/************************************************************************
 *
 * ACLEditFrame.java -
 *
 * $Id: ACLEditFrame.java,v 1.1.1.1 2006/06/30 23:38:03 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import java.lang.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import java.rmi.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import stanford.netdb.utils.*;
import javax.swing.border.*;

public class ACLEditFrame extends XFrame {

    public static ACLEditFrame THIS;

    public JPanel mainPanel = new JPanel();
    public JPanel aclPanel;
    public JPanel mainACLPanel = new JPanel(new BorderLayout());
    public JPanel tfPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
    public JTextField tf = new JTextField(11);
    public GridLayout aclLayout;
    public Hashtable acl;
    public Hashtable people;

    public ACLEditFrame() {
        super("Edit ACL");
        if( THIS != null ) {
            THIS.setState(Frame.NORMAL);
            THIS.toFront();
            return;
        }

        THIS = this;
        register();
        initFrame();
    }

    public void dispose() {
        THIS = null;
        super.dispose();
    }

    private void initFrame() {
        init(); // init XFrame bowls

        tool_bar.removeAll();
        tool_bar.add(modifyB);
        tool_bar.add(closeB);
        modifyB.setText("Save");
        modifyB.setEnabled(true);
        closeB.setText("Cancel");

        ActionListener[] al = closeB.getActionListeners();
        for( int i = 0; i < al.length; i++ )
            closeB.removeActionListener(al[i]);

        closeB.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    dispose();
                }
            });

        modifyB.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    try {
                        MainWindow.netdb.updateACL(acl);
                        closeB.doClick();

                    } catch (java.rmi.NoSuchObjectException ex) {
                        X.handleTimeout();

                    } catch (java.rmi.MarshalException ex) {
                        X.handleTimeout();

                    } catch (Exception ex) {
                        JOptionPane.showMessageDialog(THIS, ex.getMessage(),
                            "Error", JOptionPane.ERROR_MESSAGE);
                    }
                }
            });

        mainPanel.setBackground(Color.white);
        central_panel.removeAll();
        central_panel.add(mainPanel);
        current_panel = mainPanel;
        current_panel.setLayout(current_gb_layout);

        // empty row at the top
        XLabel(" ");
        tr();

        current_gbc.fill = GridBagConstraints.NONE;
        current_gbc.anchor = GridBagConstraints.WEST;
        try {
            acl = Utils.copyHash(MainWindow.acl);
        } catch (Exception ex) {
            Utils.debug(ex.getMessage());
            THIS = null;
            dispose();
        }
        people = (Hashtable)acl.get("people");
        if( people == null ) {
            people = new Hashtable();
            acl.put("people", people);
        }

        mainACLPanel.setBackground(Color.white);

        AJCheckBox denyCB =
            new AJCheckBox("Deny access to not listed bellow.",
                           acl.get("acl")!=null?true:false);
        denyCB.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                    if( cb.isSelected() )
                        acl.put("acl", new Boolean(true));
                    else
                        acl.remove("acl");
                }
            });
        JPanel topPanel = new JPanel(new BorderLayout());
        topPanel.setBackground(Color.white);
        topPanel.add(new JLabel("<html><body><h1><u>Access Control List</u></h1></body></html>"),
                     BorderLayout.NORTH);
        topPanel.add(denyCB, BorderLayout.CENTER);
        mainACLPanel.add(topPanel, BorderLayout.NORTH);

        aclLayout = new GridLayout(people.size() + 6, 2, 0, 2);
        aclPanel = new JPanel(aclLayout);
        aclPanel.setBackground(Color.white);

        aclPanel.add(new JLabel(""));
        aclPanel.add(new JLabel(""));

        aclPanel.add(new JLabel("NetID"));
        aclPanel.add(new JLabel("Admin"));

        java.util.List list = Arrays.asList(people.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();
        while( it.hasNext() ) {
            String netID = (String)it.next();
            Hashtable ref = new Hashtable();
            ref.put("NetID", netID);
            AJCheckBox cb = new AJCheckBox(netID+"       ", true, ref);
            cb.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                        aclPanel.remove(cb);
                        aclPanel.remove((AJCheckBox)cb.get("adminCB"));
                        aclLayout.setRows(aclLayout.getRows() - 1);
                        people.remove(cb.getStr("NetID"));
                        mainACLPanel.revalidate();
                        mainACLPanel.repaint();
                    }
                });
            aclPanel.add(cb);
            cb = new AJCheckBox("", people.get(netID).toString().equals("god")?true:false, ref);
            cb.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                        people.put(cb.getStr("NetID"), cb.isSelected()?"god":"");
                    }
                });
            aclPanel.add(cb);
            ref.put("adminCB", cb);
        }

        tfPanel.setBackground(Color.white);
        tf.setFont(label_font);
        tf.addActionListener( new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    String newNetID = tf.getText().trim();
                    if( people.get(newNetID) != null )
                        return;
                    people.put(newNetID, "");
                    tf.setText("");

                    // rebuild aclPanel
                    aclPanel.removeAll();
                    aclLayout.setRows(aclLayout.getRows() + 1);
                    aclPanel.add(new JLabel(""));
                    aclPanel.add(new JLabel(""));

                    aclPanel.add(new JLabel("NetID"));
                    aclPanel.add(new JLabel("Admin"));

                    java.util.List list = Arrays.asList(people.keySet().toArray());
                    Collections.sort(list, name_cmp);
                    Iterator it = list.iterator();
                    while( it.hasNext() ) {
                        String netID = (String)it.next();
                        Hashtable ref = new Hashtable();
                        ref.put("NetID", netID);
                        AJCheckBox cb = new AJCheckBox(netID+"       ", true, ref);
                        cb.addActionListener(new ActionListener() {
                                public void actionPerformed(ActionEvent e) {
                        AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                        aclPanel.remove(cb);
                        aclPanel.remove((AJCheckBox)cb.get("adminCB"));
                        aclLayout.setRows(aclLayout.getRows() - 1);
                        people.remove(cb.getStr("NetID"));
                        mainACLPanel.revalidate();
                        mainACLPanel.repaint();
                                }
                            });
                        aclPanel.add(cb);
                        cb = new AJCheckBox("", people.get(netID).toString().equals("god")?true:false, ref);
                        cb.addActionListener(new ActionListener() {
                                public void actionPerformed(ActionEvent e) {
                                    AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                                    people.put(cb.getStr("NetID"), cb.isSelected()?"god":"");
                                }
                            });
                        aclPanel.add(cb);
                        ref.put("adminCB", cb);
                        mainACLPanel.revalidate();
                        mainACLPanel.repaint();
                    }
                    aclPanel.add(tfPanel);
                    aclPanel.add(new JLabel(""));
                    tf.requestFocus();
                }
            });
        tfPanel.add(tf);
        aclPanel.add(tfPanel);
        aclPanel.add(new JLabel(""));

        mainACLPanel.add(aclPanel, BorderLayout.CENTER);
        current_gb_layout.setConstraints(mainACLPanel, current_gbc);
        current_panel.add(mainACLPanel);
        current_gbc.fill = GridBagConstraints.HORIZONTAL;
        current_gbc.anchor = GridBagConstraints.CENTER;
        tr();

        XLabel(" ");
        tr();

        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        pack();
        setSize(900, 700);
        Dimension scrnsize = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation( (int)((scrnsize.getWidth()-getWidth())/2),
                     (int)((scrnsize.getHeight()-getHeight())/2) );
        show();
    }
}
