/************************************************************************
 *
 * ATeamInfoFrame.java -
 *
 * $Id: ATeamInfoFrame.java,v 1.1.1.1 2006/06/30 23:38:03 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import java.lang.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import stanford.netdb.utils.*;

public class ATeamInfoFrame extends XFrame implements Runnable {

    public String handle;
    public Hashtable ateam;
    public JButton tmplB;

    public static JFrame THIS;

    public ATeamInfoFrame(Hashtable ateam) {
        super("ATeamInfo: "+ateam.get("name"));
        handle = (String)ateam.get("name");
        this.ateam = ateam;
        initFrame();
    }

    public ATeamInfoFrame(String handle) {
        super("ATeamInfo: "+handle);
        this.handle = handle;
        initFrame();
    }

    private void initFrame() {
        THIS = this;
        init(); // init XFrame bowls

        reloadB.addActionListener( new AActionListener(this) {
                public void actionPerformed(ActionEvent event) {
                    ((ATeamInfoFrame)parent).ateam = null;
                    (new Thread((ATeamInfoFrame)parent)).start();
                }
            });

        deleteB.addActionListener( new AActionListener(this) {
                public void actionPerformed(ActionEvent event) {
                    Hashtable ateam = ((ATeamInfoFrame)parent).ateam;
                    if( JOptionPane.showConfirmDialog(parent, "Delete ATeam '"+handle+"'?",
                                                      "Delete Ateam ?",
                                                      JOptionPane.YES_NO_OPTION) == 0 ) {
                        boolean unlocked = false;
                        int id = -1;
                        Hashtable result = new Hashtable();
                        Timestamp tm = null;
                        try {
                            id = Integer.parseInt(ateam.get("id").toString());
                            tm = MainWindow.netdb.lock(Constants.ATEAM, id);
                            result = MainWindow.netdb.delete(Constants.ATEAM, ateam.get("id").toString());
                            MainWindow.netdb.unlock(Constants.ATEAM, id, tm);
                            unlocked = true;
                            JOptionPane.showMessageDialog(parent,
                                 "Ateam has been deleted.", "Info", JOptionPane.INFORMATION_MESSAGE);
                            ((JFrame)parent).dispose();

                        } catch(RemoteNetDBException exx) {
                            Vector errors = exx.getMessages();
                            String[] msg = new String[errors.size()];
                            for( int i = 0; i < errors.size(); i++ )
                                msg[i] = (String)errors.elementAt(i);
                            JOptionPane.showMessageDialog(parent,
                              msg, "Error", JOptionPane.ERROR_MESSAGE);

                        } catch (Exception ex) {
                            JOptionPane.showMessageDialog(parent,
                              ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);

                        } finally {
                            if( !unlocked && tm != null ) {
                                // try to unlock
                                try {
                                    MainWindow.netdb.unlock(Constants.ATEAM, id, tm);
                                } catch(Exception exx) {
                                }
                            }
                        }
                    }
                }
            });

        modifyB.addActionListener( new AActionListener(this) {
                public void actionPerformed(ActionEvent event) {
                    //new ATeamModifyFrame(handle);
                }
            });

        tmplB = new JButton("Use as a template");
        tmplB.setEnabled(false);
        tool_bar.add(new JLabel("   "));
        tool_bar.add(tmplB);
        tmplB.addActionListener( new AActionListener(this) {
                public void actionPerformed(ActionEvent event) {
                    //new ATeamModifyFrame(ateam);
                }
            });

        this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        this.pack();
        this.setSize(900, 700);
        Dimension scrnsize = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation( (int)((scrnsize.getWidth()-getWidth())/2),
                     (int)((scrnsize.getHeight()-getHeight())/2) );
        this.show();

        (new Thread(this)).start();
    }

    public void run() {
        try {
            boolean hasAccess = false;
            boolean hasGenericAccess = false;

            reloadB.setEnabled(false);
            modifyB.setEnabled(false);

            if( ateam == null ) {
                ateam = MainWindow.netdb.getInfo(Constants.ATEAM, handle);
                if ( ateam.get("id") == null )
                    throw new NetDBException("ATeam "+handle+
                      " does not exist", new Vector());
            }

            // check that user has access to the admin team
            hasGenericAccess = MainWindow.netdb.checkAccess(Constants.ATEAM);
            if( hasGenericAccess && MainWindow.netdb.checkGroupAccess(ateam) )
                hasAccess = true;

            current_panel = central_panel;
            current_panel.removeAll();
            current_panel.setLayout(current_gb_layout);

            current_gbc.fill = GridBagConstraints.HORIZONTAL;
            current_gbc.weightx = 1.0;
            current_gbc.weighty = 0;
            current_gbc.gridx = 0;
            current_gbc.gridy = 0;

            // empty row at the top
            XLabel(" ");
            tr();

            XLabel("ATeam");
            td();
            XLabel(ateam.get("name").toString(), LEFT);
            tr();

            // empty row
            XLabel("");
            tr();

            XLabel("Email");
            td();
            XLabel(ateam.get("email").toString(), LEFT);
            tr();

            // empty row
            XLabel("");
            tr();

            XLabel("Phone");
            td();
            XLabel(ateam.get("phone").toString(), LEFT);
            tr();

            // empty row
            XLabel("");
            tr();

            XLabel("Groups");

            Hashtable groups = (Hashtable)ateam.get("groups");
            java.util.List list = Arrays.asList(groups.keySet().toArray());
            Collections.sort(list, name_cmp);
            Object[] groupsArray = list.toArray();
            for( int i = 0; i < groupsArray.length; i++ ) {
                td();
                XLabel(groupsArray[i].toString(), LEFT);
                tr();
            }

            // empty row
            XLabel("");
            tr();

            XLabel("Departments");

            Hashtable depts = (Hashtable)ateam.get("departments");
            if( depts != null && depts.size() > 0 ) {
                list = Arrays.asList(depts.keySet().toArray());
                Collections.sort(list, name_cmp);
                Object[] deptsArray = list.toArray();
                for( int i = 0; i < deptsArray.length; i++ ) {
                    td();
                    XLabel(deptsArray[i].toString(), LEFT);
                    tr();
                }
            } else
                tr();


            // empty row
            XLabel("");
            tr();

            XLabel("Members");

            Hashtable members = (Hashtable)ateam.get("members");
            if( members != null && members.size() > 0 ) {
                list = Arrays.asList(members.keySet().toArray());
                Collections.sort(list, name_cmp);
                Object[] membersKey = list.toArray();
                for( int i = 0; i < membersKey.length; i++ ) {
                    td();
                    Hashtable person = (Hashtable)members.get(membersKey[i].toString());
                    String name  = (String)person.get("name");
                    String dirID = (String)person.get("DirID");
                    String personStr = (name!=null?name:"")+" "+(dirID!=null?"("+dirID+")":"");
                    XLabel(personStr, LEFT);
                    tr();
                }
            } else
                tr();


            // empty row
            XLabel("");
            tr();

            XLabel("Comment");
            td();
            String comment = (String)ateam.get("comment");
            if( comment == null )
                comment = "";
            XLabel(comment, LEFT);
            tr();

            // empty row
            XLabel("");
            tr();

            XLabel("Created By");
            td();
            Hashtable person = (Hashtable)ateam.get("created.by");
            String byStr = "";
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += " ("+ateam.get("created").toString().substring(0, 16)+")";
            XLabel(byStr, LEFT);
            tr();

            // Last Modified By
            XLabel("Last Modified By");
            td();
            person = (Hashtable)ateam.get("mod.by");
            byStr = "";
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += " ("+ateam.get("modified").toString().substring(0, 16)+")";
            XLabel(byStr, LEFT);
            tr();

            // empty row
            XLabel("");
            tr();

            if( hasGenericAccess )
                tmplB.setEnabled(true);

            if( hasAccess ) {
                deleteB.setEnabled(true);
                modifyB.setEnabled(true);

            } else {
                tool_bar.remove(modifyB);
                tool_bar.remove(deleteB);
                tool_bar.validate();
                tool_bar.repaint();
            }

            central_panel.validate();
            central_panel.repaint();
            scrollpane.validate();
            reloadB.setEnabled(true);

        } catch(NetDBException ex) {
            ex.printStackTrace();
            Vector errors = ex.getMessages();
            String[] msg = new String[errors.size()];
            for( int i = 0; i < errors.size(); i++ )
                msg[i] = (String)errors.elementAt(i);
            JOptionPane.showMessageDialog(this,
                     msg, "Error", JOptionPane.ERROR_MESSAGE);

        } catch (Exception ex) {
            ex.printStackTrace();
            JOptionPane.showMessageDialog(this,
                     ex.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
        }
    }
}
