/************************************************************************
 *
 * AddrSpaceSplitJoinDialog.java -
 *
 * $Id: AddrSpaceSplitJoinDialog.java,v 1.2 2006/09/26 16:54:14 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import java.rmi.*;
import java.lang.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import stanford.netdb.utils.*;

public class AddrSpaceSplitJoinDialog extends JDialog implements Runnable {

    private NetworkModifyFrame parent;
    private String startAddrSpacePrefix;
    private GridLayout grid;
    private Hashtable addressSpaces = new Hashtable();
    public AJButton commitB;
    public JToggleButton splitB;
    public JToggleButton joinB;
    public Hashtable workingAS;
    public Hashtable labelsH = new Hashtable();
    public Comparator as_ip_cmp;

    public static Font label_font = null;
    public static Font bold_font = null;
    public JPanel labelsPanel;
    public JLabel msg;

    public AddrSpaceSplitJoinDialog THIS;

    public AddrSpaceSplitJoinDialog(NetworkModifyFrame parent, String startAddrSpacePrefix) {
        super(parent, true);
        setTitle("Addr. Space Split/Join Dialog");

        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

        as_ip_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Hashtable as1 = (Hashtable)addressSpaces.get((String)o1);
                    Hashtable as2 = (Hashtable)addressSpaces.get((String)o2);
                    return (int)(((Integer)as1.get("addr_space_ip")).intValue() -
                                 ((Integer)as2.get("addr_space_ip")).intValue());
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        THIS = this;

        this.parent = parent;
        this.startAddrSpacePrefix = startAddrSpacePrefix;

        try {
            label_font = new Font("Monospaced", Font.PLAIN, 14);
        } catch (Exception ex) {
            label_font = new Font("fixed", Font.PLAIN, 14);
            Utils.warn("Got exception while loading label font. "+ex);
        }

        try {
            bold_font = new Font("Monospaced", Font.BOLD, 14);
        } catch (Exception ex) {
            bold_font = new Font("fixed", Font.BOLD, 14);
            Utils.warn("Got exception while loading label font. "+ex);
        }

        // pull out all addr spaces in this network
        Enumeration enm = parent.addrSpacesH.keys();
        while( enm.hasMoreElements() ) {
            String id = (String)enm.nextElement();
            Hashtable as = (Hashtable)parent.addrSpacesH.get(id);
            String asPrefix = (String)as.get("addr_space_prefix");
            addressSpaces.put(asPrefix, as);
            if( asPrefix.equals(startAddrSpacePrefix) ) // label it
                as.put("shown", "");
            AJLabel label = makeLabel(asPrefix, as);
            labelsH.put(label.getText(), label);
            as.put("label", label.getText());
        }
        // mark cojoined labels
        mark_cojoined();

        Container root = this.getContentPane();
        JPanel rootPanel = new JPanel(new BorderLayout());
        rootPanel.setBackground(Color.white);
        root.add(rootPanel);
        msg = new JLabel(" ");
        JPanel msgPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        msgPanel.add(msg);
        rootPanel.add(msgPanel, BorderLayout.NORTH);

        Hashtable as = (Hashtable)addressSpaces.get(startAddrSpacePrefix);
        grid = new GridLayout(1, 1);
        labelsPanel = new JPanel(grid);
        labelsPanel.setBackground(Color.white);
        labelsPanel.add((JLabel)labelsH.get((String)as.get("label")));

        GridBagLayout gbl = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.fill = GridBagConstraints.BOTH;
        JPanel centralPanel = new JPanel(gbl);
        centralPanel.setBackground(Color.white);
        rootPanel.add(centralPanel, BorderLayout.CENTER);

        gbc.gridy = 1;
        JLabel label = new JLabel("");
        gbl.setConstraints(label, gbc);
        centralPanel.add(label);
        gbc.gridy++;

        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbl.setConstraints(labelsPanel, gbc);
        centralPanel.add(labelsPanel);
        gbc.gridy++;

        gbc.fill = GridBagConstraints.BOTH;
        label = new JLabel("");
        gbl.setConstraints(label, gbc);
        centralPanel.add(label);

        JPanel bottomPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
        commitB= new AJButton("Commit", null, false);
        commitB.obj = parent;
        commitB.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    // stuff new address spaces back to the net
                    AJButton button = (AJButton)event.getSource();
                    NetworkModifyFrame parent = (NetworkModifyFrame)button.obj;

                    // stuff new addr. spaces
                    Enumeration enm = addressSpaces.keys();
                    while( enm.hasMoreElements() ) {
                        String key = (String)enm.nextElement();
                        if( parent.addrSpacesH.get(key) == null ) {
                            Hashtable as = (Hashtable)addressSpaces.get(key);
                            as.put("newAS", "");
                            String labelKey = (String)as.remove("label");
                            if( labelKey != null )
                                labelsH.remove(labelKey);
                            parent.addrSpacesH.put(key, as);
                            Utils.debug("adding addr space "+key);
                        } else { // make sure remove all "label"
                            String labelKey = (String)((Hashtable)parent.addrSpacesH.get(key)).remove("label");
                            if( labelKey != null )
                                labelsH.remove(labelKey);
                        }
                    }

                    // remove dead addr. spaces
                    enm = parent.addrSpacesH.keys();
                    while( enm.hasMoreElements() ) {
                        String key = (String)enm.nextElement();
                        if( addressSpaces.get(key) == null ) {
                            parent.addrSpacesH.remove(key);
                            Utils.debug("removing addr space "+key);
                        }
                    }
                    dispose();

                    // now rebuild gui
                    parent.msg.setText("Rebuilding address spaces...");
                    parent.mega_pane.removeAll();
                    parent.mega_pane.add(parent.msgPanel, BorderLayout.CENTER);
                    parent.mega_pane.revalidate();
                    parent.mega_pane.repaint();
                    parent.reloadB.setEnabled(false);
                    parent.modifyB.setEnabled(false);
                    parent.header_panel.remove(parent.ePanel);
                    parent.ePanelLabel.setText("");
                    parent.header_panel.revalidate();
                    (new NetworkThread(parent, NetworkThread.REBUILD_AS, null,
                                       null, null)).start();

                }
            });
        bottomPanel.add(commitB);
        JButton cancelB = new JButton("Cancel");
        cancelB.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    // remove all notes in address spaces
                    Enumeration enm = addressSpaces.keys();
                    while( enm.hasMoreElements() ) {
                        Hashtable as = (Hashtable)addressSpaces.get((String)enm.nextElement());
                        as.remove("shown");
                        String labelKey = (String)as.remove("label");
                        if( labelKey != null )
                            labelsH.remove(labelKey);
                    }
                    dispose();
                }
            });
        bottomPanel.add(cancelB);

        ButtonGroup group = new ButtonGroup();
        bottomPanel.add(new JLabel("  Action:"));
        splitB = new JToggleButton("Split", true);
        group.add(splitB);
        bottomPanel.add(splitB);
        joinB = new JToggleButton("Join", false);
        group.add(joinB);
        bottomPanel.add(joinB);
        rootPanel.add(bottomPanel, BorderLayout.SOUTH);

        this.pack();
        this.setSize(400, 300);
        Dimension scrnsize = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation( (int)((scrnsize.getWidth() - getWidth())/2),
                     (int)((scrnsize.getHeight() - getHeight())/2) );
        this.show();
    }

    public void run() {
        try {
            // disable all labels
            Enumeration enm = addressSpaces.keys();
            while( enm.hasMoreElements() ) {
                String key = (String)enm.nextElement();
                Hashtable as = (Hashtable)addressSpaces.get(key);
                AJLabel label = (AJLabel)labelsH.get((String)as.get("label"));
                label.state = false;
            }

            if( splitB.isSelected() ) {
                msg.setText("<html><body>&nbsp;Spliting "+workingAS.get("addr_space_prefix")+" ...</body></html>");
                // do splitting
                try {
                    Hashtable result = MainWindow.netdb.splitAS(workingAS);
                    Hashtable oldAS = (Hashtable)addressSpaces.remove((String)workingAS.get("addr_space_prefix"));
                    oldAS.remove("shown");
                    String labelKey = (String)oldAS.remove("label");
                    if( labelKey != null )
                        labelsH.remove(labelKey);
                    oldAS.remove("internal");
                    enm = result.keys();
                    while( enm.hasMoreElements() ) {
                        String key = (String)enm.nextElement();
                        Hashtable as = (Hashtable)result.get(key);
                        as.put("shown", "");
                        as.put("internal", "");
                        addressSpaces.put(key, as);
                    }

                    rebuildLabels(1);

                } catch(java.rmi.ServerException ex) {
                    if( ex.getCause() instanceof SessionTimeoutException )
                        X.handleTimeout();

                    else if( ex.getCause() instanceof RemoteNetDBException ) {
                        RemoteNetDBException ne = (RemoteNetDBException)ex.getCause();
                        Vector errors = ne.getMessages();
                        String msgText = "<html><body>";
                        int esize = errors.size();
                        for( int i = 0; i < esize; i++ )
                            msgText += "<font color=\"#ff0000\">&nbsp;Error: "+(String)errors.elementAt(i)+
                                "</font>"+(i<esize-1?"<br>":"");
                        msg.setText(msgText+"</body></html>");

                    } else if( ex.getCause() instanceof RemoteException ) {
                        RemoteException re = (RemoteException)ex.getCause();
                        msg.setText("<html><body>&nbsp;<font color=\"#ff0000\">Error:&nbsp;"+
                                    re.getMessage()+"</font></body></html>");

                    } else {
                        msg.setText("<html><body>&nbsp;<font color=\"#ff0000\">Error:&nbsp;"+
                                    ex.getMessage()+"</font></body></html>");
                    }

                } catch (java.rmi.NoSuchObjectException ex) {
                    X.handleTimeout();

                } catch (java.rmi.MarshalException ex) {
                    X.handleTimeout();

                } catch (Exception ex) {
                    msg.setText("<html><body>&nbsp;<font color=\"#ff0000\">Error:&nbsp;"+
                                ex.getMessage()+"</font></body></html>");
                }

            } else { // join
                try {
                    String basePrefix = (String)workingAS.get("addr_space_prefix");
                    String newASPrefix = Utils.joinedAS(basePrefix);
                    Hashtable messages =
                        MainWindow.netdb.verifyAddressSpace(newASPrefix,
                                                            new Integer((String)parent.oldNet.get("id")));
                    String adjacentPrefix = Utils.adjacentAS(basePrefix);
                    msg.setText("<html><body>&nbsp;Joining "+workingAS.get("addr_space_prefix")+" and "+adjacentPrefix+"...</body></html>");
                    Hashtable adjacentAS = (Hashtable)addressSpaces.get(adjacentPrefix);

                    long floor1 = Utils.ipstr2long(basePrefix);
                    long floor2 = Utils.ipstr2long(adjacentPrefix);
                    if( floor2 < floor1 )
                        newASPrefix = adjacentPrefix;
                    else
                        newASPrefix = basePrefix;
                    Hashtable joinedAS = Utils.joinAS(newASPrefix, addressSpaces);
                    joinedAS.put("shown", "");

                    // remove old stuff
                    addressSpaces.remove(basePrefix);
                    workingAS.remove("shown");
                    String labelKey = (String)workingAS.remove("label");
                    if( labelKey != null )
                        labelsH.remove(labelKey);
                    workingAS.remove("internal");
                    if( adjacentAS != null ) {
                        addressSpaces.remove(adjacentPrefix);
                        adjacentAS.remove("shown");
                        labelKey = (String)adjacentAS.remove("label");
                        if( labelKey != null )
                            labelsH.remove(labelKey);
                        adjacentAS.remove("internal");
                    }
                    String joinedPrefix = (String)joinedAS.get("addr_space_prefix");

                    // find out if it is internal
                    if( isInternal(Utils.adjacentAS(joinedPrefix)) )
                        joinedAS.put("internal", "");

                    // put new in
                    addressSpaces.put(joinedPrefix, joinedAS);

                    rebuildLabels(2);

                } catch(java.rmi.ServerException ex) {
                    if( ex.getCause() instanceof SessionTimeoutException )
                        X.handleTimeout();

                    else if( ex.getCause() instanceof RemoteNetDBException ) {
                        RemoteNetDBException ne = (RemoteNetDBException)ex.getCause();
                        Vector errors = ne.getMessages();
                        String msgText = "<html><body>";
                        int esize = errors.size();
                        for( int i = 0; i < esize; i++ )
                            msgText += "<font color=\"#ff0000\">&nbsp;Error: "+(String)errors.elementAt(i)+
                                "</font>"+(i<esize-1?"<br>":"");
                        msg.setText(msgText+"</body></html>");

                    } else if( ex.getCause() instanceof RemoteException ) {
                        RemoteException re = (RemoteException)ex.getCause();
                        msg.setText("<html><body>&nbsp;<font color=\"#ff0000\">Error:&nbsp;"+
                                    re.getMessage()+"</font></body></html>");

                    } else {
                        msg.setText("<html><body>&nbsp;<font color=\"#ff0000\">Error:&nbsp;"+
                                    ex.getMessage()+"</font></body></html>");
                    }

                } catch (java.rmi.NoSuchObjectException ex) {
                    X.handleTimeout();

                } catch (java.rmi.MarshalException ex) {
                    X.handleTimeout();

                } catch (Exception ex) {
                    msg.setText("<html><body>&nbsp;<font color=\"#ff0000\">Error:&nbsp;"+
                                ex.getMessage()+"</font></body></html>");
                }
            }
        } finally {
            Enumeration enm = addressSpaces.keys();
            while( enm.hasMoreElements() ) {
                String key = (String)enm.nextElement();
                Hashtable as = (Hashtable)addressSpaces.get(key);
                AJLabel label = (AJLabel)labelsH.get((String)as.get("label"));
                label.state = true;
            }
            commitB.setEnabled(true);
        }
    } // end of run()

    // will create links between cojoined address spaces in corresponing AJLabel's
    private void mark_cojoined() {
        Enumeration enm = addressSpaces.keys();
        while( enm.hasMoreElements() ) {
            String key = (String)enm.nextElement();
            Hashtable as = (Hashtable)addressSpaces.get(key);
            String prefix = (String)as.get("addr_space_prefix");
            String adjacentPrefix = Utils.adjacentAS(prefix);
            Hashtable adjacentAS = (Hashtable)addressSpaces.get(adjacentPrefix);
            if( adjacentAS == null )
                continue;
            Utils.debug(prefix+" has adjacent addr. space "+adjacentPrefix);
            AJLabel baseLabel = (AJLabel)labelsH.get((String)as.get("label"));
            AJLabel adjacentLabel = (AJLabel)labelsH.get((String)adjacentAS.get("label"));
            baseLabel.obj = adjacentLabel;
        }
    } // mark_cojoined

    private AJLabel makeLabel(String asPrefix, Hashtable as) {
        AJLabel label = new AJLabel(asPrefix, as);
        label.setHorizontalAlignment(JLabel.RIGHT);
        label.setFont(label_font);
        label.addMouseListener(new MouseAdapter() {
                public void mouseEntered(MouseEvent event) {
                    AJLabel label = (AJLabel)event.getComponent();

                    if( ( splitB.isSelected() &&
                          ((Integer)label.box.get("prefix_length")).intValue() == 30 ) || // do now allow spliting beyond /30
                        ( joinB.isSelected() &&
                          ((Integer)label.box.get("prefix_length")).intValue() == 5 ) || // do not allow joining anything below /5
                        ( label.box.get("internal") != null &&
                          label.obj == null && joinB.isSelected() ) || // internal label without adjacent cannot be joined
                        !label.state )
                        return;
                    setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
                    label.setForeground(Color.blue);
                    label.setFont(bold_font);
                    label.repaint();

                    if( joinB.isSelected() ) {
                        // find adjacent label and highlight it as well
                        AJLabel adjacentLabel = (AJLabel)label.obj;
                        if( adjacentLabel == null )
                            return;
                        adjacentLabel.setForeground(Color.blue);
                        adjacentLabel.setFont(bold_font);
                        adjacentLabel.repaint();
                    }
                }
                public void mouseExited(MouseEvent event) {
                    AJLabel label = (AJLabel)event.getComponent();
                    if( ( splitB.isSelected() &&
                          ((Integer)label.box.get("prefix_length")).intValue() == 30 ) || // do now allow spliting beyond /30
                        ( joinB.isSelected() &&
                          ((Integer)label.box.get("prefix_length")).intValue() == 5 ) || // do not allow joining anything below /5
                        ( label.box.get("internal") != null &&
                          label.obj == null && joinB.isSelected() ) || // internal label without adjacent cannot be joined
                        !label.state )
                        return;
                    setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                    label.setForeground(Color.black);
                    label.setFont(label_font);
                    label.repaint();

                    if( joinB.isSelected() ) {
                        // find adjacent label and deselect it as well
                        AJLabel adjacentLabel = (AJLabel)label.obj;
                        if( adjacentLabel == null )
                            return;
                        adjacentLabel.setForeground(Color.black);
                        adjacentLabel.setFont(label_font);
                        adjacentLabel.repaint();
                    }
                }
                public void mouseClicked(MouseEvent event) {
                    setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                    AJLabel label = (AJLabel)event.getSource();
                    if( ( splitB.isSelected() &&
                          ((Integer)label.box.get("prefix_length")).intValue() == 30 ) || // do now allow spliting beyond /30
                        ( joinB.isSelected() &&
                          ((Integer)label.box.get("prefix_length")).intValue() == 5 ) || // do not allow joining anything below /5
                        ( label.box.get("internal") != null &&
                          label.obj == null && joinB.isSelected() ) || // internal label without adjacent cannot be joined
                        !label.state )
                        return;
                    workingAS = label.box;
                    commitB.setEnabled(false);
                    (new Thread(THIS)).start();
                }
            });
        return label;
    } // end of makeLabel(...)

    public void rebuildLabels(int removed) {
        labelsPanel.removeAll();
        grid.setRows(grid.getRows()-removed);
        java.util.List asIDs = Arrays.asList(addressSpaces.keySet().toArray());
        Collections.sort(asIDs, as_ip_cmp);
        Object[] asIDsArray = asIDs.toArray();
        for( int i = 0; i < asIDsArray.length; i++ ) {
            String id = (String)asIDsArray[i];
            Hashtable as = (Hashtable)addressSpaces.get(id);
            if( as.get("shown") == null )
                continue;
            String asPrefix = (String)as.get("addr_space_prefix");
            AJLabel label = makeLabel(asPrefix, as);
            label.state = false;
            labelsH.put(label.getText(), label);
            as.put("label", label.getText());
            labelsPanel.add(label);
        }
        mark_cojoined();
        labelsPanel.repaint();
        msg.setText(" ");
    } // end of rebuildLabels()

    private boolean isInternal(String asPrefix) {
        if( addressSpaces.get(asPrefix) != null )
            return true;
        String[] splited = Utils.splitASPrefix(asPrefix);
        if( splited == null )
            return false;
        return ( isInternal(splited[0]) || isInternal(splited[1]) );
    }
}
