/************************************************************************
 *
 * DomainInfoFrame.java -
 *
 * $Id: DomainInfoFrame.java,v 1.1.1.1 2006/06/30 23:38:03 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import java.lang.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import stanford.netdb.utils.*;

public class DomainInfoFrame extends XFrame implements Runnable {

    public String handle;
    Hashtable domain = null;
    public JButton tmplB;

    public DomainInfoFrame(Hashtable domain) {
        super("DomainInfo: "+(String)domain.get("domain"));
        this.domain = domain;
        this.handle = (String)domain.get("domain");
        initFrame();
    }

    public DomainInfoFrame(String handle) {
        super("DomainInfo: "+handle);
        this.handle = handle;
        initFrame();
    }

    private void initFrame() {
        init(); // init XFrame bowls

        //reloadB.setEnabled(true);
        //deleteB.setEnabled(true);

        reloadB.addActionListener( new AActionListener(this) {
                public void actionPerformed(ActionEvent event) {
                    ((DomainInfoFrame)parent).domain = null;
                    (new Thread((DomainInfoFrame)parent)).start();
                }
            });

        deleteB.addActionListener( new AActionListener(this) {
                public void actionPerformed(ActionEvent event) {
                    Hashtable domain = ((DomainInfoFrame)parent).domain;
                    if( JOptionPane.showConfirmDialog(parent, "Delete Domain '"+((DomainInfoFrame)parent).handle+"'?",
                                                      "Delete Domain ?",
                                                      JOptionPane.YES_NO_OPTION) == 0 ) {
                        boolean unlocked = false;
                        int id = -1;
                        Hashtable result = new Hashtable();
                        Timestamp tm = null;
                        try {
                            id = Integer.parseInt(domain.get("id").toString());
                            tm = MainWindow.netdb.lock(Constants.DOMAIN, id);
                            result = MainWindow.netdb.delete(Constants.DOMAIN, domain.get("id").toString());
                            MainWindow.netdb.unlock(Constants.DOMAIN, id, tm);
                            unlocked = true;
                            JOptionPane.showMessageDialog(parent,
                                 "Domain has been deleted.", "Info", JOptionPane.INFORMATION_MESSAGE);
                            ((JFrame)parent).dispose();

                        } catch(RemoteNetDBException exx) {
                            Vector errors = exx.getMessages();
                            String[] msg = new String[errors.size()];
                            for( int i = 0; i < errors.size(); i++ )
                                msg[i] = (String)errors.elementAt(i);
                            JOptionPane.showMessageDialog(parent,
                              msg, "Error", JOptionPane.ERROR_MESSAGE);

                        } catch (Exception ex) {
                            JOptionPane.showMessageDialog(parent,
                              ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);

                        } finally {
                            if( !unlocked && tm != null ) {
                                // try to unlock
                                try {
                                    MainWindow.netdb.unlock(Constants.DOMAIN, id, tm);
                                } catch(Exception exx) {
                                }
                            }
                        }
                    }
                }
            });

        modifyB.addActionListener( new AActionListener(this) {
                public void actionPerformed(ActionEvent event) {
                    //new DomainModifyFrame(handle);
                }
            });

        tmplB = new JButton("Use as a template");
        tmplB.setEnabled(false);
        tool_bar.add(new JLabel("   "));
        tool_bar.add(tmplB);
        tmplB.addActionListener( new AActionListener(this) {
                public void actionPerformed(ActionEvent event) {
                    //new DomainInsertFrame(domain);
                }
            });

        this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        this.pack();
        this.setSize(900, 700);
        Dimension scrnsize = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation( (int)((scrnsize.getWidth()-getWidth())/2),
                     (int)((scrnsize.getHeight()-getHeight())/2) );
        this.show();

        (new Thread(this)).start();
    } // end of initFrame()

    public void run() {
        try {
            boolean hasAccess = false;
            boolean hasGenericAccess = false;

            reloadB.setEnabled(false);
            modifyB.setEnabled(false);

            if( domain == null ) {
                domain = MainWindow.netdb.getInfo(Constants.DOMAIN, handle);
                if( domain == null || domain.get("id") == null ) {
                    label.setText("Failed to load domain. It probably does not exist.");
                    label.setForeground(Color.red);
                    reloadB.setEnabled(false);
                    deleteB.setEnabled(false);
                    return;
                }
            }

            // check that user has access to the domain
            hasGenericAccess = MainWindow.netdb.checkAccess(Constants.DOMAIN);
            if( hasGenericAccess && MainWindow.netdb.checkGroupAccess(domain) )
                hasAccess = true;

            current_panel = central_panel;
            current_panel.removeAll();
            current_panel.setLayout(current_gb_layout);

            current_gbc.fill = GridBagConstraints.HORIZONTAL;
            current_gbc.weightx = 1.0;
            current_gbc.weighty = 0;
            current_gbc.gridx = 0;
            current_gbc.gridy = 0;

            // empty row at the top
            XLabel(" ");
            tr();

            XLabel("Domain");
            td();
            XLabel(domain.get("domain").toString(), LEFT);
            tr();

            XLabel("Limited");
            td();
            XLabel(((Boolean)domain.get("limited?")).booleanValue()?"Yes":"No", LEFT);
            tr();

            XLabel("Allow Name");
            td();
            XLabel(((Boolean)domain.get("allow_name?")).booleanValue()?"Yes":"No", LEFT);
            tr();

            XLabel("Comment");
            td();
            Object comment = domain.get("comment");
            if( comment != null )
                XLabel(comment.toString(), LEFT);
            tr();

            XLabel("Groups");
            Hashtable groups = (Hashtable)domain.get("groups");
            if( groups != null ) {
                java.util.List list = Arrays.asList(groups.keySet().toArray());
                Collections.sort(list, name_cmp);
                Object[] groupsArray = list.toArray();
                for( int i = 0; i < groupsArray.length; i++ ) {
                    td();
                    XLabel(groupsArray[i].toString(), LEFT);
                    tr();
                }
            } else
                tr();

            XLabel("Created By");
            td();
            Hashtable person = (Hashtable)domain.get("created.by");
            String byStr = "";
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += " ("+domain.get("created").toString().substring(0, 16)+")";
            XLabel(byStr, LEFT);
            tr();

            // Last Modified By
            XLabel("Last Modified By");
            td();
            person = (Hashtable)domain.get("mod.by");
            byStr = "";
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += " ("+domain.get("modified").toString().substring(0, 16)+")";
            XLabel(byStr, LEFT);
            tr();

            // empty row
            XLabel("");
            tr();

            reloadB.setEnabled(true);

            if( hasGenericAccess )
                tmplB.setEnabled(true);

            if( hasAccess ) {
                modifyB.setEnabled(true);
                deleteB.setEnabled(true);

            } else {
                tool_bar.remove(modifyB);
                tool_bar.remove(deleteB);
                tool_bar.validate();
                tool_bar.repaint();
            }

            central_panel.validate();
            central_panel.repaint();
            scrollpane.validate();

        } catch(Exception ex) {
            Utils.warn(ex.toString());
            ex.printStackTrace();
        }
    } // end of run()
}
