/************************************************************************
 *
 * GroupInfoFrame.java -
 *
 * $Id: GroupInfoFrame.java,v 1.1.1.1 2006/06/30 23:38:03 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import java.lang.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import stanford.netdb.utils.*;

public class GroupInfoFrame extends XFrame implements Runnable {

    public String handle;
    public Hashtable group;

    public static JFrame THIS;
    private static int counter;
    public Integer frameID;
    public static Hashtable refs = new Hashtable(); // holds all open GroupInfoFrame frames

    public GroupInfoFrame(Hashtable group) {
        super("GroupInfo: "+group.get("name"));
        handle = (String)group.get("name");
        this.group = group;
        initFrame();
    }

    public GroupInfoFrame(String handle) {
        super("GroupInfo: "+handle);
        this.handle = handle;
        initFrame();
    }

    private static synchronized Integer getCounter() {
        return new Integer((counter++)-1);
    }

    private void initFrame() {
        THIS = this;
        init(); // init XFrame bowls

        reloadB.addActionListener( new AActionListener(this) {
                public void actionPerformed(ActionEvent event) {
                    ((GroupInfoFrame)parent).group = null;
                    (new Thread((GroupInfoFrame)parent)).start();
                }
            });

        deleteB.addActionListener( new AActionListener(this) {
                public void actionPerformed(ActionEvent event) {
                    Hashtable group = ((GroupInfoFrame)parent).group;
                    if( JOptionPane.showConfirmDialog(parent, "Delete Group '"+group.get("name")+"'?",
                                                      "Delete Group ?",
                                                      JOptionPane.YES_NO_OPTION) == 0 ) {
                        boolean unlocked = false;
                        int id = -1;
                        Hashtable result = new Hashtable();
                        Timestamp tm = null;
                        try {
                            id = Integer.parseInt(group.get("id").toString());
                            tm = MainWindow.netdb.lock(Constants.GROUP, id);
                            result = MainWindow.netdb.delete(Constants.GROUP, group.get("id").toString());
                            MainWindow.netdb.unlock(Constants.GROUP, id, tm);
                            unlocked = true;
                            JOptionPane.showMessageDialog(parent,
                                 "Group has been deleted.", "Info", JOptionPane.INFORMATION_MESSAGE);
                            ((JFrame)parent).dispose();

                        } catch(RemoteNetDBException exx) {
                            Vector errors = exx.getMessages();
                            String[] msg = new String[errors.size()];
                            for( int i = 0; i < errors.size(); i++ )
                                msg[i] = (String)errors.elementAt(i);
                            JOptionPane.showMessageDialog(parent,
                              msg, "Error", JOptionPane.ERROR_MESSAGE);

                        } catch (Exception ex) {
                            JOptionPane.showMessageDialog(parent,
                              ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);

                        } finally {
                            if( !unlocked && tm != null ) {
                                // try to unlock
                                try {
                                    MainWindow.netdb.unlock(Constants.GROUP, id, tm);
                                } catch(Exception exx) {
                                }
                            }
                        }
                    }
                }
            });

        modifyB.addActionListener( new AActionListener(this) {
                public void actionPerformed(ActionEvent event) {
                    //new GroupModifyFrame(handle);
                }
            });

        closeB.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    GroupInfoFrame.refs.remove(frameID);
                    dispose();
                }
            });

        this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        this.pack();
        this.setSize(900, 700);
        Dimension scrnsize = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation( (int)((scrnsize.getWidth()-getWidth())/2),
                     (int)((scrnsize.getHeight()-getHeight())/2) );
        this.show();

        (new Thread(this)).start();

        frameID = getCounter();
        refs.put(frameID, this);
    } // end of initFrame()

    public void run() {
        try {
            boolean hasAccess = false;

            reloadB.setEnabled(false);
            modifyB.setEnabled(false);

            if( group == null ) {
                group = MainWindow.netdb.getInfo(Constants.GROUP, handle);
                if ( group.get("id") == null ) {
                    Vector errors = new Vector();
                    throw new NetDBException("Group "+handle+
                      " does not exist", errors);
                }
            }

            // check that user has access to the group
            if( MainWindow.netdb.checkAccess(Constants.DOMAIN) && MainWindow.netdb.checkGroupAccess(group) )
                hasAccess = true;

            current_panel = central_panel;
            current_panel.removeAll();
            current_panel.setLayout(current_gb_layout);

            current_gbc.fill = GridBagConstraints.HORIZONTAL;
            current_gbc.weightx = 1.0;
            current_gbc.weighty = 0;
            current_gbc.gridx = 0;
            current_gbc.gridy = 0;

            // empty row at the top
            XLabel(" ");
            tr();

            XLabel("Group");
            td();
            XLabel(group.get("name").toString(), LEFT);
            tr();

            XLabel(" ");
            tr();

            XLabel("Comment");
            td();
            Object obj = group.get("comment");
            if( obj != null )
                XLabel(obj.toString(), LEFT);
            tr();

            XLabel(" ");
            tr();

            // Created By
            XLabel("Created By");
            td();
            Hashtable person = (Hashtable)group.get("created.by");
            String byStr = "";
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += " ("+group.get("created").toString().substring(0, 16)+")";
            XLabel(byStr, LEFT);
            tr();

            // Last Modified By
            XLabel("Last Modified By");
            td();
            person = (Hashtable)group.get("mod.by");
            byStr = "";
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += " ("+group.get("modified").toString().substring(0, 16)+")";
            XLabel(byStr, LEFT);
            tr();

            // empty row
            XLabel("");
            tr();

            if( hasAccess ) {
                modifyB.setEnabled(true);
                deleteB.setEnabled(true);

            } else {
                tool_bar.remove(modifyB);
                tool_bar.remove(deleteB);
                tool_bar.validate();
                tool_bar.repaint();
            }

            central_panel.validate();
            central_panel.repaint();
            scrollpane.validate();

            reloadB.setEnabled(true);

        } catch(NetDBException ex) {
            ex.printStackTrace();
            Vector errors = ex.getMessages();
            String[] msg = new String[errors.size()];
            for( int i = 0; i < errors.size(); i++ )
                msg[i] = (String)errors.elementAt(i);
            JOptionPane.showMessageDialog(this,
                     msg, "Error", JOptionPane.ERROR_MESSAGE);

        } catch(RemoteNetDBException ex) {
            ex.printStackTrace();
            Vector errors = ex.getMessages();
            String[] msg = new String[errors.size()];
            for( int i = 0; i < errors.size(); i++ )
                msg[i] = (String)errors.elementAt(i);
            JOptionPane.showMessageDialog(this,
                     msg, "Error", JOptionPane.ERROR_MESSAGE);

        } catch(Exception ex) {
            ex.printStackTrace();
            JOptionPane.showMessageDialog(this,
                     ex.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
        }
    } // end of run()
}
