/************************************************************************
 *
 * NetworkInfoFrame.java -
 *
 * $Id: NetworkInfoFrame.java,v 1.2 2006/09/26 16:56:07 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import java.lang.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import stanford.netdb.utils.*;

public class NetworkInfoFrame extends XFrame implements Runnable {

    public String handle = null;
    public String idNet = null;
    public Hashtable network = null;
    public NetworkInfoFrame THIS;
    public JPanel mainPanel = new JPanel();
    public Comparator as_ip_cmp = null;
    public Hashtable addrSpacesH;
    public boolean delete = false;
    public boolean close_this = false;

    public JButton templateB = new JButton("Use As Template");
    public JButton moveASB = new JButton("Move Address Spaces");

    public boolean rights = true;

    private String saved_handle = null;

    public static Hashtable netInfoFrames = new Hashtable();

    public void dispose() {
        if( handle != null )
            netInfoFrames.remove(handle.toLowerCase());
        super.dispose();

    }

    private void initFrame() {
        init(); // init XFrame bowls
        register();
        THIS = this;

        tool_bar.removeAll();
        tool_bar.add(modifyB);
        tool_bar.add(moveASB);
        tool_bar.add(templateB);
        tool_bar.add(deleteB);
        tool_bar.add(new JLabel("   "));
        tool_bar.add(reloadB);
        tool_bar.add(new JLabel("   "));
        tool_bar.add(closeB);

        templateB.setEnabled(false);
        moveASB.setEnabled(false);

        templateB.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    new NetworkModifyFrame(handle, true);
                }
            });

        moveASB.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    new NetworkMoveFrame(handle);
                }
            });

        NetworkInfoFrame f = (NetworkInfoFrame)netInfoFrames.get(handle.toLowerCase());
        if( f != null ) {
            f.setState(Frame.NORMAL);
            f.toFront();
            if( delete )
                f.deleteB.doClick();
            close_this = true;
        } else {

            as_ip_cmp = new Comparator() {
                    public int compare(Object o1, Object o2) {
                        Hashtable as1 = (Hashtable)addrSpacesH.get((String)o1);
                        Hashtable as2 = (Hashtable)addrSpacesH.get((String)o2);
                        return (int)(((Integer)as1.get("addr_space_ip")).intValue() -
                                     ((Integer)as2.get("addr_space_ip")).intValue());
                    }
                    public boolean equals(Object obj) {
                        return true;
                    }
                };

            netInfoFrames.put(handle.toLowerCase(), this);

            mainPanel.setBackground(Color.white);

            closeB.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent event) {
                        dispose();
                    }
                });

            reloadB.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    reloadB.setEnabled(false);
                    templateB.setEnabled(false);
                    deleteB.setEnabled(false);
                    modifyB.setEnabled(false);
                    moveASB.setEnabled(false);
                    central_panel.removeAll();
                    central_panel.add(label, BorderLayout.CENTER);
                    label.setText("Reloading network...");
                    central_panel.revalidate();
                    mainPanel.removeAll();
                    network = null;
                    saved_handle = handle.toLowerCase();
                    handle = null;
                    (new Thread(NetworkInfoFrame.this)).start();
                }
                });

            deleteB.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent event) {
                        if( JOptionPane.showConfirmDialog(THIS, "Delete Network '"+handle+"'?",
                                                          "Delete Network ?",
                                                          JOptionPane.YES_NO_OPTION) != 0 )
                            return;
                        deleteB.setEnabled(false);
                        templateB.setEnabled(false);
                        modifyB.setEnabled(false);
                        reloadB.setEnabled(false);
                        moveASB.setEnabled(false);
                        central_panel.removeAll();
                        central_panel.add(label, BorderLayout.CENTER);
                        label.setText("Deleting network...");
                        central_panel.revalidate();
                        (new NetworkThread(THIS, NetworkThread.DELETE_NETWORK, network,
                                           "Network has been deleted")).start();
                    }
                });

            this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
            this.pack();
            this.setSize(900, 700);

            Dimension scrnsize = Toolkit.getDefaultToolkit().getScreenSize();
            setLocation( (int)((scrnsize.getWidth()-getWidth())/2),
                         (int)((scrnsize.getHeight()-getHeight())/2) );

            this.setVisible(true);
        }

        (new Thread(this)).start();
    } // end of initFrame()

    public NetworkInfoFrame(Hashtable network) {
        super("(Hash)");
        this.network = network;
        Hashtable names = (Hashtable)network.get("names");
        Enumeration enm = names.keys();
        handle = ((String)enm.nextElement()).trim();
        initFrame();
    }

    public NetworkInfoFrame(String handle) {
        super("NetworkInfo: "+handle);
        this.handle = handle;
        initFrame();
    }

    public NetworkInfoFrame(String handle, boolean delete) {
        super("NetworkInfo: "+handle);
        this.handle = handle;
        this.delete = true;
        initFrame();
    }

    public void run() {
        if( close_this ) {
            super.dispose();
            return;
        }
        try {
            if( network == null )
                if( handle != null )
                    network = MainWindow.netdb.getInfo(Constants.NETWORK, handle);
                else if( idNet != null )
                    network = MainWindow.netdb.getInfo(Constants.NETWORK, Integer.parseInt(idNet));
                else {
                    // this should not happen
                    label.setText("<html><body><font color=\"#ff0000\">Failed to reload network info<br>"+
                                  "Net ID does not exist.</font></body></html>");
                    return;
                }

            if( network == null || network.get("id") == null ) {
                label.setText("<html><body><font color=\"#ff0000\">Network " + handle +
                              " does not exist.</font></body></html>");
                return;
            }

            label.setText("Verifying access...");
            if( !MainWindow.netdb.checkAccess(Constants.NETWORK) ||
                (handle != null && !MainWindow.netdb.checkGroupAccess(network) ) )
                rights = false;

            idNet = (String)network.get("id");
            if( handle == null ) {
                // pul out handle
                Hashtable names = (Hashtable)network.get("names");
                Enumeration enm = names.keys();
                handle = ((String)enm.nextElement()).trim().toLowerCase();
                netInfoFrames.put(handle, (NetworkInfoFrame)netInfoFrames.remove(saved_handle));
            }

            try {
                Hashtable names = Utils.nHash(network.get("names"));
                Hashtable netNameH = (Hashtable)names.get(names.keys().nextElement());
                setTitle("NetworkInfo: "+netNameH.get("name")+"."+netNameH.get("domain"));
            } catch (Exception nameEx) {}

            // now display what we have got
            current_panel = mainPanel;
            current_panel.setLayout(current_gb_layout);

            current_gbc.fill = GridBagConstraints.HORIZONTAL;
            current_gbc.weightx = 1.0;
            current_gbc.weighty = 0;
            current_gbc.gridx = 0;
            current_gbc.gridy = 0;

            Hashtable names = (Hashtable)network.get("names");
            java.util.List list = Arrays.asList(names.keySet().toArray());
            Collections.sort(list, name_cmp);
            Object[] nameArray = list.toArray();

            current_gbc.anchor = GridBagConstraints.NORTHWEST;
            for( int i = 0; i < nameArray.length; i++ )
            {
                // empty row at the top
                XLabel(" ");
                tr();

                XLabel("Names");
                td();
                Hashtable name = (Hashtable)names.get(nameArray[i]);
                if( name.get("domain").toString().trim().toLowerCase().equals(MainWindow.userDefDomain) )
                    XLabel((String)name.get("name"), LEFT);
                else
                    XLabel((String)name.get("name")+"."+(String)name.get("domain"), LEFT);
                tr();

                Hashtable aliases = (Hashtable)name.get("aliases");
                if( aliases != null )
                {
                    java.util.List alist = Arrays.asList(aliases.keySet().toArray());
                    Collections.sort(alist, name_cmp);
                    Object[] aliasArray = alist.toArray();

                    for( int j = 0; j < aliasArray.length; j++ )
                    {
                        tr();
                        if( j == 0 )
                        {
                            XLabel(" ");
                            tr();
                            XLabel("Alias(es)");
                        }
                        td();
                        XLabel("  "+aliasArray[j], LEFT);
                        tr();
                    }
                }
            }

            // empty line
            XLabel(" ");
            tr();

            XLabel("Groups");
            td();

            if (network.get("groups") != null)
                showList ((Hashtable)network.get("groups"));

            // empty line
            XLabel("");
            tr();

            XLabel("Address Spaces");
            tr ();
            if (network.get("address_spaces") != null)
            {
                addrSpacesH = (Hashtable)network.get("address_spaces");
                java.util.List asIDs = Arrays.asList(addrSpacesH.keySet().toArray());
                Collections.sort(asIDs, as_ip_cmp);
                Object[] asIDsArray = asIDs.toArray();

                for(int i = 0; i < asIDsArray.length; i++)
                {
                    Hashtable addrSpace = (Hashtable)addrSpacesH.get(asIDsArray[i]);

                    int ipInteger = ((Integer)addrSpace.get("addr_space_ip")).intValue();
                    int prefixLength = ((Integer)addrSpace.get("prefix_length")).intValue();
                    int prefixMask =  ~(0xffffffff << (32 - prefixLength));
                    int lowRes = ((Integer)addrSpace.get("low_res")).intValue() - ipInteger;
                    int highRes = ( ipInteger | prefixMask ) -
                        ((Integer)addrSpace.get("high_res")).intValue();

                    String  IP = Utils.ip2str(ipInteger);

                    XLabel (IP + "/" + prefixLength + "         ", RIGHT);
                    tr ();

                    XLabel ("Low Reserve: " + lowRes, RIGHT);
                    td ();

                    XLabel ("High Reserve: " + highRes, LEFT);
                    tr ();

                    if (addrSpace.get("groups") != null) {
                        XLabel (" ");
                        tr ();

                        XLabel ("Group(s)", RIGHT);
                        td ();

                        showList ((Hashtable)addrSpace.get("groups"));
                    }

                    if (addrSpace.get("comment") != null) {
                        XLabel ("Comment", RIGHT);
                        td ();

                        XLabel ("<html><body>"+
                                Utils.htmlize(addrSpace.get("comment").toString())
                                +"</body></html>", LEFT);

                        tr ();
                    }

                    if( addrSpace.get("dhcp_options") != null ) {
                        Hashtable dhcp_opts = (Hashtable)addrSpace.get("dhcp_options");
                        if( dhcp_opts.size() > 0 ) {
                            XLabel ("DHCP Options", RIGHT);
                            java.util.List list1 = Arrays.asList(dhcp_opts.keySet().toArray());
                            Collections.sort(list1, name_cmp);
                            Object[] optsArray = list1.toArray();
                            for( int j = 0; j < optsArray.length; j++ ) {
                                td();
                                XLabel(optsArray[j].toString().trim(), LEFT);
                                tr();
                            }
                        }
                    }

                    Hashtable dhcp_addresses = (Hashtable)addrSpace.get("dhcp_addresses");
                    if( dhcp_addresses == null )
                        dhcp_addresses = new Hashtable();
                    if( dhcp_addresses.size() > 0 ) {
                        XLabel ("DHCP addr(s)", RIGHT);
                        td();
                        JPanel aPanel = new JPanel(new GridLayout(dhcp_addresses.size(), 1, 0, 2));
                        aPanel.setBackground(Color.white);
                        java.util.List dlist = Arrays.asList(dhcp_addresses.keySet().toArray());
                        Collections.sort(dlist, ip_cmp);
                        Iterator it = dlist.iterator();
                        while( it.hasNext() ) {
                            String ipStr = (String)it.next();
                            Hashtable dhcp_ip = (Hashtable)dhcp_addresses.get(ipStr);
                            Boolean active = (Boolean)dhcp_ip.get("active?");
                            Hashtable dhcp_names = (Hashtable)dhcp_ip.get("names");
                            if( dhcp_names == null )
                                dhcp_names = new Hashtable();
                            String dhcp_name = "";
                            String dhcp_domain = "";
                            Enumeration dhcp_name_enm = dhcp_names.keys();
                            if( dhcp_name_enm.hasMoreElements() ) {
                                Hashtable dhcp_name_H = (Hashtable)dhcp_names.get((String)dhcp_name_enm.nextElement());
                                dhcp_name = ((String)dhcp_name_H.get("name")).trim();
                                dhcp_domain = ((String)dhcp_name_H.get("domain")).trim();
                                if( !dhcp_domain.toLowerCase().equals(MainWindow.userDefDomain) )
                                    dhcp_name += "."+dhcp_domain;
                            }
                            JLabel label = new JLabel(format(ipStr, 16)+(active.booleanValue()?"    ":"off ")+dhcp_name);
                            label.setFont(label_font);
                            aPanel.add(label);
                        }
                        current_gbc.gridwidth = GridBagConstraints.REMAINDER;
                        current_gb_layout.setConstraints(aPanel, current_gbc);
                        current_panel.add(aPanel);
                        tr();
                    }
                    tr();

                    if (i != asIDsArray.length - 1)
                        {
                            XLabel (" ");
                            tr ();
                        }
                }
            }

            fullSeparator ();

            XLabel("DHCP Service");
            td();

            if (network.get("dhcp_service") != null)
                XLabel(network.get("dhcp_service").toString(), LEFT);

            tr();

            XLabel("DHCP Options");
            td();

            if (network.get("dhcp_options") != null)
                showList ((Hashtable)network.get("dhcp_options"));

            tr();

            fullSeparator ();

            XLabel("Comment");
            td();

            String comment = (String)network.get("comment");
            if ( comment != null )
                XLabel("<html><body>"+Utils.htmlize(Utils.brakes(comment, 50))+"</body></html>", LEFT);

            tr();

            fullSeparator ();

            Hashtable person = (Hashtable)network.get("created.by");
            String finalStr = "";

            if( person.get("name") != null )
                finalStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                finalStr += "("+person.get("NetID")+") ";

            finalStr += " ("+network.get("created").toString().substring(0, 16)+")";

            XLabel("Created by ");
            td();
            XLabel(finalStr, LEFT);
            tr();

            person = (Hashtable)network.get("mod.by");
            finalStr = "";

            if( person.get("name") != null )
                finalStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                finalStr += "("+person.get("NetID")+") ";

            finalStr += " ("+network.get("modified").toString().substring(0, 16)+")";

            XLabel("Modified by");
            td();
            XLabel(finalStr, LEFT);
            tr();

            XLabel(" ");
            tr();

            central_panel.validate();
            central_panel.repaint();
            scrollpane.validate();

            if( rights ) {
                modifyB.setEnabled(true);
                templateB.setEnabled(true);
                moveASB.setEnabled(true);
                deleteB.setEnabled(true);
            }

            modifyB.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        new NetworkModifyFrame(handle, false);
                    }
                });

            central_panel.removeAll();
            central_panel.add(mainPanel);
            central_panel.revalidate();


            if( delete ) {
                delete = false; // reset delete
                deleteB.doClick();
            }

        } catch (java.rmi.ServerException ex) {
            if( ex.getCause() instanceof SessionTimeoutException )
                X.handleTimeout();
            Utils.debug(ex.getMessage());

        } catch (java.rmi.NoSuchObjectException ex) {
            X.handleTimeout();

        } catch (java.rmi.MarshalException ex) {
            X.handleTimeout();

        } catch(Exception ex) {
            Utils.debug(ex.getMessage());

        } finally {
            central_panel.repaint();
            reloadB.setEnabled(true);
        }
    } // end of run()

    private String longToIP (long orig)
    {
        String IP = "";

        for (int i = 0; i < 4; i++)
        {
            IP = (orig & 255) + IP;

            if (i != 3)
                IP = "." + IP;

            orig >>= 8;
        }

        return IP;
    }

    private void fullSeparator ()
    {
        XLabel(" ");
        tr();

        line();

        XLabel(" ");
        tr();

        return;
    }// END: fullSeparator

    private void showList (Hashtable options)
    {
        java.util.List list = Arrays.asList(options.keySet().toArray());
        Collections.sort(list, name_cmp);
        Object[] optionKeys = list.toArray();

        for(int i = 0; i < optionKeys.length; i++)
        {
            if (options.get(optionKeys[i]) != null)
            {
                if (i != 0)
                {
                    XLabel("");
                    td();
                }

                XLabel(optionKeys[i].toString(), LEFT);

                tr();
            }
        }
        return;
    } //END: showList
}
