/************************************************************************
 *
 * NetworkModifyFrame.java -
 *
 * $Id: NetworkModifyFrame.java,v 1.2 2006/09/26 16:56:30 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import java.lang.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import java.rmi.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import stanford.netdb.utils.*;
import javax.swing.border.*;

public class NetworkModifyFrame extends XFrame implements Runnable {

    public JPanel ePanel;
    public JLabel ePanelLabel;
    public JPanel msgPanel  = new JPanel(new BorderLayout());
    public JPanel mainPanel = new JPanel();
    public JTextField netNameTf;
    public JPanel mgroups_big_panel;
    public JPanel groups_panel;
    public JButton add_another_group;
    public JButton allB;
    public JPanel bottom_groups_panel;
    public GridLayout groups_panel_layout;
    public JScrollPane groups_sp;
    public AJList groups_list;
    public JTextArea commentTa;
    public JComboBox dhcpService;
    public JTextArea dhcpOptions;
    public JButton addAddrSpaceB;
    public Hashtable asPanels;
    public JTextField newAddrSpacePrefTf;
    public Object[] newAddrSpaceButtons;
    public JDialog addAddrSpaceDialogWin;
    public int separator_gridx;
    public int separator_gridy;
    public Font bold_font;
    public boolean template;
    public boolean close_this = false;

    public JLabel msg = new JLabel("", JLabel.CENTER);

    public String handle = null; // networks' fqdn
    public Hashtable oldNet;
    public Hashtable newNet;
    public Timestamp lockedTm = null;
    public Hashtable allGroupsH;
    public Hashtable dhcpServicesH;
    public Hashtable dhcpOptionsH;
    public Hashtable addrSpacesH;
    public int available_key = 1;
    public Hashtable reservedIPs = new Hashtable(); // used for canceling reserved but not commited ips
    public Hashtable reservedNames = new Hashtable(); // used for canceling reserved but not commited names
    // stores all reserved and/or committed names, thus all names from oldNet are here
    public Hashtable reserved_and_committed_names = new Hashtable();

    public NetworkModifyFrame THIS;

    // used for sorting of address spaces
    public Comparator as_ip_cmp = null;

    public static Hashtable netModFrames = new Hashtable();

    public NetworkModifyFrame(String handle, boolean template) {
        super("NetworkModify: "+handle);
        if( handle == null || template )
            setTitle("NetworkInsert");

        this.template = template;

        try {
            bold_font = new Font("Monospaced", Font.BOLD, 14);
        } catch (Exception ex) {
            bold_font = new Font("fixed", Font.BOLD, 14);
            Utils.warn("Got exception while loading label font. "+ex);
        }

        if( handle != null && !template ) {
            NetworkModifyFrame f = (NetworkModifyFrame)netModFrames.get(handle.toLowerCase());
            if( f != null ) {
                f.setState(Frame.NORMAL);
                f.toFront();
                close_this = true;
                initFrame();
                return;
            }
        }

        as_ip_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Hashtable as1 = (Hashtable)addrSpacesH.get((String)o1);
                    Hashtable as2 = (Hashtable)addrSpacesH.get((String)o2);
                    return (int)(((Integer)as1.get("addr_space_ip")).intValue() -
                                 ((Integer)as2.get("addr_space_ip")).intValue());
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        if( handle != null && !template )
            netModFrames.put(handle.toLowerCase(), this);

        THIS = this;
        this.handle = handle;
        register();
        initFrame();
    } // end of constructor

    private void initFrame() {
        if( close_this ) {
            (new Thread(this)).start();
            return;
        }

        init(); // init XFrame bowls

        deleteB.setText("Save and continue");
        deleteB.setEnabled(false);
        reloadB.setText("Reset");
        if( handle == null )
            tool_bar.remove(reloadB);
        modifyB.setText("Save");
        closeB.setText("Cancel");

        reloadB.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    try {
                        newNet = Utils.copyHash(oldNet);
                        if( lockedTm != null )
                            newNet.put("locked", lockedTm);

                        addrSpacesH = (Hashtable)newNet.get("address_spaces");
                        if( addrSpacesH == null ) {
                            addrSpacesH = new Hashtable();
                            newNet.put("address_spaces", addrSpacesH);
                        }

                        // reset name
                        Hashtable names = (Hashtable)newNet.get("names");
                        netNameTf.setText((String)(names.keys().nextElement()));

                        // reset groups
                        groups_panel.removeAll();
                        Hashtable groupsH = (Hashtable)newNet.get("groups");
                        if( groupsH == null ) {
                            groupsH = new Hashtable();
                            newNet.put("groups", groupsH);
                        }
                        groups_panel_layout.setRows(groupsH.size());
                        java.util.List list = Arrays.asList(groupsH.keySet().toArray());
                        Collections.sort(list, name_cmp);
                        Iterator it = list.iterator();
                        while( it.hasNext() ) {
                            String group = (String)it.next();
                            AJCheckBox cb = new AJCheckBox(group, true);
                            cb.setBackground(Color.white);
                            cb.addActionListener(new ActionListener() {
                                    public void actionPerformed(ActionEvent e) {
                                        AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                                        if( !cb.isSelected() ) {
                                            // remove associated data
                                            Hashtable groupsH = (Hashtable)newNet.get("groups");
                                            groups_panel.remove(cb);
                                            groupsH.remove((String)cb.object);
                                            groups_panel_layout.setRows(groups_panel_layout.getRows() - 1);
                                            groups_panel.doLayout();
                                            current_panel.revalidate();
                                        }
                                    }
                                });
                            cb.setFont(label_font);
                            groups_panel.add(cb);
                        }
                        groups_panel.revalidate();

                        // reset comment
                        String comment = (String)newNet.get("comment");
                        if( comment == null )
                            comment = "";
                        commentTa.setText(comment);

                        // reset dhcp service
                        Enumeration enm = dhcpServicesH.keys();
                        String dhcp_service = (String)newNet.get("dhcp_service");
                        if( dhcp_service == null )
                            dhcp_service = "---";
                        dhcpService.setSelectedItem(dhcp_service);

                        // reset dhcp options
                        dhcpOptionsH = (Hashtable)newNet.get("dhcp_options");
                        if( dhcpOptionsH == null ) {
                            dhcpOptionsH = new Hashtable();
                            newNet.put("dhcp_options", dhcpOptionsH);
                        }
                        String text = "";
                        list = Arrays.asList(dhcpOptionsH.keySet().toArray());
                        Collections.sort(list, name_cmp);
                        it = list.iterator();
                        while( it.hasNext() ) {
                            Hashtable optionH = (Hashtable)dhcpOptionsH.get(it.next());
                            String option = (String)optionH.get("option");
                            String value = (String)optionH.get("value");
                            text += option+"="+value+(it.hasNext()?"\n":"");
                        }
                        dhcpOptions.setText(text);

                        // first remove all panels
                        enm = asPanels.keys();
                        while( enm.hasMoreElements() ) {
                            Hashtable asData = (Hashtable)asPanels.get((String)enm.nextElement());
                            JPanel asPanel = (JPanel)asData.get("big_panel");
                            current_panel.remove(asPanel);
                        }
                        asPanels.clear();

                        // place them all back
                        current_gbc.gridx = separator_gridx;
                        current_gbc.gridy = separator_gridy;
                        java.util.List asIDs = Arrays.asList(addrSpacesH.keySet().toArray());
                        Collections.sort(asIDs, as_ip_cmp);
                        Object[] asIDsArray = asIDs.toArray();
                        for( int i = 0; i < asIDsArray.length; i++ ) {
                            String key = (String)asIDsArray[i];
                            Hashtable asData = makeASPanel(key, (Hashtable)addrSpacesH.get(key));
                            JPanel asPanel = (JPanel)asData.get("big_panel");
                            if( allB.getText().equals("Expand") )
                                collapseASPanel(asData);

                            current_gbc.gridwidth = 2;
                            current_gbc.fill = GridBagConstraints.NONE;
                            current_gbc.anchor = GridBagConstraints.CENTER;
                            asData.put("current_gbc.gridwidth", new Integer(current_gbc.gridwidth));
                            asData.put("current_gbc.fill", new Integer(current_gbc.fill));
                            asData.put("current_gbc.anchor", new Integer(current_gbc.anchor));
                            asData.put("gbc.gridx", new Integer(current_gbc.gridx));
                            asData.put("gbc.gridy", new Integer(current_gbc.gridy));
                            current_gb_layout.setConstraints(asPanel, current_gbc);
                            current_panel.add(asPanel);
                            current_gbc.gridwidth = 1;
                            current_gbc.fill = GridBagConstraints.HORIZONTAL;
                            tr();
                        }
                        current_panel.revalidate();

                    } catch(Exception ex) {
                        Utils.error(ex.getMessage());
                        return;
                    }
                }
            });

        deleteB.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    try {
                        pickUpData();
                        msg.setText("Saving network...");
                        mega_pane.removeAll();
                        mega_pane.add(msgPanel, BorderLayout.CENTER);
                        mega_pane.revalidate();
                        mega_pane.repaint();
                        reloadB.setEnabled(false);
                        modifyB.setEnabled(false);
                        deleteB.setEnabled(false);
                        header_panel.remove(ePanel);
                        ePanelLabel.setText("");
                        header_panel.revalidate();
                        (new NetworkThread(THIS, NetworkThread.SAVE_AND_CONTINUE, null,
                                           null, null)).start();
                    } catch(Exception ex) {
                        ePanelLabel.setText("<html><body> <font color=\"#ff0000\">&nbsp;Error: "+
                                            ex.getMessage()+"</font></body></html>");
                        header_panel.add(ePanel);
                        header_panel.revalidate();
                    }
                }
            });

        modifyB.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    try {
                        pickUpData();
                        msg.setText("Saving network...");
                        mega_pane.removeAll();
                        mega_pane.add(msgPanel, BorderLayout.CENTER);
                        mega_pane.revalidate();
                        mega_pane.repaint();
                        reloadB.setEnabled(false);
                        modifyB.setEnabled(false);
                        deleteB.setEnabled(false);
                        header_panel.remove(ePanel);
                        ePanelLabel.setText("");
                        header_panel.revalidate();
                        (new NetworkThread(THIS, NetworkThread.SAVE_NETWORK, null,
                                           null, null)).start();
                    } catch(Exception ex) {
                        ePanelLabel.setText("<html><body> <font color=\"#ff0000\">&nbsp;Error: "+
                                            ex.getMessage()+"</font></body></html>");
                        header_panel.add(ePanel);
                        header_panel.revalidate();
                    }
                }
            });

        ActionListener[] al = closeB.getActionListeners();
        for( int i = 0; i < al.length; i++ )
            closeB.removeActionListener(al[i]);
        closeB.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    if( lockedTm != null ) {
                        msg.setText("Unlocking network...");
                        mega_pane.removeAll();
                        mega_pane.add(msgPanel, BorderLayout.CENTER);
                        mega_pane.revalidate();
                        mega_pane.repaint();
                        reloadB.setEnabled(false);
                        modifyB.setEnabled(false);
                        deleteB.setEnabled(false);
                        if( header_panel != null && ePanel != null ) {
                            header_panel.remove(ePanel);
                            ePanelLabel.setText("");
                            header_panel.revalidate();
                        }
                        (new NetworkThread(THIS, NetworkThread.UNLOCK_NETWORK, null,
                                           null, null)).start();
                    }
                    THIS.dispose();
                }
            });
        mainPanel.setBackground(Color.white);

        this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        this.pack();
        this.setSize(900, 700);
        Dimension scrnsize = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation( (int)((scrnsize.getWidth()-getWidth())/2),
                     (int)((scrnsize.getHeight()-getHeight())/2) );
        central_panel.removeAll();
        msgPanel.setBackground(Color.white);
        msgPanel.add(msg, BorderLayout.CENTER);
        central_panel.add(msgPanel);
        this.show();

        (new Thread(this)).start();
    } // end of initFrame()


    public void dispose() {
        if( handle != null )
            netModFrames.remove(handle.toLowerCase());
        super.dispose();
    }

    public void run() {
        if( close_this ) {
            if( THIS != null )
                THIS.dispose();
            return;
        }
        try {
            msg.setText("Locking network...");
            if( handle != null && !template ) {
                if( NetworkMoveFrame.netMoveFrames.get(handle.toLowerCase()) != null ) {
                    msg.setText("You are holding lock on the same object in some other window.");
                    msg.setForeground(Color.red);
                    return;
                }
                lockedTm = MainWindow.netdb.lock(Constants.NETWORK, handle);
            }

            msg.setText("Loading network info...");
            if( handle == null ) {
                // network create
                oldNet = new Hashtable();
                oldNet.put("id", "");
                oldNet.put("object", "network");
                oldNet.put("groups", new Hashtable());
                oldNet.put("comment", "");
                oldNet.put("dhcp_options", new Hashtable());
                Hashtable namesH = new Hashtable();
                Hashtable nameH = new Hashtable();
                nameH.put("name", "");
                nameH.put("domain", "");
                namesH.put("", nameH);
                oldNet.put("names", namesH);
            } else {
                oldNet = MainWindow.netdb.getInfo(Constants.NETWORK, handle);
                // reset handle, we rely on handle been null to recognize that
                // it is insert not update
                if( template ) {
                    handle = null;
                    // we inherit only few parts of original network
                    oldNet.remove("address_spaces");
                    Hashtable oldNames = (Hashtable)oldNet.get("names");
                    oldNames.clear();
                    Hashtable nameH = new Hashtable();
                    oldNames.put("", nameH);
                    nameH.put("name", "");
                    nameH.put("domain", "");
                }
            }

            if( oldNet == null || oldNet.get("id") == null ) {
                msg.setText("Failed to load network. It probably does not exist.");
                msg.setForeground(Color.red);
                return;
            }

            // go over all names in oldNet and pack them in reserved_and_committed_names
            packNames((Hashtable)oldNet.get("names"));
            Hashtable ases = (Hashtable)oldNet.get("address_spaces");
            if( ases == null )
                ases = new Hashtable();
            Enumeration enm = ases.keys();
            while( enm.hasMoreElements() ) {
                Hashtable as = (Hashtable)ases.get((String)enm.nextElement());
                Hashtable dhcp_addresses = (Hashtable)as.get("dhcp_addresses");
                if( dhcp_addresses == null )
                    dhcp_addresses = new Hashtable();
                Enumeration enm1 = dhcp_addresses.keys();
                while( enm1.hasMoreElements() ) {
                    Hashtable ip = (Hashtable)dhcp_addresses.get((String)enm1.nextElement());
                    packNames((Hashtable)ip.get("names"));
                }
            }

            newNet = Utils.copyHash(oldNet);

            msg.setText("Verifying access...");
            if( !MainWindow.netdb.checkAccess(Constants.NETWORK) ||
                (handle != null && !MainWindow.netdb.checkGroupAccess(oldNet) ) )
                throw new Exception("You do not have sufficient access rights to modify this object");

            msg.setText("Loading list of groups...");
            allGroupsH = MainWindow.getGroups();
            // remove all groups which user is not a member of
            Boolean allGroupsUser = (Boolean)MainWindow.user.get("all_groups?");
            if( allGroupsUser == null || !allGroupsUser.booleanValue() ) {
                Hashtable userGroups = (Hashtable)MainWindow.user.get("groups");
                enm = allGroupsH.keys();
                while( enm.hasMoreElements() ) {
                    String group = (String)enm.nextElement();
                    if( userGroups.get(group) == null )
                        allGroupsH.remove(group);
                }
            }

            msg.setText("Loading list of dhcp services...");
            dhcpServicesH = MainWindow.getDHCPServices();
            dhcpService = new JComboBox();
            dhcpService.addItem("---");
            dhcpService.setSelectedIndex(0);
            enm = dhcpServicesH.keys();
            String dhcp_service = (String)newNet.get("dhcp_service");
            if( dhcp_service == null )
                dhcp_service = "";
            int index = 1;
            while( enm.hasMoreElements() ) {
                String service = (String)enm.nextElement();
                dhcpService.addItem(service);
                if( dhcp_service.equals(service) )
                    dhcpService.setSelectedIndex(index);
                index++;
            }
            msg.setText("Building interface...");

            // make up error/warnings panel
            ePanel = new JPanel(new BorderLayout());
            ePanel.setBackground(Color.yellow);
            JPanel closePanel = new JPanel(new BorderLayout());
            closePanel.setBackground(Color.yellow);
            JLabel closeLabel = new JLabel("x ");
            closeLabel.setFont(label_font);
            closeLabel.setCursor(new Cursor(Cursor.HAND_CURSOR));
            closeLabel.addMouseListener(new MouseAdapter() {
                public void mouseEntered(MouseEvent event) {
                    JLabel label = (JLabel)event.getComponent();
                    label.setFont(bold_font);
                    label.repaint();
                }
                public void mouseExited(MouseEvent event) {
                    JLabel label = (JLabel)event.getComponent();
                    label.setFont(label_font);
                    label.repaint();
                }
                public void mouseClicked(MouseEvent event) {
                    header_panel.remove(ePanel);
                    ePanelLabel.setText("");
                    header_panel.revalidate();
                }
            });
            closePanel.add(closeLabel, BorderLayout.NORTH);
            ePanel.add(closePanel, BorderLayout.EAST);
            ePanelLabel = new JLabel("");
            ePanel.add(ePanelLabel, BorderLayout.CENTER);

            if( lockedTm != null ) {
                oldNet.put("locked", lockedTm);
                newNet.put("locked", lockedTm);
            }

            newAddrSpacePrefTf = new JTextField("", 20);
            newAddrSpacePrefTf.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent event) {
                        ((JButton)newAddrSpaceButtons[0]).doClick();
                    }
                });
            newAddrSpaceButtons = new Object[2];
            newAddrSpaceButtons[0] = new JButton("Save");
            ((JButton)newAddrSpaceButtons[0]).addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent event) {
                        String asPrefix = newAddrSpacePrefTf.getText().trim();
                        // try to match ip address
                        if( !asPrefix.matches("(\\d|1?\\d\\d|2[0-4]\\d|25[0-5])\\.(\\d|1?\\d\\d|2[0-4]\\d|25[0-5])\\."+
                                              "(\\d|1?\\d\\d|2[0-4]\\d|25[0-5])\\.(\\d|1?\\d\\d|2[0-4]\\d|25[0-5])/(0?[1-9]|[1-2][0-9]|3[0-1])") ) {
                            Toolkit.getDefaultToolkit().beep();
                            newAddrSpacePrefTf.requestFocus();
                            return;
                        }
                        int prefixLength = Integer.parseInt(asPrefix.substring(asPrefix.indexOf("/")+1));

                        addAddrSpaceDialogWin.dispose();

                        // first remove all panels
                        Enumeration enm = asPanels.keys();
                        while( enm.hasMoreElements() ) {
                            Hashtable asData = (Hashtable)asPanels.get((String)enm.nextElement());
                            JPanel asPanel = (JPanel)asData.get("big_panel");
                            current_panel.remove(asPanel);
                        }

                        // add new address space
                        Hashtable as = new Hashtable();
                        addrSpacesH.put("AS"+(available_key++), as);
                        Integer ipAddrSpace = new Integer(Utils.ipstr2num(asPrefix));
                        as.put("addr_space_ip", ipAddrSpace);
                        as.put("prefix_length", new Integer(prefixLength));
                        as.put("addr_space_prefix", new String(asPrefix));
                        as.put("low_res", new Integer(ipAddrSpace.intValue()+1));
                        as.put("high_res", new Integer( ipAddrSpace.intValue() | (~(0xffffffff << (32 - prefixLength))) ));
                        as.put("comment", "");
                        as.put("groups", new Hashtable());
                        as.put("dhcp_addresses", new Hashtable());
                        as.put("dhcp_options", new Hashtable());
                        as.put("newAS", "");

                        // place them all back
                        current_gbc.gridx = separator_gridx;
                        current_gbc.gridy = separator_gridy;
                        java.util.List asIDs = Arrays.asList(addrSpacesH.keySet().toArray());
                        Collections.sort(asIDs, as_ip_cmp);
                        Object[] asIDsArray = asIDs.toArray();
                        for( int i = 0; i < asIDsArray.length; i++ ) {
                            Hashtable asData = (Hashtable)asPanels.get((String)asIDsArray[i]);
                            JPanel asPanel = null;
                            if( asData == null ) {
                                // create new panel
                                String key = (String)asIDsArray[i];
                                asData = makeASPanel(key, (Hashtable)addrSpacesH.get(key));
                                asPanel = (JPanel)asData.get("big_panel");
                            } else {
                                // get old panel from a hash
                                asPanel = (JPanel)asData.get("big_panel");
                            }

                            current_gbc.gridwidth = 2;
                            current_gbc.fill = GridBagConstraints.NONE;
                            current_gbc.anchor = GridBagConstraints.CENTER;
                            asData.put("current_gbc.gridwidth", new Integer(current_gbc.gridwidth));
                            asData.put("current_gbc.fill", new Integer(current_gbc.fill));
                            asData.put("current_gbc.anchor", new Integer(current_gbc.anchor));
                            asData.put("gbc.gridx", new Integer(current_gbc.gridx));
                            asData.put("gbc.gridy", new Integer(current_gbc.gridy));
                            current_gb_layout.setConstraints(asPanel, current_gbc);
                            current_panel.add(asPanel);
                            current_gbc.gridwidth = 1;
                            current_gbc.fill = GridBagConstraints.HORIZONTAL;
                            tr();
                        }
                        current_panel.revalidate();
                    }
                });

            newAddrSpaceButtons[1] = new JButton("Cancel");
            ((JButton)newAddrSpaceButtons[1]).addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent event) {
                        addAddrSpaceDialogWin.dispose();
                    }
                });

            current_panel = mainPanel;
            current_panel.setLayout(current_gb_layout);

            current_gbc.fill = GridBagConstraints.HORIZONTAL;
            current_gbc.weightx = 1.0;
            current_gbc.weighty = 0;
            current_gbc.gridx = 0;
            current_gbc.gridy = 0;

            // empty row at the top
            XLabel(" ");
            tr();

            HTMLPanel("Name <font color=\"#ff0000\">*</font>");
            td();

            // we assume here that net may have only one name
            Hashtable names = (Hashtable)newNet.get("names");
            enm = names.keys();
            if( !enm.hasMoreElements() )
                throw new Exception("Network does not have name. That does not make sense!!!");

            Hashtable name = (Hashtable)names.get((String)enm.nextElement());
            netNameTf = new JTextField(35);
            netNameTf.setFont(label_font);

            if( name.get("domain").toString().trim().toLowerCase().equals(MainWindow.userDefDomain) )
                netNameTf.setText((String)name.get("name"));
            else if( ((String)name.get("domain")).trim().length() == 0 && ((String)name.get("name")).trim().length() == 0 )
                netNameTf.setText("");
            else
                netNameTf.setText((String)name.get("name")+"."+(String)name.get("domain"));

            current_gbc.fill = GridBagConstraints.NONE;
            current_gbc.anchor = GridBagConstraints.WEST;
            current_gb_layout.setConstraints(netNameTf, current_gbc);
            current_panel.add(netNameTf);
            current_gbc.fill = GridBagConstraints.HORIZONTAL;
            current_gbc.anchor = GridBagConstraints.CENTER;
            tr();

            XLabel(" ");
            tr();

            makeMemberGroups();

            XLabel(" ");
            tr();

            current_gbc.anchor = GridBagConstraints.NORTHWEST;
            HTMLPanel("Comment");
            td();

            String comment = (String)newNet.get("comment");
            if( comment == null )
                comment = "";
            commentTa = new JTextArea(3, 35);
            commentTa.setFont(label_font);
            commentTa.setText(comment);
            commentTa.setEditable(true);
            JScrollPane sp = new JScrollPane(commentTa);
            current_gbc.fill = GridBagConstraints.NONE;
            current_gbc.anchor = GridBagConstraints.WEST;
            current_gb_layout.setConstraints(sp, current_gbc);
            current_panel.add(sp);
            current_gbc.fill = GridBagConstraints.HORIZONTAL;
            current_gbc.anchor = GridBagConstraints.CENTER;
            tr();

            XLabel(" ");
            tr();

            HTMLPanel("DHCP Service");
            td();

            current_gbc.fill = GridBagConstraints.NONE;
            current_gbc.anchor = GridBagConstraints.WEST;
            current_gb_layout.setConstraints(dhcpService, current_gbc);
            current_panel.add(dhcpService);
            current_gbc.fill = GridBagConstraints.HORIZONTAL;
            current_gbc.anchor = GridBagConstraints.CENTER;
            tr();
            dhcpService.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        String service = ((String)dhcpService.getSelectedItem()).trim();
                        if( service.equals("---") )
                            newNet.remove("dhcp_service");
                        else
                            newNet.put("dhcp_service", service);
                    }
                });

            XLabel(" ");
            tr();

            current_gbc.anchor = GridBagConstraints.NORTHWEST;
            HTMLPanel("DHCP Options");
            td();
            dhcpOptions = new JTextArea(3, 35);
            dhcpOptions.setFont(label_font);

            // make text
            dhcpOptionsH = (Hashtable)newNet.get("dhcp_options");
            if( dhcpOptionsH == null ) {
                dhcpOptionsH = new Hashtable();
                newNet.put("dhcp_options", dhcpOptionsH);
            }
            String text = "";

            java.util.List list = Arrays.asList(dhcpOptionsH.keySet().toArray());
            Collections.sort(list, name_cmp);
            Iterator it = list.iterator();
            while( it.hasNext() ) {
                Hashtable optionH = (Hashtable)dhcpOptionsH.get(it.next());
                String option = (String)optionH.get("option");
                String value = (String)optionH.get("value");
                text += option+"="+value+(it.hasNext()?"\n":"");
            }
            dhcpOptions.setText(text);
            dhcpOptions.setEditable(true);
            sp = new JScrollPane(dhcpOptions);
            current_gbc.fill = GridBagConstraints.NONE;
            current_gbc.anchor = GridBagConstraints.WEST;
            current_gb_layout.setConstraints(sp, current_gbc);
            current_panel.add(sp);
            current_gbc.fill = GridBagConstraints.HORIZONTAL;
            current_gbc.anchor = GridBagConstraints.CENTER;
            tr();

            XLabel(" ");
            tr();

            line();

            JPanel panel = new JPanel(new BorderLayout());
            panel.setBackground(new Color(178, 228, 234));
            JLabel label = new JLabel("<html><body>&nbsp;&nbsp;IP Address Spaces</body></html>");
            label.setFont(label_font);
            JPanel leftPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
            leftPanel.setBackground(new Color(178, 228, 234));
            leftPanel.add(label);

            allB = new JButton("Collapse");
            leftPanel.add(allB);
            allB.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent event) {
                        if( allB.getText().equals("Collapse") ) {
                            allB.setText("Expand");
                            Enumeration enm = asPanels.keys();
                            while( enm.hasMoreElements() ) {
                                Hashtable asData = (Hashtable)asPanels.get((String)enm.nextElement());
                                collapseASPanel(asData);
                            }
                        } else { // expand
                            allB.setText("Collapse");
                            Enumeration enm = asPanels.keys();
                            while( enm.hasMoreElements() ) {
                                Hashtable asData = (Hashtable)asPanels.get((String)enm.nextElement());
                                expandASPanel(asData);
                            }
                        }
                    }
                });

            panel.add(leftPanel, BorderLayout.WEST);
            addAddrSpaceB = new AJButton("Add'l", null, false);
            addAddrSpaceB.addActionListener(new AActionListener(this) {
                    public void actionPerformed(ActionEvent event) {
                        NetworkModifyFrame frame = (NetworkModifyFrame)parent;
                        JPanel panel = new JPanel(new GridLayout(2, 1));
                        JLabel label = new JLabel("Enter address space prefix");
                        label.setFont(label_font);
                        panel.add(label);
                        panel.add(newAddrSpacePrefTf);

                        JOptionPane addAddrSpaceDialog =
                            new JOptionPane(panel, JOptionPane.PLAIN_MESSAGE,
                                            JOptionPane.OK_CANCEL_OPTION, null,
                                            frame.newAddrSpaceButtons);
                        addAddrSpaceDialogWin =
                            addAddrSpaceDialog.createDialog(parent, "Add Address Space");
                        addAddrSpaceDialogWin.show();
                    }
                });
            JPanel rightPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
            rightPanel.setBackground(new Color(178, 228, 234));
            rightPanel.add(addAddrSpaceB);
            panel.add(rightPanel, BorderLayout.EAST);

            current_gbc.gridwidth = 2;
            current_gb_layout.setConstraints(panel, current_gbc);
            current_panel.add(panel);
            current_gbc.gridwidth = 1;
            tr();

            line();

            separator_gridx = current_gbc.gridx;
            separator_gridy = current_gbc.gridy;

            // now layout all address spaces
            asPanels = new Hashtable();
            addrSpacesH = (Hashtable)newNet.get("address_spaces");
            if( addrSpacesH == null ) {
                addrSpacesH = new Hashtable();
                newNet.put("address_spaces", addrSpacesH);
            }
            java.util.List asIDs = Arrays.asList(addrSpacesH.keySet().toArray());
            Collections.sort(asIDs, as_ip_cmp);
            Object[] asIDsArray = asIDs.toArray();
            for( int i = 0; i < asIDsArray.length; i++ ) {
                String id = (String)asIDsArray[i];
                Hashtable asData = makeASPanel(id, (Hashtable)addrSpacesH.get(id));
                JPanel asPanel = (JPanel)asData.get("big_panel");

                current_gbc.gridwidth = 2;
                current_gbc.fill = GridBagConstraints.NONE;
                current_gbc.anchor = GridBagConstraints.CENTER;
                asData.put("current_gbc.gridwidth", new Integer(current_gbc.gridwidth));
                asData.put("current_gbc.fill", new Integer(current_gbc.fill));
                asData.put("current_gbc.anchor", new Integer(current_gbc.anchor));
                asData.put("gbc.gridx", new Integer(current_gbc.gridx));
                asData.put("gbc.gridy", new Integer(current_gbc.gridy));

                current_gb_layout.setConstraints(asPanel, current_gbc);
                current_panel.add(asPanel);
                current_gbc.gridwidth = 1;
                current_gbc.fill = GridBagConstraints.HORIZONTAL;
                tr();
            }

            // empty row at the bottom
            XLabel(" ");
            tr();

            central_panel.removeAll();
            central_panel.revalidate();
            central_panel.add(current_panel);
            central_panel.revalidate();
            current_panel.revalidate();
            central_panel.repaint();

            modifyB.setEnabled(true);
            deleteB.setEnabled(true);
            reloadB.setEnabled(true);

        } catch(java.rmi.ServerException exx) {
            if( exx.getCause() instanceof SessionTimeoutException )
                X.handleTimeout();

            else if( exx.getCause() instanceof RemoteNetDBException ) {
                RemoteNetDBException ne = (RemoteNetDBException)exx.getCause();
                Vector errors = ne.getMessages();
                String[] msg = new String[errors.size()];
                for( int i = 0; i < errors.size(); i++ )
                    msg[i] = (String)errors.elementAt(i);
                JOptionPane.showMessageDialog(THIS,
                    msg, "Error", JOptionPane.ERROR_MESSAGE);

            } else if( exx.getCause() instanceof RemoteException ) {
                RemoteException re = (RemoteException)exx.getCause();
                JOptionPane.showMessageDialog(THIS,
                    re.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);

            } else {
                JOptionPane.showMessageDialog(THIS,
                    exx.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
            }
            closeB.doClick();
            return;

        } catch (java.rmi.NoSuchObjectException exx) {
            X.handleTimeout();

        } catch (java.rmi.MarshalException exx) {
            X.handleTimeout();

        } catch(Exception exx) {
            JOptionPane.showMessageDialog(THIS,
                    exx.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
            closeB.doClick();
            return;
        }

    } // end of run()

    public Hashtable makeASPanel(String id, Hashtable addrSpaceH) {
        Utils.debug("making as. panel id = "+id);
        Hashtable asData = new Hashtable();

        JPanel big_panel = new JPanel(new BorderLayout());
        big_panel.setBackground(Color.white);
        asData.put("window", THIS);
        asData.put("big_panel", big_panel);

        JLabel label = new JLabel(" ");
        big_panel.add(label, BorderLayout.NORTH);

        GridBagLayout layout = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();
        JPanel panel = new JPanel(layout);
        big_panel.add(panel, BorderLayout.CENTER);
        panel.setBackground(Color.white);
        asData.put("panel", panel);
        asData.put("id", id);
        asData.put("layout", layout);
        asData.put("gbc", gbc);
        asPanels.put(id, asData);

        // first row
        gbc.gridx = 0;
        gbc.fill = GridBagConstraints.NONE;
        gbc.anchor = GridBagConstraints.WEST;
        JPanel mini_panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 5));
        mini_panel.setBackground(Color.white);

        label = new AJLabel(MainWindow.downArrowIcon, asData);
        asData.put("expandLabel", label);
        label.addMouseListener(new MouseAdapter() {
                public void mouseEntered(MouseEvent event) {
                    setCursor(new Cursor(Cursor.HAND_CURSOR));
                }
                public void mouseExited(MouseEvent event) {
                    setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                }
                public void mouseClicked(MouseEvent event) {
                    AJLabel label = (AJLabel)event.getComponent();
                    String labelText = label.getText();
                    if( label.state )
                        collapseASPanel(label.box);
                    else
                        expandASPanel(label.box);
                }
            });
        label.setBorder(new EmptyBorder(0, 0, 0, 0));
        label.setBackground(Color.white);
        label.setFont(label_font);
        mini_panel.add(label);

        label = new JLabel(" Address Space    ");
        label.setBackground(Color.white);
        label.setFont(label_font);
        mini_panel.add(label);
        layout.setConstraints(mini_panel, gbc);
        panel.add(mini_panel);
        gbc.gridx++;

        label = new JLabel("Low Res.    ");
        label.setBackground(Color.white);
        label.setFont(label_font);
        layout.setConstraints(label, gbc);
        panel.add(label);
        gbc.gridx++;

        label = new JLabel("Hi Res.    ");
        label.setBackground(Color.white);
        label.setFont(label_font);
        layout.setConstraints(label, gbc);
        panel.add(label);
        gbc.gridx++;

        label = new JLabel(" ");
        label.setBackground(Color.white);
        label.setFont(label_font);
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        layout.setConstraints(label, gbc);
        panel.add(label);
        gbc.gridx = 0;
        gbc.gridwidth = 1;

        // second row
        int ipInteger = ((Integer)addrSpaceH.get("addr_space_ip")).intValue();
        String ipStr = Utils.ip2str(ipInteger);
        int prefixLength = ((Integer)addrSpaceH.get("prefix_length")).intValue();
        int prefixMask =  ~(0xffffffff << (32 - prefixLength));
        String prefix = ipStr+"/"+prefixLength;
        AJCheckBox asCB = new AJCheckBox(prefix, true, asData);
        asCB.addActionListener( new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                    if( cb.isSelected() ) return;
                    Hashtable asData = cb.box;
                    current_panel.remove((JPanel)cb.get("big_panel"));

                    int saved_gridwidth = current_gbc.gridwidth;
                    current_gbc.gridwidth       = ((Integer)asData.get("current_gbc.gridwidth")).intValue();

                    int saved_fill = current_gbc.fill;
                    current_gbc.fill            = ((Integer)asData.get("current_gbc.fill")).intValue();

                    int saved_anchor = current_gbc.anchor;
                    current_gbc.anchor          = ((Integer)asData.get("current_gbc.anchor")).intValue();

                    int saved_gridx = current_gbc.gridx;
                    current_gbc.gridx           = ((Integer)asData.get("gbc.gridx")).intValue();

                    int saved_gridy = current_gbc.gridy;
                    current_gbc.gridy           = ((Integer)asData.get("gbc.gridy")).intValue();

                    addrSpacesH.remove((String)asData.get("id"));

                    JPanel dummy_panel = new JPanel();
                    asData.put("dummy_panel", dummy_panel);
                    dummy_panel.setBackground(Color.white);

                    current_gb_layout.setConstraints(dummy_panel, current_gbc);
                    current_panel.add(dummy_panel);

                    current_gbc.gridwidth = saved_gridwidth;
                    saved_fill = current_gbc.fill;
                    current_gbc.anchor = saved_anchor;
                    current_gbc.gridx = saved_gridx;
                    current_gbc.gridy = saved_gridy;

                    current_panel.revalidate();
                }
            });
        asCB.setBackground(Color.white);
        layout.setConstraints(asCB, gbc);
        panel.add(asCB);
        gbc.gridx++;

        int lowRes = ((Integer)addrSpaceH.get("low_res")).intValue() - ipInteger;
        JTextField lowResTf = new JTextField(lowRes+"", 6);
        asData.put("lowResTf", lowResTf);
        lowResTf.setFont(label_font);
        layout.setConstraints(lowResTf, gbc);
        panel.add(lowResTf);
        gbc.gridx++;

        int highRes = ( ipInteger | prefixMask ) -
            ((Integer)addrSpaceH.get("high_res")).intValue();
        JTextField highResTf = new JTextField(highRes+"", 6);
        asData.put("highResTf", highResTf);
        highResTf.setFont(label_font);
        layout.setConstraints(highResTf, gbc);
        panel.add(highResTf);
        gbc.gridx++;

        JPanel xpanel = new JPanel();
        xpanel.setBackground(Color.white);
        JTextField splitPrefixTf = new JTextField("", 3);
        splitPrefixTf.setFont(label_font);
        xpanel.add(splitPrefixTf);
        AJButton split_join_b = new AJButton("Split/Join", asData, false);
        split_join_b.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    Hashtable as = (Hashtable)addrSpacesH.get((String)(((AJButton)event.getSource()).box).get("id"));
                    new AddrSpaceSplitJoinDialog(THIS, (String)as.get("addr_space_prefix"));
                }
            });
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        layout.setConstraints(split_join_b, gbc);
        panel.add(split_join_b);
        gbc.gridx = 0;
        gbc.gridwidth = 1;

        // third row
        gbc.anchor = GridBagConstraints.NORTHWEST;
        JLabel groupsLabel = new JLabel("Groups");
        asData.put("groupsLabel", groupsLabel);
        groupsLabel.setBackground(Color.white);
        groupsLabel.setFont(label_font);
        layout.setConstraints(groupsLabel, gbc);
        panel.add(groupsLabel);
        gbc.gridx++;

        makeASGroupList(layout, gbc, panel, asData);

        // forth row
        JLabel dhcpLabel = new JLabel("DHCP Options");
        asData.put("dhcpLabel", dhcpLabel);
        dhcpLabel.setBackground(Color.white);
        dhcpLabel.setFont(label_font);
        layout.setConstraints(dhcpLabel, gbc);
        panel.add(dhcpLabel);
        gbc.gridx++;

        JTextArea dhcpOptionsTA = new JTextArea(3, 35);
        asData.put("dhcpOptionsTA", dhcpOptionsTA);
        dhcpOptionsTA.setFont(label_font);
        // make up text
        dhcpOptionsH = (Hashtable)addrSpaceH.get("dhcp_options");
        if( dhcpOptionsH == null ) {
            dhcpOptionsH = new Hashtable();
                addrSpaceH.put("dhcp_options", dhcpOptionsH);
        }
        String text = "";
        java.util.List list = Arrays.asList(dhcpOptionsH.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();
        while( it.hasNext() ) {
            Hashtable optionH = (Hashtable)dhcpOptionsH.get(it.next());
            String option = (String)optionH.get("option");
            String value = (String)optionH.get("value");
            text += option+"="+value+(it.hasNext()?"\n":"");
        }
        dhcpOptionsTA.setText(text);
        dhcpOptionsTA.setEditable(true);
        JScrollPane sp = new JScrollPane(dhcpOptionsTA);
        asData.put("dhcpScrollPane", sp);
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        layout.setConstraints(sp, gbc);
        panel.add(sp);
        gbc.gridx = 0;
        gbc.gridwidth = 1;

        // fifth row (empty)
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        JLabel emptyLabel = new JLabel(" ");
        asData.put("emptyLabel", emptyLabel);
        emptyLabel.setBackground(Color.white);
        layout.setConstraints(emptyLabel, gbc);
        panel.add(emptyLabel);
        gbc.gridwidth = 1;

        // sixes row
        label = new JLabel("Comment");
        asData.put("commentLabel", label);
        label.setBackground(Color.white);
        label.setFont(label_font);
        layout.setConstraints(label, gbc);
        panel.add(label);
        gbc.gridx++;

        String comment = (String)addrSpaceH.get("comment");
        if( comment == null ) {
            comment = "";
            addrSpaceH.put("comment", comment);
        }
        AJTextField commentTf = new AJTextField(comment, 35, asData, null);
        commentTf.addFocusListener(new FocusAdapter() {
                    public void focusLost(FocusEvent e) {
                        AJTextField tf = (AJTextField)e.getSource();
                        String txt = tf.getText();
                        Hashtable as = (Hashtable)addrSpacesH.get((String)tf.box.get("id"));
                        as.put("comment", txt);
                        //if( txt != null &&
                        //    ( txt.indexOf("<") > -1 || txt.indexOf(">") > -1 ) ) {
                        //    JOptionPane.showMessageDialog(THIS,
                        //       "Comment field may not contain '<' or '>' characters.",
                        //       "Error", JOptionPane.ERROR_MESSAGE);
                        //    ((JTextField)e.getSource()).requestFocus();
                        //}
                    }
                });
        asData.put("commentTf", commentTf);
        commentTf.setFont(label_font);
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        layout.setConstraints(commentTf, gbc);
        panel.add(commentTf);
        gbc.gridx = 0;
        gbc.gridwidth = 1;

        // seventh row
        Hashtable as = (Hashtable)addrSpacesH.get((String)asData.get("id"));
        label = new JLabel("Dynamic DHCP addresses");
        asData.put("dynamicLabel", label);
        label.setBackground(Color.white);
        label.setFont(label_font);
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        layout.setConstraints(label, gbc);
        panel.add(label);
        gbc.gridwidth = 1;

        makeDHCPAddresses(layout, gbc, panel, asData);

        JPanel bottom_panel = new JPanel(new BorderLayout());
        bottom_panel.setBackground(Color.white);
        bottom_panel.add(new JLabel(" "), BorderLayout.CENTER);
        bottom_panel.add(new JSeparator(), BorderLayout.SOUTH);
        big_panel.add(bottom_panel, BorderLayout.SOUTH);

        return asData;

        //label = XLabel(" ");
        //asData.put("bottom_label", label);
        //tr();

        //asData.put("line", line());
    } // end of makeASPanel(...)

    private void makeDHCPAddresses(GridBagLayout layout,
                                   GridBagConstraints gbc,
                                   JPanel panel,
                                   Hashtable asData) {
        Hashtable as = (Hashtable)addrSpacesH.get((String)asData.get("id"));
        JPanel dhcp_big_panel = new JPanel(new BorderLayout());
        asData.put("dhcp_big_panel", dhcp_big_panel);
        dhcp_big_panel.setBackground(Color.white);
        JPanel dhcp_ip_panel = new JPanel();
        asData.put("dhcp_ip_panel", dhcp_ip_panel);
        dhcp_ip_panel.setBackground(Color.white);

        Hashtable dhcp_addresses = (Hashtable)as.get("dhcp_addresses");
        if( dhcp_addresses == null )
            dhcp_addresses = new Hashtable();

        Hashtable dhcp_addressesH = new Hashtable();
        asData.put("dhcp_addressesH", dhcp_addressesH);

        GridLayout dhcp_ip_panel_layout = new GridLayout(dhcp_addresses.size(), 1, 0, 2);
        asData.put("dhcp_ip_panel_layout", dhcp_ip_panel_layout);
        dhcp_ip_panel.setLayout(dhcp_ip_panel_layout);

        java.util.List list = Arrays.asList(dhcp_addresses.keySet().toArray());
        Collections.sort(list, ip_cmp);
        Iterator it = list.iterator();
        while( it.hasNext() ) {

            String ipStr = (String)it.next();
            Hashtable addr = (Hashtable)dhcp_addresses.get(ipStr);
            Hashtable ipH = new Hashtable();
            dhcp_addressesH.put(ipStr, ipH);

            JPanel ip_panel = new JPanel();
            ip_panel.setBackground(Color.white);
            dhcp_ip_panel.add(ip_panel);
            ipH.put("ip_panel", ip_panel);

            AJCheckBox cb = new AJCheckBox("", true, asData);
            cb.obj2 = ipStr;
            cb.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                        if( cb.isSelected() ) return;

                        ((Hashtable)((Hashtable)addrSpacesH.get((String)cb.box.get("id"))).
                         get("dhcp_addresses")).remove((String)cb.obj2);

                        JPanel dhcp_ip_panel = (JPanel)cb.box.get("dhcp_ip_panel");
                        Hashtable ipH = (Hashtable)((Hashtable)cb.box.get("dhcp_addressesH")).
                            remove((String)cb.obj2);
                        dhcp_ip_panel.remove((JPanel)ipH.get("ip_panel"));

                        GridLayout dhcp_ip_panel_layout = (GridLayout)cb.box.get("dhcp_ip_panel_layout");
                        dhcp_ip_panel_layout.setRows(dhcp_ip_panel_layout.getRows() - 1);

                        dhcp_ip_panel.revalidate();
                    }
                });
            cb.setBackground(Color.white);
            cb.setFont(label_font);
            ip_panel.add(cb);

            AJFormattedTextField ftf = new AJFormattedTextField((String)addr.get("ip_address"), 16, null, asData, ipStr);
            //ftf.setInputVerifier(new GenericInputVerifier());
            ftf.addFocusListener(new FocusAdapter() {
                    public void focusLost(FocusEvent e) {
                        doAction((AJFormattedTextField)e.getSource());
                    }
                    public void doAction(AJFormattedTextField ftf) {
                        Hashtable as = (Hashtable)addrSpacesH.get((String)ftf.box.get("id"));
                        Hashtable dhcpIP = (Hashtable)((Hashtable)as.get("dhcp_addresses")).get(ftf.key);
                        if( dhcpIP != null ) {
                            String oldIP = (String)dhcpIP.get("ip_address");
                            String newIP = ftf.getText().trim();
                            if( !oldIP.equals(newIP) ) {
                                dhcpIP.put("ip_address", newIP);
                                dhcpIP.put("dirty_ip", "");
                                dhcpIP.put("dirty", "");
                            }
                        }
                    }
                });

            ipH.put("ip", ftf);
            ftf.setEditable(true);
            ftf.setFont(label_font);
            ip_panel.add(ftf);

            label = new JLabel("  Name");
            label.setFont(label_font);
            ip_panel.add(label);

            Hashtable names = (Hashtable)addr.get("names");
            if( names == null ) {
                names = new Hashtable();
                addr.put("names", names);
            }
            String name = "";
            Enumeration enm = names.keys();
            if( enm.hasMoreElements() ) {
                Hashtable nameH = (Hashtable)names.get((String)enm.nextElement());
                name = (String)nameH.get("name");
                String domain = (String)nameH.get("domain");
                if( !domain.toLowerCase().trim().equals(MainWindow.userDefDomain) )
                    name += "."+domain;
            }

            AJTextField tf = new AJTextField(name, 17, asData, ipStr);
            tf.addFocusListener(new FocusAdapter() {
                    public void focusLost(FocusEvent e) {
                        doAction((AJTextField)e.getSource());
                    }
                    public void doAction(AJTextField tf) {
                        Hashtable as = (Hashtable)addrSpacesH.get((String)tf.box.get("id"));
                        Hashtable dhcpIP = (Hashtable)((Hashtable)as.get("dhcp_addresses")).get(tf.key);
                        Hashtable names = (Hashtable)dhcpIP.get("names");
                        if( names == null ) {
                            names = new Hashtable();
                            dhcpIP.put("names", names);
                        }
                        Enumeration enm = names.keys();
                        Hashtable name;
                        String key;
                        if( enm.hasMoreElements() ) {
                            Object obj = enm.nextElement();
                            Utils.debug(obj.getClass().getName());
                            key = (String)obj;
                            name = (Hashtable)names.get(key);
                        } else {
                            name = new Hashtable();
                            key = "name "+(available_key++);
                            names.put(key, name);
                            name.put("name", "");
                            name.put("domain", "");
                        }
                        String oldName = ((String)name.get("name")).trim()+"."+((String)name.get("domain")).trim();
                        String newName = tf.getText().trim();
                        String nameStr;
                        String domainStr;
                        if( !oldName.equals(newName) ) {
                            int dotIndex = newName.indexOf(".");
                            if( dotIndex < 0 ) {
                                nameStr = newName;
                                domainStr = MainWindow.userDefDomain;
                                if( newName.length() == 0 ) {
                                    names.remove(key);
                                    return;
                                }
                            } else {
                                nameStr = newName.substring(0, dotIndex);
                                domainStr = newName.substring(dotIndex+1);
                            }

                            name.put("name", nameStr);
                            name.put("domain", domainStr);
                            name.put("dirty_name", "");
                            dhcpIP.put("dirty", "");
                        }
                    }
                });
            ipH.put("name", tf);
            tf.setEditable(true);
            tf.setFont(label_font);
            ip_panel.add(tf);

            Boolean active = (Boolean)addr.get("active?");
            if( active == null )
                active = new Boolean(false);
            AJCheckBox activeCB = new AJCheckBox("Active", active.booleanValue(), asData);
            activeCB.obj2 = addr;
            activeCB.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                        if( cb.isSelected() )
                            ((Hashtable)(cb.obj2)).put("active?", new Boolean(true));
                        else
                            ((Hashtable)(cb.obj2)).put("active?", new Boolean(false));
                    }
                });
            activeCB.setBackground(Color.white);
            activeCB.setFont(label_font);
            ip_panel.add(activeCB);
        }

        dhcp_big_panel.add(dhcp_ip_panel, BorderLayout.CENTER);

        // place empty entry at the bottom
        JPanel bottom_panel = new JPanel(new BorderLayout());
        bottom_panel.setBackground(Color.white);
        dhcp_big_panel.add(bottom_panel, BorderLayout.SOUTH);

        JPanel panelP = new JPanel(new FlowLayout(FlowLayout.LEFT));
        panelP.setBackground(Color.white);

        label = new JLabel("  Address");
        label.setFont(label_font);
        panelP.add(label);
        JTextField newIpTf = new JTextField("", 16);
        asData.put("newIpTf", newIpTf);
        newIpTf.setBackground(Color.white);
        if( as.get("not_editable") != null || as.get("newAS") != null ) {
            newIpTf.setEditable(false);
            newIpTf.setBackground(new Color(235, 235, 235));
        }
        newIpTf.setFont(label_font);
        panelP.add(newIpTf);
        label = new JLabel("  Count");
        label.setFont(label_font);
        panelP.add(label);
        JTextField newCountTf = new JTextField("", 4);
        asData.put("newCountTf", newCountTf);
        newCountTf.setBackground(Color.white);
        if( as.get("not_editable") != null || as.get("newAS") != null ) {
            newCountTf.setEditable(false);
            newCountTf.setBackground(new Color(235, 235, 235));
        }
        newCountTf.setFont(label_font);
        panelP.add(newCountTf);

        bottom_panel.add(panelP, BorderLayout.CENTER);

        if( as.get("not_editable") == null && as.get("newAS") == null ) {
            //panelP = new JPanel(new FlowLayout(FlowLayout.LEFT));
            //panelP.setBackground(Color.white);
            AJButton verifyB = new AJButton("Verify", asData, false);
            verifyB.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        // first we better to make sure that there is something to verify
                        Hashtable asData = ((AJButton)e.getSource()).box;
                        StringBuffer errMsgB = new StringBuffer("");
                        boolean needs_verification = needVerify(asData, errMsgB);
                        String errMsg = new String(errMsgB);
                        boolean error = false;
                        if( errMsg.length() > 0 )
                            error = true;
                        if( needs_verification && !error ) {
                            // now proceed to verification
                            msg.setText("Verifying ip addresses");
                            mega_pane.removeAll();
                            mega_pane.add(msgPanel, BorderLayout.CENTER);
                            mega_pane.revalidate();
                            mega_pane.repaint();
                            reloadB.setEnabled(false);
                            modifyB.setEnabled(false);
                            deleteB.setEnabled(false);
                            header_panel.remove(ePanel);
                            ePanelLabel.setText("");
                            header_panel.revalidate();
                            (new NetworkThread(THIS, NetworkThread.VERIFY_NAME_AND_IP, asData,
                                               null, (AJButton)e.getSource())).start();
                        }

                        if( error ) {
                            ePanelLabel.setText(errMsg);
                            header_panel.add(ePanel);
                            header_panel.revalidate();
                        }
                    }
                });
            panelP.add(verifyB);
            //bottom_panel.add(panelP, BorderLayout.SOUTH);
        }

        gbc.gridwidth = GridBagConstraints.REMAINDER;
        layout.setConstraints(dhcp_big_panel, gbc);
        panel.add(dhcp_big_panel);
        gbc.gridx = 0;
        gbc.gridwidth = 1;
    } // end of makeDHCPAddresses(...)

    private void makeASGroupList(GridBagLayout layout,
                                 GridBagConstraints gbc,
                                 JPanel panel,
                                 Hashtable asData) {
        Hashtable as = (Hashtable)addrSpacesH.get((String)asData.get("id"));
        JPanel groups_big_panel = new JPanel(new BorderLayout());
        asData.put("groups_big_panel", groups_big_panel);
        groups_big_panel.setBackground(Color.white);
        JPanel groups_panel = new JPanel();
        asData.put("groups_panel", groups_panel);
        groups_panel.setBackground(Color.white);

        JLabel label = new JLabel("");
        label.setBackground(Color.white);
        label.setFont(label_font);
        groups_big_panel.add(label, BorderLayout.NORTH);
        Hashtable groupsH = (Hashtable)as.get("groups");
        if( groupsH == null ) {
            groupsH = new Hashtable();
            as.put("groups", groupsH);
        }
        GridLayout groups_panel_layout = new GridLayout(groupsH.size(), 1, 0, 2);
        asData.put("groups_panel_layout", groups_panel_layout);
        groups_panel.setLayout(groups_panel_layout);

        java.util.List list = Arrays.asList(groupsH.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();
        while( it.hasNext() ) {
            String group = (String)it.next();
            AJCheckBox cb = new AJCheckBox(group, true);
            cb.obj2 = as;
            cb.obj3 = groups_panel;
            cb.obj4 = groups_panel_layout;
            cb.setBackground(Color.white);
            cb.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                    if( !cb.isSelected() ) {
                        // remove associated data
                        Hashtable groupsH = (Hashtable)((Hashtable)cb.obj2).get("groups");
                        ((JPanel)cb.obj3).remove(cb);
                        groupsH.remove((String)cb.object);
                        ((GridLayout)cb.obj4).setRows(((GridLayout)cb.obj4).getRows() - 1);
                        ((JPanel)cb.obj3).doLayout();
                        current_panel.revalidate();
                    }
                }
            });
            cb.setFont(label_font);
            groups_panel.add(cb);
        }
        groups_big_panel.add(groups_panel, BorderLayout.CENTER);


        AJList groups_list = new AJList(asData);
        asData.put("groups_list", groups_list);

        AJButton add_another_groupB = new AJButton("Add more", asData);
        add_another_groupB.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        Hashtable asData = ((AJButton)e.getSource()).box;
                        JPanel bottom_groups_panel = (JPanel)asData.get("bottom_groups_panel");
                        Hashtable as = (Hashtable)addrSpacesH.get((String)asData.get("id"));
                        bottom_groups_panel.removeAll();
                        Hashtable groupsH = (Hashtable)as.get("groups");
                        if( groupsH == null )
                            groupsH = new Hashtable();
                        Hashtable groupsL = Utils.copyHash(allGroupsH);
                        // remove groups already in a list
                        Enumeration enm = groupsH.keys();
                        while( enm.hasMoreElements() ) {
                            String group = ((String)enm.nextElement()).trim();
                            groupsL.remove(group);
                        }

                        String[] groups = new String[groupsL.size()];
                        java.util.List list = Arrays.asList(groupsL.keySet().toArray());
                        Collections.sort(list, name_cmp);
                        Iterator it = list.iterator();
                        int counter = 0;
                        while( it.hasNext() )
                            groups[counter++] = (String)it.next();

                        AJList groups_list = (AJList)asData.get("groups_list");
                        groups_list.setListData(groups);
                        JScrollPane groups_sp = (JScrollPane)asData.get("groups_sp");
                        if( groups_sp == null ) {
                            groups_sp = new JScrollPane(groups_list);
                            asData.put("groups_sp", groups_sp);
                        }
                        bottom_groups_panel.add(groups_sp);
                        JPanel groups_big_panel = (JPanel)asData.get("groups_big_panel");
                        groups_list.requestFocus();
                        groups_big_panel.doLayout();
                        current_panel.revalidate();
                        bottom_groups_panel.revalidate();
                    } catch (Exception ex) {
                        Utils.warn(ex.getMessage());
                    }
                }
            });
        asData.put("add_another_groupB", add_another_groupB);
        add_another_groupB.setMargin(new Insets(0, 0, 0, 0));
        JPanel bottom_groups_panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        asData.put("bottom_groups_panel", bottom_groups_panel);
        bottom_groups_panel.add(add_another_groupB);
        bottom_groups_panel.setBackground(Color.white);
        groups_big_panel.add(bottom_groups_panel, BorderLayout.SOUTH);

        groups_list.addFocusListener(new FocusAdapter() {
                public void focusLost(FocusEvent e) {
                    Hashtable asData = ((AJList)e.getSource()).box;
                    JPanel bottom_groups_panel = (JPanel)asData.get("bottom_groups_panel");
                    bottom_groups_panel.removeAll();
                    bottom_groups_panel.add((AJButton)asData.get("add_another_groupB"));
                    ((JPanel)asData.get("groups_big_panel")).doLayout();
                    current_panel.revalidate();
                }
            });

        // double click should work the same as pressing Enter
        groups_list.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    if( e.getClickCount() > 1 ) {
                        JList list = (JList)e.getComponent();
                        list.dispatchEvent(new KeyEvent(list,
                                                        KeyEvent.KEY_PRESSED,
                                                        Calendar.getInstance().getTimeInMillis(),
                                                        0, KeyEvent.VK_ENTER,
                                                        KeyEvent.CHAR_UNDEFINED));
                    }
                }
            });

        groups_list.addKeyListener(new KeyAdapter() {
                public void keyPressed(KeyEvent e) {
                    Hashtable asData = ((AJList)e.getSource()).box;
                    JPanel bottom_groups_panel = (JPanel)asData.get("bottom_groups_panel");
                    Hashtable as = (Hashtable)addrSpacesH.get((String)asData.get("id"));
                    JPanel groups_panel = null;
                    int key = e.getKeyCode();
                    if( key == KeyEvent.VK_ESCAPE || key == KeyEvent.VK_ENTER ) {
                        groups_panel = (JPanel)asData.get("groups_panel");
                        bottom_groups_panel.removeAll();
                        bottom_groups_panel.add((AJButton)asData.get("add_another_groupB"));
                    }
                    if( key == KeyEvent.VK_ENTER ) {
                        AJList groups_list = (AJList)asData.get("groups_list");
                        Object[] addedGroups = groups_list.getSelectedValues();
                        Hashtable groupsH = (Hashtable)as.get("groups");
                        if( groupsH == null ) {
                            groupsH = new Hashtable();
                            as.put("groups", groupsH);
                        }
                        for( int i = 0; i < addedGroups.length; i++ )
                            groupsH.put(((String)addedGroups[i]).trim(), "");
                        // now rebuild liste of groups
                        groups_panel.removeAll();
                        GridLayout groups_panel_layout = (GridLayout)asData.get("groups_panel_layout");
                        groups_panel_layout.setRows(groupsH.size());

                        java.util.List list = Arrays.asList(groupsH.keySet().toArray());
                        Collections.sort(list, name_cmp);
                        Iterator it = list.iterator();
                        while( it.hasNext() ) {
                            String group = (String)it.next();
                            AJCheckBox cb = new AJCheckBox(group, true);
                            cb.obj2 = as;
                            cb.obj3 = groups_panel;
                            cb.obj4 = groups_panel_layout;
                            cb.setBackground(Color.white);
                            cb.addActionListener(new ActionListener() {
                                    public void actionPerformed(ActionEvent e) {
                                        AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                                        if( !cb.isSelected() ) {
                                            // remove associated data
                                            Hashtable groupsH = (Hashtable)((Hashtable)cb.obj2).get("groups");
                                            ((JPanel)cb.obj3).remove(cb);
                                            groupsH.remove((String)cb.object);
                                            ((GridLayout)cb.obj4).setRows(((GridLayout)cb.obj4).getRows() - 1);
                                            ((JPanel)cb.obj3).doLayout();
                                            current_panel.revalidate();
                                        }
                                    }
                                });
                            cb.setFont(label_font);
                            groups_panel.add(cb);
                        }
                    }
                    if( key == KeyEvent.VK_ESCAPE || key == KeyEvent.VK_ENTER ) {
                        groups_panel.doLayout();
                        current_panel.revalidate();
                    }
                }
            });

        // now place whole stuff on the screen
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        layout.setConstraints(groups_big_panel, gbc);
        panel.add(groups_big_panel);
        gbc.gridx = 0;
        gbc.gridwidth = 1;
    } // end of makeASGroupList(...)

    private void makeMemberGroups() {
        current_gbc.anchor = GridBagConstraints.NORTHWEST;
        HTMLPanel("Groups <font color=\"#ff0000\">*</font>");
        td();

        mgroups_big_panel = new JPanel(new BorderLayout());
        mgroups_big_panel.setBackground(Color.white);
        groups_panel = new JPanel();
        groups_panel.setBackground(Color.white);

        add_another_group = new JButton("Add more", null);
        add_another_group.setMargin(new Insets(0, 0, 0, 0));
        bottom_groups_panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        bottom_groups_panel.add(add_another_group);
        bottom_groups_panel.setBackground(Color.white);
        mgroups_big_panel.add(bottom_groups_panel, BorderLayout.NORTH);

        Hashtable groupsH = (Hashtable)newNet.get("groups");
        if( groupsH == null ) {
            groupsH = new Hashtable();
            newNet.put("groups", groupsH);
        }
        groups_panel_layout = new GridLayout(groupsH.size(), 1, 0, 2);
        groups_panel.setLayout(groups_panel_layout);
        java.util.List list = Arrays.asList(groupsH.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();
        while( it.hasNext() ) {
            String group = (String)it.next();
            AJCheckBox cb = new AJCheckBox(group, true);
            cb.setFont(label_font);
            //cb.setFont(label_font);
            cb.setBackground(Color.white);
            cb.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                    if( !cb.isSelected() ) {
                        // remove associated data
                        Hashtable groupsH = (Hashtable)newNet.get("groups");
                        groups_panel.remove(cb);
                        groupsH.remove((String)cb.object);
                        groups_panel_layout.setRows(groups_panel_layout.getRows() - 1);
                        //groups_panel.remove(cb);
                        groups_panel.doLayout();
                        current_panel.revalidate();
                    }
                }
            });
            cb.setFont(label_font);
            groups_panel.add(cb);
        }
        mgroups_big_panel.add(groups_panel, BorderLayout.CENTER);

        groups_list = new AJList();
        groups_sp = new JScrollPane(groups_list);
        add_another_group.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        bottom_groups_panel.removeAll();
                        Hashtable groupsH = (Hashtable)newNet.get("groups");
                        if( groupsH == null )
                            groupsH = new Hashtable();
                        Hashtable groupsL = Utils.copyHash(allGroupsH);
                        // remove groups already in a list
                        Enumeration enm = groupsH.keys();
                        while( enm.hasMoreElements() ) {
                            String group = ((String)enm.nextElement()).trim();
                            groupsL.remove(group);
                        }

                        String[] groups = new String[groupsL.size()];
                        java.util.List list = Arrays.asList(groupsL.keySet().toArray());
                        Collections.sort(list, name_cmp);
                        Iterator it = list.iterator();
                        int counter = 0;
                        while( it.hasNext() )
                            groups[counter++] = (String)it.next();

                        groups_list.setListData(groups);
                        bottom_groups_panel.add(groups_sp);
                        mgroups_big_panel.doLayout();
                        groups_list.requestFocus();
                        current_panel.revalidate();
                    } catch (Exception ex) {
                        Utils.warn(ex.getMessage());
                    }
                }
            });

        groups_list.addFocusListener(new FocusAdapter() {
                public void focusLost(FocusEvent e) {
                    bottom_groups_panel.removeAll();
                    bottom_groups_panel.add(add_another_group);
                    mgroups_big_panel.doLayout();
                    current_panel.revalidate();
                }
            });

        // double click should work the same as pressing Enter
        groups_list.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    if( e.getClickCount() > 1 ) {
                        JList list = (JList)e.getComponent();
                        list.dispatchEvent(new KeyEvent(list,
                                                        KeyEvent.KEY_PRESSED,
                                                        Calendar.getInstance().getTimeInMillis(),
                                                        0, KeyEvent.VK_ENTER,
                                                        KeyEvent.CHAR_UNDEFINED));
                    }
                }
            });

        groups_list.addKeyListener(new KeyAdapter() {
                public void keyPressed(KeyEvent e) {
                    int key = e.getKeyCode();
                    if( key == KeyEvent.VK_ESCAPE || key == KeyEvent.VK_ENTER ) {
                        bottom_groups_panel.removeAll();
                        bottom_groups_panel.add(add_another_group);
                    }
                    if( key == KeyEvent.VK_ENTER ) {
                        Object[] addedGroups = groups_list.getSelectedValues();
                        Hashtable groupsH = (Hashtable)newNet.get("groups");
                        if( groupsH == null ) {
                            groupsH = new Hashtable();
                            newNet.put("groups", groupsH);
                        }
                        for( int i = 0; i < addedGroups.length; i++ )
                            groupsH.put(((String)addedGroups[i]).trim(), "");
                        // now rebuild liste of groups
                        groups_panel.removeAll();
                        groups_panel_layout.setRows(groupsH.size());

                        java.util.List list = Arrays.asList(groupsH.keySet().toArray());
                        Collections.sort(list, name_cmp);
                        Iterator it = list.iterator();
                        while( it.hasNext() ) {
                            String group = (String)it.next();
                            AJCheckBox cb = new AJCheckBox(group, true);
                            cb.setFont(label_font);
                            cb.setBackground(Color.white);
                            cb.addActionListener(new ActionListener() {
                                    public void actionPerformed(ActionEvent e) {
                                        AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                                        if( !cb.isSelected() ) {
                                            // remove associated data
                                            Hashtable groupsH = (Hashtable)newNet.get("groups");
                                            groups_panel.remove(cb);
                                            groupsH.remove((String)cb.object);
                                            groups_panel_layout.setRows(groups_panel_layout.getRows() - 1);
                                            groups_panel.remove(cb);
                                            groups_panel.doLayout();
                                            current_panel.revalidate();
                                        }
                                    }
                                });
                            cb.setFont(label_font);
                            groups_panel.add(cb);
                        }
                    }
                    if( key == KeyEvent.VK_ESCAPE || key == KeyEvent.VK_ENTER ) {
                        groups_panel.doLayout();
                        current_panel.revalidate();
                    }
                }
            });

        current_gbc.fill = GridBagConstraints.NONE;
        current_gbc.anchor = GridBagConstraints.WEST;
        current_gb_layout.setConstraints(mgroups_big_panel, current_gbc);
        current_panel.add(mgroups_big_panel);
        current_gbc.fill = GridBagConstraints.HORIZONTAL;
        current_gbc.anchor = GridBagConstraints.CENTER;
        tr();
    } // end of makeMemberGroups()

    public void collapseASPanel(Hashtable asData) {
        AJLabel expandLabel = (AJLabel)asData.get("expandLabel");
        if( !expandLabel.state )
            return;
        expandLabel.state = false;
        expandLabel.setIcon(MainWindow.rightArrowIcon);

        JPanel panel = (JPanel)asData.get("panel");

        JLabel groupsLabel = (JLabel)asData.get("groupsLabel");
        panel.remove(groupsLabel);

        JPanel groups_big_panel = (JPanel)asData.get("groups_big_panel");
        panel.remove(groups_big_panel);

        JLabel dhcpLabel = (JLabel)asData.get("dhcpLabel");
        panel.remove(dhcpLabel);

        JScrollPane dhcpScrollPane = (JScrollPane)asData.get("dhcpScrollPane");
        panel.remove(dhcpScrollPane);

        JLabel emptyLabel = (JLabel)asData.get("emptyLabel");
        panel.remove(emptyLabel);

        JLabel commentLabel = (JLabel)asData.get("commentLabel");
        panel.remove(commentLabel);

        AJTextField commentTf = (AJTextField)asData.get("commentTf");
        panel.remove(commentTf);

        JLabel dynamicLabel = (JLabel)asData.get("dynamicLabel");
        panel.remove(dynamicLabel);

        JPanel dhcp_big_panel = (JPanel)asData.get("dhcp_big_panel");
        panel.remove(dhcp_big_panel);

        panel.revalidate();
    } // end of collapseASPanel(...)

    public void expandASPanel(Hashtable asData) {
        AJLabel expandLabel = (AJLabel)asData.get("expandLabel");
        if( expandLabel.state )
            return;
        expandLabel.state = true;
        expandLabel.setIcon(MainWindow.downArrowIcon);

        GridBagLayout layout = (GridBagLayout)asData.get("layout");
        GridBagConstraints gbc = (GridBagConstraints)asData.get("gbc");
        JPanel panel = (JPanel)asData.get("panel");

        // third row
        gbc.gridx = 0;
        gbc.gridwidth = 1;
        gbc.anchor = GridBagConstraints.NORTHWEST;
        JLabel groupsLabel = (JLabel)asData.get("groupsLabel");
        layout.setConstraints(groupsLabel, gbc);
        panel.add(groupsLabel);
        gbc.gridx++;

        JPanel groups_big_panel = (JPanel)asData.get("groups_big_panel");
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        layout.setConstraints(groups_big_panel, gbc);
        panel.add(groups_big_panel);
        gbc.gridx = 0;
        gbc.gridwidth = 1;

        JLabel dhcpLabel = (JLabel)asData.get("dhcpLabel");
        layout.setConstraints(dhcpLabel, gbc);
        panel.add(dhcpLabel);
        gbc.gridx++;

        JScrollPane sp = (JScrollPane)asData.get("dhcpScrollPane");
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        layout.setConstraints(sp, gbc);
        panel.add(sp);
        gbc.gridx = 0;
        gbc.gridwidth = 1;

        JLabel emptyLabel = (JLabel)asData.get("emptyLabel");
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        layout.setConstraints(emptyLabel, gbc);
        panel.add(emptyLabel);
        gbc.gridwidth = 1;

        JLabel commentLabel = (JLabel)asData.get("commentLabel");
        layout.setConstraints(commentLabel, gbc);
        panel.add(commentLabel);
        gbc.gridx++;

        AJTextField commentTf = (AJTextField)asData.get("commentTf");
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        layout.setConstraints(commentTf, gbc);
        panel.add(commentTf);
        gbc.gridx = 0;
        gbc.gridwidth = 1;

        JLabel dynamicLabel = (JLabel)asData.get("dynamicLabel");
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        layout.setConstraints(dynamicLabel, gbc);
        panel.add(dynamicLabel);
        gbc.gridwidth = 1;

        JPanel dhcp_big_panel = (JPanel)asData.get("dhcp_big_panel");
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        layout.setConstraints(dhcp_big_panel, gbc);
        panel.add(dhcp_big_panel);
        gbc.gridx = 0;
        gbc.gridwidth = 1;

        panel.revalidate();
    } // expandASPanel(...)

    private void pickUpData() throws Exception {
        newNet.put("comment", commentTa.getText().trim());

        String text = dhcpOptions.getText().trim();
        String[] split = text.split("\\n");
        if( text.length() > 0 ) {
            Hashtable options = new Hashtable();
            for( int i = 0; i < split.length; i++ ) {
                if( split[i].trim().length() == 0 )
                    continue;
                String[] ops = split[i].split("=", 2);
                if( ops.length != 2 )
                    throw new Exception("Invalid dhcp options");
                Hashtable optionH = new Hashtable();
                optionH.put("option", ops[0].trim());
                optionH.put("value", ops[1].trim());
                options.put(ops[0].trim()+"="+ops[1].trim(), optionH);
            }
            if( options.size() > 0 )
                newNet.put("dhcp_options", options);
            else
                newNet.remove("dhcp_options");
        } else
            newNet.remove("dhcp_options");

        Enumeration enm = asPanels.keys();
        while( enm.hasMoreElements() ) {
            String key = (String)enm.nextElement();
            Hashtable asData = (Hashtable)asPanels.get(key);
            Hashtable as = (Hashtable)addrSpacesH.get((String)asData.get("id"));
            if( as == null ) {
                asPanels.remove(key);
                continue;
            }
            int ipAddrSpace = ((Integer)as.get("addr_space_ip")).intValue();
            int prefixLength = ((Integer)as.get("prefix_length")).intValue();

            // pick up low_res
            String txt = "";
            JTextField lowResTf = (JTextField)asData.get("lowResTf");
            try {
                txt = lowResTf.getText().trim();
                Integer lowResInt = new Integer(txt);
                as.put("low_res", new Integer(ipAddrSpace + lowResInt.intValue()));
            } catch (Exception ex) {
                throw new Exception("Low Res. "+Utils.htmlize(txt)+" is not an Integer");
            }

            // pick up and high_res
            JTextField highResTf = (JTextField)asData.get("highResTf");
            try {
                txt = highResTf.getText().trim();
                Integer highResInt = new Integer(txt);
                as.put("high_res",
                       new Integer( (ipAddrSpace | (~(0xffffffff << (32 - prefixLength)))) -
                                    highResInt.intValue() ) );
            } catch (Exception ex) {
                throw new Exception("High Res. "+Utils.htmlize(txt)+" is not an Integer");
            }

            // pick up dhcp options
            text = ((JTextArea)asData.get("dhcpOptionsTA")).getText();
            split = text.split("\\n");
            if( as == null )
                continue;
            if( text.length() > 0 ) {
                Hashtable options = new Hashtable();
                for( int i = 0; i < split.length; i++ ) {
                    if( split[i].trim().length() == 0 )
                        continue;
                    String[] ops = split[i].split("=", 2);
                    if( ops.length != 2 )
                        throw new Exception("Invalid dhcp options");
                    Hashtable optionH = new Hashtable();
                    optionH.put("option", ops[0].trim());
                    optionH.put("value", ops[1].trim());
                    options.put(ops[0].trim()+"="+ops[1].trim(), optionH);
                }
                as.put("dhcp_options", options);
            } else
                as.put("dhcp_options", new Hashtable());
        }
    } // end of pickUpData()

    private void packNames(Hashtable names) {
        if( names == null )
            return;
        Enumeration enm = names.keys();
            while( enm.hasMoreElements() ) {
                Hashtable name = (Hashtable)names.get((String)enm.nextElement());
                reserved_and_committed_names.put(((String)name.get("name")).toLowerCase()+"."+
                                                 ((String)name.get("domain")).toLowerCase(), "");

                packNames((Hashtable)name.get("aliases"));
                packNames((Hashtable)name.get("mxs"));
            }
    } // end of packNames(...)

    /** analyzes if there is anything in a address space to be verified.
     Error massages acculmulate in errMsg. */
    public boolean needVerify(Hashtable asData, StringBuffer errMsg) {
        Hashtable as = (Hashtable)addrSpacesH.get((String)asData.get("id"));
        Hashtable dhcpAddrs = (Hashtable)as.get("dhcp_addresses");
        if( dhcpAddrs == null )
            dhcpAddrs = new Hashtable();

        // stuff address into the hash; we need to do
        // it despite of the code in focusLost(...)
        Hashtable dhcp_addressesH = (Hashtable)asData.get("dhcp_addressesH");
        if( dhcp_addressesH != null ) {
            Enumeration enm = dhcp_addressesH.keys();
            while( enm.hasMoreElements() ) {
                String key = (String)enm.nextElement();
                Hashtable ipH = (Hashtable)dhcp_addressesH.get(key);
                AJFormattedTextField ftf = (AJFormattedTextField)ipH.get("ip");
                ftf.doAction(ftf);
                AJTextField tf = (AJTextField)ipH.get("name");
                tf.doAction(tf);
            }
        }

        boolean needs_verification = false;
        boolean error = false;
        Hashtable reffIP = new Hashtable();
        Hashtable reffName = new  Hashtable();
        Enumeration enm = dhcpAddrs.keys();
        String errorMsg = "";
        while( enm.hasMoreElements() ) {
            String key = (String)enm.nextElement();
            Hashtable dhcpIP = (Hashtable)dhcpAddrs.get(key);
            // check that text is a valid ip string
            String ipStr = (String)dhcpIP.get("ip_address");
            if( !ipStr.matches("((\\d|1?\\d\\d|2[0-4]\\d|25[0-5])\\.){3}"+
                               "(\\d|1?\\d\\d|2[0-4]\\d|25[0-5])") ) {
                errorMsg += (error?"<br>":"")+"&nbsp;<font color=\"#ff0000\">Error: '"+ipStr+
                    "' is not a valid ip string</font>";
                error = true;
            }
            // build up new refference Hash, for checking duplicate addresses
            if( reffIP.put(ipStr, "") != null ) {
                errorMsg += (error?"<br>":"")+
                    "&nbsp;<font color=\"#ff0000\">Error: There is a already ip address '"+
                    ipStr+"'</font>";
                error = true;
            }
            // build up new refference Hash, for checking duplicate names
            Hashtable names = (Hashtable)dhcpIP.get("names");
            if( names != null ) {
                Enumeration enm1 = names.keys();
                if( enm1.hasMoreElements() ) {
                    Hashtable name = (Hashtable)names.get(enm1.nextElement());
                    String nameStr = (name.get("name")+"."+name.get("domain")).
                        trim().toLowerCase();
                    if( reffName.put(nameStr, "") != null ) {
                        errorMsg += (error?"<br>":"")+
                            "&nbsp;<font color=\"#ff0000\">Error: IP address name '"+
                            nameStr+"' occurs more than once</font>";
                        error = true;
                    }
                }
            }

            if( dhcpIP.get("dirty") != null )
                needs_verification = true;
        }

        // now check fields for new dhcp addresses
        JTextField newIpTf = (JTextField)asData.get("newIpTf");
        String newIpStr = newIpTf.getText().trim();
        boolean hasNewIP = newIpStr.length() > 0?true:false;
        // check if string is ip
        if( hasNewIP ) {
            needs_verification = true;
            if( !newIpStr.matches("((\\d|1?\\d\\d|2[0-4]\\d|25[0-5])\\.){3}"+
                                  "(\\d|1?\\d\\d|2[0-4]\\d|25[0-5])") ) {
                errorMsg += (error?"<br>":"")+"&nbsp;<font color=\"#ff0000\">Error: '"+newIpStr+
                    "' is not a valid ip string</font>";
                error = true;
            }
        }

        JTextField newCountTf = (JTextField)asData.get("newCountTf");
        String newCountStr = newCountTf.getText().trim();
        if( newCountStr.length() == 0 )
            newCountStr = "1";
        else
            needs_verification = true;
        int newCount;
        try {
            newCount = Integer.parseInt(newCountStr);
            if( newCount < -1 ) {
                errorMsg += (error?"<br>":"")+"&nbsp;<font color=\"#ff0000\">Error: count cannot be zero or negative.</font>";
                error = true;
            }
        } catch (Exception ex) {
            errorMsg += (error?"<br>":"")+"&nbsp;<font color=\"#ff0000\">Error: invalid count</font>";
            error = true;
        }

        if( error )
            errMsg.append("<html><body>"+errorMsg+"</body></html>");

        if( needs_verification && !error )
            return true;
        else
            return false;
    } // end of needVerify(...)
}

/***************************************
 * Note, asData has following structure:
 *
 * (Hashtable)
 * 'window'                -> (NetworkModifyFrame)
 * 'current_gbc.gridwidth' -> (Integer)
 * 'current_gbc.fill'      -> (Integer)
 * 'current_gbc.anchor'    -> (Integer)
 * 'gbc.gridx'             -> (Integer)
 * 'gbc.gridy'             -> (Integer)
 * 'big_panel'             -> (JPanel)
 * 'groupsLabel'           -> (JLabel)
 * 'dhcpLabel'             -> (JLabel)
 * 'dhcpScrollPane'        -> (JScrollPane)
 * 'emptyLabel'            -> (JLabel)
 * 'commentLabel'          -> (JLabel)
 * 'dynamicLabel'          -> (JLabel)
 * 'panel'                 -> (JPanel)
 * 'id'                    -> (String)
 * 'layout'                -> (GridBagLayout)
 * 'gbc'                   -> (GridBagConstraints)
 * 'dummy_panel'           -> (JPanel)
 * 'dhcpOptionsTA'         -> (JTextArea)
 * 'commentTf'             -> (AJTextField)
 * 'highResTf'             -> (JTextField)
 * 'lowResTf'              -> (JTextField)
 * 'dhcp_big_panel'        -> (JPanel)
 * 'dhcp_ip_panel'         -> (JPanel)
 * 'dhcp_addressesH'       -> (Hashtable)
 *                            %key% -> (Hashtable)
 *                                     'ip_panel' -> (JPanel)
 *                                     'ip'       -> (AJFormattedTextField)
 * 'dhcp_ip_panel_layout'  -> (GridLayout)
 * 'groups_big_panel'      -> (JPanel)
 * 'groups_panel'          -> (JPanel)
 * 'groups_panel_layout'   -> (GridLayout)
 * 'groups_list'           -> (AJList(asData))
 * 'groups_sp'             -> (JScrollPane)
 * 'add_another_groupB'    -> (AJButton(asData))
 * 'bottom_groups_panel'   -> (JPanel)
 ***************************************/
