/************************************************************************
 *
 * NetworkMoveFrame.java - GUI for moving address spaces
 *
 * $Id: NetworkMoveFrame.java,v 1.2 2006/09/26 16:56:47 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import java.lang.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import java.rmi.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import stanford.netdb.utils.*;
import javax.swing.border.*;

public class NetworkMoveFrame extends XFrame implements Runnable {

    public JPanel msgPanel  = new JPanel(new BorderLayout());
    public JPanel mainPanel = new JPanel();
    public JPanel asPanel = new JPanel();
    public GridLayout asPanelLayout;
    public JComboBox destNet;
    public Hashtable cbs = new Hashtable(); // list of check boxes

    public Hashtable net;
    public Hashtable allNets;
    public Hashtable addrSpacesH;
    public Vector asList;
    public Timestamp lockedTm = null;

    public String handle = null; // networks' fqdn

    public NetworkMoveFrame THIS;

    public static Hashtable netMoveFrames = new Hashtable();

    public JLabel msg = new JLabel("", JLabel.CENTER);

    public Comparator as_ip_cmp = null;

    public NetworkMoveFrame(String handle) {
        super("Move Address Spaces: "+handle);
        this.handle = handle;

        THIS = this;

        NetworkMoveFrame f = (NetworkMoveFrame)netMoveFrames.get(handle.toLowerCase());
        if( f != null ) {
            f.setState(Frame.NORMAL);
            f.toFront();
            return;
        }

        netMoveFrames.put(handle.toLowerCase(), this);
        register();

        as_ip_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Hashtable as1 = (Hashtable)addrSpacesH.get((String)o1);
                    Hashtable as2 = (Hashtable)addrSpacesH.get((String)o2);
                    return (int)(((Integer)as1.get("addr_space_ip")).intValue() -
                                 ((Integer)as2.get("addr_space_ip")).intValue());
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        initFrame();
    }

    public void dispose() {
        if( handle != null )
            netMoveFrames.remove(handle.toLowerCase());
        super.dispose();
    }

    private void initFrame() {
        init(); // init XFrame bowls
        tool_bar.removeAll();
        tool_bar.add(modifyB);
        tool_bar.add(new JLabel("   "));
        tool_bar.add(closeB);
        modifyB.setText("Move Address Spaces");

        closeB.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    (new NetworkThread(THIS, NetworkThread.UNLOCK_NETWORK, "")).start();
                    dispose();
                }
            });

        modifyB.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    asList = new Vector();
                    Enumeration enm = cbs.keys();
                    while( enm.hasMoreElements() ) {
                        String asPrefix = (String)enm.nextElement();
                        JCheckBox cb = (JCheckBox)cbs.get(asPrefix);
                        if( cb.isSelected() )
                            asList.addElement(asPrefix);
                    }
                    modifyB.setEnabled(false);
                    msg.setText("Moving address spaces...");
                    central_panel.removeAll();
                    central_panel.add(msgPanel);
                    central_panel.revalidate();
                    central_panel.repaint();
                    (new NetworkThread(THIS, NetworkThread.MOVE_AS,
                                       "Address spaces have been moved")).start();
                }
            });

        msgPanel.setBackground(Color.white);
        msgPanel.add(msg, BorderLayout.CENTER);
        central_panel.add(msgPanel);

        this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        this.pack();
        this.setSize(900, 500);

        Dimension scrnsize = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation( (int)((scrnsize.getWidth()-getWidth())/2),
                     (int)((scrnsize.getHeight()-getHeight())/2) );

        this.setVisible(true);

        (new Thread(this)).start();
    }

    public void run() {
        try {
            msg.setText("Locking network...");
            if( NetworkModifyFrame.netModFrames.get(handle.toLowerCase()) != null ) {
                msg.setText("You are holding lock on the same object in some other window.");
                msg.setForeground(Color.red);
                return;
            }

            lockedTm = MainWindow.netdb.lock(Constants.NETWORK, handle);

            msg.setText("Loading network info...");
            net = MainWindow.netdb.getInfo(Constants.NETWORK, handle);

            if( net == null || net.get("id") == null ) {
                msg.setText("Failed to load network. It probably does not exist.");
                msg.setForeground(Color.red);
                return;
            }

            msg.setText("Verifying access...");
            if( !MainWindow.netdb.checkAccess(Constants.NETWORK) ||
                (handle != null && !MainWindow.netdb.checkGroupAccess(net) ) )
                throw new Exception("You do not have sufficient access rights to modify this object");

            msg.setText("Loading list of networks...");
            allNets = MainWindow.netdb.listMyNetworks();

            msg.setText("Building interface...");

            mainPanel.setBackground(Color.white);
            current_panel = mainPanel;
            current_panel.setLayout(current_gb_layout);

            current_gbc.fill = GridBagConstraints.HORIZONTAL;
            current_gbc.weightx = 1.0;
            current_gbc.weighty = 0;
            current_gbc.gridx = 0;
            current_gbc.gridy = 0;

            // empty row at the top
            XLabel(" ");
            tr();

            HTMLPanel("Source Network");
            td();

            Hashtable names = (Hashtable)net.get("names");
            Enumeration enm = names.keys();
            Hashtable name = (Hashtable)names.get((String)enm.nextElement());
            XLabel(name.get("name")+"."+name.get("domain"), LEFT);
            tr();

            XLabel(" ");
            tr();

            HTMLPanel("Destination Network <font color=\"#ff0000\">*</font>");
            td();

            destNet = new JComboBox();
            java.util.List list = Arrays.asList(allNets.keySet().toArray());
            Collections.sort(list, name_cmp);
            Iterator it = list.iterator();
            while( it.hasNext() ) {
                destNet.addItem((String)it.next());
            }

            current_gbc.fill = GridBagConstraints.NONE;
            current_gbc.anchor = GridBagConstraints.WEST;
            current_gb_layout.setConstraints(destNet, current_gbc);
            current_panel.add(destNet);
            current_gbc.fill = GridBagConstraints.HORIZONTAL;
            current_gbc.anchor = GridBagConstraints.CENTER;
            tr();

            current_gbc.anchor = GridBagConstraints.NORTHWEST;
            HTMLPanel("Address Spaces");
            td();

            current_gbc.anchor = GridBagConstraints.WEST;
            asPanel.setBackground(Color.white);
            addrSpacesH = (Hashtable)net.get("address_spaces");
            asPanelLayout = new GridLayout(addrSpacesH.size(), 1, 0, 2);
            asPanel.setLayout(asPanelLayout);
            java.util.List asIDs = Arrays.asList(addrSpacesH.keySet().toArray());
            Collections.sort(asIDs, as_ip_cmp);
            Object[] asIDsArray = asIDs.toArray();
            for( int i = 0; i < asIDsArray.length; i++ ) {
                Hashtable as = (Hashtable)addrSpacesH.get((String)asIDsArray[i]);
                String asPrefix = (String)as.get("addr_space_prefix");
                JCheckBox cb = new JCheckBox(asPrefix, false);
                cb.setBackground(Color.white);
                cbs.put(asPrefix, cb);
                asPanel.add(cb);
            }
            current_gbc.fill = GridBagConstraints.NONE;
            current_gb_layout.setConstraints(asPanel, current_gbc);
            current_panel.add(asPanel);
            current_gbc.fill = GridBagConstraints.HORIZONTAL;
            tr();

            central_panel.removeAll();
            central_panel.add(mainPanel);
            central_panel.revalidate();

            modifyB.setEnabled(true);

        } catch(java.rmi.ServerException exx) {
            if( exx.getCause() instanceof SessionTimeoutException )
                X.handleTimeout();

            else if( exx.getCause() instanceof RemoteNetDBException ) {
                RemoteNetDBException ne = (RemoteNetDBException)exx.getCause();
                Vector errors = ne.getMessages();
                String[] msg = new String[errors.size()];
                for( int i = 0; i < errors.size(); i++ )
                    msg[i] = (String)errors.elementAt(i);
                JOptionPane.showMessageDialog(THIS,
                    msg, "Error", JOptionPane.ERROR_MESSAGE);

            } else if( exx.getCause() instanceof RemoteException ) {
                RemoteException re = (RemoteException)exx.getCause();
                JOptionPane.showMessageDialog(THIS,
                    re.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);

            } else {
                JOptionPane.showMessageDialog(THIS,
                    exx.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
            }
            closeB.doClick();
            return;

        } catch (java.rmi.NoSuchObjectException ex) {
            X.handleTimeout();

        } catch (java.rmi.MarshalException ex) {
            X.handleTimeout();

        } catch(Exception exx) {
            JOptionPane.showMessageDialog(THIS,
                    exx.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
            closeB.doClick();
            return;
        }
    }
}
