/************************************************************************
 *
 * NodeModifyFrame.java -
 *
 * $Id: NodeModifyFrame.java,v 1.2 2006/09/26 17:08:09 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import java.lang.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import java.awt.*;
import java.rmi.*;
import java.sql.*;
import java.text.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import stanford.netdb.utils.*;
import su.netdb.parser.*;
import javax.swing.border.*;

public class NodeModifyFrame extends XFrame implements Runnable {

    public JPanel ePanel;
    public JLabel ePanelLabel;
    public JPanel mainPanel = new JPanel();
    public JPanel msgPanel  = new JPanel(new BorderLayout());
    public JLabel msg = new JLabel("", JLabel.CENTER);

    private static final boolean EXCLUDE_DIRTY = true;
    private static final boolean INCLUDE_ALL = false;

    boolean init = true; // when rebuilding interface should be set to false
    Hashtable modelsH;

    Parser parser = new Parser();

    // access rights
    boolean advNodeAccess = false;
    boolean routerAccess = false;
    boolean ipcAccess = false;

    // names and ip verification
    public Hashtable reservedNames = new Hashtable(); // used for canceling reserved but not commited names
    public Hashtable reservedIPs = new Hashtable(); // used for canceling reserved but not commited ip addresses
    public Hashtable allOldNames;
    public Hashtable allNewNames;
    public Hashtable allOldIPs;
    public Hashtable allNewIPs;
    public Hashtable ipTextFields;

    // name panels
    Hashtable namePanels;
    AJButton add_another_name_b;

    // template stuff
    JComboBox tmplAddrSpacesJCombo;

    // ipc panels
    JPanel ipc_header_panel;
    JPanel ipc_add_another_ip_panel;
    JSeparator ipc_separator_line;
    JPanel ipc_big_panel;
    JLabel ipc_bottom_empty_label;
    GridLayout ipc_ip_layout;
    JPanel ipc_ips_panel;

    // oses panel
    public JPanel oses_big_panel;
    public JPanel oses_panel;
    public GridLayout oses_panel_layout;
    public JPanel bottom_oses_panel;
    public JPanel right_oses_panel;
    public JButton add_another_os;
    public JButton create_another_os;
    public JScrollPane oses_sp;
    public AJList list_of_oses;
    public JButton[] buttons;
    public static JDialog addOSDialogWin;
    public JTextField osNameF;

    // groups panel
    public JPanel groups_big_panel;
    public JPanel groups_panel;
    public GridLayout groups_panel_layout;
    public JPanel bottom_groups_panel;
    public JPanel right_groups_panel;
    public JButton add_another_group;
    public JScrollPane groups_sp;
    public AJList list_of_groups;

    // admins panel
    public JPanel admins_big_panel;
    public JPanel admins_panel;
    public GridLayout admins_panel_layout;
    public JPanel right_admins_panel;
    public JButton add_another_admin;

    // look up data
    public boolean lookupATeams;
    public JButton[] lookupButtons;

    // users panel
    public JPanel users_big_panel;
    public JPanel users_panel;
    public GridLayout users_panel_layout;
    public JPanel right_users_panel;
    public JButton add_another_user;

    // ip address reserve panels
    public boolean reserving_ipc;
    public Hashtable currentIntfcStore;
    public String ipAddrReserveIP;
    public JLabel ipAddrReserveNameLabel;
    public JTextField ipAddrNameReserveTf;
    public JLabel ipAddrCountLabel;
    public JTextField ipAddrCountTf;
    public String ipAddrNameReserve = "";
    public JTextField ipAddrReserveTf;
    public AJCheckBox ipAddrActiveCB;
    public boolean ipAddrActiveRes;
    public AJList addrSpacesJList;
    public static Comparator as_cmp = null;
    public JOptionPane ipAddrReserveDialog;
    public JPanel ipAddrReservePanel;
    public JDialog ipAddrReserveWin;
    public JButton[] ipAddrReserveButtons;

    // persons lookup panels and data
    public JOptionPane personsLookupDialog;
    public JLabel personsTopLabel;
    public JLabel personsMiddleLabel;
    public JPanel personsLookupPanel;
    public JPanel personsLookupCenterPanel;
    public GridLayout personsLookupGridLayout;
    public JDialog lookupPersonWin;
    public JTextArea lookup_ta;
    public Hashtable looked_up_persons;
    public String old_lookup_text = "";

    // node type checkboxes
    public AJCheckBox adv_cb;
    public AJCheckBox rout_cb;
    public AJCheckBox ipc_cb;
    public AJCheckBox tmpl_cb;
    // and their panel
    JPanel right_hp;

    // Interfaces
    public Hashtable interfacesStore = new Hashtable();
    public JButton add_intfc_b;

    public JComboBox deps_combo;
    public JComboBox loc_combo;
    public JTextField room_tf;
    public JComboBox make_and_model_combo;
    public JTextField tag1_name;
    public JTextField tag2_name;
    public JTextField tag1_value;
    public JTextField tag2_value;
    public JTextField exp_date_tf;
    public JTextArea comment_ta;

    public NodeModifyFrame THIS;

    public boolean closed               = false;
    public boolean template             = false;
    public boolean create               = false;
    public boolean open_as_pickup       = false;
    public AJButton open_as_pickup_b    = null;

    public Hashtable oldNode                    = null;
    public Hashtable newNode                    = null;
    public Hashtable allGroupsH;
    public Hashtable allDepartmentsH;
    public Hashtable locationsH;
    public Hashtable makes_and_models_H;
    public Hashtable osesH;
    public Hashtable addrSpacesH;
    public int oldNode_deps_index               = 0;
    public int oldNode_location_index           = 0;
    public int oldNode_make_and_model_index     = 0;
    public Timestamp lockedTm                   = null;
    public String handle                        = null;
    public Hashtable allIPs                     = null;
    public Comparator full_ip_cmp;

    public StringBuffer counter                 = new StringBuffer("a10000");

    public static Hashtable nodeModFrames       = new Hashtable();

    public void dispose() {
        if( handle != null )
            nodeModFrames.remove(handle.toLowerCase());
        super.dispose();
    }

    public static NodeModifyFrame MakeNodeCreateFrame() {
        NodeModifyFrame nmf = new NodeModifyFrame();
        nmf.initFrame();
        (new Thread(nmf)).start();
        return nmf;
    }

    public static NodeModifyFrame MakeNodeModifyFrame(String handle, boolean template) {
        if( handle != null && !template ) {
            NodeModifyFrame f = (NodeModifyFrame)nodeModFrames.get(handle.toLowerCase());
            if( f != null ) {
                f.setState(Frame.NORMAL);
                f.toFront();
                return f;
            }
        }

        NodeModifyFrame nmf = new NodeModifyFrame(handle, template);
        nmf.initFrame();
        (new Thread(nmf)).start();
        return nmf;
    }

    private NodeModifyFrame() {
        super("NodeCreate");
        this.template = template;
        THIS = this;

        register();
    }

    private NodeModifyFrame(String handle, boolean template) {
        super("NodeModify: "+handle);

        if( handle == null || template )
            setTitle("NodeCreate");

        this.template = template;
        this.handle = handle;
        if( this.handle.indexOf(".") < 0 )
            this.handle += ".";
        setTitle("NodeModify: "+this.handle);
        THIS = this;

        if( handle != null && !template )
            nodeModFrames.put(handle.toLowerCase(), this);

        register();
    }

    private void initFrame() {
        init(); // init XFrame bowls

        tool_bar.removeAll();

        modifyB.setText("Save");
        tool_bar.add(modifyB);

        tool_bar.add(new JLabel(" "));

        deleteB.setText("Verify");
        tool_bar.add(deleteB);

        tool_bar.add(new JLabel(" "));

        reloadB.setText("Reset");
        tool_bar.add(reloadB);

        tool_bar.add(new JLabel(" "));

        closeB.setText("Cancel");
        tool_bar.add(closeB);

        deleteB.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    try {
                        header_panel.remove(ePanel);
                        header_panel.revalidate();
                        collectAllData(true); // collects all data from all text fields and areas

                        // prepare for verification
                        msg.setText("Verifying node...");
                        central_panel.removeAll();
                        central_panel.add(msgPanel, BorderLayout.CENTER);
                        central_panel.revalidate();
                        central_panel.repaint();
                        reloadB.setEnabled(false);
                        modifyB.setEnabled(false);
                        deleteB.setEnabled(false);
                        header_panel.remove(ePanel);
                        ePanelLabel.setText("");
                        header_panel.revalidate();

                        (new NodeThread(THIS, NodeThread.VERIFY)).start();

                    } catch (NetDBException exx) {
                        Vector errors = exx.getMessages();
                        String msg = "<html><body>";
                        int esize = errors.size();
                        for( int i = 0; i < esize; i++ )
                            msg += "<font color=\"#ff0000\">&nbsp;Error: "+(String)errors.elementAt(i)+
                                "</font>"+(i<esize-1?"<br>":"");
                        ePanelLabel.setText(msg+"</body></html>");
                        header_panel.add(ePanel);
                        header_panel.revalidate();

                    } catch (Exception exx) {
                        ePanelLabel.setText("<html><body><font color=\"#ff0000\">&nbsp;Error: "+
                                            exx.getMessage()+"</font></body></html>");
                        header_panel.add(ePanel);
                        header_panel.revalidate();
                    }
                }
            });

        reloadB.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    // do full reset
                    try {
                        newNode = Utils.copyHash(oldNode);
                        // set up msg and clean for rebuild
                        msg.setText("Resetting...");
                        mainPanel.removeAll();
                        central_panel.removeAll();
                        central_panel.add(msgPanel, BorderLayout.CENTER);

                        reloadB.setEnabled(false);
                        modifyB.setEnabled(false);
                        deleteB.setEnabled(false);

                        header_panel.remove(ePanel);
                        ePanelLabel.setText("");
                        header_panel.revalidate();

                        mainPanel.removeAll();
                        central_panel.revalidate();
                        central_panel.repaint();
                        init = false; // rebuilding
                        (new Thread(THIS)).start();

                    } catch (Exception ex) {
                        ex.printStackTrace();
                        Utils.error(ex.getMessage());
                    }
                }
            });

        modifyB.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    try {
                        msg.setText("Saving node...");
                        central_panel.removeAll();
                        central_panel.add(msgPanel, BorderLayout.CENTER);
                        central_panel.revalidate();
                        central_panel.repaint();
                        reloadB.setEnabled(false);
                        modifyB.setEnabled(false);
                        deleteB.setEnabled(false);
                        header_panel.remove(ePanel);
                        ePanelLabel.setText("");
                        header_panel.revalidate();

                        Exception collection_error = null;
                        try {
                            collectAllData(true); // collects all data from all text fields and areas
                        } catch (Exception exx) {
                            collection_error = exx;
                        }

                        if( collection_error == null )
                            (new NodeThread(THIS, NodeThread.MODIFY_NODE)).start();
                        else
                            (new NodeThread(THIS, NodeThread.EXCEPTION, collection_error)).start();

                    } catch(Exception ex) {
                        ex.printStackTrace();
                        ePanelLabel.setText("<html><body> <font color=\"#ff0000\">&nbsp;Error: "+
                                            ex.getMessage()+"</font></body></html>");
                        header_panel.add(ePanel);
                        header_panel.revalidate();
                    }
                }
            });

        ActionListener[] al = closeB.getActionListeners();
        for( int i = 0; i < al.length; i++ )
            closeB.removeActionListener(al[i]);

        closeB.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    closed = true;
                    if( lockedTm != null ) {
                        msg.setText("Unlocking node");

                        central_panel.removeAll();
                        central_panel.add(msgPanel, BorderLayout.CENTER);
                        central_panel.revalidate();
                        central_panel.repaint();
                        reloadB.setEnabled(false);
                        modifyB.setEnabled(false);
                        deleteB.setEnabled(false);
                        if( header_panel != null && ePanel != null ) {
                            header_panel.remove(ePanel);
                            ePanelLabel.setText("");
                            header_panel.revalidate();
                        }

                        (new NodeThread(THIS, NodeThread.UNLOCK_NODE)).start();
                    } else {
                        (new NodeThread(THIS, NodeThread.CANCEL_RESERVES)).start();
                    }
                    if( handle != null )
                        nodeModFrames.remove(handle.toLowerCase());
                    THIS.dispose();
                }
            });

        mainPanel.setBackground(Color.white);

        this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        this.pack();
        this.setSize(900, 700);
        Dimension scrnsize = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation( (int)((scrnsize.getWidth()-getWidth())/2),
                     (int)((scrnsize.getHeight()-getHeight())/2) );
        central_panel.removeAll();
        msgPanel.setBackground(Color.white);
        msgPanel.add(msg, BorderLayout.CENTER);
        central_panel.add(msgPanel);
        this.show();
    } // end of initFrame()

    public void run() {
        try {
            //if( newNode != null )
            //    Utils.printHash(newNode);

            if( init ) {
                if( handle == null || template )
                    setTitle("NodeCreate");

                full_ip_cmp = new Comparator() {
                        public int compare(Object o1, Object o2) {
                            int ip1 = Utils.ipstr2num((String)o1);
                            int ip2 = Utils.ipstr2num((String)o2);
                            boolean active1 = ((Boolean)allIPs.get(o1)).booleanValue();
                            boolean active2 = ((Boolean)allIPs.get(o2)).booleanValue();
                            if( active1 == active2 )
                                return ip1 - ip2;
                            else if( active1 && !active2 )
                                return -1;
                            else
                                return 1;
                        }
                        public boolean equals(Object obj) {
                            return true;
                        }
                    };

                as_cmp = new Comparator() {
                        public int compare(Object o1, Object o2) {
                            Hashtable as1 = Utils.nHash(addrSpacesH.get(o1));
                            Hashtable as2 = Utils.nHash(addrSpacesH.get(o2));
                            long cmp_val = Utils.ipstr2long((String)as1.get("str_ip_address")) -
                                Utils.ipstr2long((String)as2.get("str_ip_address"));
                            return cmp_val>0?1:(cmp_val<0?-1:0);
                        }
                        public boolean equals(Object obj) {
                            return true;
                        }
                    };

                // set up errors panel
                ePanel = new JPanel(new BorderLayout());
                ePanel.setBackground(Color.yellow);
                JPanel closePanel = new JPanel(new BorderLayout());
                closePanel.setBackground(Color.yellow);
                JLabel closeLabel = makeLabel("x ");
                closeLabel.setCursor(new Cursor(Cursor.HAND_CURSOR));
                closeLabel.addMouseListener(new MouseAdapter() {
                        public void mouseEntered(MouseEvent event) {
                            JLabel label = (JLabel)event.getComponent();
                            label.setFont(bold_font);
                            label.repaint();
                        }
                        public void mouseExited(MouseEvent event) {
                            JLabel label = (JLabel)event.getComponent();
                            label.setFont(label_font);
                            label.repaint();
                        }
                        public void mouseClicked(MouseEvent event) {
                            header_panel.remove(ePanel);
                            ePanelLabel.setText("");
                            header_panel.revalidate();
                        }
                    });
                closePanel.add(closeLabel, BorderLayout.NORTH);
                ePanel.add(closePanel, BorderLayout.EAST);
                ePanelLabel = new JLabel("");
                ePanel.add(ePanelLabel, BorderLayout.CENTER);


                msg.setText("Locking node...");
                if( handle != null && !template )
                    lockedTm = MainWindow.netdb.lock(Constants.NODE, handle);

                msg.setText("Loading node info...");
                if( handle != null ) {
                    oldNode = MainWindow.netdb.getInfo(Constants.NODE, handle);
                    if( template ) {

                        handle = null;

                        // clean up names
                        Hashtable names = new Hashtable();
                        int indexID = 0;
                        synchronized( counter ) {
                            indexID = Integer.parseInt(counter.substring(1));
                            counter.replace(1, counter.length(), ""+(indexID+1));
                        }
                        Hashtable name = new Hashtable();
                        name.put("name", "");
                        name.put("domain", "");
                        names.put("A "+indexID, name);
                        oldNode.put("names", names);

                        // clean up comment
                        oldNode.put("comment", "");

                        // reset tags values
                        Hashtable tags = Utils.nHash(oldNode.get("tags"));
                        Enumeration enm = tags.keys();
                        while( enm.hasMoreElements() )
                            tags.put(enm.nextElement(),"");

                        // clean up ipc addresses
                        if( oldNode.get("ipc_addresses") != null )
                            oldNode.put("ipc_addresses", new Hashtable());

                        // AS we will use to reserve ip address
                        String as_to_reserve = null;
                        // or starting ip address
                        String ip_start = null;

                        // fiddle with types
                        if( oldNode.get("types") == null )
                            oldNode.put("types", new Hashtable());

                        if( ((Hashtable)oldNode.get("types")).get("Template") != null ) {
                            ((Hashtable)oldNode.get("types")).clear();
                            as_to_reserve = (String)oldNode.remove("addr_space");
                        }

                        // collect ips from all interfaces
                        allIPs = new Hashtable();
                        Hashtable interfaces = (Hashtable)oldNode.get("interfaces");
                        if( interfaces == null ) {
                            interfaces = new Hashtable();
                            oldNode.put("interfaces", interfaces);
                        }
                        enm = interfaces.keys();
                        while( enm.hasMoreElements() ) {
                            Hashtable intfc = (Hashtable)interfaces.get(enm.nextElement());
                            Hashtable ips = Utils.nHash(intfc.get("ips"));
                            Enumeration enm2 = ips.keys();
                            while( enm2.hasMoreElements() ) {
                                String ip = (String)enm2.nextElement();
                                Boolean active = (Boolean)((Hashtable)ips.get(ip)).get("active?");
                                allIPs.put(ip, active);
                            }
                        }
                        // now pick up first ip
                        java.util.List list = Arrays.asList(allIPs.keySet().toArray());
                        Collections.sort(list, full_ip_cmp);
                        Iterator it = list.iterator();
                        while( it.hasNext() ) {
                            ip_start = (String)it.next();
                            break;
                        }
                        interfaces.clear();

                        try {
                            synchronized( counter ) {
                                indexID = Integer.parseInt(counter.substring(1));
                                counter.replace(1, counter.length(), ""+(indexID+1));
                            }
                        } catch (Exception exx) {
                            Utils.error(exx.getMessage());
                        }
                        interfaces = new Hashtable();
                        Hashtable intfc = new Hashtable();
                        interfaces.put("-"+indexID, intfc);
                        intfc.put("roaming?", new Boolean(false));
                        intfc.put("use-dhcp?", new Boolean(false));
                        intfc.put("dhcp-options", new Hashtable());
                        intfc.put("names", new Hashtable());
                        oldNode.put("interfaces", interfaces);
                        Hashtable ips = new Hashtable();
                        intfc.put("ips", ips);
                        open_as_pickup = true;

                        // find corresponing address space
                        if( ip_start != null )
                            as_to_reserve = MainWindow.netdb.findIPAddrSpace(ip_start);

                        if( as_to_reserve != null && as_to_reserve.trim().length() > 0 )
                            reserveIPforCreate(ips, as_to_reserve,
                                 as_to_reserve.substring(0, as_to_reserve.indexOf("/")), true);
                    }

                } else {
                    // node create
                    create = true;
                    oldNode = new Hashtable();
                    oldNode.put("object", "node");

                    Hashtable names = new Hashtable();
                    int indexID = 0;
                    synchronized( counter ) {
                        indexID = Integer.parseInt(counter.substring(1));
                        counter.replace(1, counter.length(), ""+(indexID+1));
                    }
                    Hashtable name = new Hashtable();
                    name.put("name", "");
                    name.put("domain", "");
                    names.put("A "+indexID, name);
                    oldNode.put("names", names);

                    oldNode.put("admins", new Hashtable());
                    Hashtable groups = new Hashtable();
                    groups.put(MainWindow.user.get("default_group"), "");
                    oldNode.put("groups", groups);

                    // set department - first oficial department from user
                    String depName = "";;
                    Hashtable departments = Utils.nHash(MainWindow.user.get("departments"));
                    java.util.List list = Arrays.asList(departments.keySet().toArray());
                    Collections.sort(list, name_cmp);
                    Iterator it = list.iterator();
                    while( it.hasNext() ) {
                        String dep = ((String)it.next()).trim();
                        Hashtable depH = (Hashtable)departments.get(dep);
                        Boolean official = (Boolean)depH.get("official?");
                        if( official != null && official.booleanValue() ) {
                            depName = dep;
                            break;
                        }
                    }
                    oldNode.put("dept", depName);

                    oldNode.put("location", new String(""));
                    oldNode.put("room", new String(""));
                    oldNode.put("make", new String(""));
                    oldNode.put("model", new String(""));
                    oldNode.put("oses", new Hashtable());

                    String default_addr_space = (String)MainWindow.user.get("default_addr_space");
                    try {
                        synchronized( counter ) {
                            indexID = Integer.parseInt(counter.substring(1));
                            counter.replace(1, counter.length(), ""+(indexID+1));
                        }
                    } catch (Exception exx) {
                        Utils.error(exx.getMessage());
                    }
                    Hashtable interfaces = new Hashtable();
                    Hashtable intfc = new Hashtable();
                    interfaces.put("-"+indexID, intfc);
                    intfc.put("roaming?", new Boolean(false));
                    intfc.put("use-dhcp?", new Boolean(false));
                    intfc.put("dhcp-options", new Hashtable());
                    intfc.put("names", new Hashtable());
                    oldNode.put("interfaces", interfaces);
                    Hashtable ips = new Hashtable();
                    intfc.put("ips", ips);

                    open_as_pickup = true;
                    if( default_addr_space != null )
                        reserveIPforCreate(ips, default_addr_space,
                           default_addr_space.substring(0, default_addr_space.indexOf("/")), false);
                }

                newNode = Utils.copyHash(oldNode);

                msg.setText("Verifying access...");

                /* // now this is taken care in netdb.lock(...) call
                if( !MainWindow.netdb.checkAccess(Constants.NODE) ||
                    (handle != null && !MainWindow.netdb.checkGroupAccess(oldNode) ) )
                    throw new Exception("You do not have sufficient access rights to modify this object");
                */

                advNodeAccess = MainWindow.netdb.checkAccess(Constants.ADVANCED_NODE);
                routerAccess = MainWindow.netdb.checkAccess(Constants.ROUTER);
                ipcAccess = MainWindow.netdb.checkAccess(Constants.IPC_PROVIDER);

                Hashtable types = Utils.nHash(oldNode.get("types"));
                Enumeration enm = types.keys();
                while( enm.hasMoreElements() ) {
                    String type = ((String)enm.nextElement()).trim();
                    if( type.equals("Advanced") && !advNodeAccess )
                        throw new Exception("You do not have sufficient access rights to modify this object");
                    else if( type.equals("Router") && !routerAccess )
                        throw new Exception("You do not have sufficient access rights to modify this object");
                    else if( type.equals("IPCprovider") && !ipcAccess )
                        throw new Exception("You do not have sufficient access rights to modify this object");
                }

                msg.setText("Loading list of groups...");
                Boolean allGroups = (Boolean)MainWindow.user.get("all_groups?");
                if( allGroups != null && allGroups.booleanValue() )
                    allGroupsH = MainWindow.getGroups();
                else
                    allGroupsH = Utils.copyHash((Hashtable)MainWindow.user.get("groups"));

                msg.setText("Loading list of departments...");
                allDepartmentsH = MainWindow.getDepartments();
                if( create ) {
                    Hashtable dep = new Hashtable();
                    allDepartmentsH.put("--", dep);
                    dep.put("name", "");
                }

                msg.setText("Loading list of locations...");
                locationsH = MainWindow.getLocations();
                if( create ) {
                    Hashtable location = new Hashtable();
                    locationsH.put("--", location);
                }

                msg.setText("Loading list of makes and models...");
                modelsH = MainWindow.getModels();
                if( create ) {
                    Hashtable model = new Hashtable();
                    modelsH.put("--", model);
                    model.put("model", "--");
                    model.put("make", "");
                }

                msg.setText("Loading list of oses...");
                osesH = MainWindow.getOSes();

                msg.setText("Loading list of address spaces...");
                addrSpacesH = MainWindow.getMyAddrSpaces();

                msg.setText("Building interface...");
            }
            String nodeDep = ((String)newNode.get("dept")).trim();
            String nodeLocation = ((String)newNode.get("location")).trim();

            if( init ) {
                list_of_oses = new AJList();
                list_of_groups = new AJList();

                if( lockedTm != null ) {
                    oldNode.put("locked", lockedTm);
                    newNode.put("locked", lockedTm);
                }

                // make ip reserve dialog
                java.util.List list = Arrays.asList(addrSpacesH.keySet().toArray());
                Collections.sort(list, as_cmp);
                Iterator it = list.iterator();
                int index = 0;
                String addr_spaces[] = new String[addrSpacesH.size()];
                while( it.hasNext() )
                    addr_spaces[index++] =
                        (String)((Hashtable)addrSpacesH.get(it.next())).get("prefix");
                addrSpacesJList = new AJList();
                addrSpacesJList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
                addrSpacesJList.setFont(label_font);
                addrSpacesJList.setListData(addr_spaces);
                addrSpacesJList.addListSelectionListener(new ListSelectionListener() {
                        public void valueChanged(ListSelectionEvent e) {
                            ipAddrReserveTf.setText((String)((JList)e.getSource()).getSelectedValue());
                        }
                    });
                addrSpacesJList.addFocusListener(new FocusAdapter() {
                        public void focusLost(FocusEvent e) {
                            if( !(e.getOppositeComponent() instanceof JButton) &&
                                !(e.getOppositeComponent() instanceof JCheckBox) &&
                                !(e.getOppositeComponent() instanceof JTextField) ) {
                                ipAddrReserveTf.requestFocus();
                            }
                        }
                    });

                ipAddrReservePanel = new JPanel(new BorderLayout());
                ipAddrReservePanel.add(new JScrollPane(addrSpacesJList), BorderLayout.NORTH);
                ipAddrReservePanel.add(new JLabel(), BorderLayout.CENTER);

                JPanel south_panel = new JPanel(new FlowLayout());
                south_panel.add(new JLabel("IP Addr"));
                ipAddrReserveTf = new JTextField("", 16);
                ipAddrReserveTf.addFocusListener(new FocusAdapter(){
                        public void focusGained(FocusEvent e) {
                            ipAddrReserveTf.selectAll();
                        }
                    });
                ipAddrReserveTf.setFont(label_font);
                south_panel.add(ipAddrReserveTf);
                ipAddrActiveCB = new AJCheckBox("Active", true);
                ipAddrActiveCB.setBackground(south_panel.getBackground());
                south_panel.add(ipAddrActiveCB);
                ipAddrReserveNameLabel = new JLabel(" Name");
                south_panel.add(ipAddrReserveNameLabel);
                ipAddrNameReserveTf = new JTextField("", 16);
                ipAddrNameReserveTf.setFont(label_font);
                south_panel.add(ipAddrNameReserveTf);
                ipAddrCountLabel = new JLabel(" Count");
                south_panel.add(ipAddrCountLabel);
                ipAddrCountTf = new JTextField("", 4);
                ipAddrCountTf.setFont(label_font);
                south_panel.add(ipAddrCountTf);
                ipAddrReservePanel.add(south_panel, BorderLayout.SOUTH);

                ipAddrReserveButtons = new JButton[2];
                ipAddrReserveButtons[0] = new BButton("Reserve");
                ipAddrReserveButtons[0].addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            // verify that ip is actual ip
                            ipAddrReserveIP = ipAddrReserveTf.getText().trim();
                            ipAddrNameReserve = ipAddrNameReserveTf.getText().trim();
                            if( ipAddrReserveIP.indexOf("/") > -1 ) {
                                String ipPart = ipAddrReserveIP.substring(0, ipAddrReserveIP.indexOf("/"));
                                int dots = countDots(ipPart);
                                for( int i = 0; i<3-dots; i++ )
                                    ipPart += ".0";
                                ipAddrReserveIP = ipPart + ipAddrReserveIP.substring(ipAddrReserveIP.indexOf("/"));
                            } else {
                                int dots = countDots(ipAddrReserveIP);
                                for( int i = 0; i<3-dots; i++ )
                                    ipAddrReserveIP += ".0";
                            }

                            if( !ipAddrReserveIP.matches("(\\d|1?\\d\\d|2[0-4]\\d|25[0-5])\\.(\\d|1?\\d\\d|2[0-4]\\d|25[0-5])\\."+
                                               "(\\d|1?\\d\\d|2[0-4]\\d|25[0-5])\\.(\\d|1?\\d\\d|2[0-4]\\d|25[0-5])"+
                                               "(/(0?[1-9]|[1-2][0-9]|3[0-1]))?") ) {
                                Toolkit.getDefaultToolkit().beep();
                                ipAddrReserveTf.requestFocus();
                                return;
                            }

                            ipAddrReserveWin.dispose();
                            addrSpacesJList.clearSelection();
                            addrSpacesJList.ensureIndexIsVisible(0);
                            ipAddrReserveTf.setText("");
                            ipAddrNameReserveTf.setText("");
                            ipAddrActiveRes = ipAddrActiveCB.isSelected();
                            ipAddrActiveCB.setSelected(true);

                            // prepare for reserving
                            msg.setText("Reserving ip address...");
                            central_panel.removeAll();
                            central_panel.add(msgPanel, BorderLayout.CENTER);
                            central_panel.revalidate();
                            central_panel.repaint();
                            reloadB.setEnabled(false);
                            modifyB.setEnabled(false);
                            deleteB.setEnabled(false);
                            header_panel.remove(ePanel);
                            ePanelLabel.setText("");
                            header_panel.revalidate();

                            // do reserve
                            (new NodeThread(THIS, NodeThread.RESERVE_NAME_AND_IP)).start();
                        }
                    });

                ipAddrReserveButtons[1] = new BButton("Cancel");
                ipAddrReserveButtons[1].addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            ipAddrReserveWin.dispose();
                            addrSpacesJList.clearSelection();
                            addrSpacesJList.ensureIndexIsVisible(0);
                            ipAddrReserveTf.setText("");
                            ipAddrActiveCB.setSelected(true);
                        }
                    });

                ipAddrReserveDialog =
                    new JOptionPane(ipAddrReservePanel, JOptionPane.PLAIN_MESSAGE,
                                    JOptionPane.YES_NO_OPTION, null, ipAddrReserveButtons);

            }
            current_panel = mainPanel;
            current_panel.setLayout(current_gb_layout);

            current_gbc.fill = GridBagConstraints.HORIZONTAL;
            current_gbc.weightx = 1.0;
            current_gbc.weighty = 0;
            current_gbc.gridx = 0;
            current_gbc.gridy = 0;

            line();

            // gray panel with node types
            JPanel header_panel = new JPanel(new BorderLayout());
            JPanel left_hp = new JPanel(new FlowLayout());
            left_hp.add(makeLabel("<html><body>Node information. (<font color=\"#FF0000\">*</font>"+
                                  " - indicates required field)</body></html>"));
            right_hp = new JPanel(new FlowLayout(FlowLayout.RIGHT));

            Hashtable node_types = (Hashtable)newNode.get("types");
            if( node_types == null ) {
                node_types = new Hashtable();
                newNode.put("types", node_types);
            }
            adv_cb = new AJCheckBox("Advanced", node_types.get("Advanced")!=null?true:false);
            adv_cb.setFont(label_font);
            adv_cb.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 0));
            adv_cb.setBackground(right_hp.getBackground());
            rout_cb = new AJCheckBox("Router", node_types.get("Router")!=null?true:false);
            rout_cb.setFont(label_font);
            rout_cb.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 0));
            rout_cb.setBackground(right_hp.getBackground());
            ipc_cb = new AJCheckBox("IPC", node_types.get("IPCprovider")!=null?true:false);
            ipc_cb.setFont(label_font);
            ipc_cb.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 0));
            ipc_cb.setBackground(right_hp.getBackground());
            tmpl_cb = new AJCheckBox("Template", node_types.get("Template")!=null?true:false);
            tmpl_cb.setFont(label_font);
            tmpl_cb.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 0));
            tmpl_cb.setBackground(right_hp.getBackground());

            if( node_types.get("Template") == null ) {
                if( advNodeAccess )
                    right_hp.add(adv_cb);
                if( routerAccess )
                    right_hp.add(rout_cb);
                if( ipcAccess )
                    right_hp.add(ipc_cb);
            }

            if( node_types.get("Advanced") == null &&
                node_types.get("Router") == null &&
                node_types.get("IPCprovider") == null )
                right_hp.add(tmpl_cb);
            adv_cb.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent event) {
                        JCheckBox cb = (JCheckBox)event.getSource();
                        if( cb.isSelected() ) {
                            // make sure that first panel has add_another_name_b
                            java.util.List nlist = Arrays.asList(namePanels.keySet().toArray());
                            Collections.sort(nlist, name_cmp);
                            Hashtable firstPanelStore = (Hashtable)namePanels.get(nlist.iterator().next());
                            JPanel smallNamePanel = (JPanel)firstPanelStore.get("smallNamePanel");
                            smallNamePanel.add(add_another_name_b);

                            Enumeration enm = interfacesStore.keys();
                            while( enm.hasMoreElements() ) {
                                Hashtable intfcStore = (Hashtable)interfacesStore.get(enm.nextElement());
                                ((JLabel)intfcStore.get("hw_name_label")).setVisible(true);
                                ((JTextField)intfcStore.get("hw_name_tf")).setVisible(true);
                                Hashtable ipsStore = (Hashtable)intfcStore.get("ipsStore");
                                Enumeration enm2 = ipsStore.keys();
                                while( enm2.hasMoreElements() ) {
                                    Hashtable ipStore = (Hashtable)ipsStore.get(enm2.nextElement());
                                    ((JLabel)ipStore.get("ip_name_label")).setVisible(true);
                                    ((AJTextField)ipStore.get("ip_name_tf")).setVisible(true);
                                }
                            }

                        } else { // remove all names except first one

                            // set up msg and clean for rebuilding
                            try {
                                collectAllData(false);
                            } catch (Exception ex) {
                                Utils.debug(ex.getMessage());
                            }

                            Hashtable names = (Hashtable)newNode.get("names");
                            java.util.List nlist = Arrays.asList(names.keySet().toArray());
                            Collections.sort(nlist, name_cmp);
                            String key = (String)nlist.iterator().next();
                            Hashtable name = (Hashtable)names.get(key);
                            names.clear();
                            names.put(key, name);

                            // remove interface and ip address names
                            Hashtable interfaces = Utils.nHash(newNode.get("interfaces"));
                            Enumeration enm = interfaces.keys();
                            while( enm.hasMoreElements() ) {
                                Hashtable intfc = (Hashtable)interfaces.get(enm.nextElement());
                                intfc.remove("names");
                                Hashtable ips = Utils.nHash(intfc.get("ips"));
                                Enumeration enm2 = ips.keys();
                                while( enm2.hasMoreElements() )
                                    ((Hashtable)ips.get(enm2.nextElement())).remove("names");
                            }

                            msg.setText("Changing node type...");
                            central_panel.removeAll();
                            central_panel.add(msgPanel, BorderLayout.CENTER);

                            reloadB.setEnabled(false);
                            modifyB.setEnabled(false);
                            deleteB.setEnabled(false);

                            mainPanel.removeAll();
                            central_panel.revalidate();
                            central_panel.repaint();
                            init = false; // rebuilding
                            (new Thread(THIS)).start();
                        }
                    }
                });
            adv_cb.addChangeListener(new ChangeListener() {
                    public void stateChanged(ChangeEvent event) {
                        JCheckBox cb = (JCheckBox)event.getSource();
                        Hashtable node_types = (Hashtable)newNode.get("types");
                        if( cb.isSelected() ) {
                            node_types.put("Advanced", "");
                            node_types.remove("Template");
                            right_hp.remove(tmpl_cb);
                            right_hp.revalidate();

                        } else {
                            node_types.remove("Advanced");
                            if( node_types.get("Router") == null &&
                                node_types.get("IPCprovider") == null ) {
                                right_hp.add(tmpl_cb);
                                right_hp.revalidate();
                            }
                        }
                    }
                });

            rout_cb.addChangeListener(new ChangeListener() {
                    public void stateChanged(ChangeEvent event) {
                        JCheckBox cb = (JCheckBox)event.getSource();
                        Hashtable node_types = (Hashtable)newNode.get("types");
                        if( cb.isSelected() ) {
                            node_types.put("Router", "");
                            node_types.remove("Template");
                            right_hp.remove(tmpl_cb);
                            right_hp.revalidate();

                        } else {
                            node_types.remove("Router");
                            if( node_types.get("Advanced") == null &&
                                node_types.get("IPCprovider") == null ) {
                                right_hp.add(tmpl_cb);
                                right_hp.revalidate();
                            }
                        }
                    }
                });

            ipc_cb.addChangeListener(new ChangeListener() {
                    public void stateChanged(ChangeEvent event) {
                        JCheckBox cb = (JCheckBox)event.getSource();
                        Hashtable node_types = (Hashtable)newNode.get("types");
                        if( cb.isSelected() ) {
                            node_types.put("IPCprovider", "");
                            node_types.remove("Template");
                            right_hp.remove(tmpl_cb);
                            right_hp.revalidate();

                            newNode.put("ipc_addresses", new Hashtable());

                            ipc_header_panel.setVisible(true);
                            ipc_separator_line.setVisible(true);
                            ipc_big_panel.setVisible(true);
                            ipc_bottom_empty_label.setVisible(true);
                            current_panel.revalidate();

                        } else {
                            node_types.remove("IPCprovider");
                            if( node_types.get("Advanced") == null &&
                                node_types.get("Router") == null ) {
                                right_hp.add(tmpl_cb);
                                right_hp.revalidate();
                            }
                            newNode.remove("ipc_addresses");
                            ipc_ips_panel.removeAll();
                            ipc_ip_layout.setRows(1);
                            ipc_ips_panel.add(ipc_add_another_ip_panel);

                            ipc_header_panel.setVisible(false);
                            ipc_separator_line.setVisible(false);
                            ipc_big_panel.setVisible(false);
                            ipc_bottom_empty_label.setVisible(false);
                            current_panel.revalidate();
                        }
                    }
                });

            tmpl_cb.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent event) {
                        JCheckBox cb = (JCheckBox)event.getSource();
                        if( cb.isSelected() ) {
                            // at this point node has only one name
                            // remove aliases and mxs
                            Hashtable name = (Hashtable)((Hashtable)newNode.get("names")).get((String)namePanels.keys().nextElement());
                            Utils.nHash(name.get("aliases")).clear();
                            Utils.nHash(name.get("mxs")).clear();

                            // remove interfaces
                            interfacesStore.clear();
                            Utils.nHash(newNode.get("interfaces")).clear();
                        } else
                            newNode.remove("addr_space");

                        // set up msg and clean for rebuilding
                        try {
                            collectAllData(false);
                        } catch(Exception ex) {
                            Utils.debug(ex.getMessage());
                        }
                        msg.setText("Changing node type...");
                        central_panel.removeAll();
                        central_panel.add(msgPanel, BorderLayout.CENTER);

                        reloadB.setEnabled(false);
                        modifyB.setEnabled(false);
                        deleteB.setEnabled(false);

                        mainPanel.removeAll();
                        central_panel.revalidate();
                        central_panel.repaint();
                        init = false; // rebuilding
                        (new Thread(THIS)).start();
                    }
                });

            tmpl_cb.addChangeListener(new ChangeListener() {
                    public void stateChanged(ChangeEvent event) {
                        JCheckBox cb = (JCheckBox)event.getSource();
                        Hashtable node_types = (Hashtable)newNode.get("types");
                        if( cb.isSelected() ) {
                            node_types.put("Template", "");
                            node_types.remove("Advanced");
                            node_types.remove("Router");
                            node_types.remove("IPCprovider");

                            right_hp.remove(adv_cb);
                            right_hp.remove(rout_cb);
                            right_hp.remove(ipc_cb);
                            right_hp.revalidate();

                        } else {
                            node_types.remove("Template");

                            right_hp.removeAll();

                            if( advNodeAccess )
                                right_hp.add(adv_cb);
                            if( routerAccess )
                                right_hp.add(rout_cb);
                            if( ipcAccess )
                                right_hp.add(ipc_cb);
                            right_hp.add(tmpl_cb);
                            right_hp.revalidate();
                        }
                    }
                });

            header_panel.add(left_hp, BorderLayout.WEST);
            header_panel.add(right_hp, BorderLayout.EAST);

            current_gbc.gridwidth = 2;
            add(header_panel);
            current_gbc.gridwidth = 1;
            tr();

            line();

            XLabel(" ");
            tr();

            makeNamesPanels(false);

            line();

            XLabel(" ");
            tr();

            HTMLPanel("Department <font color=\"#ff0000\">*</font>");
            td();
            deps_combo = new JComboBox();

            java.util.List list = Arrays.asList(allDepartmentsH.keySet().toArray());
            Collections.sort(list, name_cmp);
            Iterator it = list.iterator();
            int index = 0;
            while( it.hasNext() ) {
                String dep = ((String)it.next()).trim();
                deps_combo.addItem(dep);
                if( nodeDep.equals(dep) ) {
                    deps_combo.setSelectedIndex(index);
                    oldNode_deps_index = index;
                }
                index++;
            }

            if( nodeDep == null || nodeDep.length() == 0 ) {
                nodeDep = ((String)deps_combo.getSelectedItem()).trim();
                if( !nodeDep.equals("--") )
                    newNode.put("dept", nodeDep);
            }

            add(deps_combo, GridBagConstraints.NONE, GridBagConstraints.WEST);
            tr();
            deps_combo.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        String dep = (String)deps_combo.getSelectedItem();
                        if( dep.equals("--") )
                            newNode.remove("dept");
                        else {
                            newNode.put("dept", dep);
                            if( ((String)deps_combo.getItemAt(0)).equals("--") )
                                deps_combo.removeItemAt(0);
                        }
                    }
                });

            XLabel("");
            tr();

            HTMLPanel("Location <font color=\"#ff0000\">*</font>");
            td();
            loc_combo = new JComboBox();
            list = Arrays.asList(locationsH.keySet().toArray());
            Collections.sort(list, name_cmp);
            it = list.iterator();
            index = 0;
            while( it.hasNext() ) {
                String location = ((String)it.next()).trim();
                loc_combo.addItem(location);
                if( nodeLocation.equals(location) ) {
                    loc_combo.setSelectedIndex(index);
                    oldNode_location_index = index;
                }
                index++;
            }

            if( nodeLocation == null || nodeLocation.length() == 0 ) {
                nodeLocation = ((String)loc_combo.getSelectedItem()).trim();
                if( !nodeLocation.equals("--") )
                    newNode.put("location", nodeLocation);
            }

            add(loc_combo, GridBagConstraints.NONE, GridBagConstraints.WEST);
            tr();
            loc_combo.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        String location = (String)loc_combo.getSelectedItem();
                        if( location.equals("--") )
                            newNode.remove("location");
                        else {
                            newNode.put("location", location);
                            if( ((String)loc_combo.getItemAt(0)).equals("--") )
                                loc_combo.removeItemAt(0);
                        }
                    }
                });

            XLabel("");
            tr();

            HTMLPanel("Room <font color=\"#ff0000\">*</font>");
            td();
            room_tf = new JTextField((String)newNode.get("room"), 10);
            room_tf.setFont(label_font);
            add(room_tf, GridBagConstraints.NONE, GridBagConstraints.WEST);
            tr();

            XLabel("");
            tr();

            HTMLPanel("Make and Model <font color=\"#ff0000\">*</font>");
            td();
            makes_and_models_H = new Hashtable();
            make_and_model_combo = new JComboBox();
            String nodeMake = ((String)newNode.get("make")).trim();
            String nodeModel = ((String)newNode.get("model")).trim();
            list = Arrays.asList(modelsH.keySet().toArray());
            Collections.sort(list, name_cmp);
            it = list.iterator();
            index = 0;
            while( it.hasNext() ) {
                Hashtable make_and_modelH = (Hashtable)modelsH.get((String)it.next());
                String make = (String)make_and_modelH.get("make");
                String model = (String)make_and_modelH.get("model");
                String newKey = make+" "+model;
                makes_and_models_H.put(newKey, make_and_modelH);
                make_and_model_combo.addItem(newKey);
                if( nodeMake.equals(make) && nodeModel.equals(model) ) {
                    make_and_model_combo.setSelectedIndex(index);
                    oldNode_make_and_model_index = index;
                }
                index++;
            }

            if( nodeMake == null || nodeMake.length() == 0 ) {
                Hashtable make_and_model = (Hashtable)makes_and_models_H.get((String)make_and_model_combo.getSelectedItem());
                nodeMake = (String)make_and_model.get("make");
                newNode.put("make", nodeMake);
            }

            if( nodeModel == null || nodeModel.length() == 0 ) {
                Hashtable make_and_model = (Hashtable)makes_and_models_H.get((String)make_and_model_combo.getSelectedItem());
                nodeModel = (String)make_and_model.get("model");
                newNode.put("model", nodeModel);
            }

            add(make_and_model_combo, GridBagConstraints.NONE, GridBagConstraints.WEST);
            tr();

            make_and_model_combo.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        String key = (String)make_and_model_combo.getSelectedItem();
                        if( key.trim().equals("--") ) {
                            newNode.remove("make");
                            newNode.remove("model");
                        } else {
                            Hashtable make_and_model = (Hashtable)makes_and_models_H.get(key);
                            newNode.put("make", (String)make_and_model.get("make"));
                            newNode.put("model", (String)make_and_model.get("model"));
                            if( ((String)make_and_model_combo.getItemAt(0)).trim().equals("--") )
                                make_and_model_combo.removeItemAt(0);
                        }
                    }
                });

            XLabel("");
            tr();

            makeOSesPanel();

            XLabel("");
            tr();

            makeGroupsPanel();

            XLabel("");
            tr();

            makeAminsPanel();

            XLabel("");
            tr();

            makeUsersPanel();

            XLabel("");
            tr();

            current_gbc.anchor = GridBagConstraints.NORTHWEST;
            HTMLPanel("Custom Fields");
            td();
            JPanel c_panel = new JPanel(new GridLayout(3, 1, 0, 5));
            c_panel.setBackground(Color.white);
            c_panel.add(makeLabel("Field Name   Field Value"));
            JPanel cf1 = new JPanel();
            cf1.setBackground(Color.white);

            String tagName[] = {"", ""};
            String tagValue[] = {"", ""};

            Hashtable node_tags = Utils.nHash(newNode.get("tags"));
            list = Arrays.asList(node_tags.keySet().toArray());
            Collections.sort(list, name_cmp);
            it = list.iterator();
            index = 0;
            while( it.hasNext() ) {
                tagName[index] = (String)it.next();
                tagValue[index] = (String)node_tags.get(tagName[index]);
                index++;
                if( index == 2 )
                    break;
            }
            tag1_name = new JTextField(tagName[0], 12);
            tag1_name.setFont(label_font);
            cf1.add(tag1_name);
            tag1_value = new JTextField(tagValue[0], 25);
            tag1_value.setFont(label_font);
            cf1.add(tag1_value);
            c_panel.add(cf1);

            cf1 = new JPanel();
            cf1.setBackground(Color.white);
            tag2_name = new JTextField(tagName[1], 12);
            tag2_name.setFont(label_font);
            cf1.add(tag2_name);
            tag2_value = new JTextField(tagValue[1], 25);
            tag2_value.setFont(label_font);
            cf1.add(tag2_value);
            c_panel.add(cf1);

            add(c_panel, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST);
            tr();

            XLabel("");
            tr();

            line();

            if( Utils.nHash(newNode.get("types")).get("Template") != null ) {
                JPanel panel = new JPanel(new BorderLayout());
                JPanel left_panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
                panel.add(left_panel, BorderLayout.WEST);
                left_panel.add(makeLabel("<html><body>&nbsp;&nbsp;Template Default Address Space</body></html>"));

                current_gbc.gridwidth = 2;
                add(panel);
                current_gbc.gridwidth = 1;
                tr();

                line();

                tmplAddrSpacesJCombo = new JComboBox();
                list = Arrays.asList(addrSpacesH.keySet().toArray());
                Collections.sort(list, number_cmp);
                it = list.iterator();
                index = 1;
                tmplAddrSpacesJCombo.addItem("---");
                while( it.hasNext() ) {
                    String asPrefix = (String)((Hashtable)addrSpacesH.get(it.next())).get("prefix");
                    tmplAddrSpacesJCombo.addItem(asPrefix);
                    if( asPrefix.equals((String)newNode.get("addr_space")) )
                        tmplAddrSpacesJCombo.setSelectedIndex(index);
                    index++;
                }
                tmplAddrSpacesJCombo.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        String asStr = (String)tmplAddrSpacesJCombo.getSelectedItem();
                        if( asStr.equals("---") )
                            newNode.remove("addr_space");
                        else
                            newNode.put("addr_space", asStr);
                    }
                });

                XLabel("");
                tr();

                td();
                panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
                panel.setBackground(Color.white);
                panel.add(tmplAddrSpacesJCombo);
                add(panel, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
                tr();

                XLabel("");
                tr();
            }

            if( Utils.nHash(newNode.get("types")).get("Template") == null ) {
                // make interface panels
                JPanel intfc_header_panel = new JPanel(new BorderLayout());
                JPanel left_panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
                intfc_header_panel.add(left_panel, BorderLayout.WEST);
                JPanel right_panel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
                intfc_header_panel.add(right_panel, BorderLayout.EAST);
                left_panel.add(makeLabel("<html><body>&nbsp;&nbsp;Interface information</body></html>"));
                add_intfc_b = new AJButton("Add another interface", null, AJButton.NOT_MOUSABLE);
                add_intfc_b.setMargin(new Insets(0, 0, 0, 0));
                right_panel.add(add_intfc_b);
                add_intfc_b.addActionListener(new ActionListener(){
                        public void actionPerformed(ActionEvent event) {
                            try {
                                int newIntfcID;
                                synchronized( counter ) {
                                    newIntfcID = Integer.parseInt(counter.substring(1));
                                    counter.replace(1, counter.length(), ""+(newIntfcID+1));
                                }
                                Hashtable interfaces = (Hashtable)newNode.get("interfaces");
                                if( interfaces == null ) {
                                    interfaces = new Hashtable();
                                    newNode.put("interfaces", interfaces);
                                }
                                Hashtable intfc = new Hashtable();
                                intfc.put("roaming?", new Boolean(false));
                                intfc.put("use-dhcp?", new Boolean(false));
                                interfaces.put("-"+newIntfcID, intfc);

                                collectAllData(false);

                                // set up msg and clean for rebuilding
                                msg.setText("Adding interface...");
                                central_panel.removeAll();
                                central_panel.add(msgPanel, BorderLayout.CENTER);

                                reloadB.setEnabled(false);
                                modifyB.setEnabled(false);
                                deleteB.setEnabled(false);

                                mainPanel.removeAll();
                                central_panel.revalidate();
                                central_panel.repaint();
                                init = false; // rebuilding
                                (new Thread(THIS)).start();

                            } catch (NetDBException exx) {
                                exx.printStackTrace();
                                Vector errors = exx.getMessages();
                                String msg = "<html><body>";
                                int esize = errors.size();
                                for( int i = 0; i < esize; i++ )
                                    msg += "<font color=\"#ff0000\">&nbsp;Error: "+(String)errors.elementAt(i)+
                                        "</font>"+(i<esize-1?"<br>":"");
                                ePanelLabel.setText(msg+"</body></html>");
                                THIS.header_panel.add(ePanel);
                                THIS.header_panel.revalidate();

                            } catch (Exception exx) {
                                exx.printStackTrace();
                                ePanelLabel.setText("<html><body><font color=\"#ff0000\">&nbsp;Error: "+
                                                    exx.getMessage()+"</font></body></html>");
                                THIS.header_panel.add(ePanel);
                                THIS.header_panel.revalidate();
                            }
                        }
                    });

                current_gbc.gridwidth = 2;
                add(intfc_header_panel);
                current_gbc.gridwidth = 1;
                tr();

                line();

                Hashtable interfaces = Utils.nHash(newNode.get("interfaces"));
                list = Arrays.asList(interfaces.keySet().toArray());
                Collections.sort(list, number_cmp);
                it = list.iterator();
                index = 1;
                while( it.hasNext() ) {
                    String intfcID = (String)it.next();
                    JPanel infcPanel = MakeInterfacePanel(index++, intfcID);
                    current_gbc.gridwidth = 2;
                    add(infcPanel, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
                    current_gbc.gridwidth = 1;
                    tr();

                    XLabel("");
                    tr();
                }

                if( interfaces.size() > 0 )
                    line();
            } // end of if( Utils.nHash(newNode.get("types")).get("Template") == null )

            // add ipc panel if applicable
            makeIPCPanel(false);

            if( Utils.nHash(newNode.get("types")).get("IPCprovider") == null ) {
                ipc_header_panel.setVisible(false);
                ipc_separator_line.setVisible(false);
                ipc_big_panel.setVisible(false);
                ipc_bottom_empty_label.setVisible(false);
            }

            line();

            // gray panel
            JPanel record_panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
            label = makeLabel("<html><body>&nbsp;&nbsp;Record information</body></html>");
            record_panel.add(label);
            current_gbc.gridwidth = 2;
            add(record_panel);
            current_gbc.gridwidth = 1;
            tr();

            line();

            XLabel("");
            tr();

            HTMLPanel("Expiration Date");
            td();

            Timestamp exp_date = (Timestamp)newNode.get("exp.date");
            exp_date_tf = new JTextField();
            exp_date_tf.addFocusListener(new FocusAdapter() {
                    public void focusLost(FocusEvent e) {
                        String exp_date = exp_date_tf.getText().trim();
                        if( exp_date.length() == 0 )
                            newNode.remove("exp.date");
                        else {
                            try {
                                if( !exp_date.matches("[0123456789/]*") )
                                    throw new Exception("");
                                DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT);
                                df.setLenient(false);
                                Timestamp exp_date_ts = new Timestamp(df.parse(exp_date).getTime());
                                newNode.put("exp.date", exp_date_ts);
                            } catch (Exception ex) { }
                        }
                    }
                });
            if( exp_date != null )
                exp_date_tf.setText((new SimpleDateFormat("MM/dd/yyyy")).format(exp_date));
            exp_date_tf.setColumns(10);
            JPanel exp_panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
            exp_panel.setBackground(Color.white);
            exp_panel.add(exp_date_tf);
            label = makeLabel("<html><body><font size=\"-1\"> (mm/dd/yyyy) Informational only.</font></body></html>");
            exp_panel.add(label);
            add(exp_panel, GridBagConstraints.NONE, GridBagConstraints.WEST);
            tr();

            XLabel("");
            tr();

            current_gbc.anchor = GridBagConstraints.NORTHWEST;
            HTMLPanel("Comment");
            td();

            comment_ta = new JTextArea(3, 35);
            comment_ta.setFont(label_font);
            String comment = (String)newNode.get("comment");
            if( comment == null )
                comment = "";
            comment_ta.setText(comment);
            comment_ta.setEditable(true);
            JScrollPane sp = new JScrollPane(comment_ta);
            add(sp, GridBagConstraints.NONE, GridBagConstraints.WEST);
            tr();

            central_panel.removeAll();
            central_panel.add(mainPanel, BorderLayout.NORTH);
            central_panel.revalidate();
            central_panel.repaint();

            modifyB.setEnabled(true);
            deleteB.setEnabled(true);
            reloadB.setEnabled(true);

            if( !closed && open_as_pickup && open_as_pickup_b != null ) {
                open_as_pickup = false;
                String default_addr_space = (String)MainWindow.user.get("default_addr_space");
                if( default_addr_space != null ) {
                    // show warning message
                    ePanelLabel.setText("<html><body><font color=\"#ff0000\">&nbsp;Warning: no more available ip addresses in your def. addr. space \""+default_addr_space.trim()+"\"</body></html>");
                    super.header_panel.add(ePanel);
                    super.header_panel.revalidate();
                }
                //open_as_pickup_b.doClick();
            }

        } catch(java.rmi.ServerException exx) {
            if( exx.getCause() instanceof SessionTimeoutException )
                X.handleTimeout();

            else if( exx.getCause() instanceof RemoteNetDBException ) {
                RemoteNetDBException ne = (RemoteNetDBException)exx.getCause();
                Vector errors = ne.getMessages();
                String[] msg = new String[errors.size()];
                for( int i = 0; i < errors.size(); i++ )
                    msg[i] = (String)errors.elementAt(i);
                JOptionPane.showMessageDialog(THIS,
                    msg, "Error", JOptionPane.ERROR_MESSAGE);

            } else if( exx.getCause() instanceof RemoteException ) {
                RemoteException re = (RemoteException)exx.getCause();
                JOptionPane.showMessageDialog(THIS,
                    re.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);

            } else {
                JOptionPane.showMessageDialog(THIS,
                    exx.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
            }
            closeB.doClick();
            return;

        } catch (java.rmi.NoSuchObjectException exx) {
            X.handleTimeout();

        } catch (java.rmi.MarshalException exx) {
            X.handleTimeout();

        } catch(Exception exx) {
            JOptionPane.showMessageDialog(THIS,
                    exx.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
            closeB.doClick();
            return;
        }
    } // end of run()

    // collects all data from all text fields and areas
    private void collectAllData(boolean for_verify) throws Exception, NetDBException {
        boolean advanced = Utils.nHash(newNode.get("types")).get("Advanced")==null?false:true;
        newNode.put("room", room_tf.getText().trim());
        Hashtable tags = (Hashtable)newNode.get("tags");
        if( tags == null ) {
            tags = new Hashtable();
            newNode.put("tags", tags);
        } else
            tags.clear();
        String tag_name = ((String)tag1_name.getText()).trim();
        String tag_value = ((String)tag1_value.getText()).trim();
        if( tag_name.length() > 0 )
            tags.put(tag_name, tag_value);
        tag_name = ((String)tag2_name.getText()).trim();
        tag_value = ((String)tag2_value.getText()).trim();
        if( tag_name.length() > 0 ) {
            if( tags.put(tag_name, tag_value) != null && for_verify ) {
                throw new Exception("Duplicate tag");
            }
        }
        String exp_date = exp_date_tf.getText().trim();
        if( exp_date.length() == 0 )
            newNode.remove("exp.date");
        else {
            try {
                if( !exp_date.matches("[0123456789/]*") && for_verify )
                    throw new Exception("");
                DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT);
                df.setLenient(false);
                Timestamp exp_date_ts = new Timestamp(df.parse(exp_date).getTime());
                newNode.put("exp.date", exp_date_ts);
            } catch (Exception ex) {
                if( for_verify )
                    throw new Exception("Expiration date is not valid");
            }
        }
        newNode.put("comment", comment_ta.getText().trim());

        // collect all old names
        allOldNames = new Hashtable();
        allOldIPs = new Hashtable();
        collectNamesAndIPs(newNode, allOldNames, allOldIPs, null, EXCLUDE_DIRTY);

        ipTextFields = new Hashtable();

        // refill names and ips
        Vector errorMessages = new Vector();
        Hashtable oldIPsAddresses = new Hashtable();
        Hashtable oldNodeNames = new Hashtable();
        try {
            oldNodeNames = Utils.nHash(newNode.remove("names"));
            Hashtable nodeNames = new Hashtable();
            newNode.put("names", nodeNames);
            Enumeration enm = namePanels.keys();
            while( enm.hasMoreElements() ) {
                Hashtable panelStore = (Hashtable)namePanels.get(enm.nextElement());
                JTextField name_tf = (JTextField)panelStore.get("name_tf");
                String panelKey = (String)panelStore.get("key");
                String nodeName = name_tf.getText().trim();
                if( nodeName.length() > 0 ) {
                    if( nodeName.indexOf(".") < 0 ) {
                        // it does not have domain part
                        nodeName += "."+MainWindow.userDefDomain;
                    }
                    Hashtable nameH = new Hashtable();
                    nameH.put("name", Utils.name(nodeName));
                    nameH.put("domain", Utils.domain(nodeName, MainWindow.userDefDomain));
                    if( !for_verify )
                        nameH.put("dirty", "");
                    nodeNames.put(panelKey, nameH);

                    // aliases
                    Hashtable aliases = new Hashtable();
                    Hashtable aPanelStore = Utils.nHash(panelStore.get("aPanelStore"));
                    Enumeration enm2 = aPanelStore.keys();
                    while( enm2.hasMoreElements() ) {
                        AJTextField alias_tf = (AJTextField)aPanelStore.get(enm2.nextElement());
                        String aliasName = alias_tf.getText().trim();
                        if( aliasName.length() > 0 ) {
                            if( aliasName.indexOf(".") < 0 ) {
                                // it does not have domain part
                                aliasName += "."+MainWindow.userDefDomain;
                            }
                            Hashtable aliasH = new Hashtable();
                            aliasH.put("name", Utils.name(aliasName));
                            aliasH.put("domain", Utils.domain(aliasName, MainWindow.userDefDomain));
                            if( !for_verify )
                                aliasH.put("dirty", "");
                            if( aliases.get(aliasName) != null )
                                errorMessages.addElement("Name (<i>"+aliasName+"</i>) used more then once");
                            aliases.put(aliasName, aliasH);
                            alias_tf.key = aliasName;
                        }
                    }
                    if( aliases.size() > 0 )
                        nameH.put("aliases", aliases);

                    // mxs
                    Hashtable mxes = new Hashtable();
                    Hashtable mxsPanelStore = Utils.nHash(panelStore.get("mxsPanelStore"));
                    enm2 = mxsPanelStore.keys();
                    while( enm2.hasMoreElements() ) {
                        Hashtable mxStore = (Hashtable)mxsPanelStore.get(enm2.nextElement());
                        AJTextField mx_tf = (AJTextField)mxStore.get("mx_tf");
                        String mxName = mx_tf.getText().trim();
                        if( mxName.length() > 0 ) {
                            if( mxName.indexOf(".") < 0 ) {
                                // it does not have domain part
                                mxName += "."+MainWindow.userDefDomain;
                            }
                            Hashtable mxH = new Hashtable();
                            mxH.put("name", Utils.name(mxName));
                            mxH.put("domain", Utils.domain(mxName, MainWindow.userDefDomain));
                            if( !for_verify )
                                mxH.put("dirty", "");
                            JTextField pref_tf = (JTextField)mxStore.get("pref_tf");
                            try {
                                mxH.put("pref", ""+Integer.parseInt(pref_tf.getText().trim()));
                            } catch(Exception ex) {
                                mxH.put("pref", "10");
                            }
                            if( mxes.get(mxName) != null )
                                errorMessages.addElement("Name (<i>"+mxName+"</i>) used more then once");
                            mxes.put(mxName, mxH);
                            mx_tf.key = mxName;
                        }
                    }
                    if( mxes.size() > 0 )
                        nameH.put("mxs", mxes);
                } else {
                    Hashtable nameH = new Hashtable();
                    nameH.put("name", "");
                    nameH.put("domain", "");
                    nameH.put("alaises", new Hashtable());
                    nameH.put("mxs", new Hashtable());
                    nodeNames.put(panelStore.get("key"), nameH);
                }
            }

            // now ipc
            oldIPsAddresses = Utils.nHash(newNode.remove("ipc_addresses"));
            Hashtable ipc_addresses = new Hashtable();
            Component[] ipc_components = ipc_ips_panel.getComponents();
            for( int i = 0; i < ipc_components.length; i++ ) {
                if( !(ipc_components[i] instanceof JPanel) )
                    continue;

                Component[] ip_panel_components = ((JPanel)ipc_components[i]).getComponents();
                if( ip_panel_components.length == 6 ) {
                    // ip
                    AJFormattedTextField ipTF = (AJFormattedTextField)ip_panel_components[2];
                    String ipStr = ipTF.getText().trim();
                    if( ipStr.length() == 0 )
                        continue;

                    if( ipStr.indexOf("/") > -1 ) {
                        String ipPart = ipStr.substring(0, ipStr.indexOf("/"));
                        int dots = countDots(ipStr);
                        for( int j = 0; j<3-dots; j++ )
                            ipPart += ".0";
                        ipStr = ipPart + ipStr.substring(ipStr.indexOf("/"));
                    } else {
                        int dots = countDots(ipStr);
                        for( int j = 0; j<3-dots; j++ )
                            ipStr += ".0";
                    }

                    ipTF.setText(ipStr);
                    ipStr = ipTF.getText().trim();

                    ipTextFields.put(ipStr, ipTF);

                    Hashtable ipH = new Hashtable();
                    if( ipc_addresses.put(ipStr, ipH) != null )
                        errorMessages.addElement("IP address (<i>"+ipStr+"</i>) used more then once");

                    // active? flag
                    ipH.put("active?", new Boolean(((AJCheckBox)ip_panel_components[3]).isSelected()));

                    // name field
                    String ipcNameStr = ((AJTextField)ip_panel_components[5]).getText().trim();
                    if( ipcNameStr.length() > 0 ) {
                        if( ipcNameStr.indexOf(".") < 0 ) {
                            // it does not have domain part
                            ipcNameStr += "."+MainWindow.userDefDomain;
                        }
                        Hashtable nameH = new Hashtable();
                        nameH.put("name", Utils.name(ipcNameStr));
                        nameH.put("domain", Utils.domain(ipcNameStr, MainWindow.userDefDomain));
                        if( !for_verify )
                            nameH.put("dirty", "");
                        Hashtable names = new Hashtable();
                        ipH.put("names", names);
                        if( names.get(ipcNameStr) != null )
                            errorMessages.addElement("Name (<i>"+ipcNameStr+"</i>) used more then once");
                        names.put(ipcNameStr, nameH);
                    }
                }
            }

            if( ipc_addresses.size() > 0 )
                newNode.put("ipc_addresses", ipc_addresses);

            // now interfaces
            enm = interfacesStore.keys();
            while( enm.hasMoreElements() ) {
                String intfcKey = (String)enm.nextElement();
                Hashtable intfcStore = (Hashtable)interfacesStore.get(intfcKey);
                Hashtable intfc = (Hashtable)((Hashtable)newNode.get("interfaces")).get(intfcStore.get("id"));

                // first hw address
                JTextField hw_tf  = (JTextField)intfcStore.get("hw_tf");
                String hwStr = hw_tf.getText().trim().toLowerCase();
                // collect hw address
                if( hwStr.length() > 0 ) {
                    Hashtable parseRes = parser.parse(hwStr);
                    try {
                        hwStr = (String)parseRes.get("hw");
                    } catch (Exception exx) { }
                    if( hwStr == null )
                        errorMessages.addElement("Incorrect format for hardware address");

                    if( hwStr != null && hwStr.length() > 0 ) {
                        String oldHW = (String)intfc.put("hw", hwStr);
                        if( oldHW != null )
                            intfc.put("old_hw", oldHW);
                    }
                } else {
                    String oldHW = (String)intfc.get("hw");
                    if( oldHW != null )
                        intfc.put("old_hw", oldHW);
                    intfc.remove("hw");
                }

                if( advanced ) {
                    // second interface name
                    JTextField hw_name_tf = (JTextField)intfcStore.get("hw_name_tf");
                    String intfcNameStr = hw_name_tf.getText().trim();
                    intfc.put("old_names", Utils.nHash(intfc.remove("names")));
                    if( intfcNameStr.length() > 0 ) {
                        Hashtable intfcNames = (Hashtable)intfc.get("names");
                        if( intfcNames == null ) {
                            intfcNames = new Hashtable();
                            intfc.put("names", intfcNames);
                        }
                        if( intfcNameStr.indexOf(".") < 0 ) // it does not have domain part
                            intfcNameStr += "."+MainWindow.userDefDomain;

                        Hashtable intfcName = new Hashtable();
                        if( intfcNames.get(intfcNameStr) != null )
                            errorMessages.addElement("Name (<i>"+intfcNameStr+"</i>) used more then once");
                        intfcNames.put(intfcNameStr, intfcName);
                        intfcName.put("name", Utils.name(intfcNameStr));
                        intfcName.put("domain", Utils.domain(intfcNameStr, MainWindow.userDefDomain));
                        if( !for_verify )
                            intfcName.put("dirty", "");
                    }
                }

                // now dhcp options
                JTextArea dhcp_options_ta = (JTextArea)intfcStore.get("dhcp_options_ta");
                if( dhcp_options_ta != null ) {
                    String dhcp_text = dhcp_options_ta.getText();
                    String[] split = dhcp_text.split("\\n");
                    if( dhcp_text.length() > 0 ) {
                        Hashtable options = new Hashtable();
                        for( int i = 0; i < split.length; i++ ) {
                            if( split[i].trim().length() == 0 )
                                continue;
                            String[] ops = split[i].split("=", 2);
                            if( ops.length != 2 && for_verify )
                                throw new Exception("Invalid dhcp options");
                            Hashtable optionH = new Hashtable();
                            optionH.put("option", ops[0].trim());
                            optionH.put("value", ops[1].trim());
                            options.put(ops[0].trim()+"="+ops[1].trim(), optionH);
                        }
                        intfc.put("dhcp-options", options);
                    } else
                        intfc.put("dhcp-options", new Hashtable());
                }

                // now on to ip addresses
                Hashtable old_ips = Utils.nHash(intfc.remove("ips"));
                intfc.put("old_ips", old_ips);
                Hashtable ips = new Hashtable();
                JPanel ips_panel = (JPanel)intfcStore.get("ips_panel");
                Component[]ip_panel  = ips_panel.getComponents();
                for( int i = 0; i < ip_panel.length - 1; i++ ) {
                    Component[]ip_components = ((JPanel)ip_panel[i]).getComponents();

                    // ip address
                    AJFormattedTextField ipTF = (AJFormattedTextField)ip_components[2];
                    String ipStr = ipTF.getText().trim();
                    if( ipStr.length() == 0 )
                        continue;

                    if( ipStr.indexOf("/") > -1 ) {
                        String ipPart = ipStr.substring(0, ipStr.indexOf("/"));
                        int dots = countDots(ipStr);
                        for( int j = 0; j<3-dots; j++ )
                            ipPart += ".0";
                        ipStr = ipPart + ipStr.substring(ipStr.indexOf("/"));
                    } else {
                        int dots = countDots(ipStr);
                        for( int j = 0; j<3-dots; j++ )
                            ipStr += ".0";
                    }

                    ipTF.setText(ipStr);
                    ipStr = ipTF.getText().trim();

                    ipTextFields.put(ipStr, ipTF);

                    Hashtable ip = new Hashtable();

                    Hashtable old_ip = (Hashtable)old_ips.get(ipStr);
                    if( old_ip != null && old_ip.get("verified") != null )
                        ip.put("verified", "");

                    if( ips.put(ipStr, ip) != null )
                        errorMessages.addElement("IP addresss (<i>"+ipStr+"</i>) used more then once");

                    // active
                    AJCheckBox ip_cb = (AJCheckBox)ip_components[3];
                    ip_cb.obj2 = ipStr; // set up a new key
                    ip.put("active?", new Boolean(ip_cb.isSelected())); // refill active? flag

                    if( advanced ) {
                        // ip names
                        String ipNameStr = ((AJTextField)ip_components[5]).getText().trim();
                        if( ipNameStr.length() > 0 ) {
                            Hashtable ipNames = new Hashtable();
                            ip.put("names", ipNames);
                            if( ipNameStr.indexOf(".") < 0 ) // it does not have domain part
                                ipNameStr += "."+MainWindow.userDefDomain;
                            Hashtable ipName = new Hashtable();
                            ipName.put("name", Utils.name(ipNameStr));
                            ipName.put("domain", Utils.domain(ipNameStr, MainWindow.userDefDomain));
                            if( !for_verify )
                                ipName.put("dirty", "");
                            if( ipNames.get(ipNameStr) != null )
                                errorMessages.addElement("Name (<i>"+ipNameStr+"</i>) used more then once");
                            ipNames.put(ipNameStr, ipName);
                        }
                    }
                }

                if( ips.size() > 0 )
                    intfc.put("ips", ips);
            }

            if( for_verify ) {
                allNewNames = new Hashtable();
                allNewIPs = new Hashtable();

                collectNamesAndIPs(newNode, allNewNames, allNewIPs, errorMessages, INCLUDE_ALL);
            }
        } catch (Exception ex) {
            // probably duplicate name or an ip; reset all names and ips
            newNode.put("names", oldNodeNames);
            newNode.put("ipc_addresses", oldIPsAddresses);

            // reset interfaces
            // note, that it is guaranteed to have old_names and old_ips (see above)
            Enumeration enm = interfacesStore.keys();
            while( enm.hasMoreElements() ) {

                Hashtable intfc = (Hashtable)((Hashtable)newNode.get("interfaces")).
                    get(((Hashtable)interfacesStore.get(enm.nextElement())).get("id"));

                intfc.put("names", Utils.nHash(intfc.remove("old_names")));
                intfc.put("ips", Utils.nHash(intfc.remove("old_ips")));
                String oldHW = (String)intfc.remove("old_hw");
                if( oldHW != null )
                    intfc.put("hw", oldHW);
                else
                    intfc.remove("hw");
            }

            // !!! RESET KEYS IN IP ADDRESSES CHECK BOXES!!!
            // TO BE DONE

            if( for_verify )
                throw ex;
        }

        // now remove old_names and old_ips
        Enumeration enm = interfacesStore.keys();
        while( enm.hasMoreElements() ) {
            Hashtable intfc = (Hashtable)((Hashtable)newNode.get("interfaces")).
                get(((Hashtable)interfacesStore.get(enm.nextElement())).get("id"));

            intfc.remove("old_names");
            intfc.remove("old_ips");
        }

    } // end of collectAllData()

    private void makeNamesPanels(boolean rebuild) {
        // save grid
        int save_gridx = current_gbc.gridx;
        int save_gridy = current_gbc.gridy;
        namePanels = new Hashtable();

        current_gbc.anchor = GridBagConstraints.NORTHWEST;
        Hashtable names = (Hashtable)newNode.get("names");
        if( names == null ) {
            names = new Hashtable();
            newNode.put("names", names);
        }
        java.util.List list = Arrays.asList(names.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();
        boolean first = true;
        while( it.hasNext() ) {
            String key = (String)it.next();
            Hashtable nameH = (Hashtable)names.get(key);

            Hashtable panelStore = new Hashtable();
            panelStore.put("key", key);
            namePanels.put(key, panelStore);

            panelStore.put("space1", XLabel(" "));
            tr();

            JPanel titlePanel = HTMLPanel("Name <font color=\"#ff0000\">*</font>");
            panelStore.put("titlePanel", titlePanel);
            td();

            JPanel smallNamePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
            smallNamePanel.setBackground(Color.white);
            panelStore.put("smallNamePanel", smallNamePanel);
            add(smallNamePanel, GridBagConstraints.NONE, GridBagConstraints.WEST);

            JTextField name_tf = new JTextField("", 30);
            if( ((String)nameH.get("domain")).trim().toLowerCase().equals(MainWindow.userDefDomain) )
                name_tf.setText((String)nameH.get("name"));
            else if( ((String)nameH.get("domain")).trim().length() == 0 && ((String)nameH.get("name")).trim().length() == 0 )
                name_tf.setText("");
            else
                name_tf.setText(nameH.get("name")+"."+nameH.get("domain"));

            name_tf.setFont(label_font);
            panelStore.put("name_tf", name_tf);
            smallNamePanel.add(name_tf);

            if( names.size() > 1 ) {
                AJCheckBox nameCb = new AJCheckBox("", true, panelStore);
                panelStore.put("nameCb", nameCb);
                nameCb.setBackground(Color.white);
                nameCb.setFont(label_font);
                smallNamePanel.add(nameCb);
                nameCb.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                            if( !cb.isSelected() ) {
                                // remove associated data
                                String key = cb.getStr("key");
                                namePanels.remove(key);
                                if( namePanels.size() == 1 ) {
                                    // remove check box from ramaining name
                                    Enumeration enm = namePanels.keys();
                                    Hashtable lastPanelStore = (Hashtable)namePanels.get(enm.nextElement());

                                    JPanel smallNamePanel = (JPanel)lastPanelStore.get("smallNamePanel");
                                    smallNamePanel.remove((AJCheckBox)lastPanelStore.get("nameCb"));
                                    smallNamePanel.doLayout();
                                    smallNamePanel.revalidate();
                                }
                                // make sure that first panel has add_another_name_b
                                java.util.List nlist = Arrays.asList(namePanels.keySet().toArray());
                                Collections.sort(nlist, name_cmp);
                                Hashtable firstPanelStore = (Hashtable)namePanels.get(nlist.iterator().next());
                                JPanel smallNamePanel = (JPanel)firstPanelStore.get("smallNamePanel");
                                smallNamePanel.remove(add_another_name_b);
                                smallNamePanel.add(add_another_name_b);

                                // now remove name panel
                                Hashtable names = (Hashtable)newNode.get("names"); // it cannot be null here
                                names.remove(key);
                                current_panel.remove((JPanel)cb.get("titlePanel"));
                                current_panel.remove((JPanel)cb.get("smallNamePanel"));
                                current_panel.remove((JPanel)cb.get("titleAliasPanel"));
                                current_panel.remove((JPanel)cb.get("aliasesBigPanel"));
                                current_panel.remove((JPanel)cb.get("titleMXPanel"));
                                current_panel.remove((JPanel)cb.get("mxsBigPanel"));
                                current_panel.remove((JLabel)cb.get("space1"));
                                current_panel.remove((JLabel)cb.get("space2"));
                                current_panel.remove((JLabel)cb.get("space3"));
                                current_panel.remove((JLabel)cb.get("space4"));
                                current_panel.remove((JSeparator)cb.get("line"));
                                current_panel.doLayout();
                                current_panel.revalidate();
                            }
                        }
                    });
            }
            smallNamePanel.add(new JLabel(" "));
            if( first ) {
                first = false;
                add_another_name_b = new AJButton("Add another", null, AJButton.NOT_MOUSABLE);
                add_another_name_b.setMargin(new Insets(0, 0, 0, 0));
                add_another_name_b.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            int uid = 0;
                            synchronized( counter ) {
                                uid = Integer.parseInt(counter.substring(1));
                                counter.replace(1, counter.length(), ""+(uid+1));
                            }
                            Hashtable name = new Hashtable();
                            name.put("name", "");
                            name.put("domain", MainWindow.userDefDomain);
                            name.put("aliases", new Hashtable());
                            name.put("mxs", new Hashtable());
                            ((Hashtable)newNode.get("names")).put(" "+uid, name);

                            // set up msg and clean for rebuild
                            msg.setText("Adding node name...");
                            central_panel.removeAll();
                            central_panel.add(msgPanel, BorderLayout.CENTER);

                            reloadB.setEnabled(false);
                            modifyB.setEnabled(false);
                            deleteB.setEnabled(false);

                            mainPanel.removeAll();
                            central_panel.revalidate();
                            central_panel.repaint();
                            init = false; // rebuilding
                            (new Thread(THIS)).start();
                        }
                    });
                if( ((Hashtable)newNode.get("types")).get("Advanced") != null ) {
                    panelStore.put("add_another_name_b", add_another_name_b);
                    smallNamePanel.add(add_another_name_b);
                }
            }
            tr();

            // aliases
            Hashtable aliases = (Hashtable)nameH.get("aliases");
            if( aliases == null ) {
                aliases = new Hashtable();
                nameH.put("aliases", aliases);
            }

            int rows = aliases.size();
            if( rows == 0 )
                rows = 1;

            panelStore.put("space2", XLabel(" "));
            tr();

            // template name may not have aliases and/or mxs
            if( ((Hashtable)newNode.get("types")).get("Template") == null ) {

                JPanel aliasesBigPanel = new JPanel(new BorderLayout(0, 0));
                panelStore.put("aliasesBigPanel", aliasesBigPanel);
                aliasesBigPanel.setBackground(Color.white);
                GridLayout aliasesPanelLayout = new GridLayout(rows, 1, 0, 5);
                JPanel aliasesPanel = new JPanel(aliasesPanelLayout);
                panelStore.put("aliasesPanelLayout", aliasesPanelLayout);
                panelStore.put("aliasesPanel", aliasesPanel);
                aliasesPanel.setBackground(Color.white);
                aliasesBigPanel.add(aliasesPanel, BorderLayout.CENTER);

                addAliases(aliases, aliasesPanelLayout, aliasesPanel, key);

                AJButton add_another_alias_b = new AJButton("Add another", panelStore, AJButton.NOT_MOUSABLE);
                add_another_alias_b.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            AJButton bt = (AJButton)e.getSource();
                            JPanel aliasesPanel = (JPanel)bt.get("aliasesPanel");
                            int uid = 0;
                            synchronized( counter ) {
                                uid = Integer.parseInt(counter.substring(1));
                                counter.replace(1, counter.length(), ""+(uid+1));
                            }
                            Hashtable aliases = (Hashtable)((Hashtable)((Hashtable)newNode.get("names")).
                                                            get(bt.getStr("key"))).get("aliases");
                            if( aliases == null ) {
                                aliases = new Hashtable();
                                ((Hashtable)((Hashtable)newNode.get("names")).
                                 get(bt.getStr("key"))).put("aliases", aliases);
                            }
                            if( aliases.size() == 0 )
                                ((AJButton)bt.get("add_another_alias_b")).setText("Add another");
                            Hashtable alias = new Hashtable();
                            alias.put("name", "");
                            alias.put("domain", MainWindow.userDefDomain);
                            aliases.put(" "+uid, alias);
                            GridLayout aliasesPanelLayout = (GridLayout)bt.get("aliasesPanelLayout");
                            aliasesPanelLayout.setRows(aliases.size());

                            // collect data
                            Component[] a_components = aliasesPanel.getComponents();
                            String[] texts = new String[a_components.length];
                            for( int i = 0; i < a_components.length; i++ ) {
                                Component[] b_components = ((JPanel)a_components[i]).getComponents();
                                AJTextField tf = (AJTextField)b_components[1];
                                String text = tf.getText().trim();
                                Hashtable aName = (Hashtable)aliases.get(tf.key);
                                String nameStr = Utils.name(text);
                                String domainStr = Utils.domain(text, MainWindow.userDefDomain);
                                boolean sameName = ((String)aName.get("name")).toLowerCase().trim().equals(nameStr.toLowerCase().trim());
                                boolean sameDomain = ((String)aName.get("domain")).toLowerCase().trim().equals(domainStr.toLowerCase().trim());
                                aName.put("name", nameStr);
                                aName.put("domain", domainStr);
                                if( !sameName || !sameDomain )
                                    aName.put("dirty", "");
                            }

                            // rebuild aliases
                            aliasesPanel.removeAll();
                            addAliases(aliases, (GridLayout)bt.get("aliasesPanelLayout"),
                                       (JPanel)bt.get("aliasesPanel"),
                                       (String)bt.get("key"));

                            aliasesPanel.doLayout();
                            aliasesPanel.revalidate();
                            current_panel.doLayout();
                            current_panel.revalidate();
                        }
                    });
                panelStore.put("add_another_alias_b", add_another_alias_b);
                add_another_alias_b.setMargin(new Insets(0, 0, 0, 0));
                if( aliases.size() == 0 ) {
                    add_another_alias_b.setText("Add Alias");
                }
                JPanel rightPanel = new JPanel(new BorderLayout(5, 0));
                rightPanel.setBackground(Color.white);
                rightPanel.add(add_another_alias_b, BorderLayout.NORTH);
                aliasesBigPanel.add(rightPanel, BorderLayout.EAST);

                JPanel titleAliasPanel = HTMLPanel("Alias(es)");
                panelStore.put("titleAliasPanel", titleAliasPanel);
                td();
                add(aliasesBigPanel, GridBagConstraints.NONE, GridBagConstraints.WEST);
                tr();

                // MXs
                Hashtable mxs = (Hashtable)nameH.get("mxs");
                if( mxs == null ) {
                    mxs = new Hashtable();
                    nameH.put("mxs", mxs);
                }

                rows = mxs.size();
                if( rows == 0 )
                    rows = 1;

                panelStore.put("space3", XLabel(" "));
                tr();

                JPanel mxsBigPanel = new JPanel(new BorderLayout(0, 0));
                panelStore.put("mxsBigPanel", mxsBigPanel);
                mxsBigPanel.setBackground(Color.white);
                GridLayout mxsPanelLayout = new GridLayout(rows, 1, 0, 5);
                JPanel mxsPanel = new JPanel(mxsPanelLayout);
                panelStore.put("mxsPanelLayout", mxsPanelLayout);
                panelStore.put("mxsPanel", mxsPanel);
                mxsPanel.setBackground(Color.white);
                mxsBigPanel.add(mxsPanel, BorderLayout.CENTER);

                addMXs(mxs, mxsPanelLayout, mxsPanel, key);

                AJButton add_another_mx_b = new AJButton("Add another", panelStore, AJButton.NOT_MOUSABLE);
                add_another_mx_b.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            AJButton bt = (AJButton)e.getSource();
                            JPanel mxsPanel = (JPanel)bt.get("mxsPanel");
                            int uid = 0;
                            synchronized( counter ) {
                                uid = Integer.parseInt(counter.substring(1));
                                counter.replace(1, counter.length(), ""+(uid+1));
                            }
                            Hashtable mxs = (Hashtable)((Hashtable)((Hashtable)newNode.get("names")).
                                                        get(bt.getStr("key"))).get("mxs");
                            if( mxs == null ) {
                                mxs = new Hashtable();
                                ((Hashtable)((Hashtable)newNode.get("names")).
                                 get(bt.getStr("key"))).put("mxs", mxs);
                            }
                            if( mxs.size() == 0 )
                                ((AJButton)bt.get("add_another_mx_b")).setText("Add another");
                            Hashtable mx = new Hashtable();
                            mx.put("name", "");
                            mx.put("domain", MainWindow.userDefDomain);
                            mx.put("pref", "10");
                            mxs.put(" "+uid, mx);
                            GridLayout mxsPanelLayout = (GridLayout)bt.get("mxsPanelLayout");
                            mxsPanelLayout.setRows(mxs.size());

                            // collect data
                            Component[] m_components = mxsPanel.getComponents();
                            String[] texts = new String[m_components.length];
                            for( int i = 0; i < m_components.length; i++ ) {
                                Component[] b_components = ((JPanel)m_components[i]).getComponents();
                                AJTextField tf = (AJTextField)b_components[1];
                                String text = tf.getText().trim();
                                Hashtable mxName = (Hashtable)mxs.get(tf.key);
                                String nameStr = Utils.name(text);
                                String domainStr = Utils.domain(text, MainWindow.userDefDomain);
                                boolean sameName = ((String)mxName.get("name")).toLowerCase().trim().equals(nameStr.toLowerCase().trim());
                                boolean sameDomain = ((String)mxName.get("domain")).toLowerCase().trim().equals(domainStr.toLowerCase().trim());
                                mxName.put("name", nameStr);
                                mxName.put("domain", domainStr);
                                if( !sameName || !sameDomain )
                                    mxName.put("dirty", "");
                            }

                            // rebuild aliases
                            mxsPanel.removeAll();
                            addMXs(mxs, (GridLayout)bt.get("mxsPanelLayout"),
                                   (JPanel)bt.get("mxsPanel"), (String)bt.get("key"));

                            mxsPanel.doLayout();
                            mxsPanel.revalidate();
                            current_panel.doLayout();
                            current_panel.revalidate();
                        }
                    });
                panelStore.put("add_another_mx_b", add_another_mx_b);
                add_another_mx_b.setMargin(new Insets(0, 0, 0, 0));
                if( mxs.size() == 0 ) {
                    add_another_mx_b.setText("Add MX");
                }
                rightPanel = new JPanel(new BorderLayout(5, 0));
                rightPanel.setBackground(Color.white);
                rightPanel.add(add_another_mx_b, BorderLayout.NORTH);
                mxsBigPanel.add(rightPanel, BorderLayout.EAST);

                JPanel titleMXPanel = HTMLPanel("MX(es)");
                panelStore.put("titleMXPanel", titleMXPanel);
                td();
                add(mxsBigPanel, GridBagConstraints.NONE, GridBagConstraints.WEST);
                tr();

                panelStore.put("space4", XLabel(" "));
                tr();
            }

            panelStore.put("line", line());
        }

        // restore grid
        if( rebuild ) {
            current_gbc.gridx = save_gridx;
            current_gbc.gridy = save_gridy;
        }
    } // end of makeNamesPanels(...)

    public void addAliases(Hashtable aliases, GridLayout aliasesPanelLayout,
                           JPanel aliasesPanel, String key)
    {
        Hashtable panelStore = (Hashtable)namePanels.get(key);
        java.util.List alist = Arrays.asList(aliases.keySet().toArray());
        Collections.sort(alist, name_number_cmp);
        Iterator ait = alist.iterator();
        Hashtable aPanelStore = new Hashtable();
        panelStore.put("aPanelStore", aPanelStore);
        while( ait.hasNext() ) {
            String akey = (String)ait.next();
            JPanel apanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
            AJCheckBox cb = new AJCheckBox("", true, new Hashtable());
            cb.put("apanel", apanel);
            cb.put("aliasesPanelLayout", aliasesPanelLayout);
            cb.put("aliasesPanel", aliasesPanel);
            cb.put("main key", key);
            cb.put("key", akey);
            cb.put("aPanelStore", aPanelStore);
            cb.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                        if( !cb.isSelected() ) {
                            // remove associated data
                            Hashtable aliases =
                                ((Hashtable)((Hashtable)((Hashtable)newNode.get("names")).
                                             get(cb.getStr("main key"))).get("aliases"));
                            aliases.remove(cb.getStr("key"));
                            if( aliases.size() == 0 )
                                ((AJButton)((Hashtable)namePanels.get(cb.get("main key"))).
                                 get("add_another_alias_b")).
                                    setText("Add Alias");
                            GridLayout aliasesPanelLayout = (GridLayout)cb.get("aliasesPanelLayout");
                            aliasesPanelLayout.setRows(aliasesPanelLayout.getRows() - 1);
                            JPanel aliasesPanel = (JPanel)cb.get("aliasesPanel");
                            aliasesPanel.remove((JPanel)cb.get("apanel"));
                            ((Hashtable)cb.get("aPanelStore")).remove(cb.get("key"));

                            aliasesPanel.doLayout();
                            aliasesPanel.revalidate();
                            current_panel.doLayout();
                            current_panel.revalidate();
                        }
                    }
                });
            cb.setFont(label_font);
            cb.setBackground(Color.white);
            apanel.add(cb);
            Hashtable alias = (Hashtable)aliases.get(akey);
            AJTextField alias_tf = new AJTextField("", 24, null, akey);
            if( alias.get("name") != null && alias.get("name").toString().trim().length() > 0 ) {
                if( ((String)alias.get("domain")).trim().toLowerCase().equals(MainWindow.userDefDomain) )
                    alias_tf.setText((String)alias.get("name"));
                else
                    alias_tf.setText(alias.get("name")+"."+alias.get("domain"));
            }
            aPanelStore.put(akey, alias_tf);

            alias_tf.setFont(label_font);
            alias_tf.setBackground(Color.white);
            apanel.setBackground(Color.white);
            apanel.add(alias_tf);
            aliasesPanel.add(apanel);
        }
    } // end of addAliases(...)

    public void addMXs(Hashtable mxs, GridLayout mxsPanelLayout,
                       JPanel mxsPanel, String key)
    {
        java.util.List mlist = Arrays.asList(mxs.keySet().toArray());
        Collections.sort(mlist, name_number_cmp);
        Iterator mit = mlist.iterator();
        Hashtable panelStore = (Hashtable)namePanels.get(key);
        Hashtable mxsPanelStore = new Hashtable();
        panelStore.put("mxsPanelStore", mxsPanelStore);
        while( mit.hasNext() ) {
            String mkey = (String)mit.next();
            Hashtable mxStore = new Hashtable();
            mxsPanelStore.put(mkey, mxStore);

            JPanel mpanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
            mpanel.setBackground(Color.white);
            AJCheckBox cb = new AJCheckBox("", true, new Hashtable());
            cb.put("mpanel", mpanel);
            cb.put("mxsPanelLayout", mxsPanelLayout);
            cb.put("mxsPanel", mxsPanel);
            cb.put("main key", key);
            cb.put("key", mkey);
            cb.put("mxsPanelStore", mxsPanelStore);
            cb.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                        if( !cb.isSelected() ) {
                            // remove associated data
                            Hashtable mxs = ((Hashtable)((Hashtable)((Hashtable)newNode.get("names")).get(cb.getStr("main key"))).get("mxs"));
                            mxs.remove(cb.getStr("key"));
                            if( mxs.size() == 0 )
                                ((AJButton)((Hashtable)namePanels.get(cb.get("main key"))).get("add_another_mx_b")).setText("Add MX");
                            GridLayout mxsPanelLayout = (GridLayout)cb.get("mxsPanelLayout");
                            mxsPanelLayout.setRows(mxsPanelLayout.getRows() - 1);
                            JPanel mxsPanel = (JPanel)cb.get("mxsPanel");
                            mxsPanel.remove((JPanel)cb.get("mpanel"));
                            mxsPanel.doLayout();
                            mxsPanel.revalidate();
                            current_panel.doLayout();
                            current_panel.revalidate();
                            ((Hashtable)cb.get("mxsPanelStore")).remove(cb.get("key"));
                        }
                    }
                });

            cb.setFont(label_font);
            cb.setBackground(Color.white);
            mpanel.add(cb);
            Hashtable mx = (Hashtable)mxs.get(mkey);
            AJTextField mx_tf = new AJTextField("", 24, null, mkey);
            if( mx.get("name") != null && ((String)mx.get("name")).trim().length() > 0 ) {
                if( ((String)mx.get("domain")).trim().toLowerCase().equals(MainWindow.userDefDomain) )
                    mx_tf.setText((String)mx.get("name"));
                else
                    mx_tf.setText(mx.get("name")+"."+mx.get("domain"));
            }
            mx_tf.setFont(label_font);
            mx_tf.setBackground(Color.white);
            mpanel.add(mx_tf);
            mxStore.put("mx_tf", mx_tf);
            JTextField pref_tf = new JTextField((String)mx.get("pref"), 3);
            pref_tf.setFont(label_font);
            pref_tf.setBackground(Color.white);
            mpanel.add(pref_tf);
            mxStore.put("pref_tf", pref_tf);
            mxsPanel.add(mpanel);
        }
    } // end of addMXs(...)

    private void makeAdminsCBs(boolean rebuild) {
        Hashtable admins_h = (Hashtable)newNode.get("admins");
        Hashtable adminsH = new Hashtable();
        Hashtable hash = Utils.nHash(admins_h.get("ateams"));
        Enumeration enm = hash.keys();
        while( enm.hasMoreElements() ) {
            adminsH.put((String)enm.nextElement()+" (ateam)", "ateam");
        }
        hash = Utils.nHash(admins_h.get("admins"));
        enm = hash.keys();
        while( enm.hasMoreElements() ) {
            String key = (String)enm.nextElement();
            Hashtable person = (Hashtable)hash.get(key);
            String name = (String)person.get("name");
            if( name == null && person.get("NetID") != null )
                name = "("+person.get("NetID")+")";
            if( name == null && person.get("RegID") != null )
                name = "["+person.get("RegID")+"]";
            if( name == null )
                name = "";
            adminsH.put(name, "person="+key);
        }
        if( rebuild ) {
            admins_panel.removeAll();
            admins_panel_layout.setRows(adminsH.size());
        } else { // building for the first time
            admins_panel = new JPanel();
            admins_panel.setBackground(Color.white);
            admins_big_panel.add(admins_panel, BorderLayout.CENTER);
            admins_panel_layout = new GridLayout(adminsH.size(), 1, 0, 2);
            admins_panel.setLayout(admins_panel_layout);
        }

        java.util.List list = Arrays.asList(adminsH.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();
        while( it.hasNext() ) {
            String display_name = (String)it.next();
            Hashtable box = new Hashtable();
            if( ((String)adminsH.get(display_name)).equals("ateam") )
                box.put("ateam", "");
            else
                box.put("person", ((String)adminsH.get(display_name)).substring(7));
            AJCheckBox cb = new AJCheckBox(display_name, true, box);
            cb.setFont(label_font);
            cb.setBackground(Color.white);
            cb.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                        if( !cb.isSelected() ) {
                            // remove associated data
                            if( cb.box.get("ateam") != null )
                                ((Hashtable)((Hashtable)newNode.get("admins")).get("ateams")).remove(cb.getText().substring(0, cb.getText().length() - 8));
                            else
                                ((Hashtable)((Hashtable)newNode.get("admins")).get("admins")).remove((String)cb.box.get("person"));
                            admins_panel.remove(cb);
                            admins_panel_layout.setRows(admins_panel_layout.getRows() - 1);
                            admins_panel.doLayout();
                            current_panel.revalidate();
                        }
                    }
                });
            admins_panel.add(cb);
        }
        if( rebuild )
            current_panel.revalidate();
    } // end of makeAdminsCBs(...)

    private void makeAminsPanel() {
        current_gbc.anchor = GridBagConstraints.NORTHWEST;
        HTMLPanel("Administrators <font color=\"#ff0000\">*</font>");
        td();
        admins_big_panel = new JPanel(new BorderLayout());
        admins_big_panel.setBackground(Color.white);

        // make check boxes for a first time
        makeAdminsCBs(false);

        personsLookupPanel = new JPanel(new BorderLayout());
        personsTopLabel = new JLabel("", JLabel.CENTER);
        personsTopLabel.setFont(label_font);
        personsLookupPanel.add(personsTopLabel, BorderLayout.NORTH);

        personsLookupGridLayout = new GridLayout(7, 1, 0, 2);
        personsLookupCenterPanel = new JPanel(personsLookupGridLayout);
        personsLookupCenterPanel.setBackground(Color.white);
        // fillers
        personsLookupCenterPanel.add(new JLabel(" "));
        personsLookupCenterPanel.add(new JLabel(" "));
        personsLookupCenterPanel.add(new JLabel(" "));
        personsLookupCenterPanel.add(new JLabel(" "));
        personsLookupCenterPanel.add(new JLabel(" "));
        personsLookupCenterPanel.add(new JLabel(" "));
        personsLookupCenterPanel.add(new JLabel(" "));
        personsLookupPanel.add(new JScrollPane(personsLookupCenterPanel), BorderLayout.CENTER);

        JPanel bottomPanel = new JPanel(new BorderLayout());
        personsMiddleLabel = new JLabel("<html><body><center><hr style=\"width: 100%; height: 2px;\"><font size=\"-1\">"+
                           "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Name, SUNet ID, or Admin Team, each on its own line."+
                           "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<br>"+
                           "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Put a semicolon after Admin Teams, as in pine-team;"+
                           "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</font></center></body></html>",
                           JLabel.CENTER);
        personsMiddleLabel.setFont(label_font);
        personsMiddleLabel.setForeground(label_color);
        bottomPanel.add(personsMiddleLabel, BorderLayout.NORTH);

        lookup_ta = new JTextArea(3, 30);
        lookup_ta.setFont(label_font);
        lookup_ta.setEditable(true);
        JScrollPane sp = new JScrollPane(lookup_ta);
        bottomPanel.add(sp, BorderLayout.CENTER);
        personsLookupPanel.add(bottomPanel, BorderLayout.SOUTH);

        lookupButtons = new JButton[3];
        lookupButtons[0] = new BButton("Submit");
        lookupButtons[0].addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if( !old_lookup_text.equals(lookup_ta.getText()) ) {
                        lookupButtons[0].setEnabled(false);
                        lookupButtons[1].setEnabled(false);
                        Thread th;
                        if( lookupATeams )
                            th = new NodeThread(THIS, NodeThread.LOOKUP_ADMINS, looked_up_persons);
                        else
                            th = new NodeThread(THIS, NodeThread.LOOKUP_USERS, looked_up_persons);
                        th.start();
                        try {
                            th.join();
                        } catch(Exception ex) {
                            return;
                        }
                        old_lookup_text = lookup_ta.getText();
                        Enumeration enm = looked_up_persons.keys();
                        boolean multiple_hits = false;
                        while( enm.hasMoreElements() ) {
                            String key = (String)enm.nextElement();
                            if( key.endsWith("uncheck") )
                                multiple_hits = true;
                        }
                        if( multiple_hits ) {
                            JOptionPane.showMessageDialog(THIS.lookupPersonWin,
                                   "Multiple hits. Please select persons to commit.",
                                   "Multiple Hits", JOptionPane.INFORMATION_MESSAGE);
                            return;
                        }
                    }
                    synchronized( looked_up_persons ) {
                        THIS.lookupPersonWin.dispose();
                        lookup_ta.setText("");
                        if( lookupATeams ) {
                            Hashtable adminsH = (Hashtable)newNode.get("admins");
                            Hashtable ateams = (Hashtable)adminsH.get("ateams");
                            if( ateams == null ) {
                                ateams = new Hashtable();
                                adminsH.put("ateams", ateams);
                            }
                            Hashtable admins = (Hashtable)adminsH.get("admins");
                            if( admins == null ) {
                                admins = new Hashtable();
                                adminsH.put("admins", admins);
                            }
                            Enumeration enm = looked_up_persons.keys();
                            while( enm.hasMoreElements() ) {
                                String key = (String)enm.nextElement();
                                if( key.endsWith(" ateam") )
                                    ateams.put(((Hashtable)looked_up_persons.get(key)).get("name"), looked_up_persons.get(key));
                                else if( key.endsWith(" person") ) // person
                                    admins.put(key, looked_up_persons.get(key));
                            }
                        } else {
                            Hashtable users = (Hashtable)newNode.get("users");
                            if( users == null ) {
                                users = new Hashtable();
                                newNode.put("users", users);
                            }
                            Enumeration enm = looked_up_persons.keys();
                            while( enm.hasMoreElements() ) {
                                String key = (String)enm.nextElement();
                                if( key.endsWith(" person") )
                                    users.put(key, looked_up_persons.get(key));
                            }
                        }
                    }
                    looked_up_persons = new Hashtable();

                    // now rebuild admins panel
                    if( lookupATeams )
                        makeAdminsCBs(true);
                    else
                        makeUsersCBs(true);

                    // clean up
                    lookup_ta.setText("");
                    personsLookupCenterPanel.removeAll();
                    personsLookupGridLayout.setRows(7);

                    // fillers
                    personsLookupCenterPanel.add(new JLabel(" "));
                    personsLookupCenterPanel.add(new JLabel(" "));
                    personsLookupCenterPanel.add(new JLabel(" "));
                    personsLookupCenterPanel.add(new JLabel(" "));
                    personsLookupCenterPanel.add(new JLabel(" "));
                    personsLookupCenterPanel.add(new JLabel(" "));
                    personsLookupCenterPanel.add(new JLabel(" "));
                    //THIS.lookupPersonWin.dispose();
                }
            });
        lookupButtons[1] = new BButton("Lookup");
        lookupButtons[1].addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    lookupButtons[0].setEnabled(false);
                    lookupButtons[1].setEnabled(false);

                    if( lookupATeams )
                        (new NodeThread(THIS, NodeThread.LOOKUP_ADMINS, looked_up_persons)).start();
                    else
                        (new NodeThread(THIS, NodeThread.LOOKUP_USERS, looked_up_persons)).start();
                    old_lookup_text = lookup_ta.getText();
                }
            });

        lookupButtons[2] = new BButton("Cancel");
        lookupButtons[2].addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    // reset hash
                    looked_up_persons = new Hashtable();

                    // clean up
                    lookup_ta.setText("");
                    personsLookupCenterPanel.removeAll();
                    personsLookupGridLayout.setRows(7);

                    // fillers
                    personsLookupCenterPanel.add(new JLabel(" "));
                    personsLookupCenterPanel.add(new JLabel(" "));
                    personsLookupCenterPanel.add(new JLabel(" "));
                    personsLookupCenterPanel.add(new JLabel(" "));
                    personsLookupCenterPanel.add(new JLabel(" "));
                    personsLookupCenterPanel.add(new JLabel(" "));
                    personsLookupCenterPanel.add(new JLabel(" "));
                    THIS.lookupPersonWin.dispose();
                    lookupButtons[0].setEnabled(true);
                    lookupButtons[1].setEnabled(true);
                }
            });
        personsLookupDialog =
            new JOptionPane(personsLookupPanel, JOptionPane.PLAIN_MESSAGE,
                            JOptionPane.YES_NO_CANCEL_OPTION, null, lookupButtons);

        add_another_admin = new BButton("Add another");
        add_another_admin.setMargin(new Insets(0, 0, 0, 0));
        add_another_admin.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        lookupATeams = true;
                        personsTopLabel.setText("<html><body><center><b>Lookup administrators</b><br><hr style=\"width: 100%; height: 2px;\">"+
                                                "<font size=\"-1\">Un-check to remove</font></center></body></html>");
                        personsMiddleLabel.setText("<html><body><center><hr style=\"width: 100%; height: 2px;\"><font size=\"-1\">"+
                                                   "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Name, SUNet ID, or Admin Team, each on its own line."+
                                                   "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<br>"+
                                                   "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Put a semicolon after Admin Teams, as in pine-team;"+
                                                   "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</font></center></body></html>");
                        looked_up_persons = new Hashtable();
                        old_lookup_text = lookup_ta.getText();
                        lookupPersonWin = personsLookupDialog.createDialog(MainWindow.win, "Lookup administrators");
                        lookupPersonWin.show();
                    }
            });

        right_admins_panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        right_admins_panel.setBackground(Color.white);
        right_admins_panel.add(add_another_admin);
        admins_big_panel.add(right_admins_panel, BorderLayout.EAST);

        add(admins_big_panel, GridBagConstraints.NONE, GridBagConstraints.WEST);
        tr();
    } // end of makeAminsPanel()

    private void makeUsersCBs(boolean rebuild) {
        Hashtable users_h = Utils.nHash(newNode.get("users"));

        Hashtable usersH = new Hashtable();
        Enumeration enm = users_h.keys();
        while( enm.hasMoreElements() ) {
            String key = (String)enm.nextElement();
            Hashtable person = (Hashtable)users_h.get(key);
            String name = (String)person.get("name");
            if( name == null && person.get("NetID") != null )
                name = "("+person.get("NetID")+")";
            if( name == null && person.get("RegID") != null )
                name = "["+person.get("RegID")+"]";
            if( name == null )
                name = "";
            usersH.put(name, key);
        }
        if( rebuild ) {
            users_panel.removeAll();
            users_panel_layout.setRows(usersH.size());
        } else { // building for the first time
            users_panel = new JPanel();
            users_panel.setBackground(Color.white);
            users_big_panel.add(users_panel, BorderLayout.CENTER);
            users_panel_layout = new GridLayout(usersH.size(), 1, 0, 2);
            users_panel.setLayout(users_panel_layout);
        }

        java.util.List list = Arrays.asList(usersH.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();
        while( it.hasNext() ) {
            String display_name = (String)it.next();
            Hashtable box = new Hashtable();
            box.put("person", usersH.get(display_name));
            AJCheckBox cb = new AJCheckBox(display_name, true, box);
            cb.setFont(label_font);
            cb.setBackground(Color.white);
            cb.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                        if( !cb.isSelected() ) {
                            // remove associated data
                            ((Hashtable)newNode.get("users")).remove((String)cb.box.get("person"));
                            users_panel.remove(cb);
                            users_panel_layout.setRows(users_panel_layout.getRows() - 1);
                            users_panel.doLayout();
                            current_panel.revalidate();
                        }
                    }
                });
            users_panel.add(cb);
        }
        if( rebuild )
            current_panel.revalidate();
    } // end of makeUsersCBs(...)

    private void makeUsersPanel() {
        current_gbc.anchor = GridBagConstraints.NORTHWEST;
        HTMLPanel("Users");
        td();
        users_big_panel = new JPanel(new BorderLayout());
        users_big_panel.setBackground(Color.white);

        // make check boxes for a first time
        makeUsersCBs(false);

        add_another_user = new BButton("Add another");
        add_another_user.setMargin(new Insets(0, 0, 0, 0));
        add_another_user.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        personsTopLabel.setText("<html><body><center><b>Lookup users</b><br><hr style=\"width: 100%; height: 2px;\">"+
                                                "<font size=\"-1\">Un-check to remove</font></center></body></html>");
                        personsMiddleLabel.setText("<html><body><center><hr style=\"width: 100%; height: 2px;\"><font size=\"-1\">"+
                                                   "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"+
                                                   "Name, or SUNet ID each on its own line."+
                                                   "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"+
                                                   "</font></center></body></html>");
                        lookupATeams = false;
                        looked_up_persons = new Hashtable();
                        old_lookup_text = lookup_ta.getText();
                        lookupPersonWin = personsLookupDialog.createDialog(MainWindow.win, "Lookup users");
                        lookupPersonWin.show();
                    }
            });

        right_users_panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        right_users_panel.setBackground(Color.white);
        right_users_panel.add(add_another_user);
        users_big_panel.add(right_users_panel, BorderLayout.EAST);

        add(users_big_panel, GridBagConstraints.NONE, GridBagConstraints.WEST);
        tr();
    }

    private void makeGroupsPanel() {
        current_gbc.anchor = GridBagConstraints.NORTHWEST;
        HTMLPanel("Groups <font color=\"#ff0000\">*</font>");
        td();
        groups_big_panel = new JPanel(new BorderLayout());
        groups_big_panel.setBackground(Color.white);

        Hashtable nodeGroupsH = (Hashtable)newNode.get("groups");
        groups_panel = new JPanel();
        groups_panel.setBackground(Color.white);
        groups_big_panel.add(groups_panel, BorderLayout.CENTER);
        groups_panel_layout = new GridLayout(nodeGroupsH.size(), 1, 0, 2);
        groups_panel.setLayout(groups_panel_layout);

        add_another_group = new BButton("Add another");
        add_another_group.setMargin(new Insets(0, 0, 0, 0));

        bottom_groups_panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        bottom_groups_panel.setBackground(Color.white);
        groups_big_panel.add(bottom_groups_panel, BorderLayout.SOUTH);

        right_groups_panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        right_groups_panel.setBackground(Color.white);
        right_groups_panel.add(add_another_group);
        groups_big_panel.add(right_groups_panel, BorderLayout.EAST);

        java.util.List list = Arrays.asList(nodeGroupsH.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();
        while( it.hasNext() ) {
            String group = (String)it.next();
            AJCheckBox cb = new AJCheckBox(group, true);
            cb.setFont(label_font);
            cb.setBackground(Color.white);
            cb.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                        if( !cb.isSelected() ) {
                            // remove associated data
                            ((Hashtable)newNode.get("groups")).remove(cb.getText());
                            groups_panel.remove(cb);
                            groups_panel_layout.setRows(groups_panel_layout.getRows() - 1);
                            groups_panel.doLayout();
                            current_panel.revalidate();
                            if( ((Hashtable)newNode.get("groups")).size() == 0 )
                                add_another_group.doClick();
                        }
                    }
                });
            groups_panel.add(cb);
        }

        add_another_group.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        right_groups_panel.removeAll();
                        Hashtable groups_in_the_list = Utils.copyHash(allGroupsH);

                        // remove groups which already selected
                        Hashtable nodeGroupsH = (Hashtable)newNode.get("groups");
                        Enumeration enm = nodeGroupsH.keys();
                        while( enm.hasMoreElements() ) {
                            String selectedGroup = (String)enm.nextElement();
                            groups_in_the_list.remove(selectedGroup);
                        }

                        // sort them
                        String[] groups_array = new String[groups_in_the_list.size()];
                        java.util.List list = Arrays.asList(groups_in_the_list.keySet().toArray());
                        Collections.sort(list, name_cmp);
                        Iterator it = list.iterator();
                        int counter = 0;
                        while( it.hasNext() )
                            groups_array[counter++] = (String)it.next();

                        list_of_groups.setListData(groups_array);
                        if( groups_sp == null )
                            groups_sp = new JScrollPane(list_of_groups);
                        bottom_groups_panel.add(groups_sp);

                        groups_big_panel.doLayout();
                        current_panel.revalidate();
                        list_of_groups.requestFocus();
                        bottom_groups_panel.revalidate();

                    } catch (Exception ex) {
                        Utils.warn(ex.getMessage());
                    }
                }
            });

        // remove it if lost focus
        list_of_groups.addFocusListener(new FocusAdapter() {
                public void focusLost(FocusEvent e) {
                    bottom_groups_panel.remove(groups_sp);
                    right_groups_panel.add(add_another_group);
                    groups_big_panel.doLayout();
                    current_panel.revalidate();
                }
            });

        // double click should work the same as pressing Enter
        list_of_groups.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    if( e.getClickCount() > 1 ) {
                        JList list = (JList)e.getComponent();
                        list.dispatchEvent(new KeyEvent(list,
                                                        KeyEvent.KEY_PRESSED,
                                                        Calendar.getInstance().getTimeInMillis(),
                                                        0, KeyEvent.VK_ENTER,
                                                        KeyEvent.CHAR_UNDEFINED));
                    }
                }
            });

        list_of_groups.addKeyListener(new KeyAdapter() {
                public void keyPressed(KeyEvent e) {
                    int key = e.getKeyCode();
                    if( key == KeyEvent.VK_ESCAPE || key == KeyEvent.VK_ENTER ) {
                        bottom_groups_panel.remove(groups_sp);
                        right_groups_panel.add(add_another_group);
                    }

                    if( key == KeyEvent.VK_ENTER || key == KeyEvent.VK_SPACE ) {
                        Object[] added_groups = list_of_groups.getSelectedValues();
                        Hashtable nodeGroupsH = (Hashtable)newNode.get("groups");
                        for( int i = 0; i < added_groups.length; i++ ) {
                            nodeGroupsH.put((String)added_groups[i], "");
                        }
                        if( added_groups.length > 0 && key == KeyEvent.VK_SPACE ) {
                            try {
                                int sel = list_of_groups.getSelectedIndex();
                                Hashtable groups_in_the_list = Utils.copyHash(allGroupsH);

                                // remove groups which already selected
                                Enumeration enm = nodeGroupsH.keys();
                                while( enm.hasMoreElements() ) {
                                    String selectedGroup = (String)enm.nextElement();
                                    groups_in_the_list.remove(selectedGroup);
                                }

                                String[] groups_array = new String[groups_in_the_list.size()];
                                java.util.List list = Arrays.asList(groups_in_the_list.keySet().toArray());
                                Collections.sort(list, name_cmp);
                                Iterator it = list.iterator();
                                int counter = 0;
                                while( it.hasNext() )
                                    groups_array[counter++] = (String)it.next();

                                list_of_groups.setListData(groups_array);
                                list_of_groups.setSelectedIndex(sel);
                            } catch (Exception ex) {
                                Utils.error(ex.getMessage());
                            }
                        }

                        groups_panel.removeAll();

                        groups_panel_layout.setRows(nodeGroupsH.size());
                        java.util.List list = Arrays.asList(nodeGroupsH.keySet().toArray());
                        Collections.sort(list, name_cmp);
                        Iterator it = list.iterator();
                        while( it.hasNext() ) {
                            String group = (String)it.next();
                            AJCheckBox cb = new AJCheckBox(group, true);
                            cb.setFont(label_font);
                            cb.setBackground(Color.white);
                            cb.addActionListener(new ActionListener() {
                                    public void actionPerformed(ActionEvent e) {
                                        AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                                        if( !cb.isSelected() ) {
                                            // remove associated data
                                            ((Hashtable)newNode.get("groups")).remove(cb.getText());
                                            groups_panel.remove(cb);
                                            groups_panel_layout.setRows(groups_panel_layout.getRows() - 1);
                                            groups_panel.doLayout();
                                            current_panel.revalidate();
                                            if( ((Hashtable)newNode.get("groups")).size() == 0 )
                                                add_another_group.doClick();
                                        }
                                    }
                                });
                            groups_panel.add(cb);
                        }
                    }
                    if( key == KeyEvent.VK_ESCAPE ||
                        key == KeyEvent.VK_ENTER ||
                        key == KeyEvent.VK_SPACE )
                        {
                            groups_big_panel.doLayout();
                            current_panel.revalidate();
                        }
                }
            });

        add(groups_big_panel, GridBagConstraints.NONE, GridBagConstraints.WEST);
        tr();
    } // end of makeGroupsPanel

    private void makeOSesPanel() {
        current_gbc.anchor = GridBagConstraints.NORTHWEST;
        HTMLPanel("Operating systems <font color=\"#ff0000\">*</font>");
        td();
        oses_big_panel = new JPanel(new BorderLayout());
        oses_big_panel.setBackground(Color.white);

        Hashtable nodeOSesH = (Hashtable)newNode.get("oses");
        oses_panel = new JPanel();
        oses_panel.setBackground(Color.white);
        oses_big_panel.add(oses_panel, BorderLayout.CENTER);
        oses_panel_layout = new GridLayout(nodeOSesH.size(), 1, 0, 2);
        oses_panel.setLayout(oses_panel_layout);

        add_another_os = new BButton("Add another");
        add_another_os.setMargin(new Insets(0, 0, 0, 0));

        create_another_os = new BButton("New");
        create_another_os.setMargin(new Insets(0, 0, 0, 0));

        create_another_os.addFocusListener(new FocusAdapter() {
                public void focusLost(FocusEvent e) {
                    if( e.getOppositeComponent().equals(list_of_oses) )
                        return;
                    bottom_oses_panel.remove(oses_sp);
                    right_oses_panel.removeAll();
                    right_oses_panel.add(add_another_os);
                    oses_big_panel.doLayout();
                    current_panel.revalidate();
                }
            });

        create_another_os.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        JPanel panel = new JPanel(new GridLayout(4, 1));
                        panel.add(new JLabel("OS Name:"));
                        panel.add(osNameF = new JTextField("", 40));
                        osNameF.addKeyListener(new KeyAdapter() {
                                public void keyPressed(KeyEvent e) {
                                    if( e.getKeyCode() == KeyEvent.VK_ENTER )
                                        ((JTextField)e.getSource()).transferFocus();
                                }
                            });
                        panel.add(new JLabel(" "));

                        THIS.buttons = new JButton[2];
                        THIS.buttons[0] = new BButton("Save");
                        ((JButton)(THIS.buttons[0])).addActionListener(new ActionListener() {
                                public void actionPerformed(ActionEvent ev) {
                                    String osName = osNameF.getText().trim();
                                    if( osName.length() == 0 )
                                        return;
                                    THIS.addOSDialogWin.dispose();
                                    Hashtable newOS = new Hashtable();
                                    newOS.put("object", "os");
                                    newOS.put("name", osName);
                                    newOS.put("comment", "");

                                    msg.setText("Adding OS...");
                                    central_panel.removeAll();
                                    central_panel.add(msgPanel, BorderLayout.CENTER);
                                    central_panel.revalidate();
                                    central_panel.repaint();

                                    (new NodeThread(THIS, NodeThread.CREATE_OS, newOS)).start();
                                }
                            });

                        THIS.buttons[1] = new BButton("Cancel");
                        ((JButton)(THIS.buttons[1])).addActionListener(new ActionListener() {
                                public void actionPerformed(ActionEvent ev) {
                                    THIS.addOSDialogWin.dispose();
                                }
                            });

                        JOptionPane modDialog =
                            new JOptionPane(panel, JOptionPane.PLAIN_MESSAGE, JOptionPane.OK_CANCEL_OPTION, null, THIS.buttons);
                        addOSDialogWin = modDialog.createDialog(THIS, "Add OS");
                        addOSDialogWin.show();

                    } catch(Exception exx) {
                        JOptionPane.showMessageDialog(THIS,
                                exx.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
                        return;
                    }
                }
            });

        bottom_oses_panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        bottom_oses_panel.setBackground(Color.white);
        oses_big_panel.add(bottom_oses_panel, BorderLayout.SOUTH);

        right_oses_panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        right_oses_panel.setBackground(Color.white);
        right_oses_panel.add(add_another_os);
        oses_big_panel.add(right_oses_panel, BorderLayout.EAST);

        java.util.List list = Arrays.asList(nodeOSesH.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();
        while( it.hasNext() ) {
            String os = (String)it.next();
            AJCheckBox cb = new AJCheckBox(os, true);
            cb.setFont(label_font);
            cb.setBackground(Color.white);
            cb.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                        if( !cb.isSelected() ) {
                            // remove associated data
                            ((Hashtable)newNode.get("oses")).remove(cb.getText());
                            oses_panel.remove(cb);
                            oses_panel_layout.setRows(oses_panel_layout.getRows() - 1);
                            oses_panel.doLayout();
                            current_panel.revalidate();
                            if( ((Hashtable)newNode.get("oses")).size() == 0 )
                                add_another_os.doClick();
                        }
                    }
                });
            oses_panel.add(cb);
        }

        add_another_os.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        right_oses_panel.removeAll();
                        right_oses_panel.add(create_another_os);
                        Hashtable oses_in_the_list = Utils.copyHash(osesH);

                        // remove oses which already selected
                        Hashtable nodeOSesH = (Hashtable)newNode.get("oses");
                        Enumeration enm = nodeOSesH.keys();
                        while( enm.hasMoreElements() ) {
                            String selectedOS = (String)enm.nextElement();
                            oses_in_the_list.remove(selectedOS);
                        }

                        // sort them
                        String[] oses_array = new String[oses_in_the_list.size()];
                        java.util.List list = Arrays.asList(oses_in_the_list.keySet().toArray());
                        Collections.sort(list, name_cmp);
                        Iterator it = list.iterator();
                        int counter = 0;
                        while( it.hasNext() )
                            oses_array[counter++] = (String)it.next();

                        list_of_oses.setListData(oses_array);
                        if( oses_sp == null )
                            oses_sp = new JScrollPane(list_of_oses);
                        bottom_oses_panel.add(oses_sp);

                        oses_big_panel.doLayout();
                        current_panel.revalidate();
                        list_of_oses.requestFocus();
                        bottom_oses_panel.revalidate();

                    } catch (Exception ex) {
                        Utils.warn(ex.getMessage());
                    }
                }
            });

        // remove it if lost focus
        list_of_oses.addFocusListener(new FocusAdapter() {
                public void focusLost(FocusEvent e) {
                    if( create_another_os != null &&
                        e.getOppositeComponent() != null &&
                        e.getOppositeComponent().equals(create_another_os) )
                        return;
                    bottom_oses_panel.remove(oses_sp);
                    right_oses_panel.removeAll();
                    right_oses_panel.add(add_another_os);
                    oses_big_panel.doLayout();
                    current_panel.revalidate();
                }
            });

        // double click should work the same as pressing Enter
        list_of_oses.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    if( e.getClickCount() > 1 ) {
                        JList list = (JList)e.getComponent();
                        list.dispatchEvent(new KeyEvent(list,
                                                        KeyEvent.KEY_PRESSED,
                                                        Calendar.getInstance().getTimeInMillis(),
                                                        0, KeyEvent.VK_ENTER,
                                                        KeyEvent.CHAR_UNDEFINED));
                    }
                }
            });

        list_of_oses.addKeyListener(new KeyAdapter() {
                public void keyPressed(KeyEvent e) {
                    int key = e.getKeyCode();
                    if( key == KeyEvent.VK_ESCAPE || key == KeyEvent.VK_ENTER ) {
                        bottom_oses_panel.remove(oses_sp);
                        right_oses_panel.add(add_another_os);
                    }
                    if( key == KeyEvent.VK_ENTER ) {
                        Object[] added_oses = list_of_oses.getSelectedValues();
                        Hashtable nodeOSesH = (Hashtable)newNode.get("oses");
                        for( int i = 0; i < added_oses.length; i++ )
                            nodeOSesH.put((String)added_oses[i], "");

                        oses_panel.removeAll();

                        oses_panel_layout.setRows(nodeOSesH.size());
                        java.util.List list = Arrays.asList(nodeOSesH.keySet().toArray());
                        Collections.sort(list, name_cmp);
                        Iterator it = list.iterator();
                        while( it.hasNext() ) {
                            String os = (String)it.next();
                            AJCheckBox cb = new AJCheckBox(os, true);
                            cb.setFont(label_font);
                            cb.setBackground(Color.white);
                            cb.addActionListener(new ActionListener() {
                                    public void actionPerformed(ActionEvent e) {
                                        AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                                        if( !cb.isSelected() ) {
                                            // remove associated data
                                            ((Hashtable)newNode.get("oses")).remove(cb.getText());
                                            oses_panel.remove(cb);
                                            oses_panel_layout.setRows(oses_panel_layout.getRows() - 1);
                                            oses_panel.doLayout();
                                            current_panel.revalidate();
                                            if( ((Hashtable)newNode.get("oses")).size() == 0 )
                                                add_another_os.doClick();
                                        }
                                    }
                                });
                            oses_panel.add(cb);
                        }
                    }
                    if( key == KeyEvent.VK_ESCAPE || key == KeyEvent.VK_ENTER ) {
                        oses_big_panel.doLayout();
                        current_panel.revalidate();
                    }
                }
            });

        add(oses_big_panel, GridBagConstraints.NONE, GridBagConstraints.WEST);
        tr();
    } // end of makeOSesPanel()

    private JPanel MakeInterfacePanel(int index, String id) {
        Utils.debug("making intfc. panel id = "+id);
        Hashtable intfcStore = new Hashtable();
        interfacesStore.put(id, intfcStore);
        Hashtable intfc = (Hashtable)((Hashtable)newNode.get("interfaces")).get(id);

        JPanel big_panel = new JPanel(new BorderLayout());
        //intfcStore.put("big_panel", big_panel);
        intfcStore.put("id", id);
        JLabel label = makeLabel("<html><body>&nbsp;&nbsp;Interface "+index+"&nbsp;&nbsp;</body></html>");
        JPanel north_panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 3));
        north_panel.add(label);
        AJCheckBox cb = new AJCheckBox("<html><body><i>(Uncheck to remove)</i></body></html>", true, intfcStore);
        cb.addActionListener( new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                    if( cb.isSelected() )
                        return;
                    interfacesStore.remove(cb.get("id"));
                    Utils.nHash(newNode.get("interfaces")).remove(cb.get("id"));

                    // set up msg and clean for rebuilding
                    msg.setText("Removing interface...");
                    central_panel.removeAll();
                    central_panel.add(msgPanel, BorderLayout.CENTER);

                    reloadB.setEnabled(false);
                    modifyB.setEnabled(false);
                    deleteB.setEnabled(false);

                    mainPanel.removeAll();
                    central_panel.revalidate();
                    central_panel.repaint();
                    init = false; // rebuilding
                    (new Thread(THIS)).start();
                }
            });
        ((FlowLayout)cb.getLayout()).setVgap(0);
        cb.setMargin(new Insets(0, 0, 0, 0));
        cb.setBackground(big_panel.getBackground());
        cb.setFont(label_font);
        north_panel.add(cb);
        big_panel.add(north_panel, BorderLayout.NORTH);

        GridBagLayout layout = new GridBagLayout();
        intfcStore.put("layout", layout);
        GridBagConstraints gbc = new GridBagConstraints();
        intfcStore.put("gbc", gbc);
        gbc.gridwidth = 1;
        JPanel panel = new JPanel(layout);
        intfcStore.put("panel", panel);
        panel.setBackground(Color.white);
        big_panel.add(panel, BorderLayout.CENTER);

        // first row
        gbc.gridx = 0;
        gbc.fill = GridBagConstraints.NONE;
        gbc.anchor = GridBagConstraints.WEST;

        label = makeLabel("Hardware Address  ");
        layout.setConstraints(label, gbc);
        panel.add(label);
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.gridx++;

        JPanel right_hw_panel = new JPanel();
        right_hw_panel.setBackground(Color.white);
        String hw = (String)intfc.get("hw");
        if( hw == null )
            hw = "";
        hw = hw.trim().toUpperCase();
        String hwStr = "";
        if( hw.length() > 0 )
            hwStr = hw.substring(0, 4)+"."+hw.substring(4, 8)+"."+hw.substring(8);
        hwStr = hwStr.toLowerCase();
        JTextField hw_tf = new JTextField(hwStr, 15);
        intfcStore.put("hw_tf", hw_tf);
        hw_tf.setFont(label_font);
        right_hw_panel.add(hw_tf);

        label = makeLabel(" Name");
        intfcStore.put("hw_name_label", label);
        right_hw_panel.add(label);

        JTextField hw_name_tf = new JTextField("", 14);
        intfcStore.put("hw_name_tf", hw_name_tf);
        hw_name_tf.setFont(label_font);
        right_hw_panel.add(hw_name_tf);
        Hashtable names = Utils.nHash(intfc.get("names"));
        if( names.size() > 0 ) {
            // pick up first name
            Hashtable nameH = (Hashtable)names.get(names.keys().nextElement());
            if( ((String)nameH.get("domain")).trim().toLowerCase().equals(MainWindow.userDefDomain) )
                hw_name_tf.setText((String)nameH.get("name"));
            else
                hw_name_tf.setText(nameH.get("name")+"."+nameH.get("domain"));
        }

        if( Utils.nHash(newNode.get("types")).get("Advanced") == null ) {
            label.setVisible(false);
            hw_name_tf.setVisible(false);
        }

        Boolean use_dhcp = (Boolean)intfc.get("use-dhcp?");
        if( use_dhcp == null )
            use_dhcp = new Boolean(false);
        cb = new AJCheckBox("DHCP/BootP", use_dhcp.booleanValue(), intfcStore);
        cb.setFont(label_font);
        ((FlowLayout)cb.getLayout()).setVgap(0);
        cb.setMargin(new Insets(0, 0, 0, 0));
        right_hw_panel.add(cb);
        cb.addActionListener( new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                    ((AJCheckBox)cb.get("roaming_cb")).setEnabled(cb.isSelected());
                    if( !cb.isSelected() )
                        ((AJCheckBox)cb.get("roaming_cb")).setSelected(false);
                    ((Hashtable)Utils.nHash(newNode.get("interfaces")).get(cb.get("id"))).
                        put("use-dhcp?", new Boolean(cb.isSelected()));
                }
            });

        Boolean roaming = (Boolean)intfc.get("roaming?");
        if( roaming == null )
            roaming = new Boolean(false);
        cb = new AJCheckBox("Roaming", roaming.booleanValue(), intfcStore);
        intfcStore.put("roaming_cb", cb);
        if( !use_dhcp.booleanValue() )
            cb.setEnabled(false);
        cb.setFont(label_font);
        ((FlowLayout)cb.getLayout()).setVgap(0);
        cb.setMargin(new Insets(0, 0, 0, 0));
        right_hw_panel.add(cb);
        cb.addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                    ((Hashtable)Utils.nHash(newNode.get("interfaces")).get(cb.get("id"))).
                        put("roaming?", new Boolean(cb.isSelected()));
                }
            });

        layout.setConstraints(right_hw_panel, gbc);
        panel.add(right_hw_panel);

        gbc.gridx = 0;
        gbc.gridwidth = 1;

        // second row
        gbc.anchor = GridBagConstraints.NORTHWEST;
        label = makeLabel("IP Addresses");
        layout.setConstraints(label, gbc);
        panel.add(label);
        gbc.anchor = GridBagConstraints.WEST;
        gbc.gridx++;
        gbc.gridwidth = GridBagConstraints.REMAINDER;

        Hashtable ipsStore = new Hashtable();
        intfcStore.put("ipsStore", ipsStore);
        Hashtable ips = Utils.nHash(intfc.get("ips"));
        GridLayout ip_layout = new GridLayout(ips.size() + 1, 1, 0, 2);
        intfcStore.put("ip_layout", ip_layout);
        JPanel ips_panel = new JPanel(ip_layout);
        intfcStore.put("ips_panel", ips_panel);
        ips_panel.setBackground(Color.white);

        makeIPs(ips, intfcStore, false);

        JPanel ip_panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 5));
        intfcStore.put("another_ip_b_panel", ip_panel);
        ip_panel.setBackground(Color.white);
        ips_panel.add(ip_panel);
        AJButton another_ip_b = new AJButton("Add another", intfcStore, AJButton.NOT_MOUSABLE);
        if( open_as_pickup )
            open_as_pickup_b = another_ip_b;

        another_ip_b.setMargin(new Insets(0, 0, 0, 0));
        ip_panel.add(another_ip_b);
        //ips_panel.add(ip_panel);
        another_ip_b.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent e) {
                    reserving_ipc = false; // reserving ip
                    ipAddrCountLabel.setVisible(false);
                    ipAddrCountTf.setVisible(false);
                    if( Utils.nHash(newNode.get("types")).get("Advanced") == null ) {
                        ipAddrReserveNameLabel.setVisible(false);
                        ipAddrNameReserveTf.setVisible(false);
                    } else {
                        ipAddrReserveNameLabel.setVisible(true);
                        ipAddrNameReserveTf.setVisible(true);
                    }
                    AJButton another_ip_b = (AJButton)e.getSource();
                    currentIntfcStore = another_ip_b.box;
                    ipAddrReserveWin = ipAddrReserveDialog.createDialog(THIS, "Reserve IP Address");
                    ipAddrReserveWin.show();
                    addrSpacesJList.requestFocus();
                }
            });

        layout.setConstraints(ips_panel, gbc);
        panel.add(ips_panel);

        gbc.gridx = 0;
        gbc.gridwidth = 1;

        gbc.anchor = GridBagConstraints.NORTHWEST;
        label = makeLabel("DHCP Options");
        layout.setConstraints(label, gbc);
        panel.add(label);
        gbc.anchor = GridBagConstraints.WEST;
        gbc.gridx++;

        Hashtable dhcp_options_h = Utils.nHash(intfc.get("dhcp-options"));
        JPanel bottom_panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
        intfcStore.put("bottom_panel", bottom_panel);
        JTextArea dhcp_options_ta = new JTextArea(3, 25);
        intfcStore.put("dhcp_options_ta", dhcp_options_ta);

        String options_text = "";
        java.util.List list = Arrays.asList(dhcp_options_h.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();
        boolean first = true;
        while( it.hasNext() ) {
            String key = ((String)it.next()).trim();
            options_text += (!first?"\n":"")+key.trim();
            first = false;
        }

        dhcp_options_ta.setText(options_text);
        dhcp_options_ta.setFont(label_font);
        dhcp_options_ta.setEditable(true);
        JScrollPane dhcp_sp = new JScrollPane(dhcp_options_ta);

        intfcStore.put("dhcp_sp", dhcp_sp);
        bottom_panel.add(dhcp_sp);

        AJButton dhcpB = new AJButton("Display", intfcStore, AJButton.NOT_MOUSABLE);
        dhcpB.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    AJButton dhcpB = (AJButton)e.getSource();
                    dhcpB.setVisible(false);
                    ((JScrollPane)dhcpB.get("dhcp_sp")).setVisible(true);
                    ((JTextArea)dhcpB.get("dhcp_options_ta")).requestFocus();
                }
            });

        intfcStore.put("dhcpB", dhcpB);
        dhcpB.setMargin(new Insets(0, 0, 0, 0));
        bottom_panel.add(dhcpB);

        if( dhcp_options_h.size() > 0 )
            dhcpB.setVisible(false);
        else
            dhcp_sp.setVisible(false);

        gbc.gridwidth = GridBagConstraints.REMAINDER;
        layout.setConstraints(bottom_panel, gbc);
        panel.add(bottom_panel);

        gbc.gridx = 0;
        gbc.gridwidth = 1;

        return big_panel;
    } // end of MakeInterfacePanel(...)

    public void makeIPs(Hashtable ips, Hashtable intfcStore, boolean rebuilding) {
        //Utils.printHash(ips);
        GridLayout ip_layout = (GridLayout)intfcStore.get("ip_layout");
        JPanel ips_panel = (JPanel)intfcStore.get("ips_panel");

        if( rebuilding )
            ips_panel.removeAll();

        ip_layout.setRows(ips.size() + 1);
        Hashtable ipsStore = (Hashtable)intfcStore.get("ipsStore");
        java.util.List list = Arrays.asList(ips.keySet().toArray());
        Collections.sort(list, ip_cmp);
        Iterator it = list.iterator();
        while( it.hasNext() ) {
            String ipStr = ((String)it.next());
            Hashtable ipStore = new Hashtable();
            ipsStore.put(ipStr, ipStore);
            Hashtable addr = (Hashtable)ips.get(ipStr);
            JPanel ip_panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 5));
            ip_panel.setBackground(Color.white);
            ips_panel.add(ip_panel);

            AJCheckBox ip_cb = new AJCheckBox("", true, intfcStore);
            ip_cb.obj2 = ipStr;
            ip_cb.obj3 = ip_panel;
            ip_cb.setBackground(Color.white);
            ip_cb.setFont(label_font);
            ip_panel.add(ip_cb);
            ip_cb.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        // remove ip address and its panel
                        AJCheckBox ip_cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                        if( ip_cb.isSelected() ) return;
                        ((Hashtable)((Hashtable)((Hashtable)newNode.get("interfaces")).
                                     get(ip_cb.box.get("id"))).get("ips")).remove(ip_cb.obj2);
                        GridLayout ip_layout = (GridLayout)ip_cb.box.get("ip_layout");
                        JPanel ips_panel = (JPanel)ip_cb.box.get("ips_panel");
                        ips_panel.remove((JPanel)ip_cb.obj3);
                        ip_layout.setRows(ip_layout.getRows() - 1);
                        ips_panel.revalidate();
                    }
                });

            ip_panel.add(new JLabel(" "));

            Hashtable intfc = (Hashtable)((Hashtable)newNode.get("interfaces")).get(intfcStore.get("id"));

            AJFormattedTextField ftf = new AJFormattedTextField(ipStr, 16, null, intfc, ipStr);
            ftf.setEditable(true);
            if( addr.get("verified") != null )
                ftf.setFont(bold_em_font);
            else
                ftf.setFont(label_font);
            ip_panel.add(ftf);

            Boolean active = (Boolean)addr.get("active?");
            if( active == null )
                active = new Boolean(false);
            AJCheckBox activeCB = new AJCheckBox("Active", active.booleanValue(), intfcStore);
            activeCB.obj2 = ipStr;
            activeCB.setBackground(Color.white);
            activeCB.setFont(label_font);
            ip_panel.add(activeCB);
            activeCB.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        // set up active? flag
                        AJCheckBox activeCB = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                        ((Hashtable)((Hashtable)((Hashtable)((Hashtable)newNode.get("interfaces")).
                                                 get(activeCB.box.get("id"))).get("ips")).get(activeCB.obj2)).
                            put("active?", new Boolean(activeCB.isSelected()));
                    }
                });

            label = new JLabel("  Name ");
            ipStore.put("ip_name_label", label);
            label.setFont(label_font);
            ip_panel.add(label);

            AJTextField name_tf = new AJTextField("", 17, null, ipStr);
            ipStore.put("ip_name_tf", name_tf);
            name_tf.setEditable(true);
            name_tf.setFont(label_font);
            ip_panel.add(name_tf);

            Hashtable names = Utils.nHash(addr.get("names"));
            if( names.size() > 0 ) {
                Hashtable nameH = (Hashtable)names.get(names.keys().nextElement());

                if( ((String)nameH.get("domain")).trim().toLowerCase().equals(MainWindow.userDefDomain) )
                    name_tf.setText((String)nameH.get("name"));
                else
                    name_tf.setText(nameH.get("name")+"."+nameH.get("domain"));

                if( nameH.get("verified") != null )
                    name_tf.setFont(bold_em_font);
            }

            if( Utils.nHash(newNode.get("types")).get("Advanced") == null ) {
                label.setVisible(false);
                name_tf.setVisible(false);
            }
        }

        if( rebuilding ) {
            ips_panel.add((JPanel)intfcStore.get("another_ip_b_panel"));
            ips_panel.revalidate();
        }
    } // end of makeIPs(...)

    public void makeIPCPanel(boolean rebuilding) {
        if( !rebuilding ) {
            ipc_header_panel = new JPanel(new BorderLayout());
            JPanel left_panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
            ipc_header_panel.add(left_panel, BorderLayout.WEST);
            left_panel.add(makeLabel("<html><body>&nbsp;&nbsp;IP Addresses (IPC provider) <i>Uncheck to remove</i></body></html>"));

            current_gbc.gridwidth = 2;
            add(ipc_header_panel);
            current_gbc.gridwidth = 1;
            tr();

            ipc_separator_line = line();

            ipc_big_panel = new JPanel(new BorderLayout());
            GridBagLayout layout = new GridBagLayout();
            GridBagConstraints gbc = new GridBagConstraints();
            gbc.gridwidth = 1;
            JPanel panel = new JPanel(layout);
            panel.setBackground(Color.white);
            ipc_big_panel.add(panel, BorderLayout.CENTER);

            ipc_ip_layout = new GridLayout(1, 1, 0, 2);
            ipc_ips_panel = new JPanel(ipc_ip_layout);
            ipc_ips_panel.setBackground(Color.white);

            gbc.gridx = 0;
            gbc.fill = GridBagConstraints.NONE;
            gbc.anchor = GridBagConstraints.WEST;
            gbc.gridwidth = GridBagConstraints.REMAINDER;
            layout.setConstraints(ipc_ips_panel, gbc);
            panel.add(ipc_ips_panel);
        } else
            ipc_ips_panel.removeAll();

        Hashtable ips = Utils.nHash(newNode.get("ipc_addresses"));
        ipc_ip_layout.setRows(ips.size() + 1);

        java.util.List list = Arrays.asList(ips.keySet().toArray());
        Collections.sort(list, ip_cmp);
        Iterator it = list.iterator();
        while( it.hasNext() ) {
            String ipStr = (String)it.next();
            Hashtable addr = (Hashtable)ips.get(ipStr);
            JPanel ip_panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 5));
            ip_panel.setBackground(Color.white);
            ipc_ips_panel.add(ip_panel);

            AJCheckBox ip_cb = new AJCheckBox("", true, null);
            ip_cb.obj2 = ipStr;
            ip_cb.obj3 = ip_panel;
            ip_cb.setBackground(Color.white);
            ip_cb.setFont(label_font);
            ip_panel.add(ip_cb);
            ip_cb.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        // remove ip address and its panel
                        AJCheckBox ip_cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                        if( ip_cb.isSelected() ) return;
                        ((Hashtable)newNode.get("ipc_addresses")).remove(ip_cb.obj2);
                        ipc_ips_panel.remove((JPanel)ip_cb.obj3);
                        ipc_ip_layout.setRows(ipc_ip_layout.getRows() - 1);
                        ipc_ips_panel.revalidate();
                    }
                });

            ip_panel.add(new JLabel(" "));

            AJFormattedTextField ftf = new AJFormattedTextField(ipStr, 16, null, addr, ipStr);
            ftf.setEditable(true);
            if( addr.get("verified") != null )
                ftf.setFont(bold_em_font);
            else
                ftf.setFont(label_font);
            ip_panel.add(ftf);

            Boolean active = (Boolean)addr.get("active?");
            if( active == null )
                active = new Boolean(false);
            AJCheckBox active_cb = new AJCheckBox("Active", active.booleanValue(), null);
            active_cb.obj2 = ipStr;
            active_cb.setBackground(Color.white);
            active_cb.setFont(label_font);
            ip_panel.add(active_cb);
            active_cb.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        // remove ip address and its panel
                        AJCheckBox active_cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                        ((Hashtable)((Hashtable)newNode.get("ipc_addresses")).get(active_cb.obj2)).
                            put("active?", new Boolean(active_cb.isSelected()));
                    }
                });

            label = new JLabel("  Name ");
            label.setFont(label_font);
            ip_panel.add(label);

            AJTextField name_tf = new AJTextField("", 17, null, ipStr);
            name_tf.setEditable(true);
            name_tf.setFont(label_font);
            ip_panel.add(name_tf);

            Hashtable names = Utils.nHash(addr.get("names"));
            if( names.size() > 0 ) {
                Hashtable nameH = (Hashtable)names.get(names.keys().nextElement());

                if( ((String)nameH.get("domain")).trim().toLowerCase().equals(MainWindow.userDefDomain) )
                    name_tf.setText((String)nameH.get("name"));
                else
                    name_tf.setText(nameH.get("name")+"."+nameH.get("domain"));

                if( nameH.get("verified") != null )
                    name_tf.setFont(bold_em_font);
            }
        }

        ipc_add_another_ip_panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 5));
        ipc_add_another_ip_panel.setBackground(Color.white);
        ipc_ips_panel.add(ipc_add_another_ip_panel);

        AJButton another_ip_b = new AJButton("Add another", null, AJButton.NOT_MOUSABLE);
        another_ip_b.setMargin(new Insets(0, 0, 0, 0));
        ipc_add_another_ip_panel.add(another_ip_b);
        another_ip_b.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent e) {
                    reserving_ipc = true;
                    ipAddrReserveNameLabel.setVisible(true);
                    ipAddrNameReserveTf.setVisible(true);
                    ipAddrCountLabel.setVisible(true);
                    ipAddrCountTf.setVisible(true);
                    ipAddrCountTf.setText("");
                    AJButton another_ip_b = (AJButton)e.getSource();
                    currentIntfcStore = another_ip_b.box;
                    ipAddrReserveWin = ipAddrReserveDialog.createDialog(THIS, "Reserve IPC IP Address");
                    ipAddrReserveWin.show();
                    addrSpacesJList.requestFocus();
                }
            });

        if( !rebuilding ) {
            current_gbc.gridwidth = 2;
            add(ipc_big_panel, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
            current_gbc.gridwidth = 1;
            tr();

            ipc_bottom_empty_label = XLabel("");
            tr();
        } else
            ipc_big_panel.revalidate();
    } // end of makeIPCPanel()

    private void collectNamesAndIPs(Hashtable nodeHash, Hashtable allNames, Hashtable allIPs, Vector errorMessages, boolean exclude_dirty)
        throws NetDBException
    {
        // error messages about duplicating names and/or ip addresses whill be collected
        // in the following vector
        if( errorMessages == null )
            errorMessages = new Vector();

        // first node names
        Hashtable nodeNames = (Hashtable)nodeHash.get("names");
        if( nodeNames == null ) {
            nodeNames = new Hashtable();
            nodeHash.put("names", nodeNames);
        }

        Enumeration enm = nodeNames.keys();
        while( enm.hasMoreElements() ) {
            Hashtable nameH = (Hashtable)nodeNames.get(enm.nextElement());
            String namePart = ((String)nameH.get("name")).trim();
            if( namePart.length() == 0 )
                continue;
            String fqdnName = namePart+"."+((String)nameH.get("domain")).trim();
            String fqdnLower = fqdnName.toLowerCase();
            if( allNames.get("NODE NAME "+fqdnLower)             != null ||
                allNames.get("NODE NAME ALIAS "+fqdnLower)       != null ||
                allNames.get("INTFC NAME "+fqdnLower)            != null ||
                allNames.get("IP NAME "+fqdnLower)               != null )
                errorMessages.addElement("Name (<i>"+fqdnName+"</i>) used more then once");
            if( !exclude_dirty || nameH.get("dirty") == null )
                allNames.put("NODE NAME "+fqdnLower, nameH);

            Hashtable nodeNameAliases = Utils.nHash(nameH.get("aliases"));
            Enumeration enm2 = nodeNameAliases.keys();
            while( enm2.hasMoreElements() ) {
                Hashtable aliasH = (Hashtable)nodeNameAliases.get(enm2.nextElement());
                namePart = ((String)aliasH.get("name")).trim();
                if( namePart.length() == 0 )
                    continue;
                fqdnName = namePart+"."+((String)aliasH.get("domain")).trim();
                fqdnLower = fqdnName.toLowerCase();
                if( allNames.get("NODE NAME "+fqdnLower)             != null ||
                    allNames.get("NODE NAME ALIAS "+fqdnLower)       != null ||
                    allNames.get("INTFC NAME "+fqdnLower)            != null ||
                    allNames.get("IP NAME "+fqdnLower)               != null )
                    errorMessages.addElement("Name (<i>"+fqdnName+"</i>) used more then once");
                if( !exclude_dirty || aliasH.get("dirty") == null )
                    allNames.put("NODE NAME ALIAS " + fqdnLower, aliasH);
            }

            Hashtable nodeNameMXs = Utils.nHash(nameH.get("mxs"));
            enm2 = nodeNameMXs.keys();
            while( enm2.hasMoreElements() ) {
                Hashtable mxH = (Hashtable)nodeNameMXs.get(enm2.nextElement());
                namePart = ((String)mxH.get("name")).trim();
                if( namePart.length() == 0 )
                    continue;
                fqdnName = namePart+"."+((String)mxH.get("domain")).trim();
                fqdnLower = fqdnName.toLowerCase();
                //if( allNames.get("NODE NAME MX "+fqdnLower) != null )
                //    errorMessages.addElement("Name (<i>"+fqdnName+"</i>) used more then once");
                if( !exclude_dirty || mxH.get("dirty") == null )
                    allNames.put("NODE NAME MX " + fqdnLower, mxH);
            }
        }

        // second ipc names and ipc addresses
        Hashtable ipc_addresses = Utils.nHash(nodeHash.get("ipc_addresses"));
        enm = ipc_addresses.keys();
        while( enm.hasMoreElements() ) {
            String ipc_addressStr = (String)enm.nextElement();
            Hashtable ipc_address = (Hashtable)ipc_addresses.get(ipc_addressStr);
            Hashtable ipcNames = Utils.nHash(ipc_address.get("names"));
            // these names do not have aliases and/or mxes

            String ipStr = ipc_addressStr.trim();
            if( !exclude_dirty || ipc_address.get("dirty") == null )
                if( allIPs.put(ipStr, ipStr) != null )
                    errorMessages.addElement("IP address (<i>"+ipStr+"</i>) used more then once");

            Enumeration enm2 = ipcNames.keys();
            while( enm2.hasMoreElements() ) {
                Hashtable ipcName = (Hashtable)ipcNames.get(enm2.nextElement());
                String namePart = ((String)ipcName.get("name")).trim();
                if( namePart.length() == 0 )
                    continue;
                String fqdnName = namePart+"."+((String)ipcName.get("domain")).trim();
                String fqdnLower = fqdnName.toLowerCase();
                if( allNames.get("NODE NAME "+fqdnLower)             != null ||
                    allNames.get("NODE NAME ALIAS "+fqdnLower)       != null ||
                    allNames.get("INTFC NAME "+fqdnLower)            != null ||
                    allNames.get("IP NAME "+fqdnLower)               != null )
                    errorMessages.addElement("Name (<i>"+fqdnName+"</i>) used more then once");
                if( !exclude_dirty || ipcName.get("dirty") == null )
                    allNames.put("IP NAME " + fqdnLower, ipcName);
            }
        }

        // third ips and thier names in interfaces
        Hashtable interfaces = Utils.nHash(nodeHash.get("interfaces"));
        enm = interfaces.keys();
        while( enm.hasMoreElements() ) {
            Hashtable intfc = (Hashtable)interfaces.get(enm.nextElement());
            Hashtable intfcNames = Utils.nHash(intfc.get("names"));
            Enumeration enm2 = intfcNames.keys();
            while( enm2.hasMoreElements() ) {
                Hashtable nameH = (Hashtable)intfcNames.get(enm2.nextElement());
                String namePart = ((String)nameH.get("name")).trim();
                if( namePart.length() == 0 )
                    continue;
                String fqdnName = namePart+"."+((String)nameH.get("domain")).trim();
                String fqdnLower = fqdnName.toLowerCase();
                if( allNames.get("NODE NAME "+fqdnLower)             != null ||
                    allNames.get("NODE NAME ALIAS "+fqdnLower)       != null ||
                    allNames.get("INTFC NAME "+fqdnLower)            != null ||
                    allNames.get("IP NAME "+fqdnLower)               != null )
                    errorMessages.addElement("Name (<i>"+fqdnName+"</i>) used more then once");
                if( !exclude_dirty || nameH.get("dirty") == null )
                    allNames.put("INTFC NAME " + fqdnLower, nameH);
            }

            Hashtable ips = Utils.nHash(intfc.get("ips"));
            enm2 = ips.keys();
            while( enm2.hasMoreElements() ) {
                String ipAddress = (String)enm2.nextElement();
                String ipStr = ipAddress.trim(); // trim it just in case
                Hashtable ipH = (Hashtable)ips.get(ipAddress);

                if( !exclude_dirty || ipH.get("dirty") == null )
                    if( allIPs.put(ipStr, ipStr) != null )
                        errorMessages.addElement("IP address (<i>"+ipStr+"</i>) used more then once");

                Hashtable ipNames = Utils.nHash(ipH.get("names"));
                Enumeration enm3 = ipNames.keys();
                while( enm3.hasMoreElements() ) {
                    Hashtable nameH = (Hashtable)ipNames.get(enm3.nextElement());
                    String namePart = ((String)nameH.get("name")).trim();
                    if( namePart.length() == 0 )
                        continue;
                    String fqdnName = namePart+"."+((String)nameH.get("domain")).trim();
                    String fqdnLower = fqdnName.toLowerCase();
                    if( allNames.get("NODE NAME "+fqdnLower)             != null ||
                        allNames.get("NODE NAME ALIAS "+fqdnLower)       != null ||
                        allNames.get("INTFC NAME "+fqdnLower)            != null ||
                        allNames.get("IP NAME "+fqdnLower)               != null )
                        errorMessages.addElement("Name (<i>"+fqdnName+"</i>) used more then once");
                    if( !exclude_dirty || nameH.get("dirty") == null )
                        allNames.put("IP NAME " + fqdnLower, nameH);
                }
            }
        }

        // check for errors
        if( errorMessages.size() > 0 ) {
            //for( int i = 0; i < errorMessages.size(); i++ ) {
                //System.out.println("ERROR: "+errorMessages.elementAt(i));
            //}
            throw new NetDBException("Duplicate name/ip use or format error", errorMessages);
        }
    } // end of collectNamesAndIPs(...)

    private void reserveIPforCreate(Hashtable ips, String start_as, String start_ip, boolean doWarn) {
        msg.setText("Reserving ip address...");
        try {
            Hashtable result =
                MainWindow.netdb.reserveIP(start_as, start_ip, 1);
            String requestedIP = (String)result.keys().nextElement();
            Hashtable reservedTable = (Hashtable)result.get(requestedIP);
            Hashtable resIPs = Utils.nHash(reservedTable.get("ip_addresses"));
            String reservedIP = (String)resIPs.keys().nextElement();
            reservedIPs.put(reservedIP.trim(), reservedIP);

            Hashtable ip = new Hashtable();
            ip.put("active?", new Boolean(true));
            ip.put("names", new Hashtable());
            ips.put(reservedIP, ip);
            open_as_pickup = false;
        } catch(java.rmi.ServerException exx) {
            if( doWarn ) {
                ePanelLabel.setText("<html><body><font color=\"#ff0000\">&nbsp;Failed to reserve ip address in "+
                                    start_as+"</body></html>");
                header_panel.add(ePanel);
                header_panel.revalidate();
            }

            if( exx.getCause() instanceof SessionTimeoutException )
                X.handleTimeout();

            else if( exx.getCause() instanceof RemoteNetDBException ) {
                RemoteNetDBException ne = (RemoteNetDBException)exx.getCause();
                Vector errors = ne.getMessages();
                for( int i = 0; i < errors.size(); i++ )
                    Utils.debug((String)errors.elementAt(i));

            } else if( exx.getCause() instanceof RemoteException ) {
                Utils.debug(((RemoteException)exx.getCause()).getMessage());

            } else
                Utils.debug(exx.getMessage());

        } catch (java.rmi.NoSuchObjectException ex) {
            X.handleTimeout();

        } catch (java.rmi.MarshalException ex) {
            X.handleTimeout();

        } catch (Exception ex) {
            if( doWarn ) {
                ePanelLabel.setText("<html><body><font color=\"#ff0000\">&nbsp;Failed to reserve ip address in "+start_as+"</body></html>");
                header_panel.add(ePanel);
                header_panel.revalidate();
            }
            Utils.debug(ex.getMessage());
        }
    } // end of reserveIPforCreate(...)

    public int countDots(String txt) {
        int dots = 0;
        int index = -1;
        for(;;) {
            index = txt.indexOf(".", index);
            if( index > -1 ) {
                dots++;
                index++;
            } else
                break;
        }
        return dots;
    }
}

/*
  intfcStore
  ----------
  key             value
  ---             -----
  id                 (String) !interface id!
  layout             (GridBagLayout)
  gbc                (GridBagConstraints)
  panel              (JPanel)
  hw_name_label      (JLabel) " Name"
  hw_name_tf         (JTextField)
  ipsStore           (Hashtable)
                     %ipStr%  -> (Hashtable)
                                 ip_name_label  -> (JLabel) "  Name "
                                 ip_name_tf     -> (AJTextField)
  ip_layout          (GridLayout)
  ips_panel          (JPanel)
  another_ip_b_panel (JPanel)
  bottom_panel       (JPanel)
  dhcp_options_ta    (JTextArea)
  dhcp_sp            (JScrollPane)
  dhcpB              (AJButton)
*/
