/************************************************************************
 *
 * QueueFrame.java -
 *
 * $Id: QueueFrame.java,v 1.1.1.1 2006/06/30 23:38:03 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import java.lang.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import java.rmi.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import stanford.netdb.utils.*;
import javax.swing.border.*;

public class QueueFrame extends XFrame {

    public static QueueFrame THIS;
    public JPanel mainPanel = new JPanel();

    public JLabel max_concurrent_calls;
    public JLabel running;
    public JLabel pending;

    public QueueFrame() {
        super("Manage Queue");
        if( THIS != null ) {
            THIS.setState(Frame.NORMAL);
            THIS.toFront();
            return;
        }

        THIS = this;
        register();
        initFrame();
    }

    public void dispose() {
        THIS = null;
        super.dispose();
    }

    private void initFrame() {
        init(); // init XFrame bowls

        tool_bar.removeAll();
        tool_bar.add(modifyB);
        tool_bar.add(closeB);

        modifyB.setText("Reload");
        modifyB.setEnabled(true);
        modifyB.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    try {
                        MainWindow.queue = MainWindow.netdb.getQueue();
                        max_concurrent_calls.setText(MainWindow.queue.get("max_concurrent_calls").toString());
                        running.setText(MainWindow.queue.get("running").toString());
                        pending.setText(MainWindow.queue.get("pending").toString());
                    } catch(java.rmi.ServerException ex) {
                        if( ex.getCause() instanceof SessionTimeoutException )
                            X.handleTimeout();

                        else {
                            JOptionPane.
                                showMessageDialog(MainWindow.win, ex.getCause().getMessage(),
                                                  "Error", JOptionPane.ERROR_MESSAGE);
                            closeB.doClick();
                        }
                    } catch (java.rmi.NoSuchObjectException ex) {
                        X.handleTimeout();

                    } catch (java.rmi.MarshalException ex) {
                        X.handleTimeout();

                    } catch (Exception ex) {
                        JOptionPane.
                            showMessageDialog(MainWindow.win, ex.getMessage(),
                                              "Error", JOptionPane.ERROR_MESSAGE);
                        closeB.doClick();
                    }
                }
            });

        ActionListener[] al = closeB.getActionListeners();
        for( int i = 0; i < al.length; i++ )
            closeB.removeActionListener(al[i]);

        closeB.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    dispose();
                }
            });

        mainPanel.setBackground(Color.white);
        central_panel.removeAll();
        central_panel.add(mainPanel);
        current_panel = mainPanel;
        current_panel.setLayout(current_gb_layout);

        // empty row at the top
        XLabel(" ");
        tr();

        HTMLPanel("Max concurrent calls");
        td();
        JPanel right_panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        right_panel.setBackground(Color.white);
        max_concurrent_calls = new JLabel(MainWindow.queue.get("max_concurrent_calls").toString());
        max_concurrent_calls.setFont(label_font);
        right_panel.add(max_concurrent_calls);

        JButton plus_b = new JButton("up");
        plus_b.setFont(label_font);
        plus_b.setMargin(new Insets(0, 0, 0, 0));
        right_panel.add(plus_b);
        plus_b.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    try {
                        Long mcc = (Long)MainWindow.queue.get("max_concurrent_calls");
                        MainWindow.netdb.setMaxConcurrent(mcc.longValue()+1);
                        MainWindow.queue = MainWindow.netdb.getQueue();
                        max_concurrent_calls.setText(MainWindow.queue.get("max_concurrent_calls").toString());
                        running.setText(MainWindow.queue.get("running").toString());
                        pending.setText(MainWindow.queue.get("pending").toString());
                    } catch(java.rmi.ServerException ex) {
                        if( ex.getCause() instanceof SessionTimeoutException )
                            X.handleTimeout();

                        else {
                            JOptionPane.
                                showMessageDialog(MainWindow.win, ex.getCause().getMessage(),
                                                  "Error", JOptionPane.ERROR_MESSAGE);
                            closeB.doClick();
                        }

                    } catch (java.rmi.NoSuchObjectException ex) {
                        X.handleTimeout();

                    } catch (java.rmi.MarshalException ex) {
                        X.handleTimeout();

                    } catch (Exception ex) {
                        JOptionPane.showMessageDialog(MainWindow.win,
                                                      ex.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
                        closeB.doClick();
                    }
                }
            });

        JButton minus_b = new JButton("down");
        minus_b.setFont(label_font);
        minus_b.setMargin(new Insets(0, 0, 0, 0));
        right_panel.add(minus_b);
        minus_b.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    try {
                        Long mcc = (Long)MainWindow.queue.get("max_concurrent_calls");
                        MainWindow.netdb.setMaxConcurrent(mcc.longValue()-1);
                        MainWindow.queue = MainWindow.netdb.getQueue();
                        max_concurrent_calls.setText(MainWindow.queue.get("max_concurrent_calls").toString());
                        running.setText(MainWindow.queue.get("running").toString());
                        pending.setText(MainWindow.queue.get("pending").toString());
                    } catch(java.rmi.ServerException ex) {
                        if( ex.getCause() instanceof SessionTimeoutException )
                            X.handleTimeout();

                        else {
                            JOptionPane.
                                showMessageDialog(MainWindow.win, ex.getCause().getMessage(),
                                                  "Error", JOptionPane.ERROR_MESSAGE);
                            closeB.doClick();
                        }

                    } catch (java.rmi.NoSuchObjectException ex) {
                        X.handleTimeout();

                    } catch (java.rmi.MarshalException ex) {
                        X.handleTimeout();

                    } catch (Exception ex) {
                        JOptionPane.showMessageDialog(MainWindow.win,
                                                      ex.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
                        closeB.doClick();
                    }
                }
            });

        add(right_panel, GridBagConstraints.NONE, GridBagConstraints.WEST);
        tr();

        HTMLPanel("Running");
        td();
        right_panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        right_panel.setBackground(Color.white);
        running = new JLabel(MainWindow.queue.get("running").toString());
        running.setFont(label_font);
        right_panel.add(running);
        add(right_panel, GridBagConstraints.NONE, GridBagConstraints.WEST);
        tr();

        HTMLPanel("Pending");

        td();
        right_panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        right_panel.setBackground(Color.white);
        pending = new JLabel(MainWindow.queue.get("pending").toString());
        pending.setFont(label_font);
        right_panel.add(pending);
        add(right_panel, GridBagConstraints.NONE, GridBagConstraints.WEST);
        tr();

        central_panel.doLayout();

        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        pack();
        setSize(900, 700);
        Dimension scrnsize = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation( (int)((scrnsize.getWidth()-getWidth())/2),
                     (int)((scrnsize.getHeight()-getHeight())/2) );
        show();
    }
}
