#!/usr/pubsw/bin/zsh

#####  NetDB CLI testing program  #####

# Do the following by hand:

#   Check the output from 'netdb --usage', 'netdb --help', and 'netdb --version'
#   Make sure that the database contains the test data specified at,
#       http://www.stanford.edu/group/networking/netdb/projects/andromeda/docs/test-data.txt

export ASTROGWC=astro-gw.Cogswell-Cogs.COM
export ASTROGWS=astro-gw.Spacely-Sprockets.COM
export ASTROGWI=astro-gw-1-1.Cogswell-Cogs.COM
export JETSONMBDSL=jetson-mbdsl.Spacely-Sprockets.COM
export MACS=mac-spacely.Spacely-Sprockets.COM
export MACSSN=mac-spacely-super-node.Spacely-Sprockets.COM
export PC=pc-cogswell.Cogswell-Cogs.COM

### Automated testing

# save pointer to the program to test; define some constants and commands
export NETDB=$1
export MODULES=$2
DIFF=/usr/pubsw/bin/diff
DIFFS=/tmp/diff.out.$$
REALDIFFS=/tmp/real.diffs.$$

SKIPHELP=yes

h1() { echo; echo '#######' $@; echo }
h2() { echo; echo '###---' $@        }


###  --usage         display this help and exit
###  --help          print a detailed description of how to use netdb
###  --version       print netdb version information

if [ -z $SKIPHELP ] ; then  # set this to skip this section

h1 "Testing '--usage', '--help', and '--version'"

$NETDB --usage >& /tmp/netdb.usage.$$     || echo "--help produces bad error code, $?"
$NETDB --help >& /tmp/netdb.help.$$       || echo "--help produces bad error code, $?"
$NETDB --version >& /tmp/netdb.version.$$ || echo "--version produces bad error code, $?"

cuhv() {
    $DIFF /tmp/netdb.$1.$$ =($NETDB $2) >& /dev/null \
    || echo "'netdb $2' output doesn't match --$1 output"
}

cuhv usage --usage; cuhv usage --usag; cuhv usage --usa; cuhv usage --us; cuhv usage --u
cuhv usage -usage;  cuhv usage -usag;  cuhv usage -usa;  cuhv usage -us;  cuhv usage -u; cuhv usage ""

cuhv help --help; cuhv help --hel; cuhv help --he; cuhv help --h
cuhv help -help;  cuhv help -hel;  cuhv help -he;  cuhv help -h

cuhv version --version; cuhv version --versio; cuhv version --versi; cuhv version --vers; cuhv version --ver; cuhv version --ve; cuhv version --v
cuhv version -version;  cuhv version -versio;  cuhv version -versi;  cuhv version -vers;  cuhv version -ver;  cuhv version -ve;  cuhv version -v

rm /tmp/netdb.usage.$$ /tmp/netdb.help.$$ /tmp/netdb.version.$$

fi  # SKIPHELP


###  netdb node info        # never skip this section

echo
echo "NetDB CLI regression testing"
echo
echo "Module : node info"

$NETDB node info $ASTROGWC > /tmp/$ASTROGWC.$$ \
    || echo "'node info' not working or bad exit code"

echo $MACS > /tmp/foo.$$
$NETDB node info --in /tmp/foo.$$ > /tmp/$MACS.$$ \
    || echo "'node info' not working or bad exit code"

$NETDB node info $MACSSN > /tmp/$MACSSN.$$ \
    || echo "'node info' not working or bad exit code"

$NETDB node info $PC > /tmp/$PC.$$ \
    || echo "'node info' not working or bad exit code"

$NETDB node info $JETSONMBDSL > /tmp/$JETSONMBDSL.$$ \
    || echo "'node info' not working or bad exit code"

rm /tmp/foo.$$
perl -pi -e 's/^(\s*date-....ted: ).*$/$1/' /tmp/$ASTROGWC.$$ /tmp/$MACS.$$ /tmp/$PC.$$ /tmp/$MACSSN.$$ /tmp/$JETSONMBDSL.$$
perl -pi -e 's/^(\s*updated-by: ).*$/$1/' /tmp/$ASTROGWC.$$ /tmp/$MACS.$$ /tmp/$PC.$$ /tmp/$MACSSN.$$ /tmp/$JETSONMBDSL.$$


###  These little functions will help us with the rest of testing

run() { echo $@; $@ }

compare() {
    $DIFF -C 2 --label $1.orig /tmp/$1.$$ \
               --label $1      =($NETDB node info $1 | perl -pi -e 's/^(\s*date-....ted: ).*$/$1/' \
                                 | perl -pi -e 's/^(\s*updated-by: ).*$/$1/' ) \
    > $DIFFS

    $DIFF -bqs $DIFFS $REALDIFFS >& /dev/null
    if [ $? -eq 0 ]; then
        #echo "  Ok"
        #echo "~~~ $1 okay"
    else
        echo '!!! TEST FAILED, diffs were:'; echo

    if [ -s $DIFFS ]; then
        perl -pi -e 's/^/  | /' < $DIFFS
    else
        echo '    *** EMPTY ***'
    fi
    echo; echo '  ... but should have been:'; echo
    if [ -s $REALDIFFS ]; then
        perl -pi -e 's/^/  | /' < $REALDIFFS
    else
        echo '    *** EMPTY ***'
    fi
    fi
}

# here is the main 'engine'

MODULES=`awk '/modules/,/\/modules/' $MODULES | awk '!/modules/'`
echo $MODULES | \
while read amodule
do
  export i=1
  echo "Testing module :" $amodule
  TESTS=`awk '/tests/,/\/tests/' $amodule.mod | awk '!/tests/' | awk '!/#/'`
  echo $TESTS | \
  while read atest
    do
      TESTDEF=`cat $amodule.mod  | perl -n -e "/^<\/$atest>\s*/ and \\$body = 'false'; \\$body eq 'true' and print;  /^<$atest>\s*/ and \\$body = 'true';"`
      MESG=`echo $TESTDEF | perl -n -e '/^<\/msg>\s*/ and $msg = "false"; $msg eq "true" and print;  /^<msg>\s*/ and $msg = "true";' `
      CMD=`echo $TESTDEF | perl -n -e '/^<\/cmd>\s*/ and $cmd = "false"; $cmd eq "true" and print;  /^<cmd>\s*/ and $cmd = "true";'`
      OBJ=`echo $TESTDEF | perl -n -e '/^<\/obj>\s*/ and $obj = "false"; $obj eq "true" and print;  /^<obj>\s*/ and $obj = "true";'`
      echo $TESTDEF | awk  "/\<diff\>/,/\<\/diff\>/"  | awk "!/diff\>/"  > $REALDIFFS
      #echo "    ### Testing : " $MESG
      echo -n $i
      #echo "    $CMD"
      echo "#!/bin/sh" > tempcmd.sh
      echo $CMD >> tempcmd.sh
      chmod u+x tempcmd.sh
      ./tempcmd.sh 2>&1 > /dev/null
      if [ -n "$OBJ" ];
      then
        compare $OBJ
      fi
      echo "  Ok"
      i=`expr $i + 1`;
    done
  echo
done

###  netdb list [ groups | locations ] [string]
