-- NetDB_InterfaceType.sql --- contains functions related to interface types

create or replace package body NetDB_InterfaceType as

    procedure del ( p_op_uid in int,
                    p_type   in varchar) as
    begin
        NetDB_Access.has_access_or_die (p_op_uid, 'interfacetype');
        delete from InterfaceType where type = p_type;
    end del;


    procedure ins ( p_op_uid in int,
                    p_type in varchar,
                    p_desc in varchar ) as
    begin
        NetDB_Access.has_access_or_die (p_op_uid, 'interfacetype');
        NetDB_Util.validate (p_type, 'type', 'Interface types must have a type.');
        NetDB_Util.validate (p_desc, 'description');

        insert into InterfaceType (type, description) values (p_type, p_desc);
    exception
        when NetDB.unique_constraint_failed then
            NetDB_Util.throw_record_already_exists ('interfacetype', p_type);
        when NetDB.value_too_large then
            NetDB_Util.throw_field_too_long (sqlerrm);
    end ins;


    procedure upd ( p_op_uid   in int,
                    p_type     in varchar,
                    p_desc_new in varchar ) as
    begin
        NetDB_Access.has_access_or_die (p_op_uid, 'interfacetype');
        NetDB_Util.validate (p_type,     'type', 'Interface types must have a type.');
        NetDB_Util.validate (p_desc_new, 'description');

        update InterfaceType set description = p_desc_new where type = p_type;
    exception
        when NetDB.unique_constraint_failed then
            NetDB_Util.throw_record_already_exists ('interfacetype', p_type);
        when NetDB.value_too_large then
            NetDB_Util.throw_field_too_long (sqlerrm);
    end upd;


    function verify_or_die (p_type in varchar) return varchar
    as
        v_type InterfaceType.type%TYPE := null;
    begin
        select type into v_type from InterfaceType where type = p_type;

        return v_type;
    exception
        when NO_DATA_FOUND then
            NetDB_Util.throw_record_does_not_exist ('interfacetype', p_type);
        when TOO_MANY_ROWS then
            NetDB_Util.throw_single_record_expected ('interfacetype');
    end verify_or_die;

end NetDB_InterfaceType;
/
