-- NetDB_Log.sql --- Logging functions

create or replace package body NetDB_Log as

    -- Save the action for logging and verify that the action is valid.
    procedure set_action (p_action in varchar) as
    begin
        if (lower (p_action) not in ('insert', 'update', 'delete')) then
            NetDB_Util.throw_unknown_action (p_action);
        end if;

        insert into log_action values (p_action);
    end set_action;


    function get_action return varchar
    as
        v_action varchar(6) := null;
    begin
        select action into v_action from log_action;

        return v_action;
    exception
        when NO_DATA_FOUND then
            return null;
        when TOO_MANY_ROWS then
            raise_application_error (NetDB.ex_idiot_programmer, 'Multiple active log operations.');
    end get_action;


    procedure insert_domain ( p_record_id in int,
                              p_log_id    in int ) as
    begin
        insert into LogName (log_id,   name)
                     values (p_log_id, NetDB_Domain.handle (p_record_id));
    end insert_domain;


    procedure insert_admin ( p_record_id in int,
                             p_log_id    in int ) as
    begin
        insert into LogName (log_id,   name)
                     values (p_log_id, NetDB_AdminTeam.handle (p_record_id));
    end insert_admin;


    procedure insert_group ( p_record_id in int,
                             p_log_id    in int ) as
    begin
        insert into LogName (log_id,   name)
                     values (p_log_id, NetDB_Owner.handle (p_record_id));
    end insert_group;


    procedure insert_network ( p_record_id in int,
                               p_log_id    in int ) as
    begin
        for r in (select distinct fullname from Name where network = p_record_id) loop
            insert into LogName (log_id, name) values (p_log_id, r.fullname);
        end loop;
    end insert_network;


    procedure insert_node ( p_record_id in int,
                            p_log_id    in int ) as
    begin
        for r in (select distinct fullname from Name where node = p_record_id) loop
            insert into LogName (log_id, name) values (p_log_id, r.fullname);
        end loop;

        for r in ( select ip.ipaddress
                     from interface inter,
                          ipaddress ip
                    where inter.node = p_record_id and inter.id = ip.interface )
        loop
            insert into LogIPaddress (log_id, ipaddress) values (p_log_id, r.ipaddress);
        end loop;

     -- We don't think this is a good idea.
     -- for r in (select * from ipaddress where ipcnode = p_record_id)
     -- loop
     --     insert into LogIPaddress (log_id, ipaddress) values (p_log_id, r.ipaddress);
     -- end loop;
    end insert_node;


    procedure insert_user ( p_record_id in int,
                            p_log_id    in int ) as
    begin
        insert into LogName (log_id,   name)
                     values (p_log_id, NetDB_User.handle (p_record_id));
    end insert_user;


    procedure ins ( p_user_id     in int,
                    p_action      in varchar,
                    p_record_id   in int,
                    p_record_type in varchar )
    as
        v_log_id       int             := null;
        v_record_type  varchar(32)     := lower (p_record_type);
        v_state        Node.state%type := null;
    begin
        if (p_action is null) then
            return;
        end if;
        
        if (v_record_type = 'node') then
            select state into v_state from Node where id = p_record_id;
        end if;
        
        insert into Log (action,   person,    record_id,   recordtype,    logdate,      state)
                 values (p_action, p_user_id, p_record_id, v_record_type, systimestamp, v_state)
        returning id into v_log_id;

           if (v_record_type = 'adminteam') then insert_admin   (p_record_id, v_log_id);
        elsif (v_record_type = 'domain')    then insert_domain  (p_record_id, v_log_id);
        elsif (v_record_type = 'owner')     then insert_group   (p_record_id, v_log_id);
        elsif (v_record_type = 'network')   then insert_network (p_record_id, v_log_id);
        elsif (v_record_type = 'node')      then insert_node    (p_record_id, v_log_id);
        elsif (v_record_type = 'user')      then insert_user    (p_record_id, v_log_id);
        else
            NetDB_Util.throw_unknown_record_type (p_record_type);
        end if;

        delete from log_action;
    end ins;

end NetDB_Log;
/
