-- NetDB_RecordType.sql --- Contains functions for translation [Rr]ecord types.

create or replace package body NetDB_RecordType as

    type record_list is table of record_names index by varchar(32);
    records record_list;

    -- Returns the access group that a [Rr]ecord belongs to.
    function access (rec_type in varchar) return varchar as
    begin
        if (rec_type is null) then
            return null;
        end if;

        return records(lower (rec_type)).access_name;
    exception
        when NO_DATA_FOUND then
            Util.debugln ('Unknown record type "' || rec_type || '". It should probably be added to the initialization block.');
            return rec_type;
    end access;


    -- Returns the the pretty name for a [Rr]ecord.
    function display (rec_type in varchar) return varchar as
    begin
        if (rec_type is null) then
            return null;
        end if;

        return records(lower (rec_type)).display_name;
    exception
        when NO_DATA_FOUND then
            Util.debugln ('Unknown record type "' || rec_type || '". It should probably be added to the initialization block.');
            return rec_type;
    end display;


    -- Returns the the pretty, plural name for a [Rr]ecord.
    function plural (rec_type in varchar) return varchar as
    begin
        if (rec_type is null) then
            return null;
        end if;

        if (records(lower (rec_type)).plural_name is not null) then
            return records(lower (rec_type)).plural_name;
        else
            return records(lower (rec_type)).display_name || 's';
        end if;
    exception
        when NO_DATA_FOUND then
            Util.debugln ('Unknown record type "' || rec_type || '". It should probably be added to the initialization block.');
            return rec_type || 's';
    end plural;

begin
    -- Records                  :=                access_name         display_name          plural_name
    records('adminteam')        := record_names ('adminteam',        'Admin Team',          null);
    records('domain')           := record_names ('domain',           'Domain',              null);
    records('network')          := record_names ('network',          'Network',             null);
    records('node')             := record_names ('node',             'Node',                null);
    records('nodegroup')        := record_names ('nodegroup',        'Node Group',          null);
    records('owner')            := record_names ('owner',            'Group',               null);
    records('access')           := record_names ( null,              'Access Group',        null);
    records('user')             := record_names ('users',            'User',                null);
    records('users')            := record_names ('users',            'User',                null);

    -- Record Sub-types         :=                access_name         display_name          plural_name
    records('advanced')         := record_names ('advancednode',     'Advanced Node',       null);
    records('advancednode')     := record_names ('advancednode',     'Advanced Node',       null);
    records('dhcpserver')       := record_names ('node',             'DHCP Server',         null);
    records('ipcprovider')      := record_names ('ipcprovider',      'IPC Provider',        null);
    records('router')           := record_names ('router',           'Router',              null);
    records('staticipclient')   := record_names ('node',             'Static IP Client',    null);
    records('template')         := record_names ('node',             'Template',            null);

    -- children of Records      :=                access_name         display_name          plural_name
    records('interface')        := record_names ('node',             'Interface',           null);
    records('address')          := record_names ( null,              'Address',             'Addresses');
    records('addressspace')     := record_names ('network',          'IP Address Space',    null);
    records('name')             := record_names ('name',             'Name',                null);

    -- records                  :=                access_name         display_name          plural_name
    records('dhcpoption')       := record_names ('systemmanagement', 'DHCP Option',         null   );
    records('dhcpservice')      := record_names ('systemmanagement', 'DHCP Service',        null   );
    records('interfacetype')    := record_names ('systemmanagement', 'Interface Type',      null   );
    records('limitedname')      := record_names ('systemmanagement', 'Limited Name',        null   );
    records('location')         := record_names ('systemmanagement', 'Location',            null   );
    records('make')             := record_names ('systemmanagement', 'Make',                null   );
    records('model')            := record_names ('systemmanagement', 'Model',               null   );
    records('modeltype')        := record_names ( null,              'Model Type',          null   );
    records('nodetype')         := record_names ('systemmanagement', 'Node Type',           null   );
    records('organization')     := record_names ('systemmanagement', 'Department',          null   );
    records('os')               := record_names ('systemmanagement', 'Operating System',    null   );
    records('person')           := record_names ( null,              'Person',             'People');
    records('resname')          := record_names ('systemmanagement', 'Reserved Name',       null   );
    records('state')            := record_names ('systemmanagement', 'State',               null   );
    records('systemmanagement') := record_names ('systemmanagement', 'System Management',  ' '     );
    records('mx')               := record_names ( null,              'MX',                 'MXes'  );

    -- other access groups.
    records('exemptnodestate')  := record_names ( 'exemptnodestate', 'Exempt Node State',   null   );
end NetDB_RecordType;
/
