-- Util.sql --- general utility functions

create or replace package body Util as

    -- datediff() : return difference between dates in 'datepart' units
    function datediff
    (
      datepart  varchar,   -- 'day(s)', 'hour(s)', 'minute(s)', or 'second(s)'
      date1     timestamp,
      date2     timestamp  := systimestamp
    )
      return float as

        type part_to_seconds is table of float index by varchar(6);

        seconds     part_to_seconds;
        v_seconds   int;
        unit        varchar(6) := regexp_replace(datepart,'s$');

    begin
        seconds('second') := 1;
        seconds('minute') := 60;
        seconds('hour')   := 60 * seconds('minute');
        seconds('day')    := 24 * seconds('hour');

        if not seconds.exists(lower(unit)) then
            raise_application_error(-20098, 'Invalid date part: ' || datepart);
        end if;

        v_seconds := extract(day    from (date2 - date1)) * seconds('day')    +
                     extract(hour   from (date2 - date1)) * seconds('hour')   +
                     extract(minute from (date2 - date1)) * seconds('minute') +
                     extract(second from (date2 - date1));

        return(v_seconds/seconds(lower(unit)));

    end datediff;


    -- drop_type_if_exists - allow 'drop type ...' in package specs w/o problems
    procedure drop_type_if_exists(p_type varchar) as
        N int;
    begin
        select count(*) into N from user_objects where object_name=upper(p_type)
                                                   and object_type='TYPE';
        if N = 1 then EXECUTE IMMEDIATE 'drop type ' || p_type; end if;
    end drop_type_if_exists;


    -- to_boolean - convert integer to boolean (0/NULL == FALSE, otherwise TRUE)
    function to_boolean(input int) return boolean as
    begin
        if input is null or input = 0 then return FALSE; else return TRUE; end if;
    end to_boolean;

    -- to_int - converts boolean to integer (NULL/FALSE == 0, otherwise 1)
    function to_int(input boolean) return int as
    begin
        if input is not null and input then return 1; else return 0; end if;
    end to_int;

    procedure print(message in varchar) as begin dbms_output.put(message); end print;

    procedure println(message in varchar) as begin dbms_output.put_line(message); end println;

    procedure println as begin dbms_output.new_line(); end println;

    procedure debug(message in varchar) as begin
        if print_debug_messages then dbms_output.put(message); end if;
    end debug;

    procedure debugln(message in varchar) as begin
        if print_debug_messages then dbms_output.put_line(message); end if;
    end debugln;

    procedure debugln as begin
        if print_debug_messages then dbms_output.new_line(); end if;
    end debugln;

end Util;
/
