-- NetDB.sql --- NetDB general package

EXEC Util.drop_type_if_exists ('result_set');
EXEC Util.drop_type_if_exists ('result_row');

create type result_row
    as object ( type               varchar(7),
                key                varchar(1024),
                hash_level         int,
                value_type         varchar(256),
                value_string       varchar(256),
                value_timestamp    timestamp,
                value_int          int );
/

create type result_set as table of result_row;
/

create or replace package NetDB as

    -- Named Oracle codes.
    ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
                                                             unique_constraint_failed      EXCEPTION;    PRAGMA EXCEPTION_INIT (unique_constraint_failed,       -00001);
                                                             check_constraint_failed       EXCEPTION;    PRAGMA EXCEPTION_INIT (check_constraint_failed,        -02290);
                                                             parent_does_not_exist         EXCEPTION;    PRAGMA EXCEPTION_INIT (parent_does_not_exist,          -02291);
                                                             children_exist                EXCEPTION;    PRAGMA EXCEPTION_INIT (children_exist,                 -02292);
                                                             conversion_error              EXCEPTION;    PRAGMA EXCEPTION_INIT (conversion_error,               -06502);
                                                             value_too_large               EXCEPTION;    PRAGMA EXCEPTION_INIT (value_too_large,                -12899);

    -- External exception codes.
    ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
    ex_idiot_programmer             constant int := -20095;  idiot_programmer              EXCEPTION;    PRAGMA EXCEPTION_INIT (idiot_programmer,               -20095);
    ex_impossible_code_path         constant int := -20096;  impossible_code_path          EXCEPTION;    PRAGMA EXCEPTION_INIT (impossible_code_path,           -20096);
    ex_bad_ip_arithmetic            constant int := -20097;  bad_ip_arithmetic             EXCEPTION;    PRAGMA EXCEPTION_INIT (bad_ip_arithmetic,              -20097);
    ex_invalid_date_part            constant int := -20098;  invalid_date_part             EXCEPTION;    PRAGMA EXCEPTION_INIT (invalid_date_part,              -20098);
    ex_IPv6_not_implemented         constant int := -20099;  IPv6_not_implemented          EXCEPTION;    PRAGMA EXCEPTION_INIT (IPv6_not_implemented,           -20099);
    ex_invalid_IP_address           constant int := -20100;  invalid_IP_address            EXCEPTION;    PRAGMA EXCEPTION_INIT (invalid_IP_address,             -20100);

    -- Exceptions blessed by the "error code tsar".
    ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
    -- generic low-level errors thrown by triggers and functions called by triggers
    ex_system_supplies_id           constant int := -20500;  system_supplies_id            EXCEPTION;    PRAGMA EXCEPTION_INIT (system_supplies_id,             -20500);
    ex_id_must_not_change           constant int := -20501;  id_must_not_change            EXCEPTION;    PRAGMA EXCEPTION_INIT (id_must_not_change,             -20501);
    ex_system_maintained            constant int := -20502;  system_maintained             EXCEPTION;    PRAGMA EXCEPTION_INIT (system_maintained,              -20502);

    -- generic programming/data errors
    ex_bad_parameter                constant int := -20510;  bad_parameter                 EXCEPTION;    PRAGMA EXCEPTION_INIT (bad_parameter,                  -20510);
    ex_too_many_characters          constant int := -20511;  too_many_characters           EXCEPTION;    PRAGMA EXCEPTION_INIT (too_many_characters,            -20511);
    ex_invalid_email_address        constant int := -20512;  invalid_email_address         EXCEPTION;    PRAGMA EXCEPTION_INIT (invalid_email_address,          -20512);

    -- netdb record errors
    ex_single_record_expected       constant int := -20520;  single_record_expected        EXCEPTION;    PRAGMA EXCEPTION_INIT (single_record_expected,         -20520);
    ex_record_already_exists        constant int := -20521;  record_already_exists         EXCEPTION;    PRAGMA EXCEPTION_INIT (record_already_exists,          -20521);
    ex_record_not_found             constant int := -20522;  record_not_found              EXCEPTION;    PRAGMA EXCEPTION_INIT (record_not_found,               -20522);
    ex_record_requires_group        constant int := -20523;  record_requires_group         EXCEPTION;    PRAGMA EXCEPTION_INIT (record_requires_group,          -20523);
    ex_record_requires_name         constant int := -20524;  record_requires_name          EXCEPTION;    PRAGMA EXCEPTION_INIT (record_requires_name,           -20524);
    ex_record_in_use                constant int := -20525;  record_in_use                 EXCEPTION;    PRAGMA EXCEPTION_INIT (record_in_use,                  -20525);
    ex_record_not_reserved          constant int := -20526;  record_not_reserved           EXCEPTION;    PRAGMA EXCEPTION_INIT (record_not_reserved,            -20526);

    -- lock errors
    ex_record_locked_by_other       constant int := -20530;  record_locked_by_other        EXCEPTION;    PRAGMA EXCEPTION_INIT (record_locked_by_other,         -20530);
    ex_record_not_locked            constant int := -20531;  record_not_locked             EXCEPTION;    PRAGMA EXCEPTION_INIT (record_not_locked,              -20531);

    -- node record errors
    ex_node_must_have_admin         constant int := -20540;  node_must_have_admin          EXCEPTION;    PRAGMA EXCEPTION_INIT (node_must_have_admin,           -20540);
    ex_node_must_have_OS            constant int := -20541;  node_must_have_OS             EXCEPTION;    PRAGMA EXCEPTION_INIT (node_must_have_OS,              -20541);
    ex_no_moving_interfaces         constant int := -20542;  no_moving_interfaces          EXCEPTION;    PRAGMA EXCEPTION_INIT (no_moving_interfaces,           -20542);

    -- IP address errors
    ex_no_such_subnet               constant int := -20550;  no_such_subnet                EXCEPTION;    PRAGMA EXCEPTION_INIT (no_such_subnet,                 -20550);
    ex_subnet_ip_mismatch           constant int := -20551;  subnet_ip_mismatch            EXCEPTION;    PRAGMA EXCEPTION_INIT (subnet_ip_mismatch,             -20551);
    ex_address_is_subnet            constant int := -20552;  address_is_subnet             EXCEPTION;    PRAGMA EXCEPTION_INIT (address_is_subnet,              -20552);
    ex_address_is_broadcast         constant int := -20553;  address_is_broadcast          EXCEPTION;    PRAGMA EXCEPTION_INIT (address_is_broadcast,           -20553);
    ex_no_moving_IPaddresses        constant int := -20554;  no_moving_IPaddresses         EXCEPTION;    PRAGMA EXCEPTION_INIT (no_moving_IPaddresses,          -20554);
    ex_cannot_reserve_address       constant int := -20555;  cannot_reserve_address        EXCEPTION;    PRAGMA EXCEPTION_INIT (cannot_reserve_address,         -20555);

    -- user record errors
    ex_user_must_have_dept          constant int := -20560;  user_must_have_dept           EXCEPTION;    PRAGMA EXCEPTION_INIT (user_must_have_dept,            -20560);
    ex_user_must_be_LNA             constant int := -20561;  user_must_be_LNA              EXCEPTION;    PRAGMA EXCEPTION_INIT (user_must_be_LNA,               -20561);
    ex_inactive_user                constant int := -20562;  inactive_user                 EXCEPTION;    PRAGMA EXCEPTION_INIT (inactive_user,                  -20562);
    ex_no_access_to_record          constant int := -20563;  no_access_to_record           EXCEPTION;    PRAGMA EXCEPTION_INIT (no_access_to_record,            -20563);
    ex_no_access_to_record_type     constant int := -20564;  no_access_to_record_type      EXCEPTION;    PRAGMA EXCEPTION_INIT (no_access_to_record_type,       -20564);
    ex_not_a_member_of_group        constant int := -20565;  not_a_member_of_group         EXCEPTION;    PRAGMA EXCEPTION_INIT (not_a_member_of_group,          -20565);

    -- name errors
    ex_no_renaming_domains          constant int := -20573;  no_renaming_domains           EXCEPTION;    PRAGMA EXCEPTION_INIT (no_renaming_domains,            -20573);
    ex_no_moving_names              constant int := -20574;  no_moving_names               EXCEPTION;    PRAGMA EXCEPTION_INIT (no_moving_names,                -20574);
    ex_name_collision               constant int := -20575;  name_collision                EXCEPTION;    PRAGMA EXCEPTION_INIT (name_collision,                 -20575);
    ex_bad_name                     constant int := -20576;  bad_name                      EXCEPTION;    PRAGMA EXCEPTION_INIT (bad_name,                       -20576);
    ex_bad_MX_target                constant int := -20577;  bad_MX_target                 EXCEPTION;    PRAGMA EXCEPTION_INIT (bad_MX_target,                  -20577);
    ex_bad_MX_name                  constant int := -20578;  bad_MX_name                   EXCEPTION;    PRAGMA EXCEPTION_INIT (bad_MX_name,                    -20578);

    -- domain errors
    ex_cannot_use_domain            constant int := -20590;  cannot_use_domain             EXCEPTION;    PRAGMA EXCEPTION_INIT (cannot_use_domain,              -20590);
    ex_cannot_use_domain_as_name    constant int := -20591;  cannot_use_domain_as_name     EXCEPTION;    PRAGMA EXCEPTION_INIT (cannot_use_domain_as_name,      -20591);

    -- address space errors
    ex_overlapping_address_spaces   constant int := -20600;  overlapping_address_spaces    EXCEPTION;    PRAGMA EXCEPTION_INIT (overlapping_address_spaces,     -20600);
    ex_prefix_len_out_of_bounds     constant int := -20601;  prefix_len_out_of_bounds      EXCEPTION;    PRAGMA EXCEPTION_INIT (prefix_len_out_of_bounds,       -20601);
    ex_reserve_out_of_bounds        constant int := -20602;  reserve_out_of_bounds         EXCEPTION;    PRAGMA EXCEPTION_INIT (reserve_out_of_bounds,          -20602);

    type ref_cursor is ref cursor;

    existence_counter int;

end NetDB;
/

grant execute on NetDB to netdb_execute;
