-- NetDB_Node.sql --- contains functions related to nodes

EXEC Util.drop_type_if_exists ('node_type_result');

create or replace type node_type_row as object (type varchar(32));
/

create or replace type node_type_result as table of node_type_row;
/

create or replace package NetDB_Node as

    function list_types (p_node_id in int) return node_type_result pipelined;

    procedure OS ( p_action  in varchar,
                   p_op_uid  in int,
                   p_lock    in timestamp,
                   p_node_id in int     := null,
                   p_node    in varchar := null,
                   p_os      in varchar,
                   p_version in varchar );

    procedure admin ( p_action    in varchar,
                      p_op_uid    in int,
                      p_lock      in timestamp,
                      p_node_id   in int     := null,
                      p_node      in varchar := null,
                      p_ateam_id  in varchar := null,
                      p_ateam     in varchar := null,
                      p_person_id in int     := null,
                      p_regid     in varchar := null );

    procedure user ( p_action    in varchar,
                     p_op_uid    in int,
                     p_lock      in timestamp,
                     p_node_id   in int     := null,
                     p_node      in varchar := null,
                     p_person_id in int     := null,
                     p_regid     in varchar := null );

    procedure type ( p_action  in varchar,
                     p_op_uid  in int,
                     p_lock    in timestamp,
                     p_node_id in int     := null,
                     p_node    in varchar := null,
                     p_type    in varchar );

    procedure tag ( p_action  in varchar,
                    p_op_uid  in int,
                    p_lock    in timestamp,
                    p_node_id in int,
                    p_node    in varchar,
                    p_label   in varchar,
                    p_value   in varchar );

    function info ( p_node_id in int,
                    p_user_id in int := null ) return result_set pipelined;

    procedure locked_or_die ( p_op_uid  in int,
                              p_lock    in timestamp,
                              p_node_id in int );

    function verify_or_die ( p_node_id  in int     := null,
                             p_fullname in varchar := null ) return int;

    function handle (p_node_id in int) return varchar;

    function master ( p_action          in varchar,
                      p_op_uid          in int       := null,
                      p_lock            in timestamp := null,
                      p_node_id         in int       := null,
                      p_node            in varchar   := null,
                      p_org             in varchar   := null,
                      p_location        in varchar   := null,
                      p_room            in varchar   := null,
                      p_state           in varchar   := null,
                      p_make            in varchar   := null,
                      p_model           in varchar   := null,
                      p_comment         in varchar   := null,
                      p_expire_date     in timestamp := null,
                      p_dhcpservice     in varchar   := null,
                      p_ipadddressspace in IP.prefix := null ) return NetDB.ref_cursor;

end NetDB_Node;
/

grant execute on NetDB_Node to netdb_execute;
