-- NetDB_Util.sql --- utility functions specific to NetDB

create or replace package NetDB_Util as

    procedure sysmaint_check (col varchar, value varchar);
    procedure validate(string varchar, thing varchar := 'input string', null_error varchar := null);

    procedure locked_or_die ( p_op_uid      in int,
                              p_lock        in timestamp,
                              p_record_id   in int,
                              p_record_type in varchar );

    function handle ( p_record_id   in int,
                      p_record_type in varchar,
                      user_readable in boolean := false) return varchar;

    function verify_or_die ( p_record_id    in int     := null,
                             p_handle       in varchar := null,
                             p_record_type  in varchar,
                             p_name_type    in varchar := null,
                             check_reserved in boolean := null ) return int;

    function fullname ( p_name   in varchar,
                        p_domain in varchar ) return varchar;

    function get_null_generic_cursor return NetDB.ref_cursor;

    procedure throw_no_record_id_handle;
    procedure throw_unknown_record_type (p_type in varchar);
    procedure throw_unknown_action (p_action in varchar);
    procedure throw_single_record_expected (p_record_type in varchar);

    procedure throw_still_in_use ( p_record_type   in varchar,
                                   p_record_handle in varchar );

    procedure throw_record_does_not_exist ( p_record_type   in varchar,
                                            p_record_handle in varchar );

    procedure throw_record_already_exists ( p_record_type   in varchar,
                                            p_record_handle in varchar );

    procedure throw_record_not_locked ( p_record_type   in varchar,
                                        p_record_handle in varchar,
                                        p_user_handle   in varchar );

    procedure throw_locked_by_other ( p_record_type   in varchar,
                                      p_record_handle in varchar,
                                      p_user_handle   in varchar );

    procedure throw_field_too_long (oracle_error_message in varchar);

    function strip_ora_code (error in varchar) return varchar;

end NetDB_Util;
/

grant execute on NetDB_Util to netdb_execute;
