-- DNS.sql --- DNS package unit tests

declare

    type strings is varray(100) of varchar(255);
    names   strings;
    domains strings;

    i       int;            -- scratch integers
    testnum int   := 1;     -- test counter

begin

    -- check_label()
    names := strings('foobar', 'foo-bar', '1as', '1-9a-z',
                     'lllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllll');
    for i in names.FIRST .. names.LAST loop
        dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
        if DNS.check_label(names(i))
            then dbms_output.put_line('OK');
            else dbms_output.put_line('Failed check_label: ' || names(i)); end if;
    end loop;

    names := strings('foobar*', '-foo-bar', '1.as', '1_-9a-z', '12345',
                     'LLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLL');
    for i in names.FIRST .. names.LAST loop
        dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
        if not DNS.check_label(names(i))
            then dbms_output.put_line('OK');
            else dbms_output.put_line('Failed check_label: ' || names(i)); end if;
    end loop;

    -- check_name()
    names := strings('foobar', 'foo-bar.stanford.edu', '1as.123c', '1-9a-z.foo.var.baz',
                     'lllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllll.com');
    for i in names.FIRST .. names.LAST loop
        dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
        if DNS.check_name(names(i))
            then dbms_output.put_line('OK');
            else dbms_output.put_line('Failed check_name: ' || names(i)); end if;
    end loop;

    names := strings('.', '..', 'stanford..edu', '.stanford', '1aa.as.', '1_-9a-z.stanford.edu', 'www.edu%.ack',
                     'LLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLL.com');
    for i in names.FIRST .. names.LAST loop
        dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
        if not DNS.check_name(names(i))
            then dbms_output.put_line('OK');
            else dbms_output.put_line('Failed check_name: ' || names(i)); end if;
    end loop;

    -- labels()
    names := strings('', '.', 'stanford', 'stanford.edu.', 'www.edu.ack');
    for i in names.FIRST .. names.LAST loop
        dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
        if DNS.labels(names(i)) = i-2
            then dbms_output.put_line('OK');
            else dbms_output.put_line('Failed labels: ' || to_char(i-2) || '/' || names(i)); end if;
    end loop;

    -- name() and domain() spec:  input           name  domain
    --                            ==============  ====  ==============
    --                            foo.bar.com     foo   bar.com
    --                            foo.bar.com.    foo   bar.com
    --
    --                            foo.com         foo   com
    --                            foo.com.        foo   com
    --
    --                            com             com   .
    --                            com.            com   .
    --
    --                            null            null  null
    --                            .               .     .

    -- name()
    names := strings('foo.bar.com', 'foo.bar.com.', 'foo.com', 'foo.com.', 'foo', 'foo.');
    for i in names.FIRST .. names.LAST loop
        dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
        if DNS.name(names(i)) = 'foo'
            then dbms_output.put_line('OK');
            else dbms_output.put_line('Failed name: ' || to_char(i-2) || '/' || names(i)); end if;
    end loop;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if DNS.name('.') = '.'
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed name: ' || to_char(i-2) || '/.'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if DNS.name('') is null
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed name: ' || to_char(i-2) || '/.'); end if;

    -- domain()
    names := strings('foo.bar.com', 'foo.bar.com.', 'foo.com', 'foo.com.', 'foo', 'foo.', '.');
    domains := strings(  'bar.com',     'bar.com',      'com',      'com', '.',   '.',    '.');
    for i in names.FIRST .. names.LAST loop
        dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
        if DNS.domain(names(i)) = domains(i)
            then dbms_output.put_line('OK');
            else dbms_output.put_line('Failed domain: ' || to_char(i-2) || '/' || names(i)); end if;
    end loop;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if DNS.domain('') is null
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed domain: ' || to_char(i-2) || '/.'); end if;


end;
/
