-- IP.sql --- IP package unit tests

declare

    anint   int;            -- scratch integer
    testnum int := 1;       -- test counter
    address varchar(39);    -- scratch IP address variable
    prefix  varchar(43);    -- scratch prefix varchar

begin

    -- family()
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.family('171.64.12.20') = 4
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed family of 4'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.family('dead:beef:cafe::0001/128') = 6
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed family of 6'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    begin
        select IP.family(null) into anint from dual;
        dbms_output.put_line('Failed checking family(NULL)');
    exception
        when NetDB.invalid_IP_address then dbms_output.put_line('OK');
        when others                   then dbms_output.put_line('Failed checking family(NULL)');
    end;

    -- normalize()
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.normalize(NULL) is null
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed normalizing NULL'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.normalize('171.64') = lpad('171.064.000.000',39)
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed normalizing abbreviated IP address'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.normalize('171.64.20.42') = lpad('171.064.020.042',39)
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed normalizing IP address'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.normalize('171.64/14') = lpad('171.064.000.000/014',43)
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed normalizing abbreviated prefix'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.normalize('171.64.20.42/31') = lpad('171.064.020.042/031',43)
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed normalizing prefix'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    begin
        select IP.normalize('288.12.16.88') into address from dual;
        dbms_output.put_line('Failed checking invalid IP address');
    exception
        when NetDB.invalid_IP_address then dbms_output.put_line('OK');
        when others                   then dbms_output.put_line('Failed checking invalid IP address');
    end;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    begin
        select IP.normalize('171.66.24.1/33') into address from dual;
        dbms_output.put_line('Failed checking invalid prefix length');
    exception
        when NetDB.invalid_IP_address then dbms_output.put_line('OK');
        when others                   then dbms_output.put_line('Failed checking invalid prefix length');
    end;

    -- host()
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.host(NULL) is null
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed familiarizing NULL as IP address'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.host('171.64.020.42') = '171.64.20.42'
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed familiarizing IP address as IP address'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.host('171.64.20.42/31') = '171.64.20.42'
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed familiarizing prefix as IP address'); end if;

    -- text()
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.text(NULL) is null
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed familiarizing NULL as prefix'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.text('171.64.020.42') = '171.64.20.42/32'
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed familiarizing IP address as prefix'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.text('171.64.20.42/31') = '171.64.20.42/31'
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed familiarizing prefix as prefix'); end if;

    -- network()
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.network(NULL) is null
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed getting network from NULL'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.network('171.64.20.10/24') = '171.64.20.0/24'
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed getting network from prefix'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.network('171.64.20.42') = '171.64.20.42/32'
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed getting network from IP address'); end if;

    -- broadcast()
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.broadcast(NULL) is null
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed getting broadcast from NULL'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.broadcast('171.64.20.10/24') = '171.64.20.255'
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed getting broadcast from prefix'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.broadcast('171.64.20.42') = '171.64.20.42'
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed getting broadcast from IP address'); end if;

    -- netmask()
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.netmask(NULL) is null
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed getting netmask from NULL'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.netmask('171.64.20.10/16') = '255.255.0.0'
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed getting netmask from prefix'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.netmask('171.64.20.42') = '255.255.255.255'
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed getting netmask from IP address'); end if;

    -- masklen()
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.masklen('171.64.20.10/24') = 24
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed getting mask length from prefix'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.masklen('171.64.20.42') = 32
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed getting mask length from IP address'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    begin
        select IP.masklen(null) into anint from dual;
        dbms_output.put_line('Failed checking masklen(NULL)');
    exception
        when NetDB.invalid_IP_address then dbms_output.put_line('OK');
        when others                   then dbms_output.put_line('Failed checking masklen(NULL)');
    end;

    -- plus()
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.plus('171.64.20.10',-12) = '171.64.19.254'
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed subtacting 12 from 171.64.20.10'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.plus('171.64.20.42',1) = '171.64.20.43'
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed adding 1 to 171.64.20.42'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.plus('255.255.255.245',10) = '255.255.255.255'
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed adding 10 to 255.255.255.245'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.plus('0.0.0.10',-10) = '0.0.0.0'
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed subtacting 10 from 0.0.0.10'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    begin
        select ip.plus('0.0.0.10',-11) into address from dual;
        dbms_output.put_line('Failed checking low bad IP arithmetic');
    exception
        when NetDB.bad_ip_arithmetic then dbms_output.put_line('OK');
        when others                  then dbms_output.put_line('Failed checking bad IP arithmetic, low');
    end;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    begin
        select ip.plus('255.255.255.245',11) into address from dual;
        dbms_output.put_line('Failed checking high bad IP arithmetic');
    exception
        when NetDB.bad_ip_arithmetic then dbms_output.put_line('OK');
        when others                  then dbms_output.put_line('Failed checking bad IP arithmetic, high');
    end;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    begin
        select ip.plus('171.64.20.42',1111111111111) into address from dual;
        dbms_output.put_line('Failed checking number precision too large');
    exception
        when others then
            if SQLCODE = -6502 then
                dbms_output.put_line('OK');
            else
                dbms_output.put_line('Failed checking number precision too large: ' || SQLCODE);
            end if;
    end;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    begin
        select IP.plus(null,1) into address from dual;
        dbms_output.put_line('Failed checking plus(NULL,1)');
    exception
        when NetDB.invalid_IP_address then dbms_output.put_line('OK');
        when others                   then dbms_output.put_line('Failed checking plus(NULL,1)');
    end;

    -- difference()
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.difference('171.64.20.10','171.64.20.10') = 0
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed subtacting 171.64.20.10 from itself'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.difference('171.64.20.42','171.64.20.40') = 2
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed subtacting 171.64.20.40 from 171.64.20.42'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    if IP.difference('171.64.20.0','171.64.20.42') = -42
        then dbms_output.put_line('OK');
        else dbms_output.put_line('Failed subtacting 171.64.20.42 from 171.64.20.0'); end if;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    begin
        select IP.difference(null,'171.64.20.42') into address from dual;
        dbms_output.put_line('Failed checking difference(NULL,171.64.20.42)');
    exception
        when NetDB.invalid_IP_address then dbms_output.put_line('OK');
        when others                   then dbms_output.put_line('Failed checking difference(NULL,171.64.20.42)');
    end;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    begin
        select IP.difference('171.64.20.42',null) into address from dual;
        dbms_output.put_line('Failed checking difference(171.64.20.42,NULL)');
    exception
        when NetDB.invalid_IP_address then dbms_output.put_line('OK');
        when others                   then dbms_output.put_line('Failed checking difference(171.64.20.42,NULL)');
    end;
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    begin
        select IP.difference(null,null) into address from dual;
        dbms_output.put_line('Failed checking difference(NULL,NULL)');
    exception
        when NetDB.invalid_IP_address then dbms_output.put_line('OK');
        when others                   then dbms_output.put_line('Failed checking difference(NULL,NULL)');
    end;

    -- IPv6 not implemented error
    dbms_output.put('Test ' || testnum || ' ... '); testnum := testnum + 1;
    begin
        select ip.normalize('03a7::a1b0') into address from dual;
        dbms_output.put_line('Failed checking for IPv6 not implemented');
    exception
        when NetDB.IPv6_not_implemented then
            dbms_output.put_line('OK');
        when others then
            dbms_output.put_line('Failed checking for IPv6 not implemented' || SQLCODE);
    end;

end;
/
