set serveroutput on size 100000;
savepoint start_access_tests;

declare
    test_number int          := 1;
    err_msg     varchar(256) := null;
    v_lock      timestamp    := null;
    v_count     int          := 0;

    test_cursor NetDB.ref_cursor;
    result      result_row := result_row (null, null, null, null, null, null, null);

    test_id     int := 0;
    test_handle AccessGroup.label%TYPE;
begin
    test_cursor := NetDB_User.master ('lock', NetDB_Test.user_0.id, null, NetDB_Test.user_1.id);
    result      := NetDB_Test.get_generic_row (test_cursor);
    v_lock      := result.value_timestamp;
    NetDB_Test.empty_cursor (test_cursor);

    for r in (select * from AccessGroup)
    loop
        begin
            NetDB_Test.output_and_increment_test (test_number);
            err_msg := 'Remove record type access for the user.';

            NetDB_User.users_access ('remove', NetDB_Test.user_0.id, v_lock, NetDB_Test.user_1.id, r.id, r.label);

            select count (*) into v_count from Users_AccessGroup
             where userid = NetDB_Test.user_1.id and accessgroup = r.id;

            if (v_count = 0) then
                NetDB_Test.pass ();
            else
                NetDB_Test.failed (err_msg);
            end if;
        exception
            when others then
                NetDB_Test.failed (err_msg, sqlerrm);
        end;

        begin
            NetDB_Test.output_and_increment_test (test_number);
            err_msg := 'Verify no record type access.';

            NetDB_Access.has_access_or_die (NetDB_Test.user_1.id, r.label, null, false);
            NetDB_Test.failed (err_msg);
        exception
            when NetDB.no_access_to_record_type then
                NetDB_Test.pass ();
            when others then
                NetDB_Test.failed (err_msg, sqlerrm);
        end;

        begin
            NetDB_Test.output_and_increment_test (test_number);
            err_msg := 'Add record type access for the user.';

            NetDB_User.users_access ('add', NetDB_Test.user_0.id, v_lock, NetDB_Test.user_1.id, r.id, r.label);

            select count (*) into v_count from Users_AccessGroup
             where userid = NetDB_Test.user_1.id and accessgroup = r.id;

            if (v_count > 0) then
                NetDB_Test.pass ();
            else
                NetDB_Test.failed (err_msg);
            end if;
        exception
            when others then
                NetDB_Test.failed (err_msg, sqlerrm);
        end;

        begin
            NetDB_Test.output_and_increment_test (test_number);
            err_msg := 'Verify record type access.';

            NetDB_Access.has_access_or_die (NetDB_Test.user_1.id, r.label, null, false);
            NetDB_Test.pass ();
        exception
            when others then
                NetDB_Test.failed (err_msg, sqlerrm);
        end;

        begin
            NetDB_Test.output_and_increment_test (test_number);
            err_msg := 'Verify an access type with the ID only.';

            test_id := NetDB_Access.verify_or_die (r.id);

            if (test_id = r.id) then
                NetDB_Test.pass ();
            else
                NetDB_Test.failed (err_msg);
            end if;
        exception
            when others then
                NetDB_Test.failed (err_msg, sqlerrm);
        end;

        begin
            NetDB_Test.output_and_increment_test (test_number);
            err_msg := 'Verify an access type with the handle only.';

            test_id := NetDB_Access.verify_or_die (null, r.label);

            if (test_id = r.id) then
                NetDB_Test.pass ();
            else
                NetDB_Test.failed (err_msg);
            end if;
        exception
            when others then
                NetDB_Test.failed (err_msg, sqlerrm);
        end;

        begin
            NetDB_Test.output_and_increment_test (test_number);
            err_msg := 'Verify an access type with the ID and handle.';

            test_id := NetDB_Access.verify_or_die (r.id, r.label);

            if (test_id = r.id) then
                NetDB_Test.pass ();
            else
                NetDB_Test.failed (err_msg);
            end if;
        exception
            when others then
                NetDB_Test.failed (err_msg, sqlerrm);
        end;

        begin
            NetDB_Test.output_and_increment_test (test_number);
            err_msg := 'Verify an access type with a bad ID and handle combination.';

            test_id := NetDB_Access.verify_or_die (r.id + 1, r.label);

            NetDB_Test.failed (err_msg);
        exception
            when NetDB.record_not_found then
                NetDB_Test.pass ();
            when others then
                NetDB_Test.failed (err_msg, sqlerrm);
        end;

        begin
            NetDB_Test.output_and_increment_test (test_number);
            err_msg := 'Verify a handle for an id.';

            test_handle := NetDB_Access.handle (r.id);

            if (test_handle = r.label) then
                NetDB_Test.pass ();
            else
                NetDB_Test.failed (err_msg);
            end if;
        exception
            when others then
                NetDB_Test.failed (err_msg, sqlerrm);
        end;
    end loop;

    test_cursor := NetDB_User.master ('unlock', NetDB_Test.user_0.id, v_lock, NetDB_Test.user_1.id);
    NetDB_Test.empty_cursor (test_cursor);
exception
    when others then
        dbms_output.put_line ('ERROR: Unhandled exception "' || sqlcode || '" with message "' || sqlerrm || '"');
        dbms_output.put_line (err_msg);
end;
/

rollback to start_access_tests;
