set serveroutput on size 100000;
savepoint start_dhcp_option_tests;

declare
    test_number int          := 1;
    err_msg     varchar(256) := null;
    v_count     int          := 0;

    test_cursor NetDB.ref_cursor;
    result      result_row := result_row (null, null, null, null, null, null, null);

    -- Values from the DB.
    db_lock timestamp(3);

    -- Options for insert and update.
    option_label     varchar(64) := 'test-1234509876-option';
    option_label_new varchar(64) := 'test-1234509876-option-new';
    option_datatype  varchar(16) := 'string';
    option_id        int         := 0;
    test_id          int;

    option_row  Network_DHCPoptions%ROWTYPE;
    verified_id int;

    unrecoverable_error EXCEPTION;
begin
    select count (*) into v_count from DHCPoption where label = option_label or label = option_label_new;

    if (v_count > 0) then
        err_msg := 'The test option "' || option_label || '" or "' || option_label_new ||
                   '" already exists. If it is not a valid option, remove the entry and run ' ||
                   'the tests again. If it is valid, the tests must be modified.';
        raise unrecoverable_error;
    end if;

    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'dhcpoption');
    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'network');

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Do anything with a user that doesn''t have access to DHCP options.';

        test_cursor := NetDB_DHCPoption.master ('insert', NetDB_Test.user_2.id, null, option_label);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insertion of an new option without a datatype.';

        test_cursor := NetDB_DHCPoption.master ('insert', NetDB_Test.user_1.id, null, option_label);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.bad_parameter then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insertion of an new option without a label.';

        test_cursor := NetDB_DHCPoption.master ('insert', NetDB_Test.user_1.id, null, null, null, option_datatype);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.bad_parameter then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insertion of an new option.';

        test_cursor := NetDB_DHCPoption.master ('insert', NetDB_Test.user_1.id, null, option_label, null, option_datatype);
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_id      := result.value_string;
        NetDB_Test.empty_cursor (test_cursor);

        verified_id := NetDB_DHCPoption.verify_or_die (test_id, option_label);

        if (verified_id = test_id) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Add an option to a network that cannot be added to networks.';

        test_cursor := NetDB_Network.master ('lock', NetDB_Test.user_1.id, null, NetDB_Test.network_1.id);
        result      := NetDB_Test.get_generic_row (test_cursor);
        db_lock     := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        option_id := NetDB_DHCPoption.verify_or_die (null, option_label);
        NetDB_DHCPoption.add (NetDB_Test.user_1.id, db_lock, NetDB_Test.network_1.id, 'network', option_id, null, 'value');

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.record_not_found then
            NetDB_Test.pass ();

            test_cursor := NetDB_Network.master ('unlock', NetDB_Test.user_1.id, db_lock, NetDB_Test.network_1.id);
            NetDB_Test.empty_cursor (test_cursor);
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Add an option to a network.';

        option_id := NetDB_DHCPoption.verify_or_die (null, option_label);
        NetDB_DHCPoption.application_add (NetDB_Test.user_1.id, 'network', option_id);

        test_cursor := NetDB_Network.master ('lock', NetDB_Test.user_1.id, null, NetDB_Test.network_1.id);
        result      := NetDB_Test.get_generic_row (test_cursor);
        db_lock     := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_DHCPoption.add (NetDB_Test.user_1.id, db_lock, NetDB_Test.network_1.id, 'network', option_id, null, 'value');

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Add an option already on a network.';

        test_cursor := NetDB_Network.master ('lock', NetDB_Test.user_1.id, null, NetDB_Test.network_1.id);
        result      := NetDB_Test.get_generic_row (test_cursor);
        db_lock     := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_DHCPoption.add (NetDB_Test.user_1.id, db_lock, NetDB_Test.network_1.id, 'network', option_id, null, 'value');

        test_cursor := NetDB_Network.master ('unlock', NetDB_Test.user_1.id, db_lock, NetDB_Test.network_1.id);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete an option still in use.';

        option_id   := NetDB_DHCPoption.verify_or_die (null, option_label);
        test_cursor := NetDB_DHCPoption.master ('delete', NetDB_Test.user_1.id, option_id);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.record_in_use then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Remove an option from a network.';

        test_cursor := NetDB_Network.master ('lock', NetDB_Test.user_1.id, null, NetDB_Test.network_1.id);
        result      := NetDB_Test.get_generic_row (test_cursor);
        db_lock     := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        option_id := NetDB_DHCPoption.verify_or_die (null, option_label);
        NetDB_DHCPoption.remove (NetDB_Test.user_1.id, db_lock, NetDB_Test.network_1.id, 'network', option_id, null);

        test_cursor := NetDB_Network.master ('unlock', NetDB_Test.user_1.id, db_lock, NetDB_Test.network_1.id);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Add an option to an address space.';

        NetDB_DHCPoption.application_add (NetDB_Test.user_1.id, 'addressspace', null, option_label);

        test_cursor := NetDB_Network.master ('lock', NetDB_Test.user_1.id, null, NetDB_Test.network_1.id);
        result      := NetDB_Test.get_generic_row (test_cursor);
        db_lock     := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_DHCPoption.add (NetDB_Test.user_1.id, db_lock, NetDB_Test.net_1_space_1.id, 'addressspace', null, option_label, 'value');

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Remove an option from an address space.';

        test_cursor := NetDB_Network.master ('lock', NetDB_Test.user_1.id, null, NetDB_Test.network_1.id);
        result      := NetDB_Test.get_generic_row (test_cursor);
        db_lock     := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_DHCPoption.remove (NetDB_Test.user_1.id, db_lock, NetDB_Test.net_1_space_1.id, 'addressspace', null, option_label);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete an option.';

        option_id   := NetDB_DHCPoption.verify_or_die (null, option_label);
        test_cursor := NetDB_DHCPoption.master ('delete', NetDB_Test.user_1.id, option_id);
        NetDB_Test.empty_cursor (test_cursor);

        verified_id := NetDB_DHCPoption.verify_or_die (option_id);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.record_not_found then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

exception
    when unrecoverable_error then
        dbms_output.put_line ('ERROR: All remaining tests aborted. An unrecoverable error occured: ' || err_msg);
    when others then
        dbms_output.put_line ('ERROR: Unhandled exception "' || sqlcode || '" with message "' || sqlerrm || '"');
        dbms_output.put_line (err_msg);
end;
/

rollback to start_dhcp_option_tests;
