set serveroutput on size 100000;

savepoint start_ip_tests;

declare
    test_number int          := 1;
    err_msg     varchar(256) := null;
    v_lock      timestamp    := null;
    v_count     int          := 0;

    v_expected  varchar(256) := null;
    v_name      varchar(256) := null;

    existing_addr IPaddress.ipaddress%TYPE := '191.64.132.253';
    valid_addr    IPaddress.ipaddress%TYPE := '191.64.132.254';
    valid_mid_res IPaddress.ipaddress%TYPE := '191.64.132.255';
    valid_contig  IPaddress.ipaddress%TYPE := '191.64.20.17';
    contig_count  int := 21;

    ias_id  int := NetDB_IPaddressSpace.containing (existing_addr);
    ias_row IPaddressspace%ROWTYPE;

    test_cursor NetDB.ref_cursor;
    result      result_row := result_row (null, null, null, null, null, null, null);

    ip1 IPaddress.ipaddress%TYPE := null;
    ip2 IPaddress.ipaddress%TYPE := null;
    ip3 IPaddress.ipaddress%TYPE := null;
    ip4 IPaddress.ipaddress%TYPE := null;
begin
    select * into ias_row from IPaddressSpace where id = ias_id;

    NetDB_Test.test_info ('address', test_number);

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Attempt to reserve addresses in an address space which the user does not have access to.';

        test_cursor := NetDB_IPaddress.reserve (NetDB_Test.user_1.id, valid_addr, 3);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Reserve three addresses.';

        test_cursor := NetDB_IPaddress.reserve (NetDB_Test.user_2.id, valid_addr, 3);
        result := NetDB_Test.get_generic_row (test_cursor); -- useless row

        result := NetDB_Test.get_generic_row (test_cursor);
        ip1    := result.key;

        result := NetDB_Test.get_generic_row (test_cursor);
        ip2    := result.key;

        result := NetDB_Test.get_generic_row (test_cursor);
        ip3    := result.key;

        NetDB_Test.empty_cursor (test_cursor);

        if ( IP.normalize (ip1) = IP.normalize ('191.64.132.254') and
             IP.normalize (ip2) = IP.normalize ('191.64.133.13')  and
             IP.normalize (ip3) = IP.normalize ('191.64.133.14') )
        then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Reserve three more addresses.';

        test_cursor := NetDB_IPaddress.reserve (NetDB_Test.user_2.id, valid_addr, 3);
        result := NetDB_Test.get_generic_row (test_cursor); -- useless row

        result := NetDB_Test.get_generic_row (test_cursor);
        ip1    := result.key;

        result := NetDB_Test.get_generic_row (test_cursor);
        ip2    := result.key;

        result := NetDB_Test.get_generic_row (test_cursor);
        ip3    := result.key;

        NetDB_Test.empty_cursor (test_cursor);

        if ( IP.normalize (ip1) = IP.normalize ('191.64.133.15') and
             IP.normalize (ip2) = IP.normalize ('191.64.133.16') and
             IP.normalize (ip3) = IP.normalize ('191.64.133.17') )
        then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Refresh the reservation of an address.';

        test_cursor := NetDB_IPaddress.reserve (NetDB_Test.user_2.id, ip1, 1);
        result := NetDB_Test.get_generic_row (test_cursor); -- useless row

        result := NetDB_Test.get_generic_row (test_cursor);
        ip1    := result.key;

        NetDB_Test.empty_cursor (test_cursor);

        if (IP.normalize (ip1) = IP.normalize ('191.64.133.15')) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Refresh the same address again.';

        test_cursor := NetDB_IPaddress.reserve (NetDB_Test.user_2.id, ip1, 1);
        result := NetDB_Test.get_generic_row (test_cursor); -- useless row

        result := NetDB_Test.get_generic_row (test_cursor);
        ip1    := result.key;

        if (IP.normalize (ip1) = IP.normalize ('191.64.133.15')) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    rollback to start_ip_tests;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Reserve some addresses with an exact starting location.';

        test_cursor := NetDB_IPaddress.reserve (NetDB_Test.user_2.id, valid_addr, 4, 1);
        result := NetDB_Test.get_generic_row (test_cursor); -- useless row

        result := NetDB_Test.get_generic_row (test_cursor);
        ip1    := result.key;

        result := NetDB_Test.get_generic_row (test_cursor);
        ip2    := result.key;

        result := NetDB_Test.get_generic_row (test_cursor);
        ip3    := result.key;

        result := NetDB_Test.get_generic_row (test_cursor);
        ip4    := result.key;

        NetDB_Test.empty_cursor (test_cursor);

        if ( IP.normalize (ip1) = IP.normalize ('191.64.132.254') and
             IP.normalize (ip2) = IP.normalize ('191.64.133.13')  and
             IP.normalize (ip3) = IP.normalize ('191.64.133.14')  and
             IP.normalize (ip4) = IP.normalize ('191.64.133.15') )
        then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Attempt to reserve some addresses with a bad, exact start.';

        test_cursor := NetDB_IPaddress.reserve (NetDB_Test.user_2.id, existing_addr, 4, 1);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.cannot_reserve_address then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    rollback to start_ip_tests;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Reserve some contiguous addresses.';

        test_cursor := NetDB_IPaddress.reserve (NetDB_Test.user_1.id, valid_contig, contig_count, 0, 1);
        NetDB_Test.empty_cursor (test_cursor);

        if (NetDB_Test.rows_skipped = contig_count + 1) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    rollback to start_ip_tests;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Attempt to reserve too many contiguous addresses.';

        test_cursor := NetDB_IPaddress.reserve (NetDB_Test.user_1.id, valid_contig, contig_count + 1, 0, 1);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.cannot_reserve_address then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Attempt to reserve some addresses starting in a midres.';

        test_cursor := NetDB_IPaddress.reserve (NetDB_Test.user_2.id, valid_mid_res, 4, 1);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.cannot_reserve_address then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    rollback to start_ip_tests;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Reserve some addresses starting in a midres, with a god user.';

        test_cursor := NetDB_IPaddress.reserve (NetDB_Test.user_0.id, valid_mid_res, 4);
        result := NetDB_Test.get_generic_row (test_cursor); -- useless row

        result := NetDB_Test.get_generic_row (test_cursor);
        ip1    := result.key;

        result := NetDB_Test.get_generic_row (test_cursor);
        ip2    := result.key;

        result := NetDB_Test.get_generic_row (test_cursor);
        ip3    := result.key;

        result := NetDB_Test.get_generic_row (test_cursor);
        ip4    := result.key;

        NetDB_Test.empty_cursor (test_cursor);

        if ( IP.normalize (ip1) = IP.normalize ('191.64.132.255') and
             IP.normalize (ip2) = IP.normalize ('191.64.133.0')   and
             IP.normalize (ip3) = IP.normalize ('191.64.133.13')  and
             IP.normalize (ip4) = IP.normalize ('191.64.133.14') )
        then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;

    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    rollback to start_ip_tests;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Attempt to exactly reserve the base address.';

        test_cursor := NetDB_IPaddress.reserve (NetDB_Test.user_2.id, ias_row.address, 1, 1);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.cannot_reserve_address then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Attempt to reserve the base address.';

        test_cursor := NetDB_IPaddress.reserve (NetDB_Test.user_2.id, ias_row.address, 1);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.pass ();
    exception
        when NetDB.cannot_reserve_address then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Attempt to reserve the broadcast.';

        test_cursor := NetDB_IPaddress.reserve (NetDB_Test.user_2.id, ias_row.broadcast, 1);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.cannot_reserve_address then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    rollback to start_ip_tests;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Create a new IP address.';

        test_cursor := NetDB_Node.master ('lock', NetDB_Test.user_2.id, null, NetDB_Test.node_4.id);
        result      := NetDB_Test.get_generic_row (test_cursor);
        v_lock      := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        test_cursor := NetDB_IPaddress.reserve (NetDB_Test.user_2.id, valid_addr, 1);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_IPaddress.accept (NetDB_Test.user_2.id, valid_addr, 1, 0, NetDB_Test.node_4.id, v_lock, 'node');

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update an IP address.';

        test_cursor := NetDB_IPaddress.master ('update', NetDB_Test.user_2.id, v_lock, null, valid_addr, 0);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'closest_name ("191.64.132.252")'; -- address
        v_expected := 'DNbf4084fc.Spacely-Sprockets.COM';

        v_name := NetDB_IPaddress.closest_name ('191.64.132.252');

        if (v_name = v_expected) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg, '  expected "' || v_expected || '"     got "' || v_name || '".');
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'closest_name ("191.64.20.1")'; -- interface
        v_expected := 'astro-gw-1-1.Cogswell-Cogs.COM';

        v_name := NetDB_IPaddress.closest_name ('191.64.20.1');

        if (v_name = v_expected) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg, '  expected "' || v_expected || '"     got "' || v_name || '".');
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'closest_name ("191.64.20.39")'; -- node
        v_expected := NetDB_Test.node_3_name;

        v_name := NetDB_IPaddress.closest_name ('191.64.20.39');

        if (v_name = v_expected) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg, '  expected "' || v_expected || '"     got "' || v_name || '".');
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;
exception
    when others then
        dbms_output.put_line ('ERROR: Unhandled exception "' || sqlcode || '" with message "' || sqlerrm || '"');
end;
/

rollback to start_ip_tests;
