set serveroutput on size 100000;
savepoint start_log_tests;

declare
    test_number int          := 1;
    err_msg     varchar(256) := null;
    v_lock      timestamp    := null;
    v_reserved  timestamp    := null;
    v_count     int          := 0;

    person_id int;
    name_id   int;
    rec_id    int;

    loc            Location.name%TYPE;
    room           Node.room%TYPE           := 'the room';
    v_state        State.state%TYPE;
    v_make         Model.make%TYPE;
    v_model        Model.modelname%TYPE;
    v_os           OS.os%TYPE;
    email          AdminTeam.email%TYPE     := 'email@address.com';
    phone          AdminTeam.phone%TYPE     := '555-555-5555';
    person_name    Person.name%TYPE         := 'Persons Name';
    last_name      Person.lastname%TYPE     := 'name';
    v_regid        Person.person_regid%TYPE := 'ThisPassesAsARegID';
    v_netid        Person.netid%TYPE        := 'someUniqueNetID';
    a_comment      AdminTeam.comments%TYPE  := 'a comment';
    a_new_comment  AdminTeam.comments%TYPE  := 'a new comment';

    valid_addr    IPaddress.ipaddress%TYPE  := '191.64.132.254';
    hw_addr       Interface.hw_address%TYPE := 'hwaddress';
    intfc_type    Interface.type%TYPE;
    test_intfc_id int;

    ateam_name      AdminTeam.name%TYPE := 'test-log-ateam-name';
    ateam_name_new  AdminTeam.name%TYPE := 'new-test-log-ateam-name';
    domain_name     Name.name%TYPE      := 'test-log-domain-name';
    domain_name_new Name.name%TYPE      := 'TEST-log-DOMAIN-name';
    owner_name      Owner.label%TYPE    := 'test-log-owner-name';
    owner_name_new  Owner.label%TYPE    := 'new-test-log-owner-name';
    network_name    Name.name%TYPE      := 'test-log-network-name';
    network_handle  Name.fullname%TYPE;
    net_name_rec    NAME%ROWTYPE;
    node_name       Name.name%TYPE      := 'test-log-node-name';
    node_handle     Name.fullname%TYPE;
    user_handle     Users.netid%TYPE;

    test_start timestamp;

    test_cursor NetDB.ref_cursor;
    result      result_row := result_row (null, null, null, null, null, null, null);

    unrecoverable_error EXCEPTION;

    procedure fill_id_lock ( the_cursor  in NetDB.ref_cursor,
                             p_id       out int,
                             p_lock     out timestamp ) is
    begin
        result := NetDB_Test.get_generic_row (the_cursor);
        p_id   := result.value_string;

        result := NetDB_Test.get_generic_row (the_cursor);
        p_lock := result.value_timestamp;

        NetDB_Test.empty_cursor (the_cursor);
    end fill_id_lock;
begin
    select type into intfc_type from InterfaceType where rownum = 1;

    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'AdminTeam');
    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'Domain');
    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'Network');
    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'Node');
    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'Owner');
    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'User');

    NetDB_Test.add_to_group (NetDB_Test.user_1.id, NetDB_Test.group_2.id);

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'Insert an Admin Team.';
        test_start := systimestamp;

        test_cursor := NetDB_AdminTeam.master ('insert', NetDB_Test.user_1.id, null, null, ateam_name, null, email, phone, a_comment);
        fill_id_lock (test_cursor, rec_id, v_lock);

        NetDB_Owner.add (NetDB_Test.user_1.id, v_lock, NetDB_Test.group_1.id, null, rec_id, 'AdminTeam');
        NetDB_Owner.add (NetDB_Test.user_1.id, v_lock, NetDB_Test.group_2.id, null, rec_id, 'AdminTeam');

        test_cursor := NetDB_AdminTeam.master ('verify', NetDB_Test.user_1.id, null, rec_id);
        NetDB_Test.empty_cursor (test_cursor);

        select count (*) into v_count
          from Log
          join LogName on Log.id = LogName.log_id
         where person         = NetDB_Test.user_1.id and
               lower (action) = 'insert'             and
               recordtype     = 'adminteam'          and
               record_id      = rec_id               and
               name           = ateam_name           and
               logdate        > test_start;

        if (v_count = 1) then NetDB_Test.pass ();
        else                  NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'Update an Admin Team.';
        test_start := systimestamp;

        NetDB_Test.grant_access (NetDB_Test.user_1.id, 'AdminTeam');

        test_cursor := NetDB_AdminTeam.master ('update', NetDB_Test.user_1.id, v_lock, rec_id, null, ateam_name_new, email, phone, a_comment);
        NetDB_Test.empty_cursor (test_cursor);

        test_cursor := NetDB_AdminTeam.master ('verify', NetDB_Test.user_1.id, null, rec_id);
        NetDB_Test.empty_cursor (test_cursor);

        select count (*) into v_count
          from Log
          join LogName on Log.id = LogName.log_id
         where person         = NetDB_Test.user_1.id and
               lower (action) = 'update'             and
               recordtype     = 'adminteam'          and
               record_id      = rec_id               and
               name           = ateam_name_new       and
               logdate        > test_start;

        if (v_count = 1) then NetDB_Test.pass ();
        else                  NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'Delete an Admin Team.';
        test_start := systimestamp;

        NetDB_Test.grant_access (NetDB_Test.user_1.id, 'AdminTeam');

        test_cursor := NetDB_AdminTeam.master ('delete', NetDB_Test.user_1.id, v_lock, rec_id);
        NetDB_Test.empty_cursor (test_cursor);

        select count (*) into v_count
          from Log
          join LogName on Log.id = LogName.log_id
         where person         = NetDB_Test.user_1.id and
               lower (action) = 'delete'             and
               recordtype     = 'adminteam'          and
               record_id      = rec_id               and
               name           = ateam_name_new       and
               logdate        > test_start;

        if (v_count = 1) then NetDB_Test.pass ();
        else                  NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'Insert a Domain.';
        test_start := systimestamp;

        test_cursor := NetDB_Name.reserve (NetDB_Test.user_1.id, domain_name, NetDB_Test.domain_1.domain, 'Domain');
        fill_id_lock (test_cursor, name_id, v_reserved);

        test_cursor := NetDB_Domain.master ('insert', NetDB_Test.user_1.id, null, null, NetDB_Name.fullname (name_id), null, a_comment);
        fill_id_lock (test_cursor, rec_id, v_lock);

        NetDB_Name.accept (NetDB_Test.user_1.id, name_id, null, 'Domain', rec_id, null, v_lock);
        NetDB_Owner.add (NetDB_Test.user_1.id, v_lock, NetDB_Test.group_1.id, null, rec_id, 'Domain');
        NetDB_Owner.add (NetDB_Test.user_1.id, v_lock, NetDB_Test.group_2.id, null, rec_id, 'Domain');

        test_cursor := NetDB_Domain.master ('verify', NetDB_Test.user_1.id, null, rec_id);
        NetDB_Test.empty_cursor (test_cursor);

        select count (*) into v_count
          from Log
          join LogName on Log.id = LogName.log_id
         where person         = NetDB_Test.user_1.id          and
               lower (action) = 'insert'                      and
               recordtype     = 'domain'                      and
               record_id      = rec_id                        and
               name           = NetDB_Name.fullname (name_id) and
               logdate        > test_start;

        if (v_count = 1) then NetDB_Test.pass ();
        else                  NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'Update a Domain.';
        test_start := systimestamp;

        test_cursor := NetDB_Domain.master ('update', NetDB_Test.user_1.id, v_lock, rec_id, null, domain_name_new, a_new_comment);
        NetDB_Test.empty_cursor (test_cursor);

        test_cursor := NetDB_Domain.master ('verify', NetDB_Test.user_1.id, null, rec_id);
        NetDB_Test.empty_cursor (test_cursor);

        select count (*) into v_count
          from Log
          join LogName on Log.id = LogName.log_id
         where person         = NetDB_Test.user_1.id          and
               lower (action) = 'update'                      and
               recordtype     = 'domain'                      and
               record_id      = rec_id                        and
               name           = NetDB_Name.fullname (name_id) and
               logdate        > test_start;

        if (v_count = 1) then NetDB_Test.pass ();
        else                  NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'Delete a Domain.';
        test_start := systimestamp;

        test_cursor := NetDB_Domain.master ('delete', NetDB_Test.user_1.id, v_lock, rec_id);
        NetDB_Test.empty_cursor (test_cursor);

        select count (*) into v_count
          from Log
          join LogName on Log.id = LogName.log_id
         where person         = NetDB_Test.user_1.id and
               lower (action) = 'delete'             and
               recordtype     = 'domain'             and
               record_id      = rec_id               and
               name           = NetDB_Util.fullname (domain_name_new, NetDB_Test.domain_1.domain) and
               logdate        > test_start;

        if (v_count = 1) then NetDB_Test.pass ();
        else                  NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'Insert an Ownership group.';
        test_start := systimestamp;

        test_cursor := NetDB_Owner.master ('insert', NetDB_Test.user_1.id, null, null, owner_name, null, a_comment);
        fill_id_lock (test_cursor, rec_id, v_lock);

        select count (*) into v_count
          from Log
          join LogName on Log.id = LogName.log_id
         where person         = NetDB_Test.user_1.id and
               lower (action) = 'insert'             and
               recordtype     = 'owner'              and
               record_id      = rec_id               and
               name           = owner_name           and
               logdate        > test_start;

        if (v_count = 1) then NetDB_Test.pass ();
        else                  NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'Update an Ownership group.';
        test_start := systimestamp;

        test_cursor := NetDB_Owner.master ('update', NetDB_Test.user_1.id, v_lock, rec_id, null, owner_name_new, a_new_comment);
        NetDB_Test.empty_cursor (test_cursor);

        select count (*) into v_count
          from Log
          join LogName on Log.id = LogName.log_id
         where person         = NetDB_Test.user_1.id and
               lower (action) = 'update'             and
               recordtype     = 'owner'              and
               record_id      = rec_id               and
               name           = owner_name_new       and
               logdate        > test_start;

        if (v_count = 1) then NetDB_Test.pass ();
        else                  NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'Delete an Ownership group.';
        test_start := systimestamp;

        test_cursor := NetDB_Owner.master ('delete', NetDB_Test.user_1.id, v_lock, rec_id);
        NetDB_Test.empty_cursor (test_cursor);

        select count (*) into v_count
          from Log
          join LogName on Log.id = LogName.log_id
         where person         = NetDB_Test.user_1.id and
               lower (action) = 'delete'             and
               recordtype     = 'owner'              and
               record_id      = rec_id               and
               name           = owner_name_new       and
               logdate        > test_start;

        if (v_count = 1) then NetDB_Test.pass ();
        else                  NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'Insert a Network.';
        test_start := systimestamp;

        test_cursor := NetDB_Name.reserve (NetDB_Test.user_1.id, network_name, NetDB_Test.domain_1.domain, 'Network');
        fill_id_lock (test_cursor, name_id, v_reserved);

        select * into net_name_rec from name where id = name_id;

        test_cursor := NetDB_Network.master ('insert', NetDB_Test.user_1.id, null, null, net_name_rec.fullname, null, a_comment);
        fill_id_lock (test_cursor, rec_id, v_lock);

        NetDB_Name.accept (NetDB_Test.user_1.id, name_id, null, 'Network', rec_id, null, v_lock);
        NetDB_Owner.add (NetDB_Test.user_1.id, v_lock, NetDB_Test.group_1.id, null, rec_id, 'Network');
        NetDB_Owner.add (NetDB_Test.user_1.id, v_lock, NetDB_Test.group_2.id, null, rec_id, 'Network');

        test_cursor := NetDB_Network.master ('verify', NetDB_Test.user_1.id, v_lock, rec_id);
        NetDB_Test.empty_cursor (test_cursor);

        network_handle := NetDB_Network.handle (rec_id);

        select count (*) into v_count
          from Log
          join LogName on Log.id = LogName.log_id
         where person         = NetDB_Test.user_1.id and
               lower (action) = 'insert'             and
               recordtype     = 'network'            and
               record_id      = rec_id               and
               name           = network_handle       and
               logdate        > test_start;

        if (v_count = 1) then NetDB_Test.pass ();
        else                  NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'Update a Network.';
        test_start := systimestamp;

        test_cursor := NetDB_Network.master ('update', NetDB_Test.user_1.id, v_lock, rec_id, null, null, a_new_comment);
        NetDB_Test.empty_cursor (test_cursor);

        test_cursor := NetDB_Network.master ('verify', NetDB_Test.user_1.id, v_lock, rec_id);
        NetDB_Test.empty_cursor (test_cursor);

        select count (*) into v_count
          from Log
          join LogName on Log.id = LogName.log_id
         where person         = NetDB_Test.user_1.id and
               lower (action) = 'update'             and
               recordtype     = 'network'            and
               record_id      = rec_id               and
               name           = network_handle       and
               logdate        > test_start;

        if (v_count = 1) then NetDB_Test.pass ();
        else                  NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'Delete a Network.';
        test_start := systimestamp;

        test_cursor := NetDB_Network.master ('delete', NetDB_Test.user_1.id, v_lock, rec_id);
        NetDB_Test.empty_cursor (test_cursor);

        select count (*) into v_count
          from Log
          join LogName on Log.id = LogName.log_id
         where person         = NetDB_Test.user_1.id and
               lower (action) = 'delete'             and
               recordtype     = 'network'            and
               record_id      = rec_id               and
               name           = network_handle       and
               logdate        > test_start;

        if (v_count = 1) then NetDB_Test.pass ();
        else                  NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'Insert a Node.';
        test_start := systimestamp;

        select name            into loc             from Location where rownum = 1;
        select state           into v_state         from State    where rownum = 1;
        select make, modelname into v_make, v_model from Model    where rownum = 1;
        select os              into v_os            from OS       where rownum = 1;

        test_cursor := NetDB_Name.reserve (NetDB_Test.user_1.id, node_name, NetDB_Test.domain_1.domain, 'Node');
        fill_id_lock (test_cursor, name_id, v_reserved);

        select fullname into node_handle from Name where name = node_name and domain = NetDB_Test.domain_1.id;

        test_cursor := NetDB_Node.master ('insert', NetDB_Test.user_1.id, null, null, node_handle, NetDB_Test.dept_1_name, loc, room, v_state, v_make, v_model, a_comment);
        fill_id_lock (test_cursor, rec_id, v_lock);

        test_cursor   := NetDB_Interface.master ('insert', NetDB_Test.user_1.id, v_lock, null, hw_addr, null, intfc_type, rec_id);
        result        := NetDB_Test.get_generic_row (test_cursor);
        test_intfc_id := result.value_string;
        NetDB_Test.empty_cursor (test_cursor);

        test_cursor := NetDB_IPaddress.reserve (NetDB_Test.user_1.id, valid_addr, 1);
        result      := NetDB_Test.get_generic_row (test_cursor);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_IPaddress.accept (NetDB_Test.user_1.id, valid_addr, 1, 0, test_intfc_id, v_lock, 'interface');

        NetDB_Name.accept (NetDB_Test.user_1.id, name_id, null, 'Node', rec_id, null, v_lock);
        NetDB_Owner.add (NetDB_Test.user_1.id, v_lock, NetDB_Test.group_1.id, null, rec_id, 'Node');
        NetDB_Owner.add (NetDB_Test.user_1.id, v_lock, NetDB_Test.group_2.id, null, rec_id, 'Node');
        NetDB_Node.admin ('add', NetDB_Test.user_1.id, v_lock, rec_id, null, null, null, NetDB_Test.user_1.id);
        NetDB_Node.OS ('add', NetDB_Test.user_1.id, v_lock, rec_id, null, v_os, null);

        test_cursor := NetDB_Node.master ('verify', NetDB_Test.user_1.id, null, rec_id);
        NetDB_Test.empty_cursor (test_cursor);

        select count (*) into v_count
          from Log
          join LogName      on Log.id = LogName.log_id
          join LogIPaddress on Log.id = LogIPaddress.log_id
         where person         = NetDB_Test.user_1.id      and
               lower (action) = 'insert'                  and
               recordtype     = 'node'                    and
               record_id      = rec_id                    and
               name           = node_handle               and
               ipaddress      = IP.normalize (valid_addr) and
               logdate        > test_start;

        if (v_count = 1) then NetDB_Test.pass ();
        else                  NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'Update a Node.';
        test_start := systimestamp;

        test_cursor := NetDB_Node.master ('update', NetDB_Test.user_1.id, v_lock, rec_id, null, NetDB_Test.dept_1_name, loc, room, v_state, v_make, v_model, a_new_comment);
        NetDB_Test.empty_cursor (test_cursor);

        test_cursor := NetDB_Node.master ('verify', NetDB_Test.user_1.id, null, rec_id);
        NetDB_Test.empty_cursor (test_cursor);

        select count (*) into v_count
          from Log
          join LogName on Log.id = LogName.log_id
         where person         = NetDB_Test.user_1.id and
               lower (action) = 'update'             and
               recordtype     = 'node'               and
               record_id      = rec_id               and
               name           = node_handle          and
               logdate        > test_start;

        if (v_count = 1) then NetDB_Test.pass ();
        else                  NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'Delete a Node.';
        test_start := systimestamp;

        test_cursor := NetDB_Node.master ('delete', NetDB_Test.user_1.id, v_lock, rec_id);
        NetDB_Test.empty_cursor (test_cursor);

        select count (*) into v_count
          from Log
          join LogName on Log.id = LogName.log_id
         where person         = NetDB_Test.user_1.id and
               lower (action) = 'delete'             and
               recordtype     = 'node'               and
               record_id      = rec_id               and
               name           = node_handle          and
               logdate        > test_start;

        if (v_count = 1) then NetDB_Test.pass ();
        else                  NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'Insert a User.';
        test_start := systimestamp;

        test_cursor := NetDB_Person.master ('insert', NetDB_Test.user_1.id, null, v_regid, person_name, v_netid, email, phone, 1, last_name);
        fill_id_lock (test_cursor, person_id, v_lock); -- there is no lock, but a variable must be used...

        test_cursor := NetDB_User.master ('insert', NetDB_Test.user_1.id, null, null, v_netid, person_id, NetDB_Test.group_1_name, NetDB_Test.domain_1_name, NetDB_Test.net_1_prefix_1, 1, 0, 0, a_comment);
        fill_id_lock (test_cursor, rec_id, v_lock);

        NetDB_User.org ('insert', NetDB_Test.user_1.id, v_lock, rec_id, NetDB_Test.dept_1.id, null, 0);
        NetDB_Owner.add (NetDB_Test.user_1.id, v_lock, NetDB_Test.group_1.id, null, rec_id, 'User');
        NetDB_Owner.add (NetDB_Test.user_1.id, v_lock, NetDB_Test.group_2.id, null, rec_id, 'User');

        test_cursor := NetDB_User.master ('verify', NetDB_Test.user_1.id, null, rec_id);
        NetDB_Test.empty_cursor (test_cursor);

        user_handle := NetDB_User.handle (rec_id);

        select count (*) into v_count
          from Log
          join LogName on Log.id = LogName.log_id
         where person         = NetDB_Test.user_1.id and
               lower (action) = 'insert'             and
               recordtype     = 'user'               and
               record_id      = rec_id               and
               name           = user_handle          and
               logdate        > test_start;

        if (v_count = 1) then NetDB_Test.pass ();
        else                  NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'Update a User.';
        test_start := systimestamp;

        test_cursor := NetDB_User.master ('update', NetDB_Test.user_1.id, v_lock, rec_id, null, null, NetDB_Test.group_2_name, NetDB_Test.domain_2_name, NetDB_Test.net_1_prefix_1, 1, 0, 0, a_new_comment);
        NetDB_Test.empty_cursor (test_cursor);

        test_cursor := NetDB_User.master ('verify', NetDB_Test.user_1.id, null, rec_id);
        NetDB_Test.empty_cursor (test_cursor);

        select count (*) into v_count
          from Log
          join LogName on Log.id = LogName.log_id
         where person         = NetDB_Test.user_1.id and
               lower (action) = 'update'             and
               recordtype     = 'user'               and
               record_id      = rec_id               and
               name           = user_handle          and
               logdate        > test_start;

        if (v_count = 1) then NetDB_Test.pass ();
        else                  NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg    := 'Delete a User.';
        test_start := systimestamp;

        test_cursor := NetDB_User.master ('delete', NetDB_Test.user_1.id, v_lock, rec_id);
        NetDB_Test.empty_cursor (test_cursor);

        select count (*) into v_count
          from Log
          join LogName on Log.id = LogName.log_id
         where person         = NetDB_Test.user_1.id and
               lower (action) = 'delete'             and
               recordtype     = 'user'               and
               record_id      = rec_id               and
               name           = user_handle          and
               logdate        > test_start;

        if (v_count = 1) then NetDB_Test.pass ();
        else                  NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;
exception
    when unrecoverable_error then
        dbms_output.put_line ('ERROR: All remaining tests aborted. An unrecoverable error occured: ' || err_msg);
    when others then
        dbms_output.put_line ('ERROR: Unhandled exception "' || sqlcode || '" with message "' || sqlerrm || '"');
        dbms_output.put_line (err_msg);
end;
/

rollback to start_log_tests;
