set serveroutput on size 100000;
savepoint start_make_tests;

declare
    test_number int          := 1;
    err_msg     varchar(256) := null;
    v_count     int          := 0;

    test_cursor NetDB.ref_cursor;
    result      result_row := result_row (null, null, null, null, null, null, null);

    -- values for insert and update.
    test_make   Make.make%TYPE := 'unlikely make name - test';
    test_make_2 Make.make%TYPE := 'test - unlikely make name';
    test_id     Make.make%TYPE := null;
    verified_id Make.make%TYPE := null;

    existing_make Make%ROWTYPE;

    unrecoverable_error EXCEPTION;
begin
    select count (*) into v_count from Make where make = test_make;

    if (v_count > 0) then
        err_msg := 'The test make "' || test_make || '" exists. If it is not a valid ' ||
                   'make, remove the entry and run the tests again. If it is valid, the ' ||
                   'tests must be modified.';
        raise unrecoverable_error;
    end if;

    select * into existing_make from Make where rownum = 1;

    NetDB_Test.revoke_access (NetDB_Test.user_1.id, 'Make');

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insert a make without permission';

        test_cursor := NetDB_Make.master ('insert', NetDB_Test.user_1.id, test_make);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update a make without permission.';

        test_cursor := NetDB_Make.master ('update', NetDB_Test.user_1.id, existing_make.make);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete a make without permission.';

        test_cursor := NetDB_Make.master ('delete', NetDB_Test.user_1.id, existing_make.make);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'Make');

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insert a make';

        test_cursor := NetDB_Make.master ('insert', NetDB_Test.user_1.id, test_make);
        NetDB_Test.empty_cursor (test_cursor);

        verified_id := NetDB_Make.verify_or_die (test_make);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update a make.';

        test_cursor := NetDB_Make.master ('update', NetDB_Test.user_1.id, test_make, test_make_2);
        NetDB_Test.empty_cursor (test_cursor);

        verified_id := NetDB_Make.verify_or_die (test_make_2);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete a make.';

        test_cursor := NetDB_Make.master ('delete', NetDB_Test.user_1.id, test_make_2);
        NetDB_Test.empty_cursor (test_cursor);

        verified_id := NetDB_Make.verify_or_die (test_make_2);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.record_not_found then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;
exception
    when unrecoverable_error then
        dbms_output.put_line ('ERROR: All remaining tests aborted. An unrecoverable error occured: ' || err_msg);
    when others then
        dbms_output.put_line ('ERROR: Unhandled exception "' || sqlcode || '" with message "' || sqlerrm || '"');
        dbms_output.put_line (err_msg);
end;
/

rollback to start_make_tests;
