set serveroutput on size 100000;
savepoint start_person_tests;

declare
    test_number int          := 1;
    err_msg     varchar(256) := null;
    v_count     int          := 0;

    test_cursor NetDB.ref_cursor;
    result      result_row := result_row (null, null, null, null, null, null, null);

    -- values for insert and update.
    test_name     Person.name%TYPE         := 'bobs name mcgee';
    test_name_2   Person.name%TYPE         := 'bobs sister mcgee';
    test_lastname Person.lastname%TYPE     := 'mcgee';
    test_regid    Person.person_regid%TYPE := 'uniqueregid';
    test_netid    Person.netid%TYPE        := 'bobmcgee';

    test_id     int := null;
    verified_id int := null;

    existing_person Person%ROWTYPE;

    unrecoverable_error EXCEPTION;
begin
    select count (*) into v_count from Person
     where lower (name) in (test_name, test_name_2);

    if (v_count > 0) then
        err_msg := 'The test person "' || test_name || '" or "' || test_name_2 || '" exists. If it is not a valid ' ||
                   'person, remove the entry and run the tests again. If it is valid, the ' ||
                   'tests must be modified.';
        raise unrecoverable_error;
    end if;

    select * into existing_person from Person where rownum = 1;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insert a person';

        test_cursor := NetDB_Person.master ('insert', NetDB_Test.user_1.id, null, test_regid, test_name, null, null, null, 1, test_lastname);
        result  := NetDB_Test.get_generic_row (test_cursor);
        test_id := result.value_string;
        NetDB_Test.empty_cursor (test_cursor);

        verified_id := NetDB_Person.verify_or_die (test_id);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update a person.';

        test_cursor := NetDB_Person.master ('insert', NetDB_Test.user_1.id, test_id, test_regid, test_name_2, null, null, null, 1, test_lastname);
        NetDB_Test.empty_cursor (test_cursor);

        verified_id := NetDB_Person.verify_or_die (test_id);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete a person.';

        test_cursor := NetDB_Person.master ('delete', NetDB_Test.user_1.id, test_id);
        NetDB_Test.empty_cursor (test_cursor);

        test_id := NetDB_Person.verify_or_die (test_id);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.record_not_found then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;
exception
    when unrecoverable_error then
        dbms_output.put_line ('ERROR: All remaining tests aborted. An unrecoverable error occured: ' || err_msg);
    when others then
        dbms_output.put_line ('ERROR: Unhandled exception "' || sqlcode || '" with message "' || sqlerrm || '"');
        dbms_output.put_line (err_msg);
end;
/

rollback to start_person_tests;
