set serveroutput on size 100000;
savepoint start_resname_tests;

declare
    test_number int          := 1;
    err_msg     varchar(256) := null;

    name_not_in_resname_table ResName.name%TYPE := 'blah';

    new_name1_id int := 0;
    new_name2_id int := 0;
    new_name3_id int := 0;

    res1 boolean;
    res2 boolean;
begin

    begin -- Begin is_reserved (...) tests.
        insert into ResName (name,          domain,                 modified_by,          modified)
                     values ('^bar[0-9]*$', NetDB_Test.domain_1.id, NetDB_Test.user_0.id, systimestamp)
        returning id into new_name1_id;

        insert into ResName (name,          domain, modified_by,          modified)
                     values ('^foo[0-9]+$', null,   NetDB_Test.user_0.id, systimestamp)
        returning id into new_name2_id;

        insert into ResName (name,       domain, modified_by,          modified)
                     values ('^barfoo$', null,   NetDB_Test.user_0.id, systimestamp)
        returning id into new_name3_id;

        insert into ResName_Owner (resname, owner) values (new_name1_id, NetDB_Test.group_1.id);
        insert into ResName_Owner (resname, owner) values (new_name1_id, NetDB_Test.group_2.id);
        insert into ResName_Owner (resname, owner) values (new_name2_id, NetDB_Test.group_1.id);

        NetDB_Test.add_to_group (NetDB_Test.user_1.id, NetDB_Test.group_1.id);
        NetDB_Test.add_to_group (NetDB_Test.user_2.id, 1); -- Prevent the trigger from firing if we end up dropping all of user_2's groups.
        NetDB_Test.remove_from_group (NetDB_Test.user_2.id, NetDB_Test.group_1.id);
        NetDB_Test.remove_from_group (NetDB_Test.user_2.id, NetDB_Test.group_2.id);

        for r in ( select NetDB_Test.user_1.id user_id, 'bar1'                    name, null                   domain, 0 ex from dual union
                   select NetDB_Test.user_1.id user_id, 'bar1'                    name, NetDB_Test.domain_1.id domain, 0 ex from dual union
                   select NetDB_Test.user_1.id user_id, 'bar1'                    name, NetDB_Test.domain_2.id domain, 0 ex from dual union
                   select NetDB_Test.user_2.id user_id, 'bar1'                    name, null                   domain, 0 ex from dual union
                   select NetDB_Test.user_2.id user_id, 'bar1'                    name, NetDB_Test.domain_1.id domain, 1 ex from dual union
                   select NetDB_Test.user_2.id user_id, 'bar1'                    name, NetDB_Test.domain_2.id domain, 0 ex from dual union

                   select NetDB_Test.user_1.id user_id, 'bar'                     name, null                   domain, 0 ex from dual union
                   select NetDB_Test.user_1.id user_id, 'bar'                     name, NetDB_Test.domain_1.id domain, 0 ex from dual union
                   select NetDB_Test.user_1.id user_id, 'bar'                     name, NetDB_Test.domain_2.id domain, 0 ex from dual union
                   select NetDB_Test.user_2.id user_id, 'bar'                     name, null                   domain, 0 ex from dual union
                   select NetDB_Test.user_2.id user_id, 'bar'                     name, NetDB_Test.domain_1.id domain, 1 ex from dual union
                   select NetDB_Test.user_2.id user_id, 'bar'                     name, NetDB_Test.domain_2.id domain, 0 ex from dual union

                   select NetDB_Test.user_1.id user_id, 'foo0'                    name, null                   domain, 0 ex from dual union
                   select NetDB_Test.user_1.id user_id, 'foo0'                    name, NetDB_Test.domain_1.id domain, 0 ex from dual union
                   select NetDB_Test.user_1.id user_id, 'foo0'                    name, NetDB_Test.domain_2.id domain, 0 ex from dual union
                   select NetDB_Test.user_2.id user_id, 'foo0'                    name, null                   domain, 1 ex from dual union
                   select NetDB_Test.user_2.id user_id, 'foo0'                    name, NetDB_Test.domain_1.id domain, 1 ex from dual union
                   select NetDB_Test.user_2.id user_id, 'foo0'                    name, NetDB_Test.domain_2.id domain, 1 ex from dual union

                   select NetDB_Test.user_1.id user_id, 'barfoo'                  name, null                   domain, 1 ex from dual union
                   select NetDB_Test.user_1.id user_id, 'barfoo'                  name, NetDB_Test.domain_1.id domain, 1 ex from dual union
                   select NetDB_Test.user_1.id user_id, 'barfoo'                  name, NetDB_Test.domain_2.id domain, 1 ex from dual union
                   select NetDB_Test.user_2.id user_id, 'barfoo'                  name, null                   domain, 1 ex from dual union
                   select NetDB_Test.user_2.id user_id, 'barfoo'                  name, NetDB_Test.domain_1.id domain, 1 ex from dual union
                   select NetDB_Test.user_2.id user_id, 'barfoo'                  name, NetDB_Test.domain_2.id domain, 1 ex from dual union

                   select NetDB_Test.user_1.id user_id, name_not_in_resname_table name, null                   domain, 0 ex from dual union
                   select NetDB_Test.user_1.id user_id, name_not_in_resname_table name, NetDB_Test.domain_1.id domain, 0 ex from dual union
                   select NetDB_Test.user_1.id user_id, name_not_in_resname_table name, NetDB_Test.domain_2.id domain, 0 ex from dual union
                   select NetDB_Test.user_2.id user_id, name_not_in_resname_table name, null                   domain, 0 ex from dual union
                   select NetDB_Test.user_2.id user_id, name_not_in_resname_table name, NetDB_Test.domain_1.id domain, 0 ex from dual union
                   select NetDB_Test.user_2.id user_id, name_not_in_resname_table name, NetDB_Test.domain_2.id domain, 0 ex from dual order by 2,3,1 )
        loop
            NetDB_Test.output_and_increment_test (test_number);
            err_msg := 'Check if the name "' || r.name || '" is reserved for the user "' || NetDB_User.handle (r.user_id) || '" in the domain "';

            if (r.domain is null) then
                err_msg := err_msg || r.domain || '".';
            else
                err_msg := err_msg || NetDB_Domain.handle (r.domain) || '".';
            end if;

            res1 := NetDB_ResName.is_reserved (r.user_id, r.name, r.domain);
            res2 := Util.to_boolean (r.ex);

            if (res1 = res2) then
                NetDB_Test.pass ();
            else
                NetDB_Test.failed (err_msg);
            end if;
        end loop;
    exception
         when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end; -- end is_reserved (...) tests.
exception
    when others then
        dbms_output.put_line ('ERROR: Unhandled exception "' || sqlcode || '" with message "' || sqlerrm || '"');
        dbms_output.put_line (err_msg);
end;
/

rollback to start_resname_tests;
