-- NetDB_Util.sql -- NetDB_Util unit tests

declare
    test_number int          := 1;
    err_msg     varchar(256) := null;

    v_string  varchar(256) := null;
    i         int; -- scratch integer
begin

-- validate()

    -- valid characters
    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Test validate (...) with a valid string.';

        NetDB_Util.validate ('ABCDEFGHIJKLMNOPQRSTUVWXYZ 1234567890 ''' ||
                             'abcdefghijklmnopqrstuvwxyz !@#$%^&*()+=~`"');

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    -- invalid characters:  chr (9)  -- horizantal tab  chr (12) -- form feed
    --                      chr (10) -- line feed       chr (13) -- carriage return
    --                      chr (11) -- vertical tab
    for i in 9 .. 13 loop
        begin
            NetDB_Test.output_and_increment_test (test_number);
            err_msg := 'Test validate (...) with the invalid character "' || i || '".';

            NetDB_Util.validate ('ABC' || chr (i));

            NetDB_Test.failed (err_msg);
        exception
            when NetDB.bad_parameter then
                NetDB_Test.pass ();
            when others then
                NetDB_Test.failed (err_msg, sqlerrm);
        end;
    end loop;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Get the fullname of (null, null).';

        v_string := NetDB_Util.fullname (null, null);

        if (v_string is null) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Get the fullname of (value, null).';

        v_string := NetDB_Util.fullname (NetDB_Test.domain_1.domain, null);

        if (v_string is null) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Get the fullname of (null, value).';

        v_string := NetDB_Util.fullname (null, 'COM');

        if (v_string is null) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Get the fullname of (value, value).';

        v_string := NetDB_Util.fullname ('   ' || NetDB_Test.name_domain_1.name || '   ', '   COM   ');

        if (v_string = NetDB_Test.domain_1.domain) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
            dbms_output.put_line ('expected "' || NetDB_Test.domain_1.domain || '"     got "' || v_string || '"');
        end if;
    end;
end;
/
