#######################################################################
#
# DNS_config.sample.m4 - DNS generation/configuration information
#
#  This file defines any number of DNS domains, nets or hosts.  Each
#  domain or net has a set of information associated with it.  Items
#  include:
#
#    hacks:       boolean - anything but 0 means there's a hacks file
#    master:      master name server (may have an NS record)
#    official:    official name servers (those with NS records)
#    stealth:     other name servers (those without NS records)
#    soa:         SOA info - ttl, master, mail,
#                            refresh, retry, expire, neg_cache_ttl
#    delegated:   a delegated subdomain or subnet
#    zonefile:    override the default zonefile name
#    ttl:         default TTL
#
#      Hacks files contain DNS resource records that can't be
#      generated from NetDB.
#
#      The soa and ttl items are only required for zones generated
#      from NetDB.  The ttl item is optional; if it not supplied,
#      the neg_cache_ttl will be used instead.
#
#      If master is set to "ALL", then all servers (official and
#      stealth) will be considered master servers.  This is useful
#      for fixed zones like 0.0.127.IN-ADDR.ARPA.  [It's also a
#      hack.  Someday 'master' should accept more than one value.]
#
#  Host entries have only one attribute:
#
#    ip:        the IP address of the host for named operations
#
#  The host entry exists to clarify the IP address of any servers
#  listed in the file.  Only servers that need clarification of
#  their IP address need be listed.  In fact, listing other servers
#  will only lead to trouble when you forget to update this file
#  after readdressing a name server.
#
#----------------------------------------------------------------------
# Copyright (c) 2007 Board of Trustees, Leland Stanford Jr. University
#######################################################################

define(MASTER,`spacewell-gears.Spacely-Sprockets.COM')

define(UNIBLAB,`uniblab.Spacely-Sprockets.COM')
define(LECTRONIMO,`lectronimo.Spacely-Sprockets.COM')

define(NSs,`UNIBLAB, LECTRONIMO')

define(SOA,`172800, $1, hostmaster.Spacely-Sprockets.COM, 1200, 600, 604800, 3600')

define(DEFAULT,`
  soa:      SOA(UNIBLAB)
  master:   MASTER
  official: NSs
')

define(HACKS,`
  hacks:    true
')

#######################################################################
# Hosts - just those that we can't get from DNS
#######################################################################

# Note: In a real application these *would* be in the DNS!

host: spacewell-gears.Spacely-Sprockets.COM {
  ip: 191.64.133.39
}
host: uniblab.Spacely-Sprockets.COM {
  ip: 191.64.132.107
}
host: lectronimo.Spacely-Sprockets.COM {
  ip: 191.64.132.108
}

#######################################################################
# Domains
#######################################################################

# Main domains

domain: Spacely-Sprockets.COM { DEFAULT HACKS
  delegated: Pinky
}

domain: Cogswell-Cogs.COM { DEFAULT }

#######################################################################
# Nets
#######################################################################

# Main nets

net: 191.64.132 { DEFAULT }
net: 191.64.20  { DEFAULT }

# Private address space

net: 10 { DEFAULT HACKS }

#######################################################################
# Secondary service for other sites
#######################################################################

# Space Cookie Corporation

define(SPACECOOK,`
  master:   ns1.space-cookies.com
  official: LECTRONIMO
')

host: ns1.space-cookies.com {
  ip: 191.64.36.151
}

domain: space-cookies.com           { SPACECOOK }
domain: chocolate.space-cookies.com { SPACECOOK }

net: 191.64.36 { SPACECOOK }

