-- dhcp.sql --- SQL to generate DHCP config file from NetDB

-- address spaces with router IP addresses and those router addresses

insert into AS_router (
  select IPaddressSpace.id,
         IPaddressSpace.network,
         IPaddressSpace.prefix,
         IPaddressSpace.address,
         IPaddressSpace.broadcast,
         min(IPaddress.IPaddress)
    from IPaddressSpace
    join IPaddress      on IPaddress.IPaddress between IPaddressSpace.address and IPaddressSpace.broadcast
    join Interface      on IPaddress.interface = Interface.id
    join Node_NodeType  on Interface.node = Node_NodeType.node
   where Node_NodeType.type = 'Router'
   group
      by IPaddressSpace.id,
         IPaddressSpace.network,
         IPaddressSpace.prefix,
         IPaddressSpace.address,
         IPaddressSpace.broadcast
);

insert into AS_router (
  select IPaddressSpace.id,
         IPaddressSpace.network,
         IPaddressSpace.prefix,
         IPaddressSpace.address,
         IPaddressSpace.broadcast,
         ''
    from IPaddressSpace
   where id not in (select id from AS_router)
);

-- shared networks

select 'NETWORK',
       fullname,
       cast( IP.host(prefix)    as varchar(15) ),
       cast( IP.netmask(prefix) as varchar(15) ),
       cast( IP.host(router)    as varchar(15) )
  from AS_router
  join Name on Name.network = AS_router.network
 where AS_router.router is not null;

-- ranges

select 'ADDRESS', cast( IP.host(prefix) as varchar(15) ), IPaddress_txt
  from AS_router, IPaddress
 where AS_router.id = IPaddress.IPaddressSpace and active = 1
   and AS_router.router is not null
 order by IPaddress;

-- nerf address to get last range to print - see code

select 'ADDRESS', '127.0.0.0', '127.0.0.127' from dual;

-- hosts with IP addresses in shared networks with routers

select /*+ ORDERED USE_NL(AS_ROUTER IPADDRESS INTERFACE NAME DOMAIN) */
      'HOST',
       name,
       Domain.domain,
       substr(HW_address,1,12),
       case when (IPaddress.active = 1 and AS_router.router is not null) then IPaddress_txt else NULL end as IPaddress,
       roam
  from      AS_router
       join IPaddress  on IPaddress.IPaddress between AS_router.address and AS_router.broadcast
       join Interface  on Interface.id = IPaddress.interface
       join Name       on Name.node = Interface.node
       join Domain     on Domain.id = Name.domain
 where Interface.HW_address is not null and Interface.DHCP = 1
   and Interface.node not in (select id from Node where state in (select state from state where denyDHCP != 0));

-- roaming-only hosts

select 'HOST',
       name,
       Domain.domain,
       substr(HW_address,1,12),
       IPaddress.IPaddress_txt,
       roam
  from      Interface
       join Name       on Name.node = Interface.node
       join Domain     on Name.domain = Domain.id
  left join IPaddress  on Interface.id = IPaddress.interface
 where Interface.HW_address is not null and Interface.DHCP = 1 and Interface.roam = 1
   and IPaddress.interface is NULL
   and Interface.node not in (select id from Node where state in (select state from state where denyDHCP != 0));

-- interface DHCP options

select 'INTopt',
       substr(HW_address,1,12),
       DHCPoption.label,
       DHCPoption.type,
       value,
       datatype
  from Interface, Interface_DHCPoptions, DHCPoption
 where Interface.HW_address is not null and Interface.DHCP = 1
   and Interface.id = Interface_DHCPoptions.interface
   and DHCPoption.id = Interface_DHCPoptions.DHCPoption;

-- network DHCP options

select 'NETopt',
       fullname,
       DHCPoption.label,
       DHCPoption.type,
       value,
       DHCPoption.datatype
  from Name, Network, Network_DHCPoptions, DHCPoption
 where Name.network = Network.id
   and Network.id = Network_DHCPoptions.network
   and DHCPoption.id = Network_DHCPoptions.DHCPoption;

-- address space DHCP options

select 'ASopt',
       cast( IP.host(prefix) as varchar(15) ),
       DHCPoption.label,
       DHCPoption.type,
       value,
       DHCPoption.datatype
  from AS_router, IPaddressSpace_DHCPoptions, DHCPoption
 where AS_router.id = IPaddressSpace_DHCPoptions.IPaddressSpace
   and DHCPoption.id = IPaddressSpace_DHCPoptions.DHCPoption
   and AS_router.router is not null;

