-- IPaddress_b_iu_r.sql --- IPaddress table before insert or update trigger

create or replace trigger IPaddress_b_iu_r
  before insert or update on IPaddress
  for each row
declare
    subnet     int        := NULL;
    address    IP.address := NULL;
    broadcast  IP.address := NULL;
begin

    -- make sure the user is not setting derivatives
    if INSERTING then
        NetDB_Util.sysmaint_check('IPaddress_txt',:NEW.IPaddress_txt);
    end if;

    -- normalize IPaddress and set IPaddress_txt
    :NEW.IPaddress     := IP.normalize(:NEW.IPaddress);
    :NEW.IPaddress_txt := IP.host(:NEW.IPaddress);

    -- get address space, address space address and address space broadcast address
    begin
        select id, address, broadcast into subnet, address, broadcast from IPaddressSpace
         where :NEW.IPaddress between address and broadcast;
    exception when NO_DATA_FOUND then
        raise_application_error(NetDB.ex_no_such_subnet,
          'There is no address space containing the IP address "' || :NEW.IPaddress_txt || '".');
    end;

    -- is the new address space correct?
    if (:NEW.IPaddressSpace is not null and :NEW.IPaddressSpace != subnet) then
        raise_application_error(NetDB.ex_subnet_ip_mismatch,
          'Specified address space does not contain IP address "' || :NEW.IPaddress_txt || '".');

    -- is the new address a subnet or broadcast address?
    elsif (:NEW.IPaddress = address) then
        raise_application_error(NetDB.ex_address_is_subnet,
          'Invalid input:  IP address "' || :NEW.IPaddress_txt ||
            '" is the address space subnet address.');
    elsif (:NEW.IPaddress = broadcast) then
        raise_application_error(NetDB.ex_address_is_broadcast,
          'Invalid input:  IP address "' || :NEW.IPaddress_txt ||
            '" is the address space broadcast address.');
    end if;

end;
/

