-- Name_b_u_r.sql --- Name table before update trigger

create or replace trigger Name_b_u_r
  before update
      of id, fullname, fullname_lc, name, IPaddress, aliasee, interface, node
      on Name
  for each row
begin

    -- user must not change the value of the id or domain columns
    if (:NEW.id != :OLD.id) then
        raise_application_error(NetDB.ex_system_supplies_id, 'ID values are supplied by the system and cannot be changed.');
    elsif (:NEW.domain != :OLD.domain) then
        raise_application_error(NetDB.ex_no_moving_names, 'Names cannot be moved to different domains.');

    -- user must not update derivative columns
    elsif (UPDATING('fullname'   )) then NetDB_Util.sysmaint_check('fullname',    'fullname'   );
    elsif (UPDATING('fullname_lc')) then NetDB_Util.sysmaint_check('fullname_lc', 'fullname_lc');

    -- allow only case changes for domain names
    elsif (UPDATING('name') and :NEW.name != :OLD.name and :NEW.label = 'Domain') then
        if (lower(:NEW.name) != lower(:OLD.name)) then
            raise_application_error(NetDB.ex_no_renaming_domains, 'Domain names cannot be modified except for case changes.');
        else
            insert into updated_domains (domain, parent) values (:NEW.id, :NEW.domain);
        end if;

    -- add new name id to dirty_related_node so related_node will get set
    elsif (UPDATING('IPaddress') or UPDATING('node'   )  or
           UPDATING('interface') or UPDATING('aliasee')) then
        insert into dirty_related_node (name) values (:NEW.id);
    end if;

end;
/

