/************************************************************************
 *
 * ATeamInsertCommand.java - db call to NetDB_AdminTeam.master 'insert', ...
 *
 * $Id: ATeamInsertCommand.java,v 1.2 2006/09/26 17:24:13 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.util.*;
import java.lang.*;
import java.sql.*;

/** db call to NetDB_AdminTeam.master 'insert' */
class ATeamInsertCommand extends ATeamCommand implements ComplexDBCall {

    public Hashtable newATeam;

    public ATeamInsertCommand(stanford.netdb.utils.Logger logger,
                              Hashtable newATeam, Integer op_uid)
    {
        super(logger);
        this.newATeam = newATeam;

        values[ACTION_INDEX]     = "insert";
        values[ID_OP_USER_INDEX] = op_uid;
        values[NAME_INDEX]       = ((String)Utils.n2s(newATeam.get("name"))).trim();
        values[EMAIL_INDEX]      = ((String)Utils.n2s(newATeam.get("email"))).trim();
        values[PHONE_INDEX]      = ((String)Utils.n2s(newATeam.get("phone"))).trim();
        values[COMMENT_INDEX]    = o2v(newATeam.get("comment"));
    }

    public Hashtable doIt(NetDBConnection con) throws NetDBException, Exception {
        BeginTransaction(con.con);
        boolean commit_finally = false;
        int id = 0;
        Timestamp lock = null;

        try {
            logger.info("inserting ateam "+values[NAME_INDEX]);
            logger.debug("do sigle value attributes");
            db_call_type = ORA_FUN_CALL;
            Hashtable result = con.callDB(this);

            // check for errors
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            id = Integer.parseInt((String)result.get("id"));
            lock = (Timestamp)result.get("locked");

            // now groups
            Hashtable newGroups = (Hashtable)newATeam.get("groups");
            Vector groups_to_insert = new Vector(newGroups.keySet());

            // insert groups to insert
            logger.debug("inserting groups");
            ComplexDBCall cmd =
                new ObjectGroupCommand(logger, ObjectGroupCommand.ADD_GROUPS,
                                       id, ObjectGroupCommand.ATEAM,
                                       lock, groups_to_insert,
                                       (Integer)values[ID_OP_USER_INDEX]);
            result = cmd.doIt(con);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            // update members
            Hashtable newMembers = Utils.nHash(newATeam.get("members"));
            Vector m2i = new Vector();
            Enumeration enm = newMembers.keys();
            while( enm.hasMoreElements() ) {
                // keys are Person IDs
                m2i.addElement( ((Hashtable)newMembers.get(enm.nextElement())).get("id") );
            }

            // insert m2i (members to insert)
            logger.debug("inserting members");
            cmd = new ATeamMembersCommand(logger, ATeamMembersCommand.INSERT, m2i,
                                          new Integer(id), lock,
                                          (Integer)values[ID_OP_USER_INDEX]);

            cmd.doIt(con);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            // update organizations/departments
            Hashtable newDepts = (Hashtable)newATeam.get("departments");
            if( newDepts == null )
                newDepts = new Hashtable();
            Vector depts_to_insert = new Vector(newDepts.keySet());

            // insert depts to insert
            logger.debug("inserting depts");
            cmd = new ATeamDepartmentsCommand(logger, ATeamDepartmentsCommand.INSERT,
                                              depts_to_insert, new Integer(id),
                                              lock, (Integer)values[ID_OP_USER_INDEX]);

            result = cmd.doIt(con);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            // now do check
            logger.debug("do check");
            // values[0] has to be reset here becouse last call filled up this with some value
            db_call_type         = ORA_FUN_IGNORE_RET_CALL;
            values[0]            = newRefCursor();
            values[ACTION_INDEX] = "verify";

            //  values[NAME_INDEX] has to be set to Null on order to verify by id
            values[ATEAM_ID_INDEX] = new Integer(id);
            result                 = con.callDB(this);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            commit_finally = true;
            logger.debug("done");

        } catch(Exception ex) {
            ex.printStackTrace();
            try {
                logger.debug("Rollback Transaction");
                RollbackTransaction(con.con);
            } catch(Exception exx) { }
            throw ex;

        } finally {
            if( commit_finally )
                try {
                    logger.info("inserting ateam "+values[NAME_INDEX]+" done");
                    // Unlock the admin team now that we're done.
                    try {
                        DBCall call =
                            new ATeamLockUnLockCommand(logger, ATeamLockUnLockCommand.UNLOCK_ACTION, null,
                                                       id, lock, (Integer)values[ID_OP_USER_INDEX]);
                        call.processResultSet(con.callDB(call));
                    } catch (Exception exxx) { }
                    CommitTransaction(con.con);
                } catch(Exception exx) { }
            else
                try {
                    logger.info("inserting ateam "+values[NAME_INDEX]+" failed");
                    logger.debug("Rollback Transaction");
                    RollbackTransaction(con.con);
                } catch(Exception exx) { }
        }
        return new Hashtable();
    }
}
