/************************************************************************
 *
 * AddrSpaceInsertCommand.java - db call pIPaddressSpace 'insert'
 *
 * $Id: AddrSpaceInsertCommand.java,v 1.3 2006/09/26 17:24:23 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import stanford.netdb.lib.*;
import java.util.*;
import java.lang.*;
import java.sql.*;

/* db call pIPaddressSpace 'insert' */
class AddrSpaceInsertCommand extends AddrSpaceCommand implements ComplexDBCall{

    private Hashtable      as;
    private Integer        op_uid;
    private NetworkCommand parent;

    public AddrSpaceInsertCommand(stanford.netdb.utils.Logger logger, NetworkCommand parent,
                                  Hashtable as, Integer idNet, Integer op_uid) {
        super(logger);

        this.parent = parent;
        this.as     = as;
        this.op_uid = op_uid;
        lock        = parent.lock;

        values[ACTION_INDEX]     = "insert";
        values[ID_OP_USER_INDEX] = op_uid;
        values[PREFIX_INDEX]     = as.get("addr_space_prefix");
        values[ID_NETWORK_INDEX] = idNet;
        values[LOWRES_INDEX]     = as.get("low_res");
        values[HIGHRES_INDEX]    = as.get("high_res");
        values[COMMENT_INDEX]    = ((String)Utils.n2s(as.get("comment"))).trim();
        values[LOCKED_INDEX]     = values[NET_LOCK_INDEX] = lock;
    }

    public Hashtable doIt(NetDBConnection con)
        throws NetDBException, Exception, NetworkCommand.MiniException {
        Hashtable result = con.callDB(this);
        checkErrors("Failed to insert address space "+values[PREFIX_INDEX], result);

        Integer idAS = new Integer((String)result.get("id"));

        // insert new groups
        Vector groups = new Vector();
        Enumeration enm = Utils.nHash(as.get("groups")).keys();
        while( enm.hasMoreElements() )
            groups.addElement(enm.nextElement());

        ComplexDBCall call =
            new ObjectGroupCommand(logger, ObjectGroupCommand.ADD_GROUPS, idAS.intValue(),
                                   ObjectGroupCommand.IPADDRSPACE, lock, groups, op_uid);
        checkErrors("Failed to insert groups", call.doIt(con));

        // insert new dhcp options
        call = new DHCPOptionsActionCommand(logger, DHCPOptionsActionCommand.INSERT_ACTION, lock,
                                            idAS, Constants.IPADDRSPACE,
                                            Utils.nHash(as.get("dhcp_options")), op_uid);
        checkErrors("Failed to insert dhcp options", call.doIt(con));

        // insert new dhcp ip addresses
        Hashtable newDHCPips = Utils.nHash(as.get("dhcp_addresses"));

        // search for blocks of ip addresses
        Hashtable blocks = new Hashtable();

        // Should be replaced with code which does not depend on transformation of ips into integers
        Comparator ip_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    long ip1 = Utils.ipstr2long((String)Utils.n2s(((Hashtable)o1).get("ip_address")));
                    long ip2 = Utils.ipstr2long((String)Utils.n2s(((Hashtable)o2).get("ip_address")));

                    long cmp_val = ip1 - ip2;
                    return cmp_val>0?1:(cmp_val<0?-1:0);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };
        java.util.List ips = Arrays.asList(newDHCPips.values().toArray());
        Collections.sort(ips, ip_cmp);
        Object[] ip = ips.toArray();
        long last_ip = 0;
        String current_block_key = null;
        int current_block_counter = 0;
        for( int i = 0; i < ip.length; i++ ) {
            String next_ip_str = (String)Utils.n2s(((Hashtable)ip[i]).get("ip_address"));
            long next_ip = Utils.ipstr2long(next_ip_str);
            if( i == 0 ) {
                current_block_key = next_ip_str;
                current_block_counter = 1;
                blocks.put(current_block_key, new Integer(current_block_counter));
            } else {
                if( last_ip + 1 == next_ip ) {
                    // append to already created block
                    blocks.put(current_block_key, new Integer(++current_block_counter));
                } else {
                    // create new block
                    current_block_key = next_ip_str;
                    current_block_counter = 1;
                    blocks.put(current_block_key, new Integer(current_block_counter));
                }
            }
            last_ip = next_ip;
        }

        // now reserve blocks
        enm = blocks.keys();
        while( enm.hasMoreElements() ) {
            String start_ip = (String)enm.nextElement();
            int quantity = ((Integer)blocks.get(start_ip)).intValue();
            DBCall cmd = new IPAddressReserveCommand(logger, quantity, start_ip, true, true, op_uid);
            checkErrors("Failed to reserve ip address "+start_ip, cmd.processResultSet(con.callDB(cmd)));
        }

        enm = newDHCPips.keys();
        Hashtable accepted_ip_addresses = new Hashtable();
        while( enm.hasMoreElements() ) {
            Hashtable ipAddr = (Hashtable)newDHCPips.get((String)enm.nextElement());
            String reserved_ip_addr = (String)ipAddr.get("ip_address");
            ((NetDBii)logger).register_reserved_ip(reserved_ip_addr);
            DBCall cmd = new IPAddressAcceptCommand(logger, reserved_ip_addr,
                                                    (Boolean)ipAddr.get("active?"), idAS, lock,
                                                    IPAddressAcceptCommand.ADDR_SPACE_TYPE,
                                                    System.getProperty("DefaultPTRpref", "0"), op_uid);

            checkErrors("Failed to insert ip address "+ipAddr.get("ip_address"),
                        cmd.processResultSet(con.callDB(cmd)));
            accepted_ip_addresses.put(reserved_ip_addr, "");
        }

        Hashtable retval = new Hashtable();
        retval.put("accepted_ip_addresses", accepted_ip_addresses);
        return retval;
    } // end of doIt(...)
}
