/************************************************************************
 *
 * AddrSpaceModifyCommand.java - db call NetDB_IPaddressSpace.master 'update'
 *
 * $Id: AddrSpaceModifyCommand.java,v 1.3 2006/09/26 17:26:21 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import stanford.netdb.lib.*;
import java.util.*;
import java.lang.*;
import java.sql.*;

/* db call NetDB_IPaddressSpace.master 'update' */
class AddrSpaceModifyCommand extends AddrSpaceCommand implements ComplexDBCall {

    private Hashtable      newAS;
    private Integer        op_uid;
    private NetworkCommand parent;

    public AddrSpaceModifyCommand(stanford.netdb.utils.Logger logger, NetworkCommand parent,
                                  Hashtable newAS,
                                  Integer idNet, Integer op_uid) {
        super(logger);

        this.parent = parent;
        this.newAS  = newAS;
        this.op_uid = op_uid;
        lock        = parent.lock;

        values[ACTION_INDEX]     = "update";
        values[ID_OP_USER_INDEX] = op_uid;
        values[PREFIX_INDEX]     = newAS.get("addr_space_prefix");
        values[ID_NETWORK_INDEX] = idNet;
        values[LOWRES_INDEX]     = newAS.get("low_res");
        values[HIGHRES_INDEX]    = newAS.get("high_res");
        values[COMMENT_INDEX]    = ((String)Utils.n2s(newAS.get("comment"))).trim();
        values[LOCKED_INDEX]     = values[NET_LOCK_INDEX] = lock;
    }

    public Hashtable doIt(NetDBConnection con)
        throws NetDBException, Exception, NetworkCommand.MiniException {
        Hashtable result = con.callDB(this);
        checkErrors("Failed to modify address space "+values[PREFIX_INDEX], result);

        Integer idAS = new Integer((String)result.get("id"));

        Vector groups_to_insert = new Vector();
        Vector groups_to_delete = new Vector();
        Vector groups_to_keep   = new Vector();
        Utils.keysIntersect(Utils.nHash(newAS.get("old.groups")), Utils.nHash(newAS.get("groups")),
                            groups_to_insert, groups_to_delete, groups_to_keep);

        // insert groups_to_insert
        logger.debug("inserting groups_to_insert");
        ComplexDBCall call =
            new ObjectGroupCommand(logger, ObjectGroupCommand.ADD_GROUPS, idAS.intValue(),
                                   ObjectGroupCommand.IPADDRSPACE, lock, groups_to_insert, op_uid);
        checkErrors("Failed to insert groups", call.doIt(con));

        // delete groups_to_delete
        logger.debug("deleting groups_to_delete");
        call =
            new ObjectGroupCommand(logger, ObjectGroupCommand.REMOVE_GROUPS, idAS.intValue(),
                                   ObjectGroupCommand.IPADDRSPACE, lock, groups_to_delete, op_uid);
        checkErrors("Failed to delete groups", call.doIt(con));

        // delete old dhcp options
        call = new DHCPOptionsActionCommand(logger, DHCPOptionsActionCommand.DELETE_ACTION, lock,
                                            idAS, Constants.IPADDRSPACE,
                                            Utils.nHash(newAS.get("old.dhcp_options")), op_uid);
        checkErrors("Failed to delete dhcp options", call.doIt(con));

        // insert new dhcp options
        call = new DHCPOptionsActionCommand(logger, DHCPOptionsActionCommand.INSERT_ACTION, lock,
                                            idAS, Constants.IPADDRSPACE,
                                            Utils.nHash(newAS.get("dhcp_options")), op_uid);
        checkErrors("Failed to insert dhcp options", call.doIt(con));

        // now sort all dhcp ip addresses
        Hashtable dhcp_ips_to_insert = new Hashtable();
        Hashtable dhcp_ips_to_delete = new Hashtable();
        Hashtable dhcp_ips_to_update = new Hashtable();

        Hashtable newDHCPips = (Hashtable)newAS.get("dhcp_addresses");
        if( newDHCPips == null )
            newDHCPips = new Hashtable();
        Enumeration enm = newDHCPips.keys();
        while( enm.hasMoreElements() ) {
            String key = (String)enm.nextElement();
            Hashtable ip = (Hashtable)newDHCPips.get(key);
            dhcp_ips_to_insert.put(((String)ip.get("ip_address")).trim(), ip.get("active?"));
        }

        Hashtable oldDHCPips = (Hashtable)newAS.get("old.dhcp_addresses");
        if( oldDHCPips == null )
            oldDHCPips = new Hashtable();
        enm = oldDHCPips.keys();
        while( enm.hasMoreElements() ) {
            String key = (String)enm.nextElement();
            Hashtable oldIP = (Hashtable)oldDHCPips.get(key);
            String oldIPstr = ((String)oldIP.get("ip_address")).trim();
            Boolean newState = (Boolean)dhcp_ips_to_insert.get(oldIPstr);
            if( newState == null ) {
                oldDHCPips.remove(key);
                dhcp_ips_to_delete.put(oldIPstr, "");
            } else if( newState.booleanValue() != ((Boolean)oldIP.get("active?")).booleanValue() )
                dhcp_ips_to_update.put(oldIPstr, dhcp_ips_to_insert.remove(oldIPstr));
            else // newState.booleanValue() == ((Boolean)oldIP.get("state?")).booleanValue()
                dhcp_ips_to_insert.remove(oldIPstr);
        }

        // delete dhcp_ips_to_delete
        enm = dhcp_ips_to_delete.keys();
        while( enm.hasMoreElements() ) {
            String ip = (String)enm.nextElement();
            DBCall cmd = new IPAddressDeleteCommand(logger, lock, ip, op_uid);
            checkErrors("Failed to delete ip address "+ip, cmd.processResultSet(con.callDB(cmd)));
        }

        // insert dhcp_ips_to_insert
        Hashtable accepted_ip_addresses = new Hashtable();
        enm = dhcp_ips_to_insert.keys();
        while( enm.hasMoreElements() ) {
            String ip = (String)enm.nextElement();
            // first reserve it
            Hashtable reserveResult = ((NetDBii)logger).reserveIP(con, null, ip, 1);
            result = (Hashtable)reserveResult.get(reserveResult.keys().nextElement());
            checkErrors("Failed to reserve ip address "+ip, result);
            Hashtable ip_addresses = Utils.nHash(result.get("ip_addresses"));
            if( ip_addresses.size() > 0 ) {
                String ipStr = ((String)ip_addresses.keys().nextElement()).trim();
                DBCall cmd = new IPAddressAcceptCommand(logger, ipStr, (Boolean)dhcp_ips_to_insert.get(ip),
                                                        idAS, lock,
                                                        IPAddressAcceptCommand.ADDR_SPACE_TYPE,
                                                        System.getProperty("DefaultPTRpref", "0"),
                                                        op_uid);
                checkErrors("Failed to insert ip address "+ip, cmd.processResultSet(con.callDB(cmd)));
                accepted_ip_addresses.put(ipStr, "");
            }

        }

        // update dhcp_ips_to_update
        enm = dhcp_ips_to_update.keys();
        while( enm.hasMoreElements() ) {
            String ip = (String)enm.nextElement();
            DBCall cmd = new IPAddressUpdateCommand(logger, lock, ip,
                                                    (Boolean)dhcp_ips_to_update.get(ip),
                                                    System.getProperty("DefaultPTRpref", "0"),
                                                    op_uid);
            checkErrors("Failed to update ip address "+ip,
                        cmd.processResultSet(con.callDB(cmd)));
        }

        Hashtable retval = new Hashtable();
        retval.put("accepted_ip_addresses", accepted_ip_addresses);
        return retval;
    } // end of doIt(...)
}
