/************************************************************************
 *
 * CheckGroupAccessCommand.java - db call NetDB_Owner.check_access
 *
 * $Id: CheckGroupAccessCommand.java,v 1.1.1.1 2006/06/30 23:38:08 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.lang.*;
import java.util.*;
import java.sql.*;

/** db call NetDB_Owner.check_access_mw */
class CheckGroupAccessCommand extends DBCall {

    public static final int ID_OP_USER_INDEX        =  1; // p_user_id
    public static final int RECORD_ID_INDEX         =  2; // p_record_id
    public static final int TYPE_INDEX              =  3; // p_record_type

    /**
     * @param object - Hashtable representing some NetDB object
     * @param op_uid - id of the operating user.
     */
    public CheckGroupAccessCommand(stanford.netdb.utils.Logger logger, Hashtable object, Integer op_uid)
        throws Exception, InternalException, IllegalArgumentException
    {
        super(logger);
        values                          = new Object[4];
        values[0]                       = new OutParam(Types.NUMERIC);
        values[ID_OP_USER_INDEX]        = op_uid;
        values[RECORD_ID_INDEX]         = new Integer(object.get("id").toString());

        // weak attempt to validate object
        Object typeO = object.get("object");
        if( typeO == null || !(typeO instanceof String) )
            throw new IllegalArgumentException("Invalid object passed to "+
                             "CheckGroupAccessCommand constructor");

        String type = (String)typeO;
        if (      type.equals("group") ) values[TYPE_INDEX] = "owner";
        else if ( type.equals("ateam") ) values[TYPE_INDEX] = "adminteam";
        else                             values[TYPE_INDEX] = type;

        db_call_type = SIMPLE_PROC_CALL;
    }

    public String getSQL() {
        return proc_call_sql("$(SCHEMA)Util.to_int($(SCHEMA)NetDB_Owner.check_access");
    }
}
