/************************************************************************
 *
 * ConsultantCommand.java - db call NetDB_Organization.consultant
 *
 * $Id: ConsultantCommand.java,v 1.1.1.1 2006/06/30 23:38:08 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.lang.*;
import java.sql.*;

/** db call NetDB_Organization.consultant */
class ConsultantCommand extends DBCall {

    public static final int ACTION_INDEX     = 0; // p_action
    public static final int ID_OP_USER_INDEX = 1; // p_op_uid
    public static final int PERSON_ID_INDEX  = 2; // p_person_id
    public static final int ORG_ID_INDEX     = 3; // p_org_id
    public static final int ORG_NAME_INDEX   = 4; // p_org_name
    public static final int PRIMARY_INDEX    = 5; // p_primary

    public static final String INSERT_ACTION = "insert";
    public static final String DELETE_ACTION = "delete";
    public static final String UPDATE_ACTION = "update";

    public ConsultantCommand(stanford.netdb.utils.Logger logger, String action,
                             Integer op_uid, Integer depID, Integer consultant_id,
                             Boolean primary) {
        super(logger);

        db_call_type = ORA_PROC_CALL;
        values       = new Object[6];

        values[ACTION_INDEX]      = o2v(action);
        values[ID_OP_USER_INDEX]  = o2n(op_uid, Types.NUMERIC);
        values[PERSON_ID_INDEX]   = o2n(consultant_id, Types.NUMERIC);
        values[ORG_ID_INDEX]      = o2n(depID, Types.NUMERIC);
        values[ORG_NAME_INDEX]    = new Null(Types.VARCHAR);
        if( primary == null )
            values[PRIMARY_INDEX] = new Null(Types.TINYINT);
        else
            values[PRIMARY_INDEX] = new Short(primary.booleanValue()?"1":"0");
    }

    public String getSQL() {
        return proc_call_sql("NetDB_Organization.consultant");
    }
}
