/************************************************************************
 *
 * DHCPOptionsActionCommand.java - db call NetDB_DHCPoption.add|remove
 *
 * $Id: DHCPOptionsActionCommand.java,v 1.2 2006/09/26 17:27:04 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.lang.*;
import java.util.*;
import java.sql.*;

/** db call NetDB_DHCPoption.add|remove */
class DHCPOptionsActionCommand extends DBCall implements ComplexDBCall {

    public static final String INSERT_ACTION = "add";
    public static final String DELETE_ACTION = "remove";

    public static final int ID_OP_USER_INDEX        = 0; // p_op_uid
    public static final int LOCK_INDEX              = 1; // p_lock
    public static final int RECORD_ID_INDEX         = 2; // p_record_id
    public static final int RECORD_TYPE_INDEX       = 3; // p_record_type
    public static final int DHCP_OPTION_ID_INDEX    = 4; // p_option_id
    public static final int DHCP_OPTION_NAME_INDEX  = 5; // p_option_label
    public static final int DHCP_OPTION_VALUE_INDEX = 6; // p_option_value

    private Hashtable options;
    private String action;

    public DHCPOptionsActionCommand(stanford.netdb.utils.Logger logger,
                                    String action, Timestamp lock, Integer objID,
                                    int objType, Hashtable options, Integer op_uid)
        throws Exception
    {
        super(logger);
        this.options = options;
        this.action  = action;
        db_call_type = ORA_PROC_CALL;

        values = new Object[action.equals(INSERT_ACTION)?7:6];

        values[ID_OP_USER_INDEX]     = op_uid;
        values[LOCK_INDEX]           = lock;
        values[RECORD_ID_INDEX]      = objID;
        values[DHCP_OPTION_ID_INDEX] = new Null(Types.NUMERIC);

        switch(objType) {
        case Constants.IPADDRSPACE:
            values[RECORD_TYPE_INDEX] = "addressspace"; break;

        case Constants.NETWORK:
            values[RECORD_TYPE_INDEX] = "network";      break;

        case Constants.INTERFACE:
            values[RECORD_TYPE_INDEX] = "interface";    break;

        case Constants.DHCP_SERVICE:
            values[RECORD_TYPE_INDEX] = "dhcpservice";  break;

        default:
            throw new Exception("Invalid object type passed to DHCPOptionsActionCommand(...)");
        }

    }

    public Hashtable doIt(NetDBConnection con) throws NetDBException, Exception {
        Enumeration enm = options.keys();
        logger.debug(action+" DHCP Options");
        while( enm.hasMoreElements() ) {
            Hashtable optionH = (Hashtable)options.get(enm.nextElement());
            String option = (String)optionH.get("option");
            String value = (String)optionH.get("value");
            values[DHCP_OPTION_NAME_INDEX]  = option;
            if( action.equals(INSERT_ACTION) )
                values[DHCP_OPTION_VALUE_INDEX] = o2v(value);

            logger.debug(action+" dhcp option "+option+"="+value);
            Hashtable result = con.callDB(this);
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;
        }
        return new Hashtable();
    }

    public String getSQL() {
        return proc_call_sql("NetDB_DHCPoption."+action);
    }
}
