/************************************************************************
 *
 * DepartmentInsertCommand.java - db call NetDB_Organization.master 'update'
 *
 * $Id: DepartmentInsertCommand.java,v 1.3 2006/09/26 17:29:54 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.util.*;
import java.lang.*;
import java.sql.*;

/** db call NetDB_Organization.master 'update' */
class DepartmentInsertCommand extends DepartmentCommand implements ComplexDBCall {

    public Hashtable newDep;
    public Integer op_uid;

    public DepartmentInsertCommand(stanford.netdb.utils.Logger logger, Hashtable newDep, Integer op_uid) {
        super(logger);
        this.newDep = newDep;
        this.op_uid = op_uid;

        values[ACTION_INDEX]            = "insert";
        values[ID_OP_USER_INDEX]        = op_uid;
        values[DEPARTMENT_NAME_INDEX]   = newDep.get("name");
        values[DEPARTMENT_REG_ID_INDEX] = o2v(newDep.get("RegID"));
    }

    public Hashtable doIt(NetDBConnection con) throws NetDBException, Exception {
        BeginTransaction(con.con);
        boolean commit_finally = false;

        try {
            logger.info("inserting department "+values[DEPARTMENT_NAME_INDEX]);
            logger.debug("do sigle value attributes");
            Hashtable result = con.callDB(this);

            // check for errors
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            Integer id = new Integer((String)result.get("id"));

            // insert new consultants
            logger.debug("inserting new consultants");
            Hashtable cons = (Hashtable)newDep.get("consultants");
            if( cons == null ) cons =  new Hashtable();
            Enumeration enm = cons.keys();
            while( enm.hasMoreElements() ) {
                String key = (String)enm.nextElement();
                Hashtable person = (Hashtable)cons.get(key);
                ConsultantCommand cmd =
                    new ConsultantCommand(logger, ConsultantCommand.INSERT_ACTION, op_uid,
                                          id, new Integer((String)person.get("id")),
                                          (Boolean)person.get("primary?"));
                logger.debug((String)person.get("RegID")+" "+(String)person.get("NetID"));
                result = cmd.processResultSet(con.callDB(cmd));

                // check for errors
                error_set = (Vector)result.get("errors");
                if( error_set != null && error_set.size() > 0 )
                    return result;
            }

            commit_finally = true;

        } catch(Exception ex) {
            ex.printStackTrace();
            try {
                logger.debug("Rollback Transaction");
                RollbackTransaction(con.con);
            } catch(Exception exx) { }
            throw ex;

        } finally {
            if( commit_finally )
                try {
                    logger.info("inserting department "+values[DEPARTMENT_NAME_INDEX]+" done");
                    CommitTransaction(con.con);
                } catch(Exception exx) { }
            else
                try {
                    logger.info("inserting department "+values[DEPARTMENT_NAME_INDEX]+" failed");
                    logger.debug("Rollback Transaction");
                    RollbackTransaction(con.con);
                } catch(Exception exx) { }
        }
        return new Hashtable();
    }

    public int errorResultSetIndex() {
        return 1;
    }
}
