/************************************************************************
 *
 * DepartmentModifyCommand.java - db call NetDB_Organization.master 'update'
 *
 * $Id: DepartmentModifyCommand.java,v 1.3 2006/09/26 17:30:03 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.util.*;
import java.lang.*;

/** db call NetDB_Organization.master 'update' */
class DepartmentModifyCommand extends DepartmentCommand implements ComplexDBCall {

    private static final int MODIFY_ACTION = 1;
    private static final int REPLACE_ACTION = 2;

    public Hashtable oldDep;
    public Hashtable newDep;
    public Integer op_uid;
    public int action;

    public DepartmentModifyCommand(stanford.netdb.utils.Logger logger,
                                   Hashtable oldDep, Hashtable newDep,
                                   Integer op_uid)
    {
        super(logger);
        db_call_type = ORA_FUN_CALL;
        this.oldDep  = oldDep;
        this.newDep  = newDep;
        this.op_uid  = op_uid;

        String oldID = (String)oldDep.get("id");
        String newID = (String)newDep.get("id");
        if( !oldID.equals(newID) ) {
            values[ACTION_INDEX] = "replace";
            action = REPLACE_ACTION;
        }
        else {
            values[ACTION_INDEX] = "update";
            action = MODIFY_ACTION;
        }

        values[ID_OP_USER_INDEX]          = op_uid;
        values[DEPARTMENT_NAME_INDEX]     = oldDep.get("name");
        values[DEPARTMENT_REG_ID_INDEX]   = o2v(newDep.get("RegID"));
        values[DEPARTMENT_ID_INDEX]       = new Integer(oldID);
        values[DEPARTMENT_NEW_NAME_INDEX] = newDep.get("name");
    }

    public Hashtable doIt(NetDBConnection con) throws NetDBException, Exception {
        BeginTransaction(con.con);
        boolean commit_finally = false;

        try {
            // delete old consultants
            logger.info("updating department "+values[DEPARTMENT_NAME_INDEX]);
            logger.debug("deleting old consultants");
            Hashtable cons = Utils.nHash(oldDep.get("consultants"));
            Enumeration enm = cons.keys();
            while( enm.hasMoreElements() ) {
                String key = (String)enm.nextElement();
                Hashtable person = (Hashtable)cons.get(key);
                ConsultantCommand cmd =
                    new ConsultantCommand(logger, ConsultantCommand.DELETE_ACTION, op_uid,
                                          new Integer((String)oldDep.get("id")),
                                          new Integer((String)person.get("id")),
                                          (Boolean)person.get("primary?"));
                logger.debug((String)person.get("RegID")+" "+(String)person.get("NetID"));
                Hashtable result = cmd.processResultSet(con.callDB(cmd));

                // check for errors
                Vector error_set = (Vector)result.get("errors");
                if( error_set != null && error_set.size() > 0 )
                    return result;
            }

            logger.debug("do sigle value attributes");
            Hashtable result = con.callDB(this);

            // check for errors
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            if( ((String)Utils.n2s(result.get("action"))).equals("replace") )
                action = REPLACE_ACTION;

            if( action == MODIFY_ACTION ) {
                // insert new consultants
                logger.debug("inserting new consultants");
                cons = (Hashtable)newDep.get("consultants");
                if( cons == null ) cons =  new Hashtable();
                enm = cons.keys();
                while( enm.hasMoreElements() ) {
                    String key = (String)enm.nextElement();
                    Hashtable person = (Hashtable)cons.get(key);
                    ConsultantCommand cmd =
                        new ConsultantCommand(logger, ConsultantCommand.INSERT_ACTION, op_uid,
                                              new Integer((String)newDep.get("id")),
                                              new Integer((String)person.get("id")),
                                              (Boolean)person.get("primary?"));
                    logger.debug((String)person.get("RegID")+" "+(String)person.get("NetID"));
                    result = cmd.processResultSet(con.callDB(cmd));

                    // check for errors
                    error_set = (Vector)result.get("errors");
                    if( error_set != null && error_set.size() > 0 )
                        return result;
                }
            }
            commit_finally = true;

        } catch(Exception ex) {
            ex.printStackTrace();
            try {
                logger.debug("Rollback Transaction");
                RollbackTransaction(con.con);
            } catch(Exception exx) { }
            throw ex;

        } finally {
            if( commit_finally )
                try {
                    logger.info("updating department "+values[DEPARTMENT_NAME_INDEX]+" done");
                    CommitTransaction(con.con);
                } catch(Exception exx) { }
            else
                try {
                    logger.info("updating department "+values[DEPARTMENT_NAME_INDEX]+" failed");
                    logger.debug("Rollback Transaction");
                    RollbackTransaction(con.con);
                } catch(Exception exx) { }
        }
        return new Hashtable();
    }

    public int errorResultSetIndex() {
        return 1;
    }
}
