/************************************************************************
 *
 * DomainModifyCommand.java - db call pDomain 'update'
 *
 * $Id: DomainModifyCommand.java,v 1.3 2006/10/25 04:19:20 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.util.*;
import java.lang.*;
import java.sql.*;

/** db call pDomain 'update' */
class DomainModifyCommand extends DomainCommand implements ComplexDBCall {

    private Hashtable oldDomain;
    private Hashtable newDomain;
    private int id;
    private String action;

    public DomainModifyCommand(stanford.netdb.utils.Logger logger,
                               Hashtable oldDomain, Hashtable newDomain,
                               Integer op_uid) {
        super(logger);

        Integer id_domain = null;
        if( oldDomain != null ) {
            id_domain = new Integer((String)oldDomain.get("id"));
            id = id_domain.intValue();
            values[ID_INDEX]       = id_domain;
            action                 = "update";
            values[NEW_NAME_INDEX] = newDomain.get("domain");
        } else {
            db_call_type       = ORA_FUN_CALL;
            oldDomain          = Utils.hash("groups", new Hashtable());
            action             = "insert";
            values[NAME_INDEX] = newDomain.get("domain");
        }

        this.oldDomain            = oldDomain;
        this.newDomain            = newDomain;

        values[ACTION_INDEX]      = action;
        values[ID_OP_USER_INDEX]  = op_uid;

        values[COMMENT_INDEX]     = newDomain.get("comment");
        if( values[COMMENT_INDEX] == null )
            values[COMMENT_INDEX] = "";
        values[LIMITED_INDEX]     =
            new Short( ((Boolean)newDomain.get("limited?")).booleanValue()?"1":"0" );

        lock = (Timestamp)newDomain.get("locked");
        values[LOCKED_INDEX] = o2n(lock, Types.TIMESTAMP);
    }

    public Hashtable doIt(NetDBConnection con) throws NetDBException, Exception {
        BeginTransaction(con.con);
        boolean commit_finally = false;
        try {
            logger.info("modifying domain "+values[NAME_INDEX]);
            logger.debug("do sigle value attributes");
            printSQL();
            Hashtable result = con.callDB(this);

            // check for errors
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            if( action.equals("insert") ) {
                id   = Integer.parseInt((String)result.get("id"));
                lock = (Timestamp)result.get("locked");

                // now accept domain name
                DBCall call = new NameAcceptCommand(logger,
                                                    new Integer(id), "domain",
                                                    new Integer(id), null,
                                                    lock, null,
                                                    (Integer)values[ID_OP_USER_INDEX]);
                result = con.callDB(call);

                // check for errors
                error_set = (Vector)result.get("errors");
                if( error_set != null && error_set.size() > 0 )
                    return result;
            }

            // domain related groups
            result = update_domain_groups(con,
                                          Utils.nHash(oldDomain.get("useAsName")),
                                          Utils.nHash(newDomain.get("useAsName")),
                                          ObjectGroupCommand.DOMAIN_USE);

            if( result != null ) return result;

            result = update_domain_groups(con,
                                          Utils.nHash(oldDomain.get("assign")),
                                          Utils.nHash(newDomain.get("assign")),
                                          ObjectGroupCommand.DOMAIN_ASSIGN);

            if( result != null ) return result;

            // now groups
            Hashtable newGroups = (Hashtable)newDomain.get("groups");
            Hashtable oldGroups = (Hashtable)oldDomain.get("groups");
            Vector groups_to_insert = new Vector();
            Vector groups_to_delete = new Vector();
            Vector groups_to_keep = new Vector();
            Utils.keysIntersect(oldGroups, newGroups, groups_to_insert,
                                groups_to_delete, groups_to_keep);

            // insert groups_to_insert
            logger.debug("inserting groups_to_insert");
            ObjectGroupCommand cmd =
                new ObjectGroupCommand(logger, ObjectGroupCommand.ADD_GROUPS,
                                       id, ObjectGroupCommand.DOMAIN,
                                       lock, groups_to_insert,
                                       (Integer)values[ID_OP_USER_INDEX]);
            result = cmd.doIt(con);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            // delete groups_to_delete
            logger.debug("deleting groups_to_delete");
            cmd =
                new ObjectGroupCommand(logger, ObjectGroupCommand.REMOVE_GROUPS,
                                       id, ObjectGroupCommand.DOMAIN,
                                       lock, groups_to_delete,
                                       (Integer)values[ID_OP_USER_INDEX]);
            result = cmd.doIt(con);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            // now do check
            logger.debug("do check");

            // values[0] has to be reset here becouse last call
            // filled up this with some value
            values[0]            = newRefCursor();
            db_call_type         = ORA_FUN_IGNORE_RET_CALL;
            values[ACTION_INDEX] = "verify";
            result = con.callDB(this);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            // unlock domain
            DBCall call = new DomainLockUnLockCommand(logger, DomainLockUnLockCommand.UNLOCK_ACTION, null,
                                                      new Integer(id), lock, (Integer)values[ID_OP_USER_INDEX]);
            result = call.processResultSet(con.callDB(call));

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            ((NetDBii)logger).unregister_lock(lock, new Integer(Constants.DOMAIN));

            commit_finally = true;

        } catch(Exception ex) {
            ex.printStackTrace();
            try {
                logger.debug("Rollback Transaction");
                RollbackTransaction(con.con);
            } catch(Exception exx) { }
            throw ex;

        } finally {
            if( commit_finally )
                try {
                    logger.info("modifying domain "+values[NAME_INDEX]+" done");
                    CommitTransaction(con.con);
                } catch(Exception exx) { }
            else
                try {
                    logger.info("modifying domain "+values[NAME_INDEX]+" failed");
                    logger.debug("Rollback Transaction");
                    RollbackTransaction(con.con);
                } catch(Exception exx) { }
        }
        return new Hashtable();
    }

    private Hashtable update_domain_groups(NetDBConnection con, Hashtable oldGroups,
                                           Hashtable newGroups, String which)
        throws NetDBException, InternalException, Exception
    {
        Vector groups_to_insert = new Vector();
        Vector groups_to_delete = new Vector();
        Vector groups_to_keep = new Vector();
        Utils.keysIntersect(oldGroups, newGroups, groups_to_insert,
                            groups_to_delete, groups_to_keep);

        // insert groups_to_insert
        logger.debug("inserting groups_to_insert");
        ObjectGroupCommand cmd =
            new ObjectGroupCommand(logger, ObjectGroupCommand.ADD_GROUPS,
                                   id, which, lock, groups_to_insert,
                                   (Integer)values[ID_OP_USER_INDEX]);

        Hashtable result = cmd.doIt(con);

        // check for errors
        Vector error_set = (Vector)result.get("errors");
        if( error_set != null && error_set.size() > 0 )
            return result;

        // delete groups_to_delete
        logger.debug("deleting groups_to_delete");
        cmd =
            new ObjectGroupCommand(logger, ObjectGroupCommand.REMOVE_GROUPS,
                                   id, which, lock, groups_to_delete,
                                   (Integer)values[ID_OP_USER_INDEX]);

        result = cmd.doIt(con);

        // check for errors
        error_set = (Vector)result.get("errors");
        if( error_set != null && error_set.size() > 0 )
            return result;

        return null;
    }
}
