/************************************************************************
 *
 * ExpressionParser.java -
 * Parses the search string and generates the SQL clause
 * Method to Access : generateSQLclause(sf,sv)
 * Valid SearchString seperators:
 * 1.AND(case insensitive)
 * 2.OR(       "         )
 * 3.space = OR
 * 4.carriage return = OR
 * 5.,=OR
 * 6.Quoted Strings
 * 7.Embedded quotes
 * 8.or not = OR NOT (     "        )
 * 9.not = AND NOT(case insensitive)
 * 10.or not not = OR NOT NOT
 * 11.NOT NOT = AND NOT NOT
 * 12. AND NOT
 * 13. AND NOT NOT
 *
 * Some field names are hardcoded and that part of the code will be replaced to get them dynamically
 *
 * $Id: ExpressionParser.java,v 1.1.1.1 2006/06/30 23:38:09 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import java.util.*;
import java.io.*;

class ExpressionParser{

    public ExpressionParser(){ }

    private String sfield_p;
    private String svalue_p;
    public String err_msg=new String("");

    public void setSfield(String s){sfield_p=s;}
    public void setSvalue(String s){svalue_p=s;}
    public String getSfield(){return sfield_p;}
    public String getSvalue(){return svalue_p;}
    public void setErrorMsg(String err) {err_msg=err;}
    public String getErrorMsg(){return err_msg;}

    public ExpressionParser(String sfield33,String svalue33){setSfield(sfield33);setSvalue(svalue33);}

    /*Check for IPaddress validity
     */
    public int isValidIPSearchString(String svalue) {

        int rval=0;
        if (svalue.indexOf('/') != -1){
            if(svalue.indexOf('[') != -1 || svalue.indexOf(']') != -1)
                rval=-1;
            if (svalue.indexOf('/') != svalue.lastIndexOf('/'))
                rval=-1;
            else {
                int zz = Integer.valueOf(svalue.substring(svalue.indexOf('/')+1,svalue.length())).intValue();
                if (zz > 32 || zz <= 0)
                    rval=-1;
            }
        }
        else if(svalue.indexOf('[') != -1 && svalue.indexOf(']') != -1 && svalue.indexOf("-") != -1)
            rval=-1;
        return rval;
    }

    /*Checks whether the date entered is a range*/
    public int isDateRange(String svalue) {
        int rval=-1;
        if (svalue.indexOf(" - ") != -1)
            rval=1;
        else
            rval=2;
        return rval;
    }

    /*Process the date value if it is a range*/
    public String evaluateDateRange(String sfield,String svalue,int i) {
        String retval="";
        if (i==1)
            retval= sfield+" between "+"'"+svalue.substring(0,svalue.indexOf(" - "))+"'"+" and "+"'"+ svalue.substring(svalue.indexOf(" - ")+3,svalue.length())+"'";
        if (i==2){
            if (svalue.indexOf(">") != -1){
                retval= sfield+" >= "+"'"+svalue.substring(0,svalue.indexOf(">"))+"'";
                System.out.println("Return val for date >:"+sfield+":"+":"+retval+":"+i);
            }
            else if(svalue.indexOf("<") != -1) {
                retval= sfield+" <= "+"'"+svalue.substring(0,svalue.indexOf("<"))+"'";
                System.out.println("Return val for date <:"+sfield+":"+":"+retval+":"+i);
            }
        }
        return retval;

    }


    /*Validates the ipaddress to determine whether it is a range*/
    public int isIPRange(String svalue) {
        int z=-1;
        int ndots=0;
        String sdots="";
        int i=isValidIPSearchString(svalue);
        if (i==0){
            if (svalue.indexOf('/') != -1)
                z=1;
            else if(svalue.indexOf('/') == -1 && svalue.indexOf('[') == -1 && svalue.indexOf(']') == -1 && svalue.indexOf('-') != -1)
                z=2;
            else if (svalue.indexOf('/') == -1 && svalue.indexOf('.') != -1) {
                sdots=svalue;
                while(sdots.indexOf('.') != -1)    {
                    ndots = ndots+1;
                    sdots=sdots.substring(sdots.indexOf('.')+1,sdots.length());
                }
                if (ndots == 2 && (svalue.indexOf('*') == -1 && svalue.indexOf('%') == -1) && (svalue.indexOf('[') == -1 || svalue.indexOf(']') == -1) )
                    z=3;
                else if (ndots == 3 && (svalue.indexOf('*') == -1 && svalue.indexOf('%') == -1) && (svalue.indexOf('[') == -1 || svalue.indexOf(']') == -1) && svalue.length() == svalue.lastIndexOf('.')+1)
                    z=4;
                else if (ndots == 3 && (svalue.indexOf('*') == -1 && svalue.indexOf('%') == -1) && (svalue.indexOf('[') == -1 || svalue.indexOf(']') == -1) && svalue.length() > svalue.lastIndexOf('.')+1)
                    z=5;
            }

        }
        else{
            /*the input is not a valid ipaddress*/
            System.out.println("BOGUS ADDRESS:"+svalue);
            z = -999;
        }
        return z;
    }

    /*Process the ipaddress to make out if it is a range or a string*/
    public String evaluateIPRange(String sfield,String svalue,int i){
        String sdots="";
        String retval="";
        if ( i == 1 ){
            sdots=svalue.substring(0,svalue.indexOf('/'));
            if (svalue.indexOf('.') == svalue.lastIndexOf('.'))
                sdots = sdots + ".";
            int zz = Integer.valueOf(svalue.substring(svalue.indexOf('/')+1,svalue.length())).intValue();
            zz = 32 - zz;
            String low1  =  iptostr(strtoip(sdots) & bitstomask(zz));
            String high1 = iptostr(strtoip(low1) | ~bitstomask(zz));
            String low2 = String.valueOf(strtoip(low1));
            String high2 = String.valueOf(strtoip(high1));
            sfield="ipAddress";
            retval=sfield+" between "+low2+" and "+high2;
        }
        else if( i == 2 ){
            String low1=String.valueOf(strtoip(svalue.substring(0,svalue.indexOf('-'))));
            String high1=String.valueOf(strtoip(svalue.substring(svalue.indexOf('-')+1,svalue.length()).trim()));
            sfield="ipAddress";
            retval=sfield+" between "+low1+" and "+high1;
        }
        else if( i == 3 )
            retval="'"+svalue+".0"+"'";
        else if( i == 4 ){
            System.out.println("<< i in 4 = ["+String.valueOf(i)+"]>>");
            String low1=String.valueOf(strtoip(svalue));
            sfield="ipAddress";
            retval=sfield+" between "+low1+" and "+low1;
            //retval="'"+svalue+"0"+"'";
        }
        else if( i == 5){
            if( sfield.trim().equalsIgnoreCase("strPrefix")){
                sfield="ipAddress";
                String low1=String.valueOf(strtoip(svalue.substring(0,svalue.lastIndexOf("."))));
                String high1=String.valueOf(strtoip(svalue));
                retval=sfield+" between "+low1+" and "+high1;
            }
            else{
                sfield="ipAddress";
                String low1=String.valueOf(strtoip(svalue));
                retval=sfield+" between "+low1+" and "+low1;
            }
        }


        return retval;

    }


    /*Replace these tokens with */
    public Vector SQLClauseChars(String sfield){
        // All the code that is commented was used when space( ) ,comma(,)
        // and parenthesis("(" ")") were operators.Not deleting this code
        // because fullsearch might take a "U" turn soon

        Vector v=new Vector();
        v.addElement("'and''and'"); //1
        v.addElement("'or''or'");//2
        v.addElement("' '' '");//3
        v.addElement("'or'' ''not'");//4
        v.addElement("'and''or'");//5
        v.addElement("'or''and'");//6
        //      v.addElement("'and'','");//7
        //      v.addElement("',''and'");//8
        //     v.addElement("'or'");//9
        //      v.addElement("' '' '");//10
        //      v.addElement("'('' '");//11
        //     v.addElement("' '')'");//12
        v.addElement("'not'' '");//13
        v.addElement("' ''not'");//14
        v.addElement("'not''and'");//15
        //    v.addElement("' '");//16
        //      v.addElement("','','");//17
        v.addElement("' ''and'' '");//18
        v.addElement("' ''and'");//19
        v.addElement("' ''or'");//46
        v.addElement("'or'' '");//47
        v.addElement("'not''or'");//20
        v.addElement("'and''not''not'");//21
        v.addElement("'and''not'");//22
        v.addElement("'or''not''not'");//23
        v.addElement("'or''not'");//24
        v.addElement("'not''not'");//25
        v.addElement("'not'");//26
        v.addElement("' '");//16
        v.addElement("'and'");//27
        v.addElement("'or'");//28
        v.addElement("' ornotnot '");//29
        v.addElement("' andnotnot '");//30
        v.addElement("' ornot '");//31
        v.addElement(sfield+" like "+"'(''not'"+"'not'");//32
        v.addElement(sfield+" like "+"'not'"+"'not'");//33
        v.addElement(sfield+" like "+"'(''not'");//34
        v.addElement(sfield+" like "+"'not'");//35
        v.addElement("' andnot '");//36
        v.addElement(sfield+" like '('");//37
        v.addElement(sfield+" not like '('");//38
        //  v.addElement("')'");//39
        //   v.addElement("' ) '");//40
        v.addElement(sfield+" like  and");//41
        v.addElement(sfield+" like "+"ipAddress"+" between ");//42
        v.addElement(sfield+" like "+sfield+" between ");//43
        v.addElement(sfield+" like "+sfield+" >= ");//44
        v.addElement(sfield+" like "+sfield+" <= ");//45


        return v;
    }

    /*Replace SQLClauseChars(String sfield) with SQLClauseReplaceCharsWith(String sfield) */
    public Vector SQLClauseReplaceCharsWith(String sfield,String viewName,String idColumn){
        Vector v=new Vector();

        v.addElement("'and'");//1
        v.addElement("'or'");//2
        v.addElement("' '");//3
        v.addElement("'or''not'");//4
        v.addElement("'or'");//5
        v.addElement("'or'");//6
        //      v.addElement("','");//7
        //      v.addElement("','");//8
        //   v.addElement("' '");//9
        //      v.addElement("' '");//10
        //    v.addElement("'('");//11
        //   v.addElement("')'");//12
        v.addElement("'not'");//13
        v.addElement("'not'");//14
        v.addElement("'not'");//15
        // v.addElement("+ ' '");//16 changed
        //      v.addElement("','");//17
        v.addElement("'and'");//18
        v.addElement("'and'");//19
        v.addElement("'or'");//46
        v.addElement("'or'");//47
        v.addElement("'not'");//20
        v.addElement("' andnotnot '");//21
        v.addElement("' andnot '");//22
        v.addElement("' ornotnot '");//23
        v.addElement("' ornot '");//24
        v.addElement("' andnotnot '");//25
        v.addElement("' andnot '");//26
        v.addElement("+"+String.valueOf('"')+" "+String.valueOf('"') + "+");//16 changed
        //v.addElement(" and "+sfield+" like ");
        v.addElement(" and "+viewName+"."+idColumn+" in (Select "+ idColumn+" from "+viewName+" where "+sfield+" like ");//27
        v.addElement(" Or "+sfield+" like ");//28
        v.addElement(" or "+" not "+sfield+" not "+"like ");//29
        v.addElement(" and "+" not "+sfield+" not "+"like ");//30
        v.addElement(" Or "+sfield+" not "+" like ");//31
        v.addElement("(not "+sfield+" not "+"like ");//32
        v.addElement("not "+sfield+" not "+"like ");//33
        v.addElement("("+sfield+" not "+"like ");//34
        v.addElement(sfield+" not "+"like ");//35
        // v.addElement(" and "+sfield+" not "+" like ");
        v.addElement(" and "+viewName+"."+idColumn +" not in(Select "+idColumn+" from "+viewName+" where "+sfield+" like ");//36
        v.addElement("("+sfield+" like ");//37
        v.addElement("("+sfield+" not like ");//38
        //   v.addElement("' ) '");//39
        // v.addElement(")");//40
        v.addElement("");//41
        v.addElement("ipAddress"+" between ");//42
        v.addElement(sfield+" between ");//43
        v.addElement(sfield+" >= ");//44
        v.addElement(sfield+" <= ");//45

        return v;
    }

    //Generates SQL clause with appropriate replacement of the tokens
    // Don't remember the reason for this name (Maybe I have to replace this)
    public StringBuffer replaceSQLclauseChars(StringBuffer sbuffer1,String sfield,String viewName,String idColumn){
        Vector s1 = SQLClauseChars(sfield);
        Vector repls = SQLClauseReplaceCharsWith(sfield,viewName,idColumn);
        String st4="";
        String st5="";
        SBreplace sbr = new SBreplace();
        try {
            for (int i=0;i<s1.size();i++) {
                st4=(String)s1.elementAt(i);
                st5=(String)repls.elementAt(i);
                while (sbuffer1.toString().indexOf(st4) != -1) {
                    sbuffer1 = sbr.replace(sbuffer1.toString().indexOf(st4),sbuffer1.toString().indexOf(st4) + st4.length(),st5,sbuffer1);
                }
            }
        } catch(NoSuchElementException e){
            e.printStackTrace();}
        return sbuffer1;
    }

    //Split the search string into tokens
    public String TokenizeSvalue(String sfield1,String svalue1,String isSubstring,String shouldBeLowered,String sValueType) {
        String st2="";
        String st3="";
        String st2_cp="";
        int idx=0;

        svalue1=svalue1.trim();
        StringBuffer sbuffer1 = new StringBuffer(svalue1);
        SBreplace sbr=new SBreplace();

        //if search field is address get rid of the spaces between numbers(space is a separator)
        if(sValueType.equalsIgnoreCase("Address")) {
            while(sbuffer1.toString().indexOf("  ") != -1){
                sbuffer1 = sbr.replace(sbuffer1.toString().indexOf("  "),sbuffer1.toString().indexOf("  ") + "  ".length()," ",sbuffer1);
            }
            while(sbuffer1.toString().indexOf("--") != -1) {
                sbuffer1 = sbr.replace(sbuffer1.toString().indexOf("--"),sbuffer1.toString().indexOf("--") + "--".length(),"-",sbuffer1);
            }
            while(sbuffer1.toString().indexOf(" -") != -1) {
                sbuffer1 = sbr.replace(sbuffer1.toString().indexOf(" -"),sbuffer1.toString().indexOf(" -") + " -".length(),"-",sbuffer1);
            }
            while(sbuffer1.toString().indexOf("- ") != -1) {
                sbuffer1 = sbr.replace(sbuffer1.toString().indexOf("- "),sbuffer1.toString().indexOf("- ") + "- ".length(),"-",sbuffer1);
            }

            svalue1=sbuffer1.toString();
        }

        //if search field is a text replace embedded quotes with special words
        // '' with DSQ (double single quotes)
        //' with SQ (single quote)
        // " with DQ  (double quotes)
        // "" with DDQ (double doble quotes)

        else {
            if (sbuffer1.toString().indexOf("''") != -1){
                while (sbuffer1.toString().indexOf("''") != -1)   {
                    sbuffer1=sbr.replace(sbuffer1.toString().indexOf("''"),sbuffer1.toString().indexOf("''") + "''".length(),"<DSQ>",sbuffer1);
                }
            }
            svalue1=sbuffer1.toString();
            StringBuffer sbuffer2=new StringBuffer(svalue1);
            int ich=0;
            int ich1=0;
            if (sbuffer2.toString().indexOf("'") != -1) {
                try{
                    while (sbuffer2.toString().indexOf("'") != -1) {
                        if (sbuffer2.toString().indexOf("'") != 0 && sbuffer2.toString().indexOf("'") != sbuffer2.toString().length()) {
                            Character cc=new Character(sbuffer2.toString().charAt(sbuffer2.toString().indexOf("'")-1));
                            Character cc1=new Character(sbuffer2.toString().charAt(sbuffer2.toString().indexOf("'")+1));
                            ich = cc.hashCode();
                            ich1 = cc1.hashCode();
                            if (((ich==33) || (ich>=35 && ich<=38) || (ich==43) || (ich==45)|| (ich>=46 && ich<=122)) && ((ich1==33) || (ich1>=35 && ich1<=38) || (ich1==43) || (ich1==45)|| (ich1>=46 && ich1<=122))){
                                sbuffer1=sbr.replace(sbuffer2.toString().indexOf("'")+idx,sbuffer2.toString().indexOf("'") + "'".length()+idx,"<SQ>",sbuffer1);
                                sbuffer2=sbr.replace(sbuffer2.toString().indexOf("'"),sbuffer2.toString().indexOf("'") + "'".length(),"<SQ>",sbuffer2);
                            }
                            else{
                                sbuffer2=sbr.replace(sbuffer2.toString().indexOf("'"),sbuffer2.toString().indexOf("'") + "'".length(),"",sbuffer2);
                                idx=idx+1;
                            }
                        }
                        else if (sbuffer2.toString().indexOf("'") == 0 || sbuffer2.toString().indexOf("'") == sbuffer2.toString().length()){
                            sbuffer2=sbr.replace(sbuffer2.toString().indexOf("'"),sbuffer2.toString().indexOf("'") + "'".length(),"",sbuffer2);
                            idx=idx+1;
                        }
                    }

                }catch (IndexOutOfBoundsException e){};
            }
            idx=0;

            String dq=String.valueOf('"')+String.valueOf('"');
            if (sbuffer1.toString().indexOf(dq) != -1){
                while (sbuffer1.toString().indexOf(dq) != -1){
                    sbuffer1=sbr.replace(sbuffer1.toString().indexOf(dq),sbuffer1.toString().indexOf(dq) + dq.length(),"<DDQ>",sbuffer1);
                }
            }
            svalue1=sbuffer1.toString();
            sbuffer2=new StringBuffer(svalue1);
            if (sbuffer2.toString().indexOf('"') != -1){
                try{
                    while (sbuffer2.toString().indexOf('"') != -1) {
                        if (sbuffer2.toString().indexOf('"') != 0 && sbuffer2.toString().indexOf('"') != sbuffer2.toString().length()){
                            Character cc=new Character(sbuffer2.toString().charAt(sbuffer2.toString().indexOf('"')-1));
                            Character cc1=new Character(sbuffer2.toString().charAt(sbuffer2.toString().indexOf('"')+1));
                            ich = cc.hashCode();
                            ich1 = cc1.hashCode();

                            if (((ich==33) || (ich>=35 && ich<=38) || (ich==43) || (ich==45)|| (ich>=46 && ich<=122)) && ((ich1==33) || (ich1>=35 && ich1<=38) || (ich1==43) || (ich1==45)|| (ich1>=46 && ich1<=122))) {
                                sbuffer1=sbr.replace(sbuffer2.toString().indexOf('"')+idx,sbuffer2.toString().indexOf('"') + String.valueOf('"').length()+idx,"<DQ>",sbuffer1);
                                sbuffer2=sbr.replace(sbuffer2.toString().indexOf('"'),sbuffer2.toString().indexOf('"') + String.valueOf('"').length(),"<DQ>",sbuffer2);
                            }
                            else {
                                sbuffer2=sbr.replace(sbuffer2.toString().indexOf('"'),sbuffer2.toString().indexOf('"') + "'".length(),"",sbuffer2);
                                idx=idx+1;
                            }
                        }
                        else if (sbuffer2.toString().indexOf('"') == 0 || sbuffer2.toString().indexOf('"') == sbuffer2.toString().length()){
                            sbuffer2=sbr.replace(sbuffer2.toString().indexOf('"'),sbuffer2.toString().indexOf('"') + "'".length(),"",sbuffer2);
                            idx=idx+1;
                        }
                    }
                }catch (IndexOutOfBoundsException e){};
            }

            svalue1=sbuffer1.toString();
        }
        // Process each token
        // WORD
        //Quoted String
        //Number


        boolean isDone=false;
        ExpressionParser p = new ExpressionParser(sfield1,svalue1);
        StreamTokenizer stz = p.setParseChars(sfield1,svalue1,sValueType);
        while (! isDone){
            int c = StreamTokenizer.TT_EOF;
            try{
                c = stz.nextToken();
            } catch (IOException e) {break;}
            switch (c) {
            case StreamTokenizer.TT_EOF: // CASE END OF FILE
                isDone = true;
                break;
            case StreamTokenizer.TT_WORD: // TOKEN IS A WORD
                if(sValueType.equalsIgnoreCase("Address")) {
                    int ir=isIPRange(stz.sval);
                    if (ir > 0)
                        st3=evaluateIPRange(sfield1,stz.sval,ir)+"' '";
                    else
                        st3="'"+stz.sval+"'"+"' '";
                }
                else if(sValueType.equalsIgnoreCase("date")){
                    int idt=isDateRange(stz.sval);
                    if (idt>0)
                        st3=evaluateDateRange(sfield1,stz.sval,idt)+"' '";
                    else
                        st3="'"+stz.sval+"'"+"' '";
                }
                else{
                    int LowerIt=0;
                    String hasDomain="";
                    int sLength=stz.sval.length();
                    if(sValueType.equalsIgnoreCase("NodeDomain")){
                        if (stz.sval.indexOf(".") != -1) {
                            int hasDot=stz.sval.indexOf(".");
                            String isDomn=stz.sval.substring(hasDot+1,sLength);
                            stz.sval=stz.sval.substring(0,hasDot);
                            if (sLength  > hasDot && (isDomn.compareTo("*")!=0  && isDomn.compareTo("%")!=0)) {
                                if ( isDomn.equals(""))
                                    isDomn = "%";
                                hasDomain="  and fqdnLower like "+"'"+isDomn+"'";
                                LowerIt=1;
                            }
                        }
                    }
                    if(sValueType.equalsIgnoreCase("DomainName")){
                        if ( stz.sval.equalsIgnoreCase("or")
                             || stz.sval.equalsIgnoreCase("and")
                             ||  stz.sval.equalsIgnoreCase("not"))
                            stz.sval = stz.sval;
                        else{
                            if(stz.sval.indexOf(".") != -1
                               &&  stz.sval.substring(stz.sval.indexOf("."),stz.sval.length()).equals("."))
                                stz.sval = stz.sval.substring(0,stz.sval.length() - 1);
                            else if (stz.sval.indexOf(".") == -1 && (stz.sval.indexOf("*") == -1 && stz.sval.indexOf("%") == -1))
                                stz.sval = stz.sval+".%";
                        }
                    }
                    if(sValueType.equalsIgnoreCase("DomainLog")){
                        if ( stz.sval.equalsIgnoreCase("or")
                             || stz.sval.equalsIgnoreCase("and")
                             ||  stz.sval.equalsIgnoreCase("not"))
                            stz.sval = stz.sval;
                        else{
                            if(stz.sval.indexOf(".") != -1
                               &&  stz.sval.substring(stz.sval.indexOf("."),stz.sval.length()).equals("."))
                                stz.sval=stz.sval+"%";
                            else if (stz.sval.indexOf(".") == -1)
                                stz.sval=stz.sval+".%";
                        }
                    }

                    if (stz.sval.indexOf("<DQ>") != -1){
                        if(isSubstring.equalsIgnoreCase("yes")) {
                            if (stz.sval.equalsIgnoreCase("Or") || stz.sval.equalsIgnoreCase("and") || stz.sval.equalsIgnoreCase("not"))
                                st3="'"+stz.sval+"'";
                            else
                                st3="'"+"%"+stz.sval+"%"+"'"+"' '";
                        }
                        else
                            st3="'"+stz.sval+"'";
                    }
                    else if (stz.sval.indexOf("<SQ>") != -1){
                        if(isSubstring.equalsIgnoreCase("yes")) {
                            if (stz.sval.equalsIgnoreCase("Or") || stz.sval.equalsIgnoreCase("and") || stz.sval.equalsIgnoreCase("not"))
                                st3="'"+stz.sval+"'";
                            else
                                st3="'"+"%"+stz.sval+"%"+"'"+"' '";
                        }
                        else
                            st3='"'+stz.sval+'"';
                    }

                    else if (stz.sval.indexOf("<DSQ>") != -1)
                        {
                            if(isSubstring.equalsIgnoreCase("yes"))
                                {
                                    if (stz.sval.equalsIgnoreCase("Or") || stz.sval.equalsIgnoreCase("and") || stz.sval.equalsIgnoreCase("not"))
                                        st3="'"+stz.sval+"'";
                                    else
                                        st3="'"+"%"+stz.sval+"%"+"'"+"' '";
                                }
                            else
                                st3='"'+stz.sval+'"';
                        }
                    else if (stz.sval.indexOf("<DDQ>") != -1){
                        if(isSubstring.equalsIgnoreCase("yes")){
                            if (stz.sval.equalsIgnoreCase("Or") || stz.sval.equalsIgnoreCase("and") || stz.sval.equalsIgnoreCase("not"))
                                st3="'"+stz.sval+"'";
                            else
                                st3="'"+"%"+stz.sval+"%"+"'"+"' '";
                        }
                        else
                            st3="'"+stz.sval+"'";
                    }
                    else if(isSubstring.equalsIgnoreCase("yes")){
                        if (stz.sval.equalsIgnoreCase("Or") || stz.sval.equalsIgnoreCase("and") || stz.sval.equalsIgnoreCase("not"))
                            st3="'"+stz.sval+"'";
                        else
                            st3="'"+"%"+stz.sval+"%"+"'"+"' '";
                    }
                    else
                        st3="'"+stz.sval+"'"+"' '";
                    if (LowerIt==1){
                        if (st3.indexOf("' '")!=-1)
                            st3=st3.substring(0,st3.indexOf("' '"))+hasDomain+"' '";
                        else
                            st3+=hasDomain;
                        LowerIt=0;
                    }
                }
                if (stz.sval.equalsIgnoreCase("OR") || stz.sval.equalsIgnoreCase("AND") || stz.sval.equalsIgnoreCase("NOT")) {
                    st3 = st3.toLowerCase();
                }

                st2+=st3;
                st2=st2.replace('*','%');
                break;
            default:
                boolean wquote = false; // TOKEN IS A QUOTED STRING
                if ((c == 0x22) || (c == 0x27)) {
                    if (stz.sval.equalsIgnoreCase("OR") || stz.sval.equalsIgnoreCase("AND") || stz.sval.equalsIgnoreCase("NOT")){
                        if(isSubstring.equalsIgnoreCase("yes"))
                            st3 = '"'+"%"+stz.sval+"%"+'"';
                        else
                            st3 = '"'+stz.sval+'"';
                        System.out.println("<<Happens here 1>>");
                    }
                    else{
                        if (stz.sval.indexOf("<DQ>") != -1){
                            if(isSubstring.equalsIgnoreCase("yes")){
                                if (stz.sval.equalsIgnoreCase("Or") || stz.sval.equalsIgnoreCase("and") || stz.sval.equalsIgnoreCase("not")) {
                                    System.out.println("<<Happens here 2>>");
                                    st3="'"+stz.sval+"'"; }
                                else
                                    st3="'"+"%[-./ ]"+stz.sval+"[-./ ]%"+"'"+"' '";
                            }
                            else
                                st3="'"+stz.sval+"'";
                        }
                        else if (stz.sval.indexOf("<SQ>") != -1){
                            if(isSubstring.equalsIgnoreCase("yes")) {
                                if (stz.sval.equalsIgnoreCase("Or") || stz.sval.equalsIgnoreCase("and") || stz.sval.equalsIgnoreCase("not"))
                                    st3="'"+stz.sval+"'";
                                else
                                    st3="'"+"%"+stz.sval+"%"+"'"+"' '";
                            }
                            else
                                st3='"'+stz.sval+'"';
                        }
                        else if (stz.sval.indexOf("<DSQ>") != -1) {
                            if(isSubstring.equalsIgnoreCase("yes")) {
                                if (stz.sval.equalsIgnoreCase("Or") || stz.sval.equalsIgnoreCase("and") || stz.sval.equalsIgnoreCase("not"))
                                    st3="'"+stz.sval+"'";
                                else
                                    st3="'"+"%"+stz.sval+"%"+"'"+"' '";
                            }
                            else
                                st3='"'+stz.sval+'"';
                        }
                        else if (stz.sval.indexOf("<DDQ>") != -1) {
                            if(isSubstring.equalsIgnoreCase("yes")){
                                if (stz.sval.equalsIgnoreCase("Or") || stz.sval.equalsIgnoreCase("and") || stz.sval.equalsIgnoreCase("not"))
                                    st3="'"+stz.sval+"'";
                                else
                                    st3="'"+"%"+stz.sval+"%"+"'"+"' '";
                            }
                            else
                                st3="'"+stz.sval+"'";
                        }
                        else if(/*sValueType.equalsIgnoreCase("Person") ||*/isSubstring.equalsIgnoreCase("yes") ) {
                            if(stz.sval.toLowerCase().compareTo("and")!= 0 && stz.sval.toLowerCase().compareTo("not")!= 0 && stz.sval.toLowerCase().compareTo("or")!= 0 ){
                                st3="'"+"%"+stz.sval+"%"+"'"+"' '";
                            }
                            else
                                st3="'"+stz.sval+"'";
                        }
                        else
                            st3="'"+stz.sval+"'";
                    }
                    st2+=st3;
                    wquote=true;
                    break;
                }
                if (! wquote){ // ANY OTHER TYPE OF TOKEN
                    if (c == 0x2E || c==0x2F || c == 0x30 || c==0x31 || c==0x32 || c==0x33 || c==0x34 || c==0x35 || c==0x36 || c==0x37 || c==0x38 || c==0x39 )
                        st3=""+(char)stz.ttype+"";
                    else if(c==0x28 || c==0x29)
                        st3=String.valueOf('"')+(char)stz.ttype+String.valueOf('"')+"' '";
                    else{
                        st3="'"+(char)stz.ttype+"'";
                    }
                    st2+=st3;
                    break;
                }
            }
            st2=st2.replace('*','%');
            st2=st2.replace('?','_');
        }

        //Replace the special embedded quote chars with appropriate type of quotes
        StringBuffer sbuffersc=new StringBuffer(st2);
        if (sbuffersc.toString().indexOf("<DSQ>") != -1){
            while (sbuffersc.toString().indexOf("<DSQ>") != -1) {
                sbuffersc=sbr.replace(sbuffersc.toString().indexOf("<DSQ>"),sbuffersc.toString().indexOf("<DSQ>") + "<DSQ>".length(),"''''",sbuffersc);
            }
        }
        if (sbuffersc.toString().indexOf("<SQ>") != -1){
            while (sbuffersc.toString().indexOf("<SQ>") != -1) {
                sbuffersc=sbr.replace(sbuffersc.toString().indexOf("<SQ>"),sbuffersc.toString().indexOf("<SQ>") + "<SQ>".length(),"''",sbuffersc);
            }
        }
        if (sbuffersc.toString().indexOf("<DQ>") != -1){
            while (sbuffersc.toString().indexOf("<DQ>") != -1) {
                sbuffersc=sbr.replace(sbuffersc.toString().indexOf("<DQ>"),sbuffersc.toString().indexOf("<DQ>") + "<DQ>".length(),String.valueOf('"')+String.valueOf('"'),sbuffersc);
            }
        }
        if (sbuffersc.toString().indexOf("<DDQ>") != -1){
            while (sbuffersc.toString().indexOf("<DDQ>") != -1) {
                sbuffersc=sbr.replace(sbuffersc.toString().indexOf("<DDQ>"),sbuffersc.toString().indexOf("<DDQ>") + "<DDQ>".length(),String.valueOf('"')+String.valueOf('"')+String.valueOf('"')+String.valueOf('"'),sbuffersc);
            }
        }

        st2=sbuffersc.toString();

        st2_cp=st2;
        if (st2_cp.substring(st2_cp.length()-3,st2_cp.length()).equals("' '"))
            st2=st2.substring(0,st2.length()-3);
        st2=sfield1+" "+"like "+ st2;
        return st2;

    }

    // Set the Grammar for the parser
    public StreamTokenizer setParseChars(String sfield22,String svalue22,String sValueType)
    {
        StreamTokenizer stz = new StreamTokenizer(new StringReader(svalue22));
        stz.quoteChar('"');
        stz.whitespaceChars(0,31);
        if(sValueType.equalsIgnoreCase("date")) {
            stz.wordChars(33,33);
            stz.wordChars(35,38);
            //      stz.ordinaryChars(40,41);
            stz.wordChars(40,47);
            //      stz.wordChars(42,47);//Original before 11/09/2000
            stz.wordChars(32,32);//Space is part of the word
            stz.ordinaryChars(48,57);
            stz.wordChars(43,43);
            stz.wordChars(45,45);
            stz.wordChars(46,122);
            //      stz.ordinaryChars(44,44);
        }
        else
            {
                stz.wordChars(33,33);
                stz.wordChars(35,38);
                //      stz.ordinaryChars(40,41);
                stz.wordChars(40,47);
                //      stz.wordChars(42,47); Orginal before 11/09/2000
                stz.ordinaryChars(32,32);//Space as a seperator
                stz.ordinaryChars(48,57);
                stz.wordChars(43,43);
                stz.wordChars(45,45);
                stz.wordChars(46,122);
                //      stz.ordinaryChars(44,44);
            }
        return stz;
    }

    public String generateSQLClause(String sfield,String svalue,String viewName,String idColumn,String isSubstring,String shouldBeLowered,String valueType){
        int hasParanthesis = 0;
        String st9="";
        String st2 = TokenizeSvalue(sfield,svalue,isSubstring,shouldBeLowered,valueType);
        if (st2.indexOf("'('") != -1)
            hasParanthesis=1;
        StringBuffer sbuffer1 = new StringBuffer(st2);
        sbuffer1=replaceSQLclauseChars(sbuffer1,sfield,viewName,idColumn);
        String st4=sbuffer1.toString();
        st9=st4;
        String st10="";
        String st11="";
        String stWhere=st9;
        String stWLen="";
        String stSubWL="";
        String _stSub="";
        if (st4.length() > 0) {
            // Pad a single parenthesis ")" to the end of each condition (SPECIAL FOR "AND" and "AND NOT")
            if(stWhere.indexOf(" where "+sfield) != -1){
                String idColumnName = viewName+"."+idColumn;

                while (stWhere.indexOf(" where "+sfield) != -1) {
                    stWLen=stWhere.substring(stWhere.indexOf(" where "+sfield)+7+sfield.length(),stWhere.length());
                    stSubWL=stWhere.substring(0,stWhere.indexOf(" where "+sfield)+7+sfield.length());

                    if (stWLen.indexOf(" "+sfield) != -1  || stWLen.indexOf(" "+idColumnName)!=-1){
                        if(stWLen.indexOf(" "+sfield)!= -1 && stWLen.indexOf(" and "+idColumnName) != -1) {
                            if(stWLen.indexOf(" "+sfield) < stWLen.indexOf(" "+idColumnName))
                                _stSub+=stSubWL+stWLen.substring(0,stWLen.indexOf(" "+sfield)-3)+")";
                            else if(stWLen.indexOf(" and "+idColumnName) < stWLen.indexOf(" "+sfield) )
                                _stSub+=stSubWL+stWLen.substring(0,stWLen.indexOf(" and "+idColumnName))+")";
                        }
                        else if(stWLen.indexOf(" "+sfield)!= -1 && stWLen.indexOf(" and "+idColumnName) == -1)
                            _stSub+=stSubWL+stWLen.substring(0,stWLen.indexOf(" "+sfield)-3)+")";
                        else if(stWLen.indexOf(" "+sfield)== -1 && stWLen.indexOf(" and "+idColumnName) != -1)
                            _stSub+=stSubWL+stWLen.substring(0,stWLen.indexOf(" and "+idColumnName))+")";
                    }
                    else

                        _stSub+=stWhere+")";
                    stWhere=stWhere.substring(stWhere.indexOf(" where "+sfield)+7+sfield.length(),stWhere.length());

                    if (stWhere.indexOf(" "+sfield) != -1  || stWhere.indexOf(" "+idColumnName)!=-1){
                        if(stWhere.indexOf(" "+sfield)!= -1 && stWhere.indexOf(" and "+idColumnName) != -1) {
                            if(stWhere.indexOf(" "+sfield) < stWhere.indexOf(" "+idColumnName))
                                stWhere=stWhere.substring(stWhere.indexOf(" "+sfield)-3,stWhere.length());
                            else if(stWhere.indexOf(" and "+idColumnName) < stWhere.indexOf(" "+sfield) )
                                stWhere=stWhere.substring(stWhere.indexOf(" and "+idColumnName),stWhere.length());

                        }
                        else if(stWhere.indexOf(" "+sfield)!= -1 && stWhere.indexOf(" and "+idColumnName) == -1)

                            stWhere=stWhere.substring(stWhere.indexOf(" "+sfield)-3,stWhere.length());


                        else if(stWhere.indexOf(" "+sfield)== -1 && stWhere.indexOf(" and "+idColumnName) != -1)
                            stWhere=stWhere.substring(stWhere.indexOf(" and "+idColumnName),stWhere.length());


                    }

                }
                if (stWLen.indexOf(" "+sfield) !=-1)
                    _stSub+=stWLen.substring(stWLen.indexOf(" "+sfield)-3,stWLen.length());

                st9=_stSub;
                st4=st9;
            }
            stWhere=new String("");
            stWLen=new String("");
            stSubWL=new String("");
            if (_stSub.length() > 0)
                stWhere =  _stSub;
            else
                stWhere=st9;

            if(stWhere.indexOf(" where ("+sfield) != -1){
                _stSub=new String("");

                String idColumnName = viewName+"."+idColumn;

                while (stWhere.indexOf(" where ("+sfield) != -1) {
                    stWLen=stWhere.substring(stWhere.indexOf(" where ("+sfield)+8+sfield.length(),stWhere.length());
                    stSubWL=stWhere.substring(0,stWhere.indexOf(" where ("+sfield)+8+sfield.length());

                    if (stWLen.indexOf(" "+sfield) != -1  || stWLen.indexOf(" "+idColumnName)!=-1){
                        if(stWLen.indexOf(" "+sfield)!= -1 && stWLen.indexOf(" and "+idColumnName) != -1){
                            if(stWLen.indexOf(" "+sfield) < stWLen.indexOf(" "+idColumnName))
                                _stSub+=stSubWL+stWLen.substring(0,stWLen.indexOf(" "+sfield)-3)+")";
                            else if(stWLen.indexOf(" and "+idColumnName) < stWLen.indexOf(" "+sfield) )
                                _stSub+=stSubWL+stWLen.substring(0,stWLen.indexOf(" and "+idColumnName))+")";
                        }
                        else if(stWLen.indexOf(" "+sfield)!= -1 && stWLen.indexOf(" and "+idColumnName) == -1)
                            _stSub+=stSubWL+stWLen.substring(0,stWLen.indexOf(" "+sfield)-3)+")";
                        else if(stWLen.indexOf(" "+sfield)== -1 && stWLen.indexOf(" and "+idColumnName) != -1)
                            _stSub+=stSubWL+stWLen.substring(0,stWLen.indexOf(" and "+idColumnName))+")";
                    }
                    else

                        _stSub+=stWhere+")";
                    stWhere=stWhere.substring(stWhere.indexOf(" where ("+sfield)+8+sfield.length(),stWhere.length());

                    if (stWhere.indexOf(" "+sfield) != -1  || stWhere.indexOf(" "+idColumnName)!=-1) {
                        if(stWhere.indexOf(" "+sfield)!= -1 && stWhere.indexOf(" and "+idColumnName) != -1)          {
                            if(stWhere.indexOf(" "+sfield) < stWhere.indexOf(" "+idColumnName))
                                stWhere=stWhere.substring(stWhere.indexOf(" "+sfield)-3,stWhere.length());
                            else if(stWhere.indexOf(" and "+idColumnName) < stWhere.indexOf(" "+sfield) )
                                stWhere=stWhere.substring(stWhere.indexOf(" and "+idColumnName),stWhere.length());

                        }
                        else if(stWhere.indexOf(" "+sfield)!= -1 && stWhere.indexOf(" and "+idColumnName) == -1)

                            stWhere=stWhere.substring(stWhere.indexOf(" "+sfield)-3,stWhere.length());
                        else if(stWhere.indexOf(" "+sfield)== -1 && stWhere.indexOf(" and "+idColumnName) != -1)
                            stWhere=stWhere.substring(stWhere.indexOf(" and "+idColumnName),stWhere.length());


                    }

                }
                /*      if (stWLen.indexOf(" "+sfield) !=-1)
                        _stSub+=stWLen.substring(stWLen.indexOf(" "+sfield)-3,stWLen.length());
                */
                _stSub+=stWhere;
                st9=_stSub;
                st4=st9;
            }

            //   System.out.println("Search SQL Clause:"+_stSub);
            // If there is no user specified grouping in the search string then
            // group first by "and" and then "or"

            //      if (hasParanthesis==0) //New change do grouping all the time
            //   { // grouping by and
            while (st9.indexOf(" and ") != -1) {
                st10=st9.substring(0,st9.indexOf(" and ")+4);
                if (st10.lastIndexOf("Or "+sfield) != -1)
                    st10=st10.substring(0,st10.lastIndexOf("Or "+sfield)+"Or ".length())+"("+st10.substring(st10.lastIndexOf("Or "+sfield)+"Or ".length(),st10.length());
                else
                    st10="("+st10;
                st9=st9.substring(st10.length() - 1,st9.length());
                if (st9.indexOf("Or "+sfield) != -1)
                    st11=st11+st10+" "+st9.substring(1,st9.indexOf("Or "+sfield))+")";
                else
                    st11=st11+st10+st9+")";
                if (st9.indexOf(" Or "+sfield) != -1)
                    st9=st9.substring(st9.indexOf(" Or "+sfield),st9.length());
                else
                    st9="";
                //}

            }
            st11=st11+st9;

            if(st11.length() <= 0)
                st9=st4;
            else {
                st9=st11;
                st11="";
            }
            // grouping by OR
            while (st9.indexOf(" Or ") != -1) {
                st10=st9.substring(0,st9.indexOf(" Or ")+3);
                if (st10.lastIndexOf("and "+sfield) != -1)
                    st10=st10.substring(0,st10.lastIndexOf("and "+sfield)+"and ".length())+"("+st10.substring(st10.lastIndexOf("and "+sfield)+"and ".length(),st10.length());
                else
                    st10="("+st10;
                st9=st9.substring(st10.length() - 1,st9.length());
                if (st9.indexOf("and "+sfield) != -1)
                    st11=st11+st10+" "+st9.substring(1,st9.indexOf("and "+sfield))+")";
                else
                    st11=st11+st10+st9+")";
                if (st9.indexOf(" and "+sfield) != -1)
                    st9=st9.substring(st9.indexOf(" and "+sfield),st9.length());
                else
                    st9="";
            }
            st11=st11+st9;
            //}

            return st11;
        }
        else
            return st4;
        //  } //do grouping all the time
        // else
        // return st2;
    }

    public int bitstomask(int i) {
        return (0xffffffff << i) ;
    }

    public int strtoip(String ss) {
        int s1=0;
        int i=0;
        int z=3;
        while (ss.indexOf('.') != -1){
            s1=Integer.valueOf(ss.substring(0,ss.indexOf('.'))).intValue();
            if (s1 > 255){
                System.out.println(ss+" BOGUS bad address expression");
                break;
            }
            else{
                i += s1 << (8 * z);
                z=z-1;
                ss=ss.substring(ss.indexOf('.')+1,ss.length());

            }
        }
        if (ss.length() > 0 && ss.indexOf('.') == -1 && z==1)
            i+=Integer.valueOf(ss).intValue() << 8;
        else if (ss.length() > 0 && ss.indexOf('.') == -1 && z==0)
            i+=Integer.valueOf(ss).intValue();
        return i;
    }


    public String iptostr(int ip)
    {

        return (String.valueOf(((ip >> 24) & 0xff)) + '.' + String.valueOf(((ip >> 16) & 0xff)) + '.' + String.valueOf(((ip >> 8) & 0xff)) + '.' + String.valueOf((ip & 0xff)));
    }

    public static void ParseValue(String args[]){
        String sst=null;

        String st3="";
        //        st3=p.generateSQLClause(args[1],args[0]);
        sst=st3+"\n";
        System.out.println("SQL CONDITION:" + sst+ "\n");
    }

    public String ParsePersonName(String personName)
    {
        String person=new String("");
        person = replaceStringBufferChars(personName,"' '","'and'");
        return person;
    }

    public String replaceStringBufferChars(String strName,String strReplace,String strReplaceWith)
    {
        SBreplace sr=new SBreplace();
        StringBuffer sb=new StringBuffer(strName);
        String rstr=new String(strName);
        while (sb.toString().indexOf(strReplace) != -1) {

            sb=sr.replace(sb.toString().indexOf(strReplace),sb.toString().indexOf(strReplace) + strReplace.length(),strReplaceWith,sb);
            rstr=sb.toString();
        }
        return rstr;

    }
}
