/************************************************************************
 *
 * IPAddressAcceptCommand.java - db call NetDB_IPaddress.accept
 *
 * $Id: IPAddressAcceptCommand.java,v 1.3 2006/09/07 21:26:06 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2006, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import java.lang.*;
import java.sql.*;

/** db call NetDB_IPaddress.accept */
class IPAddressAcceptCommand extends DBCall {

    public static final String IPC_TYPE         = "node";
    public static final String INTERFACE_TYPE   = "interface";
    public static final String ADDR_SPACE_TYPE  = "addressspace";

    public static final int ID_OP_USER_INDEX    = 0; // p_op_uid
    public static final int ADDRESS_INDEX       = 1; // p_address
    public static final int ACTIVE_INDEX        = 2; // p_active
    public static final int PTR_PREF_INDEX      = 3; // p_ptr_pref
    public static final int REFERENT_ID_INDEX   = 4; // p_referent_id
    public static final int LOCK_INDEX          = 5; // p_lock
    public static final int REFERENT_TYPE_INDEX = 6; // p_referent_type

    public IPAddressAcceptCommand(stanford.netdb.utils.Logger logger,
                                  String ip,        Boolean active,
                                  Integer referent, Timestamp lock,
                                  String ref_type,  String ptr_pref,
                                  Integer op_uid) {
        super(logger);
        db_call_type = ORA_PROC_CALL;
        values = new Object[7];

        values[ID_OP_USER_INDEX]    = op_uid;
        values[ADDRESS_INDEX]       = ip;
        values[ACTIVE_INDEX]        = new Short(active.booleanValue()?"1":"0");
        if( ptr_pref != null && ptr_pref.trim().length() > 0 )
            values[PTR_PREF_INDEX]  = new Integer(ptr_pref);
        else
            values[PTR_PREF_INDEX]  = new Integer(System.getProperty("DefaultPTRpref", "0"));
        values[REFERENT_ID_INDEX]   = referent;
        values[LOCK_INDEX]          = lock;
        values[REFERENT_TYPE_INDEX] = ref_type;
    }

    public String getSQL() {
        return proc_call_sql("NetDB_IPaddress.accept");
    }
}
