/************************************************************************
 *
 * InterfaceCommand.java - base class for calls to NetDB_Interface.master
 *
 * $Id: InterfaceCommand.java,v 1.1.1.1 2006/06/30 23:38:09 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.lang.*;
import java.sql.*;

/** base class for all db calls to NetDB_Interface.master */
abstract class InterfaceCommand extends DBCall {

    public static final String DELETE_ACTION     = "delete";
    public static final String INFO_ACTION       = "info";
    public static final String INSERT_ACTION     = "insert";
    public static final String UPDATE_ACTION     = "update";

    public static final String ETHERNET_TYPE     = "Ethernet";

    public static final int ACTION_INDEX         = 1;  // p_action
    public static final int ID_OP_USER_INDEX     = 2;  // p_op_uid
    public static final int LOCK_INDEX           = 3;  // p_lock
    public static final int INTFC_ID_INDEX       = 4;  // p_intfc_id
    public static final int HW_ADDRESS_INDEX     = 5;  // p_hw_addr
    public static final int NEW_HW_ADDRESS_INDEX = 6;  // p_hw_addr_new
    public static final int INTFC_TYPE_INDEX     = 7;  // p_type
    public static final int NODE_ID_INDEX        = 8;  // p_node_id
    public static final int ROAMING_FLAG_INDEX   = 9;  // p_use_roam
    public static final int DHCP_FLAG_INDEX      = 10; // p_use_dhcp
    public static final int COMMENT_INDEX        = 11; // p_comment

    public InterfaceCommand(stanford.netdb.utils.Logger logger) {
        super(logger);
        values = new Object[12];

        values[0]                    = ref_cursor;
        values[ACTION_INDEX]         = new Null(Types.VARCHAR);
        values[ID_OP_USER_INDEX]     = new Null(Types.NUMERIC);
        values[LOCK_INDEX]           = new Null(Types.TIMESTAMP);
        values[INTFC_ID_INDEX]       = new Null(Types.NUMERIC);
        values[HW_ADDRESS_INDEX]     = new Null(Types.VARCHAR);
        values[NEW_HW_ADDRESS_INDEX] = new Null(Types.VARCHAR);
        values[INTFC_TYPE_INDEX]     = new Null(Types.VARCHAR);
        values[NODE_ID_INDEX]        = new Null(Types.NUMERIC);
        values[ROAMING_FLAG_INDEX]   = new Null(Types.TINYINT);
        values[DHCP_FLAG_INDEX]      = new Null(Types.TINYINT);
        values[COMMENT_INDEX]        = new Null(Types.VARCHAR);
    }

    public String getSQL() {
        return proc_call_sql("NetDB_Interface.master");
    }
}
