/************************************************************************
 *
 * LockUnLockObjectCommand.java -
 *
 * $Id: LockUnLockObjectCommand.java,v 1.1.1.1 2006/06/30 23:38:09 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, 2002 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.lang.*;
import java.util.*;
import java.sql.*;

class LockUnLockObjectCommand extends DBCall {

    private int action = -1; // LOCK, UNLOCK

    public static final int LOCK                        = 1;
    public static final int UNLOCK                      = 2;

    public static final int REG_ID_OP_USER_INDEX        = 1;  // @dwrdPerRegId
    public static final int NAME_INDEX                  = 2;  // @nmName
    public static final int DOMAIN_INDEX                = 3;  // @fqdDomain
    public static final int OBJECT_ID_INDEX             = 4;  // @idObject
    public static final int OBJECT_TYPE_INDEX           = 5;  // @wrdObject

    /**
     * Constructor for LockObjectCommand which calls pLock
     * @param action - NetDB.{LOCK, UNLOCK}
     * @param what   = NetDB.{ATEAM, GROUP, DOMAIN, USER, NETWORK, NODE}
     * @param oid    - objects id
     * @param regID  - RegID of operating user.
     */
    public LockUnLockObjectCommand(stanford.netdb.utils.Logger logger, int action, int what, Integer oid, String regID) {
        super(logger);
        this.action = action;

        values = new Object[6];
        values[0] = new OutParam(Types.NUMERIC);
        values[REG_ID_OP_USER_INDEX] = regID;
        values[NAME_INDEX] = new Null(Types.VARCHAR);
        values[DOMAIN_INDEX] = new Null(Types.VARCHAR);
        values[OBJECT_ID_INDEX] = oid;

        switch(what) {
        case Constants.ATEAM:
            values[OBJECT_TYPE_INDEX] = "adminteam";
            break;

        case Constants.GROUP:
            values[OBJECT_TYPE_INDEX] = "owner";
            break;

        case Constants.DOMAIN:
            values[OBJECT_TYPE_INDEX] = "domain";
            break;

        case Constants.USER:
            values[OBJECT_TYPE_INDEX] = "users";
            break;

        case Constants.NETWORK:
            values[OBJECT_TYPE_INDEX] = "network";
            break;

        case Constants.NODE:
            values[OBJECT_TYPE_INDEX] = "node";
            break;
        }
    }

    public String getSQL() {
        if( action == LOCK )
            return "{?= call pLock (?, ?, ?, ?, ?)}";
        else
            return "{?= call pUnLock (?, ?, ?, ?, ?)}";
    }

    public Hashtable processResultSet(Hashtable res) throws Exception {
        return res;
    }
}
