/************************************************************************
 *
 * MXSimpleSearchCommand.java - db call to wiSearchByNode
 *
 * $Id: MXSimpleSearchCommand.java,v 1.1.1.1 2006/06/30 23:38:09 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.util.*;
import java.lang.*;

/** db call to wiSearchByNode */
class MXSimpleSearchCommand extends SimpleSearchCommand {

    public MXSimpleSearchCommand(stanford.netdb.utils.Logger logger) {
        super(logger);
        values = new Object[2];
    }

    public String getSQL() {
        return "{call SimpleSearchForMXName (?, ?)}";
    }

    /**
     * custom result set processing. Returned Hashtable has structure:
     *     (Hash)
     *     `mxname handle` -> (Hash)
     *               'mx'     -> (String)
     *               'pref' -> (Int)
     *               'handle' -> (String) FQDN.
     *               'name'   -> (String) short version of handle without default domain
     */
    public Hashtable processResultSet(Hashtable res)
        throws Exception {
        if( res.get("errors") != null )
            return res;

        Hashtable result = new Hashtable();

        Vector result_sets = (Vector)res.get("result_sets");
        for( int i = 0; i < result_sets.size(); i++ ) {
            Hashtable result_set = (Hashtable)result_sets.elementAt(i);
            Vector rs = (Vector)result_set.get("data");

            // uncomment below to see raw data
            //printRawData(result_set);

            for( int j = 0; j < rs.size(); j++ ) {
                Vector row = (Vector)rs.elementAt(j);
                String mx = row.elementAt(1).toString().trim();

                Object obj = row.elementAt(2);
                String handle = obj.toString().trim();

                Hashtable record = getChild(result, mx+" "+handle.toLowerCase());

                record.put("mx", mx);
                record.put("handle", handle);

                obj = row.elementAt(3);
                String name = obj.toString().trim();
                if( !(obj instanceof Null) )
                    record.put("name", name);

                obj = row.elementAt(4);
                if( obj instanceof Integer )
                    record.put("pref", obj);
            }
        }
        return result;
    } // end of processResultSet(...)
}
