/************************************************************************
 *
 * NameAcceptCommand.java - db call NetDB_Name.accept
 *
 * $Id: NameAcceptCommand.java,v 1.1.1.1 2006/06/30 23:38:09 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import java.lang.*;
import java.sql.*;

/** db call NetDB_Name.accept */
class NameAcceptCommand extends DBCall {

    public static final int ID_OP_USER_INDEX    = 0; // p_op_uid
    public static final int NAME_ID_INDEX       = 1; // p_name_id
    public static final int FULLNAME_INDEX      = 2; // p_fullname
    public static final int TYPE_INDEX          = 3; // p_name_type
    public static final int REFERENT_ID_INDEX   = 4; // p_referent_id
    public static final int REFERENT_NAME_INDEX = 5; // p_referent_name
    public static final int MASTER_LOCK_INDEX   = 6; // p_lock
    public static final int MX_PREF_INDEX       = 7; // p_mx_pref


    public NameAcceptCommand(stanford.netdb.utils.Logger logger,
                             Integer idName,           String type,           Integer idMasterObj,
                             String referent_fullname, Timestamp master_lock, Integer mx_pref,
                             Integer op_uid) {
        this(logger,      idName,            null,          type,
             idMasterObj, referent_fullname, master_lock,   mx_pref, op_uid);
    }
    /**
     * type = Constants.{DOMAIN_TYPE, NODE_NAME_TYPE, ADDRESS_NAME_TYPE, INTERFACE_NAME_TYPE,
     *                   NETWORK_NAME_TYPE, NAME_ALIAS_TYPE, MX_TYPE}
     **/
    public NameAcceptCommand(stanford.netdb.utils.Logger  logger,
                             Integer idName,      String fullname,          String type,
                             Integer idMasterObj, String referent_fullname, Timestamp master_lock,
                             Integer mx_pref,     Integer op_uid)
    {
        super(logger);
        db_call_type = ORA_PROC_CALL;
        values = new Object[8];

        values[ID_OP_USER_INDEX]   = op_uid;
        values[NAME_ID_INDEX]      = o2n(idName, Types.NUMERIC);
        values[FULLNAME_INDEX]     = o2v(fullname);
        values[TYPE_INDEX]         = type;
        values[REFERENT_ID_INDEX]  = o2n(idMasterObj, Types.NUMERIC);
        values[REFERENT_NAME_INDEX]= o2v(referent_fullname);
        values[MASTER_LOCK_INDEX]  = o2n(master_lock, Types.TIMESTAMP);
        values[MX_PREF_INDEX]      = o2n(mx_pref, Types.NUMERIC);
    }

    public String getSQL() {
        return proc_call_sql("NetDB_Name.accept");
    }
}
