/************************************************************************
 *
 * NetworkCommand.java - base class for db call NetDB_Network.master
 *
 * $Id: NetworkCommand.java,v 1.1.1.1 2006/06/30 23:38:09 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.lang.*;
import java.util.*;
import java.sql.*;

/** base class for db call NetDB_Network.master */
abstract class NetworkCommand extends DBCall {

    public Timestamp lock;

    protected Integer op_uid;
    protected Integer idNet;

    protected Hashtable oldNet;
    protected Hashtable newNet;

    public static final int ACTION_INDEX     = 1; // p_action
    public static final int ID_OP_USER_INDEX = 2; // p_op_uid
    public static final int LOCKED_INDEX     = 3; // p_lock
    public static final int ID_NETWORK_INDEX = 4; // p_net_id
    public static final int NETWORK_INDEX    = 5; // p_network
    public static final int SERVICE_INDEX    = 6; // p_service
    public static final int COMMENT_INDEX    = 7; // p_comment

    public NetworkCommand(stanford.netdb.utils.Logger logger) {
        super(logger);
        values = new Object[8];
        reset();
    }

    public String getSQL() {
        return proc_call_sql("NetDB_Network.master");
    }

    protected Hashtable doCheck(NetDBConnection con) throws NetDBException, Exception {
        reset();
        db_call_type = ORA_FUN_IGNORE_RET_CALL;

        values[ACTION_INDEX]     = "verify";
        values[ID_OP_USER_INDEX] = op_uid;
        values[ID_NETWORK_INDEX] = idNet;

        return con.callDB(this);
    }

    private void reset() {
        values[0]                = ref_cursor;
        values[ACTION_INDEX]     = new Null(Types.VARCHAR);
        values[ID_OP_USER_INDEX] = new Null(Types.NUMERIC);
        values[LOCKED_INDEX]     = new Null(Types.TIMESTAMP);
        values[ID_NETWORK_INDEX] = new Null(Types.NUMERIC);
        values[NETWORK_INDEX]    = new Null(Types.VARCHAR);
        values[SERVICE_INDEX]    = new Null(Types.VARCHAR);
        values[COMMENT_INDEX]    = new Null(Types.VARCHAR);
    }

    public void checkErrors(Hashtable result, Hashtable warnings) throws MiniException {
        Vector error_set = (Vector)result.get("errors");
        if( error_set != null && error_set.size() > 0 ) {
            if( warnings == null )
                throw new MiniException(result);

            for( int i = 0; i < error_set.size(); i++ ) {
                String element = error_set.elementAt(i).toString().trim();
                logger.debug(element);
                if( element.startsWith("WARNING") )
                    warnings.put(element, "");
                else
                    throw new MiniException(result);
            }
        }
    }

    class MiniException extends Exception {
        public Hashtable result;

        public MiniException(Hashtable result) {
            this.result = result;
        }
    } // end of class MiniException
}
