/************************************************************************
 *
 * NetworkFullSearchCommand.java -
 *
 * $Id: NetworkFullSearchCommand.java,v 1.3 2006/11/01 21:50:57 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import java.util.*;
import java.text.*;
import java.lang.*;
import stanford.netdb.utils.*;

class NetworkFullSearchCommand extends FullSearchCommand {

    public NetworkFullSearchCommand(stanford.netdb.utils.Logger logger,
                                    Hashtable search_params)
    {
        super(logger, search_params);
    }

    public String getSearchSQL() throws Exception {
        String sql = "WITH Records AS ( SELECT unique(id) record_id FROM ";

        // fill in display order vector
        Vector displayOrder = new Vector();
        init_result(search_params, displayOrder, "Networks");

        // go over displayOrder.XXX parameters and assemble displayOrder Vector
        assmDisplayOrder(displayOrder, search_params);

        // special treatment of network 'dhcp options' field
        String dhcp_options = (String)search_params.get("txt.NetDHCPoptions");
        if( dhcp_options != null && dhcp_options.trim().length() > 0 )
            search_params.put("fs.FS_NetworkDHCPoptions.txt.lower(dhcp_option)",
                              dhcp_options.replaceAll("#", "#hsh").replaceAll("\\\\\\\\", "&bsl;").
                              replaceAll("\\\\=", "#eql").replaceAll("&bsl;", "\\\\\\\\"));

        // special treatment of addr. space 'dhcp options' field
        dhcp_options = (String)search_params.get("txt.ASDHCPoptions");
        if( dhcp_options != null && dhcp_options.trim().length() > 0 )
            search_params.put("fs.FS_AddressSpace.txt.lower(dhcp_option)",
                              dhcp_options.replaceAll("#", "#hsh").replaceAll("\\\\\\\\", "&bsl;").
                              replaceAll("\\\\=", "#eql").replaceAll("&bsl;", "\\\\\\\\"));

        // assemble where clause
        sql += assmCommonSQL(search_params) + " ) ";

        // assemble select clause
        Hashtable views = new Hashtable();
        String info_select = "\nSELECT id \"ID\", to_char(id) \"handle\", "+
            "as_prefix, ip_as_prefix, IPaddress_txt";

        info_select +=
            selected("displayOrder.txt.1.Name:Network")?", name_id, net_name, net_domain":"";

        info_select +=
            selected("displayOrder.txt.2.Location")?", location":"";

        if( selected("displayOrder.ni.3.AddressSpace") )
            views.put("FS_AddressSpaceInfo", "");

        info_select +=
            selected("displayOrder.txt.4.Service")?", dhcp_service \"Service\"":"";


        if( selected("txt.5.Network DHCPoption") ) {
            info_select += ", net_dhcp_option \"Network DHCPoption\"";
            views.put("FS_NetworkDHCPoptionsInfo", "");
        }

        info_select +=
            selected("displayOrder.txt.6.Network Group")?", owner \"Network Group\"":"";

        info_select += selected("displayOrder.txt.7.Comment")?", comments \"Comment\"":"";

        if( selected("displayOrder.ns.8.AddressSpace DHCP Option") ) {
            info_select += ", as_dhcp_option";
            views.put("FS_AddressSpaceInfo", "");
        }

        if( selected("displayOrder.ns.9.AS Group") ) {
            info_select += ", as_owner";
            views.put("FS_AddressSpaceOwnerInfo", "");
        }

        info_select += selected("displayOrder.ns.10.AScomment")?", as_comments":"";

        if( selected("displayOrder.ni.11.Address") )
            views.put("FS_AddressSpaceIPaddressInfo", "");

        if( selected("displayOrder.ns.12.Name:DHCP Address") ) {
            info_select += ", ip_name, ip_domain";
            views.put("FS_AddressSpaceIPaddressInfo", "");
        }

        if( selected("displayOrder.ns.13.Active Address") ) {
            info_select += ", ip_active \"Active Address\"";
            views.put("FS_AddressSpaceIPaddressInfo", "");
        }

        if( selected("displayOrder.ns.18.LowRes") ) {
            info_select += ", trim(lowres) lowres";
            views.put("FS_AddressSpaceInfo", "");
        }

        if( selected("displayOrder.ns.19.HighRes") ) {
            info_select += ", trim(highres) highres";
            views.put("FS_AddressSpaceInfo", "");
        }

        info_select += modBlock(search_params);

        views.put("FS_NetworkInfo", "");

        Enumeration enm = views.keys();
        int i = 0;
        while( enm.hasMoreElements() ) {
            sql += info_select + " FROM " + enm.nextElement()+
            " JOIN Records on ID = Records.record_id";
            if( i+1 < views.size() ) sql += "\nUNION";
            i++;
        }

        return sql;
    }

    public Hashtable processResultSet(Hashtable res) throws Exception {
        if( res.get("errors") != null ) {
            Hashtable errors = new Hashtable();
            errors.put("errors", res.get("errors"));
            return errors;
        }

        String default_domain = (String)((NetDBii)logger).getUser().get("default_domain");

        Vector column_names = (Vector)res.get("column_names");
        Hashtable result = new Hashtable();

        Vector result_set = (Vector)res.get("resultSet");
        // first pass
        for( int i = 0; i < result_set.size(); i++ ) {
            Vector row = (Vector)result_set.elementAt(i);
            String id = row.elementAt(0).toString();

            Hashtable record = get_hash_or_create(result, id);
            Hashtable net_name = new Hashtable();
            Hashtable as = new Hashtable();
            Hashtable ip = new Hashtable();
            for( int j = 0; j < row.size(); j++ ) {
                String column = (String)column_names.elementAt(j);

                if( row.elementAt(j) == null )
                    continue;

                if( column.equals("NAME_ID") ) {
                    net_name = get_hash_or_create(get_hash_or_create(record, "names"),
                                                  row.elementAt(j));
                    continue;
                }

                if( column.equals("NET_NAME") ) {
                    net_name.put("name", row.elementAt(j));
                    continue;
                }

                if( column.equals("NET_DOMAIN") ) {
                    net_name.put("domain", row.elementAt(j));
                    continue;
                }

                if( column.equals("Network DHCPoption") ) {
                    get_hash_or_create(record, "Network DHCPoption").put(row.elementAt(j), "");
                    continue;
                }

                if( column.equals("Network Group") ) {
                    get_hash_or_create(record, "groups").put(row.elementAt(j), "");
                    continue;
                }

                if( column.equals("AS_PREFIX") ) {
                    as = get_hash_or_create(get_hash_or_create(record, "address_spaces"),
                                            row.elementAt(j));
                    continue;
                }

                if( column.equals("AS_OWNER") ) {
                    get_hash_or_create(as, "groups").put(row.elementAt(j), "");
                    continue;
                }

                if( column.equals("AS_DHCP_OPTION") ) {
                    get_hash_or_create(as, "dhcp_options").put(row.elementAt(j), "");
                    continue;
                }

                if( column.equals("AS_COMMENTS") ) {
                    as.put("comments", row.elementAt(j));
                    continue;
                }

                if( column.equals("IP_AS_PREFIX") ) {
                    as = get_hash_or_create(get_hash_or_create(record, "address_spaces"),
                                            row.elementAt(j));
                    continue;
                }

                if( column.equals("IPADDRESS_TXT") ) {
                    ip = get_hash_or_create(get_hash_or_create(as, "ips"), row.elementAt(j));
                    continue;
                }

                if( column.equals("IP_NAME") ) {
                    ip.put("name", row.elementAt(j));
                    continue;
                }

                if( column.equals("IP_DOMAIN") ) {
                    ip.put("domain", row.elementAt(j));
                    continue;
                }

                if( column.equals("Active Address") ) {
                    ip.put("active", row.elementAt(j));
                    continue;
                }

                if( column.equals("HIGHRES") || column.equals("LOWRES") ) {
                    as.put(column, row.elementAt(j));
                    continue;
                }

                if( ( column.equals("Created By") || column.equals("Modified By") ) &&
                    row.elementAt(j).toString().trim().equals("()") )
                    continue;

                // handle the rest of the columns
                Hashtable cell = get_hash_or_create(record, column);

                if( row.elementAt(j) != null ) {
                    // reqular columns
                    Object element = row.elementAt(j);
                    if( element instanceof oracle.sql.DATE )
                        cell.put(((oracle.sql.DATE)element).timestampValue(), "");
                    else if( element instanceof oracle.sql.TIMESTAMP )
                        cell.put(((oracle.sql.TIMESTAMP)element).timestampValue(), "");
                    else
                        cell.put(element, "");
                }
            }
        }

        // second pass
        Enumeration enm = result.keys();
        while( enm.hasMoreElements() ) {
            Hashtable record = (Hashtable)result.get(enm.nextElement());

            // following columns have to be created here
            Vector locations = new Vector();
            record.put("Location", locations);

            Vector address_spaces = new Vector();
            record.put("AddressSpace", address_spaces);

            Vector as_lowres = new Vector();
            record.put("LowRes", as_lowres);

            Vector as_highres = new Vector();
            record.put("HighRes", as_highres);

            Vector as_dhcp_options = new Vector();
            record.put("AddressSpace DHCP Option", as_dhcp_options);

            Vector as_groups = new Vector();
            record.put("AS Group", as_groups);

            Vector as_comment = new Vector();
            record.put("AScomment", as_comment);

            Vector as_ips = new Vector();
            if( selected("displayOrder.ni.11.Address") )
                record.put("Address", as_ips);

            Vector as_ip_names = new Vector();
            if( selected("displayOrder.ns.12.Name:DHCP Address") )
                record.put("Name:DHCP Address", as_ip_names);

            Vector as_ip_status = new Vector();
            if( selected("displayOrder.ns.13.Active Address") )
                record.put("Active Address", as_ip_status);

            Vector cell = new Vector();

            cell = new Vector();
            cell.addElement(((Hashtable)record.get("handle")).keys().nextElement());
            record.put("handle", cell);

            if( selected("displayOrder.txt.1.Name:Network") ) {
                cell = new Vector();
                Hashtable name = (Hashtable)((Hashtable)record.get("names")).elements().nextElement();
                cell.addElement(Utils.fullDisplayName((String)name.get("name"),
                                                      (String)name.get("domain"),
                                                      default_domain));
                record.put("Name:Network", cell);
            }

            // Locations
            if( selected("displayOrder.txt.2.Location") ) {
                Enumeration loc = Utils.nHash(record.get("LOCATION")).keys();
                while( loc.hasMoreElements() ) {
                    locations.addElement(loc.nextElement());
                }
            }

            if( selected("displayOrder.txt.4.Service") ) {
                cell = new Vector();
                Hashtable service = (Hashtable)record.get("Service");
                if( service != null && service.size() > 0 )
                    cell.addElement(service.keys().nextElement());
                record.put("Service", cell);
            }

            if( selected("txt.5.Network DHCPoption") ) {
                cell = new Vector();
                Enumeration options = Utils.nHash(record.get("Network DHCPoption")).keys();
                while( options.hasMoreElements() ) {
                    cell.addElement(options.nextElement());
                }
                record.put("Network DHCPoption", cell);
            }

            if( selected("displayOrder.txt.6.Network Group") ) {
                cell = new Vector();
                Enumeration groups = Utils.nHash(record.get("groups")).keys();
                while( groups.hasMoreElements() )
                    cell.addElement(groups.nextElement());
                record.put("Network Group", cell);
            }

            if( selected("displayOrder.txt.7.Comment") ) {
                cell = new Vector();
                Hashtable comment = (Hashtable)record.get("Comment");
                if( comment != null && comment.size() > 0 )
                    cell.addElement(comment.keys().nextElement());
                record.put("Comment", cell);
            }

            // created and mod fields
            cell = new Vector();
            Hashtable h = Utils.nHash(record.get("Created By"));
            if( h.size() > 0 ) cell.addElement(h.keys().nextElement());
            record.put("Created By", cell);

            cell = new Vector();
            h = Utils.nHash(record.get("Date Created"));
            if( h.size() > 0 ) cell.addElement(h.keys().nextElement());
            record.put("Date Created", cell);

            cell = new Vector();
            h = Utils.nHash(record.get("Modified By"));
            if( h.size() > 0 ) cell.addElement(h.keys().nextElement());
            record.put("Modified By", cell);

            cell = new Vector();
            h = Utils.nHash(record.get("Date Modified"));
            if( h.size() > 0 ) cell.addElement(h.keys().nextElement());
            record.put("Date Modified", cell);

            // now go over all address spaces
            Hashtable ases = Utils.nHash(record.get("address_spaces"));
            Object[] as_array = sorted_keys(ases, ip_cmp);
            for (int j = 0; j < as_array.length; j++) {
                Object as_key = as_array[j];
                Hashtable as = (Hashtable)ases.get(as_key);

                // padd dhcp options column
                int pad_amout = address_spaces.size() - as_dhcp_options.size();
                for( int k = 0; k < pad_amout; k++ )
                    as_dhcp_options.addElement("");

                // padd AS groups column
                pad_amout = address_spaces.size() - as_groups.size();
                for( int k = 0; k < pad_amout; k++ )
                    as_groups.addElement("");

                // padd ip related columns
                pad_amout = address_spaces.size() - as_ips.size();
                for( int k = 0; k < pad_amout; k++ ) {
                    as_ips.addElement("");
                    as_ip_names.addElement("");
                    as_ip_status.addElement("");
                }

                pad_amout = address_spaces.size() - as_comment.size();
                for( int k = 0; k < pad_amout; k++ )
                    as_comment.addElement("");

                // padd as_lowres
                pad_amout = address_spaces.size() - as_lowres.size();
                for( int k = 0; k < pad_amout; k++ )
                    as_lowres.addElement("");

                // padd as_highres
                pad_amout = address_spaces.size() - as_highres.size();
                for( int k = 0; k < pad_amout; k++ )
                    as_highres.addElement("");

                address_spaces.addElement(as_key);
                int as_padding = 1;

                // AS dhcp options column
                Object[] dhcp_options =
                    sorted_keys(Utils.nHash(as.get("dhcp_options")), string_cmp);
                for (int k = 0; k < dhcp_options.length; k++)
                    as_dhcp_options.addElement(dhcp_options[k]);

                if( as_padding < dhcp_options.length )
                    as_padding = dhcp_options.length;

                // AS groups column
                Object[] groups =
                    sorted_keys(Utils.nHash(as.get("groups")), string_cmp);
                for (int k = 0; k < groups.length; k++)
                    as_groups.addElement(groups[k]);

                if( as_padding < groups.length )
                    as_padding = groups.length;

                // AS comments
                as_comment.addElement(Utils.n2s(as.get("comments")));

                // LowRes and HighRes
                as_lowres.addElement(Utils.n2s(as.get("LOWRES")));
                as_highres.addElement(Utils.n2s(as.get("HIGHRES")));

                // ip addresses
                Hashtable ipAddresses = Utils.nHash(as.get("ips"));
                Object[] ips = sorted_keys(ipAddresses, ip_cmp);
                for (int k = 0; k < ips.length; k++) {
                    as_ips.addElement(ips[k]);
                    Hashtable ip = Utils.nHash(ipAddresses.get(ips[k]));
                    if( ip.get("name") == null )
                        as_ip_names.addElement("");
                    else
                        as_ip_names.
                            addElement(Utils.fullDisplayName((String)ip.get("name"),
                                                             (String)ip.get("domain"),
                                                             default_domain));

                    String active = (String)ip.get("active");
                    as_ip_status.addElement(active==null?"No":active);
                }

                if( as_padding < ips.length ) as_padding = ips.length;

                // padd current address space
                for( int k = 1; k < as_padding; k++ )
                    address_spaces.addElement("");
            }
            //record.remove("address_spaces");
        }

        this.result.put("result", result);
        return this.result;
    }
}
