/************************************************************************
 *
 * NodeCommand.java - contains all constants for NetDB_Node.master db call
 *
 * $Id: NodeCommand.java,v 1.1.1.1 2006/06/30 23:38:09 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.lang.*;
import java.util.*;
import java.sql.*;

/** contains all constants for NetDB_Node.master procedure call */
abstract class NodeCommand extends DBCall {

    protected Integer   op_uid;
    protected Integer   idNode;
    protected Timestamp lock;


    public static final int ACTION_INDEX                = 1;  // p_action
    public static final int OP_USER_INDEX               = 2;  // p_op_uid
    public static final int LOCKED_INDEX                = 3;  // p_lock
    public static final int ID_NODE_INDEX               = 4;  // p_node_id
    public static final int NAME_INDEX                  = 5;  // p_node
    public static final int ORGANIZATION_INDEX          = 6;  // p_org
    public static final int LOCATION_NAME_INDEX         = 7;  // p_location
    public static final int ROOM_INDEX                  = 8;  // p_room
    public static final int STATE_INDEX                 = 9;  // p_state
    public static final int MAKE_INDEX                  = 10; // p_make
    public static final int MODEL_INDEX                 = 11; // p_model
    public static final int COMMENT_INDEX               = 12; // p_comment
    public static final int EXPIRATION_INDEX            = 13; // p_expire_date
    public static final int DHCP_SERVICE_INDEX          = 14; // p_dhcpservice
    public static final int AS_INDEX                    = 15; // p_ipadddressspace

    public NodeCommand(stanford.netdb.utils.Logger logger) {
        super(logger);
        values = new Object[16];
        reset();
        values[ACTION_INDEX]                    = new Null(Types.VARCHAR);
        values[OP_USER_INDEX]                   = new Null(Types.NUMERIC);
        values[ID_NODE_INDEX]                   = new Null(Types.NUMERIC);
    }

    public String getSQL() {
        return proc_call_sql("NetDB_Node.master");
    }

    private void reset() {
        values[0]                   = ref_cursor;
        values[LOCKED_INDEX]        = new Null(Types.TIMESTAMP);
        values[NAME_INDEX]          = new Null(Types.VARCHAR);
        values[ORGANIZATION_INDEX]  = new Null(Types.VARCHAR);
        values[LOCATION_NAME_INDEX] = new Null(Types.VARCHAR);
        values[ROOM_INDEX]          = new Null(Types.VARCHAR);
        values[STATE_INDEX]         = new Null(Types.VARCHAR);
        values[MAKE_INDEX]          = new Null(Types.VARCHAR);
        values[MODEL_INDEX]         = new Null(Types.VARCHAR);
        values[COMMENT_INDEX]       = new Null(Types.VARCHAR);
        values[EXPIRATION_INDEX]    = new Null(Types.TIMESTAMP);
        values[DHCP_SERVICE_INDEX]  = new Null(Types.VARCHAR);
        values[AS_INDEX]            = new Null(Types.VARCHAR);
    }

    protected Hashtable doCheck(NetDBConnection con) throws NetDBException, Exception {
        reset();
        values[ACTION_INDEX]  = "verify";
        values[OP_USER_INDEX] = op_uid;
        values[ID_NODE_INDEX] = idNode;

        return con.callDB(this);
    }

    protected void throwSimpleNetDBException(String msgHeader, String msg) throws NetDBException {
        Vector errors = new Vector();
        errors.addElement(msg);
        throw new NetDBException(msgHeader, errors);
    }
}
