/************************************************************************
 *
 * NodeFullSearchCommand.java -
 *
 * $Id: NodeFullSearchCommand.java,v 1.5 2006/11/01 21:51:53 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.text.*;
import java.util.*;

class NodeFullSearchCommand extends FullSearchCommand {

    public NodeFullSearchCommand(stanford.netdb.utils.Logger logger,
                                 Hashtable search_params)
    {
        super(logger, search_params);
    }

    public String getSearchSQL() throws Exception {

        String sql = "WITH Records AS ( SELECT unique(id) record_id FROM ";

        // fill in display order vector
        Vector displayOrder = new Vector();
        init_result(search_params, displayOrder, "Nodes");

        // go over displayOrder.XXX parameters and assemble displayOrder Vector
        assmDisplayOrder(displayOrder, search_params);

        // do not search by name if no name type is selected
        if( spNull("cb.i.Name:Node")      &&
            spNull("cb.i.Name:Alias")     &&
            spNull("cb.i.Name:Interface") &&
            spNull("cb.i.Name:IP")        &&
            spNull("cb.i.Name:IPC") )
            search_params.remove("fs.FS_NameNode.txt.fullname_lc");

        // do some lower case conversion
        if( !spNull("fs.FS_NameNode.txt.fullname_lc") )
            search_params.put("fs.FS_NameNode.txt.fullname_lc",
                ((String)search_params.get("fs.FS_NameNode.txt.fullname_lc")).toLowerCase());

        if( !spNull("fs.FS_NodeMX.txt.fullname_lc") )
            search_params.put("fs.FS_NodeMX.txt.fullname_lc",
                ((String)search_params.get("fs.FS_NodeMX.txt.fullname_lc")).toLowerCase());

        // parse 'custom field' field
        String customFields = (String)search_params.get("txt.customFields");
        if( customFields != null && customFields.trim().length() > 0 )
            search_params.put("fs.FS_Node.txt.lower(custom_field)",
                              customFields.replaceAll("#", "#hsh").replaceAll("\\\\\\\\", "&bsl;").replaceAll("\\\\=", "#eql").replaceAll("&bsl;", "\\\\\\\\"));

        // parse 'dhcp options' field
        String dhcp_options = (String)search_params.get("txt.DHCPoptions");
        if( dhcp_options != null && dhcp_options.trim().length() > 0 )
            search_params.put("fs.FS_NodeIPaddress.txt.lower(dhcp_option)",
                              dhcp_options.replaceAll("#", "#hsh").replaceAll("\\\\\\\\", "&bsl;").replaceAll("\\\\=", "#eql").replaceAll("&bsl;", "\\\\\\\\"));

        // we need to trick here sql builder to do the search even if no actuall
        // search parameters provied but "InMyGroups" checkbox is checked,
        // which is done by forcing search for name '*'
        if( ((String)Utils.n2s(search_params.get("cb.i.f.InMyGroups"))).length()>0 &&
            ((String)Utils.n2s(search_params.get("fs.FS_NameNode.txt.fullname_lc"))).trim().equals("") ) {
            search_params.put("fs.FS_NameNode.txt.fullname_lc", "*");
            search_params.put("cb.i.Name:Node", "node is not null");
        }


        // continue with assembling sql 'where' clause
        sql += assmCommonSQL(search_params) + " ) ";

        Vector views = new Vector();
        boolean node_info_names_added = false;
        String info_select = "\nSELECT id \"ID\", to_char(id) \"handle\"";
        if( displayed("Name:Node")      ||
            displayed("Name:Alias")     ||
            displayed("Name:Interface") ||
            displayed("Name:IP")        ||
            displayed("Name:IPC") ) {
            info_select += ", name_id, name, domain, aliasee, interface, node, IPaddress, ip_name_interface";
            node_info_names_added = true;
            views.addElement("FS_NodeInfoNames");
        }

        if( displayed("Address")        ||
            displayed("Active Address") ||
            displayed("PTR Pref") ) {
            info_select += ", ipc_address_txt, ipc_active, ip_interface, ip_address_txt, ip_active, PTRpref";
            views.addElement("FS_NodeInfoIPaddresses");
            views.addElement("FS_NodeInfoIPC");
        }

        if( displayed("Hardware Address") ||
            displayed("DHCP")             ||
            displayed("DHCP Option")      ||
            displayed("Roaming") ) {
            info_select += ", interface_id, interface_hw, interface_roam, interface_dhcp, dhcp_options";
            views.addElement("FS_NodeInfoInterfaces");
        }

        if( displayed("Receives Mail For") ) {
            info_select += ", mx_name, mx_domain, mx_received_by, mx_pref";
            if( !node_info_names_added )
                views.addElement("FS_NodeInfoNames");
        }

        info_select +=
            (!displayed("Node Type")          ?"":", type \"Node Type\"")+
            (!displayed("State")              ?"":", state \"State\"")+
            (!displayed("Group")              ?"":", owner \"Group\"")+
            (!displayed("Department")         ?"":", organization \"Department\"")+
            (!displayed("Location")           ?"":", location \"Location\"")+
            (!displayed("Site")               ?"":", site \"Site\"")+
            (!displayed("Code")               ?"":", code \"Code\"")+
            (!displayed("Room")               ?"":", room \"Room\"")+
            (!displayed("Expires On")         ?"":", expiration \"Expires On\"")+
            (!displayed("Make/Model")         ?"":", make_and_model \"Make/Model\"")+
            (!displayed("Operating System")   ?"":", os \"Operating System\"")+
            (!displayed("Node Administrator") ?"":", node_admin \"Node Administrator\"")+
            (!displayed("Node User")          ?"":", node_user \"Node User\"")+
            (!displayed("Node Tag")           ?"":", tag \"Node Tag\"")+
            (!displayed("Comment")            ?"":", comments \"Comment\"")+
            modBlock(search_params);

        views.addElement("FS_NodeInfo");

        for( int i = 0; i < views.size(); i++ ) {
            sql += info_select + " FROM " + views.elementAt(i)+
            " JOIN Records on ID = Records.record_id";
            if( i+1 < views.size() )
                sql += "\nUNION";

        }
        return sql;
    }

    // Node Full Search has custom picker
    public Hashtable processResultSet(Hashtable res) throws Exception {
        if( res.get("errors") != null ) {
            Hashtable errors = new Hashtable();
            errors.put("errors", res.get("errors"));
            return errors;
        }

        Vector column_names = (Vector)res.get("column_names");
        Hashtable result = new Hashtable();

        Vector result_set = (Vector)res.get("resultSet");
        for( int i = 0; i < result_set.size(); i++ ) {
            Vector row = (Vector)result_set.elementAt(i);
            String id = row.elementAt(0).toString();

            Hashtable record = get_hash_or_create(result, id);

            Hashtable ip_addresses = get_hash_or_create(record, "ip_addresses");

            Hashtable name   = new Hashtable();
            Hashtable mx     = new Hashtable();
            String name_fqdn = "";
            String mx_fqdn   = "";
            Hashtable ipH    = new Hashtable();
            Hashtable ipcH   = new Hashtable();
            Hashtable intfcH = new Hashtable();

            for( int j = 0; j < row.size(); j++ ) {
                String column = (String)column_names.elementAt(j);
                // handle name related columns
                if( ( column.equals("NAME_ID")           ||
                      column.equals("NAME")              ||
                      column.equals("MX_NAME")           ||
                      column.equals("DOMAIN")            ||
                      column.equals("MX_DOMAIN")         ||
                      column.equals("ALIASEE")           ||
                      column.equals("INTERFACE")         ||
                      column.equals("MX_RECEIVED_BY")    ||
                      column.equals("MX_PREF")           ||
                      column.equals("IPADDRESS")         ||
                      column.equals("IP_NAME_INTERFACE") ||
                      column.equals("NODE") ) && row.elementAt(j) != null )
                    {

                        if( !column.equals("MX_NAME") && !column.equals("MX_DOMAIN") && !column.equals("MX_RECEIVED_BY") )
                            name.put(column, row.elementAt(j).toString());

                        if( column.equals("MX_NAME") || column.equals("MX_DOMAIN") || column.equals("MX_RECEIVED_BY") || column.equals("MX_PREF") )
                            mx.put(column, row.elementAt(j).toString());

                        if( column.equals("NAME") || column.equals("DOMAIN") )
                            name_fqdn += (name_fqdn.length()>0?".":"")+row.elementAt(j).toString().toLowerCase().trim();

                        if( column.equals("MX_NAME") || column.equals("MX_DOMAIN") || column.equals("MX_RECEIVED_BY") )
                            mx_fqdn += (mx_fqdn.length()>0?".":"")+row.elementAt(j).toString().toLowerCase().trim();

                        continue;
                    }

                // handle ipc related columns
                if( ( column.equals("IPC_ADDRESS_TXT") || column.equals("IPC_ACTIVE") ) &&
                    row.elementAt(j) != null ) {
                    ipcH.put(column, row.elementAt(j));
                    continue;
                }

                if( ( column.equals("IP_INTERFACE")   ||
                      column.equals("IP_ADDRESS_TXT") ||
                      column.equals("IP_ACTIVE")      ||
                      column.equals("PTRPREF") ) && row.elementAt(j) != null ) {
                    ipH.put(column, row.elementAt(j));
                    continue;
                }

                if( ( column.equals("INTERFACE_ID")   ||
                      column.equals("INTERFACE_HW")   ||
                      column.equals("DHCP_OPTIONS")   ||
                      column.equals("INTERFACE_DHCP") ||
                      column.equals("INTERFACE_ROAM") ) && row.elementAt(j) != null ) {
                    intfcH.put(column, row.elementAt(j));
                    continue;
                }

                if( ( column.equals("Created By") || column.equals("Modified By") ) &&
                    row.elementAt(j) != null && row.elementAt(j).toString().trim().equals("()") )
                    continue;

                // handle the rest of the columns
                Hashtable cell = (Hashtable)record.get(column_names.elementAt(j));
                if( cell == null ) {
                    cell = new Hashtable();
                    record.put(column_names.elementAt(j), cell);
                }

                if( row.elementAt(j) != null ) {
                    // reqular columns
                    Object element = row.elementAt(j);
                    if( element instanceof oracle.sql.DATE )
                        cell.put(((oracle.sql.DATE)element).timestampValue(), "");
                    else if( element instanceof oracle.sql.TIMESTAMP )
                        cell.put(((oracle.sql.TIMESTAMP)element).timestampValue(), "");
                    else
                        cell.put(element, "");
                }
            }

            if( ipcH.size() > 0 )
                get_hash_or_create(get_hash_or_create(get_hash_or_create(record, "interfaces"), "ipc"), "ips").
                    put(ipcH.get("IPC_ADDRESS_TXT"), ipcH);

            if( ipH.size() > 0 ) {
                if( ipH.get("IP_INTERFACE") != null )
                    get_hash_or_create(get_hash_or_create(get_hash_or_create(record, "interfaces"), ipH.get("IP_INTERFACE").toString()), "ips").
                        put(ipH.get("IP_ADDRESS_TXT"), ipH);
                else // ip address is a prefix in a template
                    get_hash_or_create(get_hash_or_create(get_hash_or_create(record, "interfaces"), "AS"), "ips").
                        put(ipH.get("IP_ADDRESS_TXT"), ipH);
            }

            if( intfcH.size() > 0 ) {
                get_hash_or_create(get_hash_or_create(record, "interfaces"), intfcH.get("INTERFACE_ID").toString()).put("flags", intfcH);
                if( intfcH.get("DHCP_OPTIONS") != null )
                    get_hash_or_create(
                     get_hash_or_create(
                      get_hash_or_create(record, "interfaces"),
                      intfcH.get("INTERFACE_ID").toString()),
                     "dhcp options").put(intfcH.get("DHCP_OPTIONS"), "");
            }

            if( name_fqdn.length() > 0 )
                get_hash_or_create(record, "names").put(name_fqdn, name);

            if( mx_fqdn.length() > 0 )
                get_hash_or_create(record, "names").put(mx_fqdn, mx);
        }

        String default_domain = (String)((NetDBii)logger).getUser().get("default_domain");

        // go over the result and convert cells into vectors
        Comparator name_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Hashtable name1 = (Hashtable)((Map.Entry)o1).getValue();
                    Hashtable name2 = (Hashtable)((Map.Entry)o2).getValue();

                    if( name1.get("INTERFACE") != null && name2.get("INTERFACE") != null &&
                        !name1.get("INTERFACE").equals(name2.get("INTERFACE")) )
                        return name1.get("INTERFACE").toString().
                            compareToIgnoreCase(name2.get("INTERFACE").toString());

                    return (name1.get("NAME")+"."+name1.get("DOMAIN")).
                        compareToIgnoreCase(name2.get("NAME")+"."+name2.get("DOMAIN"));
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        Comparator intfc_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    if( o1.toString().equals("ipc") )
                        return 1;

                    if( o2.toString().equals("ipc") )
                        return -1;

                    long cmp_val = Long.parseLong(o1.toString())-Long.parseLong(o2.toString());
                    return cmp_val>0?1:(cmp_val<0?-1:0);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        Enumeration enm = result.keys();
        while( enm.hasMoreElements() ) {
            Hashtable record = (Hashtable)result.get(enm.nextElement());

            // following columns have to be created here
            Vector name_node = new Vector();
            record.put("Name:Node", name_node);

            Vector name_alias = new Vector();
            record.put("Name:Alias", name_alias);

            Vector intfcs = new Vector();
            record.put("Interface#", intfcs);

            Vector name_interface = new Vector();
            record.put("Name:Interface", name_interface);

            Vector name_ip = new Vector();
            record.put("Name:IP", name_ip);

            Vector name_ipc = new Vector();
            record.put("Name:IPC", name_ipc);

            Vector name_mx = new Vector();
            record.put("Receives Mail For", name_mx);

            Vector hw_address = new Vector();
            record.put("Hardware Address", hw_address);

            Vector dhcp_options = new Vector();
            record.put("DHCP Option", dhcp_options);

            Vector dhcp_flag = new Vector();
            record.put("DHCP", dhcp_flag);

            Vector roaming_flag = new Vector();
            record.put("Roaming", roaming_flag);

            Vector ip_addresses = new Vector();
            record.put("Address", ip_addresses);

            Vector ptr_prefs = new Vector();
            record.put("PTR Pref", ptr_prefs);

            Vector ip_addresses_status = new Vector();
            record.put("Active Address", ip_addresses_status);

            // we will reffer node names from this hashtable by their ids
            Hashtable node_names = new Hashtable();

            // names of interfaces
            Hashtable intfc_names = new Hashtable();

            // names of ip addresses
            Hashtable ip_names = new Hashtable();

            Hashtable interfaces = get_hash_or_create(record, "interfaces");

            Enumeration cells = record.keys();
            while( cells.hasMoreElements() ) {
                Object column_name = cells.nextElement();
                if( column_name.equals("names") ) {
                    Hashtable names = Utils.nHash(record.get(column_name));

                    Object[] names_array = Arrays.asList(names.keySet().toArray()).toArray();
                    // first walk through all names
                    for (int i = 0; i < names_array.length; ++i) {
                        Hashtable name = (Hashtable)names.get(names_array[i]);

                        if(        name.get("NODE") != null ) { // node name
                            node_names.put(name.get("NAME_ID"), name);
                            name.put("aliases", new Hashtable());
                            name.put("mxes", new Hashtable());

                        } else if( name.get("INTERFACE") != null ) { // interface name
                            get_hash_or_create(get_hash_or_create(interfaces, name.get("INTERFACE")),
                                               "names").put(name.get("NAME_ID"), name);

                        } else if( name.get("IPADDRESS") != null ) { // ip name
                            Hashtable intfc = get_hash_or_create(interfaces, name.get("IP_NAME_INTERFACE"));
                            Hashtable ips = get_hash_or_create(intfc, "ips");
                            Hashtable ipaddress = get_hash_or_create(ips, name.get("IPADDRESS"));
                            get_hash_or_create(ipaddress, "names").put(name.get("NAME_ID"), name);
                        }
                    }

                    // second walk through all names, pick up aliases and mxes
                    for (int i = 0; i < names_array.length; ++i) {
                        Hashtable name = (Hashtable)names.get(names_array[i]);

                        if( name.get("ALIASEE") != null )
                            ((Hashtable)(Utils.nHash(node_names.get(name.get("ALIASEE")))).
                             get("aliases")).put(name.get("NAME_ID"), name);
                        if( name.get("MX_NAME") != null )
                            ((Hashtable)(Utils.nHash(node_names.get(name.get("MX_RECEIVED_BY")))).
                             get("mxes")).put(names_array[i], name);
                    }

                    continue;
                } else if( column_name.equals("interfaces") ) {
                    continue;
                }

                Object cell = record.get(column_name);
                if( cell instanceof Hashtable ) {
                    Vector vCell = new Vector();
                    Enumeration cell_enm = ((Hashtable)cell).keys();
                    while( cell_enm.hasMoreElements() ) {
                        vCell.addElement(cell_enm.nextElement());
                    }
                    record.put(column_name, vCell);
                }
            }

            // prepare vector columns for names
            java.util.List alist = Arrays.asList(node_names.entrySet().toArray());
            Collections.sort(alist, name_cmp);
            Object[] names_array = alist.toArray();
            for (int i = 0; i < names_array.length; i++) {
                Hashtable name = (Hashtable)((Map.Entry)names_array[i]).getValue();
                int padded = 0;
                int total_aliases_and_mxes_shown = displayed("Name:Node")?1:0;
                name_node.addElement(Utils.fullDisplayName((String)name.get("NAME"),
                                                           (String)name.get("DOMAIN"),
                                                           default_domain));

                int aliases_total = 0;
                int mxes_total    = 0;

                // aliases if we need them
                if( displayed("Name:Alias") ) {
                    Hashtable aliases = Utils.nHash(name.get("aliases"));
                    total_aliases_and_mxes_shown = total_aliases_and_mxes_shown<aliases.size()?aliases.size():total_aliases_and_mxes_shown;
                    aliases_total = aliases.size();
                    java.util.List clist = Arrays.asList(aliases.entrySet().toArray());
                    Collections.sort(clist, name_cmp);
                    Object[] aliases_array = clist.toArray();
                    for (int j = 0; j < aliases_array.length; j++) {
                        Hashtable alias = (Hashtable)((Map.Entry)aliases_array[j]).getValue();
                        name_alias.addElement(Utils.fullDisplayName((String)alias.get("NAME"),
                                                                    (String)alias.get("DOMAIN"),
                                                                    default_domain));
                        if( j > 0 ) { // padding element
                            name_node.addElement("");
                            padded++;
                        }
                    }
                }

                // mxes if we need them
                if( displayed("Receives Mail For") ) {
                    Hashtable mxes = Utils.nHash(name.get("mxes"));
                    total_aliases_and_mxes_shown = total_aliases_and_mxes_shown<mxes.size()?mxes.size():total_aliases_and_mxes_shown;
                    mxes_total = mxes.size();
                    java.util.List mlist = Arrays.asList(mxes.entrySet().toArray());
                    Collections.sort(mlist, name_cmp);
                    Object[] mxes_array = mlist.toArray();
                    for (int j = 0; j < mxes_array.length; j++) {
                        Hashtable mx = (Hashtable)((Map.Entry)mxes_array[j]).getValue();
                        name_mx.addElement(Utils.fullDisplayName((String)mx.get("MX_NAME"),
                                                                 (String)mx.get("MX_DOMAIN"),
                                                             default_domain)+" ("+mx.get("MX_PREF")+")");
                        if( j > padded ) // padding element
                            name_node.addElement("");
                    }
                }

                // pad mx column
                for( int j=0; j<(total_aliases_and_mxes_shown-mxes_total); j++)
                    name_mx.addElement("");

                for( int j=0; j<(total_aliases_and_mxes_shown-aliases_total); j++)
                    name_alias.addElement("");
            }

            // now assemble information related to interfaces
            Object[] interfaces_array = sorted_keys(interfaces, intfc_cmp);
            int intfc_counter = 0;
            for (int i = 0; i < interfaces_array.length; i++) {
                Object key = interfaces_array[i];
                Hashtable intfc = (Hashtable)interfaces.get(key);
                if( key.toString().equals("ipc") )
                    intfcs.addElement("IPC");
                else if( key.toString().equals("AS") )
                    intfcs.addElement("AS");
                else
                    intfcs.addElement(""+(++intfc_counter));
                // interface names
                if( displayed("Name:Interface") ) {
                    Hashtable names = get_hash_or_create(intfc, "names");
                    if( names.size() > 0 ) { // interface may have only one name
                        Hashtable name = (Hashtable)names.elements().nextElement();
                        name_interface.addElement(Utils.fullDisplayName((String)name.get("NAME"),
                                                                        (String)name.get("DOMAIN"),
                                                                        default_domain));
                    } else
                        name_interface.addElement("");
                }

                int total_rows_displayed     = 1; // there is always at least one row in interface
                int ipc_names_displayed    = 0;
                int ip_names_displayed     = 0;
                int ips_displayed          = 0;
                int dhcp_options_displayed = 0;
                Hashtable ips = get_hash_or_create(intfc, "ips");
                Object[] ips_array = sorted_keys(ips, ip_cmp);
                for (int j = 0; j < ips_array.length; j++) {
                    Object ip_key = ips_array[j];
                    Hashtable ipH = (Hashtable)ips.get(ip_key);
                    Hashtable names = get_hash_or_create(ipH, "names");
                    if( key.toString().equals("ipc") ) {
                        if( displayed("Name:IPC") ) {
                            if( names.size() > 0 ) {
                                Hashtable name = (Hashtable)names.elements().nextElement();
                                name_ipc.addElement(Utils.fullDisplayName((String)name.get("NAME"),
                                                                          (String)name.get("DOMAIN"),
                                                                          default_domain));
                                ipc_names_displayed++;
                             } else {
                                 if( displayed("Address")        ||
                                     displayed("Active Address") ||
                                     displayed("PTR Pref") ) {
                                     name_ipc.addElement("");
                                     ipc_names_displayed++;
                                 }
                            }
                        }

                    } else { // regular interface
                        if( displayed("Name:IP") ) {
                            if( names.size() > 0 ) {
                                Hashtable name = (Hashtable)names.elements().nextElement();
                                name_ip.addElement(Utils.fullDisplayName((String)name.get("NAME"),
                                                                          (String)name.get("DOMAIN"),
                                                                          default_domain));
                                if( displayed("Name:IPC") ) {
                                    name_ipc.addElement("");
                                    ipc_names_displayed++;
                                }
                                ip_names_displayed++;
                            } else {
                                if( displayed("Address")        ||
                                    displayed("Active Address") ||
                                    displayed("PTR Pref") ) {
                                    name_ip.addElement("");
                                    ip_names_displayed++;
                                    if( displayed("Name:IPC") ) {
                                        name_ipc.addElement("");
                                        ipc_names_displayed++;
                                    }
                                }
                            }
                        }
                    }

                    if( displayed("Address") ) {
                        ip_addresses.addElement(ip_key);
                        ips_displayed++;
                    }

                    if( displayed("PTR Pref") && ipH.get("PTRPREF") != null ) {
                        ptr_prefs.addElement(ipH.get("PTRPREF"));
                        if( !displayed("Address") && !displayed("Active Address") ) // 'PTR Pref' is a driving column here
                            ips_displayed++;
                    }

                    if( displayed("Active Address") ) {
                        Object active = ipH.get("IP_ACTIVE");
                        if( active == null ) active = ipH.get("IPC_ACTIVE");
                        ip_addresses_status.addElement(active);
                        if( !displayed("Address") ) // active address is a driving column here
                            ips_displayed++;
                    }

                } // end of for(...) loop

                if( displayed("Hardware Address") )
                    hw_address.addElement(n2s(get_hash_or_create(intfc, "flags").get("INTERFACE_HW")));

                if( displayed("DHCP") )
                    dhcp_flag.addElement(n2s(get_hash_or_create(intfc, "flags").get("INTERFACE_DHCP")));

                if( displayed("Roaming") )
                    roaming_flag.addElement(n2s(get_hash_or_create(intfc, "flags").get("INTERFACE_ROAM")));

                if( displayed("DHCP Option") ) {
                    Object[] dhcp_options_array =
                        sorted_keys(get_hash_or_create(intfc, "dhcp options"), string_cmp);
                    for (int j = 0; j < dhcp_options_array.length; j++)
                        dhcp_options.addElement(dhcp_options_array[j]);
                    dhcp_options_displayed = dhcp_options_array.length;
                }

                total_rows_displayed = ipc_names_displayed>total_rows_displayed?ipc_names_displayed:total_rows_displayed;
                total_rows_displayed = ip_names_displayed>total_rows_displayed?ip_names_displayed:total_rows_displayed;
                total_rows_displayed = ips_displayed>total_rows_displayed?ips_displayed:total_rows_displayed;
                total_rows_displayed = dhcp_options_displayed>total_rows_displayed?dhcp_options_displayed:total_rows_displayed;

                // add empty rows to interface, hw and other columns
                for( int j = 1; j < total_rows_displayed; j++ ) {
                    intfcs.addElement("");
                    hw_address.addElement("");
                    dhcp_flag.addElement("");
                    roaming_flag.addElement("");
                    name_interface.addElement("");
                }

                for( int j = 0; j < total_rows_displayed-ips_displayed; j++ ) {
                    ip_addresses.addElement("");
                    ip_addresses_status.addElement("");
                }

                for( int j = 0; j < total_rows_displayed-ipc_names_displayed; j++ )
                    name_ipc.addElement("");

                for( int j = 0; j < total_rows_displayed-ip_names_displayed; j++ )
                    name_ip.addElement("");

                for( int j = 0; j < total_rows_displayed-dhcp_options_displayed; j++ )
                    dhcp_options.addElement("");
            }
        }

        this.result.put("result", result);
        return this.result;
    } // end of processResultSet(...)

}
