/************************************************************************
 *
 * SystemParameterCommand.java - glue for old full search
 *
 * $Id: SystemParameterCommand.java,v 1.1.1.1 2006/06/30 23:38:10 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.lang.*;
import java.util.*;
import java.sql.*;

/** contains all constants for pNode procedure call */
class SystemParameterCommand extends DBCall {

    public static final int ACTION_INDEX                = 1;  // @wrdAction
    public static final int LABEL_INDEX                 = 2;  // @snmLabel
    public static final int DEFAULT_ID_INDEX            = 3;  // @idDefault
    public static final int STR_VALUE_INDEX             = 4;  // @strValue
    public static final int INT_VALUE_INDEX             = 5;  // @intValue
    public static final int DESCRIPTION_INDEX           = 6;  // @dsDescription
    public static final int VALUE_ID_INDEX              = 7;  // @idValue

    public SystemParameterCommand(stanford.netdb.utils.Logger logger,
                                  String paramName)
    {
        super(logger);
        values = new Object[8];
        values[0]                       = new OutParam(Types.NUMERIC);
        values[ACTION_INDEX]            = "info";
        values[LABEL_INDEX]             = paramName;
        values[DEFAULT_ID_INDEX]        = new Null(Types.NUMERIC);
        values[STR_VALUE_INDEX]         = new Null(Types.VARCHAR);
        values[INT_VALUE_INDEX]         = new Null(Types.NUMERIC);
        values[DESCRIPTION_INDEX]       = new Null(Types.VARCHAR);
        values[VALUE_ID_INDEX]          = new Null(Types.NUMERIC);
    }

    public int errorResultSetIndex() {
        return -1;
    }

    public String getSQL() {
        return "{?= call pSystemParameters ( ?, ?, ?, ?, ?, ?, ?)}";
    }

    public SystemParameter packResultSet(Hashtable res) throws Exception {
        Vector result_sets = (Vector)res.get("result_sets");
        for( int i = 0; i < result_sets.size(); i++ ) {
            Hashtable result_set = (Hashtable)result_sets.elementAt(i);
            Vector rs = (Vector)result_set.get("data");
            ResultSetMetaData md = (ResultSetMetaData)result_set.get("meta_data");

            // uncomment to see the row data
            //printRawData(result_set);

            if( md.getColumnName(2).equals("Label") ) { // single node information
                for( int j = 0; j < rs.size(); j++ ) {
                    Vector row = (Vector)rs.elementAt(j);

                    Object label = row.elementAt(1);

                    Object strValue = row.elementAt(2);
                    Object intValue = row.elementAt(3);
                    Object idValue = row.elementAt(4);
                    Object description = row.elementAt(2);

                    return new SystemParameter((label instanceof String)?((String)label).trim():null,
                                               null, (description instanceof String)?((String)description).trim():null,
                                               (strValue instanceof String)?((String)strValue).trim():null,
                                               (strValue instanceof String)?((String)strValue).trim().toLowerCase():null,
                                               (intValue instanceof Integer)?((Integer)intValue).intValue():0,
                                               (idValue instanceof Integer)?((Integer)idValue).intValue():0);
                }
            }
        }
        return null;
    }
}
