/************************************************************************
 *
 * UserFullSearchCommand.java -
 *
 * $Id: UserFullSearchCommand.java,v 1.3 2006/10/06 20:01:48 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.util.*;
import java.text.*;

class UserFullSearchCommand extends FullSearchCommand {

    public UserFullSearchCommand(stanford.netdb.utils.Logger logger,
                                 Hashtable search_params)
    {
        super(logger, search_params);
    }

    public String getSearchSQL() throws Exception {
        String sql =
            "SELECT id \"ID\", netid \"handle\", name || '(' || netid || ')' \"User Name\""+
            (!selected("displayOrder.b.2.Active")?"":", decode(active, 1, 'Yes', 'No') \"Active\"")+
            (!selected("displayOrder.tst.3.Last Login")?"":", last_login \"Last Login\"")+
            ", decode(official, 1, 'Yes', 'No') \"LNA\", organization \"Department\", nvl(cons_name, ' ') \"Network Consultant\""+
            (!selected("displayOrder.txt.7.Group")?"":", owner \"Group\", decode(all_groups, 1, 'Yes', 'No') \"All Groups\"")+
            (!selected("displayOrder.txt.8.Record Type")?"":", nvl(access_dsp, ' ') \"Record Type\", decode(all_records, 1, 'Yes', 'No') \"All Types\"")+
            (!selected("displayOrder.txt.9.Comment")?"":", nvl(comments, ' ') \"Comment\"")+
            (!selected("displayOrder.txt.10.Phone")?"":", nvl(phone, ' ') \"Phone\"")+
            (!selected("displayOrder.txt.11.Email")?"":", nvl(email, ' ') \"Email\"")+
            (!selected("displayOrder.txt.12.Default Domain")?"":", nvl(def_domain, ' ') \"Default Domain\"")+
            (!selected("displayOrder.ip.13.Default AddressSpace")?"":", decode(def_addr_space, null, ' ', IP.text(def_addr_space)) \"Default AddressSpace\"")+
            (!selected("displayOrder.txt.14.Default Group")?"":", nvl(def_group, ' ') \"Default Group\"")+
            modBlock(search_params)+
            " FROM "+search_params.get("fs.table")+
            " WHERE ID IN ( SELECT unique(id) FROM ";

        search_params.put("displayOrder.txt.7.All Groups", search_params.get("displayOrder.txt.7.Group"));
        search_params.put("displayOrder.txt.8.All Types",  search_params.get("displayOrder.txt.8.Record Type"));

        // fill in display order vector
        Vector displayOrder = new Vector();
        init_result(search_params, displayOrder, "Users");

        // go over displayOrder.XXX parameters and assemble displayOrder Vector
        assmDisplayOrder(displayOrder, search_params);

        // some fiddling with access groups
        if( n2s(search_params.get("rb.b.records")).equals("0") ) {
            // we are not searching by the following fields unless rb.b.all_records is 0
            search_params.remove("cb.i.Node");
            search_params.remove("cb.i.Network");
            search_params.remove("cb.i.Domain");
            search_params.remove("cb.i.User");
            search_params.remove("cb.i.Group");
            search_params.remove("cb.i.AdminTeam");
            search_params.remove("cb.i.IPC");
            search_params.remove("cb.i.AdvancedNode");
            search_params.remove("cb.i.Router");
            search_params.remove("cb.i.SystemManagement");
        }

        // continue with assembling sql 'where' clause
        sql += assmCommonSQL(search_params) + " ) ";

        return sql;
    }

    public Hashtable processResultSet(Hashtable res) throws Exception {
        if( res.get("errors") != null ) {
            Hashtable errors = new Hashtable();
            errors.put("errors", res.get("errors"));
            return errors;
        }

        Vector column_names = (Vector)res.get("column_names");
        Hashtable result = new Hashtable();
        Vector result_set = (Vector)res.get("resultSet");
        Object lna_flag = "";
        Object department_name = "";
        Hashtable dept = new Hashtable();

        for( int i = 0; i < result_set.size(); i++ ) {
            Vector row = (Vector)result_set.elementAt(i);

            String id = row.elementAt(0).toString();
            Hashtable rawH = (Hashtable)result.get(id);
            if( rawH == null ) {
                rawH = new Hashtable();
                result.put(id, rawH);
            }

            Hashtable depts_and_lna = get_hash_or_create(rawH, "Departments");

            for( int j = 0; j < row.size(); j++ ) {
                Object column_name = column_names.elementAt(j);

                Hashtable cell = (Hashtable)rawH.get(column_names.elementAt(j));
                if( cell == null ) {
                    cell = new Hashtable();
                    rawH.put(column_name, cell);
                }

                if( row.elementAt(j) != null ) {
                    Object element = row.elementAt(j);

                    if( column_name.equals("LNA") )
                        lna_flag = element;

                    if( column_name.equals("Department") )
                        department_name = element;

                    if( column_name.equals("Network Consultant") ) {
                        // we are relying here on the fact that this column will always show after "LNA" and "Department"
                        dept = get_hash_or_create(depts_and_lna, department_name);
                        dept.put("LNA", lna_flag);
                        get_hash_or_create(dept, "cons").put(element, "");
                    }

                    if( element instanceof oracle.sql.DATE )
                        cell.put(((oracle.sql.DATE)element).timestampValue(), "");
                    else if( element instanceof oracle.sql.TIMESTAMP )
                        cell.put(((oracle.sql.TIMESTAMP)element).timestampValue(), "");
                    else
                        cell.put(element, "");
                }
                rawH.put(column_name, cell);
            }
        }

        // go over the result and convert cells into vectors
        Enumeration enm = result.keys();
        while( enm.hasMoreElements() ) {
            Hashtable rawH = (Hashtable)result.get(enm.nextElement());
            Enumeration cells = rawH.keys();
            while( cells.hasMoreElements() ) {
                Object column_name = cells.nextElement();
                if( column_name.equals("Departments") ) continue;
                Hashtable cell = (Hashtable)rawH.get(column_name);
                Vector vCell = new Vector();
                Enumeration cell_enm = cell.keys();
                while( cell_enm.hasMoreElements() ) {
                    vCell.addElement(cell_enm.nextElement());
                }
                rawH.put(column_name, vCell);
            }

            // fiddle with departments
            Vector department     = new Vector();
            Vector consultant     = new Vector();
            Vector lna            = new Vector();
            Hashtable departments = Utils.nHash(rawH.get("Departments"));
            Object[] departments_array = sorted_keys(departments, string_cmp);
            for (int j = 0; j < departments_array.length; j++) {
                department.addElement(departments_array[j]);
                dept = Utils.nHash(departments.get(departments_array[j]));
                lna.addElement(Utils.n2s(dept.get("LNA")));
                Hashtable cons = Utils.nHash(dept.get("cons"));
                Object[] cons_array = sorted_keys(cons, string_cmp);
                for (int k = 0; k < cons_array.length; k++)
                    consultant.addElement(cons_array[k]);

                if( selected("displayOrder.ns.6.Network Consultant") ) {
                    // we need to add empty rows to lna and department columns
                    for (int k = 0; k < cons_array.length - 1; k++ ) {
                        lna.addElement("");
                        department.addElement("");
                    }
                }
            }

            // now if user did not ask to display neither LNA flag nor Department name then
            // we resort "Network Consultant" column and display only unique rows
            if( !selected("displayOrder.ns.4.LNA") && !selected("displayOrder.ns.5.Department") ) {
                Hashtable cons = new Hashtable();
                for (int j = 0; j < consultant.size(); j++ ) {
                    String cons_name = consultant.elementAt(j).toString().trim();
                    if( cons_name.length() > 0 ) cons.put(cons_name, "");
                }

                consultant = new Vector();
                Object[] cons_array = sorted_keys(cons, string_cmp);
                for (int k = 0; k < cons_array.length; k++)
                    consultant.addElement(cons_array[k]);
            }

            // now put everything back where it was
            rawH.put("LNA",                lna);
            rawH.put("Department",         department);
            rawH.put("Network Consultant", consultant);
        }

        this.result.put("result", result);

        return this.result;
    } // end of processResultSet(...)
}
