/************************************************************************
 *
 * UserModifyCommand.java - db call pUser 'update'
 *
 * $Id: UserModifyCommand.java,v 1.1.1.1 2006/06/30 23:38:10 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.lang.*;
import java.util.*;
import java.sql.*;

/** db call pUser 'update' */
class UserModifyCommand extends UserCommand implements ComplexDBCall {

    private Hashtable oldUser;
    private Hashtable newUser;

    public UserModifyCommand(stanford.netdb.utils.Logger logger, Hashtable oldUser, Hashtable newUser, Integer op_uid) {
        super(logger);
        this.oldUser                    = oldUser;
        this.newUser                    = newUser;
        values[ACTION_INDEX]            = "update";
        values[OP_USER_INDEX]           = op_uid;
        values[ID_INDEX]                = new Integer((String)oldUser.get("id"));
        values[DEF_GROUP_INDEX]         = newUser.get("default_group");
        values[DEF_DOMAIN_INDEX]        = newUser.get("default_domain");

        String defAddrSpace = (String)newUser.get("default_addr_space");
        values[DEF_ADDR_SPACE_INDEX]    = defAddrSpace != null?defAddrSpace:"";

        values[ACTIVE_INDEX]            = new Short( ((Boolean)newUser.get("active?")).booleanValue()?(short)1:(short)0 );
        values[ALL_OBJ_INDEX]           = new Short( ((Boolean)newUser.get("all_objects?")).booleanValue()?(short)1:(short)0 );
        values[ALL_GROUPS_INDEX]        = new Short( ((Boolean)newUser.get("all_groups?")).booleanValue()?(short)1:(short)0 );

        String comment = (String)newUser.get("comment");
        if( comment != null )
            values[COMMENT_INDEX] = comment;

        values[LOCKED_INDEX] = lock     = (Timestamp)oldUser.get("locked");
    }

    public Hashtable doIt(NetDBConnection con) throws NetDBException, Exception {
        logger.info("Modifying user: "+values[NET_ID_INDEX]);
        BeginTransaction(con.con);
        boolean commit_finally = false;
        try {
            //
            int id = Integer.parseInt((String)oldUser.get("id"));

            logger.debug("do sigle value attributes");
            Hashtable result = con.callDB(this);

            // check for errors
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            // update groups
            Hashtable oldGroups = Utils.nHash(oldUser.get("groups"));
            Hashtable newGroups = Utils.nHash(newUser.get("groups"));

            Vector groups_to_insert = new Vector();
            Vector groups_to_delete = new Vector();
            Vector groups_to_keep = new Vector();
            Utils.keysIntersect(oldGroups, newGroups, groups_to_insert, groups_to_delete, groups_to_keep);

            // insert groups_to_insert
            logger.debug("inserting groups_to_insert");
            ComplexDBCall cmd =
                new ObjectGroupCommand(logger, ObjectGroupCommand.ADD_GROUPS, id,
                                       ObjectGroupCommand.USER, lock, groups_to_insert,
                                       (Integer)values[OP_USER_INDEX]);
            result = cmd.doIt(con);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            logger.debug("deleting groups_to_delete");
            cmd = new ObjectGroupCommand(logger, ObjectGroupCommand.REMOVE_GROUPS, id,
                                         ObjectGroupCommand.USER,
                                         lock, groups_to_delete, (Integer)values[OP_USER_INDEX]);
            result = cmd.doIt(con);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;


            // update departements
            Hashtable oldDepartments = Utils.nHash(oldUser.get("departments"));
            Hashtable newDepartments = Utils.nHash(newUser.get("departments"));

            Vector deps_to_insert = new Vector();
            Vector deps_to_delete = new Vector();
            Vector deps_to_keep = new Vector();
            Utils.keysIntersect(oldDepartments, newDepartments, deps_to_insert, deps_to_delete, deps_to_keep);

            logger.debug("inserting deps_to_insert");
            for( int i = 0; i < deps_to_insert.size(); i++ ) {
                String depName = (String)deps_to_insert.elementAt(i);
                Hashtable dep = (Hashtable)newDepartments.get(depName);
                DBCall call =
                    new UserDepartmentCommand(logger, UserDepartmentCommand.INSERT_ACTION,
                                              (Integer)values[ID_INDEX], depName,
                                              ((Boolean)dep.get("official?")).booleanValue(),
                                              lock, (Integer)values[OP_USER_INDEX]);
                result = call.processResultSet(con.callDB(call));

                // check for errors
                error_set = (Vector)result.get("errors");
                if( error_set != null && error_set.size() > 0 )
                    return result;
            }

            logger.debug("deleting deps_to_delete");
            for( int i = 0; i < deps_to_delete.size(); i++ ) {
                String depName = (String)deps_to_delete.elementAt(i);
                Hashtable dep = (Hashtable)oldDepartments.get(depName);
                DBCall call =
                    new UserDepartmentCommand(logger, UserDepartmentCommand.DELETE_ACTION,
                                              (Integer)values[ID_INDEX], depName,
                                              ((Boolean)dep.get("official?")).booleanValue(),
                                              lock, (Integer)values[OP_USER_INDEX]);
                result = call.processResultSet(con.callDB(call));

                // check for errors
                error_set = (Vector)result.get("errors");
                if( error_set != null && error_set.size() > 0 )
                    return result;
            }

            logger.debug("updating deps_to_keep");
            for( int i = 0; i < deps_to_keep.size(); i++ ) {
                String depName = (String)deps_to_keep.elementAt(i);
                Hashtable dep = (Hashtable)newDepartments.get(depName);
                Hashtable oldDep = (Hashtable)oldDepartments.get(depName);
                if( ((Boolean)dep.get("official?")).booleanValue() != ((Boolean)oldDep.get("official?")).booleanValue() ) {
                    // update only if official status changed
                    DBCall call =
                        new UserDepartmentCommand(logger, UserDepartmentCommand.UPDATE_ACTION,
                                                  (Integer)values[ID_INDEX], depName,
                                                  ((Boolean)dep.get("official?")).booleanValue(),
                                                  lock,(Integer)values[OP_USER_INDEX]);
                    result = call.processResultSet(con.callDB(call));

                    // check for errors
                    error_set = (Vector)result.get("errors");
                    if( error_set != null && error_set.size() > 0 )
                        return result;
                }
            }

            // handle records
            Hashtable oldRecords = Utils.nHash(oldUser.get("records"));
            Hashtable newRecords = Utils.nHash(newUser.get("records"));

            Vector recs_to_insert = new Vector();
            Vector recs_to_delete = new Vector();
            Vector recs_to_keep = new Vector();
            Utils.keysIntersect(oldRecords, newRecords, recs_to_insert, recs_to_delete, recs_to_keep);

            logger.debug("insering recs_to_insert");
            for( int i = 0; i < recs_to_insert.size(); i++ ) {
                DBCall call = new UserAccessCommand(logger, UserAccessCommand.INSERT_ACTION, (Integer)values[ID_INDEX],
                                                    (String)recs_to_insert.elementAt(i),
                                                    lock, (Integer)values[OP_USER_INDEX]);
                result = call.processResultSet(con.callDB(call));

                // check for errors
                error_set = (Vector)result.get("errors");
                if( error_set != null && error_set.size() > 0 )
                    return result;
            }

            logger.debug("deleting recs_to_delete");
            for( int i = 0; i < recs_to_delete.size(); i++ ) {
                DBCall call = new UserAccessCommand(logger, UserAccessCommand.DELETE_ACTION, (Integer)values[ID_INDEX],
                                                    (String)recs_to_delete.elementAt(i),
                                                    lock, (Integer)values[OP_USER_INDEX]);
                result = call.processResultSet(con.callDB(call));

                // check for errors
                error_set = (Vector)result.get("errors");
                if( error_set != null && error_set.size() > 0 )
                    return result;
            }

            // do check
            logger.debug("do check");

            // values[0] has to be reset here becouse last call filled up this with some value
            values[0]                   = ref_cursor;
            values[ACTION_INDEX]        = "verify";

            result = con.callDB(this);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            DBCall call = new UserLockUnLockCommand(logger, UserLockUnLockCommand.UNLOCK_ACTION,
                                                    ((Integer)values[ID_INDEX]), null,
                                                    lock, (Integer)values[OP_USER_INDEX]);
            result = call.processResultSet(con.callDB(call));

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            ((NetDBii)logger).unregister_lock(lock, new Integer(Constants.USER));

            commit_finally = true;

        } catch(Exception ex) {
            ex.printStackTrace();
            try {
                logger.info("Rollback Transaction");
                RollbackTransaction(con.con);
            } catch(Exception exx) { }
            throw ex;

        } finally {
            if( commit_finally )
                try {
                    logger.info("modifying user "+values[NET_ID_INDEX]+" done");
                    CommitTransaction(con.con);
                } catch(Exception exx) { }
            else
                try {
                    logger.info("modifying user "+values[NET_ID_INDEX]+" failed");
                    logger.debug("Rollback Transaction");
                    RollbackTransaction(con.con);
                } catch(Exception exx) { }
        }
        return new Hashtable();
    }
}
