/************************************************************************
 *
 * DateFormatter.java -
 *
 * $Id: DateFormatter.java,v 1.1.1.1 2006/06/30 23:38:10 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2006, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.utils;

import java.text.*;
import java.util.*;

public class DateFormatter extends Object {

    private SimpleDateFormat[] formatters = null;

    private String lock = "lock";

    public DateFormatter(String[] formats) {

        formatters = new SimpleDateFormat[formats.length];
        for( int i = 0; i < formats.length; i++ ) {
            formatters[i] = new SimpleDateFormat(formats[i]);
            formatters[i].setLenient(false);
        }
    }

    public DateFormatter(String format) {
        formatters = new SimpleDateFormat[1];
        formatters[0] = new SimpleDateFormat(format);
    }

    public Date parse(String src) throws ParseException {
        for( int i = 0; i < formatters.length; i++ ) {
            try {
                // date formatters are not thread safe, thus synchronized clause is used
                synchronized(lock) {
                    return formatters[i].parse(src);
                }
            } catch (Exception ex) { }
        }
        throw new ParseException("Unable to recognize date in string \""+src+"\".", 0);
    }
}
