/************************************************************************
 *
 * DefaultLogger.java -
 *
 * $Id: DefaultLogger.java,v 1.1.1.1 2006/06/30 23:38:10 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.utils;

import java.util.*;
import java.lang.*;
import java.net.*;

public class DefaultLogger implements Logger {

    private boolean do_syslog     = false;
    private String syslog_host    = "localhost";
    private int syslog_port       = 514;
    private int syslog_facility   = 19; // local3
    private int syslog_priority   = 6; // info

    public DefaultLogger() { }

    public void debug (Object msg) {
        if( Boolean.getBoolean("LOG_DEBUG") )
            if( do_syslog )
                doSysLog("[DEBUG] "+msg);
            else
                System.out.println("[DEBUG] "+msg);
    }

    public void warn  (Object msg) {
        if( Boolean.getBoolean("LOG_WARN") )
            if( do_syslog )
                doSysLog("[WARN] "+msg);
            else
                System.out.println("[WARN]  "+msg);
    }

    public void error (Object msg) {
        if( Boolean.getBoolean("LOG_ERROR") )
            if( do_syslog )
                doSysLog("[ERROR] "+msg);
            else
                System.out.println("[ERROR] "+msg);
    }

    public void info  (Object msg) {
        if( Boolean.getBoolean("LOG_INFO") )
            if( do_syslog )
                doSysLog("[INFO] "+msg);
            else
                System.out.println("[INFO]  "+msg);
    }

    private void doSysLog(String msg) {
        try {
            InetAddress syslogAddr = InetAddress.getByName(syslog_host);
            DatagramSocket socket = new DatagramSocket();
            int syslog_code = (syslog_facility << 3) | syslog_priority;
            String scode = (new Integer(syslog_code)).toString();
            String data = "<"+scode+">"+System.getProperty(Constants.SYSLOG_ID_STR_KEY, "netdb")+": "+msg;
            byte[] rowdata = data.getBytes();
            DatagramPacket packet = new DatagramPacket(rowdata, rowdata.length, syslogAddr, syslog_port);
            socket.send(packet);
        } catch(Exception ex) {
            System.out.println(ex);
        }
    }

    public void set (String what, Object value) {
        if( what.equals(Constants.SYSLOG_HOST_KEY) )
            syslog_host = (String)value;

        else if( what.equals(Constants.DO_SYSLOG_KEY) )
            do_syslog = ((Boolean)value).booleanValue();

        else if( what.equals(Constants.SYSLOG_PORT_KEY) )
            syslog_port = ((Integer)value).intValue();

        else if( what.equals(Constants.SYSLOG_FACILITY_KEY) ) {
            String sfacility = ((String)value);
            if(      sfacility.equals("user" ) )
                syslog_facility = 1;
            else if( sfacility.equals("kern" ) )
                syslog_facility = 0;
            else if( sfacility.equals("mail" ) )
                syslog_facility = 2;
            else if( sfacility.equals("daemon" ) )
                syslog_facility = 3;
            else if( sfacility.equals("auth" ) )
                syslog_facility = 4;
            else if( sfacility.equals("lpr" ) )
                syslog_facility = 6;
            else if( sfacility.equals("news" ) )
                syslog_facility = 7;
            else if( sfacility.equals("uucp" ) )
                syslog_facility = 8;
            else if( sfacility.equals("cron" ) )
                syslog_facility = 9;
            else if( sfacility.equals("local0" ) )
                syslog_facility = 16;
            else if( sfacility.equals("local1" ) )
                syslog_facility = 17;
            else if( sfacility.equals("local2" ) )
                syslog_facility = 18;
            else if( sfacility.equals("local3" ) )
                syslog_facility = 19;
            else if( sfacility.equals("local4" ) )
                syslog_facility = 20;
            else if( sfacility.equals("local5" ) )
                syslog_facility = 21;
            else if( sfacility.equals("local6" ) )
                syslog_facility = 22;
            else if( sfacility.equals("local7" ) )
                syslog_facility = 23;
            else // local3
                syslog_facility = 19;

        } else if( what.equals(Constants.SYSLOG_PRIORITY_KEY) ) {
            String sprio = ((String)value);
            if(      sprio.equals("emerg" ) )
                syslog_priority = 0;
            else if( sprio.equals("alert" ) )
                syslog_priority = 1;
            else if( sprio.equals("crit" ) )
                syslog_priority = 2;
            else if( sprio.equals("err" ) )
                syslog_priority = 3;
            else if( sprio.equals("warning" ) )
                syslog_priority = 4;
            else if( sprio.equals("notice" ) )
                syslog_priority = 5;
            else if( sprio.equals("info" ) )
                syslog_priority = 6;
            else if( sprio.equals("debug" ) )
                syslog_priority = 7;
            else // info
                syslog_priority = 6;
        }
    }
}
