/************************************************************************
 *
 * TableHashSort.java -
 *
 * $Id: TableHashSort.java,v 1.2 2006/09/11 19:38:45 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2006, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.utils;

import java.util.*;

/** This class is used for sorting hashtables which would normally represent tables and are in the form
 * <pre>
 * (Hashtable)
 * %key% -> (Hashtable)  --- this is a row hash
 *          'column1' -> (String)
 *          'column2' -> (String)
 *          ...
 *          'columnN' -> (String)
 * </pre>
 */
public class TableHashSort extends Object {

    /** value type "String" */
    public static final int STRING = 0;

    /** value type "Numeric", i.e. value type is string representation of numeric */
    public static final int NUMBER = 1;

    /** Direction of the sort 'down' */
    public static final int DOWN   = 0;

    /** Direction of the sort 'up' */
    public static final int UP     = 1;

    Hashtable table;
    Object    sort_key;
    int       sort_type;
    int       dir;

    private TableHashSort(Hashtable table, Object sort_key, int sort_type, int dir) {
        super();

        this.table     = table;
        this.sort_key  = sort_key;
        this.sort_type = sort_type;
        this.dir       = dir;
    }

    /** returns array of row hashes (see above) */
    public static Object[] doSort(Hashtable table, Object sort_key, int sort_type, int dir) throws Exception {
        return (new TableHashSort(table, sort_key, sort_type, dir)).sort_it();
    }

    /** returns array of row hashes (see above), with sort dirrection 'down' */
    public static Object[] doSort(Hashtable table, Object sort_key, int sort_type) throws Exception {
        return (new TableHashSort(table, sort_key, sort_type, DOWN)).sort_it();
    }

    private Object[] sort_it() throws Exception {
        Comparator cmp;

        switch(sort_type) {
        case STRING:
            cmp = new Comparator() {
                    public int compare(Object o1, Object o2) {
                        String cmp1 = Utils.nStr(((Hashtable)((Map.Entry)o1).getValue()).get(sort_key));
                        String cmp2 = Utils.nStr(((Hashtable)((Map.Entry)o2).getValue()).get(sort_key));

                        if( dir == DOWN )
                            return cmp1.compareToIgnoreCase(cmp2);
                        else
                            return cmp2.compareToIgnoreCase(cmp1);
                    }
                };
            break;

        case NUMBER:
            cmp = new Comparator() {
                    public int compare(Object o1, Object o2) {
                        Long cmp1 = new Long((String)((Hashtable)((Map.Entry)o1).getValue()).get(sort_key));
                        Long cmp2 = new Long((String)((Hashtable)((Map.Entry)o2).getValue()).get(sort_key));

                        if( dir == DOWN )
                            return cmp1.compareTo(cmp2);
                        else
                            return cmp2.compareTo(cmp1);
                    }
                };
            break;

        default:
            throw new Exception("Unsupported sort type");
        }

        java.util.List list = Arrays.asList(table.entrySet().toArray());
        Collections.sort(list, cmp);
        return list.toArray();
    }
}
