/************************************************************************
 *
 * NetDBImpl.java -
 *
 * $Id: NetDBImpl.java,v 1.4 2006/10/04 20:37:08 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import org.ietf.jgss.*;
import java.lang.*;
import java.util.*;
import java.sql.*;
import java.rmi.*;
import java.io.*;
import java.net.*;
import java.rmi.server.*;
import javax.security.auth.*;
import javax.security.auth.login.*;
import javax.security.auth.callback.*;
import java.rmi.registry.*;
import java.rmi.server.UnicastRemoteObject;
import javax.net.ssl.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;
import java.security.Security;
import javax.security.auth.Subject;
import com.sun.security.auth.callback.TextCallbackHandler;

public class NetDBImpl
    extends UnicastRemoteObject
    implements NetDBi, SigTERMHandler, stanford.netdb.utils.Logger {

    private long timestamp;
    private static long start_time;
    private boolean root;
    private NetDBpi netdb;
    private boolean invalidated = false;
    private static LoginContext loginContext;
    static stanford.netdb.utils.Logger logger = null;

    public GSSManager manager;
    public GSSContext context;
    public boolean pass = false;
    public String name = null;

    private static Hashtable netdbs = new Hashtable();

    // queue
    private static long running = 0;
    private static LinkedList queue = new LinkedList();
    private static long max_concurrent_calls = 1;
    private static long nextTicket = 0;
    private static SigTERMHandler sigTERMHandler;
    private static Posix sigCaller;

    public NetDBImpl(boolean root) throws RemoteException {
        super(0, new ClientSocketFactory(), new ServerSocketFactory());
        this.root = root;
        timestamp = Calendar.getInstance().getTimeInMillis();
        debug("Created "+timestamp);
    }

    public NetDBi getInstance() throws RemoteException {
        return new NetDBImpl(false);
    }

    /** Creates instance of NetDBi object */
    public Object klogin(byte[] token) throws RemoteException {
        if( root )
            return null;
        if( token == null && pass ) {
            // read acl first
            BufferedReader file = null;
            boolean allow = false;
            boolean acl_found = false;
            boolean god = false;
            boolean is_acl = false;
            try{
                file = new BufferedReader( new FileReader(System.getProperty("acl",
                                                                             Constants.ACL_FILENAME)) );
                String login = name.trim();
                String line;
                int index = 0;
                while( ( line = file.readLine() ) != null ){
                    line = line.trim();
                    if( index == 0 && line.equals("*acl*") ) {
                        is_acl = true;
                        continue;
                    }
                    if( line.equals("") ) // skip empty lines
                        continue;
                    if( line.endsWith("*") ) {
                        if( line.substring(0, line.length() - 1).trim().equals(login) ){
                            allow = true;
                            god = true;
                            break;
                        }
                    } else if( line.equals(login) ){
                        allow = true;
                        break;
                    }
                    index++;
                }
                acl_found = true;

            } catch (FileNotFoundException ex) {
                debug("acl file not found, not using acl.");

            } catch (Exception ex) {
                debug(ex.getMessage());

            } finally {
                try {
                    file.close();
                } catch (Exception ex) {
                }
            }

            if( acl_found && is_acl && !allow )
                throw new RemoteException("User ["+name+"] not allowed to login.");

            try {
                netdb = NetDBHook.createInstance(name, god);
            } catch (Exception ex) {
                throw new RemoteException(ex.toString());
            }

            synchronized (netdbs) {
                Hashtable state = netdb.getState();
                netdbs.put((String)state.get("timestamp"), this);
            }

            return this;
        }
        LoginContext lc = null;
        try {
            lc = new LoginContext("NetDBImpl", new TextCallbackHandler());
        } catch (LoginException le) {
            error("Cannot create LoginContext. "
                        + le.getMessage());
            throw new RemoteException("Cannot create LoginContext. "
                                      + le.getMessage());
        } catch (SecurityException se) {
            error("Cannot create LoginContext. "
                        + se.getMessage());
            throw new RemoteException("Cannot create LoginContext. "
                                      + se.getMessage());
        }

        try {

            // attempt authentication
            lc.login();

        } catch (AccountExpiredException aee) {

            error("Your account has expired.  " +
                        "Please notify your administrator.");
            throw new RemoteException("Server account has expired");

        } catch (CredentialExpiredException cee) {

            error("Your credentials have expired.");
            System.exit(-1);

        } catch (FailedLoginException fle) {

            error("Authentication Failed");
            throw new RemoteException("Server Authentication Failed");

        } catch (Exception e) {

            error("Unexpected Exception - unable to continue. "+e.getMessage());
            throw new RemoteException("Unexpected Exception - unable to continue. "+e.getMessage());
        }

        byte[] retToken = new byte[token.length];

        try {
            Subject.doAsPrivileged(lc.getSubject(),
                       new  MyAction(token, retToken), null);
        } catch (Exception ex) {
            throw new RemoteException(ex.toString());
        }

        if( token != null )
            return retToken;
        else
            throw new RemoteException("Authenitication failed");
    } // end of klogin(...)

    public void logout() throws RemoteException {
        synchronized (netdbs) {
            netdbs.remove((String)netdb.getState().get("timestamp"));
            info("LOGOUT");
            invalidated = true;
            netdb = null;
        }
    }

    class MyAction implements java.security.PrivilegedExceptionAction {

        private byte[] token;
        private byte[] retToken;
        private String tokenStr;

        public MyAction(byte[] token, byte[] retToken) {
            super();
            this.token = token;
            this.retToken = retToken;
        }

        public Object run() throws Exception {
            try {
                if( manager == null )
                    manager = GSSManager.getInstance();
                if( context == null )
                    context = manager.createContext((GSSCredential)null);

                debug("context.isEstablished() = "+context.isEstablished());
                if (!context.isEstablished()) {
                    byte[] rToken = context.acceptSecContext(token, 0, token.length);
                    for( int i = 0; i < rToken.length; i++ )
                        retToken[i] = rToken[i];
                }
                if (context.isEstablished()) {

                    debug("Context Established! ");
                    debug("Client is " + context.getSrcName());
                    debug("Server is " + context.getTargName());

                    //
                    // If mutual authentication did not take place, then
                    // only the client was authenticated to the
                    // server. Otherwise, both client and server were
                    // authenticated to each other.
                    //
                    if (context.getMutualAuthState()) {
                        debug("Mutual authentication took place!");
                        pass = true;
                        String principal = context.getSrcName().toString();
                        name = principal.substring(0, principal.indexOf("@"));
                    }

                    context.dispose();
                    manager = null;
                }
                debug("context.isEstablished() = "+context.isEstablished());
                return null;
            } catch(Exception ex) {
                //ex.printStackTrace();
                error(ex.getMessage());
                throw ex;
            }
        }
    }

    private void checkEndSession() throws SessionTimeoutException, RemoteException {
        if( root )
            throw new SessionTimeoutException("This is a root session.");
        if( invalidated )
            throw new SessionTimeoutException("Your session was invalidated.");
        if( netdb == null )
            throw new SessionTimeoutException("You session timed out.");
        requestService();
    }

    public Hashtable getGlobalDefaults() throws RemoteException, SessionTimeoutException {
        checkEndSession();
        try {
            try {
                return netdb.getGlobalDefaults();
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public Hashtable getCurrentUser () throws RemoteException, SessionTimeoutException {
        checkEndSession();
        try {
            try {
                return netdb.getCurrentUser();
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    } // end of getCurrentUser(

    public Hashtable loadACL () throws RemoteException, SessionTimeoutException {
        // this call is not placed in a queue but executed immediately
        if( root )
            throw new SessionTimeoutException("This is a root session.");
        if( netdb == null )
            throw new SessionTimeoutException("You session timed out.");
        BufferedReader file = null;
        Hashtable acl = new Hashtable();
        Hashtable people = new Hashtable();
        acl.put("people", people);
        try{
            Hashtable user = netdb.getCurrentUser();
            if( user.get("god") ==  null )
                throw new Exception("You do not have sufficient privileges to perform this call");
            file = new BufferedReader( new FileReader(Constants.ACL_FILENAME) );
            String line = null;
            int index = 0;
            while( ( line = file.readLine() ) != null ){
                line = line.trim();
                if( index == 0 && line.equals("*acl*") ) {
                    acl.put("acl", new Boolean(true));
                    continue;
                }
                if( line.equals("") ) // skip empty lines
                    continue;
                if( line.endsWith("*") )
                    people.put(line.substring(0, line.length() - 1).trim(), "god");
                else
                    people.put(line, "");
                index++;
            }

        } catch (FileNotFoundException ex) {
            return null;

        } catch (Exception ex) {
            debug(ex.getMessage());
            return null;

        } finally {
            try {
                file.close();
            } catch (Exception ex) {
            }
        }
        return acl;
    } // end of loadACL()

    public Hashtable createACL () throws RemoteException, SessionTimeoutException {
        // this call is not placed in a queue but executed immediately
        if( root )
            throw new SessionTimeoutException("This is a root session.");
        if( netdb == null )
            throw new SessionTimeoutException("You session timed out.");
        try {
            Hashtable user = netdb.getCurrentUser();
            if( user.get("god") ==  null )
                throw new Exception("You do not have sufficient privileges to perform this call");
        } catch (Exception ex) {
            throw new RemoteException(ex.getMessage());
        }
        try {
            (new File(Constants.ACL_FILENAME)).createNewFile();
        } catch (Exception ex) {
            debug(ex.getMessage());
        }
        return loadACL();
    } // end of createACL()

    public void updateACL (Hashtable acl) throws RemoteException, SessionTimeoutException {
        // this call is not placed in a queue but executed immediately
        if( root )
            throw new SessionTimeoutException("This is a root session.");
        if( netdb == null )
            throw new SessionTimeoutException("You session timed out.");
        PrintWriter file = null;
        try {
            Hashtable user = netdb.getCurrentUser();
            if( user.get("god") ==  null )
                throw new Exception("You do not have sufficient privileges to perform this call");
            file = new PrintWriter(new BufferedWriter(new FileWriter(Constants.ACL_FILENAME)));
            if( acl.get("acl") != null )
                file.println("*acl*");
            Hashtable people = (Hashtable)acl.get("people");
            if( people == null )
                return;
            Enumeration enm = people.keys();
            while( enm.hasMoreElements() ) {
                String netID = (String)enm.nextElement();
                file.println(netID+(people.get(netID).toString().equals("god")?"*":""));
            }
        } catch (Exception ex) {
            throw new RemoteException(ex.getMessage());
        } finally {
            try {
                file.close();
            } catch (Exception ex) {
            }
        }
    }

    public Hashtable getQueue() throws RemoteException, SessionTimeoutException {
        // this call is not placed in a queue but executed immediately
        if( root )
            throw new SessionTimeoutException("This is a root session.");
        if( netdb == null )
            throw new SessionTimeoutException("You session timed out.");
        try {
            Hashtable user = netdb.getCurrentUser();
            if( user.get("god") ==  null )
                throw new Exception("You do not have sufficient privileges to perform this call");
            synchronized (queue) {
                Hashtable result = new Hashtable();
                result.put("running", new Long(running));
                result.put("max_concurrent_calls", new Long(max_concurrent_calls));
                result.put("pending", new Integer(queue.size()));
                return result;
            }
        } catch ( Exception ex ) {
            throw new RemoteException(ex.getMessage());
        }
    }

    public void setMaxConcurrent (long mcc) throws RemoteException, SessionTimeoutException {
        // this call is not placed in a queue but executed immediately
        if( root )
            throw new SessionTimeoutException("This is a root session.");
        if( netdb == null )
            throw new SessionTimeoutException("You session timed out.");
        try {
            Hashtable user = netdb.getCurrentUser();
            if( user.get("god") ==  null )
                throw new RemoteException("You do not have sufficient privileges to perform this call");
            synchronized (queue) {
                max_concurrent_calls = mcc;
            }
        } catch ( Exception ex ) {
            throw new RemoteException(ex.getMessage());
        }
    }

    public void shutdown () throws RemoteException, SessionTimeoutException {
        // this call is not placed in a queue but executed immediately
        if( root )
            throw new SessionTimeoutException("This is a root session.");
        if( netdb == null )
            throw new SessionTimeoutException("You session timed out.");

        try {
            debug("Shutting down");
            Hashtable user = netdb.getCurrentUser();
            if( user.get("god") ==  null )
                throw new Exception("You do not have sufficient privileges to perform this call");

            // lock the queue which will prevent any future calls and do shutdown
            synchronized (queue) {
                doShutdown();
            }

        } catch ( Exception ex ) {
            throw new RemoteException(ex.getMessage());

        } finally {
            debug("System.exit");
            System.exit(0);
        }
    }

    private void doShutdown() {
        try {
            synchronized (netdbs) {
                Enumeration enm = netdbs.keys();
                while( enm.hasMoreElements() ) {
                    String key = (String)enm.nextElement();
                    NetDBImpl netdb = (NetDBImpl)netdbs.get(key);
                    try {
                        netdb.finalize();
                    } catch (Throwable th) {
                        error(th.toString());
                    }
                }
            }
        } catch (Exception ex) {
        } finally {
            debug("System.exit");
            System.exit(0);
        }
    }

    public String      getDefaultDomain () throws RemoteException, SessionTimeoutException {
        checkEndSession();
        try {
            return netdb.getDefaultDomain();
        } finally {
            endCall();
        }
    }

    public void        cancelSearch (String searchID) throws RemoteException, SessionTimeoutException {
        checkEndSession();
        try {
            try {
                netdb.cancelSearch(searchID);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public String      createSearchID () throws RemoteException, SessionTimeoutException {
        checkEndSession();
        try {
            try {
                return netdb.createSearchID();
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    /** performs simple search, see NetDB.simpleSearch(...) for details */
    public Hashtable   simpleSearch (int what, String searchStr, String searchID)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.simpleSearch(what, searchStr, searchID);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }


    /** performs simple search, see NetDB.simpleSearch(...) for details */
    public Hashtable    simpleSearch (int what, String searchStr)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.simpleSearch(what, searchStr);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    /** performs directory search, see NetDB.dirSearch(...) for details */
    public Hashtable    dirSearch (int byWhat, String searchStr)
        throws RemoteException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.dirSearch(byWhat, searchStr);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    /** performs full search, see NetDB.fullSearch(...) for details */
    public Hashtable fullSearch (int what, Hashtable search_params, String searchID)
        throws RemoteException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.fullSearch(what, search_params, searchID);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public String      findIPAddrSpace          (String ip)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.findIPAddrSpace(ip);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public void         moveAS (Vector asList, String targetNetFQDN)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                netdb.moveAS(asList, targetNetFQDN);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    /** locks NetDB object, see NetDB.lock(...) for details */
    public Timestamp    lock (int what, String handle)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.lock(what, handle);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    /** locks NetDB object, see NetDB.lock(...) for details */
    public Timestamp    lock (int what, int oid)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.lock(what, oid);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    /** unlocks NetDB object, see NetDB.unlock(...) for details */
    public Timestamp   unlock (int what, String handle, Timestamp tm)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.unlock(what, handle, tm);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    /** unlocks NetDB object, see NetDB.unlock(...) for details */
    public void       unlock (int what, int oid, Timestamp tm)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                netdb.unlock(what, oid, tm);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    /** splits address space, see NetDB.splitAS(...) for details */
    public Hashtable   splitAS (Hashtable as)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.splitAS(as);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public Hashtable verifyAddressSpace(String prefix, Integer idNet)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.verifyAddressSpace(prefix, idNet);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    /** performs update, see NetDB.update(...) for details */
    public Hashtable    update (Hashtable oldObj, Hashtable newObj)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.update(oldObj, newObj);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    /** performs delete, see NetDB.delete(...) for details */
    public Hashtable    delete (int what, String oid)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.delete(what, oid);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    /** performs delete, see NetDB.delete(...) for details */
    public Hashtable    delete (Hashtable obj)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.delete(obj);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    /** performs insert, see NetDB.insert(...) for details */
    public Hashtable   insert (Hashtable obj)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.insert(obj);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public Hashtable   reserveIP (String asPrefix, String ip, int count)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.reserveIP(asPrefix, ip, count);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public void        cancelIP (String ip)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                netdb.cancelIP(ip);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public String      reserveName              (String fqdnName, String nameType)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.reserveName(fqdnName, nameType);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public void        cancelName               (String fqdnName, String nameType)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                netdb.cancelName(fqdnName, nameType);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    /** performs generic access check, see NetDB.checkAccess(...) for details */
    public boolean      checkAccess (int objectType)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.checkAccess(objectType);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    /** performs group access check, see NetDB.checkGroupAccess(...) for details */
    public boolean      checkGroupAccess(Hashtable object)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.checkGroupAccess(object);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    /** performs getInfo db call, see NetDB.getInfo(...) for details */
    public Hashtable    getInfo (int what, int oid)
        throws RemoteException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.getInfo(what, oid);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    /** performs getInfo db call, see NetDB.getInfo(...) for details */
    public Hashtable    getInfo (int what, String handle)
        throws RemoteException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.getInfo(what, handle);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public Hashtable   listMyNetworks ()
        throws RemoteException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.listMyNetworks();
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public Hashtable   listMyDomains ()
        throws RemoteException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.listMyDomains();
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public Hashtable listMyGroups(String filter)
        throws RemoteException, RemoteNetDBException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.listMyGroups(filter);
            } catch ( NetDBException ex ) {
                throw new RemoteNetDBException(ex);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public Hashtable   listMyAddrSpaces ()
        throws RemoteException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.listMyAddrSpaces();
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public Hashtable   listAddrSpaces ()
        throws RemoteException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.listAddrSpaces();
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public Hashtable   listDHCPServices ()
        throws RemoteException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.listDHCPServices();
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public Hashtable   listRecords ()
        throws RemoteException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.listRecords();
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public Hashtable   listStates ()
        throws RemoteException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.listStates();
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public Hashtable   list                     (int what, String filter, boolean mine)
        throws RemoteException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.list(what, filter, mine);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public Hashtable   listMyStates (String filter)
        throws RemoteException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                return netdb.listMyStates(filter);
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    public Hashtable   generateHexName (String ip, int salt)
        throws RemoteException, SessionTimeoutException
    {
        checkEndSession();
        try {
            return netdb.generateHexName(ip, salt);
        } finally {
            endCall();
        }
    }

    public Hashtable   getState ()
        throws RemoteException, SessionTimeoutException
    {
        checkEndSession();
        try {
            Hashtable state = new Hashtable();
            state.put("timestamp millis", new Long(timestamp));
            state.put("timestamp", new java.util.Date(timestamp));
            long  uptime = Calendar.getInstance().getTimeInMillis()-start_time;
            state.put("uptime", (uptime/86400000)+"d "+((uptime/3600000)%60)+"h "+
                      ((uptime/60000)%60)+"m "+((uptime/1000)%60)+"s");
            boolean god = netdb.getCurrentUser().get("god")!=null?true:false;
            synchronized (netdbs) {
                Hashtable sessions = new Hashtable();
                Enumeration keys = netdbs.keys();
                while( keys.hasMoreElements() ) {
                    String timestamp = (String)keys.nextElement();
                    NetDBImpl netdb = (NetDBImpl)netdbs.get(timestamp);
                    Hashtable user = netdb.getCurrentUser();
                    Hashtable session = new Hashtable();
                    session.put("SessionID", timestamp);
                    session.put("NetID", user.get("NetID"));
                    if( god )
                        session.put("logged_in", ""+(Calendar.getInstance().getTimeInMillis()  - Long.parseLong(timestamp)));
                    sessions.put(user.get("NetID")+" "+timestamp, session);
                    state.put("sessions", sessions);
                }
            }
            return state;

        } catch (Exception ex) {
            debug(ex.getMessage());
            throw new RemoteException(ex.getMessage());

        } finally {
            endCall();
        }
    }

    public void        reloadConfig ()
        throws RemoteException, SessionTimeoutException
    {
        checkEndSession();
        try {
            try {
                netdb.reloadConfig();
            } catch ( Exception ex ) {
                throw new RemoteException(ex.getMessage());
            }
        } finally {
            endCall();
        }
    }

    protected void finalize() throws Throwable {
        debug("Destroyed "+timestamp);
        Hashtable state = netdb.getState();
        netdbs.remove((String)state.get("timestamp"));
        netdb.finalize();
        netdb = null;
        super.finalize();
    }

    public static void main(String[] cmd) {
        start_time = Calendar.getInstance().getTimeInMillis();

        try {
            loginContext = new LoginContext("NetDBImpl");
            loginContext.login();
        } catch (Exception ex) {
            System.out.println(ex.getMessage());
            System.exit(1);
        }

        Properties netdb_props = new Properties();
        FileInputStream in = null;
        try {
            in = new FileInputStream(System.getProperty("config.file",
                                                        Constants.NETDB_PROPERTIES_FILENAME));
            netdb_props.load(in);
        } catch (Exception ex) {
        } finally {
            try {
                if( in != null )
                    in.close();
            } catch (Exception ex) { }
        }

        // override by system properties
        Properties system_props = System.getProperties();
        Enumeration enm = system_props.keys();
        while( enm.hasMoreElements() ) {
            String key = (String)enm.nextElement();
            netdb_props.put(key, system_props.get(key));
        }

        // now override system properties
        enm = netdb_props.keys();
        while( enm.hasMoreElements() ) {
            String key = (String)enm.nextElement();
            system_props.put(key, netdb_props.get(key));
        }

        max_concurrent_calls = (new Long(System.getProperties().
                                getProperty(Constants.QUEUE_DEPTH_KEY, "10"))).longValue();

        // set up logger
        if( logger == null )
            logger = new DefaultLogger();

        if( Boolean.getBoolean(Constants.DO_SYSLOG_KEY) ) {
            logger.set(Constants.DO_SYSLOG_KEY, new Boolean(true));
            logger.set(Constants.SYSLOG_PORT_KEY,
                       new Integer(netdb_props.getProperty(Constants.SYSLOG_PORT_KEY, "514")));
            logger.set(Constants.SYSLOG_HOST_KEY,
                       netdb_props.getProperty(Constants.SYSLOG_HOST_KEY, "localhost"));
            logger.set(Constants.SYSLOG_FACILITY_KEY,
                       netdb_props.getProperty(Constants.SYSLOG_FACILITY_KEY, "local3"));
            logger.set(Constants.SYSLOG_PRIORITY_KEY,
                       netdb_props.getProperty(Constants.SYSLOG_PRIORITY_KEY, "info"));
        }

        // disable SIGHUP
        try {
            Posix.disableSIGHUP();
        } catch (Exception ex) {
            System.err.println(ex);
        }

        // install SIGTERM handler
        try {
            sigTERMHandler = new NetDBImpl(true);
            sigCaller = new Posix();
            sigCaller.registerSigTERMaction(sigTERMHandler);
            sigCaller.sigterm();
        } catch (Exception ex) {
            System.err.println("Failed to install SIGTERM handler: "+ex);
        }

        // print pid, needed for manage
        Utils.write_pid_file();

        // change egid and euid
        if( netdb_props.getProperty(Constants.GID_KEY) != null ) {
            try {
                Posix.setgid(netdb_props.getProperty(Constants.GID_KEY));
            } catch(Exception ex) {
                // Something went wrong
                System.out.println("failed to set gid ["+
                    netdb_props.getProperty(Constants.GID_KEY)+"]");
            }
        }

        if( netdb_props.getProperty(Constants.UID_KEY) != null ) {
            try {
                Posix.setuid(netdb_props.getProperty(Constants.UID_KEY));
            } catch(Exception ex) {
                // Something went wrong
                System.out.println("failed to set uid ["+
                    netdb_props.getProperty(Constants.UID_KEY)+"]");
            }
        }

        try {
            Registry reg = LocateRegistry.getRegistry(Integer.parseInt(System.getProperty("registry.port", "20005")));
            reg.list();
        } catch (Exception ex) {
            try {
                LocateRegistry.createRegistry(Integer.parseInt(System.getProperty("registry.port", "20005")));
            } catch (Exception exx) {
                ((NetDBImpl)sigTERMHandler).debug(exx.getMessage());
            }
        }

        // start timeout thread
        Thread timeoutThread = new Thread() {
                public void run() {
                    for(;;) {
                        try {
                            sleep(5000); // sleep 5 sec.
                        } catch (Exception ex) { }
                        synchronized (netdbs) {
                            Enumeration enm = netdbs.keys();
                            while( enm.hasMoreElements() ) {
                                String key = (String)enm.nextElement();
                                NetDBImpl netdb = (NetDBImpl)netdbs.get(key);
                                if( netdb.netdb.isTimeout() )
                                    try {
                                        netdb.finalize();
                                    } catch (Throwable th) { };
                            }
                        }
                    }
                }
            };
        timeoutThread.start();

        try {
            NetDBImpl server = new NetDBImpl(true);
            String[] names = Naming.list("//:"+System.getProperty("registry.port", "20005"));
            Hashtable namesH = new Hashtable();
            for( int i = 0; i < names.length; i++ )
                namesH.put(names[i], "");

            if( namesH.get("//:"+System.getProperty("registry.port", "20005")+"/NetDB") == null )
                Naming.bind("//:"+System.getProperty("registry.port", "20005")+"/NetDB", server);
            else
                Naming.rebind("//:"+System.getProperty("registry.port", "20005")+
                              "/NetDB-"+System.getProperty("user.name"), server);
        } catch (Exception ex) {
            ex.printStackTrace();
            ((NetDBImpl)sigTERMHandler).error(ex);
        }
    } // end of main(...)

    private class KerberosCallBackHandler implements CallbackHandler {
        private String name;
        private String password;

        public KerberosCallBackHandler(String name, String password) {
            this.name = name;
            this.password = password;
        }

        public void handle(Callback[] callbacks)
            throws UnsupportedCallbackException, IOException {
            for (int i = 0; i < callbacks.length; i++) {
                if (callbacks[i] instanceof TextOutputCallback) {
                    // display the message according to the specified type
                    TextOutputCallback toc = (TextOutputCallback)callbacks[i];
                    switch (toc.getMessageType()) {
                    case TextOutputCallback.INFORMATION:
                        //System.out.println(toc.getMessage());
                        break;
                    case TextOutputCallback.ERROR:
                        //System.out.println("ERROR: " + toc.getMessage());
                        break;
                    case TextOutputCallback.WARNING:
                        //System.out.println("WARNING: " + toc.getMessage());
                        break;
                    default:
                        throw new IOException("Unsupported message type: " +
                                              toc.getMessageType());
                    }

                } else if (callbacks[i] instanceof NameCallback) {
                    // prompt the user for a username
                    NameCallback nc = (NameCallback)callbacks[i];

                    // ignore the provided defaultName
                    //System.err.println(nc.getPrompt());
                    //System.err.flush();
                    nc.setName(name);

                } else if (callbacks[i] instanceof PasswordCallback) {
                    // prompt the user for sensitive information
                    PasswordCallback pc = (PasswordCallback)callbacks[i];
                    //System.err.print(pc.getPrompt());
                    //System.err.flush();
                    pc.setPassword(password.toCharArray());

                } else
                    throw new UnsupportedCallbackException(callbacks[i], "Unrecognized Callback");
            }
        }
    }

    private void requestService() {
        long ticket = -1;
        boolean first_attempt = true;
        synchronized (queue) {
            ticket = nextTicket;
            nextTicket++;
            for(;;) {
                if(first_attempt && running < max_concurrent_calls) {
                    // proceed to execution
                    running++;
                    break;
                }
                // place a ticket in a queue and fall asleep
                if( first_attempt ) // do it only once
                    queue.add(new Long(ticket));
                try {
                    queue.wait(); // lock on queue released while sleeping
                } catch (Exception ex) {
                    //System.out.println(ex.getMessage());
                }
                // here we wake up and should check what is in a top of the queue
                long queue_top = ((Long)queue.getFirst()).longValue();
                if( queue_top == ticket && running < max_concurrent_calls ) {
                    // remove the ticket from a queue
                    queue.removeFirst();
                    // increase execution counter and proceed to execution
                    running++;
                    break;
                } else
                    first_attempt = false;
            }
        }
    } // end of request service

    private void endCall() {
        synchronized (queue) {
            running--;
            try {
                queue.notifyAll();
            } catch (Exception ex) {
                //ex.printStackTrace();
            }
        }
    }

    public void handleSigTERM() {
        debug("Going down on SIGTERM");
        doShutdown();
    }

    /** logs debug message */
    public void debug(Object msg) {
        doLog(Constants.DEBUG, msg.toString());
    }

    /** logs warning message */
    public void warn(Object msg) {
        doLog(Constants.WARN, msg.toString());
    }

    /** logs error message */
    public void error(Object msg) {
        doLog(Constants.ERROR, msg.toString());
    }

    /** logs info message */
    public void info(Object msg) {
        doLog(Constants.INFO, msg.toString());
    }

    /** Does actuall logging */
    private void doLog(int level, String msg) {
        StackTraceElement[] st = (new Throwable()).getStackTrace();
        switch( level ) {
        case Constants.DEBUG:
            logger.debug(timestamp+" "+st[2].getClassName()+"::"+st[2].getMethodName()+"() "+
                               st[2].getLineNumber()+" | "+msg);
            break;

        case Constants.WARN:
            logger.warn(timestamp+" "+st[2].getClassName()+"::"+st[2].getMethodName()+"() "+
                               st[2].getLineNumber()+" | "+msg);
            break;

        case Constants.ERROR:
            logger.error(timestamp+" "+st[2].getClassName()+"::"+st[2].getMethodName()+"() "+
                               st[2].getLineNumber()+" | "+msg);
            break;

        case Constants.INFO:
            logger.info(timestamp+" "+st[2].getClassName()+"::"+st[2].getMethodName()+"() "+
                               st[2].getLineNumber()+" |  "+msg);
            break;
        }
    }

    // does nothing, exists just to satisfy Logger interface
    public void set (String what, Object value) {
    }
}
