/************************************************************************
 *
 * Department.java -
 *
 * $Id: Department.java,v 1.4 2006/09/26 22:44:51 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class Department extends ModifyPage {

    public Department() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {

        Hashtable store = new Hashtable();
        store.put("type", "Department");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        try {

            String netID = (String)session.getAttribute("NetID");
            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String rootURL = request.getServletPath();

            if( rootURL.equals("/Department") ) {
                try {
                    Hashtable departmentList = netdb.getInfo(Constants.DEPARTMENT, "");
                    setData(session, "departments", departmentList);
                    store.put("departmentList", departmentList);
                } catch(Exception exx) {
                    exx.printStackTrace();
                    throw new Exception("Failed to load list of departments.");
                }

            } else if( rootURL.equals("/DepartmentsList") ) {
                try {
                    Hashtable departmentList = netdb.getInfo(Constants.DEPARTMENT, "");
                    setData(session, "departments", departmentList);
                    store.put("departmentList", departmentList);
                } catch(Exception exx) {
                    exx.printStackTrace();
                    throw new Exception("Failed to load list of departments.");
                }


            } else {
                try {
                    try {
                        Hashtable oldDepartment = new Hashtable();
                        String department = ((String)Utils.n2s(request.getParameter("departmentName"))).trim();
                        if( department.length() > 0 ) {
                            oldDepartment = netdb.getInfo(Constants.DEPARTMENT, decode(department));
                            if( oldDepartment.size() == 0 )
                                throw new Exception("Department \""+decode(department)+"\" does not exist.");
                        } else {
                            // creating new department
                            oldDepartment.put("department", decode(department));
                            oldDepartment.put("site", "");
                            oldDepartment.put("code", "");
                        };
                        oldDepartment.put("object", "department");
                        store.put("oldDepartment", oldDepartment);
                        store.put("newDepartment", Utils.copyHash(oldDepartment));

                    } catch (NetDBException ex) {
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int j = 0; j < esize; j++ )
                            msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                        throw new Exception(msg);
                    }
                } catch (Exception ex) {
                    addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                    response.sendRedirect("/Department");
                    return;
                }
            }

            PrintWriter out = response.getWriter();
            printOutHtml(session, request, out, store, rootURL);

        } catch (Exception ex) {
            ex.printStackTrace();
            addSessionMsg(session, "Error.errorMsg", "Error: "+ex);
            response.sendRedirect(store!=null?(String)store.get("fromPage"):"/home");
        }
    } // end of doGet(...)

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        HttpSession session = begin(request, response, null);
        if( session == null )
            return;

        String rootURL = request.getServletPath();

        NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");

        String sessionKey = decode(request.getParameter("sessionKey"));
        Hashtable store = (Hashtable)session.getAttribute(sessionKey);
        Hashtable departmentList = (Hashtable)store.get("departmentList");

        try {
            PrintWriter out = response.getWriter();
            String action = request.getParameter("action");
            if( action == null || action.trim().length() == 0 ) {
                if( rootURL.equals("/AddDepartment") )
                    action = "Create Department";
                else if( rootURL.equals("/ModifyDepartment") )
                    action = "Modify Department";
            }

            if( action.equals("Verify") || action.equals("Create Department") || action.equals("Modify Department") ) {
                // pick up new dep. name
                Hashtable newDepartment = (Hashtable)store.get("newDepartment");
                newDepartment.put("name", Utils.nStr(request.getParameter("department")).trim());
                newDepartment.put("RegID", Utils.nStr(request.getParameter("RegID")).trim());

                // fill up all consultants with "not selected" flag
                Hashtable consultants = nHash(newDepartment.get("consultants"));

                for( Hashtable consultant : (Collection<Hashtable>)consultants.values() )
                    consultant.put("not selected", "");

                // pick up consultants on check boxes
                String[] values = request.getParameterValues("consultant");
                if( values != null ) {
                    for( int i = 0; i < values.length; i++ ) {
                        String key = decode(values[i]);
                        ((Hashtable)consultants.get(key)).remove("not selected");
                    }
                }

                // pick up all stale consultants, i.e. those with "not selected" flag
                Vector keysToRemove = new Vector();
                for( Object key : consultants.keySet() )
                    if( Utils.nHash(consultants.get(key)).get("not selected") != null )
                        keysToRemove.addElement(key);

                // now remove them
                for( int i = 0; i < keysToRemove.size(); i++ )
                    consultants.remove(keysToRemove.elementAt(i));

                newDepartment.get("consultants raw data");
                String usersStr = request.getParameter("consultants");
                String newUsersStr = "";
                String[] split = usersStr.split("\\n");
                if( split != null ) {
                    boolean multiple_matches = false;
                    boolean verified = false;
                    boolean no_match = false;
                    for( int i = 0; i < split.length; i++ ) {
                        String userStr = split[i].trim();
                        if( userStr.length() == 0 )
                            continue;
                        Hashtable foundPeople;
                        try {
                            try {
                                boolean got_match = false;
                                if( userStr.endsWith("@") )
                                    foundPeople =
                                        netdb.dirSearch(Constants.NETID,
                                                        userStr.substring(0, userStr.length()-1));
                                else
                                    foundPeople = netdb.dirSearch(Constants.LOOKUP, userStr);

                                // fill them in
                                consultants = Utils.get_hash_or_create(newDepartment, "consultants");
                                if( foundPeople == null )
                                    no_match = true;
                                else if( foundPeople.size() == 0 )
                                    no_match = true;
                                else {
                                    verified = true;
                                    got_match = true;
                                }

                                if( foundPeople.size() > 1 )
                                    multiple_matches = true;

                                if( !got_match )
                                    newUsersStr += (newUsersStr.length()>0?"\n":"")+
                                        userStr;

                                fillInPeople(consultants, foundPeople);

                                // fill in primary
                                for( Hashtable person : (Collection<Hashtable>)consultants.values() )
                                    if( person.get("primary?") == null ) person.put("primary?", Boolean.FALSE);

                            } catch (NetDBException ex) {
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int j = 0; j < esize; j++ )
                                    msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                                throw new Exception(msg);
                            }

                        } catch (Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                        }
                    }

                    if( multiple_matches )
                        addSessionMsg(session, "Error.errorMsg", "ERROR: Multiple matches for consultant.");
                    else {
                        if( no_match ) {
                            addSessionMsg(session, "Error.errorMsg", "ERROR: No matches for consultant.");
                            newDepartment.put("consultants raw data", newUsersStr); // save raw data

                        } else if( verified )
                            addSessionMsg(session, "info_msg", "SUCCESS: Consultants verified.");
                    }
                }

                // pick up primary?
                consultants = nHash(newDepartment.get("consultants"));
                String primary_person_key = decode((String)Utils.n2s(request.getParameter("primary")));

                for( Hashtable consultant : (Collection<Hashtable>)consultants.values() )
                    consultant.put("primary?", Boolean.FALSE);

                nHash(consultants.get(primary_person_key)).put("primary?", Boolean.TRUE);

                // now check if there is only one person left and if so make him/her primary consultant
                if( consultants.size() == 1 )
                    ((Hashtable)consultants.get(consultants.keys().nextElement())).put("primary?", new Boolean(true));

            } // end of if( action.equals("Verify") )


            if( rootURL.equals("/Department") && action.equals("Delete") ) {
                String[] values = request.getParameterValues("departmentlist1");
                if( values != null ) {
                    if( values.length == 0 )
                        addSessionMsg(session, "Error.errorMsg", "Error: Please select department you wish to delete.");
                    else if( values.length > 1 )
                        addSessionMsg(session, "Error.errorMsg", "Error: Please select only one department you wish to delete.");
                    else {
                        // delete
                        try {
                            try {
                                Hashtable department = (Hashtable)departmentList.get(decode(values[0]));
                                if( department == null )
                                    throw new Exception("Selected department does not exist");
                                department.put("object", "department");
                                netdb.delete(department);
                                session.setAttribute("info_msg", "SUCCESS: Department '"+decode(values[0])+"' has been deleted");

                                departmentList = netdb.getInfo(Constants.DEPARTMENT, "");
                                setData(session, "departments", departmentList);
                                store.put("departmentList", departmentList);
                            } catch (NetDBException ex) {
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int j = 0; j < esize; j++ )
                                    msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                                throw new Exception(msg);
                            }
                        } catch (Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                        }
                    }
                } else
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select department you wish to delete.");

            } else if( rootURL.equals("/Department") && action.equals("Replace") ) {
                String[] values1 = request.getParameterValues("departmentlist1");
                String[] values2 = request.getParameterValues("departmentlist2");
                if( values1 == null || values1.length == 0 )
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select one or more departments to replace.");
                else if( values2 == null || values2.length != 1 )
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select department to replace with.");
                else {
                    Hashtable department2 = (Hashtable)departmentList.get(decode(values2[0]));
                    if( department2 == null )
                        throw new Exception("Selected department does not exist");
                    department2.put("object", "department");

                    boolean gotErrors = false;
                    for( int i = 0; i < values1.length; i++ ) {
                        try {
                            try {
                                Hashtable department1 = (Hashtable)departmentList.get(decode(values1[i]));
                                if( department1 == null )
                                    throw new Exception("Selected department does not exist");
                                department1.put("object", "department");
                                netdb.update(department1, department2);
                            } catch (NetDBException ex) {
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int j = 0; j < esize; j++ )
                                    msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                                throw new Exception(msg);
                            }
                        } catch (Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                            gotErrors = true;
                        }
                    }

                    if( !gotErrors )
                        session.setAttribute("info_msg", "SUCCESS: Departments have been replaced");
                    try {
                        departmentList = netdb.getInfo(Constants.DEPARTMENT, "");
                        setData(session, "departments", departmentList);
                        store.put("departmentList", departmentList);
                    } catch(Exception exx) {
                        throw new Exception("Failed to load list of departments.");
                    }
                }

            } else if( rootURL.equals("/Department") && action.equals("Add") ) {
                response.sendRedirect("/AddDepartment");
                return;

            } else if( rootURL.equals("/Department") && action.equals("Modify") ) {
                String[] values1 = request.getParameterValues("departmentlist1");
                if( values1 == null || values1.length == 0 )
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select department you wish to modify.");
                else if( values1.length > 1 )
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select only one department to modify.");
                else {
                    response.sendRedirect("/ModifyDepartment?departmentName="+encode(values1[0]));
                    return;
                }

            } else if( action.equals("Reset") ) {
                store.put("newDepartment", store.get("oldDepartment"));

            } else if( action.equals("Cancel") ) {
                response.sendRedirect("/Department");
                return;

            } else if( ( action.equals("Create Department") || action.equals("Modify Department") ) &&
                       session.getAttribute("Error.errorMsg") == null  ) {
                Hashtable newDepartment = (Hashtable)store.get("newDepartment");
                try {
                    try {
                        Utils.validate(Constants.DEPARTMENT, newDepartment, Constants.NARROW);

                        updatePersons(netdb, nHash(newDepartment.get("consultants")));

                        if( action.equals("Create Department") ) {
                            netdb.insert(newDepartment);
                            session.setAttribute("info_msg", "SUCCESS: Department has been created");
                        } else {
                            netdb.update((Hashtable)store.get("oldDepartment"), newDepartment);
                            session.setAttribute("info_msg", "SUCCESS: Department has been modified.");
                        }
                        response.sendRedirect("/Department");
                        return;

                    } catch (NetDBException ex) {
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int j = 0; j < esize; j++ )
                            msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                        throw new Exception(msg);
                    }
                } catch (Exception ex) {
                    addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                }
            }

            printOutHtml(session, request,out, store, rootURL);

        } catch (Exception ex) {
            ex.printStackTrace();
            addSessionMsg(session, "Error.errorMsg", "Error: "+ex);
            response.sendRedirect(store!=null?(String)store.get("fromPage"):"/home");
        }
    } // end of doPost(...)

    public void printOutHtml(HttpSession session, HttpServletRequest request, PrintWriter out, Hashtable store, String rootURL) throws Exception {
        String sessionKey = (String)store.get("sessionKey");
        out.println(commonHtmlBegin(rootURL.equals("/Department")?"Manage Departments":
                                    (rootURL.equals("/ModifyDepartment")?"Modify Department":
                                     (rootURL.equals("/DepartmentsList")?"List of Departments":"Create Department")))+
                    commonHeader(session, "/Dispatch?Action=redirect&to=/home",
                                 SHOW_HOME_LINK, SHOW_PROFILE_LINK, SHOW_LOGOUT_LINK,
                                 "", rootURL.equals("/Department")?"Manage Departments":
                                 (rootURL.equals("/ModifyDepartment")?"Modify Department":
                                  (rootURL.equals("/DepartmentsList")?"List of Departments":"Create Department")),
                                 "hlpnetdb.html", decode((String)store.get("newFromPage")))+
                    msgPanels(session)+
                    "<DIV id=\"infoPage\">\n"+
                    "<FORM METHOD=\"POST\" NAME=\"Form\" ACTION='"+rootURL+"'>"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"sessionKey\" VALUE=\""+encode(sessionKey)+"\">"+
                    (!rootURL.equals("/DepartmentsList")?("<DIV class='modHeader'>\n"+
                                                          "<TABLE WIDTH='100%'>\n"+
                                                          "<TR><TD NOWRAP>"+
                                                          (rootURL.equals("/Department")?"Select Department(s) to repoint from the left. Select the Department to point to, from the right.":
                                                           (rootURL.equals("/ModifyDepartment")?"Modify an existing Department.":"Create new Department."))+
                                                          "</TD></TR></TABLE></DIV>"):""));


        if( rootURL.equals("/Department") ) {
            out.println("<BLOCKQUOTE>&nbsp;Sortable list of <a href='/DepartmentsList' target='_blank'>departments</a>."+
                        "<TABLE WIDTH='100%'>");
            Hashtable departmentList = (Hashtable)store.get("departmentList");
            out.println("<TR><TD WIDTH='1%'>&nbsp;<BR><SELECT ID='Selector_Department' SIZE=30 NAME='departmentlist1' "+
                        "ondblclick=\"document.Form.Button_Modify.click()\" "+
                        "onKeyDown=\"if( event.keyCode == 13 ) document.Form.Button_Modify.click();\" MULTIPLE>");
            java.util.List list = Arrays.asList(departmentList.keySet().toArray());
            Collections.sort(list, name_cmp);
            Iterator it = list.iterator();
            while( it.hasNext() ) {
                Object key = it.next();
                String reg_id = Utils.nStr(((Hashtable)departmentList.get(key)).get("RegID")).trim();
                String department = ((String)key).trim();
                out.println("<OPTION "+(reg_id.length()==0?"STYLE='font-style: italic;' ":"")+
                            "VALUE='"+encode(department)+"'>"+Utils.htmlize(department, false));
            }

            out.println("</SELECT><BR><BR><INPUT ID='Button_Add' TYPE='SUBMIT' NAME='action' VALUE='Add'> "+
                        "<INPUT ID='Button_Modify' TYPE='SUBMIT' NAME='action' VALUE='Modify'> "+
                        "<INPUT ID='Button_Delete' TYPE='SUBMIT' NAME='action' VALUE='Delete'></TD>");

            out.println("<TD WIDTH='1%'> Becomes </TD><TD>&nbsp;<BR><SELECT ID='Selector_Replacement' SIZE=30 NAME='departmentlist2'>");
            list = Arrays.asList(departmentList.keySet().toArray());
            Collections.sort(list, name_cmp);
            it = list.iterator();
            while( it.hasNext() ) {
                Object key = it.next();
                String reg_id = Utils.nStr(((Hashtable)departmentList.get(key)).get("RegID")).trim();
                String department = ((String)key).trim();
                out.println("<OPTION "+(reg_id.length()==0?"STYLE='font-style: italic;' ":"")+"VALUE='"+
                            encode(department)+"'>"+Utils.htmlize(department, false));
            }

            out.println("</SELECT><BR><BR><INPUT ID='Button_Replace' TYPE='SUBMIT' NAME='action' VALUE='Replace'></TD></TR>"+
                        "</TABLE></BLOCKQUOTE>\n"+
                        "<HR NOSHADE SIZE=2>\n"+
                        "</FORM>\n<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");

        } else if( rootURL.equals("/DepartmentsList") ) {
            // sortable list of locations
            String sorted_column = request.getParameter("sort");
            if( sorted_column == null || sorted_column.trim().length() == 0 )
                sorted_column = "name";
            String sort_dir = request.getParameter("dir");
            if( sort_dir == null || sort_dir.trim().length() == 0 )
                sort_dir = "down";
            String dir_name  = sorted_column.equals("name")?(sort_dir.equals("up")?"down":"up"):"down";
            String dir_regid = sorted_column.equals("RegID")?(sort_dir.equals("up")?"down":"up"):"down";

            out.println("<BLOCKQUOTE><TABLE><THEAD frame=bellow><TR>"+
                        "<TD NOWRAP><A HREF=\"/DepartmentsList?sort=name&dir="+encode(dir_name)+"\">"+
                        "Department</A>&nbsp;&nbsp;</TD>"+
                        "<TD NOWRAP><A HREF=\"/DepartmentsList?sort=RegID&dir="+encode(dir_regid)+"\">"+
                        "RegID</A>&nbsp;&nbsp;</TD>"+
                        "</TR></THEAD>\n<TBODY>");

            Object[] departments =
                TableHashSort.doSort((Hashtable)store.get("departmentList"), sorted_column,
                                     TableHashSort.STRING, sort_dir.equals("down")?TableHashSort.DOWN:TableHashSort.UP);
            for( int i = 0; i < departments.length; i++ ) {
                out.println("<TR><TD NOWRAP>"+Utils.htmlize((String)((Hashtable)((Map.Entry)departments[i]).getValue()).get("name"), false)+
                            "</TD><TD NOWRAP>"+Utils.htmlize(Utils.nStr(((Hashtable)((Map.Entry)departments[i]).getValue()).get("RegID")), false)+
                            "</TD></TR>");
            }

            out.println(""+
                        "</TABLE></BLOCKQUOTE>\n"+
                        "<HR NOSHADE SIZE=2>\n"+
                        "</FORM>\n<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");

        } else { // adding or modifying department
            out.println("<TABLE WIDTH='100%'>");
            Hashtable newDepartment = (Hashtable)store.get("newDepartment");
            out.println("<TR><TD VALIGN='TOP' WIDTH='2%'>&nbsp;<font class='star'>*</font> Department</TD>"+
                        "<TD><INPUT ID='TextField_Dept_Name' TYPE='TEXT' SIZE='30' NAME='department' value='"+
                        Utils.htmlize((String)Utils.n2s(newDepartment.get("name")), false)+
                        "'></TD></TR>"+
                        "<TR><TD VALIGN='TOP' WIDTH='2%'>&nbsp;RegID</TD>"+
                        "<TD><INPUT ID='TextField_Dept_RegID' TYPE='TEXT' SIZE='30' NAME='RegID' value='"+
                        Utils.htmlize((String)Utils.n2s(newDepartment.get("RegID")), false)+
                        "'></TD></TR>");

            Hashtable consultants = nHash(newDepartment.get("consultants"));

            out.println("<TR><TD VALIGN='TOP' WIDTH='2%'>&nbsp;Consultants</TD>\n"+
                        "<TD VALIGN='TOP'><TABLE>"+
                        (consultants.size()>0?("<TR><TD WIDTH='2%' NOWRAP>Uncheck to remove"+
                                               "&nbsp;&nbsp;&nbsp;&nbsp;</TD><TD NOWRAP>Select primary</TD></TR>"):""));
            java.util.List list = Arrays.asList(consultants.keySet().toArray());
            Collections.sort(list, name_cmp);
            Iterator it = list.iterator();
            int index = 0;

            while( it.hasNext() ) {
                String key = (String)it.next();
                Hashtable person = (Hashtable)consultants.get(key);
                String dirID = (String)person.get("DirID");
                String regID = (String)person.get("RegID");
                String dep = (String)person.get("department");

                String displayName = (String)person.get("name");
                if( displayName == null || displayName.trim().length() == 0 )
                    displayName = (String)person.get("NetID");
                if( displayName == null || displayName.trim().length() == 0 )
                    displayName = (String)person.get("DirID");
                if( displayName == null || displayName.trim().length() == 0 )
                    displayName = regID;

                boolean active = activep(person);

                Boolean primary = (Boolean)person.get("primary?");
                if( primary == null )
                    primary = new Boolean(false);

                index++;
                out.println("<TR><TD WIDTH='2%' NOWRAP><INPUT ID='CheckBox_Consultant_"+index+"' TYPE='checkbox' "+
                            (person.get("not selected")==null?"CHECKED ":"")+
                            "NAME='consultant' value='"+
                            encode(key)+"'>"+
                            (active?"<A HREF=\""+
                             System.getProperty("stanfordwho.url")+dirID.trim()+"\">":"")+
                            displayName+(active?"</A>":"")+
                            ((dep!=null&&dep.trim().length()>0)?" <strong>"+dep+"</strong>":"")+
                            (!active?" ( <em>inactive</em>)":"")+"</TD><TD>"+
                            "<INPUT ID='Radio_Primary_"+index+"' TYPE=RADIO NAME='primary' VALUE='"+encode(key)+"'"+
                            (primary.booleanValue()?" CHECKED":"")+"></TD></TR>");
            }

            out.println("</TABLE>"+
                        "(<em>List name or SUNet ID, one per line. Append @ if searching only "+
                        "by sunet ID, as in sunetid@</em>)<BR>"+
                        "<TEXTAREA ID='TextArea_Admins' NAME='consultants' ROWS='3' COLS='21'>"+
                        (newDepartment.get("consultants raw data")!=null?
                         Utils.htmlize((String)newDepartment.get("consultants raw data"), false):"")+
                        "</TEXTAREA>"+
                        "&nbsp;<INPUT ID='Button_Verify' TYPE='SUBMIT' NAME='action' VALUE='Verify'>"+
                        "</TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                        "<TR COLSPAN='2'><TD NOWRAP><INPUT ID='Button_Create_Or_Modify' TYPE='SUBMIT' NAME='action' VALUE='"+
                        (rootURL.equals("/AddDepartment")?"Create Department":"Modify Department")+
                        "'> <INPUT ID='Button_Reset' TYPE='SUBMIT' NAME='action' VALUE='Reset'> <INPUT ID='Button_Cancel' TYPE='SUBMIT' NAME='action' VALUE='Cancel'></TD></TR>"+
                        "</TABLE>\n"+
                        "<HR NOSHADE SIZE=2>\n"+
                        "</FORM>\n<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");
        }
    } // end of printOutHtml(...)

}
