/************************************************************************
 *
 * Dispatch.java -
 *
 * $Id: Dispatch.java,v 1.2 2006/09/26 23:12:41 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.sql.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class Dispatch extends HttpServlet
{
    public Dispatch() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        // no session? relogin user at home page
        HttpSession session = request.getSession(false);
        if (session == null)
        {
            response.sendRedirect("/home");
            return;
        }

        // build redirection string
        String url = "/";
        String object = request.getParameter("object");
        String action = request.getParameter("Action");
        if( action.equals("delete") )
            url = "/Delete?object="+object+"&";

        else if( action.equals("info") )
            url = "/Info"+object+"?";

        else if( action.equals("modify") )
            url = "/Modify"+object+"?";
        else if( action.equals("use") || action.equals("create") )
            url = "/Create"+object+"?";

        else if( action.equals("redirect") ) {
            String pre_action = (String)Utils.n2s(request.getParameter("pre"));
            // do the pre action
            String to = request.getParameter("object");
            if( to == null ) {
                to = request.getParameter("to");
                if( to == null )
                    to = "/home";
            }

            if( pre_action.equals("unlock") ) {
                try {
                    String sessionKey =
                        URLDecoder.decode((String)Utils.n2s(request.getParameter("sessionKey")),
                                          "UTF-8");
                    NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
                    Hashtable store = (Hashtable)session.getAttribute(sessionKey);
                    to = request.getParameter("to");
                    String handle = (String)store.get("handle");
                    Timestamp lockTm = (Timestamp)store.get("lockTm");
                    if( handle != null && lockTm != null ) {
                        try {
                            int what = -1;
                            if( object.equals("Node") )
                                what = Constants.NODE;

                            else if( object.equals("AdminTeam") )
                                what = Constants.ATEAM;

                            else if( object.equals("Network") )
                                what = Constants.NETWORK;

                            else if( object.equals("User") )
                                what = Constants.USER;

                            else if( object.equals("Group") )
                                what = Constants.GROUP;

                            else if( object.equals("Domain") )
                                what = Constants.DOMAIN;

                            if( ( what == Constants.NODE || what == Constants.NETWORK  ) && Utils.isNumber(handle) )
                                netdb.unlock(what, Integer.parseInt(handle), lockTm);
                            else
                                netdb.unlock(what, handle, lockTm);

                        } catch (NetDBException ex) {
                            /*
                            Vector errors = ex.getMessages();
                            String msg = "";
                            int esize = errors.size();
                            for( int i = 0; i < esize; i++ )
                                msg += "Error: "+(String)errors.elementAt(i)+(i<esize-1?"\n":"");
                            throw new Exception(msg);
                            */
                        }
                    }

                    try {
                        for( String ip : (Set<String>)Utils.nHash(store.get("reserved_ips")).keySet() ) {
                            try {
                                netdb.cancelIP(ip);
                            } catch (NetDBException ex) {
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int i = 0; i < esize; i++ )
                                    msg += "Error: "+(String)errors.elementAt(i)+(i<esize-1?"\n":"");
                                throw new Exception(msg);
                            }
                        }

                        for( Hashtable nameStore : (Collection<Hashtable>)Utils.nHash(store.get("reserved_names")).values() ) {
                            try {
                                netdb.cancelName((String)nameStore.get("fqdn"), (String)nameStore.get("type"));
                            } catch (NetDBException ex) {
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int i = 0; i < esize; i++ )
                                    msg += "Error: "+(String)errors.elementAt(i)+(i<esize-1?"\n":"");
                                throw new Exception(msg);
                            }
                        }

                    } catch (NetDBException ex) {
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int i = 0; i < esize; i++ )
                            msg += "Error: "+(String)errors.elementAt(i)+(i<esize-1?"\n":"");
                        throw new Exception(msg);
                    }

                } catch(Exception ex) {
                    ex.printStackTrace();
                    session.setAttribute("Error.errorMsg", ex.getMessage());

                } finally {
                    response.sendRedirect(to);
                }
                return;

            } else if( pre_action.equals("remove_from_session") )
                session.removeAttribute((String)Utils.n2s(request.getParameter("key")));

            response.sendRedirect(to);
            return;
        }

        Enumeration enm = request.getParameterNames();
        while( enm.hasMoreElements() ) {
            String param = (String)enm.nextElement();
            String value = request.getParameter(param);
            if( param.equals("object") || param.equals("Action") )
                continue;
            url += param + "=" + URLEncoder.encode(value, "UTF-8") +
                (enm.hasMoreElements()?"&":"");
        }

        response.sendRedirect(url);
    }
}
