/************************************************************************
 *
 * FullSearchSort.java -
 *
 * $Id: FullSearchSort.java,v 1.2 2006/10/27 21:33:49 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2005, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.util.*;
import java.text.*;
import java.sql.*;
import stanford.netdb.utils.*;

public class FullSearchSort extends CommonPage {

    Hashtable search_data;
    String sorting_column;      // name of the column to sort
    String type;                // type of the fields
    String dir;                 // direction

    static Comparator string_cmp;
    static Comparator timestamp_cmp;
    static Comparator ip_cmp;

    static {
        string_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    if( o1.toString().trim().length() == 0 && o2.toString().trim().length() != 0 )
                        return 1;
                    if( o2.toString().trim().length() == 0 && o1.toString().trim().length() != 0 )
                        return -1;
                    return (o1.toString()).compareToIgnoreCase(o2.toString());
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        timestamp_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    return ((Timestamp)o1).compareTo((Timestamp)o2);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        ip_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    long cmp_val =
                        Utils.ipstr2long((String)o1) - Utils.ipstr2long((String)o2);
                    return cmp_val>0?1:(cmp_val<0?-1:0);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        num_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    long cmp_val =
                        Integer.parseInt((String)o1) - Integer.parseInt((String)o2);
                    return cmp_val>0?1:(cmp_val<0?-1:0);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };
    }

    public FullSearchSort(Hashtable search_data, String sorting_column, String type, String dir) {
        super();
        this.search_data        = search_data;
        this.sorting_column     = sorting_column;
        this.type               = type;
        this.dir                = (dir==null||dir.trim().equals(""))?"down":dir;
    }

    public Object[] doSort() {
        Comparator cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Hashtable row1 = (Hashtable)((Map.Entry)o1).getValue();
                    Hashtable row2 = (Hashtable)((Map.Entry)o2).getValue();

                    Vector cell1 = (Vector)row1.get(sorting_column);
                    Vector cell2 = (Vector)row2.get(sorting_column);

                    Comparator cell_cmp = string_cmp;
                    if( type.equals("Timestamp") )
                        cell_cmp = timestamp_cmp;
                    else if( type.equals("IP") || type.equals("NoSortIP") )
                        cell_cmp = ip_cmp;
                    else if ( type.equals("NoSortNumber") )
                        cell_cmp = num_cmp;

                    // convert to string and run string_cmp
                    java.util.List clist = Arrays.asList(cell1.toArray());
                    Collections.sort(clist, cell_cmp);
                    Object[] cdata = clist.toArray();

                    Object cmp1 = null;
                    if( cdata.length > 0 )
                        cmp1 = cdata[0];

                    clist = Arrays.asList(cell2.toArray());
                    Collections.sort(clist, cell_cmp);
                    cdata = clist.toArray();

                    Object cmp2 = null;
                    if( cdata.length > 0 )
                        cmp2 = cdata[0];

                    if( type.equals("Timestamp") ) {
                        if( cmp1 == null )
                            cmp1 = new Timestamp(0);
                        if( cmp2 == null )
                            cmp2 = new Timestamp(0);
                        if( dir.equals("down") )
                            return ((Timestamp)cmp1).compareTo((Timestamp)cmp2);
                        else
                            return ((Timestamp)cmp2).compareTo((Timestamp)cmp1);
                    } else if( type.equals("IP") || type.equals("NoSortIP") ) {
                        if( cmp1 == null )
                            cmp1 = "0.0.0.0";
                        else
                            cmp1 = cmp1.toString();

                        if( cmp2 == null )
                            cmp2 = "0.0.0.0";
                        else
                            cmp2 = cmp2.toString();
                        long cmp_val =
                            Utils.ipstr2long((String)cmp1) - Utils.ipstr2long((String)cmp2);

                        if( dir.equals("down") )
                            return cmp_val>0?1:(cmp_val<0?-1:0);
                        else
                            return cmp_val>0?-1:(cmp_val<0?1:0);
                    } else if( type.equals("Numeric") || type.equals("NoSortNumber") ) {
                        try {
                            long val1 = Long.parseLong((String)cmp1);
                            long val2 = Long.parseLong((String)cmp2);
                            if( dir.equals("down") )
                                return val1>val2?1:(val1==val2?0:-1);
                            else
                                return val1>val2?-1:(val1==val2?0:1);
                        } catch (Exception ex) { }
                    }

                    // do string cmp
                    if( cmp1 == null )
                        cmp1 = "";
                    else
                        cmp1 = cmp1.toString();

                    if( cmp2 == null )
                        cmp2 = "";
                    else
                        cmp2 = cmp2.toString();

                    if( dir.equals("down") )
                        return ((String)cmp1).compareToIgnoreCase((String)cmp2);
                    else
                        return ((String)cmp2).compareToIgnoreCase((String)cmp1);
                }

                public boolean equals(Object obj) {
                    return true;
                }
            };

        java.util.List list = Arrays.asList(search_data.entrySet().toArray());
        Collections.sort(list, cmp);
        return list.toArray();
    }
}
