/************************************************************************
 *
 * HomeServlet.java - main home page
 *
 * $Id: HomeServlet.java,v 1.6 2007/04/17 21:11:42 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class HomeServlet extends ModifyPage
{
    public HomeServlet() {
    }

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        response.setContentType("text/html");

        String netID = null;
        NetDBpi netdb = null;

        // no session? relogin user with msg
        HttpSession session = request.getSession(false);
        if (session == null)
        {

            session = request.getSession(true);
            System.gc();

            // enfore session timeout. def. value 15 minutes
            session.setMaxInactiveInterval(Integer.parseInt(System.getProperty(Constants.TIMEOUT_KEY, "900000"))/1000);

            session.setAttribute("Error.errorMsg",
                                 "Your previous session has expired. This is a new session.");

            String webID = request.getHeader(System.getProperty(Constants.AUTH_USER_KEY, "X-WebAuth-User"));

            // following two lines added for testing cases
            if( netID == null &&
                System.getProperty(Constants.CHEAT_USER_KEY) != null &&
                System.getProperty(Constants.CHEAT_USER_KEY).trim().length() > 0 )
                webID = System.getProperty(Constants.CHEAT_USER_KEY);

            if (webID == null || webID.trim().length() == 0)
            {
                response.getWriter().println(
                    getCommonHtmlBegin("Auth failed")+
                    "<BR><H2 ALIGN=\"center\"><FONT COLOR=\"#AA0000\">Authentication failed</FONT></H2>\n"+
                    "</BODY>\n</HTML>"
                );
                return;
            }
            session.setAttribute("NetID", webID.trim());
            try {
                netdb = NetDBHook.createInstance(webID.trim(), false);
                session.setAttribute("sid", (String)(nHash(netdb.getState())).get("timestamp"));
                session.setAttribute("netdb", netdb);
                session.setAttribute("user", netdb.getCurrentUser());
                session.setAttribute("X-Forwarded-For", Utils.n2s(request.getHeader("X-Forwarded-For")));
                (new LoadData(session, netdb)).start();
                ((stanford.netdb.utils.Logger)netdb).info("http session started ["+session.getId()+"]");
            } catch (Exception ex) {
                response.sendRedirect("/LoginFailed?error_msg="+encode(ex.getMessage()));
                return;
            }

            session.setAttribute("master", new NetDBMaster());
        }

        // reset qsearch values
        String search_string = "";
        String search_object = "Nodes";

        try
        {
            netdb = (NetDBpi)session.getAttribute("netdb");
            netID = (String)session.getAttribute("netID");

            // check user access
            Hashtable currentUser = netdb.getCurrentUser();
            Boolean checkValue = (Boolean)currentUser.get("all_objects?");
            boolean allObjects = (checkValue == null ? false : checkValue.booleanValue());
            Hashtable records = Utils.nHash(currentUser.get("records"));

            boolean nodeObject          = false;
            boolean userObject          = false;
            boolean networkObject       = false;
            boolean adminTeamObject     = false;
            boolean groupObject         = false;
            boolean domainObject        = false;

            // cache user access flags in session (first time only)
            nodeObject = checkAccess(session, records, "hasNodeObjectAccess?", "Node", allObjects);
            userObject = checkAccess(session, records, "hasUserObjectAccess?", "Users", allObjects);
            networkObject = checkAccess(session, records, "hasNetworkObjectAccess?", "Network", allObjects);
            adminTeamObject = checkAccess(session, records, "hasATeamObjectAccess?", "AdminTeam", allObjects);
            groupObject = checkAccess(session, records, "hasGroupObjectAccess?", "Owner", allObjects);
            domainObject = checkAccess(session, records, "hasDomainObjectAccess?", "Domain", allObjects);

            // cache user's default domain in session (first time only)
            String default_domain = (String)session.getAttribute("default_domain");
            if (default_domain == null)
            {
               default_domain = (String)currentUser.get("default_domain");
               session.setAttribute("default_domain", default_domain);
            }

            session.setAttribute("user_name", Utils.nStr(currentUser.get("name")));
            String welcome_msg = "Welcome to NetDB,<BR>" + session.getAttribute("user_name");

            response.getWriter().println(
                getCommonHtmlBegin("NetDB Home")+
                getCommonHeaderTable("", welcome_msg, netID, "/home")+
                msgPanels(session)+
                "<BR>\n"+
                getCommonQSearchForm(search_string, search_object, encode("/home"))+
                ((nodeObject || userObject || networkObject || adminTeamObject || groupObject || domainObject)?
                    "<BR>\n\n"+
                    "<DIV id=\"create_panel\">\n"+
                    "<TABLE ALIGN='CENTER' WIDTH='90%'   BORDER='1' CELLSPACING='0' CELLPADDING='3' CALS=3>\n"+
                    "<TR><TD>\n"+
                    "      <TABLE ALIGN='CENTER' WIDTH='100%' BORDER='0' CELLSPACING='0' CELLPADDING='2'>\n"+
                    "      <TR>\n"+
                    "        <TD ALIGN='LEFT' WIDTH='20%'><B>Create</B></TD>\n<TD ALIGN='LEFT'>\n"+
                    (nodeObject?"<A ID='Link_Create_Node' HREF='/Dispatch?Action=create&object=Node&fromPage=/home'>Node</A>&nbsp;&nbsp;\n":"")+
                    (userObject?"<A ID='Link_Create_User' HREF='/Dispatch?Action=create&object=User&fromPage=/home'>User</A>&nbsp;&nbsp;\n":"")+
                    (networkObject?"<A ID='Link_Create_Network' HREF='/Dispatch?Action=create&object=Network&fromPage=/home'>Network</A>&nbsp;&nbsp;\n":"")+
                    (groupObject?"<A ID='Link_Create_Group' HREF='/Dispatch?Action=create&object=Group&fromPage=/home'>Group</A>&nbsp;&nbsp;\n":"")+
                    (adminTeamObject?"<A ID='Link_Create_AdminTeam' HREF='/Dispatch?Action=create&object=AdminTeam&fromPage=/home'>Admin&nbsp;Team</A>&nbsp;&nbsp;\n":"")+
                    (domainObject?"<A ID='Link_Create_Domain' HREF='/Dispatch?Action=create&object=Domain&fromPage=/home'>Domain</A>\n":"")+
                    "        </TD>\n"+
                    "      </TR>"+
                    "      </TABLE>\n"+
                    "    </TD></TR>\n"+
                    "</TABLE>\n</DIV>\n":"")+
                ((records != null && records.get("SystemManagement") != null)?
                    "<BR>\n\n"+
                    "<DIV id=\"sys_management\">\n"+
                    "<TABLE ALIGN='CENTER' WIDTH='90%'   BORDER='1' CELLSPACING='0' CELLPADDING='3' CALS=3>\n"+
                    "<TR><TD>\n"+
                    "      <TABLE ALIGN='CENTER' WIDTH='100%' BORDER='0' CELLSPACING='0' CELLPADDING='2'>\n"+
                    "      <TR>\n"+
                    "        <TD ALIGN='LEFT' WIDTH='20%'><B>Manage</B></TD>\n<TD ALIGN='LEFT'>\n"+
                    "          <A ID='Manage_Department_Link' HREF='/Department'>Department</A>&nbsp;&nbsp;\n"+
                    "          <A ID='Manage_Location_Link' HREF='/Location'>Location</A>&nbsp;&nbsp;\n"+
                    "          <A ID='Manage_Make_Link' HREF='/Make'>Make</A>&nbsp;&nbsp;\n"+
                    "          <A ID='Manage_Model_Link' HREF='/Model'>Model</A>&nbsp;&nbsp;\n"+
                    "          <A ID='Manage_OS_Link' HREF='/OS'>OS</A>&nbsp;&nbsp;\n"+
                    "          <A ID='Manage_State_Link' HREF='/State'>State</A>\n"+
                    "        </TD>\n"+
                    "      </TR>"+
                    "      </TABLE>\n"+
                    "    </TD></TR>\n"+
                    "</TABLE>\n</DIV>\n":"")+
                "<BR>\n"+getCommonBottomLabel()+
                "</BODY>\n"+
                "</HTML>\n"
            );
        } catch (Exception ex) {
            printCommonErrorPage(response, ex);
            return;
        }
    }

    private boolean checkAccess(HttpSession session, Hashtable records, String atype, String stype, boolean allObjects) {
        Boolean checkValue = (Boolean)session.getAttribute(atype);
        if (checkValue == null) {
            if (allObjects || records.get(stype) != null)
                checkValue = Boolean.TRUE;
            else
                checkValue = Boolean.FALSE;
            session.setAttribute(atype, checkValue);
        }
        return checkValue.booleanValue();
    }

    class LoadData extends Thread {
        private HttpSession session;
        private NetDBpi netdb;

        public LoadData(HttpSession session, NetDBpi netdb) {
            this.session = session;
            this.netdb = netdb;
        }

        public void run() {
            try {
                session.setAttribute("departments",       netdb.getInfo(Constants.DEPARTMENT, ""));
                session.setAttribute("locations",         netdb.getInfo(Constants.LOCATION, ""));
                session.setAttribute("dhcp_options",      netdb.getInfo(Constants.DHCP_OPTIONS, ""));
                session.setAttribute("models",            netdb.getInfo(Constants.MODEL, 0));
                session.setAttribute("model_types",       netdb.getInfo(Constants.MODEL_TYPE, 0));
                session.setAttribute("oses",              netdb.getInfo(Constants.OS, 0));
                session.setAttribute("myGroups",          netdb.listMyGroups("*"));
                session.setAttribute("myAddrSpaces",      netdb.listMyAddrSpaces());
                session.setAttribute("dhcp_services",     netdb.listDHCPServices());
                session.setAttribute("domains",           netdb.listMyDomains());
                session.setAttribute("allRecords",        netdb.listRecords());
                session.setAttribute("states",            netdb.listMyStates(null));
                session.setAttribute("custom.node.views", (new CNVParser()).loadCustomNodeViewsData());
                session.setAttribute("node types",        netdb.list(Constants.NODE_TYPE, null, false));
                session.setAttribute("my node types",     netdb.list(Constants.NODE_TYPE, null, true));

                Hashtable global_defaults = netdb.getGlobalDefaults();
                session.setAttribute("global.defaults",   global_defaults);

                Integer custom_fields_total = new Integer(Constants.MAX_CUSTOM_FIELDS);
                try {
                    if( global_defaults.get("MaxNodeCustomFields") != null )
                        custom_fields_total = (Integer)global_defaults.get("MaxNodeCustomFields");
                } catch (Exception exx) { }
                session.setAttribute("custom_fields_total", custom_fields_total);

                session.setAttribute("dataReady", "");
            } catch (Exception ex) {
            }
        }
    }
}
