/************************************************************************
 *
 * InfoAdminTeam.java - displays info for an admin-team
 *
 * $Id: InfoAdminTeam.java,v 1.2 2006/11/16 23:20:38 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;  // SimpleDateFormat
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class InfoAdminTeam extends InfoPage
{
    public InfoAdminTeam() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("no session store", "");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        try {
            PrintWriter out = response.getWriter();

            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String netID = (String)session.getAttribute("NetID");

            // obtain admin team key attribute from session
            String object = (String)request.getParameter("handle");
            String oid = (object == null ? "???" : URLEncoder.encode(object, "UTF-8"));

            // lookup admin team info
            Hashtable info = netdb.getInfo(Constants.ATEAM, object);
            if (info == null || info.get("id") == null ) {
                session.setAttribute("Error.errorMsg",
                                     "No info found for admin team "+object);
                response.sendRedirect((String)store.get("fromPage"));
                return;
            }

            // obtain admin team access flag from session
            Boolean checkValue = (Boolean)session.getAttribute("hasATeamObjectAccess?");
            boolean hasAccess = (checkValue != null && checkValue.booleanValue());
            if( hasAccess && !netdb.checkGroupAccess(info) )
                hasAccess = false;

            // info header and function bar
            out.println(commonHtmlBegin("Admin Team Info: "+Utils.htmlize(object, false))+
                        commonHeader(session, "/home", SHOW_HOME_LINK, SHOW_PROFILE_LINK, SHOW_LOGOUT_LINK,
                                     "", "Admin Team Info<br>"+Utils.htmlize(object), "hlpateam.html",
                                     decode((String)store.get("newFromPage")))+
                        msgPanels(session)+
                        "<DIV id=\"infoPage\">\n"+
                        commonLinksPanel("AdminTeam", hasAccess, object, store));

            printOutInfo(session, response, out, info, (String)store.get("newFromPage"));


            // info function bar, qsearch panel, footer
            out.println(commonLinksPanel("AdminTeam", hasAccess, object, store)+
                        "<HR NOSHADE SIZE=1>\n"+
                        getCommonQSearchForm("", "Nodes", (String)store.get("newFromPage"))+
                        "<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");

            end(request, response);

        } catch (Exception ex) {
            // display error page
            handleException(response, ex);
            return;
        }
    } // end of doGet(...)

    public static void printOutInfo(HttpSession session, HttpServletResponse response, PrintWriter out,
                                    Hashtable info, String newFromPage)
        throws Exception
    {
        String o_name = (String)info.get("name");
        o_name = (o_name == null ? "" : o_name);

        String o_email = (String)info.get("email");
        o_email = (o_email == null ? "" : o_email);

        String o_phone = (String)info.get("phone");
        o_phone = (o_phone == null ? "" : o_phone);

        String o_comment = (String)info.get("comment");
        o_comment = (o_comment == null ? "" : o_comment);

        String o_groups = "";
        Hashtable ht = (Hashtable)info.get("groups");
        if (ht != null)
            {
                java.util.List alist = Arrays.asList(ht.keySet().toArray());
                Collections.sort(alist, name_cmp);
                Object[] oa = alist.toArray();
                for (int i = 0; i < oa.length; ++i)
                    {
                        if (o_groups.length() > 0)
                            o_groups += ", ";
                        o_groups += Utils.htmlize((String)oa[i], false);
                    }
            }

        String o_depts = "";
        ht = (Hashtable)info.get("departments");
        if (ht != null)
            {
                java.util.List alist = Arrays.asList(ht.keySet().toArray());
                Collections.sort(alist, name_cmp);
                Object[] oa = alist.toArray();
                for (int i = 0; i < oa.length; ++i)
                    {
                        if (o_depts.length() > 0)
                            o_depts += ", ";
                        o_depts += Utils.htmlize((String)oa[i], false);
                    }
            }

        String o_members = "";
        ht = (Hashtable)info.get("members");
        if (ht != null)
            {
                java.util.List alist = Arrays.asList(ht.values().toArray());  // <PERSON>s
                Collections.sort(alist, person_cmp);  // special comparator
                Object[] oa = alist.toArray();
                for (int i = 0; i < oa.length; ++i)
                    {
                        Hashtable person = (Hashtable)oa[i];
                        String m_name = (String)person.get("name");
                        String m_link = makePersonLink(person);
                        String m_data =
                            (m_link != null ? "<A HREF='"+m_link+"'>" : "")+
                            Utils.htmlize(m_name, false)+
                            (m_link != null ? "</A>" : "");
                        if (o_members.length() > 0)
                            o_members += "<BR>";
                        o_members += m_data;
                    }
            }

        Hashtable person = (Hashtable)info.get("created.by");
        String o_clink = makePersonLink(person);
        String o_cdate = sdf.format(info.get("created"));
        String o_cacct = (String)person.get("NetID");
        o_cacct = (o_cacct == null ? "" : o_cacct);
        String o_create =
            (o_clink != null ? "<A HREF='"+o_clink+"'>" : "")+
            (String)person.get("name")+
            " ("+o_cacct+")" +
            (o_clink != null ? "</A>" : "")+
            "&nbsp; "+o_cdate;

        person = (Hashtable)info.get("mod.by");
        String o_mlink = makePersonLink(person);
        String o_mdate = sdf.format(info.get("modified"));
        String o_macct = (String)person.get("NetID");
        o_macct = (o_macct == null ? "" : o_macct);
        String o_modify =
            (o_mlink != null ? "<A HREF='"+o_mlink+"'>" : "")+
            (String)person.get("name") +
            " ("+o_macct+")" +
            (o_mlink != null ? "</A>" : "")+
            "&nbsp; "+o_mdate;

        // admin team info content
        out.println(
                    "<TABLE WIDTH='100%'>\n"+
                    "<TR><TD COLSPAN='3' HEIGHT='5'></TD></TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Name</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+Utils.htmlize(o_name, false)+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Email</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+Utils.htmlize(o_email, false)+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Phone</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+Utils.htmlize(o_phone, false)+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Groups</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_groups+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Depts</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_depts+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Members</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_members+"</B></TD>\n"+
                    "</TR>\n"+
                    (o_comment.length() > 0 ?
                     "<TR>\n"+
                     "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Comment</TD>\n"+
                     "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                     "  <TD VALIGN='TOP'><B>"+Utils.htmlize(o_comment)+"</B></TD>\n"+
                     "</TR>\n":"")+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Created By</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_create+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Last Modified By</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_modify+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR><TD COLSPAN='3' HEIGHT='5'></TD></TR>\n"+
                    "</TABLE>\n"
                    );

    } // end of printOutInfo(...)
}
