/************************************************************************
 *
 * InfoDomain.java - displays info for a domain
 *
 * $Id: InfoDomain.java,v 1.4 2007/01/09 00:43:09 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;  // SimpleDateFormat
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class InfoDomain extends InfoPage
{
    public InfoDomain() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("no session store", "");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;


        try {
            PrintWriter out = response.getWriter();
            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String netID = (String)session.getAttribute("NetID");

            // obtain domain key attribute from request
            String object = request.getParameter("handle");
            String handle = new String(object);

            // obtain domain access flag from session
            Boolean checkValue = (Boolean)session.getAttribute("hasDomainObjectAccess?");
            boolean hasAccess = (checkValue != null && checkValue.booleanValue());

            // lookup domain info
            Hashtable info = netdb.getInfo(Constants.DOMAIN, object);
            if (info == null || info.get("id") == null ) {
                session.setAttribute("Error.errorMsg",
                                     "Error: No info found for domain "+object);
                response.sendRedirect((String)store.get("fromPage"));
                return;
            }

            if( hasAccess && !netdb.checkGroupAccess(info) )
                hasAccess = false;

            // info header and function bar
            out.println(commonHtmlBegin("DomainInfo: "+object)+
                        commonHeader(session, "/home", SHOW_HOME_LINK, SHOW_PROFILE_LINK, SHOW_LOGOUT_LINK,
                                     "", "Domain Info<br>"+object, "hlpdomain.html",
                                     decode((String)store.get("newFromPage")))+
                        msgPanels(session)+
                        "<DIV id=\"infoPage\">\n"+
                        commonLinksPanel("Domain", hasAccess, handle, store));

            printOutInfo(session, response, out, info, (String)store.get("newFromPage"));

            // info function bar, qsearch panel, footer
            out.println(commonLinksPanel("Domain", hasAccess, handle, store)+
                        "<HR NOSHADE SIZE=1>\n"+
                        getCommonQSearchForm("", "Nodes", (String)store.get("newFromPage"))+
                        "<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");
        } catch (Exception ex) {
            printCommonErrorPage(response, ex);
            return;
        }
    } // end of doGet(...)

    public static void printOutInfo(HttpSession session,
                                    HttpServletResponse response,
                                    PrintWriter out, Hashtable info,
                                    String newFromPage)
        throws Exception
    {
        String o_domain = (String)info.get("domain");
        o_domain = (o_domain == null ? "" : o_domain);

        Boolean b = (Boolean)info.get("limited?");
        String o_limited = (b == null ? "No" : (b.booleanValue() ? "Yes" : "No"));

        b = (Boolean)info.get("allow_name?");
        String o_allowname = (b == null ? "No" : (b.booleanValue() ? "Yes" : "No"));

        Hashtable person = (Hashtable)info.get("created.by");
        String o_clink = makePersonLink(person);
        String o_cdate = sdf.format(info.get("created"));
        String o_cacct = (String)person.get("NetID");
        o_cacct = (o_cacct == null ? "" : o_cacct);
        String o_create =
            (o_clink != null ? "<A HREF='"+o_clink+"'>" : "")+
            (String)person.get("name")+
            " ("+o_cacct+")" +
            (o_clink != null ? "</A>" : "")+
            "&nbsp; "+o_cdate;

        person = (Hashtable)info.get("mod.by");
        String o_mlink = makePersonLink(person);
        String o_mdate = sdf.format(info.get("modified"));
        String o_macct = (String)person.get("NetID");
        o_macct = (o_macct == null ? "" : o_macct);
        String o_modify =
            (o_mlink != null ? "<A HREF='"+o_mlink+"'>" : "")+
            (String)person.get("name") +
            " ("+o_macct+")" +
            (o_mlink != null ? "</A>" : "")+
            "&nbsp; "+o_mdate;

        // domain info content
        out.println(
                    "<TABLE WIDTH='100%'>\n"+
                    "<TR><TD COLSPAN='3' HEIGHT='5'></TD></TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Domain</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_domain+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Limited</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_limited+"<BR></B></TD>\n"+
                    "</TR><TR><TD COLSPAN='3'>&nbsp;</TD></TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Use as name (Groups)</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>");

        int index = 0;
        Hashtable use_as_name_gr = Utils.nHash(info.get("useAsName"));
        for( String group : (java.util.List<String>)Utils.sort(use_as_name_gr.keySet(), name_cmp) )
            out.println(Utils.htmlize(group, false)+(++index<use_as_name_gr.size()?",":""));

        out.println("</B></TD><TR><TD COLSPAN='3'>&nbsp;</TD></TR>"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Assign names in (Groups)</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>");

        index = 0;
        Hashtable assign_gr = Utils.nHash(info.get("assign"));
        for( String group : (java.util.List<String>)Utils.sort(assign_gr.keySet(), name_cmp) )
            out.println(Utils.htmlize(group, false)+(++index<assign_gr.size()?",":""));

        out.println("</B></TD></TR>\n"+
                    "<TR><TD COLSPAN='3'>&nbsp;</TD></TR>"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Groups</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>");

        index = 0;
        Hashtable groups = Utils.nHash(info.get("groups"));
        for( String group : (java.util.List<String>)Utils.sort(groups.keySet(), name_cmp) )
            out.println(Utils.htmlize(group, false)+(++index<groups.size()?",":""));

        out.println("</B></TD>\n"+
                    "</TR>\n"+
                    "<TR><TD COLSPAN='3'>&nbsp;</TD></TR>"+
                    (Utils.nStr(info.get("comment")).length() > 0 ?
                     "<TR>\n"+
                     "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Comment</TD>\n"+
                     "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                     "  <TD VALIGN='TOP'><B>"+Utils.htmlize(Utils.nStr(info.get("comment")))+"</B></TD>\n"+
                     "</TR>\n":"")+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Created By</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_create+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Last Modified By</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_modify+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR><TD COLSPAN='3' HEIGHT='5'></TD></TR>\n"+
                    "</TABLE>\n"
                    );
    }
}
