/************************************************************************
 *
 * InfoGroup.java - displays info for a group
 *
 * $Id: InfoGroup.java,v 1.1.1.1 2006/06/30 23:38:10 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;  // SimpleDateFormat
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class InfoGroup extends InfoPage
{
    public InfoGroup() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("no session store", "");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        try {
            PrintWriter out = response.getWriter();

            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String netID = (String)session.getAttribute("NetID");
            String default_domain = (String)session.getAttribute("default_domain");

            // obtain group key attribute from session
            String object = request.getParameter("handle");
            String handle = new String(object);

            // obtain group access flag from session
            Boolean checkValue = (Boolean)session.getAttribute("hasGroupObjectAccess?");
            boolean hasAccess = (checkValue != null && checkValue.booleanValue());

            Hashtable info = netdb.getInfo(Constants.GROUP, object);
            if (info == null || info.get("id") == null ) {
                session.setAttribute("Error.errorMsg",
                                     "Error: No info found for group "+object);
                response.sendRedirect((String)store.get("fromPage"));
                return;
            }

            // group info header and function bar
            out.println(commonHtmlBegin("GroupInfo: "+Utils.htmlize(object, false))+
                        commonHeader(session, "/home", SHOW_HOME_LINK, SHOW_PROFILE_LINK, SHOW_LOGOUT_LINK,
                                     "", "Group Info<br>"+Utils.htmlize(object, false), "hlpgroup.html",
                                     decode((String)store.get("newFromPage")))+
                        msgPanels(session)+
                        "<DIV id=\"infoPage\">\n"+
                        commonLinksPanel("Group", hasAccess, handle, store));

            printOutInfo(session, response, out, info, (String)store.get("newFromPage"));

            out.println(commonLinksPanel("Group", hasAccess, handle, store)+
                        "<HR NOSHADE SIZE=1>\n"+
                        getCommonQSearchForm("", "Nodes", (String)store.get("newFromPage"))+
                        "<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");

        } catch (Exception ex) {
            printCommonErrorPage(response, ex);
            return;
        }
    } // end of doGet(...)

    public static void printOutInfo(HttpSession session, HttpServletResponse response, PrintWriter out,
                                    Hashtable info, String newFromPage)
        throws Exception
    {
        String o_name = (String)info.get("name");
        o_name = (o_name == null ? "" : o_name);

        String o_comment = (String)info.get("comment");
        o_comment = (o_comment == null ? "" : o_comment);

        Hashtable person = (Hashtable)info.get("created.by");
        String o_clink = makePersonLink(person);
        String o_cdate = sdf.format(info.get("created"));
        String o_cacct = (String)person.get("NetID");
        o_cacct = (o_cacct == null ? "" : o_cacct);
        String o_create =
            (o_clink != null ? "<A HREF='"+o_clink+"'>" : "")+
            (String)person.get("name")+
            " ("+o_cacct+")" +
            (o_clink != null ? "</A>" : "")+
            "&nbsp; "+o_cdate;

        person = (Hashtable)info.get("mod.by");
        String o_mlink = makePersonLink(person);
        String o_mdate = sdf.format(info.get("modified"));
        String o_macct = (String)person.get("NetID");
        o_macct = (o_macct == null ? "" : o_macct);
        String o_modify =
            (o_mlink != null ? "<A HREF='"+o_mlink+"'>" : "")+
            (String)person.get("name") +
            " ("+o_macct+")" +
            (o_mlink != null ? "</A>" : "")+
            "&nbsp; "+o_mdate;

        // group info content
        out.println(
                    "<TABLE>\n"+
                    "<TR><TD COLSPAN='3' HEIGHT='5'></TD></TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' NOWRAP>Group</TD>\n"+
                    "  <TD WIDTH='20'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+Utils.htmlize(o_name, false)+"</B></TD>\n"+
                    "</TR>\n"+
                    (o_comment.length() > 0 ?
                     "<TR>\n"+
                     "  <TD VALIGN='TOP' NOWRAP>Comment</TD>\n"+
                     "  <TD WIDTH='20'>&nbsp;</TD>\n"+
                     "  <TD VALIGN='TOP'><B>"+Utils.htmlize(o_comment)+"</B></TD>\n"+
                     "</TR>\n":"")+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' NOWRAP>Created By</TD>\n"+
                    "  <TD WIDTH='20'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_create+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' NOWRAP>Last Modified By</TD>\n"+
                    "  <TD WIDTH='20'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_modify+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR><TD COLSPAN='3' HEIGHT='5'></TD></TR>\n"+
                    "</TABLE>\n"
                    );

    } // end of printOutInfo(...)
}
