/************************************************************************
 *
 * LoginServlet.java - initial http session creation and loggin in
 *
 * $Id: LoginServlet.java,v 1.4 2007/01/04 01:02:48 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class LoginServlet extends ModifyPage
{
    public LoginServlet() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        response.setContentType("text/html");

        HttpSession session = request.getSession(false);
        if( session != null ) {
            session.invalidate();
        }
        session = request.getSession(true);
        System.gc();

        // enfore session timeout. def. value 15 minutes
        session.setMaxInactiveInterval(Integer.parseInt(System.getProperty(Constants.TIMEOUT_KEY, "900000"))/1000);

        String netID = request.getHeader(System.getProperty(Constants.AUTH_USER_KEY, "X-WebAuth-User"));

        // following two lines added for testing cases
        if( netID == null &&
            System.getProperty(Constants.CHEAT_USER_KEY) != null &&
            System.getProperty(Constants.CHEAT_USER_KEY).trim().length() > 0 ) {
            netID = System.getProperty(Constants.CHEAT_USER_KEY);
            if( request.getParameter("uid") != null )
                netID = request.getParameter("uid");
        }

        if (netID == null || netID.trim().length() == 0) {
            response.getWriter().println(
                "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n"+
                "<HTML>\n<HEAD>"+
                "<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=ISO-8859-1\">\n"+
                "<TITLE>Auth failed</TITLE>\n"+
                "</HEAD>\n\n"+
                "<BODY BGCOLOR=\"#FFFFFF\">\n"+
                "<BR><H2 ALIGN=\"center\"><FONT COLOR=\"#AA0000\">Authentication failed</FONT></H2>\n"+
                "</BODY>\n</HTML>"
            );
            return;
        }
        session.setAttribute("NetID", netID.trim());

        try {
            NetDBpi netdb = NetDBHook.createInstance(netID.trim(), false);
            session.setAttribute("sid", (String)(nHash(netdb.getState())).get("timestamp"));
            session.setAttribute("netdb", netdb);
            session.setAttribute("user", netdb.getCurrentUser());
            session.setAttribute("X-Forwarded-For", Utils.n2s(request.getHeader("X-Forwarded-For")));
            (new LoadData(session, netdb)).start();
            ((stanford.netdb.utils.Logger)netdb).info("http session started ["+session.getId()+"]");
            response.sendRedirect("/home");

        } catch (Exception ex) {
            response.sendRedirect("/LoginFailed?error_msg="+encode(ex.getMessage()));
            return;
        }

        session.setAttribute("master", new NetDBMaster());
    }

    class LoadData extends Thread {
        private HttpSession session;
        private NetDBpi netdb;

        public LoadData(HttpSession session, NetDBpi netdb) {
            this.session = session;
            this.netdb = netdb;
        }

        public void run() {
            try {
                session.setAttribute("departments",       netdb.getInfo(Constants.DEPARTMENT, ""));
                session.setAttribute("locations",         netdb.getInfo(Constants.LOCATION, ""));
                session.setAttribute("dhcp_options",      netdb.getInfo(Constants.DHCP_OPTIONS, ""));
                session.setAttribute("models",            netdb.getInfo(Constants.MODEL, 0));
                session.setAttribute("model_types",       netdb.getInfo(Constants.MODEL_TYPE, 0));
                session.setAttribute("oses",              netdb.getInfo(Constants.OS, 0));
                session.setAttribute("myGroups",          netdb.listMyGroups("*"));
                session.setAttribute("myAddrSpaces",      netdb.listMyAddrSpaces());
                session.setAttribute("dhcp_services",     netdb.listDHCPServices());
                session.setAttribute("domains",           netdb.listMyDomains());
                session.setAttribute("allRecords",        netdb.listRecords());
                session.setAttribute("states",            netdb.listMyStates(null));
                session.setAttribute("custom.node.views", (new CNVParser()).loadCustomNodeViewsData());
                session.setAttribute("node types",        netdb.list(Constants.NODE_TYPE, null, false));
                session.setAttribute("my node types",     netdb.list(Constants.NODE_TYPE, null, true));

                Hashtable global_defaults = netdb.getGlobalDefaults();
                session.setAttribute("global.defaults",   global_defaults);

                Integer custom_fields_total = new Integer(Constants.MAX_CUSTOM_FIELDS);
                try {
                    if( global_defaults.get("MaxNodeCustomFields") != null )
                        custom_fields_total = (Integer)global_defaults.get("MaxNodeCustomFields");
                } catch (Exception exx) { }
                session.setAttribute("custom_fields_total", custom_fields_total);

                session.setAttribute("dataReady", "");
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }
}
