/************************************************************************
 *
 * Model.java -
 *
 * $Id: Model.java,v 1.2 2006/09/08 18:01:32 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class Model extends ModifyPage {

    public Model() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("type", "Model");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        try {
            String netID = (String)session.getAttribute("NetID");
            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String rootURL = request.getServletPath();

            if( rootURL.equals("/Model") ) {
                try {
                    Hashtable modelList = netdb.getInfo(Constants.MODEL, "");
                    setData(session, "models", Utils.copyHash(modelList));
                    store.put("modelList", modelList);
                } catch(Exception exx) {
                    throw new Exception("Failed to load list of models.");
                }

            } else {
                Hashtable oldModel = new Hashtable();
                String modelName = request.getParameter("modelName");

                if( modelName == null ) {
                    oldModel.put("model", "");
                    oldModel.put("make", "");
                    oldModel.put("type", "");
                } else {
                    Hashtable modelList = netdb.getInfo(Constants.MODEL, "");
                    setData(session, "models", Utils.copyHash(modelList));
                    oldModel = (Hashtable)modelList.get(decode(modelName));
                    if( oldModel == null )
                        throw new Exception("Model '"+decode(modelName)+"' does not exist.");
                }

                oldModel.put("object", "model");

                store.put("oldModel", oldModel);
                store.put("newModel", Utils.copyHash(oldModel));

                Hashtable makes = netdb.getInfo(Constants.MAKE, "");
                store.put("makes", makes);
            }

            PrintWriter out = response.getWriter();
            printOutHtml(session, out, store, rootURL);

        } catch (Exception ex) {
            ex.printStackTrace();
            addSessionMsg(session, "Error.errorMsg", "Error: "+ex);
            response.sendRedirect(store!=null?(String)store.get("fromPage"):"/home");
        }
    } // end of doGet(...)

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        HttpSession session = begin(request, response, null);
        if( session == null )
            return;

        String rootURL = request.getServletPath();

        NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");

        String sessionKey = decode(request.getParameter("sessionKey"));
        Hashtable store = (Hashtable)session.getAttribute(sessionKey);
        Hashtable modelList = (Hashtable)store.get("modelList");

        try {
            PrintWriter out = response.getWriter();
            String action = request.getParameter("action");
            if( action == null || action.trim().length() == 0 ) {
                if( rootURL.equals("/AddModel") )
                    action = "Create Model";
                else if( rootURL.equals("/ModifyModel") )
                    action = "Modify Model";
            }

            if( rootURL.equals("/Model") && action.equals("Delete") ) {
                String[] values = request.getParameterValues("modellist1");
                if( values != null ) {
                    if( values.length == 0 )
                        addSessionMsg(session, "Error.errorMsg", "Error: Please select model you wish to delete.");
                    else if( values.length > 1 )
                        addSessionMsg(session, "Error.errorMsg", "Error: Please select only one model you wish to delete.");
                    else {
                        // delete
                        try {
                            try {
                                Hashtable model = (Hashtable)modelList.get(decode(values[0]));
                                if( model == null )
                                    throw new Exception("Selected model does not exist");
                                model.put("object", "model");
                                netdb.delete(model);
                                session.setAttribute("info_msg", "SUCCESS: Model '"+decode(values[0])+"' has been deleted");

                                modelList = netdb.getInfo(Constants.MODEL, "");
                                setData(session, "models", Utils.copyHash(modelList));
                                store.put("modelList", modelList);
                            } catch (NetDBException ex) {
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int j = 0; j < esize; j++ )
                                    msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                                throw new Exception(msg);
                            }
                        } catch (Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                        }
                    }
                } else
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select model you wish to delete.");

            } else if( rootURL.equals("/Model") && action.equals("Replace") ) {
                String[] values1 = request.getParameterValues("modellist1");
                String[] values2 = request.getParameterValues("modellist2");
                if( values1 == null || values1.length == 0 )
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select one or more models to replace.");
                else if( values2 == null || values2.length != 1 )
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select model to replace with.");
                else {
                    Hashtable model2 = (Hashtable)modelList.get(decode(values2[0]));
                    if( model2 == null )
                        throw new Exception("Selected model does not exist");
                    model2.put("object", "model");

                    boolean gotErrors = false;
                    for( int i = 0; i < values1.length; i++ ) {
                        try {
                            try {
                                Hashtable model1 = (Hashtable)modelList.get(decode(values1[i]));
                                if( model1 == null )
                                    throw new Exception("Selected model does not exist");
                                model1.put("object", "model");
                                netdb.update(model1, model2);
                            } catch (NetDBException ex) {
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int j = 0; j < esize; j++ )
                                    msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                                throw new Exception(msg);
                            }
                        } catch (Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                            gotErrors = true;
                        }
                    }

                    if( !gotErrors )
                        session.setAttribute("info_msg", "SUCCESS: Models have been replaced");
                    try {
                        modelList = netdb.getInfo(Constants.MODEL, "");
                        setData(session, "models", Utils.copyHash(modelList));
                        store.put("modelList", modelList);
                    } catch(Exception exx) {
                        throw new Exception("Failed to load list of models.");
                    }
                }

            } else if( rootURL.equals("/Model") && action.equals("Add") ) {
                response.sendRedirect("/AddModel");
                return;

            } else if( rootURL.equals("/Model") && action.equals("Modify") ) {
                String[] values1 = request.getParameterValues("modellist1");
                if( values1 == null || values1.length == 0 )
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select model you wish to modify.");
                else if( values1.length > 1 )
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select only one model to modify.");
                else {
                    response.sendRedirect("/ModifyModel?modelName="+encode(values1[0]));
                    return;
                }

            } else if( action.equals("Cancel") ) {
                response.sendRedirect("/Model");
                return;

            } else if( action.equals("Create Model") || action.equals("Modify Model") ) {
                Hashtable newModel = (Hashtable)store.get("newModel");
                newModel.put("model", ((String)Utils.n2s(request.getParameter("model"))).trim());
                newModel.put("make", (decode((String)Utils.n2s(request.getParameter("make")))).trim());
                String type = decode(request.getParameter("type")).trim();
                newModel.put("type", type.equals("--None--")?"":type);
                try {
                    try {
                        Utils.validate(Constants.MODEL, newModel, Constants.NARROW);

                        if( action.equals("Create Model") ) {
                            netdb.insert(newModel);
                            session.setAttribute("info_msg", "SUCCESS: Model has been created");
                        } else {
                            netdb.update((Hashtable)store.get("oldModel"), newModel);
                            session.setAttribute("info_msg", "SUCCESS: Model has been modified.");
                        }
                        response.sendRedirect("/Model");
                        return;

                    } catch (NetDBException ex) {
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int j = 0; j < esize; j++ )
                            msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                        throw new Exception(msg);
                    }
                } catch (Exception ex) {
                    addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                }
            }

            printOutHtml(session, out, store, rootURL);

        } catch (Exception ex) {
            ex.printStackTrace();
            addSessionMsg(session, "Error.errorMsg", "Error: "+ex);
            response.sendRedirect(store!=null?(String)store.get("fromPage"):"/home");
        }
    } // end of doPost(...)

    public void printOutHtml(HttpSession session, PrintWriter out, Hashtable store, String rootURL) throws Exception {
        String sessionKey = (String)store.get("sessionKey");
        out.println(commonHtmlBegin(rootURL.equals("/Model")?"Manage Models":(rootURL.equals("/ModifyModel")?"Modify Model":"Create Model"))+
                    commonHeader(session, "/Dispatch?Action=redirect&to=/home",
                                 SHOW_HOME_LINK, SHOW_PROFILE_LINK, SHOW_LOGOUT_LINK,
                                 "", rootURL.equals("/Model")?"Manage Models":(rootURL.equals("/ModifyModel")?"Modify Model":"Create Model"),
                                 "hlpnetdb.html", decode((String)store.get("newFromPage")))+
                    msgPanels(session)+
                    "<DIV id=\"infoPage\">\n"+
                    "<FORM METHOD=\"POST\" NAME=\"Form\" ACTION='"+rootURL+"'>"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"sessionKey\" VALUE=\""+encode(sessionKey)+"\">"+
                    "<DIV class='modHeader'>\n"+
                    "<TABLE WIDTH='100%'>\n"+
                    "<TR><TD NOWRAP>"+
                    (rootURL.equals("/Model")?"Select Model(s) to repoint from the left. Select the Model to point to, from the right.":
                     (rootURL.equals("/ModifyModel")?"Modify an existing Model.":"Create new Model."))+
                    "</TD></TR></TABLE></DIV>"+
                    "<TABLE WIDTH='100%'>");

        if( rootURL.equals("/Model") ) {
            Hashtable modelList = (Hashtable)store.get("modelList");
            out.println("<TR><TD WIDTH='1%'>&nbsp;<BR><SELECT ID='Selector_Models' SIZE=30 NAME='modellist1' "+
                        "ondblclick=\"document.Form.Button_Modify.click()\" "+
                        "onKeyDown=\"if( event.keyCode == 13 ) document.Form.Button_Modify.click();\" MULTIPLE>");
            java.util.List list = Arrays.asList(modelList.keySet().toArray());
            Collections.sort(list, name_cmp);
            Iterator it = list.iterator();
            while( it.hasNext() ) {
                String model = ((String)it.next()).trim();
                out.println("<OPTION VALUE='"+encode(model)+"'>"+Utils.htmlize(model, false));
            }

            out.println("</SELECT><BR><BR><INPUT ID='Button_Add' TYPE='SUBMIT' NAME='action' VALUE='Add'> "+
                        "<INPUT ID='Button_Modify' TYPE='SUBMIT' NAME='action' VALUE='Modify'> "+
                        "<INPUT ID='Button_Delete' TYPE='SUBMIT' NAME='action' VALUE='Delete'></TD>");

            out.println("<TD WIDTH='1%'> Becomes </TD><TD>&nbsp;<BR><SELECT ID='Selector_Replacement' SIZE=30 NAME='modellist2'>");
            list = Arrays.asList(modelList.keySet().toArray());
            Collections.sort(list, name_cmp);
            it = list.iterator();
            while( it.hasNext() ) {
                String model = ((String)it.next()).trim();
                out.println("<OPTION VALUE='"+encode(model)+"'>"+Utils.htmlize(model, false));
            }

            out.println("</SELECT><BR><BR><INPUT ID='Button_Replace' TYPE='SUBMIT' NAME='action' VALUE='Replace'></TD></TR>"+
                        "</TABLE>\n"+
                        "<HR NOSHADE SIZE=2>\n"+
                        "</FORM>\n<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");

        } else { // adding or modifying model
            Hashtable model_types = nHash(getData(session, "model_types"));
            Hashtable newModel = (Hashtable)store.get("newModel");
            String model_type = ((String)Utils.n2s(newModel.get("type"))).trim();
            out.println("<TR><TD VALIGN='TOP' WIDTH='2%'>&nbsp;<font class='star'>*</font> Model</TD>"+
                        "<TD><INPUT ID='TextField_Model' TYPE='TEXT' SIZE='21' NAME='model' value=\""+
                        Utils.htmlize((String)Utils.n2s(newModel.get("model")), false)+
                        "\"></TD></TR>\n"+
                        "<TR><TD VALIGN='TOP' WIDTH='2%'>&nbsp;Type</TD>\n<TD>"+
                        "<SELECT ID='Selector_Type' SIZE=1 NAME='type'>\n"+
                        "<OPTION VALUE='"+encode("--None--")+"'>--None--");

            java.util.List list = Arrays.asList(model_types.keySet().toArray());
            Collections.sort(list, name_cmp);
            Iterator it = list.iterator();
            while( it.hasNext() ) {
                String type = (String)it.next();
                out.println("<OPTION VALUE='"+encode(type)+"'"+(model_type.equals(type)?" SELECTED":"")+">"+type);
            }

            out.println("</SELECT></TD></TR>\n"+
                        "<TR><TD VALIGN='TOP' WIDTH='2%'>&nbsp;<font class='star'>*</font> Make</TD><TD>"+
                        "<SELECT ID='Selector_Make' SIZE=16 NAME='make'>");

            String selectedMake = (String)Utils.n2s(newModel.get("make"));
            Hashtable makes = nHash(store.get("makes"));
            list = Arrays.asList(makes.keySet().toArray());
            Collections.sort(list, name_cmp);
            it = list.iterator();
            while( it.hasNext() ) {
                String make = ((String)it.next()).trim();
                out.println("<OPTION VALUE='"+encode(make)+"'"+(selectedMake.equals(make)?" SELECTED":"")+">"+
                            Utils.htmlize(make, false));
            }

            out.println("</SELECT>\n"+
                        "</TD></TR>\n"+
                        "<TR COLSPAN='2'><TD NOWRAP><INPUT ID='Button_Create_Or_Modify' TYPE='SUBMIT' NAME='action' VALUE='"+
                        (rootURL.equals("/AddModel")?"Create Model":"Modify Model")+
                        "'> <INPUT ID='Button_Cancel' TYPE='SUBMIT' NAME='action' VALUE='Cancel'></TD></TR>"+
                        "</TABLE>\n"+
                        "<HR NOSHADE SIZE=2>\n"+
                        "</FORM>\n<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");
        }
    } // end of printOutHtml(...)
}
