/************************************************************************
 *
 * ModifyAdminTeam.java -
 *
 * $Id: ModifyAdminTeam.java,v 1.4 2006/11/16 23:20:38 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2004, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.sql.*;
import java.net.*;
import java.util.*;
import java.text.*;  // SimpleDateFormat
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import su.netdb.parser.*;
import stanford.netdb.utils.*;

public class ModifyAdminTeam extends ModifyPage {

    public ModifyAdminTeam() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("type", "AdminTeam");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        try {
            PrintWriter out = response.getWriter();
            String rootURL = request.getServletPath();

            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String netID = (String)session.getAttribute("NetID");

            // obtain key adminTeam attributes from request
            String object = request.getParameter("handle");
            String handle = null;
            Timestamp lockTm = null;
            if( object != null ) {
                handle = new String(object);
                store.put("handle", handle);
                if( rootURL.equals("/ModifyAdminTeam") ) {
                    // lock
                    try {
                        lockTm = netdb.lock(Constants.ATEAM, handle);
                        if( lockTm == null )
                            throw new Exception("Failed to lock admin team "+handle);

                    } catch (Exception ex) {
                        ex.printStackTrace();
                        session.setAttribute("Error.errorMsg", ex.getMessage());
                        response.sendRedirect((String)store.get("fromPage"));
                        return;
                    }
                    store.put("lockTm", lockTm);
                }
            }

            Hashtable oldAdminTeam = null;
            if( rootURL.equals("/CreateAdminTeam") ) {

                if( !netdb.checkAccess(Constants.ATEAM) )
                    throw new Exception("User '"+session.getAttribute("NetID")+"' can not create admin teams.");

                if( handle == null ) { // creating clean ateam
                    oldAdminTeam = new Hashtable();
                    oldAdminTeam.put("id", "");
                    oldAdminTeam.put("object", "ateam");
                    oldAdminTeam.put("name", "");
                    oldAdminTeam.put("email", "");
                    oldAdminTeam.put("phone", "");

                    oldAdminTeam.put("groups", new Hashtable());
                    oldAdminTeam.put("departments", new Hashtable());
                    oldAdminTeam.put("members", new Hashtable());

                    oldAdminTeam.put("comment", "");
                } else { // creating ateam from template
                    Hashtable tmpOldATeam = netdb.getInfo(Constants.ATEAM, object);
                    if ( tmpOldATeam == null || tmpOldATeam.get("id") == null ) {
                        session.setAttribute("Error.errorMsg",
                                             "Error: No info found for admin team "+object);
                        response.sendRedirect((String)store.get("fromPage"));
                        return;
                    }
                    handle = null;
                    // we inherit only few parts of original AdminTeam
                    oldAdminTeam = new Hashtable();
                    oldAdminTeam.put("id", "");
                    oldAdminTeam.put("object", "ateam");
                    oldAdminTeam.put("name", "");
                    oldAdminTeam.put("email", "");
                    oldAdminTeam.put("phone", "");
                    oldAdminTeam.put("comment", "");

                    oldAdminTeam.put("groups", nHash(tmpOldATeam.get("groups")));
                    oldAdminTeam.put("departments", nHash(tmpOldATeam.get("departments")));
                    oldAdminTeam.put("members", nHash(tmpOldATeam.get("members")));
                }

            } else { // Admin Team modify
                oldAdminTeam = netdb.getInfo(Constants.ATEAM, object);
                if (oldAdminTeam == null || oldAdminTeam.get("id") == null ) {
                    session.setAttribute("Error.errorMsg",
                                         "Error: No info found for admin team "+object);
                    response.sendRedirect((String)store.get("fromPage"));
                    return;
                }
            }

            if( lockTm != null )
                oldAdminTeam.put("locked", lockTm);

            Hashtable newAdminTeam = Utils.copyHash(oldAdminTeam);
            store.put("oldAdminTeam", oldAdminTeam);
            store.put("newAdminTeam", newAdminTeam);
            store.put("oldNewAdminTeam", Utils.copyHash(newAdminTeam));

            if( object != null )
                store.put("object", object);

            printOutModifyHTML(session, out, store, rootURL, "0", "0");

        } catch (Exception ex) {
            // display error page
            ex.printStackTrace();
            String errMsg = ex.getMessage();
            session.setAttribute("Error.errorMsg", errMsg == null ? "???" : errMsg);
            response.sendRedirect((String)store.get("fromPage"));
            return;
        }
    } // end of doGet(...)

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        HttpSession session = begin(request, response, null);
        if( session == null )
            return;

        PrintWriter out = response.getWriter();
        String rootURL = request.getServletPath();

        String sessionKey = decode(request.getParameter("sessionKey"));
        String default_domain = (String)session.getAttribute("default_domain");
        Hashtable store = (Hashtable)session.getAttribute(sessionKey);
        NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
        String handle = (String)store.get("handle");
        Timestamp lockTm = (Timestamp)store.get("lockTm");
        Hashtable newAdminTeam = (Hashtable)store.get("newAdminTeam");
        Hashtable oldAdminTeam = (Hashtable)store.get("oldAdminTeam");
        Hashtable oldNewAdminTeam = (Hashtable)store.get("oldNewAdminTeam");

        String action = (String)Utils.n2s((String)request.getParameter("Action"));

        if( request.getParameter("verify_ateam_members") != null )
            action = "verify_ateam_members";

        try {
            if( !action.equals("Reset") && !action.equals("Cancel") ) {
                // pick up name
                newAdminTeam.put("name",
                              ((String)Utils.n2s((String)request.getParameter("name"))).trim());

                // pick up email
                newAdminTeam.put("email",
                              ((String)Utils.n2s((String)request.getParameter("email"))).trim());

                // pick up phone
                newAdminTeam.put("phone",
                              ((String)Utils.n2s((String)request.getParameter("phone"))).trim());

                // pick up groups
                Hashtable groups = new Hashtable();
                newAdminTeam.put("groups", groups);

                String[] newGroups = request.getParameterValues("groups");
                if( newGroups != null ) {
                    for( int i = 0; i < newGroups.length; i++ )
                        groups.put(decode(newGroups[i]), "");
                }

                // pick up departments
                Hashtable departments = new Hashtable();
                newAdminTeam.put("departments", departments);

                String[] newDeps = request.getParameterValues("departments");
                if( newDeps != null ) {
                    for( int i = 0; i < newDeps.length; i++ )
                        departments.put(decode(newDeps[i]), "");
                }

                // pick up members
                // fill up all members with "not selected" flag
                for( Hashtable member : (Collection<Hashtable>)nHash(newAdminTeam.get("members")).values() )
                        member.put("not selected", "");

                // pick up checked members
                String[] values = request.getParameterValues("member");
                if( values != null ) {
                    for( int i = 0; i < values.length; i++ ) {
                        String key = decode(values[i]);
                        nHash(nHash(newAdminTeam.get("members")).get(key))
                            .remove("not selected");
                    }
                }

                // pick up all stale members, i.e. those with "not selected" flag
                Vector keysToRemove = new Vector();
                Hashtable members = nHash(newAdminTeam.get("members"));
                for( String key : (Set<String>)members.keySet() )
                    if( ((Hashtable)members.get(key)).get("not selected") != null )
                        keysToRemove.addElement(key);

                // now remove them
                for( Object key : keysToRemove )
                    members.remove(key);

                // pick up comment
                newAdminTeam.put("comment",
                              ((String)Utils.n2s((String)request.getParameter("comment"))).trim());

                String newAdminTeamName = ((String)Utils.n2s(newAdminTeam.get("name"))).trim();
                String oldAdminTeamName = ((String)Utils.n2s(oldNewAdminTeam.get("name"))).trim();

                if( oldAdminTeamName.length() > 0 && newAdminTeamName.length() > 0 &&
                    !newAdminTeamName.toLowerCase().equals(oldAdminTeamName.toLowerCase()) )
                    newAdminTeam.remove("name_error");
            } // end of 'if( !action.equals("Reset") && !action.equals("Cancel") )'

            if( !action.equals("verify_ateam_members")  )
                newAdminTeam.put("members raw data", request.getParameter("members"));

            if( action.equals("verify_ateam_members") ||
                action.equals("Save") ||
                action.equals("Verify") ) {

                newAdminTeam.remove("members raw data");
                String usersStr = request.getParameter("members");
                String newUsersStr = "";
                String[] split = usersStr.split("\\n");
                if( split != null ) {
                    boolean multiple_matches = false;
                    boolean verified = false;
                    boolean no_match = false;
                    for( int i = 0; i < split.length; i++ ) {
                        String userStr = split[i].trim();
                        if( userStr.length() == 0 )
                            continue;
                        Hashtable foundPeople;
                        try {
                            try {
                                boolean got_match = false;
                                if( userStr.endsWith("@") )
                                    foundPeople =
                                        netdb.dirSearch(Constants.NETID,
                                                        userStr.substring(0, userStr.length()-1));
                                else
                                    foundPeople = netdb.dirSearch(Constants.LOOKUP, userStr);

                                // fill them in
                                Hashtable users = (Hashtable)newAdminTeam.get("members");
                                if( users == null ) {
                                    users = new Hashtable();
                                    newAdminTeam.put("members", users);
                                }
                                if( foundPeople == null )
                                    no_match = true;
                                else if( foundPeople.size() == 0 )
                                    no_match = true;
                                else {
                                    verified = true;
                                    got_match = true;
                                }

                                if( foundPeople.size() > 1 )
                                    multiple_matches = true;

                                if( !got_match )
                                    newUsersStr += (newUsersStr.length()>0?"\n":"")+
                                        userStr;

                                fillInPeople(users, foundPeople);

                            } catch (NetDBException ex) {
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int j = 0; j < esize; j++ )
                                    msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                                throw new Exception(msg);
                            }

                        } catch (Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                        }
                    }

                    if( multiple_matches )
                        addSessionMsg(session, "Error.errorMsg", "ERROR: Multiple matches for admin team members.");
                    else {
                        if( no_match ) {
                            addSessionMsg(session, "Error.errorMsg", "ERROR: No matches for admin team members.");
                            newAdminTeam.put("members raw data", newUsersStr); // save raw data

                        } else if( verified )
                            addSessionMsg(session, "info_msg", "SUCCESS: Admin team members verified.");
                    }
                }
            } // end of 'if( action.equals("verify_ateam_members") ... )'

            if( action.equals("Verify") || action.equals("Save") ) {
                try {
                    Utils.validate(Constants.ATEAM, newAdminTeam, Constants.NARROW);

                    try {
                        String newAdminTeamName = ((String)Utils.n2s(newAdminTeam.get("name"))).trim();
                        String oldAdminTeamName = ((String)Utils.n2s(oldNewAdminTeam.get("name"))).trim();
                        String oldOldAdminTeamName = ((String)Utils.n2s(oldAdminTeam.get("name"))).trim();
                        if( !oldAdminTeamName.toLowerCase().equals(newAdminTeamName.toLowerCase()) &&
                            !oldOldAdminTeamName.toLowerCase().equals(newAdminTeamName.toLowerCase()) ) {
                            Hashtable adminTeamH = nHash(netdb.getInfo(Constants.ATEAM, newAdminTeamName));
                            if( adminTeamH.get("id") != null ) {
                                addSessionMsg(session, "Error.errorMsg", "Error: Admin Team with this name already exists");
                                newAdminTeam.put("name_error", "");
                            }
                        }

                    } catch (NetDBException exx) {
                        Vector messages = exx.getMessages();
                        if( messages != null && messages.size() > 0 ) {
                            for( int i = 0; i < messages.size(); i++ )
                                addSessionMsg(session, "Error.errorMsg", "Error: "+messages.elementAt(i));
                        }
                        newAdminTeam.put("name_error", "");

                    } catch (Exception exx) {
                        addSessionMsg(session, "Error.errorMsg", "Error: "+exx.getMessage());
                        newAdminTeam.put("name_error", "");
                    }

                } catch (NetDBException ex) {
                    Vector messages = ex.getMessages();
                    if( messages != null && messages.size() > 0 ) {
                        for( int i = 0; i < messages.size(); i++ )
                            addSessionMsg(session, "Error.errorMsg", "Error: "+messages.elementAt(i));
                    }
                } catch (Exception ex) {
                    addSessionMsg(session, "Error.errorMsg", "Error: "+ex.getMessage());
                }
            } // end of 'if( action.equals("Verify") || action.equals("Save") )'

            if( action.equals("Cancel") ) {
                try {
                    try {
                        if( lockTm != null )
                            netdb.unlock(Constants.ATEAM, handle, lockTm);

                    } catch (NetDBException ex) {
                        /*
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int i = 0; i < esize; i++ )
                            msg += "Error: "+(String)errors.elementAt(i)+(i<esize-1?"\n":"");
                        throw new Exception(msg);
                        */
                    }

                } catch (Exception ex) {
                    session.setAttribute("Error.errorMsg", ex.getMessage());
                }
                response.sendRedirect((String)store.get("fromPage"));
                return;

            } else if( action.equals("Reset") ) {
                newAdminTeam = Utils.copyHash((Hashtable)store.get("oldAdminTeam"));
                store.put("newAdminTeam", newAdminTeam);
                store.put("oldNewAdminTeam", Utils.copyHash(newAdminTeam));
            }

            if( action.equals("Save") &&
                session.getAttribute("Error.errorMsg") == null ) {
                session.removeAttribute("info_msg");
                session.removeAttribute("Error.errorMsg");

                try {
                    try {
                        handle = ((String)Utils.n2s(newAdminTeam.get("name"))).trim();

                        updatePersons(netdb, nHash(newAdminTeam.get("members")));

                        if( rootURL.equals("/ModifyAdminTeam") ) {
                            netdb.update(oldAdminTeam, newAdminTeam);
                            session.setAttribute("info_msg", "SUCCESS: Admin Team '"+handle+"' has been updated.");
                        } else {
                            netdb.insert(newAdminTeam);
                            session.setAttribute("info_msg", "SUCCESS: Admin Team '"+handle+"' has been created.");
                        }

                        String fromFromPage = (String)store.get("fromFromPage");
                        response.sendRedirect("/Dispatch?Action=info&object=AdminTeam&fromPage="+
                                              encode(fromFromPage!=null?fromFromPage:(String)store.get("fromPage"))+
                                              "&handle="+encode(handle));
                        return;

                    } catch (NetDBException ex) {
                        ex.printStackTrace();
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int j = 0; j < esize; j++ )
                            msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                        throw new Exception(msg);
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                    addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                }
            } // end of 'if( action.equals("Save") ... )'

            printOutModifyHTML(session, out, store, rootURL,
                               request.getParameter("PosX"),
                               request.getParameter("PosY"));

        } catch (Exception ex) {
            ex.printStackTrace();
            session.setAttribute("Error.errorMsg", ex.getMessage());
            response.sendRedirect((String)store.get("fromPage"));
        }
    } // end of doPost(...)

    public void printOutModifyHTML(HttpSession session, PrintWriter out, Hashtable store,
                                   String rootURL, String pos_x, String pos_y)
        throws Exception
    {
        String default_domain = (String)session.getAttribute("default_domain");

        String sessionKey = (String)store.get("sessionKey");
        Timestamp lockTm = (Timestamp)store.get("lockTm");
        Hashtable newAdminTeam = (Hashtable)store.get("newAdminTeam");
        Hashtable oldAdminTeam = (Hashtable)store.get("oldAdminTeam");
        String handle = (String)store.get("handle");
        String object = (String)store.get("object");

        if( Utils.nStr(session.getAttribute("Error.errorMsg")).trim().length() > 0  ) pos_x = pos_y = "0";

        store.put("PosX", pos_x);
        store.put("PosY", pos_y);

        modTop(session, out, object, handle, "AdminTeam", "Admin Team", "hlpateam.html", sessionKey, rootURL);

        // ateam name
        String name = ((String)Utils.n2s(newAdminTeam.get("name"))).trim();
        out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font> Name</TD>\n"+
                    "<TD VALIGN='TOP'><INPUT ID='TextField_Name' "+
                    (newAdminTeam.get("name_error")!=null?"CLASS='error_form_font' ":
                     (newAdminTeam.get("name_reserved")!=null?"CLASS='reserved_form_font' ":""))+
                    "TYPE='TEXT' SIZE='21' NAME='name' value=\""+Utils.htmlize(name)+"\">"+
                    "</TD></TR>\n");

        // email
        String email = ((String)Utils.n2s(newAdminTeam.get("email"))).trim();
        out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font> Email</TD>\n"+
                    "<TD VALIGN='TOP'><INPUT ID='TextField_Email' "+
                    (newAdminTeam.get("email_error")!=null?"CLASS='error_form_font' ":"")+
                    "TYPE='TEXT' SIZE='21' NAME='email' value=\""+Utils.htmlize(email)+"\">"+
                    "</TD></TR>\n");

        // phone
        String phone = ((String)Utils.n2s(newAdminTeam.get("phone"))).trim();
        out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font> Phone</TD>\n"+
                    "<TD VALIGN='TOP'><INPUT ID='TextField_Phone' "+
                    (newAdminTeam.get("phone_error")!=null?"CLASS='phone_form_font' ":"")+
                    "TYPE='TEXT' SIZE='21' NAME='phone' value=\""+Utils.htmlize(phone)+"\">"+
                    "</TD></TR>\n"+
                    "<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // NetDB Groups
        Hashtable groups = (Hashtable)newAdminTeam.get("groups");
        if( groups == null ) { // I know that adminTeam always have groups, but just in case
            groups = new Hashtable();
            newAdminTeam.put("groups", groups);
        }
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font>&nbsp;"+
                    "Groups<br>&nbsp;(1 or more)</TD>\n"+
                    "<TD VALIGN='TOP'>Selected (<em>uncheck to remove</em>):<BR>");
        java.util.List list = Arrays.asList(groups.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();
        int index = 0;
        while( it.hasNext() ) {
            index++;
            String group = (String)it.next();
            out.println("<INPUT ID='CheckBox_Group_"+index+"' TYPE='checkbox' CHECKED NAME='groups' value='"+
                        encode(group)+"'>"+Utils.htmlize(group.trim(), false)+"<BR>");
        }
        if( groups.size() > 0 )
            out.println("<BR>");
        out.println("<SELECT ID='Selector_Groups' NAME='groups' MULTIPLE SIZE='6'>");
        Hashtable myGroups = (Hashtable)getData(session, "myGroups");
        list = Arrays.asList(myGroups.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        while( it.hasNext() ) {
            String group = (String)it.next();
            out.println("<OPTION VALUE='"+encode(group)+"'>"+Utils.htmlize(group.trim(), false)+"</OPTION>");
        }
        out.println("</SELECT></TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // departments
        Hashtable depts = (Hashtable)newAdminTeam.get("departments");
        if( depts == null ) {
            depts = new Hashtable();
            newAdminTeam.put("departments", depts);
        }
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Departments</TD>\n"+
                    "<TD VALIGN='TOP'>Selected (<em>uncheck to remove</em>):<BR>");
        list = Arrays.asList(depts.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        index = 0;
        while( it.hasNext() ) {
            index++;
            String dep = (String)it.next();
            out.println("<INPUT ID='CheckBox_Department_"+index+"' TYPE='checkbox' "+
                        "CHECKED NAME='departments' value='"+encode(dep)+"'>"+
                        Utils.htmlize(dep.trim(), false)+"<BR>");
        }
        if( depts.size() > 0 )
            out.println("<BR>");
        out.println("<SELECT ID='Selector_Departments' NAME='departments' MULTIPLE SIZE='6'>");
        Hashtable departments = (Hashtable)getData(session, "departments");
        list = Arrays.asList(departments.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        while( it.hasNext() ) {
            String dep = ((String)it.next()).trim();
            out.println("<OPTION VALUE='"+encode(dep)+"'>"+Utils.htmlize(dep, false)+"</OPTION>");
        }
        out.println("</SELECT>\n</TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // members
        Hashtable members = nHash(newAdminTeam.get("members"));
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Members</TD>\n"+
                    "<TD VALIGN='TOP'>");
        list = Arrays.asList(members.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        index = 0;
        while( it.hasNext() ) {
            index++;
            String key = (String)it.next();
            Hashtable person = (Hashtable)members.get(key);
            String dirID = (String)person.get("DirID");
            String regID = (String)person.get("RegID");
            String dep = (String)person.get("department");

            String displayName = (String)person.get("name");
            if( displayName == null || displayName.trim().length() == 0 )
                displayName = (String)person.get("NetID");
            if( displayName == null || displayName.trim().length() == 0 )
                displayName = (String)person.get("DirID");
            if( displayName == null || displayName.trim().length() == 0 )
                displayName = regID;

            boolean active = activep(person);

            out.println("<INPUT ID='CheckBox_Member_"+index+"' TYPE='checkbox' "+
                        (person.get("not selected")==null?"CHECKED ":"")+
                        "NAME='member' value='"+
                        encode(key)+"'>"+
                        (active?"<A HREF=\""+
                         System.getProperty("stanfordwho.url")+dirID.trim()+"\">":"")+
                        displayName+(active?"</A>":"")+
                        ((dep!=null&&dep.trim().length()>0)?" <strong>"+dep+"</strong>":"")+
                        (!active?" ( <em>inactive</em>)":"")+"<BR>");
        }

        out.println("(<em>List name or SUNet ID, one per line. Append @ if searching only "+
                    "by sunet ID, as in sunetid@</em>)<BR>"+
                    "<TEXTAREA ID='TextArea_Members' NAME='members' ROWS='5' COLS='21'>"+
                    (newAdminTeam.get("members raw data")!=null?Utils.htmlize((String)newAdminTeam.get("members raw data"), false):"")+
                    "</TEXTAREA>"+
                    "&nbsp;<INPUT ID='Button_Verify_Members' TYPE='SUBMIT' NAME='verify_ateam_members' VALUE='Verify'>"+
                    "</TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // comment
        String comment = (String)Utils.n2s((String)newAdminTeam.get("comment"));
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Comment</TD>\n"+
                    "<TD VALIGN='TOP'>"+
                    "<TEXTAREA ID='TextArea_Comment' NAME='comment' ROWS='5' COLS='60'>"+
                    Utils.htmlize(comment.trim(), false)+"</TEXTAREA>"+
                    "</TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        Hashtable person = (Hashtable)newAdminTeam.get("created.by");
        if( person != null ) {
            out.println("<TR><TD COLSPAN='2'><HR NOSHADE SIZE=1></TD></TR>");

            String dirID = (String)person.get("DirID");
            boolean active = person.get("DirID")==null?false:true;

            String byStr = (active?"<A HREF=\""+
                            System.getProperty("stanfordwho.url")+dirID.trim()+"\">":"");
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += (active?"</A>":"")+" ("+newAdminTeam.get("created").toString().substring(0, 16)+")";
            out.println("<TR><TD VALIGN='TOP'>&nbsp;Created By</TD>\n"+
                        "<TD VALIGN='TOP'>"+
                        byStr+"</TD></TR>");

            // modified by
            person = (Hashtable)newAdminTeam.get("mod.by");
            dirID = (String)person.get("DirID");
            active = activep(person);

            byStr = (active?"<A HREF=\""+
                     System.getProperty("stanfordwho.url")+dirID.trim()+"\">":"");
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += (active?"</A>":"")+" ("+newAdminTeam.get("modified").toString().substring(0, 16)+")";
            out.println("<TR><TD VALIGN='TOP'>&nbsp;Last Modified By</TD>\n"+
                        "<TD VALIGN='TOP'>"+
                        byStr+"</TD></TR>");
        }

        modBottom(out);
    } // end of printOutModifyHTML(...)
}
