/************************************************************************
 *
 * ModifyGroup.java -
 *
 * $Id: ModifyGroup.java,v 1.1.1.1 2006/06/30 23:38:11 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2004, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.sql.*;
import java.net.*;
import java.util.*;
import java.text.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import su.netdb.parser.*;
import stanford.netdb.utils.*;

public class ModifyGroup extends ModifyPage {

    public ModifyGroup() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("type", "");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        try {
            PrintWriter out = response.getWriter();
            String rootURL = request.getServletPath();

            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String netID = (String)session.getAttribute("NetID");

            // obtain key group attributes from request
            String object = request.getParameter("handle");
            String handle = null;
            Timestamp lockTm = null;
            if( object != null ) {
                handle = new String(object);
                store.put("handle", handle);
                if( rootURL.equals("/ModifyGroup") ) {
                    // lock
                    try {
                        lockTm = netdb.lock(Constants.GROUP, handle);
                        if( lockTm == null )
                            throw new Exception("Failed to lock group "+handle);

                    } catch (Exception ex) {
                        ex.printStackTrace();
                        session.setAttribute("Error.errorMsg", ex.getMessage());
                        response.sendRedirect((String)store.get("fromPage"));
                        return;
                    }
                    store.put("lockTm", lockTm);
                }
            }

            Hashtable oldGroup = null;
            if( rootURL.equals("/CreateGroup") ) {

                if( !netdb.checkAccess(Constants.GROUP) )
                    throw new Exception("User '"+session.getAttribute("NetID")+"' can not create groups.");

                oldGroup = new Hashtable();
                oldGroup.put("id", "");
                oldGroup.put("object", "group");
                oldGroup.put("name", "");
                oldGroup.put("comment", "");

            } else { // group modify
                oldGroup = netdb.getInfo(Constants.GROUP, object);
                if (oldGroup == null || oldGroup.get("id") == null ) {
                    session.setAttribute("Error.errorMsg",
                                         "Error: No info found for group "+object);
                    response.sendRedirect((String)store.get("fromPage"));
                    return;
                }
            }

            if( lockTm != null )
                oldGroup.put("locked", lockTm);

            Hashtable newGroup = Utils.copyHash(oldGroup);
            store.put("oldGroup", oldGroup);
            store.put("newGroup", newGroup);
            store.put("oldNewGroup", Utils.copyHash(newGroup));

            if( object != null )
                store.put("object", object);

            printOutModifyHTML(session, out, store, rootURL);

        } catch (Exception ex) {
            // display error page
            ex.printStackTrace();
            String errMsg = ex.getMessage();
            errMsg = (errMsg == null ? "???" : errMsg);
            session.setAttribute("Error.errorMsg", errMsg);
            response.sendRedirect((String)store.get("fromPage"));
            return;
        }
    } // end of doGet(...)

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        HttpSession session = begin(request, response, null);
        if( session == null )
            return;

        PrintWriter out = response.getWriter();
        String rootURL = request.getServletPath();

        String sessionKey = decode(request.getParameter("sessionKey"));
        Hashtable store = (Hashtable)session.getAttribute(sessionKey);
        NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
        String handle = (String)store.get("handle");
        Timestamp lockTm = (Timestamp)store.get("lockTm");
        Hashtable newGroup = (Hashtable)store.get("newGroup");
        Hashtable oldGroup = (Hashtable)store.get("oldGroup");
        Hashtable oldNewGroup = (Hashtable)store.get("oldNewGroup");

        String action = (String)Utils.n2s((String)request.getParameter("Action"));

        try {
            if( !action.equals("Reset") && !action.equals("Cancel") ) {
                // pick up group name
                newGroup.put("name",
                              ((String)Utils.n2s((String)request.getParameter("group"))).trim());

                // pick up comment
                newGroup.put("comment",
                               ((String)Utils.n2s((String)request.getParameter("comment"))).trim());

                String newGroupName = ((String)Utils.n2s(newGroup.get("name"))).trim();
                String oldGroupName = ((String)Utils.n2s(oldNewGroup.get("name"))).trim();

                if( oldGroupName.length() > 0 && newGroupName.length() > 0 &&
                    !newGroupName.toLowerCase().equals(oldGroupName.toLowerCase()) )
                    newGroup.remove("group_name_error");
            } // end of 'if( !action.equals("Reset") && !action.equals("Cancel") )'

            if( action.equals("Verify") || action.equals("Save") ) {
                try {
                    Utils.validate(Constants.GROUP, newGroup, Constants.NARROW);

                    try {
                        String newGroupName = ((String)Utils.n2s(newGroup.get("name"))).trim();
                        String oldGroupName = ((String)Utils.n2s(oldNewGroup.get("name"))).trim();
                        String oldOldGroupName = ((String)Utils.n2s(oldGroup.get("name"))).trim();
                        if( !oldGroupName.toLowerCase().equals(newGroupName.toLowerCase()) &&
                            !oldOldGroupName.toLowerCase().equals(newGroupName.toLowerCase()) ) {
                            Hashtable groupH = nHash(netdb.getInfo(Constants.GROUP, newGroupName));
                            if( groupH.get("id") != null ) {
                                addSessionMsg(session, "Error.errorMsg", "Error: Group with this name already exists");
                                newGroup.put("group_name_error", "");
                            }
                        }

                    } catch (NetDBException exx) {
                        Vector messages = exx.getMessages();
                        if( messages != null && messages.size() > 0 ) {
                            for( int i = 0; i < messages.size(); i++ )
                                addSessionMsg(session, "Error.errorMsg", "Error: "+messages.elementAt(i));
                        }
                        newGroup.put("group_name_error", "");

                    } catch (Exception exx) {
                        addSessionMsg(session, "Error.errorMsg", "Error: "+exx.getMessage());
                        newGroup.put("group_name_error", "");
                    }

                } catch (NetDBException ex) {
                    Vector messages = ex.getMessages();
                    if( messages != null && messages.size() > 0 ) {
                        for( int i = 0; i < messages.size(); i++ )
                            addSessionMsg(session, "Error.errorMsg", "Error: "+messages.elementAt(i));
                    }
                } catch (Exception ex) {
                    addSessionMsg(session, "Error.errorMsg", "Error: "+ex.getMessage());
                }
            } // end of 'if( action.equals("Verify") || action.equals("Save") )'

            if( action.equals("Cancel") ) {
                try {
                    try {
                        if( lockTm != null )
                            netdb.unlock(Constants.GROUP, handle, lockTm);

                    } catch (NetDBException ex) {
                        /*
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int i = 0; i < esize; i++ )
                            msg += "Error: "+(String)errors.elementAt(i)+(i<esize-1?"\n":"");
                        throw new Exception(msg);
                        */
                    }
                } catch (Exception ex) {
                    session.setAttribute("Error.errorMsg", ex.getMessage());
                }
                response.sendRedirect((String)store.get("fromPage"));
                return;

            } else if( action.equals("Reset") ) {
                newGroup = Utils.copyHash((Hashtable)store.get("oldGroup"));
                store.put("newGroup", newGroup);
                store.put("oldNewGroup", Utils.copyHash(newGroup));
            }

            if( action.equals("Save") &&
                session.getAttribute("Error.errorMsg") == null &&
                session.getAttribute("Warning") == null ) {
                session.removeAttribute("info_msg");
                session.removeAttribute("Error.errorMsg");
                session.removeAttribute("Warning");
                try {
                    try {
                        handle = ((String)Utils.n2s(newGroup.get("name"))).trim();

                        if( rootURL.equals("/ModifyGroup") ) {
                            netdb.update(oldGroup, newGroup);
                            session.setAttribute("info_msg", "SUCCESS: Group '"+handle.replace('"', '\'')+"' has been updated.");
                        } else {
                            netdb.insert(newGroup);
                            session.setAttribute("info_msg", "SUCCESS: Group '"+handle.replace('"', '\'')+"' has been created.");
                        }

                        // reload global list of groups
                        setData(session, "myGroups", netdb.listMyGroups("*"));

                        String fromFromPage = (String)store.get("fromFromPage");
                        response.sendRedirect("/Dispatch?Action=info&object=Group&fromPage="+
                                              encode(fromFromPage!=null?fromFromPage:(String)store.get("fromPage"))+
                                              "&handle="+encode(handle));
                        return;

                    } catch (NetDBException ex) {
                        ex.printStackTrace();
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int j = 0; j < esize; j++ )
                            msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                        throw new Exception(msg);
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                    addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                }
            }

            printOutModifyHTML(session, out, store, rootURL);
        } catch (Exception ex) {
            ex.printStackTrace();
            session.setAttribute("Error.errorMsg", ex.getMessage());
            response.sendRedirect((String)store.get("fromPage"));
        }
    }

    public void printOutModifyHTML(HttpSession session, PrintWriter out, Hashtable store, String rootURL)
        throws Exception
    {
        String sessionKey = (String)store.get("sessionKey");
        Timestamp lockTm = (Timestamp)store.get("lockTm");
        Hashtable newGroup = (Hashtable)store.get("newGroup");
        Hashtable oldGroup = (Hashtable)store.get("oldGroup");
        String handle = (String)store.get("handle");
        String object = (String)store.get("object");

        modTop(session, out, object, handle, "Group", "Group", "hlpgroup.html", sessionKey, rootURL);

        // group name
        String group_name = ((String)Utils.n2s(newGroup.get("name"))).trim();
        out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font> Group</TD>\n"+
                    "<TD VALIGN='TOP'><INPUT ID='TextField_Name' "+
                    (newGroup.get("group_name_error")!=null?"CLASS='error_form_font' ":"")+
                    "TYPE='TEXT' SIZE='45' NAME='group' value=\""+Utils.htmlize(group_name, false)+"\">"+
                    "</TD></TR>\n");

        // comment
        String comment = ((String)Utils.n2s(newGroup.get("comment"))).trim();
        out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Comment</TD>\n"+
                    "<TD VALIGN='TOP'>"+
                    "<TEXTAREA ID='TextArea_Comment' NAME='comment' ROWS='3' COLS='60'>"+
                    Utils.htmlize(comment, false)+
                    "</TEXTAREA></TD></TR>\n");

        Hashtable person = (Hashtable)newGroup.get("created.by");
        if( person != null ) {
            out.println("<TR><TD COLSPAN='2'><HR NOSHADE SIZE=1></TD></TR>");
            String dirID = (String)person.get("DirID");
            boolean active = activep(person);

            String byStr = (active?"<A HREF=\""+
                            System.getProperty("stanfordwho.url")+dirID.trim()+"\">":"");
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += (active?"</A>":"")+" ("+newGroup.get("created").toString().substring(0, 16)+")";
            out.println("<TR><TD VALIGN='TOP'>&nbsp;Created By</TD>\n"+
                        "<TD VALIGN='TOP'>"+
                        byStr+"</TD></TR>");

            // modified by
            person = (Hashtable)newGroup.get("mod.by");
            dirID = (String)person.get("DirID");
            active = activep(person);

            byStr = (active?"<A HREF=\""+
                     System.getProperty("stanfordwho.url")+dirID.trim()+"\">":"");
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += (active?"</A>":"")+" ("+newGroup.get("modified").toString().substring(0, 16)+")";
            out.println("<TR><TD VALIGN='TOP'>&nbsp;Last Modified By</TD>\n"+
                        "<TD VALIGN='TOP'>"+
                        byStr+"</TD></TR>");
        }

        modBottom(out);
    } // end of printOutModifyHTML(...)
}
