/************************************************************************
 *
 * ModifyNetwork.java -
 *
 * $Id: ModifyNetwork.java,v 1.7 2007/01/03 23:44:32 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2004, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.sql.*;
import java.net.*;
import java.util.*;
import java.text.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import su.netdb.parser.*;
import stanford.netdb.utils.*;

public class ModifyNetwork extends ModifyPage {

    public ModifyNetwork() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("type", "Network");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        try {
            PrintWriter out = response.getWriter();
            String rootURL = request.getServletPath();

            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String default_domain = (String)session.getAttribute("default_domain");

            // obtain key network attributes from request
            String object = request.getParameter("handle");
            String handle = null;
            Timestamp lockTm = null;
            int oid = -1;
            boolean numeric_handle = false;
            if( object != null ) {
                try {
                    oid = Integer.parseInt(object);
                    numeric_handle = true;
                } catch(Exception ex) { }
                handle = new String(object);
                store.put("handle", handle);
                if( rootURL.equals("/ModifyNetwork") ) {
                    // lock
                    try {
                        if( numeric_handle )
                            lockTm = netdb.lock(Constants.NETWORK, oid);
                        else
                            lockTm = netdb.lock(Constants.NETWORK, handle);

                        if( lockTm == null )
                            throw new Exception("Failed to lock network "+handle);

                    } catch (Exception ex) {
                        ex.printStackTrace();
                        session.setAttribute("Error.errorMsg", ex.getMessage());
                        response.sendRedirect((String)store.get("fromPage"));
                        return;
                    }
                    store.put("lockTm", lockTm);
                }
            }

            Hashtable oldNetwork = null;
            if( rootURL.equals("/CreateNetwork") ) {

                if( !netdb.checkAccess(Constants.NETWORK) )
                    throw new Exception("User '"+session.getAttribute("NetID")+"' can not create networks.");

                Hashtable address_spaces = new Hashtable();
                if( handle == null ) { // creating clean network
                    object = "";
                    oldNetwork = new Hashtable();
                    oldNetwork.put("id", "");
                    oldNetwork.put("object", "network");
                    oldNetwork.put("groups", new Hashtable());
                    oldNetwork.put("comment", "");
                    oldNetwork.put("dhcp_options", new Hashtable());
                    oldNetwork.put("address_spaces", address_spaces);
                    Hashtable namesH = new Hashtable();
                    Hashtable nameH = new Hashtable();
                    nameH.put("name", "");
                    nameH.put("domain", "");
                    namesH.put("", nameH);
                    oldNetwork.put("names", namesH);

                } else { // creating network from template
                    if( numeric_handle )
                        oldNetwork = netdb.getInfo(Constants.NETWORK, oid);
                    else
                        oldNetwork = netdb.getInfo(Constants.NETWORK, object);

                    if (oldNetwork == null || oldNetwork.get("id") == null ) {
                        session.setAttribute("Error.errorMsg",
                                             "Error: No info found for network "+object);
                        response.sendRedirect((String)store.get("fromPage"));
                        return;
                    }
                    handle = null;
                    // we inherit only few parts of original network
                    oldNetwork.put("address_spaces", address_spaces);
                    Hashtable oldNames = (Hashtable)oldNetwork.get("names");
                    oldNames.clear();
                    Hashtable nameH = new Hashtable();
                    oldNames.put("", nameH);
                    nameH.put("name", "");
                    nameH.put("domain", "");
                    oldNetwork.remove("created.by");
                }


                Hashtable global_defaults = Utils.nHash(getData(session, "global.defaults"));

                Object def_high_res = global_defaults.get("DefaultHighRes");
                if( def_high_res == null ) def_high_res = "0";

                Object def_low_res  = global_defaults.get("DefaultLowRes");
                if( def_low_res == null ) def_low_res = "1";

                Hashtable as = new Hashtable();
                address_spaces.put("as "+getNextKey(), as);
                as.put("raw_low_res", def_low_res.toString());
                as.put("raw_high_res", def_high_res.toString());
                as.put("raw_as_prefix", ""); // putting this one indicates that it is a new address space
                as.put("addr_space_ip", "");
                as.put("prefix_length", new Integer(0));
                as.put("low_res", new Integer(def_low_res.toString()));
                as.put("high_res", new Integer(def_high_res.toString()));

            } else { // network modify
                if( numeric_handle )
                    oldNetwork = netdb.getInfo(Constants.NETWORK, oid);
                else
                    oldNetwork = netdb.getInfo(Constants.NETWORK, object);


                // extract name with original case
                for( Hashtable name : (Collection<Hashtable>)Utils.nHash(oldNetwork.get("names")).values() ) {
                    store.put("handle", name.get("name")+"."+name.get("domain"));
                    break;
                }

                if (oldNetwork == null || oldNetwork.get("id") == null ) {
                    session.setAttribute("Error.errorMsg",
                                         "Error: No info found for network "+object);
                    response.sendRedirect((String)store.get("fromPage"));
                    return;
                }
            }

            if( lockTm != null )
                oldNetwork.put("locked", lockTm);

            Hashtable newNetwork = Utils.copyHash(oldNetwork);

            store.put("oldNetwork",     oldNetwork);
            store.put("newNetwork",     newNetwork);
            store.put("oldNetwork",     Utils.copyHash(oldNetwork));
            store.put("oldNewNetwork",  Utils.copyHash(newNetwork));
            store.put("troubled_ips",   new Hashtable());
            store.put("troubled_names", new Hashtable());
            store.put("reserved_names", new Hashtable());
            store.put("reserved_ips",   new Hashtable());

            // extact name with original case
            object = extractName(oldNetwork, object);

            // compress network name by stripping default domain
            object = compressName(object, default_domain);
            store.put("object", object);

            printOutModifyHTML(session, out, store, rootURL, "0", "0");

        } catch (Exception ex) {
            // display error page
            ex.printStackTrace();
            String errMsg = ex.getMessage();
            errMsg = (errMsg == null ? "???" : errMsg);
            session.setAttribute("Error.errorMsg", errMsg);
            response.sendRedirect((String)store.get("fromPage"));
            return;
        }
    } // end of doGet(...)

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        HttpSession session = begin(request, response, null);
        if( session == null )
            return;

        PrintWriter out          = response.getWriter();
        String rootURL           = request.getServletPath();

        String sessionKey        = decode(request.getParameter("sessionKey"));
        String default_domain    = (String)session.getAttribute("default_domain");
        Hashtable store          = (Hashtable)session.getAttribute(sessionKey);
        NetDBpi netdb            = (NetDBpi)session.getAttribute("netdb");
        String handle            = (String)store.get("handle");
        Timestamp lockTm         = (Timestamp)store.get("lockTm");
        Hashtable newNetwork     = (Hashtable)store.get("newNetwork");
        Hashtable oldNetwork     = (Hashtable)store.get("oldNetwork");
        Hashtable oldNewNetwork  = (Hashtable)store.get("oldNewNetwork");
        Hashtable troubled_names = (Hashtable)store.get("troubled_names");
        Hashtable troubled_ips   = (Hashtable)store.get("troubled_ips");
        String target            = null;
        String[] targets         = new String[0];

        String action = (String)Utils.n2s((String)request.getParameter("Action"));
        try {

            if( request.getParameter("ready") != null ) {
                target = getParameterNameStaringWith(request, "display_dhcp_options");
                if( target != null ) {
                    action = "display_dhcp_options";
                    target = target.substring(21);

                } else {
                    target = getParameterNameStaringWith(request, "as verify ip ");
                    if( target != null ) {
                        action = "verify_ip";
                        target = target.substring(13);
                        targets = new String[1];
                        targets[0] = target;
                    }
                }

                if( target == null ) {
                    target = getParameterNameStaringWith(request, "split_join ");
                    if( target != null ) {
                        action = "split_join";
                        target = target.substring(11);
                    }
                }

                if( !action.equals("Reset") && !action.equals("Cancel") ) {
                    // pick up new data
                    boolean addressesMerged = false;

                    // pick up network name
                    Hashtable names = (Hashtable)newNetwork.get("names");
                    String[] names_cb = getParametersStaringWith(request, "network_name_tf_");
                    for( int i = 0; i < names_cb.length; i += 2 ) {
                        String key = names_cb[i].substring(16);
                        String name = names_cb[i+1];
                        Hashtable nameH = new Hashtable();
                        nameH.put("name", Utils.name(name).trim());
                        nameH.put("domain", Utils.domain(name, default_domain).trim());
                        names.put(key, nameH);
                    }

                    // pick up locations
                    Hashtable locations = Utils.get_hash_or_create(newNetwork, "locations");
                    locations.clear();
                    String[] locKeys = getParameterNamesStaringWith(request, "location_");
                    for( int i = 0; i < locKeys.length; i++ ) {
                        String location_name = decode(request.getParameter(locKeys[i]));
                        if( location_name.equals(" --None-- ") )
                            location_name = " --None-- "+getNextKey();
                        Hashtable location = new Hashtable();
                        location.put("name", location_name);
                        location.put("site", "");
                        location.put("code", "");
                        locations.put(location_name, location);
                    }

                    // pick up groups
                    Hashtable groups = new Hashtable();
                    newNetwork.put("groups", groups);

                    String[] newGroups = request.getParameterValues("groups");
                    if( newGroups != null ) {
                        for( int i = 0; i < newGroups.length; i++ )
                            groups.put(decode(newGroups[i]), "");
                    }

                    // pick up comment
                    newNetwork.put("comment",
                                   ((String)Utils.n2s((String)request.getParameter("comment"))).trim());

                    // pick up dhcp service
                    String dhcpService = ((String)Utils.n2s((String)request.getParameter("dhcpService"))).trim();
                    if( dhcpService.equals("--") )
                        newNetwork.remove("dhcp_service");
                    else
                        newNetwork.put("dhcp_service", dhcpService);

                    Hashtable dhcp_options = nHash(getData(session, "dhcp_options"));

                    // pick up dhcp options
                    String dhcp_text = request.getParameter("network_dhcp_options");
                    boolean badDHCPOptions = false;
                    if( dhcp_text != null ) {
                        newNetwork.put("dhcp_options_raw", dhcp_text);

                        String[] split = dhcp_text.split("\\n");
                        Hashtable options = new Hashtable();
                        if( dhcp_text.length() > 0 ) {
                            for( int j = 0; j < split.length; j++ ) {
                                if( split[j].trim().length() == 0 )
                                    continue;
                                String[] ops = split[j].split("=", 2);
                                if( ops.length != 2 ) {
                                    addSessionMsg(session, "Error.errorMsg",
                                                  "Error: Network's DHCP option \""+split[j]+"\" does not have value.");
                                    badDHCPOptions = true;
                                } else {
                                    Hashtable optionH = new Hashtable();
                                    optionH.put("option", ops[0].trim());
                                    optionH.put("value", ops[1].trim());
                                    options.put(ops[0].trim()+"="+ops[1].trim(), optionH);

                                    if( nHash(dhcp_options.get(ops[0].trim().toLowerCase())).get("network") == null ) {
                                        badDHCPOptions = true;
                                        addSessionMsg(session, "Error.errorMsg",
                                                      "Error: Illegal DHCP option \""+ops[0].trim()+"\" in network.");
                                    }
                                }
                            }
                        }
                        newNetwork.put("dhcp_options", options);
                    }
                    if( badDHCPOptions )
                        newNetwork.put("bad dhcp options", "");
                    else
                        newNetwork.remove("bad dhcp options");

                    // pick up addresses spaces and all related data
                    Hashtable ases = (Hashtable)newNetwork.get("address_spaces");
                    if( ases == null ) {
                        ases = new Hashtable();
                        newNetwork.put("address_spaces", ases);
                    }

                    String[] ases_to_keep = request.getParameterValues("address_space");
                    if( ases_to_keep == null )
                        ases_to_keep = new String[0];
                    Hashtable keptAses = new Hashtable();
                    for( int i = 0; i < ases_to_keep.length; i++ )
                        keptAses.put(decode(ases_to_keep[i]), "");

                    // now pick up adderss spaces to be removed
                    Vector ases_to_remove = new Vector();
                    for( Object key : ases.keySet() )
                        if( keptAses.get(key) == null ) ases_to_remove.addElement(key);

                    // now remove address spaces not intended to be kept
                    for( Object key : ases_to_remove ) ases.remove(key);

                    // pick up ases_count
                    String raw_ases_count = ((String)Utils.n2s(request.getParameter("ases_count"))).trim();
                    newNetwork.put("raw_ases_count", raw_ases_count);
                    if( raw_ases_count.length() > 0 ) {
                        try {
                            int add_count = Integer.parseInt(raw_ases_count);
                            if( add_count < 0 )
                                throw new Exception ("");
                            newNetwork.remove("ases_count_error");
                        } catch (Exception ex) {
                            newNetwork.put("ases_count_error", "");
                        }
                    }

                    // now go over all keptAsses and pick up related data
                    for( String key : (Set<String>)keptAses.keySet() ) {
                        Hashtable as = (Hashtable)ases.get(key);

                        // pick up raw_as_prefix if there is one
                        String raw_as_prefix = request.getParameter(encode("raw_as_prefix "+key));
                        if( raw_as_prefix != null ) {
                            raw_as_prefix = raw_as_prefix.trim();
                            as.put("raw_as_prefix", raw_as_prefix);
                        }


                        // if the prefix length is greater than zero and the action is Save or Verify
                        // ie, The prefix is given while saving or verifying, fill in the data.
                        if( raw_as_prefix != null && raw_as_prefix.trim().length() > 0 &&
                            (action.equals("Save") || action.equals("Verify")) ) {
                            try {
                                // add zeroes if needed
                                raw_as_prefix = Utils.normIp(raw_as_prefix);

                                Hashtable parseRes = (new Parser()).parse(raw_as_prefix);
                                String ip_low = (String)parseRes.get("ip_low");
                                String ip_high = (String)parseRes.get("ip_high");
                                if( ( ip_low == null || ip_high == null ) )
                                    throw new Exception();
                                if( ip_low.equals(ip_high) )
                                    throw new IllegalArgumentException("Address space prefix must be between 8 and 30 inclusive.");

                                Integer prefix = new Integer(raw_as_prefix.substring(raw_as_prefix.indexOf("/")+1));
                                if( prefix.intValue() > 30 )
                                    throw new IllegalArgumentException("Address space prefix must be between 8 and 30 inclusive.");

                                as.put("addr_space_ip", ip_low);
                                as.put("prefix_length", prefix);

                                as.put("addr_space_prefix", raw_as_prefix);
                            } catch (IllegalArgumentException ex) {
                                addSessionMsg(session, "Error.errorMsg", "Error: "+ex.getMessage());
                            } catch (Exception ex) {
                                String msg = (String)Utils.n2s(ex.getMessage());
                                if( msg.startsWith("bad prefix length, ") )
                                    addSessionMsg(session, "Error.errorMsg", "Error: Address space prefix must be between 8 and 30 inclusive.");
                                else
                                    addSessionMsg(session, "Error.errorMsg", "Error: Incorrect address space prefix format '"+raw_as_prefix+"'");
                            }
                        }

                        int prefixLength = ((Integer)as.get("prefix_length")).intValue();

                        // pick up low reserve
                        String low_res = ((String)Utils.n2s(request.getParameter(encode("low_res "+key)))).trim();
                        as.put("raw_low_res", low_res);
                        if( low_res.length() == 0 )
                            addSessionMsg(session, "Error.errorMsg",
                                          "Error: Address space "+as.get("addr_space_prefix")+" does not have low reserve");
                        else if( as.get("addr_space_prefix") != null ) {
                            try {
                                int low_res_int = Integer.parseInt(low_res);
                                as.put("low_res", new Integer(low_res_int));

                                if( low_res_int < 0 )
                                    throw new Exception();

                                int total_ips = (0x00000001 << (32 - prefixLength));
                                if( low_res_int >= total_ips ) {
                                    addSessionMsg(session, "Error.errorMsg",
                                                  "Error: Low reserve value in address space "+as.get("addr_space_prefix")+" exceeds total amount of avaliable ip addresses");
                                    as.put("bad low res", "");
                                } else
                                    as.remove("bad low res");

                            } catch(Exception ex) {
                                addSessionMsg(session, "Error.errorMsg",
                                              "Error: Illegal low reserve value \""+low_res+"\"");
                                as.put("bad low res", "");
                            }
                        }

                        // pick up prefix (it is used for joining or splitting address spaces)
                        String prefix = ((String)Utils.n2s(request.getParameter(encode("prefix length "+key)))).trim();
                        as.put("raw_prefix_length", prefix);

                        // pick up high reserve
                        String high_res =((String)Utils.n2s(request.getParameter(encode("high_res "+key)))).trim();
                        as.put("raw_high_res", high_res);
                        if( high_res.length() == 0 )
                            addSessionMsg(session, "Error.errorMsg",
                                          "Error: Address space "+as.get("addr_space_prefix")+" does not have high reserve");
                        else if( as.get("addr_space_prefix") != null ) {
                            try {
                                int high_res_int = Integer.parseInt(high_res);
                                as.put("high_res", new Integer(high_res_int));
                                if( high_res_int < 0 )
                                    throw new Exception();

                                int total_ips = (0x00000001 << (32 - prefixLength));
                                if( high_res_int >= total_ips ) {
                                    addSessionMsg(session, "Error.errorMsg",
                                                  "Error: High reserve value in address space "+as.get("addr_space_prefix")+" exceeds total amount of avaliable ip addresses");
                                    as.put("bad high res", "");
                                } else
                                    as.remove("bad high res");
                            } catch(Exception ex) {
                                addSessionMsg(session, "Error.errorMsg",
                                              "Error: Illegal high reserve value \""+high_res+"\"");
                                as.put("bad high res", "");
                            }
                        }

                        // pick up groups
                        groups = new Hashtable();
                        as.put("groups", groups);

                        newGroups = request.getParameterValues(encode("as_groups "+key));
                        if( newGroups != null ) {
                            for( int i = 0; i < newGroups.length; i++ )
                                groups.put(decode(newGroups[i]), "");
                        }

                        // pick up dhcp options if any
                        badDHCPOptions = false;
                        dhcp_text = request.getParameter(encode("dhcp_options "+key));
                        if( dhcp_text != null ) {
                            as.put("dhcp_options_raw", dhcp_text);

                            String[] split = dhcp_text.split("\\n");
                            Hashtable options = new Hashtable();
                            if( dhcp_text.length() > 0 ) {
                                for( int j = 0; j < split.length; j++ ) {
                                    if( split[j].trim().length() == 0 )
                                        continue;
                                    String[] ops = split[j].split("=", 2);
                                    if( ops.length != 2 ) {
                                        badDHCPOptions = true;
                                        addSessionMsg(session, "Error.errorMsg",
                                                      "Error: DHCP option \""+split[j]+"\" in address space "+as.get("addr_space_prefix")+" does not have value.");
                                    } else {
                                        Hashtable optionH = new Hashtable();
                                        optionH.put("option", ops[0].trim());
                                        optionH.put("value", ops[1].trim());
                                        options.put(ops[0].trim()+"="+ops[1].trim(), optionH);

                                        if( nHash(dhcp_options.get(ops[0].trim().toLowerCase())).get("addressspace") == null ) {
                                            badDHCPOptions = true;
                                            addSessionMsg(session, "Error.errorMsg",
                                                          "Error: Illegal DHCP option \""+ops[0].trim()+"\" in address space "+as.get("addr_space_prefix"));
                                        }
                                    }
                                }
                            }
                            as.put("dhcp_options", options);
                        } else
                            as.put("dhcp_options", new Hashtable());

                        if( badDHCPOptions )
                            as.put("bad dhcp options", "");
                        else
                            as.remove("bad dhcp options");

                        // pick up comment
                        as.put("comment",
                               ((String)Utils.n2s(request.getParameter(encode("asComment "+key)))).trim());

                        if( as.get("mute") != null )
                            continue;

                        // pick up ip addresses, thier names and statuses
                        Hashtable ips = new Hashtable();
                        as.put("dhcp_addresses", ips);

                        String[] params = request.getParameterValues(encode("ip cb ["+key+"]"));
                        if( params != null ) {
                            for( int i = 0; i < params.length; i++ ) {
                                String ipKey = params[i];
                                String ipStr = request.getParameter(encode("ip addr ["+key+"] ["+ipKey+"]")).trim();
                                if( ipStr == null || ipStr.length() == 0 )
                                    continue;

                                ipStr = Utils.normIp(ipStr);

                                Hashtable ip = new Hashtable();
                                if( ips.put(ipStr, ip) != null )
                                    addressesMerged = true;

                                if( request.getParameter(encode("ip status ["+key+"] ["+ipKey+"]")) != null )
                                    ip.put("active?", Boolean.TRUE);
                                else
                                    ip.put("active?", Boolean.FALSE);

                                names = new Hashtable();
                                ip.put("names", names);

                                String nameStr =
                                    request.getParameter(encode("ip name ["+key+"] ["+ipKey+"]"));
                                if( nameStr != null && nameStr.trim().length() > 0 ) {
                                    Hashtable nameH = new Hashtable();
                                    String name = Utils.name(nameStr).trim();
                                    nameH.put("name", name);
                                    String domain = Utils.domain(nameStr, default_domain).trim();
                                    nameH.put("domain", domain);
                                    names.put(name+"."+domain+" "+getNextKey(), nameH);
                                }
                            }
                        }

                        // pick up "new ip addr ..." and an error if any
                        String raw_new_ip_addr =
                            ((String)Utils.n2s(request.getParameter(encode("new ip addr "+key)))).trim();
                        raw_new_ip_addr = Utils.normIp(raw_new_ip_addr);
                        as.put("raw_new_ip_addr", raw_new_ip_addr);
                        if( raw_new_ip_addr.length() > 0 ) {
                            try {
                                Hashtable parseRes = (new Parser()).parse(raw_new_ip_addr);
                                if( parseRes.get("ip_low") == null || parseRes.get("ip_high") == null )
                                    addSessionMsg(session, "Error.errorMsg", "Error: Incorrect ip address format '"+raw_new_ip_addr+"'");
                            } catch (Exception ex) {
                                addSessionMsg(session, "Error.errorMsg", "Error: Incorrect ip address format '"+raw_new_ip_addr+"'");
                            }
                        }

                        // pick up "new ip status ..."
                        as.remove("raw_new_ip_status_inactive");
                        if( request.getParameter(encode("new ip status "+key)) == null )
                            as.put("raw_new_ip_status_inactive", "");

                        // pick up "new ip count ..."
                        String raw_new_ip_count =
                            ((String)Utils.n2s(request.getParameter(encode("new ip count "+key)))).trim();
                        as.put("raw_new_ip_count", raw_new_ip_count);
                        if( raw_new_ip_count.length() > 0 ) {
                            try {
                                int new_ip_count = Integer.parseInt(raw_new_ip_count);
                                if( new_ip_count <= 0 )
                                    addSessionMsg(session, "Error.errorMsg",
                                                  "Error: Illegal IP Count value '"+raw_new_ip_count+"'.");
                            } catch (Exception ex) {
                                addSessionMsg(session, "Error.errorMsg",
                                              "Error: IP Count value '"+raw_new_ip_count+"' is not an integer.");
                            }
                        }

                    } // end of going over all address spaces

                    if( !action.equals("Verify") ) {
                        // if action is Verify check for a duplicate names will
                        // be done at the end of verify stage
                        String err = checkDuplicateNames(newNetwork);
                        if( err != null && err.trim().length() > 0 )
                            addSessionMsg(session, "Error.errorMsg", err);
                    }

                    if( addressesMerged )
                        addSessionMsg(session, "Warning", "WARNING: Duplicate ip addresses were merged.");

                } // end of 'if( !action.equals("Reset") )'

                if( action.equals("display_dhcp_options") ) {
                    Hashtable dhcpOptions = new Hashtable();
                    Hashtable dhcpOption = new Hashtable();
                    dhcpOption.put("option", "");
                    dhcpOption.put("value", "");
                    dhcpOptions.put(" ", dhcpOption);
                    Hashtable as = nHash(nHash(newNetwork.get("address_spaces")).get(target));
                    as.put("dhcp_options", dhcpOptions);
                    as.put("dhcp_options_raw", "");

                }

                if( action.equals("Verify") || action.equals("Save") ) {

                    // clean up locations from irrelevan items
                    Hashtable locations = Utils.get_hash_or_create(newNetwork, "locations");
                    Vector locations_to_remove = new Vector();
                    for( String location : (Set<String>)locations.keySet() )
                        if( location.startsWith(" --None--") )
                            locations_to_remove.addElement(location);

                    // now remove them, locations that is.
                    for( Object location : locations_to_remove ) locations.remove(location);

                    // check network's comment
                    if( ((String)Utils.n2s(newNetwork.get("comment"))).length() > 240 )
                        newNetwork.put("bad comment", "");
                    else
                        newNetwork.remove("bad comment");

                    Hashtable ases = nHash(newNetwork.get("address_spaces"));
                    for( Hashtable as : (Collection<Hashtable>)ases.values() ) {
                        if( ((String)Utils.n2s(as.get("comment"))).length() > 60 )
                            as.put("bad comment", "");
                        else
                            as.remove("bad comment");
                    }

                    Hashtable allOldNames = new Hashtable();
                    Hashtable allOldIPs = new Hashtable();
                    try {
                        collectNamesAndIPs(oldNetwork, allOldNames, allOldIPs, new Hashtable());
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }

                    Hashtable allNewNames = new Hashtable();
                    Hashtable allNewIPs = new Hashtable();
                    Hashtable ipParents = new Hashtable();
                    try {
                        collectNamesAndIPs(newNetwork, allNewNames, allNewIPs, ipParents);
                    } catch (Exception ex) {
                        addSessionMsg(session, "Error.errorMsg", "Error: "+ex.getMessage());
                    }

                    Vector names_to_reserve = new Vector();
                    Vector names_to_cancel = new Vector();
                    Vector names_no_change = new Vector();

                    Utils.keysIntersect(allOldNames, allNewNames,
                                        names_to_reserve, names_to_cancel, names_no_change);

                    Hashtable reserved_names = (Hashtable)store.get("reserved_names");

                    boolean got_some_errors = false;
                    for( int i = 0; i < names_to_reserve.size(); i++ ) {
                        String key = (String)names_to_reserve.elementAt(i);
                        Hashtable nameH = (Hashtable)allNewNames.get(key);
                        String name = ((String)nameH.get("name")).trim()+"."+((String)nameH.get("domain")).trim();
                        String nameType = nameType(key);
                        try {
                            try {
                                String r_name = netdb.reserveName(name, nameType);

                                if( r_name != null && r_name.trim().length() > 0 ) {
                                    nameH.put("reserved", "");
                                    nameH.remove("error");
                                    synchronized(nextKey) {
                                        Hashtable nameStore = new Hashtable();
                                        nameStore.put("fqdn", r_name);
                                        nameStore.put("type", nameType);
                                        reserved_names.put(nameType+" "+r_name, nameStore);
                                        troubled_names.remove(nameType+" "+r_name);
                                    }
                                } else {
                                    throw new Exception("Error: Failed to reserve name '"+name+"'");
                                }
                            } catch (NetDBException ex) {
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int j = 0; j < esize; j++ )
                                    msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                                throw new Exception(msg);
                            }
                        } catch (Exception ex) {
                            if( name != null && name.trim().length() > 0 ) {
                                synchronized(nextKey) {
                                    Hashtable nameStore = new Hashtable();
                                    nameStore.put("fqdn", name);
                                    nameStore.put("type", nameType);
                                    troubled_names.put(nameType+" "+name, nameStore);
                                }
                            }

                            got_some_errors = true;
                            nameH.put("error", "");
                            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                        }
                    }
                    if( !got_some_errors && names_to_reserve.size() > 0 )
                        addSessionMsg(session, "info_msg", "SUCCESS: Network names verified.");

                    String err = checkDuplicateNames(newNetwork);
                    if( err != null && err.trim().length() > 0 )
                        addSessionMsg(session, "Error.errorMsg", err);

                    // now ip addresses
                    Vector ips_to_reserve = new Vector();
                    Vector ips_to_cancel = new Vector();
                    Vector ips_no_change = new Vector();
                    got_some_errors = false;
                    try {
                        Utils.keysIntersect(allOldIPs, allNewIPs,
                                            ips_to_reserve, ips_to_cancel, ips_no_change);
                    } catch (Exception ex) {
                        ex.printStackTrace();
                        throw ex;
                    }

                    Hashtable reserved_ips = (Hashtable)store.get("reserved_ips");
                    if( reserved_ips == null ) {
                        reserved_ips = new Hashtable();
                        store.put("reserved_ips", reserved_ips);
                    }

                    for( int i = 0; i < ips_to_reserve.size(); i++ ) {
                        String requested_ip = (String)ips_to_reserve.elementAt(i);
                        Hashtable ipH = new Hashtable();
                        try {
                            try {
                                requested_ip = Utils.normIp(requested_ip);
                                String ip_low = null;
                                String ip_high = null;
                                try {
                                    Hashtable parseRes = (new Parser()).parse(requested_ip);
                                    ip_low = (String)parseRes.get("ip_low");
                                    ip_high = (String)parseRes.get("ip_high");
                                } catch (Exception exx) { }

                                if( ( ip_low == null || ip_high == null ) ) {
                                    addSessionMsg(session, "Error.errorMsg", "Incorrect ip address format '"+requested_ip+"'");
                                    troubled_ips.put(requested_ip.trim().toLowerCase(), "");
                                    got_some_errors = true;
                                    continue;
                                }

                                Hashtable result = netdb.reserveIP(ip_low.equals(ip_high)?null:requested_ip, ip_low, 1);

                                result = nHash(result.get(result.keys().nextElement()));
                                Hashtable warnings = nHash(result.get("warnings"));
                                if( warnings.size() > 0 ) {
                                    for( String msg : (Set<String>)warnings.keySet() )
                                        addSessionMsg(session, "Warning", msg.trim());
                                }

                                ipH = new Hashtable();
                                String reserved_ip = null;
                                for( String key : (Set<String>)nHash(result.get("ip_addresses")).keySet() ) {
                                    reserved_ip = key; ipH.put("ip", reserved_ip);
                                }

                                if( reserved_ip == null )
                                    throw new Exception("Error: Failed to reserve ip address '"+requested_ip+"'.");

                                synchronized(nextKey) {
                                    reserved_ips.put(ipH.get("ip"), "");
                                    troubled_ips.remove(((String)ipH.get("ip")).trim());
                                }

                                Hashtable masterIpHash = (Hashtable)allNewIPs.get(requested_ip);
                                ipH.put("names", nHash(masterIpHash.get("names")));
                                ipH.put("active?", masterIpHash.get("active?"));

                                Hashtable parentHash = nHash(ipParents.get(masterIpHash.get("parent")));
                                parentHash.remove(requested_ip);
                                parentHash.put(reserved_ip, ipH);

                            } catch (NetDBException ex) {
                                ex.printStackTrace();
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int j = 0; j < esize; j++ )
                                    msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                                throw new Exception(msg);
                            }
                        } catch (Exception ex) {
                            troubled_ips.put(requested_ip.toLowerCase().trim(), "");
                            ex.printStackTrace();
                            got_some_errors = true;
                            if( ipH != null )
                                ipH.put("error", "");
                            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                        }
                    }

                    if( !got_some_errors && ips_to_reserve.size() > 0 )
                        addSessionMsg(session, "info_msg", "SUCCESS: Ip addresses verified.");

                    oldNewNetwork = Utils.copyHash(newNetwork);
                    store.put("oldNewNetwork", oldNewNetwork);

                    fillIPsInASes(nHash(newNetwork.get("address_spaces")));
                    try {
                        purgeEmptyASes(newNetwork);
                        Utils.validate(Constants.NETWORK, newNetwork, Constants.NARROW);

                    } catch (NetDBException ex) {
                        Vector messages = ex.getMessages();
                        if( messages != null && messages.size() > 0 ) {
                            for( int i = 0; i < messages.size(); i++ ) {
                                Object msg = messages.elementAt(i);
                                if( msg.equals("address space does not have addr_space_prefix field") )
                                    // we want to suppress this error message
                                    continue;
                                addSessionMsg(session, "Error.errorMsg", "Error: "+msg);
                            }
                        }
                    } catch (Exception ex) {
                        addSessionMsg(session, "Error.errorMsg", "Error: "+ex.getMessage());
                    }

                } // end of 'if( action.equals("Verify") || action.equals("Save") )'

                if( action.equals("verify_ip") || action.equals("Verify") || action.equals("Save") ) {
                    if( action.equals("Verify") || action.equals("Save") ) {
                        Hashtable ases = nHash(newNetwork.get("address_spaces"));
                        targets = new String[ases.size()];
                        int i = 0;
                        for( String as : (Set<String>)ases.keySet() )
                            targets[i++] = as;
                    }
                    for( int i = 0; i < targets.length; i++ ) {
                        target = targets[i];
                        Hashtable as = (Hashtable)nHash(newNetwork.get("address_spaces")).get(target);
                        // we need to check for address space existance
                        // here becouse it could have been gone by now
                        if( as != null ) {
                            String as_prefix = (String)as.get("addr_space_prefix");

                            String requested_ip = ((String)Utils.n2s(request.getParameter(encode("new ip addr "+target)))).trim();
                            requested_ip = Utils.normIp(requested_ip);
                            String requested_count = ((String)Utils.n2s(request.getParameter(encode("new ip count "+target)))).trim();
                            if( requested_count.length() == 0 ) {
                                if( ( action.equals("Verify") || action.equals("Save") ) && requested_ip.length() == 0 )
                                    requested_count = "0";
                                else
                                    requested_count = "1";
                            }
                            Boolean active =
                                new Boolean(request.getParameter(encode("new ip status "+target))==null?false:true);
                            for(;;) {
                                try {
                                    Hashtable parseRes = (new Parser()).parse(as_prefix);
                                    long as_low_ip = Utils.ipstr2long((String)parseRes.get("ip_low"));
                                    long as_high_ip = Utils.ipstr2long((String)parseRes.get("ip_high"));

                                    int requested_ips = Integer.parseInt(requested_count);
                                    if( requested_ips > 0 ) {

                                        if( requested_ip.length() == 0 )
                                            requested_ip = as_prefix;

                                        String requested_as = null;
                                        if( !requested_ip.equals(as_prefix) ) {
                                            String ip_low = null;
                                            String ip_high = null;
                                            try {
                                                parseRes = (new Parser()).parse(requested_ip);
                                                ip_low = (String)parseRes.get("ip_low");
                                                ip_high = (String)parseRes.get("ip_high");
                                            } catch (Exception exx) { }

                                            if( ( ip_low == null || ip_high == null )  ) {
                                                //addSessionMsg(session, "Error.errorMsg", "Error: Incorrect ip address format '"+requested_ip+"'");
                                                break;
                                            }

                                            if( !ip_low.equals(ip_high) ) {
                                                requested_as = requested_ip;
                                                requested_ip = null;
                                            }

                                            long req_ip = Utils.ipstr2long(ip_low);
                                            if( req_ip < as_low_ip || req_ip > as_high_ip ) {
                                                addSessionMsg(session, "Error.errorMsg", "Error: The start address '"+ip_low+
                                                              "' does not correspond to the specified address space '"+as_prefix+"'");
                                                break;
                                            }
                                        } else {
                                            requested_as = as_prefix;
                                            requested_ip = null;
                                        }

                                        Hashtable ips = (Hashtable)as.get("dhcp_addresses");
                                        if( ips == null ) {
                                            ips = new Hashtable();
                                            as.put("dhcp_addresses", ips);
                                        }
                                        if( reserveIPandName(session, netdb, ips, store, active, requested_ip,
                                                             requested_as, null, requested_ips, "DHCP", true, true) ) {
                                            // remove saved states
                                            as.remove("raw_new_ip_addr");
                                            as.remove("raw_new_ip_count");
                                            as.remove("raw_new_ip_status_inactive");
                                        }
                                        //Hashtable result = netdb.reserveIP(null, requested_ip, 1);
                                    }
                                } catch(Exception ex) {}
                                break;
                            }
                        }
                    }
                }

                if( action.equals("Reset") ) {
                    newNetwork = Utils.copyHash((Hashtable)store.get("oldNetwork"));
                    store.put("newNetwork", newNetwork);
                    store.put("oldNewNetwork", Utils.copyHash(newNetwork));

                } else if( action.equals("Cancel") ) {
                    try {
                        try {
                            if( lockTm != null ) {
                                if( Utils.isNumber(handle) )
                                    netdb.unlock(Constants.NETWORK, Integer.parseInt(handle), lockTm);
                                else
                                    netdb.unlock(Constants.NETWORK, handle, lockTm);
                            }

                        } catch (NetDBException ex) {
                            /*
                              Vector errors = ex.getMessages();
                              String msg = "";
                              int esize = errors.size();
                              for( int i = 0; i < esize; i++ )
                              msg += "Error: "+(String)errors.elementAt(i)+(i<esize-1?"\n":"");
                              throw new Exception(msg);
                            */
                        }

                        try {
                            Hashtable reserved_ips = nHash(store.get("reserved_ips"));
                            for( String ip : (Set<String>)reserved_ips.keySet() ) {
                                try {
                                    netdb.cancelIP(ip);
                                } catch (NetDBException ex) {
                                    Vector errors = ex.getMessages();
                                    String msg = "";
                                    int esize = errors.size();
                                    for( int i = 0; i < esize; i++ )
                                        msg += "Error: "+(String)errors.elementAt(i)+(i<esize-1?"\n":"");
                                    throw new Exception(msg);
                                }
                            }

                            Hashtable reserved_names = nHash(store.get("reserved_names"));
                            for( Hashtable nameStore : (Collection<Hashtable>)reserved_names.values() ) {
                                try {
                                    netdb.cancelName((String)nameStore.get("fqdn"), (String)nameStore.get("type"));
                                } catch (NetDBException ex) {
                                    Vector errors = ex.getMessages();
                                    String msg = "";
                                    int esize = errors.size();
                                    for( int i = 0; i < esize; i++ )
                                        msg += "Error: "+(String)errors.elementAt(i)+(i<esize-1?"\n":"");
                                    throw new Exception(msg);
                                }
                            }

                        } catch (NetDBException ex) {
                            Vector errors = ex.getMessages();
                            String msg = "";
                            int esize = errors.size();
                            for( int i = 0; i < esize; i++ )
                                msg += "Error: "+(String)errors.elementAt(i)+(i<esize-1?"\n":"");
                            throw new Exception(msg);
                        }
                    } catch (Exception ex) {
                        session.setAttribute("Error.errorMsg", ex.getMessage());
                    }
                    response.sendRedirect((String)store.get("fromPage"));
                    return;

                } else if( action.equals("Add'l") ) {
                    // add new address space(s)
                    Hashtable address_spaces = (Hashtable)newNetwork.get("address_spaces");
                    if( address_spaces == null ) {
                        address_spaces = new Hashtable();
                        newNetwork.put("address_spaces", address_spaces);
                    }

                    boolean got_error = false;
                    int add_count = 1;
                    String raw_ases_count = (String)Utils.n2s(newNetwork.get("raw_ases_count"));
                    if( raw_ases_count.length() > 0 ) {
                        try {
                            add_count = Integer.parseInt(raw_ases_count);
                            if( add_count < 0 )
                                throw new Exception ("");
                        } catch (Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", "Error: Illegal count value '"+raw_ases_count+"'.");
                            got_error = true;
                        }
                    }

                    if( got_error )
                        newNetwork.put("ases_count_error", "");
                    else {
                        newNetwork.remove("raw_ases_count");
                        newNetwork.remove("ases_count_error");

                        Hashtable global_defaults = Utils.nHash(getData(session, "global.defaults"));

                        Object def_high_res = global_defaults.get("DefaultHighRes");
                        if( def_high_res == null ) def_high_res = "0";

                        Object def_low_res  = global_defaults.get("DefaultLowRes");
                        if( def_low_res == null ) def_low_res = "1";

                        for( int i = 0; i < add_count; i++ ) {
                            Hashtable as = new Hashtable();
                            address_spaces.put("as "+getNextKey(), as);
                            as.put("raw_low_res", def_low_res.toString());
                            as.put("raw_high_res", def_high_res.toString());
                            as.put("raw_as_prefix", ""); // putting this one indicates that it is a new address space
                            as.put("addr_space_ip", "");
                            as.put("prefix_length", new Integer(0));
                            as.put("low_res", new Integer(def_low_res.toString()));
                            as.put("high_res", new Integer(def_high_res.toString()));
                        }
                    }

                } else if( action.equals("split_join") ) {
                    Hashtable address_spaces = nHash(newNetwork.get("address_spaces"));
                    if( address_spaces == null ) {
                        address_spaces = new Hashtable();
                        newNetwork.put("address_spaces", address_spaces);
                    }
                    Hashtable as = (Hashtable)address_spaces.get(target);
                    // we need to check for address space existance
                    // here becouse it could have been gone by now
                    for(;;) {
                        if( as == null )
                            break;

                        int orig_as_prefix_length = ((Integer)as.get("prefix_length")).intValue();
                        String new_as_prefix_length = ((String)Utils.n2s(request.getParameter(encode("prefix length "+target)))).trim();

                        if( new_as_prefix_length.length() == 0 )
                            break;

                        int new_prefix_length = -1;
                        try {
                            new_prefix_length = Integer.parseInt(new_as_prefix_length);
                            if( new_prefix_length < 2 || new_prefix_length > 30 )
                                throw new Exception();

                        } catch(Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", "Illegal prefix length \""+new_as_prefix_length+"\"");
                            break;
                        }
                        try {
                            try {
                                if( new_prefix_length < orig_as_prefix_length ) {
                                    // join
                                    String baseAS = (String)as.get("addr_space_prefix");
                                    int int_prefix_length = orig_as_prefix_length;
                                    String newASPrefix = new String(baseAS);
                                    for(;;) {
                                        String int_prefix = Utils.joinedAS(newASPrefix);
                                        int_prefix_length--;
                                        newASPrefix = int_prefix;
                                        if( int_prefix_length == new_prefix_length )
                                            break;

                                        if( int_prefix_length <= 0 )
                                            throw new Exception("New AS prefix is past zero");
                                    }
                                    String newSubBase = newASPrefix.substring(0, newASPrefix.indexOf("/"))+"/"+(new_prefix_length+1);

                                    netdb.verifyAddressSpace(newASPrefix,
                                                             new Integer((String)((Hashtable)store.get("oldNetwork")).get("id")));
                                    Hashtable joinedAS = Utils.joinAS(newSubBase, address_spaces);
                                    joinedAS.put("raw_as_prefix", joinedAS.get("addr_space_prefix")); // putting this one indicates that it is a new address space
                                    joinedAS.put("mute", "");

                                } else if( new_prefix_length > orig_as_prefix_length ) {
                                    // split
                                    fillIPsInAS(as);
                                    Hashtable result = splitAS(netdb, as, new_prefix_length);
                                    fillIPsInASes(result);
                                    address_spaces.remove(target);
                                    Utils.appendHash(address_spaces, result);
                                }
                            } catch (NetDBException ex) {
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int j = 0; j < esize; j++ )
                                    msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                                throw new Exception(msg);
                            }
                        } catch (Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                        }
                        break;
                    }
                }

                if( action.equals("Save") &&
                    session.getAttribute("Error.errorMsg") == null ) {
                    session.removeAttribute("info_msg");
                    session.removeAttribute("Error.errorMsg");
                    try {
                        try {
                            // extract handle out of node hash
                            Hashtable names = nHash(newNetwork.get("names"));
                            Hashtable nameH = nHash(names.get(names.keys().nextElement()));
                            String handle_fqdn = nameH.get("name")+"."+nameH.get("domain");

                            Utils.purgeObject(newNetwork);
                            purgeEmptyASes(newNetwork);

                            // save all "ip_address" fields
                            Hashtable ases = nHash(newNetwork.get("address_spaces"));
                            for( Hashtable as : (Collection<Hashtable>)ases.values() ) {
                                Hashtable dhcp_addresses = nHash(as.get("dhcp_addresses"));
                                for( String ip_addr_str : (Set<String>)dhcp_addresses.keySet() )
                                    ((Hashtable)dhcp_addresses.get(ip_addr_str)).put("ip_address", new String(ip_addr_str));
                            }

                            if( rootURL.equals("/ModifyNetwork") ) {
                                netdb.update((Hashtable)store.get("oldNetwork"), newNetwork);
                                session.setAttribute("info_msg", "SUCCESS: Network '"+handle_fqdn+"' has been updated.");
                            } else {
                                netdb.insert(newNetwork);
                                session.setAttribute("info_msg", "SUCCESS: Network '"+handle_fqdn+"' has been created.");
                            }

                            // reload my addr. spaces
                            setData(session, "myAddrSpaces", netdb.listMyAddrSpaces());

                            String fromFromPage = (String)store.get("fromFromPage");
                            response.sendRedirect("/Dispatch?Action=info&object=Network&fromPage="+
                                                  encode(fromFromPage!=null?fromFromPage:(String)store.get("fromPage"))+
                                                  "&handle="+handle_fqdn);
                            return;

                        } catch (NetDBException ex) {
                            ex.printStackTrace();
                            Vector errors = ex.getMessages();
                            String msg = "";
                            int esize = errors.size();
                            for( int j = 0; j < esize; j++ )
                                msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                            throw new Exception(msg);
                        }
                    } catch (Exception ex) {
                        ex.printStackTrace();
                        addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                    }
                } // end of 'if( action.equals("Save") )'

                if( action.equals("Add another") ) {
                    Hashtable location = new Hashtable();
                    location.put("name", " --None-- ");
                    location.put("site", "");
                    location.put("code", "");
                    Utils.get_hash_or_create(newNetwork, "locations").put(" --None-- "+getNextKey(), location);;
                }

            } else
                System.out.println("NOT READY");

            printOutModifyHTML(session, out, store, rootURL,
                               request.getParameter("PosX"),
                               request.getParameter("PosY"));
        } catch (Exception ex) {
            ex.printStackTrace();
            // if we are here something unexpected happened
            // we should never end up here
            /*
            String msg = "";
            try {
                if( lockTm != null )
                    netdb.unlock(Constants.Network, handle, lockTm);
            } catch (NetDBException exx) {
                Vector errors = exx.getMessages();
                int esize = errors.size();
                for( int i = 0; i < esize; i++ )
                    msg += "Error: "+(String)errors.elementAt(i)+(i<esize-1?"<br>\n":"");
            } catch (Exception exx) {
                msg += (msg.length()>0?"<br>\n":"")+"Error: "+exx.getMessage();
            }

            msg += (msg.length()>0?"<br>\n":"")+"Error: "+ex.getMessage();
            */
            response.sendRedirect((String)store.get("fromPage"));
        }
    } // end of doPost(...)

    public void printOutModifyHTML(HttpSession session, PrintWriter out, Hashtable store,
                                   String rootURL, String pos_x, String pos_y)
        throws Exception
    {
        String default_domain = (String)session.getAttribute("default_domain");

        String sessionKey = (String)store.get("sessionKey");
        Hashtable newNetwork = (Hashtable)store.get("newNetwork");
        String handle = (String)store.get("handle");
        String object = (String)store.get("object");
        Hashtable reserved_names = (Hashtable)store.get("reserved_names");
        Hashtable troubled_names = (Hashtable)store.get("troubled_names");
        Hashtable reserved_ips = nHash(store.get("reserved_ips"));
        Hashtable troubled_ips = (Hashtable)store.get("troubled_ips");

        if( Utils.nStr(session.getAttribute("Error.errorMsg")).trim().length() > 0  ) pos_x = pos_y = "0";

        store.put("PosX", pos_x);
        store.put("PosY", pos_y);

        modTop(session, out, object, handle, "Network", "Network", "hlpnetwork.html", sessionKey, rootURL);

        // network name
        Hashtable names = (Hashtable)newNetwork.get("names");

        java.util.List list = Arrays.asList(names.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();
        while( it.hasNext() ) {
            String key = (String)it.next();
            Hashtable name  = (Hashtable)names.get(key);
            String domain = ((String)name.get("domain")).trim();
            String displayName = ((String)name.get("name")).trim();
            String fqdn = displayName+"."+domain;
            if( displayName.length() > 0 && !domain.equalsIgnoreCase(default_domain) )
                displayName += "."+domain;

            out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                        "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font> Name</TD>\n"+
                        "<TD VALIGN='TOP'><INPUT ID='TextField_Network_Name' "+
                        (nameIn(Constants.NETWORK_NAME_TYPE, fqdn, troubled_names)?"CLASS='error_form_font' ":
                         (nameIn(Constants.NETWORK_NAME_TYPE, fqdn, reserved_names)?"CLASS='reserved_form_font' ":""))+
                        "TYPE='TEXT' SIZE='20' NAME='"+
                        encode("network_name_tf_"+key)+"' value='"+
                        Utils.htmlize(displayName, false)+"'>"+
                        //"<INPUT TYPE='SUBMIT' NAME='verify_network_names' VALUE='Verify Name, Alias and MX'>"+
                        "</TD></TR>\n");
        }

        out.println("</TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // Locations
        Hashtable net_locations = Utils.get_hash_or_create(newNetwork, "locations");
        if( net_locations.size() == 0 ) {
            Hashtable location = new Hashtable();
            location.put("name", " --None-- ");
            location.put("site", "");
            location.put("code", "");
            net_locations.put(" --None-- "+getNextKey(), location);
        }

        Object[] net_loc = sorted_keys(net_locations, name_cmp);
        Hashtable locations = (Hashtable)getData(session, "locations");
        Object[] loc = sorted_keys(locations, name_cmp);

        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Location</TD><TD>");

        for (int i = 0; i < net_loc.length; i++) {
            String net_location_name = Utils.nStr(((Hashtable)net_locations.get(net_loc[i])).get("name"));
            out.println("<SELECT ID='Selector_Location_"+i+"' SIZE=1 NAME='"+encode("location_"+i)+"'>\n"+
                        "<OPTION VALUE='"+encode(" --None-- ")+"'> --None-- ");
            for (int j = 0; j < loc.length; j++) {
                String name = Utils.nStr(((Hashtable)locations.get(loc[j])).get("location"));
                out.println("<OPTION VALUE='"+encode(name)+"'"+(name.equals(net_location_name)?" SELECTED":"")+">"+
                            Utils.htmlize(name, false));
            }
            out.println("</SELECT>"+(i==0?("&nbsp;&nbsp;<INPUT ID='Button_Add_Location' TYPE='SUBMIT' NAME='Action' "+
                                           "VALUE=\"Add another\">"):"")+
                        (i+1<net_loc.length?"<BR>":""));
        }
        out.println("</TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // NetDB Groups
        Hashtable groups = (Hashtable)newNetwork.get("groups");
        if( groups == null ) { // I know that network always have groups, but just in case
            groups = new Hashtable();
            newNetwork.put("groups", groups);
        }
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font>&nbsp;"+
                    "Groups<br>&nbsp;(1 or more)</TD>\n"+
                    "<TD VALIGN='TOP'>Selected (<em>uncheck to remove</em>):<BR>");
        list = Arrays.asList(groups.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        int index = 0;
        while( it.hasNext() ) {
            String group = (String)it.next();
            out.println("<INPUT ID='CheckBox_Network_Group_"+(index++)+"' TYPE='checkbox' CHECKED NAME='groups' value='"+
                        encode(group)+"'>"+Utils.htmlize(group.trim(), false)+"<BR>");
        }
        if( groups.size() > 0 )
            out.println("<BR>");
        out.println("<SELECT ID='Selector_Network_Groups' NAME='groups' multiple size='6'>");
        Hashtable myGroups = (Hashtable)getData(session, "myGroups");
        list = Arrays.asList(myGroups.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        while( it.hasNext() ) {
            String group = (String)it.next();
            out.println("<OPTION VALUE='"+encode(group)+"'>"+Utils.htmlize(group.trim(), false)+"</OPTION>");
        }
        out.println("</SELECT></TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // Comment
        String comment = (String)Utils.n2s((String)newNetwork.get("comment"));
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Comment</TD>\n"+
                    "<TD VALIGN='TOP'>"+
                    "<TEXTAREA ID='TextArea_Network_Comment' "+(newNetwork.get("bad comment")!=null?"CLASS='error_form_font' ":"")+
                    "NAME='comment' ROWS='3' COLS='60'>"+
                    Utils.htmlize(comment.trim(), false)+"</TEXTAREA></TD></TR>\n"+
                    "<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // DHCP Service
        String netDHCPService = (String)newNetwork.get("dhcp_service");
        Hashtable dhcp_services = nHash(getData(session, "dhcp_services"));
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;DHCP Service</TD>\n"+
                    "<TD VALIGN='TOP'>\n"+
                    "<SELECT ID='Selector_DHCP_Service' SIZE=1 NAME='dhcpService'>\n"+
                    "<OPTION VALUE='"+encode("--")+"'"+
                    (netDHCPService==null?" SELECTED":"")+">--</OPTION>");
        netDHCPService = (String)Utils.n2s(netDHCPService);
        for( String service : (Set<String>)dhcp_services.keySet() )
            out.println("<OPTION VALUE='"+encode(service)+"'"+
                        (netDHCPService.equals(service)?" SELECTED":"")+">"+
                        Utils.htmlize(service)+"</OPTION>");

        out.println("</SELECT>\n"+
                    "</TD></TR>\n"+
                    "<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // DHCP Options
        String dhcpTxt  = (String)newNetwork.get("dhcp_options_raw");
        if( dhcpTxt == null ) {
            dhcpTxt = "";
            Hashtable dhcp_options = nHash(newNetwork.get("dhcp_options"));
            java.util.List alist2 = Arrays.asList(dhcp_options.keySet().toArray());
            Collections.sort(alist2, name_cmp);
            Object[] oa2 = alist2.toArray();
            for (int j = 0; j < oa2.length; j++) {
                Hashtable dhcp_option_H = (Hashtable)dhcp_options.get(oa2[j]);

                String opt = (String)dhcp_option_H.get("option");
                String val = (String)dhcp_option_H.get("value");
                if (dhcpTxt.length() > 0)
                    dhcpTxt += "\n";
                if( (opt.trim().length()+val.trim().length()) > 0 )
                    dhcpTxt += opt.trim() + "=" + val.trim();
            }
        }

        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;DHCP Options</TD>\n"+
                    "<TD VALIGN='TOP'>"+
                    "<TEXTAREA ID='TextArea_Network_DHCP_Options' "+(newNetwork.get("bad dhcp options")!=null?"CLASS='error_form_font' ":"")+
                    "NAME='network_dhcp_options' ROWS='5' COLS='25'>"+
                    Utils.htmlize(dhcpTxt.trim(), false)+"</textarea></TD></TR>\n"+
                    "<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        String raw_ases_count = ((String)Utils.n2s(newNetwork.get("raw_ases_count"))).trim();
        out.println("<TR class='modHeader'><TD><B>"+
                        "&nbsp;IP Address Space(s)</B><BR>&nbsp;&nbsp;&nbsp;<em>uncheck to remove</em></TD>\n"+
                        "<TD ALIGN='RIGHT' VALIGN='TOP'>Count <INPUT ID='TextField_Addr_Space_Counter' "+
                    (newNetwork.get("ases_count_error")!=null?"CLASS='error_form_font' ":"")+"TYPE='TEXT' SIZE='4' NAME = 'ases_count' value ='"+
                    Utils.htmlize(raw_ases_count)+"'>&nbsp;&nbsp;&nbsp;&nbsp;<INPUT ID='Button_Add_Addr_Space' TYPE='SUBMIT' NAME='Action' "+
                        "VALUE=\"Add'l\">&nbsp;</TD></TR>");

        Hashtable ases = Utils.get_hash_or_create(newNetwork, "address_spaces");

        // address spaces
        for( String key : (Set<String>)ases.keySet() )
            ((Hashtable)ases.get(key)).put("key", key);

        list = Arrays.asList(ases.entrySet().toArray());
        Collections.sort(list, as_ip_cmp);
        it = list.iterator();
        index = -1;
        while( it.hasNext() ) {
            index++;
            Map.Entry entry = (Map.Entry)it.next();
            Hashtable as    = (Hashtable)entry.getValue();

            String addr_space_ip  = (String)as.get("addr_space_ip");
            Integer lowRes        = (Integer)as.get("low_res");
            Integer highRes       = (Integer)as.get("high_res");

            Integer prefix_length = (Integer)as.get("prefix_length");
            int prefixLength      = prefix_length==null?0:prefix_length.intValue();
            String asPrefix       = (String)as.get("addr_space_prefix");
            String raw_low_res    = (String)as.get("raw_low_res");
            String raw_high_res   = (String)as.get("raw_high_res");

            String raw_prefix_length = (String)as.get("raw_prefix_length");
            boolean prefix_length_error = false;
            try {
                if( raw_prefix_length != null && raw_prefix_length.trim().length() > 0 ) {
                    int prefix_int= Integer.parseInt(raw_prefix_length);
                    if( prefix_int < 2 || prefix_int > 30 )
                        prefix_length_error = true;
                }
            } catch (Exception ex) {
                prefix_length_error = true;
            }

            String raw_new_ip_addr = ((String)Utils.n2s(as.get("raw_new_ip_addr"))).trim();
            boolean new_ip_addr_error = false;
            if( raw_new_ip_addr.length() > 0 ) {
                try {
                    Hashtable parseRes = (new Parser()).parse(raw_new_ip_addr);
                    if( parseRes.get("ip_low") == null || parseRes.get("ip_high") == null )
                        new_ip_addr_error = true;
                } catch (Exception ex) {
                    new_ip_addr_error = true;
                }
            }

            String raw_new_ip_count = ((String)Utils.n2s(as.get("raw_new_ip_count"))).trim();
            boolean new_ip_count_error = false;
            if( raw_new_ip_count.length() > 0 ) {
                try {
                    int new_ip_count = Integer.parseInt(raw_new_ip_count);
                    if( new_ip_count <= 0 )
                        throw new Exception();
                } catch (Exception ex) {
                    new_ip_count_error = true;
                }
            }

            String key = (String)as.get("key");

            boolean raw_as_error = false;
            String raw_as_prefix = (String)as.get("raw_as_prefix");
            if( raw_as_prefix != null && raw_as_prefix.trim().length() > 0 ) {
                try {
                    Hashtable parseRes = (new Parser()).parse(raw_as_prefix);
                    String ip_low = (String)parseRes.get("ip_low");
                    String ip_high = (String)parseRes.get("ip_high");
                    if( ( ip_low == null || ip_high == null ) || ip_low.equals(ip_high))
                        raw_as_error = true;
                } catch (Exception ex) {
                    raw_as_error = true;
                }
            }
            out.println("<TR><TD></TD><TD><TABLE CELLSPACING='0' CELLPADDING='0'>"+
                        "<TR><TD NOWRAP>Address Space&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"+
                        "</TD><TD NOWRAP>Low Reserve&nbsp;</TD>"+
                        "<TD NOWRAP>High Reserve&nbsp;</TD><TD NOWRAP>"+
                        (raw_as_prefix==null?"Prefix&nbsp;":"")+"</TD></TR>"+
                        "<TR><TD NOWRAP><INPUT ID='CheckBox_Addr_Space_"+index+"' TYPE='checkbox' NAME='address_space' "+
                        "VALUE='"+encode(key)+
                        "' CHECKED> "+(raw_as_prefix!=null&&as.get("mute")==null?("<INPUT ID='TextField_Prefix_"+index+"' "+
                                                            (raw_as_error?"CLASS='error_form_font' ":"")+
                                                            "TYPE='TEXT' NAME='"+encode("raw_as_prefix "+key)+
                                                            "' SIZE='15' VALUE='"+Utils.htmlize(raw_as_prefix)+"'>"):
                                       asPrefix)+"&nbsp;</TD><TD NOWRAP><INPUT ID='TextField_Low_Res_"+index+"' "+
                        (as.get("bad low res")!=null?"CLASS='error_form_font' ":"")+"TYPE='TEXT' NAME='"+
                        encode("low_res "+key)+
                        "' SIZE='5' VALUE='"+Utils.htmlize(raw_low_res!=null?raw_low_res:(lowRes+""))+"'></TD><TD NOWRAP>"+
                        "<INPUT ID='TextField_High_Res_"+index+"' "+
                        (as.get("bad high res")!=null?"CLASS='error_form_font' ":"")+"TYPE='TEXT' NAME='"+
                        encode("high_res "+key)+
                        "' SIZE='5' VALUE='"+Utils.htmlize(raw_high_res!=null?raw_high_res:highRes+"")+
                        "'></TD><TD NOWRAP>"+

                        (raw_as_prefix==null?
                         ("<INPUT ID='TextField_New_Prefix_Length_"+index+"' "+
                          (prefix_length_error?"CLASS='error_form_font' ":"")+"TYPE='TEXT' NAME='"+
                          encode("prefix length "+key)+
                          "' SIZE='5' VALUE='"+Utils.htmlize(raw_prefix_length!=null?raw_prefix_length:"")+"'> "+

                          "<INPUT ID='Button_Split_Or_Join_"+index+"' TYPE='SUBMIT' NAME='"+
                          encode("split_join "+key)+
                          "' VALUE='Split/Join'>"):"")+

                        "</TD></TR>\n"+
                        "<TR><TD COLSPAN='4'>&nbsp;</TD></TR>\n<TR><TD COLSPAN='4'>"+
                        "<TABLE CLASS='net_address_space' WIDTH='100%'><TR><TD>Groups</TD>"+
                        "<TD>Selected (<em>uncheck to remove</em>)</TD></TR><TR><TD</TD><TD>");

            // Groups
            groups = (Hashtable)as.get("groups");
            if( groups == null ) {
                groups = new Hashtable();
                as.put("groups", groups);
            }

            java.util.List list2 = Arrays.asList(groups.keySet().toArray());
            Collections.sort(list2, name_cmp);
            Iterator it2  = list2.iterator();
            int sub_index = 0;
            while( it2.hasNext() ) {
                String group = (String)it2.next();
                out.println("<INPUT ID='CheckBox_Addr_Space_Group_"+index+"_"+(sub_index++)+"' TYPE='checkbox' CHECKED NAME='"+
                            encode("as_groups "+key)+
                            "' value='"+encode(group)+"'>"+
                            Utils.htmlize(group.trim(), false)+"<BR>");
            }

            if( groups.size() > 0 )
                out.println("<BR>");
            out.println("<SELECT ID='Selector_Addr_Space_Groups_"+index+"' NAME='"+encode("as_groups "+key)+"' multiple size='6'>");
            list2 = Arrays.asList(myGroups.keySet().toArray());
            Collections.sort(list2, name_cmp);
            it2 = list2.iterator();
            while( it2.hasNext() ) {
                String group = (String)it2.next();
                out.println("<OPTION VALUE='"+encode(group)+"'>"+Utils.htmlize(group.trim(), false)+"</OPTION>");
            }
            out.println("</SELECT></TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

            // DHCP Options
            boolean show_dhcp_options = false;
            dhcpTxt  = (String)as.get("dhcp_options_raw");
            if( dhcpTxt == null ) {
                dhcpTxt = "";
                Hashtable dhcp_options = nHash(as.get("dhcp_options"));
                java.util.List alist2 = Arrays.asList(dhcp_options.keySet().toArray());
                Collections.sort(alist2, name_cmp);
                Object[] oa2 = alist2.toArray();
                for (int j = 0; j < oa2.length; j++) {
                    Hashtable dhcp_option_H = (Hashtable)dhcp_options.get(oa2[j]);

                    String opt = (String)dhcp_option_H.get("option");
                    String val = (String)dhcp_option_H.get("value");
                    if (dhcpTxt.length() > 0)
                        dhcpTxt += "\n";
                    if( (opt.trim().length()+val.trim().length()) > 0 )
                        dhcpTxt += opt.trim() + "=" + val.trim();
                }
                if( oa2.length > 0 )
                    show_dhcp_options = true;

            } else
                show_dhcp_options = true;

            out.println("<TR><TD VALIGN='TOP'>DHCP Options</TD><TD>"+
                        (show_dhcp_options?
                         "<TEXTAREA ID='TextArea_Addr_Space_DHCP_Options_"+index+"' "+
                         (as.get("bad dhcp options")!=null?"CLASS='error_form_font' ":"")+"NAME='"+
                         encode("dhcp_options "+key)+
                         "' rows=2 cols=30>"+
                         Utils.htmlize(dhcpTxt.trim(), false)+"</TEXTAREA>":
                         "<INPUT ID='Button_Display_DHCP_"+index+"' TYPE='SUBMIT' NAME='"+
                         encode("display_dhcp_options "+key)+
                         "' VALUE='Display'></TD></TR>")+
                        "<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                        "<TR><TD>Comment</TD><TD><INPUT ID='TextField_Addr_Space_Comment_"+index+"' "+
                        (as.get("bad comment")!=null?"CLASS='error_form_font' ":"")+
                        "TYPE='TEXT' SIZE='40' NAME="+
                        encode("asComment "+key)+" value='"+
                        Utils.htmlize((String)Utils.n2s(as.get("comment")), false)+"'></TD></TR>\n"+
                        "<TR><TD COLSPAN='2'>&nbsp;</TD></TR>");

            Hashtable dhcp_addresses = nHash(as.get("dhcp_addresses"));
            if( raw_as_prefix == null || dhcp_addresses.size() > 0 )
            out.println("<TR><TD COLSPAN='2' NOWRAP>Dynamic DHCP addresses</TD></TR>");

            // DHCP ip addresses
            list2 = Arrays.asList(dhcp_addresses.keySet().toArray());
            Collections.sort(list2, ip_cmp);
            Object[] ip = list2.toArray();
            for (int j = 0; j < ip.length; j++) {
                String ipStr = (String)ip[j];
                Hashtable ipH = (Hashtable)dhcp_addresses.get(ipStr);

                Boolean active = (Boolean)ipH.get("active?");
                if( active == null )
                    active = Boolean.FALSE;

                String ipNameStr = null;
                Hashtable ipNames = nHash(ipH.get("names"));
                boolean dup_name = false;
                String fqdn = "";
                for( Hashtable ipName : (Collection<Hashtable>)ipNames.values() ) {
                    ipNameStr = ((String)ipName.get("name")).trim();
                    String domain = ((String)ipName.get("domain")).trim();
                    fqdn = ipNameStr+"."+domain;
                    if( ipNameStr.length() > 0 && !domain.equalsIgnoreCase(default_domain) )
                        ipNameStr += "."+domain;
                    if( ipName.get("error") != null )
                        dup_name = true;
                }
                if( ipNameStr == null )
                    ipNameStr = "";

                out.println("<TR><TD COLSPAN='2'>"+
                            "<INPUT ID='CheckBox_Ip_Addr_"+index+"_"+j+"' "+
                            (raw_as_prefix!=null?"DISABLED ":"")+"TYPE ='CHECKBOX' VALUE ='"+j+"' NAME='"+encode("ip cb ["+key+"]")+"' CHECKED>"+
                            " <INPUT ID='TextField_Ip_Addr_"+index+"_"+j+"' "+(troubled_ips.get(ipStr.trim().toLowerCase())!=null?"CLASS='error_form_font' ":
                                        (reserved_ips.get(ipStr)!=null?"CLASS='reserved_form_font' ":""))+
                            (raw_as_prefix!=null?"READONLY CLASS='mute_form' ":"")+"TYPE='TEXT' SIZE='15' NAME = '"+encode("ip addr ["+key+"] ["+j+"]")+"' VALUE ='"+
                            Utils.htmlize(ipStr)+"'>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"+
                            "Name <INPUT ID='TextField_Ip_Addr_Name_"+index+"_"+j+"' "+(raw_as_prefix!=null?"READONLY CLASS='mute_form' ":"")+
                            (dup_name||nameIn(Constants.ADDRESS_NAME_TYPE, fqdn, troubled_names)?"CLASS='error_form_font' ":
                             (nameIn(Constants.ADDRESS_NAME_TYPE, fqdn, reserved_names)?"CLASS='reserved_form_font' ":""))+
                            "TYPE='TEXT' SIZE='15' NAME = '"+encode("ip name ["+key+"] ["+j+"]")+"' VALUE ='"+
                            Utils.htmlize(ipNameStr)+"'> "+
                            "<INPUT ID='CheckBox_Ip_Addr_Active_"+index+"_"+j+"' "+(raw_as_prefix!=null?"DISABLED ":"")+
                            "TYPE ='CHECKBOX' VALUE ='YES' NAME='"+encode("ip status ["+key+"] ["+j+"]")+"' "+
                            (active.booleanValue()?" CHECKED":"")+"> Active"+
                            "<BR>&nbsp;</TD></TR>");
            }

            if( raw_as_prefix == null )
                out.println("<TR><TD COLSPAN='2' NOWRAP>"+
                            "IP Addr <INPUT ID='TextField_New_Ip_Addr_"+index+"' "+
                            (new_ip_addr_error?"CLASS='error_form_font' ":"")+
                            "TYPE='TEXT' SIZE='15' NAME = '"+
                            encode("new ip addr "+key)+"' value ='"+Utils.htmlize(raw_new_ip_addr)+"'>"+
                            " <INPUT ID='CheckBox_New_Ip_Addr_Active_"+index+"' TYPE ='CHECKBOX' VALUE ='YES' NAME='"+
                            encode("new ip status "+key)+"'"+(as.get("raw_new_ip_status_inactive")==null?" CHECKED":"")+"> Active"+
                            "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Count <INPUT ID='TextField_New_Ip_Addr_Count_"+index+"' "+
                            (new_ip_count_error?"CLASS='error_form_font' ":"")+"TYPE='TEXT' SIZE='4' NAME = '"+
                            encode("new ip count "+key)+"' value ='"+Utils.htmlize(raw_new_ip_count)+"'>"+
                            "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<INPUT ID='Button_Verify_New_Ip_Addr_"+index+
                            "' TYPE='SUBMIT' NAME='"+
                            encode("as verify ip "+key)+"' VALUE=\"Verify\">"+
                            "</TD></TR>");

            out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                        "</TABLE>"+
                        "</TD></TR></TABLE></TD></TR>"+
                        "<TR><TD COLSPAN='2'><HR NOSHADE SIZE=1></TD></TR>");
        }

        // created by
        Hashtable person = (Hashtable)newNetwork.get("created.by");
        if( person != null ) {
            String dirID = (String)person.get("DirID");
            boolean active = activep(person);

            String byStr = (active?"<A HREF=\""+
                            System.getProperty("stanfordwho.url")+dirID.trim()+"\">":"");
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += (active?"</A>":"")+" ("+newNetwork.get("created").toString().substring(0, 16)+")";
            out.println("<TR><TD VALIGN='TOP'>&nbsp;Created By</TD>\n"+
                        "<TD VALIGN='TOP'>"+
                        byStr+"</TD></TR>");

            // modified by
            person = (Hashtable)newNetwork.get("mod.by");
            dirID = (String)person.get("DirID");
            active = activep(person);

            byStr = (active?"<A HREF=\""+
                     System.getProperty("stanfordwho.url")+dirID.trim()+"\">":"");
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += (active?"</A>":"")+" ("+newNetwork.get("modified").toString().substring(0, 16)+")";
            out.println("<TR><TD VALIGN='TOP'>&nbsp;Last Modified By</TD>\n"+
                        "<TD VALIGN='TOP'>"+
                        byStr+"</TD></TR>");
        }

        modBottom(out);
    } // end of printOutModifyHTML(...)

    private void collectNamesAndIPs(Hashtable netHash, Hashtable allNames, Hashtable allIPs, Hashtable parents)
        throws Exception
    {
        // first network names
        Hashtable netNames = (Hashtable)netHash.get("names");
        if( netNames == null ) {
            netNames = new Hashtable();
            netHash.put("names", netNames);
        }

        for( Hashtable nameH : (Collection<Hashtable>)netNames.values() ) {
            String namePart = ((String)nameH.get("name")).trim();
            if( namePart.length() == 0 )
                continue;
            String fqdnName = namePart+"."+((String)nameH.get("domain")).trim();
            String fqdnLower = fqdnName.toLowerCase();
            allNames.put("NET NAME "+fqdnLower, nameH);
        }

        // now address spaces
        for( Hashtable as : (Collection<Hashtable>)nHash(netHash.get("address_spaces")).values() ) {
            Hashtable ips = nHash(as.get("dhcp_addresses"));

            for( String ipAddress : (Set<String>)ips.keySet() ) {
                String ipStr = ipAddress.trim(); // trim it just in case
                Hashtable ipH = (Hashtable)ips.get(ipAddress);

                String parent_key = getNextKey();
                parents.put(parent_key, ips);
                ipH.put("parent", parent_key);
                allIPs.put(ipStr, ipH);

                Hashtable ipNames = Utils.nHash(ipH.get("names"));
                for( Hashtable nameH : (Collection<Hashtable>)ipNames.values() ) {
                    String namePart = ((String)nameH.get("name")).trim();
                    if( namePart.length() == 0 )
                        continue;
                    String fqdnName = namePart+"."+((String)nameH.get("domain")).trim();
                    String fqdnLower = fqdnName.toLowerCase();
                    allNames.put("IP NAME " + fqdnLower, nameH);
                }
            }
        }
    } // end of collectNamesAndIPs(...)

    private Hashtable splitAS(NetDBpi netdb, Hashtable masterAS, int target_prefix_length)
        throws NetDBException, Exception
    {
        Hashtable result = nHash(netdb.splitAS(masterAS));
        fillIPsInASes(result);
        Object[] keys = result.keySet().toArray();
        for( int i = 0; i < keys.length; i++ ) {
            String key = (String)keys[i];
            Hashtable as = (Hashtable)result.get(key);
            as.put("raw_as_prefix", as.get("addr_space_prefix")); // putting this one indicates that it is a new address space
            as.put("mute", "");
            int prefix_length = ((Integer)as.get("prefix_length")).intValue();
            if( target_prefix_length > prefix_length ) {
                Hashtable result2 = nHash(splitAS(netdb, as, target_prefix_length));
                result.remove(key);
                Utils.appendHash(result, result2);
            }
        }
        return result;
    }

    public void fillIPsInASes(Hashtable ases) {
        for( Hashtable as : (Collection<Hashtable>)ases.values() )
            fillIPsInAS(as);
    }

    public void fillIPsInAS(Hashtable as) {
        fillIPs(nHash(as.get("dhcp_addresses")));
    }

    public void fillIPs(Hashtable ips) {
        for( String ipStr : (Set<String>)ips.keySet() )
            ((Hashtable)ips.get(ipStr)).put("ip_address", ipStr);
    }

    public void purgeEmptyASes(Hashtable net) {
        Hashtable ases = nHash(net.get("address_spaces"));
        Vector keys_to_delete = new Vector();
        for( Object key : ases.keySet() ) {
            Hashtable as = (Hashtable)ases.get(key);
            String raw_as_prefix = (String)as.get("raw_as_prefix");
            if( raw_as_prefix != null && raw_as_prefix.trim().length() == 0 )
                keys_to_delete.addElement(key);
        }

        for( Object key : keys_to_delete ) ases.remove(key);
    } // purgeEmptyASes(...)
}
