/************************************************************************
 *
 * ModifyNode.java - node modify and insert pages
 *
 * $Id: ModifyNode.java,v 1.30 2007/03/29 18:31:55 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2004, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.sql.*;
import java.net.*;
import java.util.*;
import java.text.*;  // SimpleDateFormat
import java.util.regex.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import su.netdb.parser.*;
import stanford.netdb.utils.*;

import static stanford.netdb.utils.Utils.hash;

public class ModifyNode extends ModifyPage {

    public ModifyNode() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = hash();
        store.put("type", "Node");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        try {
            PrintWriter out = response.getWriter();
            String rootURL = request.getServletPath();

            Hashtable registrar = hash();
            store.put("registrar", registrar);

            Hashtable troubled_ips = hash();
            store.put("troubled_ips", troubled_ips);

            Hashtable troubled_hws = hash();
            store.put("troubled_hws", troubled_hws);

            Hashtable reserved_ips = hash();
            store.put("reserved_ips", reserved_ips);

            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String netID = (String)session.getAttribute("NetID");
            String default_domain =
                (String)session.getAttribute("default_domain");

            // obtain node key attribute from session
            String object = request.getParameter("handle");
            String handle = null;
            Timestamp lockTm = null;
            int oid = -1;
            boolean numeric_handle = false;
            if( object != null ) {
                try {
                    oid = Integer.parseInt(object);
                    numeric_handle = true;
                } catch(Exception ex) { }

                handle = new String(object);
                store.put("handle", handle);
                if( rootURL.equals("/ModifyNode") ) {
                    // lock
                    try {
                        if( numeric_handle )
                            lockTm = netdb.lock(Constants.NODE, oid);
                        else
                            lockTm = netdb.lock(Constants.NODE, handle);

                        if( lockTm == null )
                            throw new Exception("Failed to lock node "+handle);

                    } catch (Exception ex) {
                        ex.printStackTrace();
                        session.setAttribute("Error.errorMsg", ex.getMessage());
                        response.sendRedirect((String)store.get("fromPage"));
                        return;
                    }
                    store.put("lockTm", lockTm);
                }
            }

            Hashtable oldNode = null;
            if( rootURL.equals("/CreateNode") ) {

                if( !netdb.checkAccess(Constants.NODE) )
                    throw new Exception("User '"+session.getAttribute("NetID")+
                                        "' can not create nodes.");

                if( handle == null ) { // creating clean node
                    Hashtable user = (Hashtable)session.getAttribute("user");

                    oldNode = hash("object", "node",
                                   "types",    hash(),
                                   "names",    hash(getNextKey(), hash("name", "", "domain", "")),
                                   "admins",   hash(),
                                   "groups",   hash(user.get("default_group"), ""),
                                   "location", "",
                                   "room",     "",
                                   "make",     "",
                                   "model",    "",
                                   "state",    Utils.n2s(nHash(getData(session, "global.defaults")).get("DefaultState")),
                                   "oses",     hash(" --None--"+getNextKey(), ""));


                    // set department - first oficial department from user
                    String depName = "";;
                    Hashtable departments = nHash(user.get("departments"));
                    Iterator it = Utils.sortedKeys(departments, name_cmp);
                    while( it.hasNext() ) {
                        String dep = ((String)it.next()).trim();
                        Hashtable depH = (Hashtable)departments.get(dep);
                        Boolean official = (Boolean)depH.get("official?");
                        if( official != null && official.booleanValue() ) {
                            depName = dep;
                            break;
                        }
                    }
                    oldNode.put("dept", depName);

                    String default_addr_space =
                        (String)user.get("default_addr_space");
                    Hashtable interfaces = hash();
                    Hashtable intfc = hash("roaming?",     Boolean.FALSE,
                                           "use-dhcp?",    Boolean.FALSE,
                                           "dhcp-options", hash(),
                                           "names",        hash());
                    interfaces.put(getNextKey(), intfc);
                    oldNode.put("interfaces", interfaces);
                    Hashtable ips = hash();
                    intfc.put("ips", ips);

                    if( default_addr_space != null &&
                        !reserveIPandName(session, netdb, ips, store,
                                          new Boolean(true), null,
                                          default_addr_space, null, 0,
                                          "NodeCreate", false, true) )
                        intfc.put("show select", "");

                } else { // creating node from template
                    if( numeric_handle )
                        oldNode = netdb.getInfo(Constants.NODE, oid);
                    else
                        oldNode = netdb.getInfo(Constants.NODE, object);
                    if (oldNode == null || oldNode.get("id") == null ) {
                        session.setAttribute("Error.errorMsg",
                                             "Error: No info found for node "+
                                             object);
                        response.sendRedirect((String)store.get("fromPage"));
                        return;
                    }

                    // clean up created.by and mod.by
                    oldNode.remove("created.by");
                    oldNode.remove("mod.by");

                    // clean up names
                    Hashtable names = hash();
                    Hashtable name = hash("name", "", "domain", "");
                    names.put(getNextKey(), name);
                    oldNode.put("names", names);

                    // clean up comment
                    oldNode.put("comment", "");

                    // reset tags values. If source node is of type "Template" then we inherit tags and its values
                    if( find_by_key(nHash(oldNode.get("types")), "extkey", "Template", "type") == null ) {
                        Hashtable tags = nHash(oldNode.get("tags"));
                        for( Object key : tags.keySet() )
                            tags.put(key, "");
                    }

                    // clean up ipc addresses
                    if( oldNode.get("ipc_addresses") != null )
                        oldNode.put("ipc_addresses", hash());

                    // AS we will use to reserve ip address
                    String as_to_reserve = null;
                    // or starting ip address
                    String ip_start = null;

                    // fiddle with types
                    if( oldNode.get("types") == null )
                        oldNode.put("types", hash());

                    if( find_by_key(nHash(oldNode.get("types")), "extkey", "Template", "type") != null ){
                        nHash(oldNode.get("types")).clear();
                        as_to_reserve = (String)oldNode.remove("addr_space");
                    }

                    // collect ips from all interfaces
                    Hashtable allIPs = hash();
                    Hashtable interfaces = (Hashtable)oldNode.get("interfaces");
                    if( interfaces == null ) {
                        interfaces = hash();
                        oldNode.put("interfaces", interfaces);
                    }

                    for( Hashtable intfc : (Collection<Hashtable>)interfaces.values() ) {
                        Hashtable ips = nHash(intfc.get("ips"));
                        for( Object ip : ips.keySet() ) {
                            allIPs.put(ip,
                                       hash("active?", (Boolean)((Hashtable)ips.get(ip)).get("active?"),
                                            "ip",       ip));
                        }
                    }

                    // now pick up first ip
                    Iterator it = Utils.sortedEntries(allIPs, full_ip_cmp);
                    while( it.hasNext() ) {
                        ip_start = (String)((Hashtable)((Map.Entry)it.next()).getValue()).get("ip");
                        break;
                    }
                    interfaces.clear();

                    interfaces = hash();
                    Hashtable intfc = hash("roaming?",     Boolean.FALSE,
                                           "use-dhcp?",    Boolean.FALSE,
                                           "dhcp-options", hash(),
                                           "names",        hash());
                    interfaces.put(getNextKey(), intfc);
                    oldNode.put("interfaces", interfaces);
                    Hashtable ips = hash();
                    intfc.put("ips", ips);

                    // find corresponing address space
                    if( ip_start != null )
                        as_to_reserve = netdb.findIPAddrSpace(ip_start);

                    if( as_to_reserve != null && as_to_reserve.trim().length() > 0 &&
                        !reserveIPandName(session, netdb, ips, store, new Boolean(true),
                                          as_to_reserve.substring(0, as_to_reserve.indexOf("/")),
                                          as_to_reserve, null, 0, false) )
                        intfc.put("show select", "");
                } // end of 'creating node from template'

            } else { // node modify
                if( numeric_handle )
                    oldNode = netdb.getInfo(Constants.NODE, oid);
                else
                    oldNode = netdb.getInfo(Constants.NODE, object);
                if (oldNode == null || oldNode.get("id") == null ) {
                    session.setAttribute("Error.errorMsg",
                                         "Error: No info found for node "+object);
                    response.sendRedirect((String)store.get("fromPage"));
                    return;
                }

                // extact name with original case
                Hashtable names = nHash(oldNode.get("names"));
                for( Object key : names.keySet() ) {
                    Hashtable nameH = (Hashtable)names.get(key);
                    String name     = key.toString().trim().toLowerCase();
                    if( numeric_handle ) { // first name will be handle
                        object = nameH.get("name")+"."+nameH.get("domain");
                        break;
                    }
                    if( name.equals(object.trim().toLowerCase()) ) {
                        object = nameH.get("name")+"."+nameH.get("domain");
                        break;
                    }
                }
            }

            // register all names as reserved and committed
            registerNames(registrar, oldNode);

            if( lockTm != null )
                oldNode.put("locked", lockTm);

            oldNode.put("no ip format check", "");
            Hashtable newNode = Utils.copyHash(oldNode);
            if( newNode.get("types") == null ) newNode.put("types", hash());

            store.put("oldNode", oldNode);
            store.put("newNode", newNode);
            store.put("oldNewNode", Utils.copyHash(newNode));

            // compress node name by stripping default domain
            object = compressName(object, default_domain);
            store.put("object", object);

            printOutModifyHTML(session, out, store, rootURL, "0", "0");

        } catch (Exception ex) {
            // display error page
            ex.printStackTrace();
            String errMsg = ex.getMessage();
            errMsg = (errMsg == null ? "???" : errMsg);
            session.setAttribute("Error.errorMsg", errMsg);
            response.sendRedirect((String)store.get("fromPage"));
            return;
        }
    } // end of doGet(...)

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        HttpSession session = begin(request, response, null);
        if( session == null )
            return;

        String sessionKey       = decode(request.getParameter("sessionKey"));
        String default_domain   = (String)session.getAttribute("default_domain");
        Hashtable store         = (Hashtable)session.getAttribute(sessionKey);
        Hashtable registrar     = (Hashtable)store.get("registrar");
        NetDBpi netdb           = (NetDBpi)session.getAttribute("netdb");
        String handle           = (String)store.get("handle");
        Timestamp lockTm        = (Timestamp)store.get("lockTm");
        Hashtable newNode       = (Hashtable)store.get("newNode");
        Hashtable oldNewNode    = (Hashtable)store.get("oldNewNode");
        String target           = null;
        String last_view_name   = "";
        String next_view_name   = "";
        int custom_fields_total = ((Integer)getData(session, "custom_fields_total")).intValue();
        Hashtable my_node_types = nHash(getData(session, "my node types"));
        String[] targets        = new String[0];
        try {
            PrintWriter out = response.getWriter();
            String rootURL = request.getServletPath();

            if( request.getParameter("ready") != null ) {

                Hashtable troubled_ips = (Hashtable)store.get("troubled_ips");
                if( troubled_ips == null ) {
                    troubled_ips = hash();
                    store.put("troubled_ips", troubled_ips);
                }

                Hashtable troubled_hws = (Hashtable)store.get("troubled_hws");
                if( troubled_hws == null ) {
                    troubled_hws = hash();
                    store.put("troubled_hws", troubled_hws);
                }

                String action = request.getParameter("Action");
                String superAction = Utils.nStr(request.getParameter("SuperAction"));
                if( superAction.length() > 0 ) action = superAction;

                if( request.getParameter("verify_node_users") != null )
                    action = "verify_node_users";

                else if( request.getParameter("verify_node_admins") != null )
                    action = "verify_node_admins";

                else if( request.getParameter("add_node_name") != null )
                    action = "add_node_name";

                else if( request.getParameter("verify_node_names") != null )
                    action = "verify_node_names";

                else if( request.getParameter("new_model") != null )
                    action = "new_model";

                else if( request.getParameter("new_os") != null )
                    action = "new_os";

                else if( request.getParameter("add_os") != null )
                    action = "add_os";

                else if( request.getParameter("ipc_add") != null )
                    action = "add ipc";

                else if( request.getParameter("ipc_verify") != null )
                    action = "verify ipc";

                else if( request.getParameter("ipc_reserve_and_add") != null )
                    action = "reserve ipc and add";

                else if( request.getParameter("ipc_reserve") != null )
                    action = "reserve ipc";

                else if( request.getParameter("View") != null ) {
                    action = "View";
                    target = request.getParameter("View");
                }

                else {
                    for(;;) {

                        target = getParameterNameStaringWith(request, "ip reserve ");
                        if( target != null ) {
                            action = "reserve ip";
                            targets = new String[1];
                            targets[0] = target.substring(11);
                            break;
                        }

                        target = getParameterNameStaringWith(request, "ip reserve&add ");
                        if( target != null ) {
                            action = "reserve ip and add";
                            targets = new String[1];
                            targets[0] = target.substring(15);
                            break;
                        }

                        target = getParameterNameStaringWith(request, "ip add ");
                        if( target != null ) {
                            action = "add ip";
                            target = target.substring(7);
                            break;
                        }

                        target = getParameterNameStaringWith(request, "ip verify ");
                        if( target != null ) {
                            action = "verify names and ips";
                            target = target.substring(10);
                            break;
                        }

                        target = getParameterNameStaringWith(request, "add_alias_name_");
                        if( target != null ) {
                            action = "add_alias_name";
                            target = target.substring(15);
                            break;
                        }

                        target = getParameterNameStaringWith(request, "add_mx_name_");
                        if( target != null ) {
                            action = "add_mx_name";
                            target = target.substring(12);
                            break;
                        }

                        target = getParameterNameStaringWith(request, "hw_verify_name");
                        if( target != null ) {
                            action = "hw_verify_name";
                            target = target.substring(15);
                            break;
                        }

                        target = getParameterNameStaringWith(request, "display_dhcp_options");
                        if( target != null ) {
                            action = "display_dhcp_options";
                            target = target.substring(21);
                            break;
                        }

                        break;
                    }
                }

                if( action == null )
                    action = "";

                if( action.equals("verify ipc") ) {
                    session.setAttribute("Error.errorMsg",
                                         "Error: Verify ipc addresses action is not implemented yet");

                } else if( action.equals("verify names and ips") ) {
                    session.setAttribute("Error.errorMsg",
                                         "Error: Verify names and ip addresses action is not implemented yet");

                } else if( action.equals("hw_verify_name") ) {
                    session.setAttribute("Error.errorMsg",
                                         "Error: Verify hw addr. name action is not implemented yet");

                } else if( action.equals("Create Model") ) {
                    Hashtable newModel = (Hashtable)store.get("newModel");
                    newModel.put("model", ((String)Utils.n2s(request.getParameter("model"))).trim());
                    newModel.put("make", ((String)Utils.n2s(request.getParameter("make"))).trim());
                    //String type = decode(request.getParameter("type")).trim();
                    //newModel.put("type", type.equals("--None--")?"":type);
                    newModel.put("type", "");
                    try {
                        try {
                            Utils.validate(Constants.MODEL, newModel, Constants.NARROW);

                            netdb.insert(newModel);
                            session.setAttribute("info_msg", "SUCCESS: Model has been created");
                            store.remove("new_model");

                            // reload list of models
                            setData(session, "models", netdb.getInfo(Constants.MODEL, 0));

                            // now add newly created model and its make
                            newNode.put("make", newModel.get("make"));
                            newNode.put("model", newModel.get("model"));

                        } catch (NetDBException ex) {
                            Vector errors = ex.getMessages();
                            String msg = "";
                            int esize = errors.size();
                            for( int j = 0; j < esize; j++ )
                                msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                            throw new Exception(msg);
                        }
                    } catch (Exception ex) {
                        addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                    }

                } else if( action.equals("Create OS") ) {
                    Hashtable newOS = hash("object",  "os",
                                           "name",    ((String)Utils.n2s(request.getParameter("osName"))).trim(),
                                           "comment", ((String)Utils.n2s(request.getParameter("osComment"))).trim());
                    store.put("newOS", newOS);

                    try {
                        try {
                            Utils.validate(Constants.OS, newOS, Constants.NARROW);

                            netdb.insert(newOS);
                            session.setAttribute("info_msg", "SUCCESS: OS has been created");
                            store.remove("new_os");

                            // reload list of oses
                            setData(session, "oses", netdb.getInfo(Constants.OS, 0));

                            // now add newly created os
                            Hashtable oses = (Hashtable)newNode.get("oses");
                            if( oses == null ) {
                                oses = hash();
                                newNode.put("oses", oses);
                            }
                            oses.put(newOS.get("name"), "");

                        } catch (NetDBException ex) {
                            Vector errors = ex.getMessages();
                            String msg = "";
                            int esize = errors.size();
                            for( int j = 0; j < esize; j++ )
                                msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                            throw new Exception(msg);
                        }
                    } catch (Exception ex) {
                        addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                    }

                } else if( action.equals("new_os")  ) {
                    Hashtable newOS = hash("object",  "os",
                                           "name",    ((String)Utils.n2s(request.getParameter("osName"))).trim(),
                                           "comment", ((String)Utils.n2s(request.getParameter("osComment"))).trim());
                    store.put("newOS", newOS);
                    store.put("new_os", "");

                } else if( action.equals("new_model")  ) {
                    Hashtable newModel =
                        hash("model",  "",
                             "make",   "",
                             "type",   "",
                             "object", "model");
                    store.put("newModel", newModel);
                    store.put("new_model", "");

                    Hashtable makes = netdb.getInfo(Constants.MAKE, "");
                    store.put("makes", makes);

                } else if( action.equals("verify_node_names") ) {
                    session.setAttribute("Error.errorMsg",
                                         "Error: Verify names action is not implemented yet");

                } else if( action.equals("Save") ) {

                } else if( action.equals("Go") ) {
                    // change custom view
                    next_view_name = decode((String)Utils.n2s(request.getParameter("custom_view"))).trim();
                    last_view_name = (String)newNode.get("enabled view");
                    if( last_view_name == null ) last_view_name = next_view_name;

                    if( next_view_name.equals("--") )
                        newNode.remove("enabled view");
                    else {
                        newNode.put("enabled view", next_view_name);
                        try {
                            pick_up_tags_separated(newNode, request, session);
                            parse_tags_for_custom_view(newNode,
                                                       nHash(nHash(getData(session, "custom.node.views")).get(next_view_name)),
                                                       next_view_name, custom_fields_total, false);
                        } catch (Exception ex) {
                            last_view_name = null;
                            next_view_name = null;
                            newNode.remove("enabled view");
                            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                        }
                    }

                } else if( action.equals("View") ) {

                    if( target.startsWith("[ ]") ) {
                        // enable the view
                        last_view_name = target.substring(4);
                        newNode.put("enabled view", last_view_name);
                        try {
                            pick_up_tags_separated(newNode, request, session);
                            parse_tags_for_custom_view(newNode,
                                                       nHash(nHash(getData(session, "custom.node.views")).get(last_view_name)),
                                                       last_view_name, custom_fields_total, false);
                        } catch (Exception ex) {
                            last_view_name = null;
                            newNode.remove("enabled view");
                            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                        }
                    } else
                        // disable the view
                        last_view_name = (String)newNode.remove("enabled view");

                } else if( action.equals("Cancel") ) {
                    if( store.get("new_os") != null )
                        store.remove("new_os");
                    else if( store.get("new_model") != null )
                        store.remove("new_model");
                    else {
                        try {
                            try {
                                if( lockTm != null ) {
                                    if( Utils.isNumber(handle) )
                                        netdb.unlock(Constants.NODE, Integer.parseInt(handle), lockTm);
                                    else
                                        netdb.unlock(Constants.NODE, handle, lockTm);
                                }
                            } catch (NetDBException ex) { }

                            try {
                                Hashtable reserved_ips = nHash(store.get("reserved_ips"));

                                for( String key : (Set<String>)reserved_ips.keySet() ) {
                                    try {
                                        netdb.cancelIP(key);
                                    } catch (NetDBException ex) {
                                        Vector errors = ex.getMessages();
                                        String msg = "";
                                        int esize = errors.size();
                                        for( int i = 0; i < esize; i++ )
                                            msg += "Error: "+(String)errors.elementAt(i)+(i<esize-1?"\n":"");
                                        throw new Exception(msg);
                                    }
                                }

                                Hashtable reserved_names = nHash(store.get("reserved_names"));
                                for( Object key : reserved_names.keySet() ) {
                                    Hashtable nameStore = (Hashtable)reserved_names.get(key);
                                    try {
                                        netdb.cancelName((String)nameStore.get("fqdn"), (String)nameStore.get("type"));
                                    } catch (NetDBException ex) {
                                        Vector errors = ex.getMessages();
                                        String msg = "";
                                        int esize = errors.size();
                                        for( int i = 0; i < esize; i++ )
                                            msg += "Error: "+(String)errors.elementAt(i)+(i<esize-1?"\n":"");
                                        throw new Exception(msg);
                                    }
                                }

                            } catch (NetDBException ex) {
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int i = 0; i < esize; i++ )
                                    msg += "Error: "+(String)errors.elementAt(i)+(i<esize-1?"\n":"");
                                throw new Exception(msg);
                            }
                        } catch (Exception ex) {
                            session.setAttribute("Error.errorMsg", ex.getMessage());
                        }
                        response.sendRedirect((String)store.get("fromPage"));
                        return;
                    }
                }

                // reset custom view, when switching node types
                if( action.matches("^\\[(  |X)\\]\\s.*$") )
                    last_view_name = (String)newNode.remove("enabled view");


                // add node type
                if( action.startsWith("[  ] ") ) {
                    String type_to_add = Utils.nStr(find_by_key(my_node_types, "type", action.substring(5), "type"));
                    ((Hashtable)newNode.get("types")).put(type_to_add, my_node_types.get(type_to_add));

                    if( type_to_add.equals("Template") ) {
                        // remove all aliases and mxes from a node
                        Hashtable names = (Hashtable)newNode.get("names");

                        for( Object key : names.keySet() ) {
                            Hashtable nameH = (Hashtable)names.get(key);
                            nameH.put("aliases", hash());
                            nameH.put("mxs",     hash());
                        }

                        // put appropriate def. addr. space
                        Hashtable user = (Hashtable)session.getAttribute("user");
                        String default_addr_space = (String)user.get("default_addr_space");
                        if( default_addr_space != null && default_addr_space.trim().length() > 0 )
                            newNode.put("addr_space", default_addr_space.trim());
                    }
                }

                // remove node type
                if( action.startsWith("[X] ") ) {
                    String type_to_remove = Utils.nStr(find_by_key(my_node_types, "type", action.substring(4), "type"));
                    ((Hashtable)newNode.get("types")).remove(type_to_remove);

                    if( Utils.nStr(nHash(my_node_types.get(type_to_remove)).get("extkey")).equals("Advanced")  ) {
                        // remove all node names except one
                        Hashtable names = (Hashtable)newNode.get("names");
                        java.util.List list = Arrays.asList(names.keySet().toArray());
                        Collections.sort(list, name_cmp);
                        Iterator it = list.iterator();
                        int index = 0;
                        String key = null;
                        Hashtable nameH = null;
                        while( it.hasNext() ) {
                            key = (String)it.next();
                            nameH = (Hashtable)names.get(key);
                            if( ((String)nameH.get("name")).trim().length() != 0 )
                                break;
                        }
                        names.clear();
                        names.put(key, nameH);
                    }
                }

                if( !action.equals("Reset") && !action.equals("Cancel") &&
                    !action.equals("Create OS") && !action.equals("Create Model") ) {
                    // pick up new data
                    // uncomment here to view all POST data received
                    /*
                      System.out.println("\nparameteres -------------- begin");
                      Enumeration enmm =  request.getParameterNames();
                      while( enmm.hasMoreElements() ) {
                          String key = (String)enmm.nextElement();
                          String decodedKey = decode(key);
                          String[] values = request.getParameterValues(key);
                          System.out.print("["+decodedKey+"] => [");
                          if( values != null ) {
                              for( int i = 0; i < values.length; i++ )
                                  System.out.print(values[i]+(i+1<values.length?", ":""));
                          }
                          System.out.println("]");
                      }
                      System.out.println("parameteres -------------- end\n");
                    */
                    boolean addressesMerged = false;

                    // pick up department
                    String department = request.getParameter("department");
                    if( department != null ) {
                        department = decode(department);
                        if( department.trim().length() > 0 )
                            newNode.put("dept", department.trim());
                    }

                    // pick up location
                    String location = request.getParameter("location");
                    if( location != null ) {
                        location = decode(location);
                        if( location.trim().length() > 0 )
                            newNode.put("location", location);
                    }

                    // pick up room
                    String room = request.getParameter("room");
                    if( room != null )
                        newNode.put("room", room.trim());

                    // pick up make and model
                    String make_and_model = request.getParameter("make_and_model");
                    if( make_and_model != null ) {
                        make_and_model = decode(make_and_model);
                        if( make_and_model.trim().length() > 0 ) {
                            Hashtable models = (Hashtable)getData(session, "models");
                            Hashtable make_and_model_h = (Hashtable)models.get(make_and_model);
                            newNode.put("make", make_and_model_h.get("make"));
                            newNode.put("model", make_and_model_h.get("model"));
                        }
                    }

                    // pick up oses. On 'Save' oses starting with ' --None--' will be removed.
                    Hashtable oses = (Hashtable)newNode.get("oses");
                    if( oses == null ) {
                        oses = hash();
                        newNode.put("oses", oses);
                    }
                    oses.clear();
                    String[] osKeys = getParameterNamesStaringWith(request, "os_");
                    for( int i = 0; i < osKeys.length; i++ ) {
                        String osName =
                            decode(request.getParameter(osKeys[i]));
                        if( osName.equals("--None--") )
                            osName = " --None--"+getNextKey();
                        oses.put(osName, "");
                    }

                    // pick up tags
                    Hashtable tags = Utils.get_hash_or_create(newNode, "tags");
                    tags.clear();

                    // override by custom tags
                    String enabled_view_name = (String)newNode.get("enabled view");

                    if( enabled_view_name != null &&
                        ( last_view_name == null || last_view_name.trim().length() == 0 ) )
                        last_view_name = enabled_view_name;

                    // we need this to actually store both tags, even when they are the same
                    // unless custom view is enabled, since tags with the same name (aka. section)
                    // would screw up custom view currently displayed. tags_separated is removed
                    // down the road if necessary
                    pick_up_tags_separated(newNode, request, session);

                    if( last_view_name != null ) {
                        Hashtable view_data = nHash(nHash(getData(session, "custom.node.views")).get(last_view_name));
                        Pattern tag_p = Pattern.compile("^\\s*"+last_view_name+"_Tag_\\d+_Tag_([^_]+)_(.+)$");
                        Pattern cb_p  = Pattern.compile("^\\s*"+last_view_name+"_Tag_\\d+_TagCB_([^_]+)_.*$");
                        Pattern sl_p  = Pattern.compile("^\\s*"+last_view_name+"_Tag_\\d+_TagSL_([^_]+)_(.*)$");
                        String[] tagKeys = getParameterNamesStaringWith(request, last_view_name+"_Tag");
                        java.util.List list = Arrays.asList(tagKeys);
                        Collections.sort(list, name_cmp);
                        tagKeys = (String[])list.toArray();

                        tags = pick_up_tags(newNode, request, session);
                        newNode.remove("tags_separated");

                        if( tagKeys.length > 0 ) {
                            for( int i = 0; i < tagKeys.length; i++ ) {
                                String decoded_key = decode(tagKeys[i]);
                                Matcher tag_m = tag_p.matcher(decoded_key);
                                Matcher cb_m  = cb_p.matcher(decoded_key);
                                Matcher sl_m  = sl_p.matcher(decoded_key);
                                if( tag_m.matches() ) {
                                    String value = (String)Utils.n2s(tags.get(CNVParser.savedSection(view_data, tag_m.group(1))));
                                    String section = tag_m.group(1);
                                    String name    = tag_m.group(2);
                                    String evalue  = request.getParameter(tagKeys[i]);

                                    // find appropriate entry and check value of "save default" parameter
                                    Hashtable entry_def = hash();

                                    for( Hashtable entry : (Collection<Hashtable>)nHash(view_data.get("entries")).values() ) {
                                        if( Utils.n2s(entry.get("name"   )).equals(name) &&
                                            Utils.n2s(entry.get("section")).equals(section) ) {
                                            entry_def = entry;
                                            break;
                                        }
                                    }

                                    // special treatment of NIL default values in text elements
                                    if( Utils.nStr(entry_def.get("html element")).equals("text-field") &&
                                        Utils.nStr(entry_def.get("save default")).equalsIgnoreCase("no") &&
                                        Utils.nStr(entry_def.get("default value")).equalsIgnoreCase("nil") &&
                                        evalue.trim().length() == 0 )
                                        continue;

                                    if( !Utils.n2s(entry_def.get("default value")).equals(evalue) ||
                                        Utils.nStr(entry_def.get("save default")).equalsIgnoreCase("yes") )
                                        tags.put(CNVParser.savedSection(view_data, section),
                                                 value + (value.length()==0?"":",")+name+"="+evalue);

                                } else if( cb_m.matches() ) {
                                    String value = (String)Utils.n2s(tags.get(CNVParser.savedSection(view_data, cb_m.group(1))));
                                    tags.put(CNVParser.savedSection(view_data, cb_m.group(1)),
                                             value+(value.length()==0?"":",")+request.getParameter(tagKeys[i]));

                                } else if( sl_m.matches() ) {
                                    String value = (String)Utils.n2s(tags.get(CNVParser.savedSection(view_data, sl_m.group(1))));
                                    Hashtable selector = findSelector(view_data, sl_m.group(2));

                                    String[] values = request.getParameterValues(tagKeys[i]);
                                    if( Utils.nStr(selector.get("name")).trim().length() == 0 )
                                        for( int j = 0; j < values.length; j++ )
                                            value += (value.length()==0?"":",") + decode(values[j]);
                                    else {
                                        value += (value.length()==0?"":",")+selector.get("name")+"=";
                                        for( int j = 0; j < values.length; j++ )
                                            value += decode(values[j])+(j+1<values.length?selector.get("separator"):"");
                                    }

                                    tags.put(CNVParser.savedSection(view_data, sl_m.group(1)), value);
                                }
                            }

                            // try to rebuild custom view tags data structure
                            try {
                                parse_tags_for_custom_view(newNode, view_data, last_view_name,
                                                           custom_fields_total, false);
                            } catch (Exception ex) {
                                newNode.remove("enabled view");
                                addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                            }
                        }

                        // ensure that section names are filled in even when no data is present in tag values
                        Hashtable sections = nHash(view_data.get("sections"));

                        for( Hashtable section_h : (Collection<Hashtable>)sections.values() ) {
                            String section = Utils.nStr(section_h.get("name"));
                            if( tags.get(CNVParser.savedSection(view_data, section)) == null )
                                tags.put(CNVParser.savedSection(view_data, section), "");
                        }
                    }

                    // pick up exp.date
                    String exp_date = request.getParameter("expirationDate");
                    newNode.put("exp_date_string", exp_date.trim());
                    if( exp_date.trim().length() == 0 )
                        newNode.remove("exp.date");
                    else {
                        try {
                            Timestamp exp_date_ts = new Timestamp(Constants.df.parse(exp_date).getTime());
                            newNode.put("exp.date", exp_date_ts);
                            newNode.remove("bad_exp_date_format");
                        } catch(Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", "Error: Illegal date format.");
                            newNode.put("bad_exp_date_format", "");
                        }
                    }

                    // pick up groups
                    Hashtable groups = hash();
                    newNode.put("groups", groups);

                    String[] newGroups = request.getParameterValues("groups");
                    if( newGroups != null ) {
                        for( int i = 0; i < newGroups.length; i++ )
                            groups.put(decode(newGroups[i]), "");
                    }

                    // pick up names_count
                    String raw_names_count = ((String)Utils.n2s(request.getParameter("names_count"))).trim();
                    newNode.put("raw_names_count", raw_names_count);
                    if( raw_names_count.length() > 0 )
                        try {
                            int add_count = Integer.parseInt(raw_names_count);
                            if( add_count < 0 )
                                throw new Exception ("");
                            newNode.remove("names_count_error");
                        } catch (Exception ex) {
                            newNode.put("names_count_error", "");
                        };

                    // pick up interfaces_count
                    String raw_interfaces_count = ((String)Utils.n2s(request.getParameter("interfaces_count"))).trim();
                    newNode.put("raw_interfaces_count", raw_interfaces_count);
                    if( raw_interfaces_count.length() > 0 )
                        try {
                            int add_count = Integer.parseInt(raw_interfaces_count);
                            if( add_count < 0 )
                                throw new Exception ("");
                            newNode.remove("interfaces_count_error");
                        } catch (Exception ex) {
                            newNode.put("interfaces_count_error", "");
                        };

                    // pick up comment
                    newNode.put("comment",
                                ((String)Utils.n2s((String)request.getParameter("comment"))).trim());

                    // pick up state
                    newNode.put("state",
                                decode((String)Utils.n2s((String)request.getParameter("state"))).trim());

                    // pick up addr space
                    String addr_space = request.getParameter("addr_space");
                    if( addr_space != null &&
                        find_by_key(nHash(newNode.get("types")), "extkey", "Template", "type") != null ) {
                        String real_addr_space = decode(addr_space);
                        if( real_addr_space.trim().equals("---") )
                            newNode.remove("addr_space");
                        else
                            newNode.put("addr_space", real_addr_space);
                    }

                    if( find_by_key(nHash(newNode.get("types")), "extkey", "Template", "type") == null )
                        newNode.remove("addr_space");

                    // remove interfaces no longer checked
                    Hashtable intfcs_to_keep = Utils.makeHash(decode(request.getParameterValues("intfc")), null);
                    Hashtable interfaces = nHash(newNode.get("interfaces"));
                    Vector intfcs_to_remove = new Vector();

                    for( String intfcID : (Set<String>)interfaces.keySet() ) {
                        Hashtable intfc = (Hashtable)interfaces.get(intfcID);
                        if( intfcs_to_keep.get(intfcID) == null && intfc.get("just added") == null )
                            intfcs_to_remove.addElement(intfcID);
                        intfc.remove("just added");
                    }

                    for( int i = 0; i < intfcs_to_remove.size(); i++ )
                        interfaces.remove(intfcs_to_remove.elementAt(i));

                    Hashtable dhcp_options = nHash(getData(session, "dhcp_options"));

                    // set roaming and dhcp flags to false on all interfaces
                    // also clear up dhcp options.
                    // they will be refilled below.
                    interfaces = nHash(newNode.get("interfaces"));
                    for( Hashtable intfc : (Collection<Hashtable>)interfaces.values() ) {
                        intfc.put("roaming?",     Boolean.FALSE);
                        intfc.put("use-dhcp?",    Boolean.FALSE);
                        intfc.put("dhcp-options", hash());
                    }


                    for( String intfcID : (Set<String>)intfcs_to_keep.keySet() ) {
                        Hashtable intfc = nHash(nHash(newNode.get("interfaces")).get(intfcID));

                        // pick up hardware addresses
                        String hwStr = decode(request.getParameter(encode("hw_addr "+intfcID)));

                        if( hwStr == null || hwStr.trim().length() == 0 )
                            intfc.remove("hw");
                        else {
                            String parsedHW = null;
                            try {
                                Hashtable parseRes = (new Parser()).parse(hwStr);
                                parsedHW = (String)parseRes.get("hw");
                            } catch (Exception exx) {
                                troubled_hws.put(hwStr.trim().toLowerCase(), "");
                            }
                            if( parsedHW == null ) {
                                troubled_hws.put(hwStr.trim().toLowerCase(), "");
                                addSessionMsg(session, "Error.errorMsg", "Error: Incorrect format for hardware address.");
                            } else {
                                troubled_hws.remove(hwStr.trim().toLowerCase());
                                hwStr = parsedHW;
                            }

                            intfc.put("hw", hwStr);
                        }

                        // pick up interface name
                        String hwName =
                            decode(((String)Utils.n2s(request.getParameter(encode("hw_name "+intfcID)))).trim()).trim();
                        if( hwName.length() == 0 )
                            intfc.put("names", hash());
                        else {
                            if( hwName.indexOf(".") < 0 ) {
                                // it does not have domain part
                                hwName  += "."+default_domain;
                            }
                            Hashtable nameH = hash();
                            nameH.put("name", Utils.name(hwName));
                            nameH.put("domain", Utils.domain(hwName, default_domain));

                            Hashtable names = hash(nameH.get("name")+"."+nameH.get("domain")+" "+getNextKey(), nameH);
                            intfc.put("names", names);
                        }

                        // pick up roam and dhcp flags
                        String dhcp_flag = request.getParameter(encode("dhcp_flag "+intfcID));
                        if( dhcp_flag != null )
                            intfc.put("use-dhcp?", new Boolean(true));

                        String roam_flag = request.getParameter(encode("roam_flag "+intfcID));
                        if( roam_flag != null && dhcp_flag != null )
                            intfc.put("roaming?", new Boolean(true));

                        // pick up dhcp options
                        String dhcp_text = request.getParameter(encode("dhcp_options "+intfcID));
                        String badIntfcDHCP = null;
                        if( dhcp_text != null ) {
                            intfc.put("dhcp-options-raw", dhcp_text);

                            String[] split = dhcp_text.split("\\n");
                            Hashtable options = hash();
                            if( dhcp_text.length() > 0 ) {
                                for( int j = 0; j < split.length; j++ ) {
                                    if( split[j].trim().length() == 0 )
                                        continue;
                                    String[] ops = split[j].split("=", 2);
                                    if( ops.length != 2 ) {
                                        addSessionMsg(session, "Error.errorMsg",
                                                      "Error: DHCP option \""+split[j]+"\" does not have value.");
                                        badIntfcDHCP = intfcID;
                                    } else {
                                        options.put(ops[0].trim()+"="+ops[1].trim(),
                                                    hash("option", ops[0].trim(),
                                                         "value",  ops[1].trim()));

                                        if( nHash(dhcp_options.get(ops[0].trim().toLowerCase())).get("interface") == null ) {
                                            badIntfcDHCP = intfcID;
                                            addSessionMsg(session, "Error.errorMsg",
                                                          "Error: Illegal DHCP option \""+ops[0].trim()+"\"");
                                        }
                                    }
                                }
                            }

                            if( badIntfcDHCP == null )
                                intfc.remove("bad dhcp options");
                            else
                                intfc.put("bad dhcp options", "");

                            intfc.put("dhcp-options", options);
                        }

                        // pick up ip addresses their names and statuses
                        Hashtable ips = hash();
                        intfc.put("ips", ips);
                        String[] params = getParametersStaringWith(request, "ip addr cb ["+intfcID+"] [");
                        if( params != null ) {
                            for( int i = 0; i < params.length; i += 2 ) {
                                String ipKey = params[i+1];
                                String ipStr = request.getParameter(encode("ip addr txt ["+intfcID+"] ["+ipKey+"]")).trim();
                                if( ipStr.length() == 0 )
                                    continue;

                                ipStr = Utils.normIp(ipStr);
                                Hashtable ip = hash();
                                if( ips.put(ipStr, ip) != null )
                                    addressesMerged = true;

                                if( request.getParameter(encode("ip addr status ["+intfcID+"] ["+ipKey+"]")) != null )
                                    ip.put("active?", new Boolean(true));
                                else
                                    ip.put("active?", new Boolean(false));

                                String ptr_pref =
                                    request.getParameter(encode("ip ptr pref ["+intfcID+"] ["+ipKey+"]"));
                                if( ptr_pref != null ) ip.put("PTRpref", ptr_pref);

                                if( find_by_key(nHash(newNode.get("types")), "extkey", "Advanced", "type") != null ) {
                                    Hashtable names = hash();
                                    ip.put("names", names);

                                    String nameStr =
                                        request.getParameter(encode("ip addr name ["+intfcID+"] ["+ipKey+"]"));
                                    if( nameStr != null && nameStr.trim().length() > 0 ) {
                                        Hashtable nameH = hash();
                                        String name = Utils.name(nameStr).trim();
                                        nameH.put("name", name);
                                        String domain = Utils.domain(nameStr, default_domain).trim();
                                        nameH.put("domain", domain);
                                        names.put(name+"."+domain+" "+getNextKey(), nameH);
                                    }
                                }
                            }
                        }

                        // pick up selections for new ip and name
                        intfc.put("saved addr space",
                                  (String)Utils.n2s(getDecodedParameter(request, "addr space "+intfcID)));

                        intfc.put("saved ip addr",
                                  (String)Utils.n2s(getDecodedParameter(request, "new ip txt "+intfcID)));

                        String requested_ip_status = getDecodedParameter(request, "new ip active "+intfcID);
                        if( requested_ip_status == null )
                            intfc.put("saved ip status inactive", "");
                        else
                            intfc.remove("saved ip status inactive");

                        // pick up ip count
                        String raw_ip_count =
                            (String)Utils.n2s(getDecodedParameter(request, "new ip count "+intfcID));
                        intfc.put("raw ip count", raw_ip_count);
                        if( raw_ip_count.length() > 0 )
                            try {
                                int add_count = Integer.parseInt(raw_ip_count);
                                if( add_count < 0 )
                                    throw new Exception ("");
                                intfc.remove("ip_count_error");
                            } catch (Exception ex) {
                                intfc.put("ip_count_error", "");
                            };

                        intfc.put("saved ip name", (String)Utils.n2s(getDecodedParameter(request, "new ip name "+intfcID)));
                    } // end of picking up interfaces information

                    // pick up ipc
                    newNode.remove("ipc_addresses");
                    if( find_by_key(nHash(newNode.get("types")), "extkey", "IPCprovider", "type") != null ) {
                        Hashtable ipc_addresses = hash();
                        newNode.put("ipc_addresses", ipc_addresses);
                        String[] ipc_key = request.getParameterValues(encode("ipc addr cb"));
                        if( ipc_key == null )
                            ipc_key = new String[0];
                        for( int i = 0; i < ipc_key.length; i++ ) {
                            String ipcStr =
                                request.getParameter(encode("ipc addr txt "+ipc_key[i])).trim();

                            if( ipcStr.length() == 0 )
                                continue;

                            ipcStr = Utils.normIp(ipcStr);

                            Hashtable ipH = hash();
                            if( ipc_addresses.put(ipcStr, ipH) != null )
                                addressesMerged = true;

                            if( request.getParameter(encode("ipc addr status "+ipc_key[i])) != null )
                                ipH.put("active?", new Boolean(true));
                            else
                                ipH.put("active?", new Boolean(false));

                            Hashtable names = hash();
                            ipH.put("names", names);
                            String ipcName = request.getParameter(encode("ipc addr name "+ipc_key[i])).trim();
                            if( ipcName.length() > 0 ) {
                                String name = Utils.name(ipcName).trim();
                                String domain = Utils.domain(ipcName, default_domain).trim();
                                if( name.length() > 0 ) {
                                    names.put(name+"."+domain+" "+getNextKey(),
                                              hash("name",   name,
                                                   "domain", domain));
                                }
                            }
                        }

                        // pick up selections for new ip and name
                        if( request.getParameter("shown_ipc_select") == null ) {
                            newNode.remove("saved ipc addr space");
                            newNode.remove("saved ipc addr");
                            newNode.remove("saved ipc status inactive");
                            newNode.remove("saved ipc count");
                        } else {
                            newNode.put("saved ipc addr space", (String)Utils.n2s(getDecodedParameter(request, "ipc addr space")));
                            newNode.put("saved ipc addr", (String)Utils.n2s(getDecodedParameter(request, "new ipc txt")));
                            String requested_ipc_status = getDecodedParameter(request, "new ipc active");
                            if( requested_ipc_status == null )
                                newNode.put("saved ipc status inactive", "");
                            else
                                newNode.remove("saved ipc status inactive");
                            newNode.put("saved ipc count", (String)Utils.n2s(getDecodedParameter(request, "new ipc count")));
                        }
                    }

                    // fill up all users with "not selected" flag
                    Hashtable users = nHash(newNode.get("users"));
                    for( Hashtable user : (Collection<Hashtable>)users.values() )
                        user.put("not selected", "");


                    // pick up checked users
                    String[] values = request.getParameterValues("user");
                    if( values != null ) {
                        for( int i = 0; i < values.length; i++ ) {
                            String key = decode(values[i]);
                            nHash(nHash(newNode.get("users")).get(key))
                                .remove("not selected");
                        }
                    }

                    // pick up all stale users, i.e. those with "not selected" flag
                    Vector keysToRemove = new Vector();
                    users = nHash(newNode.get("users"));

                    for( Object key : users.keySet() )
                        if( ((Hashtable)users.get(key)).get("not selected") != null )
                            keysToRemove.addElement(key);

                    // now remove them
                    for( int i = 0; i < keysToRemove.size(); i++ )
                        users.remove(keysToRemove.elementAt(i));

                    // now onto admins
                    Hashtable admins = nHash(nHash(newNode.get("admins")).get("admins"));
                    for( Hashtable admin : (Collection<Hashtable>)admins.values() )
                        admin.put("not selected", "");

                    Hashtable ateams = nHash(nHash(newNode.get("admins")).get("ateams"));
                    for( Hashtable ateam : (Collection<Hashtable>)ateams.values() )
                        ateam.put("not selected", "");

                    // pick up checked admins
                    values = request.getParameterValues("admin");
                    if( values != null ) {
                        for( int i = 0; i < values.length; i++ ) {
                            String key = decode(values[i]);
                            nHash(nHash(nHash(newNode.get("admins")).get("admins")).get(key))
                                .remove("not selected");
                        }
                    }

                    // pick up checked ateams
                    values = request.getParameterValues("ateam");
                    if( values != null ) {
                        for( int i = 0; i < values.length; i++ ) {
                            String key = decode(values[i]);
                            nHash(nHash(nHash(newNode.get("admins")).get("ateams")).get(key))
                                .remove("not selected");
                        }
                    }

                    // pick up all stale users, i.e. those with "not selected" flag
                    keysToRemove = new Vector();
                    admins = nHash(nHash(newNode.get("admins")).get("admins"));
                    for( Object key : admins.keySet() )
                        if( ((Hashtable)admins.get(key)).get("not selected") != null )
                            keysToRemove.addElement(key);

                    // now remove them
                    for( int i = 0; i < keysToRemove.size(); i++ )
                        admins.remove(keysToRemove.elementAt(i));

                    // pick up all stale ateams, i.e. those with "not selected" flag
                    keysToRemove = new Vector();
                    ateams = nHash(nHash(newNode.get("admins")).get("ateams"));
                    for( Object key : ateams.keySet() )
                        if( ((Hashtable)ateams.get(key)).get("not selected") != null )
                            keysToRemove.addElement(key);

                    // now remove them
                    for( int i = 0; i < keysToRemove.size(); i++ )
                        ateams.remove(keysToRemove.elementAt(i));

                    // pick up node names, aliases and mxes
                    // node names
                    Hashtable names = hash();
                    newNode.put("names", names);
                    String[] names_cb = getParametersStaringWith(request, "node_name_cb_");
                    for( int i = 0; i < names_cb.length; i += 2 ) {
                        String key = names_cb[i].substring(13);

                        String name = request.getParameter(encode("node_name_tf_"+key));
                        Hashtable nameH = hash("name", Utils.name(name).trim(),
                                               "domain", Utils.domain(name, default_domain).trim());

                        String has_error = request.getParameter(encode("node_name_error_"+key));
                        if( has_error != null )
                            nameH.put("error", "");

                        String was_reserved = request.getParameter(encode("node_name_reserved_"+key));
                        if( was_reserved != null )
                            nameH.put("reserved", "");

                        // pick up aliases_count_tf_...
                        String raw_aliases_count = ((String)Utils.n2s(request.getParameter(encode("aliases_count_tf_"+key)))).trim();
                        nameH.put("raw_aliases_count", raw_aliases_count);
                        if( raw_aliases_count.length() > 0 )
                            try {
                                int add_count = Integer.parseInt(raw_aliases_count);
                                if( add_count < 0 )
                                    throw new Exception ("");
                                nameH.remove("aliases_count_error");
                            } catch (Exception ex) {
                                nameH.put("aliases_count_error", "");
                            };

                        // pick up mxs_count_tf_...
                        String raw_mxs_count = ((String)Utils.n2s(request.getParameter(encode("mxs_count_tf_"+key)))).trim();
                        nameH.put("raw_mxs_count", raw_mxs_count);
                        if( raw_mxs_count.length() > 0 )
                            try {
                                int add_count = Integer.parseInt(raw_mxs_count);
                                if( add_count < 0 )
                                    throw new Exception ("");
                                nameH.remove("mxs_count_error");
                            } catch (Exception ex) {
                                nameH.put("mxs_count_error", "");
                            };

                        Hashtable aliases = hash();
                        nameH.put("aliases", aliases);
                        Hashtable mxs = hash();
                        nameH.put("mxs", mxs);

                        names.put(key, nameH);

                        // aliases
                        int base_key_length = (new String("alias_name_cb_"+key+" ")).length();
                        String[] aliases_cb = getParametersStaringWith(request, "alias_name_cb_"+key+" ");
                        for( int j = 0; j < aliases_cb.length; j += 2 ) {
                            String akey = aliases_cb[j].substring(base_key_length);
                            String alias = request.getParameter(encode("alias_name_tf_"+key+" "+akey));
                            if( alias != null ) {
                                aliases.put(akey,
                                            hash("name",   Utils.name(alias).trim(),
                                                 "domain", Utils.domain(alias, default_domain).trim()));
                            }
                        }

                        // mxes
                        base_key_length = (new String("mx_name_cb_"+key+" ")).length();
                        String[] mxs_cb = getParametersStaringWith(request, "mx_name_cb_"+key+" ");
                        for( int j = 0; j < mxs_cb.length; j += 2 ) {
                            String mkey = mxs_cb[j].substring(base_key_length);
                            String mx = request.getParameter(encode("mx_name_tf_"+key+" "+mkey));
                            String pref = request.getParameter(encode("mx_pref_tf_"+key+" "+mkey));
                            if( pref != null && mx != null ) {
                                mxs.put(mkey,
                                        hash("name", Utils.name(mx).trim(),
                                             "domain", Utils.domain(mx, default_domain).trim(),
                                             "pref", pref.trim()));
                            }
                        }
                    }

                    if( !action.equals("Verify") ) {
                        // if action is Verify check for a duplicate names will
                        // be done at the end of verify stage
                        String err = checkDuplicateNames(newNode);
                        if( err != null && err.trim().length() > 0 )
                            addSessionMsg(session, "Error.errorMsg", err);
                    }

                    if( addressesMerged )
                        addSessionMsg(session, "Warning", "WARNING: Duplicate ip addresses were merged.");
                } // end of 'if( !action.equals("Reset") )'

                boolean reserve_ipc = false;
                boolean reserve_ip = false;

                if( action.equals("Verify") || action.equals("Save") ) {
                    Hashtable oldNames = hash();
                    try {
                        collectNamesAndIPs((Hashtable)store.get("oldNode"), oldNames, hash(), hash());
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }


                    Hashtable allOldNames = hash();
                    Hashtable allOldIPs   = hash();
                    try {
                        collectNamesAndIPs(oldNewNode, allOldNames, allOldIPs, hash());
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }

                    Hashtable allNewNames = hash();
                    Hashtable allNewIPs   = hash();
                    Hashtable ipParents   = hash();
                    try {
                        collectNamesAndIPs(newNode, allNewNames, allNewIPs, ipParents);
                    } catch (Exception ex) {
                        addSessionMsg(session, "Error.errorMsg", "Error: "+ex.getMessage());
                    }

                    Vector names_to_reserve = new Vector();
                    Vector names_to_cancel  = new Vector();
                    Vector names_no_change  = new Vector();

                    Utils.keysIntersect(allOldNames, allNewNames,
                                        names_to_reserve, names_to_cancel, names_no_change);

                    Hashtable reserved_names = Utils.get_hash_or_create(store, "reserved_names");
                    Hashtable troubled_names = Utils.get_hash_or_create(store, "troubled_names");

                    boolean got_some_errors = false;
                    for( int i = 0; i < names_to_reserve.size(); i++ ) {
                        String key = (String)names_to_reserve.elementAt(i);
                        Hashtable nameH = (Hashtable)allNewNames.get(key);

                        String nName   = ((String)nameH.get("name")).trim();
                        String nDomain = ((String)nameH.get("domain")).trim();

                        boolean skip = false;
                        for( Hashtable oldName : (Collection<Hashtable>)oldNames.values() ) {
                            if( oldName.get("name").equals(nName) && oldName.get("domain").equals(nDomain) ) {
                                skip = true;
                                break;
                            }
                        }

                        if( skip ) continue;

                        String name = nName+"."+nDomain;
                        String r_name = null;
                        String nameType = nameType(key);
                        try {
                            try {
                                String sType = simplifiedType(nameType);
                                if( !checkNameReserve(registrar, name, sType) )
                                    continue;

                                if( name.length() > 255 )
                                    throw new Exception("");

                                r_name = netdb.reserveName(name, nameType);

                                if( r_name != null && r_name.trim().length() > 0 ) {
                                    registerName(registrar, r_name.trim(), sType);

                                    nameH.put("reserved", "");
                                    nameH.remove("error");
                                    synchronized(nextKey) {
                                        reserved_names.put(nameType+" "+r_name,
                                                           hash("fqdn", r_name,
                                                                "type", nameType));
                                    }
                                } else
                                    throw new Exception("Error: Failed to reserve name '"+name+"'");

                            } catch (NetDBException ex) {
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int j = 0; j < esize; j++ )
                                    msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                                throw new Exception(msg);
                            }
                        } catch (Exception ex) {
                            if( name != null && name.trim().length() > 0 ) {
                                synchronized(nextKey) {
                                    troubled_names.put(nameType+" "+name,
                                                       hash("fqdn", name,
                                                            "type", nameType));
                                }
                            }

                            got_some_errors = true;
                            nameH.put("error", "");
                            String msg = ex.getMessage();
                            if( msg != null && msg.trim().length() > 0 )
                                addSessionMsg(session, "Error.errorMsg", msg.trim());
                        }
                    }
                    if( !got_some_errors && names_to_reserve.size() > 0 )
                        addSessionMsg(session, "info_msg", "SUCCESS: Node names verified.");

                    String err = checkDuplicateNames(newNode);
                    if( err != null && err.trim().length() > 0 )
                        addSessionMsg(session, "Error.errorMsg", err);

                    // now ip addresses
                    Vector ips_to_reserve = new Vector();
                    Vector ips_to_cancel = new Vector();
                    Vector ips_no_change = new Vector();
                    got_some_errors = false;
                    try {
                        Utils.keysIntersect(allOldIPs, allNewIPs,
                                            ips_to_reserve, ips_to_cancel, ips_no_change);
                    } catch (Exception ex) {
                        ex.printStackTrace();
                        throw ex;
                    }

                    Hashtable reserved_ips = (Hashtable)store.get("reserved_ips");
                    if( reserved_ips == null ) {
                        reserved_ips = hash();
                        store.put("reserved_ips", reserved_ips);
                    }
                    for( int i = 0; i < ips_to_reserve.size(); i++ ) {
                        String requested_ip = (String)ips_to_reserve.elementAt(i);
                        Hashtable ipH = hash();
                        try {
                            try {
                                String ip_low = null;
                                String ip_high = null;
                                try {
                                    Hashtable parseRes = (new Parser()).parse(requested_ip);
                                    ip_low = (String)parseRes.get("ip_low");
                                    ip_high = (String)parseRes.get("ip_high");
                                } catch (Exception exx) { }

                                if( ( ip_low == null || ip_high == null ) /* || !ip_low.equals(ip_high) */ ) {
                                    addSessionMsg(session, "Error.errorMsg", "Incorrect ip address format '"+requested_ip+"'");
                                    troubled_ips.put(requested_ip.trim().toLowerCase(), "");
                                    got_some_errors = true;
                                    continue;
                                }

                                Hashtable result = netdb.reserveIP(!ip_low.equals(ip_high)?requested_ip:null,
                                                                   !ip_low.equals(ip_high)?ip_low:requested_ip, 1);

                                result = nHash(result.get(result.keys().nextElement()));
                                Hashtable warnings = nHash(result.get("warnings"));
                                if( warnings.size() > 0 ) {
                                    for( String warning : (Set<String>)warnings.keySet() )
                                        addSessionMsg(session, "Warning", warning.trim());
                                }

                                ipH = hash();
                                String reserved_ip = null;
                                Hashtable ip_addresses = nHash(result.get("ip_addresses"));

                                for( String the_ip : (Set<String>)ip_addresses.keySet() ) {
                                    reserved_ip = the_ip;
                                    ipH.put("ip", reserved_ip);
                                    reserved_ips.put(reserved_ip, "");
                                    troubled_ips.remove(reserved_ip.trim().toLowerCase());
                                }

                                if( reserved_ip == null )
                                    throw new Exception("Error: Failed to reserve ip address '"+requested_ip+"'.");

                                Hashtable masterIpHash = (Hashtable)allNewIPs.get(requested_ip);
                                ipH.put("names", nHash(masterIpHash.get("names")));
                                ipH.put("active?", masterIpHash.get("active?"));

                                Hashtable parentHash = nHash(ipParents.get(masterIpHash.get("parent")));
                                parentHash.remove(requested_ip);
                                parentHash.put(reserved_ip, ipH);

                            } catch (NetDBException ex) {
                                ex.printStackTrace();
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int j = 0; j < esize; j++ ) {
                                    String eMsg = ((String)Utils.n2s((String)errors.elementAt(j))).trim();
                                    if( eMsg.matches("Address Space \".*\" does not exist\\.") )
                                        setData(session, "myAddrSpaces", netdb.listMyAddrSpaces());
                                    msg += "Error: "+eMsg+(j<esize-1?"\n":"");
                                }
                                throw new Exception(msg);
                            }
                        } catch (Exception ex) {
                            troubled_ips.put(requested_ip.toLowerCase().trim(), "");
                            ex.printStackTrace();
                            got_some_errors = true;
                            if( ipH != null )
                                ipH.put("error", "");
                            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                        }
                    }

                    if( !got_some_errors && ips_to_reserve.size() > 0 )
                        addSessionMsg(session, "info_msg", "SUCCESS: Ip addresses verified.");

                    oldNewNode = Utils.copyHash(newNode);
                    store.put("oldNewNode", oldNewNode);

                    if( request.getParameter("shown_ipc_select") != null )
                        reserve_ipc = true;

                    String[] keys = request.getParameterValues("shown_ip_select");
                    if( keys == null )
                        targets = new String[0];
                    else {
                        targets = new String[keys.length];
                        for( int i = 0; i < keys.length; i++ )
                            targets[i] = decode(keys[i]);
                    }
                    reserve_ip = true;

                    // clean up node oses from irrelevan items
                    Hashtable oses = Utils.get_hash_or_create(newNode, "oses");
                    Vector oses_to_remove = new Vector();

                    for( String os : (Set<String>)oses.keySet() )
                        if( os.startsWith(" --None--") )
                            oses_to_remove.addElement(os);

                    // now remove them, oses that is.
                    for( int i = 0; i < oses_to_remove.size(); i ++ )
                        oses.remove(oses_to_remove.elementAt(i));

                    err = checkDuplicateNames(newNode);
                    if( err != null && err.trim().length() > 0 )
                        addSessionMsg(session, "Error.errorMsg", err);

                } // end of 'if( action.equals("Verify") )'

                if( action.equals("reserve ipc and add") ) {

                    String requested_ipc_addr = getDecodedParameter(request, "new ipc txt");
                    if( requested_ipc_addr != null && requested_ipc_addr.trim().length() > 0 )
                        requested_ipc_addr = Utils.normIp(requested_ipc_addr);
                    String requested_ipc_count = getDecodedParameter(request, "new ipc count");
                    String requested_ipc_status = getDecodedParameter(request, "new ipc active");
                    String requested_addr_space = getDecodedParameter(request, "ipc addr space");

                    boolean parse_ip_count_failed = false;
                    int ipc_count = -1;
                    if( requested_ipc_count == null || requested_ipc_count.trim().length() == 0 )
                        ipc_count = 1;

                    if( ipc_count == -1 ) {
                        try {
                            ipc_count = Integer.parseInt(requested_ipc_count);
                        } catch (Exception pex) {
                            addSessionMsg(session, "Error.errorMsg", "Error: Illegal ipc count value");
                            parse_ip_count_failed = true;
                        }
                    }
                    if( !parse_ip_count_failed ) {
                        if( ipc_count < 1 || ipc_count > 10000 ) {
                            addSessionMsg(session, "Error.errorMsg", "Error: Illegal ipc count value");
                            parse_ip_count_failed = true;
                        }
                    }

                    if( !parse_ip_count_failed ) {
                        if( requested_addr_space != null ) {
                            requested_addr_space = decode(requested_addr_space);
                            newNode.put("saved ipc addr space", requested_addr_space);
                        } else
                            newNode.remove("saved ipc addr space");

                        newNode.put("saved ipc addr", requested_ipc_addr);
                        newNode.put("saved ipc count", requested_ipc_count);
                        if( requested_ipc_status == null )
                            newNode.put("saved ipc status inactive", "");
                        else
                            newNode.remove("saved ipc status inactive");

                        Hashtable ips = (Hashtable)newNode.get("ipc_addresses");
                        if( ips == null ) {
                            ips = hash();
                            newNode.put("ipc_addresses", ips);
                        }

                        if( (requested_addr_space == null || requested_addr_space.trim().length() == 0) &&
                            requested_ipc_addr != null && requested_ipc_addr.indexOf("/") > -1 ) {
                            requested_addr_space = requested_ipc_addr;
                            requested_ipc_addr = null;
                        }

                        if( ( requested_ipc_addr == null || requested_ipc_addr.trim().length() == 0 ) &&
                            ( requested_addr_space == null || requested_addr_space.trim().length() == 0 ) ) {
                            addSessionMsg(session, "Error.errorMsg", "Error: Please provide ip address or address space to reserve requested ipc address(es).");

                        } else {
                            String requested_ipc_name = null;
                            if( reserveIPandName(session, netdb, ips, store,
                                                 new Boolean(requested_ipc_status!=null?true:false),
                                                 requested_ipc_addr, requested_addr_space, requested_ipc_name,
                                                 ipc_count) ) {
                                newNode.remove("saved ipc addr space");
                                newNode.remove("saved ipc addr");
                                newNode.remove("saved ipc status inactive");
                                newNode.remove("saved ipc count");
                            }
                        }
                    }
                } // end of action "reserve ipc and add"

                if( action.equals("reserve ipc") || reserve_ipc ) {

                    String requested_ipc_addr = getDecodedParameter(request, "new ipc txt");
                    if( requested_ipc_addr != null && requested_ipc_addr.trim().length() > 0 )
                        requested_ipc_addr = Utils.normIp(requested_ipc_addr);
                    String requested_ipc_count = getDecodedParameter(request, "new ipc count");
                    String requested_ipc_status = getDecodedParameter(request, "new ipc active");
                    String requested_addr_space = getDecodedParameter(request, "ipc addr space");

                    boolean reserve_failed = false;

                    boolean parse_ip_count_failed = false;
                    int ipc_count = -1;
                    if( requested_ipc_count == null || requested_ipc_count.trim().length() == 0 )
                        ipc_count = 1;

                    if( ipc_count == -1 ) {
                        try {
                            ipc_count = Integer.parseInt(requested_ipc_count);
                        } catch (Exception pex) {
                            addSessionMsg(session, "Error.errorMsg", "Error: Illegal ipc count value");
                            reserve_failed = true;
                            parse_ip_count_failed = true;
                        }
                    }
                    if( !parse_ip_count_failed ) {
                        if( ipc_count < 1 || ipc_count > 10000 ) {
                            addSessionMsg(session, "Error.errorMsg", "Error: Illegal ipc count value");
                            reserve_failed = true;
                            parse_ip_count_failed = true;
                        }
                    }

                    if( !parse_ip_count_failed ) {
                        if( requested_addr_space != null )
                            requested_addr_space = decode(requested_addr_space);

                        if( ( requested_ipc_addr == null || requested_ipc_addr.trim().length() == 0 ) &&
                            ( requested_addr_space == null || requested_addr_space.trim().length() == 0 ) ) {
                            if( !reserve_ipc || ( requested_ipc_count != null && requested_ipc_count.trim().length() > 0 ) )
                                addSessionMsg(session, "Error.errorMsg", "Error: Please provide ip address or address space to reserve requested ipc address(es).");
                            reserve_failed = true;

                        } else {
                            Hashtable ips = (Hashtable)newNode.get("ipc_addresses");
                            if( ips == null ) {
                                ips = hash();
                                newNode.put("ipc_addresses", ips);
                            }

                            String requested_ipc_name = null;
                            if( (requested_addr_space == null || requested_addr_space.trim().length() == 0) &&
                                requested_ipc_addr != null && requested_ipc_addr.indexOf("/") > -1 ) {
                                requested_addr_space = requested_ipc_addr;
                                requested_ipc_addr = null;
                            }

                            if( !reserveIPandName(session, netdb, ips, store,
                                                  new Boolean(requested_ipc_status!=null?true:false),
                                                  requested_ipc_addr, requested_addr_space, requested_ipc_name,
                                                  ipc_count) )
                                reserve_failed = true;
                        }
                    }

                    if( reserve_failed ) {
                        if( requested_addr_space != null )
                            newNode.put("saved ipc addr space", requested_addr_space);

                        if( requested_ipc_status == null )
                            newNode.put("saved ipc status inactive", "");
                        else
                            newNode.remove("saved ipc status inactive");

                        newNode.put("saved ipc addr", requested_ipc_addr);
                        newNode.put("saved ipc count", requested_ipc_count);
                    } else {
                        newNode.remove("saved ipc addr space");
                        newNode.remove("saved ipc status inactive");
                        newNode.remove("saved ipc addr");
                        newNode.remove("saved ipc count");
                        newNode.remove("show ipc select");
                    }
                } // end of action "reserve ipc"

                if( action.equals("reserve ip and add") ) {
                    if( targets == null )
                        targets = new String[0];
                    for( int k = 0; k < targets.length; k++ ) {
                        target = targets[k];
                        String requested_ip_addr = getDecodedParameter(request, "new ip txt "+target);
                        if( requested_ip_addr != null && requested_ip_addr.trim().length() > 0 )
                            requested_ip_addr = Utils.normIp(requested_ip_addr);
                        String requested_ip_name = ((String)Utils.n2s(getDecodedParameter(request, "new ip name "+target))).trim();
                        String requested_ip_status = getDecodedParameter(request, "new ip active "+target);
                        String requested_addr_space = ((String)Utils.n2s(getDecodedParameter(request, "addr space "+target))).trim();

                        boolean parse_ip_count_failed = false;
                        int ip_count = -1;
                        String requested_ip_count = getDecodedParameter(request, "new ip count "+target);

                        if( requested_ip_count == null || requested_ip_count.trim().length() == 0 )
                            ip_count = 1;

                        if( ip_count == -1 ) {
                            try {
                                ip_count = Integer.parseInt(requested_ip_count);
                            } catch (Exception pex) {
                                addSessionMsg(session, "Error.errorMsg", "Error: Illegal ip count value");
                                parse_ip_count_failed = true;
                            }
                        }

                        if( !parse_ip_count_failed ) {
                            if( ip_count < 1 || ip_count > 10000 ) {
                                addSessionMsg(session, "Error.errorMsg", "Error: Illegal ip count value");
                                parse_ip_count_failed = true;
                            }
                        }

                        if( !parse_ip_count_failed ) {
                            if( ip_count == 1 )
                                ip_count = 0;
                            Hashtable intfc = nHash(nHash(newNode.get("interfaces")).get(target));
                            if( requested_addr_space != null ) {
                                requested_addr_space = decode(requested_addr_space);
                                intfc.put("saved addr space", requested_addr_space);
                            } else
                                intfc.remove("saved addr space");

                            intfc.put("saved ip addr", requested_ip_addr);
                            intfc.put("saved ip name", requested_ip_name);
                            if( requested_ip_status == null )
                                intfc.put("saved ip status inactive", "");
                            else
                                intfc.remove("saved ip status inactive");

                            Hashtable ips = (Hashtable)intfc.get("ips");
                            if( ips == null ) {
                                ips = hash();
                                intfc.put("ips", ips);
                            }
                            if( (requested_addr_space == null || requested_addr_space.trim().length() == 0) &&
                                requested_ip_addr != null && requested_ip_addr.indexOf("/") > -1 ) {
                                requested_addr_space = requested_ip_addr;
                                requested_ip_addr = null;
                            }
                            if( ( requested_ip_addr == null || requested_ip_addr.trim().length() == 0 ) &&
                                ( requested_addr_space == null || requested_addr_space.trim().length() == 0 ) )
                                addSessionMsg(session, "Error.errorMsg", "Error: Please provide address space or ip address to reserve.");
                            else {
                                if( reserveIPandName(session, netdb, ips, store,
                                                     new Boolean(requested_ip_status!=null?true:false),
                                                     requested_ip_addr, requested_addr_space,
                                                     requested_ip_name, ip_count, "IP", true, false) ) {
                                    intfc.remove("saved addr space");
                                    intfc.remove("saved ip status inactive");
                                    intfc.remove("saved ip addr");
                                    intfc.remove("saved ip name");
                                    intfc.remove("raw ip count");
                                }
                            }
                        }
                    } // end of loop over targets
                } // end of action "reserve ip and add"

                if( action.equals("reserve ip") || reserve_ip ) {
                    if( targets == null )
                        targets = new String[0];
                    for( int k = 0; k < targets.length; k++ ) {
                        target = targets[k];
                        String requested_ip_addr = getDecodedParameter(request, "new ip txt "+target);
                        if( requested_ip_addr != null && requested_ip_addr.trim().length() > 0 )
                            requested_ip_addr = Utils.normIp(requested_ip_addr);
                        String requested_ip_name = ((String)Utils.n2s(getDecodedParameter(request, "new ip name "+target))).trim();
                        String requested_ip_status = getDecodedParameter(request, "new ip active "+target);
                        String requested_addr_space = ((String)Utils.n2s(getDecodedParameter(request, "addr space "+target))).trim();
                        if( requested_addr_space != null )
                            requested_addr_space = decode(requested_addr_space);

                        boolean parse_ip_count_failed = false;
                        int ip_count = -1;
                        String requested_ip_count = getDecodedParameter(request, "new ip count "+target);

                        if( requested_ip_count == null || requested_ip_count.trim().length() == 0 )
                            ip_count = 1;

                        if( ip_count == -1 ) {
                            try {
                                ip_count = Integer.parseInt(requested_ip_count);
                            } catch (Exception pex) {
                                addSessionMsg(session, "Error.errorMsg", "Error: Illegal ip count value");
                                parse_ip_count_failed = true;
                            }
                        }

                        if( !parse_ip_count_failed ) {
                            if( ip_count < 1 || ip_count > 10000 ) {
                                addSessionMsg(session, "Error.errorMsg", "Error: Illegal ip count value");
                                parse_ip_count_failed = true;
                            }
                        }

                        if( !parse_ip_count_failed ) {
                            if( ip_count == 1 )
                                ip_count = 0;

                            Hashtable intfc = nHash(nHash(newNode.get("interfaces")).get(target));
                            Hashtable ips = (Hashtable)intfc.get("ips");
                            if( ips == null ) {
                                ips = hash();
                                intfc.put("ips", ips);
                            }
                            if( (requested_addr_space == null || requested_addr_space.trim().length() == 0) &&
                                requested_ip_addr != null && requested_ip_addr.indexOf("/") > -1 ) {
                                requested_addr_space = requested_ip_addr;
                                requested_ip_addr = null;
                            }

                            if( ( requested_ip_addr == null || requested_ip_addr.trim().length() == 0 ) &&
                                ( requested_addr_space == null || requested_addr_space.trim().length() == 0 ) ) {
                                if( !reserve_ip )
                                    addSessionMsg(session, "Error.errorMsg",
                                                  "Error: Please provide address space or ip address to reserve.");

                            } else {
                                if( reserveIPandName(session, netdb, ips, store,
                                                     new Boolean(requested_ip_status!=null?true:false),
                                                     requested_ip_addr, requested_addr_space,
                                                     requested_ip_name, ip_count, "IP", true, false) ) {
                                    intfc.remove("show select");
                                    intfc.remove("saved ip addr");
                                    intfc.remove("saved ip name");
                                    intfc.remove("saved ip status inactive");
                                    intfc.remove("saved addr space");

                                } else {
                                    if( requested_ip_addr != null )
                                        intfc.put("saved ip addr", requested_ip_addr);
                                    else
                                        intfc.remove("saved ip addr");

                                    if( requested_ip_name != null )
                                        intfc.put("saved ip name", requested_ip_name);
                                    else
                                        intfc.remove("saved ip name");

                                    if( requested_addr_space != null )
                                        intfc.put("saved addr space", requested_addr_space);
                                    else
                                        intfc.remove("saved addr space");

                                    if( requested_ip_status == null )
                                        intfc.put("saved ip status inactive", "");
                                    else
                                        intfc.remove("saved ip status inactive");
                                }
                            }
                        }
                    }
                } // end of 'if( action.equals("reserve ip") )'

                if( reserve_ip || reserve_ipc ) {
                    // some new ip addresses and names may have been reserved
                    // here outside of 'Verify' block
                    oldNewNode = Utils.copyHash(newNode);
                    store.put("oldNewNode", oldNewNode);
                }

                if( action.equals("add ipc") )
                    newNode.put("show ipc select", "");

                if( action.equals("add ip") )
                    nHash(nHash(newNode.get("interfaces")).get(target)).put("show select", "");

                if( !action.equals("verify_node_admins") && !action.equals("Cancel") &&
                    !action.equals("Create OS") && !action.equals("Create Model") )
                    nHash(newNode.get("admins")).put("raw data", request.getParameter("admins"));

                if( !action.equals("verify_node_users") && !action.equals("Cancel") &&
                    !action.equals("Create OS") && !action.equals("Create Model") )
                    newNode.put("users raw data", request.getParameter("users"));

                if( action.equals("Add another interface") ) {
                    Hashtable interfaces = (Hashtable)newNode.get("interfaces");
                    if( interfaces == null ) {
                        interfaces = hash();
                        newNode.put("interfaces", interfaces);
                    }

                    boolean got_error = false;
                    int add_count = 1;
                    String raw_interfaces_count = (String)Utils.n2s(newNode.get("raw_interfaces_count"));
                    if( raw_interfaces_count.length() > 0 ) {
                        try {
                            add_count = Integer.parseInt(raw_interfaces_count);
                            if( add_count < 0 )
                                throw new Exception ("");
                        } catch (Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", "Error: Illegal count value '"+raw_interfaces_count+"'.");
                            got_error = true;
                        }
                    }

                    if( got_error )
                        newNode.put("interfaces_count_error", "");
                    else {
                        newNode.remove("raw_interfaces_count");
                        newNode.remove("interfaces_count_error");

                        for( int i = 0; i < add_count; i++ ) {
                            interfaces.put(" i"+getNextKey(),
                                           hash("roaming?",   Boolean.FALSE,
                                                "use-dhcp?",  Boolean.FALSE,
                                                "just added", "")); // this will prevented it from been removed down the line;
                        }
                    }
                }

                if( action.equals("verify_node_admins") || action.equals("Save") || action.equals("Verify") ) {
                    nHash(newNode.get("admins")).remove("raw data");

                    String adminsStr = request.getParameter("admins");
                    String newAdminsStr = "";
                    String[] split = adminsStr.split("\\n");
                    if( split != null ) {
                        boolean multiple_matches = false;
                        boolean verified = false;
                        boolean no_match = false;

                        for( int i = 0; i < split.length; i++ ) {
                            String searchStr = split[i].trim();
                            if( searchStr.length() == 0 )
                                continue;
                            Hashtable foundPeople = null;
                            Hashtable foundATeam = null;
                            try {
                                try {
                                    boolean got_match = false;
                                    if( searchStr.endsWith("@") )
                                        foundPeople =
                                            netdb.dirSearch(Constants.NETID,
                                                            searchStr.substring(0, searchStr.length()-1));

                                    else if (searchStr.endsWith(";") )
                                        foundATeam = netdb.getInfo(Constants.ATEAM,
                                                                   searchStr.substring(0, searchStr.length()-1));

                                    else
                                        foundPeople = netdb.dirSearch(Constants.LOOKUP, searchStr);

                                    if( foundPeople == null && foundATeam == null )
                                        no_match = true;

                                    // fill people in
                                    Hashtable adminsM = Utils.get_hash_or_create(newNode, "admins");
                                    Hashtable admins  = Utils.get_hash_or_create(adminsM, "admins");

                                    if( foundPeople != null ) {
                                        if( foundPeople.size() > 0 ) {
                                            verified = true;
                                            got_match = true;
                                        }
                                        if( foundPeople.size() > 1 ) {
                                            multiple_matches = true;
                                            got_match = true;
                                        }
                                        if( foundPeople.size() == 0 )
                                            no_match = true;
                                        fillInPeople(admins, foundPeople);
                                    }

                                    // fill in ateams
                                    Hashtable ateams = Utils.get_hash_or_create(adminsM, "ateams");
                                    if( foundATeam != null ) {
                                        if( foundATeam.size() == 0 )
                                            no_match = true;
                                        if( foundATeam.get("id") != null ) {
                                            verified = true;
                                            got_match = true;
                                            ateams.put(foundATeam.get("name"), foundATeam);
                                        } else
                                            no_match = true;
                                    }

                                    if( !got_match )
                                        newAdminsStr += (newAdminsStr.length()>0?"\n":"")+
                                            searchStr;

                                } catch (NetDBException ex) {
                                    Vector errors = ex.getMessages();
                                    String msg = "";
                                    int esize = errors.size();
                                    for( int j = 0; j < esize; j++ )
                                        msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                                    throw new Exception(msg);
                                }

                            } catch (Exception ex) {
                                addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                            }
                        }

                        if( multiple_matches ) {
                            addSessionMsg(session, "Error.errorMsg", "ERROR: Multiple matches for Node Administrator.");
                            Utils.get_hash_or_create(newNode, "admins").put("raw data", newAdminsStr); // save raw data
                        } else {
                            if( no_match ) {
                                addSessionMsg(session, "Error.errorMsg", "ERROR: No matches for Node Administrator.");
                                Utils.get_hash_or_create(newNode, "admins").put("raw data", newAdminsStr); // save raw data
                            } else if( verified )
                                addSessionMsg(session, "info_msg", "SUCCESS: Node Administrators verified.");
                        }
                    }
                }

                if( action.equals("verify_node_users") || action.equals("Save") || action.equals("Verify") ) {
                    newNode.remove("users raw data");
                    String usersStr = request.getParameter("users");
                    String newUsersStr = "";
                    String[] split = usersStr.split("\\n");
                    if( split != null ) {
                        boolean multiple_matches = false;
                        boolean verified = false;
                        boolean no_match = false;
                        for( int i = 0; i < split.length; i++ ) {
                            String userStr = split[i].trim();
                            if( userStr.length() == 0 )
                                continue;
                            Hashtable foundPeople;
                            try {
                                try {
                                    boolean got_match = false;
                                    if( userStr.endsWith("@") )
                                        foundPeople =
                                            netdb.dirSearch(Constants.NETID,
                                                            userStr.substring(0, userStr.length()-1));
                                    else
                                        foundPeople = netdb.dirSearch(Constants.LOOKUP, userStr);

                                    // fill them in
                                    Hashtable users = Utils.get_hash_or_create(newNode, "users");

                                    if( foundPeople == null )
                                        no_match = true;
                                    else if( foundPeople.size() == 0 )
                                        no_match = true;
                                    else {
                                        verified = true;
                                        got_match = true;
                                    }

                                    if( foundPeople.size() > 1 )
                                        multiple_matches = true;

                                    if( !got_match )
                                        newUsersStr += (newUsersStr.length()>0?"\n":"")+
                                            userStr;

                                    fillInPeople(users, foundPeople);

                                } catch (NetDBException ex) {
                                    Vector errors = ex.getMessages();
                                    String msg = "";
                                    int esize = errors.size();
                                    for( int j = 0; j < esize; j++ )
                                        msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                                    throw new Exception(msg);
                                }

                            } catch (Exception ex) {
                                addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                            }
                        }

                        if( multiple_matches ) {
                            addSessionMsg(session, "Error.errorMsg", "ERROR: Multiple matches for Node User.");
                            newNode.put("users raw data", newUsersStr); // save raw data
                        } else {
                            if( no_match ) {
                                addSessionMsg(session, "Error.errorMsg", "ERROR: No matches for Node User.");
                                newNode.put("users raw data", newUsersStr); // save raw data

                            } else if( verified )
                                addSessionMsg(session, "info_msg", "SUCCESS: Node Users verified.");
                        }
                    }

                } else if( action.equals("add_os") ) {
                    Hashtable oses = Utils.get_hash_or_create(newNode, "oses");
                    oses.put(" --None--"+getNextKey(), "");

                } else if( action.equals("add_mx_name") ) {
                    // at this point it guaranteed that mxes hash does exist
                    Hashtable nameH = (Hashtable)((Hashtable)newNode.get("names")).get(target);
                    Hashtable mxs = (Hashtable)nameH.get("mxs");

                    boolean got_error = false;
                    int add_count = 1;
                    String raw_mxs_count = (String)Utils.n2s(nameH.get("raw_mxs_count"));
                    if( raw_mxs_count.length() > 0 ) {
                        try {
                            add_count = Integer.parseInt(raw_mxs_count);
                            if( add_count < 0 )
                                throw new Exception ("");
                        } catch (Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", "Error: Illegal count value '"+raw_mxs_count+"'.");
                            got_error = true;
                        }
                    }

                    if( got_error )
                        nameH.put("mxs_count_error", "");
                    else {
                        nameH.remove("raw_mxs_count");
                        nameH.remove("mxs_count_error");

                        for( int i = 0; i < add_count; i++ ) {
                            mxs.put(getNextKey(),
                                    hash("name",   "",
                                         "domain", "",
                                         "pref",   "10" /* default value */ ));
                        }
                    }

                } else if( action.equals("add_alias_name") ) {
                    // at this point it guaranteed that aliases hash does exist
                    Hashtable nameH = (Hashtable)((Hashtable)newNode.get("names")).get(target);
                    Hashtable aliases = (Hashtable)nameH.get("aliases");

                    boolean got_error = false;
                    int add_count = 1;
                    String raw_aliases_count = (String)Utils.n2s(nameH.get("raw_aliases_count"));
                    if( raw_aliases_count.length() > 0 ) {
                        try {
                            add_count = Integer.parseInt(raw_aliases_count);
                            if( add_count < 0 )
                                throw new Exception ("");
                        } catch (Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", "Error: Illegal count value '"+raw_aliases_count+"'.");
                            got_error = true;
                        }
                    }

                    if( got_error )
                        nameH.put("aliases_count_error", "");
                    else {
                        nameH.remove("raw_aliases_count");
                        nameH.remove("aliases_count_error");

                        for( int i = 0; i < add_count; i++ ) {
                            aliases.put(getNextKey(),
                                        hash("name", "",
                                             "domain", ""));
                        }
                    }

                } else if( action.equals("add_node_name") ) {
                    boolean got_error = false;
                    int add_count = 1;
                    String raw_names_count = (String)Utils.n2s(newNode.get("raw_names_count"));
                    if( raw_names_count.length() > 0 ) {
                        try {
                            add_count = Integer.parseInt(raw_names_count);
                            if( add_count < 0 )
                                throw new Exception ("");
                        } catch (Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", "Error: Illegal count value '"+raw_names_count+"'.");
                            got_error = true;
                        }
                    }

                    if( got_error )
                        newNode.put("names_count_error", "");
                    else {
                        newNode.remove("raw_names_count");
                        newNode.remove("names_count_error");

                        for( int i = 0; i < add_count; i++ ) {
                            ((Hashtable)newNode.get("names")).
                                put(getNextKey(), hash("name", "", "domain", ""));
                        }
                    }

                } else if( action.equals("display_dhcp_options") ) {
                    nHash(nHash(newNode.get("interfaces")).get(target)).
                        put("dhcp-options", hash(" ", hash("option", "", "value", "")));

                } else if( action.equals("Reset") ) {
                    store.put("newNode", Utils.copyHash((Hashtable)store.get("oldNode")));
                    newNode = (Hashtable)store.get("newNode");

                }

                if( action.equals("Save") &&
                    session.getAttribute("Error.errorMsg") == null ) {
                    session.removeAttribute("info_msg");
                    session.removeAttribute("Error.errorMsg");
                    try {
                        try {
                            Utils.purgeObject(newNode);
                            Utils.validate(Constants.NODE, newNode, Constants.NARROW);
                            purgeEmptyInterfaces(newNode);

                            // extract handle out of node hash
                            Hashtable names = nHash(newNode.get("names"));
                            if( names.size() == 0 )
                                names.put(getNextKey(), hash("name", "", "domain", ""));

                            String handle_fqdn = firstName(names);

                            Hashtable personsForUpdate = hash();
                            Hashtable users = nHash(newNode.get("users"));
                            int index = 0;
                            for( Object key : users.keySet() ) {
                                Hashtable person = (Hashtable)users.get(key);
                                if( ((String)Utils.n2s(person.get("object"))).equals("person") )
                                    personsForUpdate.put(""+(index++), person);
                            }
                            Hashtable admins = nHash(nHash(newNode.get("admins")).get("admins"));
                            for( Object key : admins.keySet() ) {
                                Hashtable person = (Hashtable)admins.get(key);
                                if( ((String)Utils.n2s(person.get("object"))).equals("person") )
                                    personsForUpdate.put(""+(index++), person);
                            }

                            updatePersons(netdb, personsForUpdate);

                            Hashtable results = hash();
                            if( rootURL.equals("/ModifyNode") ) {
                                results = netdb.update((Hashtable)store.get("oldNode"), newNode);
                                session.setAttribute("info_msg", "SUCCESS: Node '"+handle_fqdn+"' has been updated.");
                            } else {
                                results = netdb.insert(newNode);
                                session.setAttribute("info_msg", "SUCCESS: Node '"+handle_fqdn+"' has been created.");
                            }

                            Vector warnings = (Vector)results.get("warnings");
                            if( warnings != null ) {
                                for( int i = 0; i < warnings.size(); i++ )
                                    addSessionMsg(session, "Warning", ((String)warnings.elementAt(i)).trim());
                            }

                            String fromFromPage = (String)store.get("fromFromPage");
                            response.sendRedirect("/Dispatch?Action=info&object=Node&fromPage="+
                                                  encode(fromFromPage!=null?fromFromPage:(String)store.get("fromPage"))+
                                                  "&handle="+handle_fqdn);
                            return;

                        } catch (NetDBException ex) {
                            ex.printStackTrace();
                            Vector errors = ex.getMessages();
                            String msg = "";
                            int esize = errors.size();
                            for( int j = 0; j < esize; j++ )
                                msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                            throw new Exception(msg);
                        }
                    } catch (Exception ex) {
                        ex.printStackTrace();
                        addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                    }
                } // end of 'if( action.equals("Save") )'

                if( action.equals("Verify") ) {
                    try {
                        Utils.validate(Constants.NODE, newNode, Constants.NARROW);

                    } catch (NetDBException ex) {
                        Vector messages = ex.getMessages();
                        if( messages != null && messages.size() > 0 ) {
                            for( int i = 0; i < messages.size(); i++ )
                                addSessionMsg(session, "Error.errorMsg", "Error: "+messages.elementAt(i));
                        }
                    } catch (Exception ex) {
                        addSessionMsg(session, "Error.errorMsg", "Error: "+ex.getMessage());
                    }
                }

            } else
                System.out.println("NOT READY");

            printOutModifyHTML(session, out, store, rootURL,
                               request.getParameter("PosX"),
                               request.getParameter("PosY"));

        } catch (Exception ex) {
            ex.printStackTrace();
            // if we are here something unexpected happened
            // we should never end up here
            String msg = "";
            try {
                if( lockTm != null ) {
                    if( Utils.isNumber(handle) )
                        netdb.unlock(Constants.NODE, Integer.parseInt(handle), lockTm);
                    else
                        netdb.unlock(Constants.NODE, handle, lockTm);
                }
            } catch (NetDBException exx) {
                /*
                  Vector errors = exx.getMessages();
                  int esize = errors.size();
                  for( int i = 0; i < esize; i++ )
                  msg += "Error: "+(String)errors.elementAt(i)+(i<esize-1?"<br>\n":"");
                */
            } catch (Exception exx) {
                msg += (msg.length()>0?"<br>\n":"")+"Error: "+exx.getMessage();
            }

            msg += (msg.length()>0?"<br>\n":"")+"Error: "+ex.getMessage();
            addSessionMsg(session, "Error.errorMsg", msg);
            response.sendRedirect((String)store.get("fromPage"));
        }
    } // end of doPost(...)

    public void printOutModifyHTML(HttpSession session, PrintWriter out, Hashtable store,
                                   String rootURL, String pos_x, String pos_y)
        throws Exception
    {
        if( store.get("new_os") != null ) {
            newOSHTML(session, out, store, rootURL);
            return;

        } else if( store.get("new_model") != null ) {
            newModelHTML(session, out, store, rootURL);
            return;
        }

        int custom_fields_total = ((Integer)getData(session, "custom_fields_total")).intValue();

        String default_domain   = (String)session.getAttribute("default_domain");
        String sessionKey       = (String)store.get("sessionKey");
        Timestamp lockTm        = (Timestamp)store.get("lockTm");
        Hashtable newNode       = (Hashtable)store.get("newNode");
        Hashtable oldNode       = (Hashtable)store.get("oldNode");
        Hashtable cnvs          = nHash(getData(session, "custom.node.views"));
        Hashtable my_node_types = nHash(getData(session, "my node types"));
        String handle           = (String)store.get("handle");
        String object           = (String)store.get("object");

        Hashtable nodeOSes      = nHash(newNode.get("oses"));

        if( Utils.nStr(session.getAttribute("Error.errorMsg")).trim().length() > 0  ) pos_x = pos_y = "0";

        out.println(getCommonHtmlBegin((rootURL.equals("/ModifyNode")?("NodeModify: "+object):"NodeCreate"),
                                       "<script language=\"JavaScript\">\n"+
                                       "<!--\n"+
                                       " String.prototype.trim=\n"+
                                       "  new Function(\"return this.replace(/^\\\\s+|\\\\s+$/g,'')\")\n\n"+

                                       " function alert_animate_field(id) {\n"+
                                       "  field = document.getElementById(id);\n"+
                                       "  field.style.backgroundColor = '#ff0000'\n"+
                                       "  setTimeout(\"field.style.backgroundColor = '#ff1111';\", 10);\n"+
                                       "  setTimeout(\"field.style.backgroundColor = '#ff2222';\", 27);\n"+
                                       "  setTimeout(\"field.style.backgroundColor = '#ff3333';\", 34);\n"+
                                       "  setTimeout(\"field.style.backgroundColor = '#ff4444';\", 43);\n"+
                                       "  setTimeout(\"field.style.backgroundColor = '#ff5555';\", 54);\n"+
                                       "  setTimeout(\"field.style.backgroundColor = '#ff6666';\", 67);\n"+
                                       "  setTimeout(\"field.style.backgroundColor = '#ff7777';\", 82);\n"+
                                       "  setTimeout(\"field.style.backgroundColor = '#ff8888';\", 99);\n"+
                                       "  setTimeout(\"field.style.backgroundColor = '#ff9999';\", 118);\n"+
                                       "  setTimeout(\"field.style.backgroundColor = '#ffaaaa';\", 139);\n"+
                                       "  setTimeout(\"field.style.backgroundColor = '#ffbbbb';\", 163);\n"+
                                       "  setTimeout(\"field.style.backgroundColor = '#ffcccc';\", 186);\n"+
                                       "  setTimeout(\"field.style.backgroundColor = '#ffdddd';\", 212);\n"+
                                       "  setTimeout(\"field.style.backgroundColor = '#ffeeee';\", 240);\n"+
                                       "  setTimeout(\"field.style.backgroundColor = '#ffffff';\", 270);\n"+
                                       " }\n\n"+

                                       " function ensure_proper_roaming(id) {\n"+
                                       "  if( !document.getElementById('CheckBox_DHCP_'+id).checked )\n"+
                                       "   document.getElementById('CheckBox_Roaming_'+id).checked = false;\n"+
                                       "  else if( document.getElementById('TextField_HW_Address_'+id).value.trim().length == 0 )\n"+
                                       "   alert_animate_field('TextField_HW_Address_'+id);\n"+
                                       " }\n\n"+

                                       " function ensure_proper_dhcp(id) {\n"+
                                       "  if( document.getElementById('CheckBox_Roaming_'+id).checked ) {\n"+
                                       "   document.getElementById('CheckBox_DHCP_'+id).checked = true;\n"+
                                       "   if( document.getElementById('TextField_HW_Address_'+id).value.trim().length == 0 )\n"+
                                       "    alert_animate_field('TextField_HW_Address_'+id);\n"+
                                       "  }\n"+
                                       " }\n"+
                                       "//-->\n"+
                                       "</script>", pos_x, pos_y)+
                    commonHeader(session, "/Dispatch?Action=redirect&to=/home&pre=unlock&object=Node&sessionKey="+
                                 encode(sessionKey),
                                 SHOW_HOME_LINK, NO_PROFILE_LINK, NO_LOGOUT_LINK,
                                 "", (rootURL.equals("/ModifyNode")?("Modify Node<br>"+object):"Create Node"),
                                 "hlpnode.html")+
                    msgPanels(session)+
                    "<DIV id=\"infoPage\">\n"+
                    "<FORM NAME=\"ModifyNode\" METHOD=\"POST\" "+
                    "onSubmit=\"document.ModifyNode.PosX.value = (document.all)?document.body.scrollLeft:window.pageXOffset; "+
                    "document.ModifyNode.PosY.value = (document.all)?document.body.scrollTop:window.pageYOffset;\" ACTION='"+rootURL+"'>"+
                    "<INPUT TYPE='HIDDEN' VALUE='' NAME='PosX'>"+
                    "<INPUT TYPE='HIDDEN' VALUE='' NAME='PosY'>"+
                    "<P CLASS=\"form_buttons\">\n"+
                    "<INPUT ACCESSKEY=\"S\" ID=\"Button_Save\" TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Save\"> "+
                    "<INPUT ACCESSKEY=\"V\" ID=\"Button_Verify\" TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Verify\">\n"+
                    "<INPUT ACCESSKEY=\"R\" ID=\"Button_Reset\" TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Reset\">\n"+
                    "<INPUT ACCESSKEY=\"C\" ID=\"Button_Cancel\" TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Cancel\">\n"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"sessionKey\" VALUE=\""+encode(sessionKey)+"\">"+
                    "</P>\n"+
                    "<DIV class='modHeader'>\n"+
                    "<TABLE WIDTH='100%'>\n"+
                    "<TR><TD WIDTH='1%' NOWRAP><B>Node&nbsp;Information</B><BR>(<font class='star'>*</font>&nbsp;=&nbsp;required)</TD>\n"+
                    "<TD VALIGN='TOP' ALIGN='RIGHT'>Special node types:&nbsp;&nbsp;");
        // collect node types and display them
        Hashtable types = nHash(newNode.get("types"));
        boolean exclude_template = false;
        if( find_by_key(types, "extkey", "Template", "type") != null )
            out.println("<INPUT ID=\"Button_Template\" TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"[X] Template\">");
        else {
            if( types.size() > 0 )
                exclude_template = true;

            for( Map.Entry ent : (java.util.List<Map.Entry>)Utils.sort(my_node_types.entrySet(), Utils.node_type_cmp) ) {
                String type         = (String)ent.getKey();
                String extkey       = Utils.nStr(((Hashtable)ent.getValue()).get("extkey"));
                if( extkey.equals("Template") && exclude_template ) continue;
                out.println("<INPUT ID=\"Button_"+extkey+"\" TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"["+
                            (types.get(type)!=null?"X":"  ")+"] "+type+"\">\n");
            }
        }
        out.println("</TD></TR>\n"+
                    "</TABLE>\n"+
                    "</DIV>"+
                    "<TABLE CELLSPACING='0' WIDTH='100%'>\n");

        // node names
        Hashtable names = (Hashtable)newNode.get("names");
        Hashtable reserved_names = nHash(store.get("reserved_names"));
        Hashtable troubled_names = nHash(store.get("troubled_names"));
        Hashtable troubled_hws = nHash(store.get("troubled_hws"));

        String raw_names_count = ((String)Utils.n2s(newNode.get("raw_names_count"))).trim();
        java.util.List list = Arrays.asList(names.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();
        int index = 0;
        while( it.hasNext() ) {
            String key = (String)it.next();
            Hashtable name  = (Hashtable)names.get(key);
            String domain = ((String)name.get("domain")).trim();
            String displayName = ((String)name.get("name")).trim();
            String fqdn = displayName+"."+domain;
            if( displayName.length() > 0 && !domain.equalsIgnoreCase(default_domain) )
                displayName += "."+domain;

            out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font>&nbsp;Name</TD>\n"+
                        "<TD VALIGN='TOP'>"+
                        (name.get("error")!=null?("<INPUT TYPE='Hidden' NAME='"+encode("node_name_error_"+key)+"' VALUE=' '>"):"")+
                        (names.size()==1?("<INPUT TYPE='Hidden' NAME='"+encode("node_name_cb_"+key)+"' VALUE='YES'>"):
                         ("<INPUT ID='CheckBox_Node_Name_"+index+"' TYPE='checkbox' name='"+encode("node_name_cb_"+key)+
                          "' VALUE='YES' CHECKED>"))+"<INPUT ID='TextField_Node_Name_"+index+"' "+
                        (nameIn(Constants.NODE_NAME_TYPE, fqdn, troubled_names)?"CLASS='error_form_font' ":
                         (nameIn(Constants.NODE_NAME_TYPE, fqdn, reserved_names)?"CLASS='reserved_form_font' ":""))+
                        "TYPE='TEXT' SIZE='27' NAME='"+
                        encode("node_name_tf_"+key)+"' value='"+
                        Utils.htmlize(displayName, false)+"'>&nbsp;"+
                        ((find_by_key(types, "extkey", "Advanced", "type")!=null&&index==0)?
                         ("<INPUT ID='Button_Add_Another_Node_Name' TYPE='SUBMIT' NAME='add_node_name' VALUE='Add another'>&nbsp;&nbsp;Count:&nbsp;"+
                          "<INPUT ID='TextField_Node_Name_Count' "+(newNode.get("names_count_error")!=null?"CLASS='error_form_font' ":"")+
                          "TYPE='TEXT' SIZE='4' NAME='names_count' VALUE='"+Utils.htmlize(raw_names_count, false)+"'>"):
                         "")+
                        "</TD></TR>\n");

            if( find_by_key(types, "extkey", "Template", "type") == null ) {
                // print out aliases
                Hashtable aliases = (Hashtable)name.get("aliases");
                if( aliases == null ) {
                    aliases = hash();
                    name.put("aliases", aliases);
                }
                String raw_aliases_count = ((String)Utils.n2s(name.get("raw_aliases_count"))).trim();
                java.util.List alist = Arrays.asList(aliases.keySet().toArray());
                Collections.sort(alist, name_cmp);
                Iterator ait = alist.iterator();
                int aindex = 0;
                while( ait.hasNext() ) {
                    String akey = (String)ait.next();
                    Hashtable alias = (Hashtable)aliases.get(akey);
                    String aname = ((String)alias.get("name")).trim();
                    String adomain = ((String)alias.get("domain")).trim();
                    fqdn = aname+"."+adomain;
                    if( aname.length() > 0 && !adomain.equalsIgnoreCase(default_domain) )
                        aname += "."+adomain;

                    out.println("<TR><TD VALIGN='TOP' ALIGN='RIGHT'>"+(aindex==0?"Alias&nbsp;":"")+"</TD>\n"+
                                "<TD VALIGN='TOP'><INPUT ID='CheckBox_Alias_Name_"+index+"_"+aindex+"' TYPE='checkbox' name='"+
                                encode("alias_name_cb_"+key+" "+akey)+
                                "' VALUE='YES' CHECKED><INPUT ID='TextField_Alias_Name_"+index+"_"+aindex+"' "+
                                (nameIn(Constants.NAME_ALIAS_TYPE, fqdn, troubled_names)?"CLASS='error_form_font' ":
                                 (nameIn(Constants.NAME_ALIAS_TYPE, fqdn, reserved_names)?"CLASS='reserved_form_font' ":""))+
                                "TYPE='TEXT' SIZE='27' NAME='"+
                                encode("alias_name_tf_"+key+" "+akey)+"' value='"+Utils.htmlize(aname, false)+"'>"+
                                (aindex==0?"&nbsp;<INPUT ID='Button_Add_Another_Alias_"+index+"' TYPE='SUBMIT' NAME='"+
                                 encode("add_alias_name_"+key)+
                                 "' VALUE='Add another'>&nbsp;&nbsp;Count:&nbsp;"+
                                 "<INPUT ID='TextField_Alias_Count_"+index+"' "+
                                 (name.get("aliases_count_error")!=null?"CLASS='error_form_font' ":"")+
                                 "TYPE='TEXT' SIZE='4' NAME='"+encode("aliases_count_tf_"+key)+"' VALUE='"+
                                 Utils.htmlize(raw_aliases_count, false)+"'>":"")+
                                "</TD></TR>\n");
                    aindex++;
                }
                if( aliases.size() == 0 )
                    out.println("<TR><TD></TD><TD VALIGN='TOP'><INPUT ID='Button_Add_Alias_"+index+"' TYPE='SUBMIT' NAME='"+
                                encode("add_alias_name_"+key)+"' VALUE='Add Alias'>&nbsp;&nbsp;Count:&nbsp;"+
                                 "<INPUT ID='TextField_Alias_Count_"+index+"' "+(name.get("aliases_count_error")!=null?"CLASS='error_form_font' ":"")+
                                 "TYPE='TEXT' SIZE='4' NAME='"+encode("aliases_count_tf_"+key)+"' VALUE='"+
                                Utils.htmlize(raw_aliases_count, false)+"'></TD></TR>\n");

                // print out mxes
                Hashtable mxs = (Hashtable)name.get("mxs");
                if( mxs == null ) {
                    mxs = hash();
                    name.put("mxs", mxs);
                }
                Hashtable bad_mx_pref = nHash(newNode.get("bad_mx_pref"));
                String raw_mxs_count = ((String)Utils.n2s(name.get("raw_mxs_count"))).trim();
                java.util.List mlist = Arrays.asList(mxs.keySet().toArray());
                Collections.sort(mlist, name_cmp);
                Iterator mit = mlist.iterator();
                int mindex = 0;
                while( mit.hasNext() ) {
                    String mkey = (String)mit.next();
                    Hashtable mx = (Hashtable)mxs.get(mkey);
                    String mname = ((String)mx.get("name")).trim();
                    String mdomain = ((String)mx.get("domain")).trim();
                    String pref = ((String)mx.get("pref")).trim();
                    fqdn = mname+"."+mdomain;
                    if( mname.length() > 0 && !mdomain.equalsIgnoreCase(default_domain) )
                        mname += "."+mdomain;

                    out.println("<TR><TD VALIGN='TOP' ALIGN='RIGHT'>"+(mindex==0?"Receive mail for&nbsp;":"")+"</TD>\n"+
                                "<TD VALIGN='TOP'><INPUT ID='CheckBox_MX_Name_"+index+"_"+mindex+"' TYPE='checkbox' name='"+
                                encode("mx_name_cb_"+key+" "+mkey)+
                                "' VALUE='YES' CHECKED><INPUT ID='TextField_MX_Name_"+index+"_"+mindex+"' "+
                                (nameIn(Constants.MX_TYPE, fqdn, troubled_names)?"CLASS='error_form_font' ":
                                 (nameIn(Constants.MX_TYPE, fqdn, reserved_names)?"CLASS='reserved_form_font' ":""))+
                                "TYPE='TEXT' SIZE='27' NAME='"+
                                encode("mx_name_tf_"+key+" "+mkey)+"' value='"+Utils.htmlize(mname, false)+"'> "+
                                "<INPUT ID='TextField_MX_Name_Pref_"+index+"_"+mindex+"' "+
                                (bad_mx_pref.get(pref.trim().toLowerCase())!=null?"CLASS='error_form_font' ":"")+
                                "TYPE='TEXT' SIZE='5' NAME='"+
                                encode("mx_pref_tf_"+key+" "+mkey)+"' value='"+Utils.htmlize(pref, false)+"'>"+
                                (mindex==0?"&nbsp;<INPUT ID='Button_Add_Another_MX_"+index+"' TYPE='SUBMIT' NAME='"+
                                 encode("add_mx_name_"+key)+
                                 "' VALUE='Add another'>&nbsp;&nbsp;Count:&nbsp;"+
                                 "<INPUT ID='TextField_MX_Count_"+index+"' "+(name.get("mxs_count_error")!=null?"CLASS='error_form_font' ":"")+
                                 "TYPE='TEXT' SIZE='4' NAME='"+encode("mxs_count_tf_"+key)+"' VALUE='"+
                                 Utils.htmlize(raw_mxs_count, false)+"'>":"")+
                                "</TD></TR>\n");
                    mindex++;
                }
                if( mxs.size() == 0 )
                    out.println("<TR><TD></TD><TD VALIGN='TOP'><INPUT ID='Button_Add_MX_"+index+"' TYPE='SUBMIT' NAME='"+
                                encode("add_mx_name_"+key)+"' VALUE='Add MX'>&nbsp;&nbsp;Count:&nbsp;"+
                                "<INPUT ID='TextField_MX_Count_"+index+"' "+(name.get("mxs_count_error")!=null?"CLASS='error_form_font' ":"")+
                                "TYPE='TEXT' SIZE='4' NAME='"+encode("mxs_count_tf_"+key)+"' VALUE='"+
                                Utils.htmlize(raw_mxs_count, false)+
                                "'>&nbsp;&nbsp;(Receive mail for other hosts)</TD></TR>\n");
            }

            out.println("<TR><TD COLSPAN='2'><HR NOSHADE SIZE=1></TD></TR>\n");
            index++;
        }

        // state
        String nodeState = (String)Utils.n2s((String)newNode.get("state"));
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font>&nbsp;State</TD>\n"+
                    "<TD VALIGN='TOP'>");
        Hashtable states = (Hashtable)getData(session, "states");
        if( !nodeState.equals("") && states.get(nodeState) == null ) {
            out.println(Utils.htmlize(nodeState)+
                        "<INPUT TYPE=\"Hidden\" NAME=\"state\" VALUE=\""+encode(nodeState)+"\">");
        } else {
            out.println("<SELECT ID='Selector_State' NAME='state' SIZE='1'>");

            list = Arrays.asList(states.entrySet().toArray());
            Collections.sort(list, state_cmp);
            it = list.iterator();
            while( it.hasNext() ) {
                String state = Utils.nStr(((Hashtable)((Map.Entry)it.next()).getValue()).get("name"));
                out.println("<OPTION VALUE='"+encode(state)+"'"+(nodeState.equalsIgnoreCase(state)?"SELECTED":"")+ ">"+Utils.htmlize(state, false)+"</OPTION>");
            }
            out.println("</SELECT>\n");
        }
        out.println("</TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // departments
        String nodeDep = ((String)newNode.get("dept")).trim();
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font>&nbsp;Department</TD>\n"+
                    "<TD VALIGN='TOP'>"+Utils.htmlize(nodeDep, false)+(nodeDep.length()>0?"<BR>":"")+"<em>or choose a different department below</em><BR>\n"+
                    "<SELECT ID='Selector_Department' NAME='department' SIZE='6'>");
        Hashtable departments = (Hashtable)getData(session, "departments");
        list = Arrays.asList(departments.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        while( it.hasNext() ) {
            String dep = ((String)it.next()).trim();
            out.println("<OPTION VALUE='"+encode(dep)+"'>"+Utils.htmlize(dep, false)+"</OPTION>");
        }
        out.println("</SELECT>\n</TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // location
        String nodeLocation = ((String)newNode.get("location")).trim();
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font>&nbsp;Location<br></TD>\n"+
                    "<TD VALIGN='TOP'>"+Utils.htmlize(nodeLocation, false)+(nodeLocation.length()>0?"<BR>":"")+
                    "<em>or choose a different location below&nbsp;&nbsp;&nbsp;&nbsp;"+
                    "<a ID='Link_List_Of_Locations' href='/LocationsList' target='_blank'>List</a></em><BR>\n"+
                    "<SELECT ID='Selector_Location' NAME='location' SIZE='6'>");
        Hashtable locations = (Hashtable)getData(session, "locations");
        list = Arrays.asList(locations.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        while( it.hasNext() ) {
            String loc = ((String)it.next()).trim();
            out.println("<OPTION VALUE='"+encode(loc)+"'>"+Utils.htmlize(loc)+"</OPTION>");
        }
        out.println("</SELECT>\n</TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // room
        String room = ((String)newNode.get("room")).trim();
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font>&nbsp;Room</TD>\n"+
                    "<TD VALIGN='TOP'><INPUT ID='TextField_Room' TYPE='TEXT' SIZE='20' NAME='room' value=\""+
                    Utils.htmlize(room, false)+
                    "\"></TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // models
        String nodeMake = ((String)newNode.get("make")).trim();
        String nodeModel = ((String)newNode.get("model")).trim();
        String makeANDmodel = Utils.htmlize(nodeMake, false)+" "+Utils.htmlize(nodeModel, false);
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font>&nbsp;Make&nbsp;and&nbsp;Model</TD>\n"+
                    "<TD VALIGN='TOP'>"+Utils.htmlize(makeANDmodel, false)+(makeANDmodel.length()>1?"<BR>":"")+"<em>or choose a different "+
                    "Make and Model below</em><BR>\n"+
                    "<SELECT ID='Selector_Make_And_Model' NAME='make_and_model' SIZE='4'>");
        Hashtable models = (Hashtable)getData(session, "models");
        list = Arrays.asList(models.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        while( it.hasNext() ) {
            String key = (String)it.next();
            Hashtable make_and_model = (Hashtable)models.get(key);
            String make = ((String)make_and_model.get("make")).trim();
            String model = ((String)make_and_model.get("model")).trim();
            out.println("<OPTION VALUE='"+encode(key)+"'>"+Utils.htmlize(make, false)+" "+Utils.htmlize(model, false)+"</OPTION>");
        }
        out.println("</SELECT>\n<INPUT ID='Button_New_Model' TYPE='SUBMIT' NAME='new_model' VALUE='New'>\n"+
                    "</TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // os
        Hashtable oses = (Hashtable)getData(session, "oses");
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font>&nbsp;Operating&nbsp;System</TD>\n"+
                    "<TD VALIGN='TOP'>");
        if( nodeOSes.size() == 0 )
            nodeOSes.put(" --None--"+getNextKey(), "");
        list = Arrays.asList(nodeOSes.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        index = 0;
        while( it.hasNext() ) {
            String nodeOS = (String)it.next();
            out.println("<SELECT ID='Selector_OS' SIZE=1 NAME='"+encode("os_"+index)+"'>\n"+
                        "<OPTION VALUE='"+encode("--None--")+"'>--None--");
            java.util.List list2 = Arrays.asList(oses.keySet().toArray());
            Collections.sort(list2, name_cmp);
            Iterator it2 = list2.iterator();
            while( it2.hasNext() ) {
                String os = (String)it2.next();
                out.println("<OPTION VALUE='"+encode(os)+"'"+(os.equals(nodeOS)?" SELECTED":"")+">"+Utils.htmlize(os, false));
            }
            out.println("</SELECT>"+(index==0?("&nbsp;<INPUT ID='Button_Add_Another_OS' TYPE='SUBMIT' NAME='add_os' VALUE='Add another'>&nbsp;"+
                                     "<INPUT ID='Button_New_OS' TYPE='SUBMIT' NAME='new_os' VALUE='New'>"):"")+
                        (it.hasNext()?"<BR>":"")+"\n");
            index++;
        }
        out.println("</TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // admins
        // first rearrange them
        Hashtable allAdmins = (Hashtable)newNode.get("admins");
        if( allAdmins == null ) {
            allAdmins = hash();
            newNode.put("admins", allAdmins);
        }
        Hashtable admins = hash();
        // copy over ateams
        Hashtable ateams = nHash(allAdmins.get("ateams"));
        for( String name : (Set<String>)ateams.keySet() )
            admins.put(name+" ateam", name);

        // now persons
        Hashtable persons = nHash(allAdmins.get("admins"));
        for( String key : (Set<String>)persons.keySet() ) {
            Hashtable person = (Hashtable)persons.get(key);
            person.put("master key", key);
            admins.put(key+" person", person);
        }

        // admins
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font>&nbsp;Administrators</TD>\n"+
                    "<TD VALIGN='TOP'>");
        list = Arrays.asList(admins.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        int counter = 0;
        while( it.hasNext() ) {
            counter++;
            String key = (String)it.next();
            if( key.endsWith(" ateam") ) {
                String name = (String)admins.get(key);
                out.println("<INPUT ID='CheckBox_Admin_"+counter+"' TYPE='checkbox' CHECKED NAME='ateam' value='"+
                            encode(name)+"'>"+name+";<BR>");
            } else {
                Hashtable person = (Hashtable)admins.get(key);
                String dirID = (String)person.get("DirID");
                String regID = (String)person.get("RegID");
                String masterKey = (String)person.get("master key");
                String dep = (String)person.get("department");

                String displayName = (String)person.get("name");
                if( displayName == null || displayName.trim().length() == 0 )
                    displayName = (String)person.get("NetID");
                if( displayName == null || displayName.trim().length() == 0 )
                    displayName = (String)person.get("DirID");
                if( displayName == null || displayName.trim().length() == 0 )
                    displayName = regID;

                boolean active = activep(person);

                out.println("<INPUT ID='CheckBox_Admin_"+counter+"' TYPE='checkbox' "+
                            (person.get("not selected")==null?"CHECKED ":"")+
                            " NAME='admin' value='"+
                            encode(masterKey)+"'>"+
                            (active?"<A HREF=\""+
                             System.getProperty("stanfordwho.url")+dirID.trim()+"\">":"")+
                            displayName+(active?"</A>":"")+
                            ((dep!=null&&dep.trim().length()>0)?" <strong>"+dep+"</strong>":"")+
                            (!active?" ( <em>inactive</em>)":"")+"<BR>");
            }
        }

        out.println("(<em>Name, SUNet ID, or Admin Team, each on its own line. "+
                    "Put an at sign (@) if searching only by SUNet ID, as in SUNetID@. "+
                    "Put a semicolon after Admin Teams, as in pine-team;<em>)<BR>"+
                    "<TEXTAREA ID='TextArea_Admins' NAME='admins' ROWS='3' COLS='21'>"+
                    (nHash(newNode.get("admins")).get("raw data")!=null?Utils.htmlize((String)nHash(newNode.get("admins")).get("raw data"), false):"")+
                    "</TEXTAREA>"+
                    "&nbsp;<INPUT ID='Button_Verify_Node_Admins' TYPE='SUBMIT' NAME='verify_node_admins' VALUE='Verify'>"+
                    "</TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // users
        Hashtable users = nHash(newNode.get("users"));
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Users</TD>\n"+
                    "<TD VALIGN='TOP'>");
        list = Arrays.asList(users.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        counter=0;
        while( it.hasNext() ) {
            counter++;
            String key = (String)it.next();
            Hashtable person = (Hashtable)users.get(key);
            String dirID = (String)person.get("DirID");
            String regID = (String)person.get("RegID");
            String dep = (String)person.get("department");

            String displayName = (String)person.get("name");
            if( displayName == null || displayName.trim().length() == 0 )
                displayName = (String)person.get("NetID");
            if( displayName == null || displayName.trim().length() == 0 )
                displayName = (String)person.get("DirID");
            if( displayName == null || displayName.trim().length() == 0 )
                displayName = regID;

            boolean active = activep(person);

            out.println("<INPUT ID='CheckBox_User_"+counter+"' TYPE='checkbox' "+
                        (person.get("not selected")==null?"CHECKED ":"")+
                        "NAME='user' value='"+
                        encode(key)+"'>"+
                        (active?"<A HREF=\""+
                         System.getProperty("stanfordwho.url")+dirID.trim()+"\">":"")+
                        displayName+(active?"</A>":"")+
                        ((dep!=null&&dep.trim().length()>0)?" <strong>"+dep+"</strong>":"")+
                        (!active?" ( <em>inactive</em>)":"")+"<BR>");
        }

        out.println("(<em>Name or SUNet ID, each on a new line. Put an at sign (@) "+
                    "if searching only by SUNet ID, as in SUNetID@</em>)<BR>"+
                    "<TEXTAREA ID='TextArea_Users' NAME='users' ROWS='3' COLS='21'>"+
                    (newNode.get("users raw data")!=null?Utils.htmlize((String)newNode.get("users raw data"), false):"")+
                    "</TEXTAREA>"+
                    "&nbsp;<INPUT ID='Button_Verify_Node_Users' TYPE='SUBMIT' NAME='verify_node_users' VALUE='Verify'>"+
                    "</TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // custom fields (aka. tags)
        String enabled_view = Utils.nStr(newNode.get("enabled view"));
        String help_url   = Utils.nStr(nHash(cnvs.get(enabled_view)).get("help url")).trim();

        out.println("<TR class='modHeader'><TD WIDTH='100%' COLSPAN='2'><TABLE WIDTH='100%'><TD VALIGN='TOP' NOWRAP>"+
                    "&nbsp;Custom Fields"+
                    (enabled_view.length()!=0?(" in "+
                                               (help_url.length()>0?("<a href=\""+help_url+"\">"+enabled_view+"</a>"):
                                                enabled_view)+
                                               " view"):"")+
                    "</TD><TD WIDTH='100%'>&nbsp;</TD><TD ALIGN='RIGHT' NOWRAP>");

        if( cnvs.get("error") == null ) {
            // first see if we need to display any custom view at all
            boolean display = false;
            it = Utils.sortedKeys(cnvs, name_cmp);
            while( it.hasNext() ) {
                Hashtable view   = (Hashtable)cnvs.get(it.next());
                Hashtable allow_node_types    = nHash(view.get("allow node types"));
                Hashtable disallow_node_types = nHash(view.get("disallow node types"));

                boolean allow_this_view     = allow_node_types.size()==0?true:false;

                for( String node_type : (Set<String>)types.keySet() ) {
                    if( disallow_node_types.get(Utils.nStr(nHash(types.get(node_type)).get("extkey"))) != null ) {
                        allow_this_view = false;
                        break;
                    }
                    if( allow_node_types.get(Utils.nStr(nHash(types.get(node_type)).get("extkey"))) != null )
                        allow_this_view = true;
                }

                if( !allow_this_view ) continue;
                display = true;
                break;
            }

            if( display ) {
                out.println("<INPUT TYPE='HIDDEN' VALUE='' NAME='SuperAction'>"+
                            "Custom View <SELECT ID='Selector_Tag_Custom_View' "+
                            "NAME='custom_view' SIZE='1' "+
                            "onChange='document.ModifyNode.PosX.value = (document.all)?document.body.scrollLeft:window.pageXOffset; "+
                            "document.ModifyNode.PosY.value = (document.all)?document.body.scrollTop:window.pageYOffset; "+
                            "document.ModifyNode.SuperAction.value=\"Go\"; "+
                            "document.ModifyNode.submit()'>"+
                            "<OPTION VALUE='"+encode("--")+"'>--");

                // display all allowed custom view buttons
                it = Utils.sortedKeys(cnvs, name_cmp);
                while( it.hasNext() ) {
                    String view_name = ((String)it.next());
                    Hashtable view   = (Hashtable)cnvs.get(view_name);

                    Hashtable allow_node_types    = nHash(view.get("allow node types"));
                    Hashtable disallow_node_types = nHash(view.get("disallow node types"));

                    boolean allow_this_view     = allow_node_types.size()==0?true:false;

                    for( String node_type : (Set<String>)types.keySet() ) {
                        if( disallow_node_types.get(Utils.nStr(nHash(types.get(node_type)).get("extkey"))) != null ) {
                            allow_this_view = false;
                            break;
                        }
                        if( allow_node_types.get(Utils.nStr(nHash(types.get(node_type)).get("extkey"))) != null )
                            allow_this_view = true;
                    }

                    if( !allow_this_view ) continue;
                    out.println("<OPTION VALUE='"+encode(view_name)+"'"+
                                (enabled_view.equals(view_name)?" SELECTED":"")+">"+
                                Utils.htmlize(view_name, false));
                }
                out.println("</SELECT>&nbsp;"+
                            "<NOSCRIPT><INPUT ID=\"Button_Go_View\" TYPE=\"SUBMIT\" "+
                            "NAME=\"Action\" VALUE=\"Go\">&nbsp;</NOSCRIPT>");
            }
        }
        out.println("</TD></TR></TABLE></TD>");

        Hashtable view_data = nHash(nHash(getData(session, "custom.node.views")).get(enabled_view));
        Hashtable sections = nHash(view_data.get("sections"));

        if( cnvs.get("error") == null && enabled_view.length() != 0 ) {
            // display custom view
            out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;</TD>\n"+
                        "<TD VALIGN='TOP'><TABLE BORDER='0' CELLSPACING='0' CELLPADDING='0'>");

            Hashtable custom_tags = nHash(newNode.get("custom tags "+enabled_view));
            Hashtable tags = nHash(custom_tags.get("entries"));
            sections = nHash(custom_tags.get("sections"));
            it = Utils.sortedKeys(sections, num_cmp);
            while( it.hasNext() ) {
                Hashtable section_h = (Hashtable)sections.get(it.next());
                String section      = Utils.nStr(section_h.get("name"));
                String section_dsp  = Utils.nStr(section_h.get("display label")).trim();
                if( section_dsp.length() > 0 )
                    out.println("<TR><TD COLSPAN='2'><B>"+section_dsp+"</B></TD></TR>");

                int tag_counter = 10;
                Iterator entr_it = Utils.sortedKeys(tags, num_cmp);
                while( entr_it.hasNext() ) {
                    Hashtable entry = (Hashtable)tags.get(entr_it.next());
                    if( !entry.get("section").equals(section) ) continue;
                    String display_label = (String)Utils.n2s(entry.get("display label"));
                    out.println("<TR><TD VALIGN='TOP'>"+display_label+
                                "&nbsp;<BR>&nbsp;</TD><TD VALIGN='TOP'>"+
                                build_html_element(newNode, tag_counter++, entry, enabled_view)+
                                "</TD></TR>");
                }
                if( it.hasNext() && section_dsp.length() > 0 )
                    out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>");
            }
            out.println("</TABLE></TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");
            newNode.put("looked view "+enabled_view, "yes");
        }

        Hashtable tags = Utils.get_hash_or_create(newNode, "tags");

        // use 'tags_separated' instead of 'tags' if present
        Hashtable tags_separated = (Hashtable)newNode.get("tags_separated");
        if( tags_separated != null )
            tags = tags_separated;


        if( sections.size() > 0 && sections.size()<custom_fields_total )
            out.println("<TR class='modHeader'><TD COLSPAN='2'>&nbsp;Remaining custom fields"+
                        "</TD></TR><TR class='interfaceHeader'>"+
                        "<TD>&nbsp;&nbsp;Label</TD><TD>Value</TD></TR>");

        list = Arrays.asList(tags.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();

        String[] parameters = new String[custom_fields_total];
        String[] values     = new String[custom_fields_total];
        index = 0;

        while( it.hasNext() ) {
            String tag = (String)it.next();
            if( tags_separated != null ) {
                Hashtable tagH = (Hashtable)tags.get(tag);
                parameters[index] = (String)tagH.get("tag");
                values[index++] = (String)tagH.get("value");
            } else {
                parameters[index] = tag;
                values[index++] = (String)tags.get(tag);
            }
            if( index == custom_fields_total ) break;
        }

        int matched_sections = 0;
        for( int i = 0; i < parameters.length; i++ )
            if( sections.get(Utils.n2s(parameters[i])) != null ) matched_sections++;

        int skipped_sections = 0;
        for( int i = 0; i < parameters.length; i++ ) {
            if( parameters[i] == null ) parameters[i] = "";
            if( values[i] == null ) values[i] = "";

            if( ( parameters[i].trim().equals("")      &&
                  sections.size() > matched_sections   &&
                  skipped_sections < sections.size() ) ||
                CNVParser.foundSection(sections, parameters[i]) ) {
                skipped_sections++;
                continue;
            }

            out.println("<TR><TD>&nbsp;&nbsp;<INPUT ID='TextField_Tag_Name_"+i+"' TYPE='text' SIZE='18' NAME='nodeTag"+(i+1)+"' value =\""+
                        Utils.htmlize(parameters[i])+"\"></TD>"+
                        "<TD><INPUT ID='TextField_Tag_Value_"+i+"' TYPE='text' SIZE='64' name='nodeValue"+(i+1)+"' value=\""+
                        Utils.htmlize(values[i])+"\"></TD></TR>\n");
        }

        out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        Hashtable reserved_ips = nHash(store.get("reserved_ips"));
        Hashtable troubled_ips = nHash(store.get("troubled_ips"));

        if( find_by_key(nHash(newNode.get("types")), "extkey", "Template", "type") != null ) {
            out.println("<TR class='modHeader'><TD COLSPAN='2'>&nbsp;<B>"+
                        "Template Default Address Space</B></TD></TR>");
            //Hashtable user = (Hashtable)session.getAttribute("user");
            String nodeAddrSpace = (String)newNode.get("addr_space");
            //String default_addr_space = (String)user.get("default_addr_space");
            //if( nodeAddrSpace == null )
            //    nodeAddrSpace = default_addr_space==null?"":default_addr_space;

            Hashtable myAddrSpaces = (Hashtable)getData(session, "myAddrSpaces");
            out.println("<TR><TD></TD><TD>"+
                        "<SELECT ID='Selector_Address_Space' SIZE='1' NAME='addr_space'>");

            nodeAddrSpace = (String)Utils.n2s(nodeAddrSpace);

            out.println("<OPTION VALUE='"+
                        encode("---")+"'"+
                        ((nodeAddrSpace.trim().length()==0)?" SELECTED":"")+">---");

            list = Arrays.asList(myAddrSpaces.keySet().toArray());
            Collections.sort(list, ip_str_cmp);
            it = list.iterator();
            while( it.hasNext() ) {
                String prefix = (String)((Hashtable)myAddrSpaces.get(it.next())).get("prefix");
                out.println("<OPTION VALUE='"+
                            encode(prefix)+"'"+
                            (prefix.equals(nodeAddrSpace)?" SELECTED":"")+">"+Utils.htmlize(prefix, false));
            }
            out.println("</SELECT></TD></TR>\n"+
                        "<TR><TD COLSPAN='2'>&nbsp;Note: Nodes of type Template may "+
                        "not have interfaces</TD></TR>"+
                        "<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");
        } else {
            String raw_interfaces_count = ((String)Utils.n2s(newNode.get("raw_interfaces_count"))).trim();
            out.println("<TR class='modHeader'><TD><B>"+
                        "&nbsp;Interface&nbsp;information</B></TD>\n"+
                        "<TD ALIGN='RIGHT'>Count <INPUT ID='TextField_Interfaces_Count' "+
                        (newNode.get("interfaces_count_error")!=null?"CLASS='error_form_font' ":"")+"TYPE='TEXT' SIZE='4' NAME = 'interfaces_count' value ='"+
                        Utils.htmlize(raw_interfaces_count)+"'>&nbsp;&nbsp;&nbsp;&nbsp;<INPUT ID='Button_Add_Another_Interface' TYPE='SUBMIT' NAME='Action' "+
                        "VALUE='Add another interface'></TD></TR>");
            Hashtable interfaces = nHash(newNode.get("interfaces"));

            list = Arrays.asList(interfaces.entrySet().toArray());
            Collections.sort(list, intfc_cmp);
            it = list.iterator();
            index = 1;
            while( it.hasNext() ) {
                Map.Entry entry = (Map.Entry)it.next();
                String intfcID = (String)entry.getKey();
                Hashtable intfc = (Hashtable)entry.getValue();

                String hw = ((String)Utils.n2s(intfc.get("hw"))).trim();
                String hwStr = "";
                if( hw.length() > 0 ) {
                    String parsedHW = null;
                    try {
                        Hashtable parseRes = (new Parser()).parse(hw.toUpperCase());
                        parsedHW = (String)parseRes.get("hw");
                    } catch (Exception exx) { }
                    if( parsedHW == null )
                        hwStr = hw.trim();
                    else
                        hwStr = (hw.substring(0, 4)+"."+hw.substring(4, 8)+"."+hw.substring(8)).toLowerCase();
                }

                // pull interface name if any
                String hwName = "";
                String fqdn = "";
                Hashtable intfcNames = nHash(intfc.get("names"));

                for( Hashtable intfcName : (Collection<Hashtable>)intfcNames.values() ) {
                    hwName = ((String)intfcName.get("name")).trim();
                    String domain = ((String)intfcName.get("domain")).trim();
                    fqdn = hwName+"."+domain;
                    if( hwName.length() > 0 && !domain.equalsIgnoreCase(default_domain) )
                        hwName += "."+domain;
                }
                if( hwName == null )
                    hwName = "";

                Boolean use_dhcp = (Boolean)intfc.get("use-dhcp?");
                if( use_dhcp == null )
                    use_dhcp = new Boolean(false);

                Boolean roaming = (Boolean)intfc.get("roaming?");
                if( roaming == null )
                    roaming = new Boolean(false);

                out.println("<TR class=\"interfaceHeader\"><TD>&nbsp;&nbsp;&nbsp;Interface "+
                            index+"</TD><TD><INPUT ID='CheckBox_Interface_"+index+"' TYPE='checkbox' CHECKED NAME='intfc' value='"+
                            encode(intfcID)+
                            "'>Include this interface(<em>uncheck to remove</em>)</TD></TR>\n"+
                            "<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                            (( find_by_key(nHash(newNode.get("types")), "extkey", "Advanced", "type")!=null )?
                             ("<TR><TD>&nbsp;Interface Name</TD><TD><INPUT ID='TextField_HW_Name_"+index+"' "+
                              (nameIn(Constants.INTERFACE_NAME_TYPE, fqdn, troubled_names)?"CLASS='error_form_font' ":
                                 (nameIn(Constants.INTERFACE_NAME_TYPE, fqdn, reserved_names)?"CLASS='reserved_form_font' ":""))+
                              "TYPE='text' SIZE='15' name='"+
                              encode("hw_name "+intfcID)+
                              "'value="+Utils.htmlize(hwName)+"> "+
                              "</TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"):"")+
                            "<TR><TD>&nbsp;Hardware Address</TD><TD><INPUT ID='TextField_HW_Address_"+index+"' "+
                            (troubled_hws.get(hwStr.trim().toLowerCase())!=null?"CLASS='error_form_font' ":"")+
                            "TYPE='text' SIZE='15' name='"+
                            encode("hw_addr "+intfcID)+
                            "'value="+Utils.htmlize(hwStr, false)+">"+
                            "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<INPUT ID='CheckBox_DHCP_"+index+
                            "' onChange=\"ensure_proper_roaming('"+index+"')\" TYPE='checkbox' name='"+
                            encode("dhcp_flag "+intfcID)+
                            "' VALUE='YES'"+(use_dhcp.booleanValue()?" CHECKED":"")+
                            "> DHCP/BootP&nbsp;&nbsp;"+
                            "<INPUT ID='CheckBox_Roaming_"+index+"' onChange=\"ensure_proper_dhcp('"+index+
                            "')\" TYPE='checkbox' name='"+
                            encode("roam_flag "+intfcID)+
                            "' VALUE='YES'"+(roaming.booleanValue()?" CHECKED":"")+
                            "> Roaming"+
                            "</TD></TR>");

                // ip addresses
                out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                            "<TR><TD>&nbsp;IP Addresses</TD>");
                Hashtable ips = nHash(intfc.get("ips"));
                java.util.List ilist = Arrays.asList(ips.keySet().toArray());
                Collections.sort(ilist, ip_cmp);
                Object[] ip = ilist.toArray();
                for (int j = 0; j < ip.length; j++) {
                    Hashtable ipH = (Hashtable)ips.get(ip[j]);
                    Boolean active = (Boolean)ipH.get("active?");
                    if( active == null )
                        active = new Boolean(false);

                    String ipNameStr = null;
                    boolean dup_name = false;
                    boolean reserved_name = false;

                    for( Hashtable ipName : (Collection<Hashtable>)nHash(ipH.get("names")).values() ) {
                        ipNameStr = ((String)ipName.get("name")).trim();
                        String domain = ((String)ipName.get("domain")).trim();
                        fqdn = ipNameStr+"."+domain;
                        if( ipNameStr.length() > 0 && !domain.equalsIgnoreCase(default_domain) )
                            ipNameStr += "."+domain;
                        if( ipName.get("error") != null )
                            dup_name = true;
                        if( ipName.get("reserved") != null )
                            reserved_name = true;
                    }
                    if( ipNameStr == null )
                        ipNameStr = "";

                    out.println((j>0?"<TR><TD></TD>":"")+
                                "<TD><INPUT ID='CheckBox_IP_"+index+"_"+j+"' TYPE='checkbox' NAME='"+
                                encode("ip addr cb ["+intfcID+"] ["+j+"]")+
                                "' VALUE='"+j+"' CHECKED> <INPUT ID='TextField_IP_"+index+"_"+j+"' "+
                                (troubled_ips.get(((String)ip[j]).trim().toLowerCase())!=null?"CLASS='error_form_font' ":
                                 (reserved_ips.get((String)ip[j])!=null?"CLASS='reserved_form_font' ":""))+
                                "TYPE='text' SIZE='15' NAME='"+
                                encode("ip addr txt ["+intfcID+"] ["+j+"]")+
                                "' value='"+
                                Utils.htmlize((String)ip[j], false)+"'>"+
                                "<INPUT ID='CheckBox_IP_Active_"+index+"_"+j+"' TYPE='checkbox' NAME='"+
                                encode("ip addr status ["+intfcID+"] ["+j+"]")+
                                "'"+(active.booleanValue()?" CHECKED":"")+"> Active");

                    out.println((( find_by_key(nHash(newNode.get("types")), "extkey", "Advanced", "type")!=null )?
                                 ("&nbsp;&nbsp;&nbsp;Name <INPUT ID='TextField_IP_Name_"+index+"_"+j+"' "+
                                  (nameIn(Constants.ADDRESS_NAME_TYPE, fqdn, troubled_names)?"CLASS='error_form_font' ":
                                   (nameIn(Constants.ADDRESS_NAME_TYPE, fqdn, reserved_names)?"CLASS='reserved_form_font' ":""))+
                                  "TYPE='text' SIZE='15' name='"+
                                  encode("ip addr name ["+intfcID+"] ["+j+"]")+
                                  "'value="+Utils.htmlize(ipNameStr.trim(), false)+">"):""));
                    // PTRpref display
                    if( find_by_key(nHash(newNode.get("types")), "extkey", "Advanced", "type") != null ) {
                        String ip_ptr_pref = Utils.nStr(ipH.get("PTRpref"));

                        out.println("&nbsp;&nbsp;PTR Pref <SELECT ID='Selector_PTRpref_"+index+"_"+j+"' SIZE='1' NAME='"+
                                    encode("ip ptr pref ["+intfcID+"] ["+j+"]")+"'>");
                        Iterator it_ptr = Utils.sortedKeys(CommonPage.ptr_pref, num_cmp);
                        while( it_ptr.hasNext() ) {
                            Hashtable ptr_pref = (Hashtable)CommonPage.ptr_pref.get(it_ptr.next());
                            String value = Utils.nStr(ptr_pref.get("value"));
                            out.println("<OPTION VALUE='"+value+"'"+
                                        (value.equals(ip_ptr_pref)||(ip_ptr_pref.equals("")&&ptr_pref.get("default")!=null)?" SELECTED":"")+
                                        ">"+Utils.htmlize(Utils.nStr(ptr_pref.get("short name")), false));
                        }
                        out.println("</SELECT>");
                    }


                    out.println("</TD><TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");
                }

                if( ips.size() == 0 )
                    intfc.put("show select", "");

                if( intfc.get("show select") == null )
                    out.println("<TR><TD></TD><TD><INPUT TYPE='HIDDEN' VALUE='Yes' NAME='"+
                                encode("new ip active "+intfcID)+"'><INPUT "+
                                "ID='Button_Add_Another_IP_"+index+"' TYPE='SUBMIT' NAME='"+
                                encode("ip add "+intfcID)+
                                "' VALUE='Add another'> "+

                                //"<INPUT TYPE='SUBMIT' NAME='"+
                                //encode("ip verify "+intfcID)+
                                //"' VALUE='Verify Address & Name'>"+

                                "</TD></TR>");
                else {
                    out.println("<TR><TD>"+
                                "<INPUT TYPE='Hidden' NAME='shown_ip_select' VALUE='"+
                                encode(intfcID)+"'>"+
                                "</TD><TD><SELECT ID='Selector_IP_"+index+"' SIZE='6' NAME='"+
                                encode("addr space "+intfcID)+"'>");

                    Hashtable user = (Hashtable)session.getAttribute("user");
                    String default_addr_space = (String)Utils.n2s(user.get("default_addr_space"));
                    Hashtable myAddrSpaces = (Hashtable)getData(session, "myAddrSpaces");
                    out.println("<OPTION VALUE='"+
                                encode("select an address space")+"'>select an address space");
                    String selected_addr_space = ""; // default_addr_space;
                    if( intfc.get("saved addr space") != null )
                        selected_addr_space = (String)intfc.get("saved addr space");

                    list = Arrays.asList(myAddrSpaces.keySet().toArray());
                    Collections.sort(list, ip_str_cmp);
                    Iterator it2 = list.iterator();
                    while( it2.hasNext() ) {
                        String prefix = (String)((Hashtable)myAddrSpaces.get(it2.next())).get("prefix");
                        out.println("<OPTION VALUE='"+
                                    encode(prefix)+"'"+
                                    (prefix.equals(selected_addr_space)?" SELECTED":"")+
                                    ">"+prefix);
                    }
                    String saved_ip_name = ((String)Utils.n2s(intfc.get("saved ip name"))).trim();
                    String saved_ip_name_fqdn = Utils.name(saved_ip_name)+"."+Utils.domain(saved_ip_name, default_domain);
                    out.println("</SELECT> or <INPUT ID='TextField_New_IP_"+index+"' TYPE='text' SIZE='15' NAME='"+
                                encode("new ip txt "+intfcID)+
                                "' VALUE='"+Utils.htmlize((String)Utils.n2s(intfc.get("saved ip addr")), false)+
                                "'> <INPUT ID='CheckBox_New_IP_Active_"+index+"' TYPE='checkbox' VALUE='Yes' NAME='"+
                                encode("new ip active "+intfcID)+
                                "'"+(intfc.get("saved ip status inactive")!=null?"":" CHECKED")+
                                "> Active"+
                                (( find_by_key(nHash(newNode.get("types")), "extkey", "Advanced", "type")!=null )?
                                 ("&nbsp;&nbsp;&nbsp;&nbsp;Name <INPUT ID='TextField_New_IP_Name_"+index+"' "+
                                  (nameIn(Constants.ADDRESS_NAME_TYPE, saved_ip_name_fqdn, troubled_names)?"CLASS='error_form_font' ":"")+
                                  "TYPE='text' SIZE='15' NAME='"+
                                  encode("new ip name "+intfcID)+
                                  "' VALUE='"+Utils.htmlize(saved_ip_name, false)+"'>"):"")+
                                "&nbsp;&nbsp;&nbsp;&nbsp;Count <INPUT ID='TextField_New_IP_Count_"+index+"' "+
                                (intfc.get("ip_count_error")!=null?"CLASS='error_form_font' ":"")+"TYPE='text' SIZE='5' NAME='"+
                                encode("new ip count "+intfcID)+
                                "' VALUE='"+Utils.htmlize((String)Utils.n2s(intfc.get("raw ip count")), false)+"'>"+
                                "</TD></TR>\n"+
                                "<TR><TD></TD><TD><INPUT ID='Button_Add_Another_IP_"+index+"' TYPE='SUBMIT' NAME='"+
                                encode("ip reserve&add "+intfcID)+
                                "' VALUE='Add another'> "+
                                "<INPUT ID='Button_Reserve_Address_And_Name_"+index+"' TYPE='SUBMIT' NAME='"+
                                encode("ip reserve "+intfcID)+
                                "' VALUE='Reserve Address & Name'>"+
                                "</TD></TR>");
                    //out.println("</TD></TR>");
                }

                if( ip.length == 0 )
                    out.println("<TD></TD></TR>");


                // dhcp options
                boolean show_dhcp_options = false;
                String dhcpTxt  = (String)intfc.get("dhcp-options-raw");
                if( dhcpTxt == null ) {
                    dhcpTxt = "";
                    Hashtable dhcp_options = nHash(intfc.get("dhcp-options"));
                    java.util.List alist2 = Arrays.asList(dhcp_options.keySet().toArray());
                    Collections.sort(alist2, name_cmp);
                    Object[] oa2 = alist2.toArray();
                    for (int j = 0; j < oa2.length; j++) {
                        Hashtable dhcp_option_H = (Hashtable)dhcp_options.get(oa2[j]);

                        String opt = (String)dhcp_option_H.get("option");
                        String val = (String)dhcp_option_H.get("value");
                        if (dhcpTxt.length() > 0)
                            dhcpTxt += "\n";
                        if( (opt.trim().length()+val.trim().length()) > 0 )
                            dhcpTxt += opt.trim() + "=" + val.trim();
                    }
                    if( oa2.length > 0 )
                        show_dhcp_options = true;

                } else
                    show_dhcp_options = true;

                out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                            "<TR><TD VALIGN='TOP'>&nbsp;DHCP Options</TD><TD>"+
                            (show_dhcp_options?
                             "<TEXTAREA ID='TextArea_DHCP_Options_"+index+"' "+
                             (intfc.get("bad dhcp options")!=null?"CLASS='error_form_font' ":"")+"NAME='"+
                             encode("dhcp_options "+intfcID)+
                             "' rows=2 cols=30>"+
                             Utils.htmlize(dhcpTxt.trim(), false)+"</TEXTAREA>":
                             "<INPUT ID='Button_Display_DHCP_Options_"+index+"' TYPE='SUBMIT' NAME='"+
                              encode("display_dhcp_options "+intfcID)+
                             "' VALUE='Display'></TD></TR>"));
                index++;
                out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");
            } // end of 'while( it.hasNext() )'

            if( interfaces.size() == 0 )
                out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");
        }

        if( find_by_key(nHash(newNode.get("types")), "extkey", "IPCprovider", "type") != null ) {
            out.println("<TR class='modHeader'><TD COLSPAN='2'><B>&nbsp;"+
                        "IP Addresses (IPC provider).</B><em>Uncheck to remove</em></TD></TR>\n");

            // ipc addresses
            out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                            "<TR><TD></TD>");
            Hashtable ips = nHash(newNode.get("ipc_addresses"));
            java.util.List ilist = Arrays.asList(ips.keySet().toArray());
            Collections.sort(ilist, ip_cmp);
            Object[] ip = ilist.toArray();
            for (int j = 0; j < ip.length; j++) {
                Hashtable ipH = (Hashtable)ips.get(ip[j]);
                Boolean active = (Boolean)ipH.get("active?");
                if( active == null )
                    active = new Boolean(false);

                String ipNameStr = null;
                String fqdn = "";
                Hashtable ipNames = nHash(ipH.get("names"));
                Hashtable ipName = null;
                for( Object key : ipNames.keySet() ) {
                    ipName = (Hashtable)ipNames.get(key);
                    ipNameStr = ((String)ipName.get("name")).trim();
                    String domain = ((String)ipName.get("domain")).trim();
                    fqdn = ipNameStr+"."+domain;
                    if( ipNameStr.length() > 0 && !domain.equalsIgnoreCase(default_domain) )
                        ipNameStr += "."+domain;
                }

                if( ipNameStr == null ) ipNameStr = "";

                if( ipName == null ) ipName = hash();

                out.println("<TR><TD></TD>"+
                            "<TD><INPUT ID='CheckBox_IPC_IP_"+j+"' TYPE='checkbox' NAME='"+
                            encode("ipc addr cb")+
                            "' VALUE='"+j+"'CHECKED> <INPUT ID='TextField_IPC_IP_"+j+"' "+
                            (troubled_ips.get(((String)ip[j]).trim().toLowerCase())!=null?"CLASS='error_form_font' ":
                             (reserved_ips.get((String)ip[j])!=null?"CLASS='reserved_form_font' ":""))+
                            "TYPE='text' SIZE='15' NAME='"+
                            encode("ipc addr txt "+j)+
                            "' value='"+Utils.htmlize((String)ip[j], false)+"'>"+
                            "<INPUT ID='CheckBox_IP_IP_Active_"+j+"' TYPE='checkbox' NAME='"+
                            encode("ipc addr status "+j)+
                            "'"+(active.booleanValue()?" CHECKED":"")+"> Active"+
                            "&nbsp;&nbsp;&nbsp;Name <INPUT ID='TextField_IPC_IP_Name_"+j+"' "+
                            (nameIn(Constants.ADDRESS_NAME_TYPE, fqdn, troubled_names)?"CLASS='error_form_font' ":
                             (nameIn(Constants.ADDRESS_NAME_TYPE, fqdn, reserved_names)?"CLASS='reserved_form_font' ":""))+
                            "TYPE='text' SIZE='15' name='"+
                            encode("ipc addr name "+j)+
                            "' value='"+Utils.htmlize(ipNameStr.trim(), false)+"'>"+
                            "</TD><TR>");

                out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");
            }

            if( ips.size() == 0 )
                newNode.put("show ipc select", "");

            if( newNode.get("show ipc select") == null )
                out.println("<TR><TD>"+
                            "</TD><TD><INPUT ID='CheckBox_Add_Another_IPC_IP' TYPE='SUBMIT' NAME='ipc_add'"+
                            " VALUE='Add another'> "+
                            //"<INPUT TYPE='SUBMIT' NAME='ipc_verify'"+
                            //" VALUE='Verify Address & Name'>"+
                            "</TD></TR>"+
                            "<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");
            else {
                out.println("<TR><TD>"+
                            "<INPUT TYPE='Hidden' NAME='shown_ipc_select' VALUE='yes'>"+
                            "</TD><TD><SELECT ID='Selector_IPC' SIZE='6' NAME='"+
                            encode("ipc addr space")+"'>");

                String savedIPCAddrSpace = (String)Utils.n2s(newNode.get("saved ipc addr space"));

                Hashtable myAddrSpaces = (Hashtable)getData(session, "myAddrSpaces");
                out.println("<OPTION VALUE='"+
                            encode("select an address space")+"'"+
                            ">select an address space");

                list = Arrays.asList(myAddrSpaces.keySet().toArray());
                Collections.sort(list, ip_str_cmp);
                it = list.iterator();
                while( it.hasNext() ) {
                    String prefix = (String)((Hashtable)myAddrSpaces.get(it.next())).get("prefix");
                    out.println("<OPTION VALUE='"+
                                encode(prefix)+"'"+
                                (prefix.equals(savedIPCAddrSpace)?" SELECTED":"")+
                                ">"+prefix);
                }
                out.println("</SELECT> or <INPUT ID='TextField_New_IPC_IP' TYPE='text' SIZE='15' NAME='"+
                            encode("new ipc txt")+
                            "' VALUE='"+Utils.htmlize((String)Utils.n2s(newNode.get("saved ipc addr")), false)+
                            "'> <INPUT ID='CheckBox_New_IPC_IP_Active' TYPE='checkbox' VALUE='Yes' NAME='"+
                            encode("new ipc active")+
                            "'"+(newNode.get("saved ipc status inactive")!=null?"":" CHECKED")+
                            "> Active&nbsp;&nbsp;&nbsp;&nbsp;Count <INPUT ID='TextField_New_IPC_IP_Name' TYPE='text' SIZE='5' NAME='"+
                            encode("new ipc count")+
                            "' VALUE='"+Utils.htmlize((String)Utils.n2s(newNode.get("saved ipc count")), false)+"'>"+
                            "</TD></TR>\n"+
                            "<TR><TD></TD><TD><INPUT ID='Button_Add_Another_IPC_IP' TYPE='SUBMIT' "+
                            "NAME='ipc_reserve_and_add' VALUE='Add another'> "+
                            "<INPUT ID='Button_Reserve_IPC_Address_And_Name' TYPE='SUBMIT' "+
                            "NAME='ipc_reserve' VALUE='Reserve Address & Name'>"+
                            "</TD></TR>");
                //out.println("</TD></TR>");
                out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");
            }
        }
        out.println("<TR class='modHeader'><TD COLSPAN='2'><B>&nbsp;Record Information</B></TD></TR>\n");

        // Expiration Date
        Timestamp exp_date = (Timestamp)newNode.get("exp.date");
        String exp_date_string = (String)newNode.get("exp_date_string");

        out.println("<TR><TD COLSPAN='2'></TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Expiration Date</TD>\n"+
                    "<TD VALIGN='TOP'>"+
                    "<INPUT ID='TextField_Expiration_Date' "+(newNode.get("bad_exp_date_format")!=null?"CLASS='error_form_font' ":"")+
                    "TYPE='text' SIZE='10' name='expirationDate' VALUE='"+
                    Utils.htmlize(exp_date_string!=null?exp_date_string.trim():
                                  (exp_date!=null?(new SimpleDateFormat("MM/dd/yyyy")).format(exp_date):""), false)+
                    "'> (mm/dd/yyyy) Informational only. No action taken on this date.</TD></TR>\n"+
                    "<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // NetDB Groups
        Hashtable groups = (Hashtable)newNode.get("groups");
        if( groups == null ) { // I know that node always have groups, but just in case
            groups = hash();
            newNode.put("groups", groups);
        }
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font>&nbsp;"+
                    "Groups<br>&nbsp;(1 or more)</TD>\n"+
                    "<TD VALIGN='TOP'>Selected (<em>uncheck to remove</em>):<BR>");
        list = Arrays.asList(groups.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        index = 0;
        while( it.hasNext() ) {
            String group = (String)it.next();
            out.println("<INPUT ID='CheckBox_Group_"+index+"' TYPE='checkbox' CHECKED NAME='groups' value='"+
                        encode(group)+"'>"+Utils.htmlize(group.trim(), false)+"<BR>");
        }
        out.println("<SELECT ID='Selector_Groups' NAME='groups' multiple size='6'>");
        Hashtable myGroups = (Hashtable)getData(session, "myGroups");
        list = Arrays.asList(myGroups.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        while( it.hasNext() ) {
            String group = (String)it.next();
            out.println("<OPTION VALUE='"+encode(group)+"'>"+Utils.htmlize(group.trim(), false)+"</OPTION>");
        }
        out.println("</SELECT></TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // Comment
        String comment = (String)Utils.n2s((String)newNode.get("comment"));

        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Comment</TD>\n"+
                    "<TD VALIGN='TOP'>"+
                    "<TEXTAREA ID='TextArea_Comment' name='comment' rows='3' cols='80'>"+
                    Utils.htmlize(comment.trim(), false)+"</textarea></TD></TR>\n"+
                    "<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // created by
        Hashtable person = (Hashtable)newNode.get("created.by");
        if( person != null ) {
            String dirID = (String)person.get("DirID");
            boolean active = activep(person);

            String byStr = (active?"<A HREF=\""+
                            System.getProperty("stanfordwho.url")+dirID.trim()+"\">":"");
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += (active?"</A>":"")+" ("+newNode.get("created").toString().substring(0, 16)+")";
            out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Created By</TD>\n"+
                        "<TD VALIGN='TOP'>"+
                        byStr+"</TD></TR>");

            // modified by
            person = (Hashtable)newNode.get("mod.by");
            dirID = (String)person.get("DirID");
            active = activep(person);

            byStr = (active?"<A HREF=\""+
                     System.getProperty("stanfordwho.url")+dirID.trim()+"\">":"");
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += (active?"</A>":"")+" ("+newNode.get("modified").toString().substring(0, 16)+")";
            out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Last Modified By</TD>\n"+
                        "<TD VALIGN='TOP'>"+
                        byStr+"</TD></TR>");
        }

        out.println("</TABLE>\n"+
                    "<HR NOSHADE SIZE=2>\n"+
                    "<P CLASS=\"form_buttons\">\n"+
                    "<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Save\">\n"+
                    "<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Verify\">\n"+
                    "<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Reset\">\n"+
                    "<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Cancel\">\n"+
                    "</P>\n"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"ready\" VALUE=\"yes\">"+
                    "</FORM>\n<BR>\n"+
                    getCommonBottomLabel()+
                    "</DIV>\n"+
                    "</BODY>\n"+
                    "</HTML>\n");
    } // end of printOutModifyHTML(...)

    public void newOSHTML(HttpSession session, PrintWriter out, Hashtable store, String rootURL) throws Exception {
        String sessionKey = (String)store.get("sessionKey");

        out.println(commonHtmlBegin("Create OS")+
                    commonHeader(session, "/Dispatch?Action=redirect&to=/home&pre=unlock&object=Node&sessionKey="+
                                 encode(sessionKey),
                                 SHOW_HOME_LINK, NO_PROFILE_LINK, NO_LOGOUT_LINK,
                                 "", "Create OS", "hlpnode.html")+
                    msgPanels(session)+
                    "<DIV id=\"infoPage\">\n"+
                    "<FORM METHOD=\"POST\" ACTION='"+rootURL+"'>"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"sessionKey\" VALUE=\""+encode(sessionKey)+"\">"+
                    "<DIV class='modHeader'>\n"+

                    "<TABLE WIDTH='100%'>\n"+
                    "<TR><TD NOWRAP>Create new OS.</TD></TR>\n"+
                    "</TABLE></DIV>"+
                    "<TABLE WIDTH='100%'>");

        Hashtable newOS = (Hashtable)store.get("newOS");
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font>&nbsp;OS</TD>"+
                    "<TD><INPUT TYPE='TEXT' SIZE='21' NAME='osName' value='"+
                    Utils.htmlize((String)Utils.n2s(newOS.get("name")), false)+
                    "'></TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Comment</TD><TD>"+
                    "<TEXTAREA NAME='osComment' ROWS='2' COLS='40'>"+
                    Utils.htmlize((String)Utils.n2s(newOS.get("comment")), false)+
                    "</TEXTAREA></TD></TR>"+
                    "<TR COLSPAN='2'><TD><INPUT TYPE='SUBMIT' NAME='Action' VALUE='Create OS'> "+
                    "<INPUT TYPE='SUBMIT' NAME='Action' VALUE='Cancel'></TD></TR>"+
                    "</TABLE>\n"+
                    "<HR NOSHADE SIZE=2>\n"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"ready\" VALUE=\"yes\">"+
                    "</FORM>\n<BR>\n"+
                    getCommonBottomLabel()+
                    "</DIV>\n"+
                    "</BODY>\n"+
                    "</HTML>\n");
    }

    public void newModelHTML(HttpSession session, PrintWriter out, Hashtable store, String rootURL) throws Exception {
        String sessionKey = (String)store.get("sessionKey");

        out.println(commonHtmlBegin("Create Model")+
                    commonHeader(session, "/Dispatch?Action=redirect&to=/home&pre=unlock&object=Node&sessionKey="+
                                 encode(sessionKey),
                                 SHOW_HOME_LINK, NO_PROFILE_LINK, NO_LOGOUT_LINK,
                                 "", "Create Model", "hlpnode.html")+
                    msgPanels(session)+
                    "<DIV id=\"infoPage\">\n"+
                    "<FORM METHOD=\"POST\" ACTION='"+rootURL+"'>"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"sessionKey\" VALUE=\""+encode(sessionKey)+"\">"+
                    "<DIV class='modHeader'>\n"+

                    "<TABLE WIDTH='100%'>\n"+
                    "<TR><TD NOWRAP>Create new Model.</TD></TR>\n"+
                    "</TABLE></DIV>"+
                    "<TABLE WIDTH='100%'>");

        Hashtable model_types = nHash(getData(session, "model_types"));
        Hashtable newModel = (Hashtable)store.get("newModel");
        String model_type = ((String)Utils.n2s(newModel.get("type"))).trim();
        out.println("<TR><TD VALIGN='TOP' WIDTH='2%'>&nbsp;<font class='star'>*</font>&nbsp;Model</TD>"+
                    "<TD><INPUT TYPE='TEXT' SIZE='21' NAME='model' value='"+
                    Utils.htmlize((String)Utils.n2s(newModel.get("model")), false)+
                    "'></TD></TR>\n");
        /*
        out.println("<TR><TD VALIGN='TOP' WIDTH='2%'>&nbsp;Type</TD>\n<TD>"+
                    "<SELECT SIZE=1 NAME='type'>\n"+
                    "<OPTION VALUE='"+encode("--None--")+"'>--None--");

        java.util.List list = Arrays.asList(model_types.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();
        while( it.hasNext() ) {
            String type = (String)it.next();
            out.println("<OPTION VALUE='"+encode(type)+"'"+(model_type.equals(type)?" SELECTED":"")+">"+type);
        }

        out.println("</SELECT></TD></TR>\n");
        */
        out.println("<TR><TD VALIGN='TOP' WIDTH='2%'>&nbsp;<font class='star'>*</font>&nbsp;Make</TD><TD>"+
                    "<SELECT SIZE=16 NAME='make'>");

        String selectedMake = (String)Utils.n2s(newModel.get("make"));
        Hashtable makes = nHash(store.get("makes"));
        java.util.List list = Arrays.asList(makes.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();
        while( it.hasNext() ) {
            String make = ((String)it.next()).trim();
            out.println("<OPTION VALUE='"+encode(make)+"'"+(selectedMake.equals(make)?" SELECTED":"")+">"+make);
        }

        out.println("</SELECT>\n"+
                    "</TD></TR>\n"+
                    "<TR COLSPAN='2'><TD NOWRAP><INPUT TYPE='SUBMIT' NAME='Action' VALUE='Create Model'> "+
                    "<INPUT TYPE='SUBMIT' NAME='Action' VALUE='Cancel'></TD></TR>"+
                    "</TABLE>\n"+
                    "<HR NOSHADE SIZE=2>\n"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"ready\" VALUE=\"yes\">"+
                    "</FORM>\n<BR>\n"+
                    getCommonBottomLabel()+
                    "</DIV>\n"+
                    "</BODY>\n"+
                    "</HTML>\n");
    }

    private void collectNamesAndIPs(Hashtable nodeHash, Hashtable allNames, Hashtable allIPs, Hashtable parents)
        throws Exception
    {
        // first node names
        for( Hashtable nameH : (Collection<Hashtable>)Utils.get_hash_or_create(nodeHash, "names").values() ) {
            String namePart = ((String)nameH.get("name")).trim();
            if( namePart.length() == 0 )
                continue;
            String fqdnName = namePart+"."+((String)nameH.get("domain")).trim();
            String fqdnLower = fqdnName.toLowerCase();
            allNames.put("NODE NAME "+fqdnLower, nameH);

            for( Hashtable aliasH : (Collection<Hashtable>)nHash(nameH.get("aliases")).values() ) {
                namePart = ((String)aliasH.get("name")).trim();
                if( namePart.length() == 0 )
                    continue;
                fqdnName = namePart+"."+((String)aliasH.get("domain")).trim();
                fqdnLower = fqdnName.toLowerCase();
                allNames.put("NODE NAME ALIAS " + fqdnLower, aliasH);
            }

            for( Hashtable mxH : (Collection<Hashtable>)nHash(nameH.get("mxs")).values() ) {
                namePart = ((String)mxH.get("name")).trim();
                if( namePart.length() == 0 )
                    continue;
                fqdnName = namePart+"."+((String)mxH.get("domain")).trim();
                fqdnLower = fqdnName.toLowerCase();
                allNames.put("NODE NAME MX " + fqdnLower, mxH);
            }
        }

        // second ipc names and ipc addresses
        Hashtable ipc_addresses = nHash(nodeHash.get("ipc_addresses"));
        for( String ipc_addressStr : (Set<String>)ipc_addresses.keySet() ) {
            Hashtable ipc_address = (Hashtable)ipc_addresses.get(ipc_addressStr);
            // these names do not have aliases and/or mxes

            String parent_key = getNextKey();
            parents.put(parent_key, ipc_addresses);
            ipc_address.put("parent", parent_key);
            String ipStr = ipc_addressStr.trim();
            allIPs.put(ipStr, ipc_address);

            for( Hashtable ipcName : (Collection<Hashtable>)nHash(ipc_address.get("names")).values() ) {
                String namePart = ((String)ipcName.get("name")).trim();
                if( namePart.length() == 0 )
                    continue;
                String fqdnName = namePart+"."+((String)ipcName.get("domain")).trim();
                String fqdnLower = fqdnName.toLowerCase();
                allNames.put("IP NAME " + fqdnLower, ipcName);
            }
        }

        // third ips and thier names in interfaces
        for( Hashtable intfc : (Collection<Hashtable>)nHash(nodeHash.get("interfaces")).values() ) {
            for( Hashtable nameH : (Collection<Hashtable>)nHash(intfc.get("names")).values() ) {
                String namePart = ((String)nameH.get("name")).trim();
                if( namePart.length() == 0 )
                    continue;
                String fqdnName = namePart+"."+((String)nameH.get("domain")).trim();
                String fqdnLower = fqdnName.toLowerCase();
                allNames.put("INTFC NAME " + fqdnLower, nameH);
            }

            Hashtable ips = nHash(intfc.get("ips"));
            for( String ipAddress : (Set<String>)ips.keySet() ) {
                String ipStr = ipAddress.trim(); // trim it just in case
                Hashtable ipH = (Hashtable)ips.get(ipAddress);

                String parent_key = getNextKey();
                parents.put(parent_key, ips);
                ipH.put("parent", parent_key);
                allIPs.put(ipStr, ipH);

                for( Hashtable nameH : (Collection<Hashtable>)nHash(ipH.get("names")).values() ) {
                    String namePart = ((String)nameH.get("name")).trim();
                    if( namePart.length() == 0 )
                        continue;
                    String fqdnName = namePart+"."+((String)nameH.get("domain")).trim();
                    String fqdnLower = fqdnName.toLowerCase();
                    allNames.put("IP NAME " + fqdnLower, nameH);
                }
            }
        }
    } // end of collectNamesAndIPs(...)

    private void registerNames(Hashtable registrar, Hashtable node) {
        for( Hashtable name : (Collection<Hashtable>)nHash(node.get("names")).values() )
            registerNameRecursive(registrar, name);

        for( Hashtable intfc : (Collection<Hashtable>)nHash(node.get("interfaces")).values() ) {
            for( Hashtable name : (Collection<Hashtable>)nHash(intfc.get("names")).values() )
                registerNameRecursive(registrar, name);

            for( Hashtable ip : (Collection<Hashtable>)nHash(intfc.get("ips")).values() )
                for( Hashtable name : (Collection<Hashtable>)nHash(ip.get("names")).values() )
                    registerNameRecursive(registrar, name);
        } // end of loop over interfaces

        for( Hashtable ip : (Collection<Hashtable>)nHash(node.get("ipc_addresses")).values() )
            for( Hashtable name : (Collection<Hashtable>)nHash(ip.get("names")).values() )
                registerNameRecursive(registrar, name);
    } // end of registerNames(...)

    private void registerNameRecursive(Hashtable registrar, Hashtable name) {
        registerName(registrar, name.get("name")+"."+name.get("domain"), "name");
        for( Hashtable alias : (Collection<Hashtable>)nHash(name.get("aliases")).values() )
            registerName(registrar, alias.get("name")+"."+alias.get("domain"), "alias");

        for( Hashtable mx : (Collection<Hashtable>)nHash(name.get("mxs")).values() )
            registerName(registrar, mx.get("name")+"."+mx.get("domain"), "mx");
    } // end of registerNameRecursive(...)

    private void registerName(Hashtable registrar, String fqdn, String type) {
        fqdn = fqdn.trim().toLowerCase();
        Hashtable name = Utils.get_hash_or_create(registrar, fqdn);
        Hashtable types = Utils.get_hash_or_create(name, "types");
        types.put(type, "");
    } // end of registerName(...)

    // if throws exception it means that this reserve will fail
    // if returns true then we should reserve this name, if false then we don't need to reserve this name
    // as it already reserved
    private boolean checkNameReserve(Hashtable registrar, String fqdn, String type) throws Exception {
        Hashtable name = (Hashtable)registrar.get(fqdn.trim().toLowerCase());
        if( name == null )
            return true;

        Hashtable types = (Hashtable)name.get("types");
        if( types.get(type) != null )
            return false;

        // order of all ifs bellow is important!!!
        if( type.equals("name") ) {
            if( types.get("alias") != null )
                return false;
            if( types.get("mx") != null )
                return true;
        }

        if( type.equals("alias") ) {
            if( types.get("mx") != null )
                throw new Exception("Name '"+fqdn.trim()+"' of type mx already exists and cannot be reserved as an alias.");
            if( types.get("name") != null )
                return false;
        }

        if( type.equals("mx") ) {
            if( types.get("alias") != null )
                throw new Exception("Name '"+fqdn.trim()+"' of type alias already exists and cannot be reserved as an mx.");
            if( types.get("name") != null )
                return true;
        }

        return true; // it will never get to this point but compiler does not know about it.
    } // end of checkNameReserve(...)

    public void purgeEmptyInterfaces(Hashtable node) {
        Hashtable interfaces = nHash(node.get("interfaces"));
        Vector keys_to_delete = new Vector();
        for( Object key : interfaces.keySet() ) {
            Hashtable intfc = (Hashtable)interfaces.get(key);

            String hw = (String)intfc.get("hw");
            if( hw!=null && hw.trim().length()>0 )
                continue;

            Hashtable ips = (Hashtable)intfc.get("ips");
            if( ips != null && ips.size() > 0 )
                continue;

            Hashtable dhcp_options = (Hashtable)intfc.get("dhcp-options");
            if( dhcp_options != null && dhcp_options.size() > 0 )
                continue;

            String dhcp_options_raw = (String)intfc.get("dhcp-options-raw");
            if( dhcp_options_raw != null && dhcp_options_raw.trim().length() > 0 )
                continue;

            keys_to_delete.addElement(key);
        }

        // now remove offending interfaces
        for( int i = 0; i < keys_to_delete.size(); i++ )
            interfaces.remove(keys_to_delete.elementAt(i));
    } // end of purgeEmptyInterfaces(...)


    // picks up tags_separated from POST data
    public void pick_up_tags_separated(Hashtable node, HttpServletRequest request, HttpSession session)
        throws Exception
    {
        int custom_fields_total = ((Integer)getData(session, "custom_fields_total")).intValue();

        Hashtable tags_separated = hash();
        node.put("tags_separated", tags_separated);
        Hashtable tags = hash();

        for( int i = 1; i <= custom_fields_total; i++ ) {
            String tag   = ((String)Utils.n2s(request.getParameter("nodeTag"+i))).trim();
            String value = ((String)Utils.n2s(request.getParameter("nodeValue"+i))).trim();
            if( tags.get(tag) != null )
                addSessionMsg(session, "Error.errorMsg", "Error: Duplicate custom field names.");
            else if( tag.length() > 0 )
                tags.put(tag, value);

            tags_separated.put("a"+i, hash("tag", tag, "value", value));
        }
    } // pick_up_tags_separated(...)

    // picks up tags keyed on tag names ( not tags_separated )
    public Hashtable pick_up_tags(Hashtable node, HttpServletRequest request, HttpSession session) throws Exception {
        int custom_fields_total = ((Integer)getData(session, "custom_fields_total")).intValue();
        Hashtable tags = Utils.get_hash_or_create(node, "tags");

        for( int i = 1; i <= custom_fields_total; i++ ) {
            String tag   = ((String)Utils.n2s(request.getParameter("nodeTag"+i))).trim();
            String value = ((String)Utils.n2s(request.getParameter("nodeValue"+i))).trim();
            if( tags.get(tag) != null )
                addSessionMsg(session, "Error.errorMsg", "Error: Duplicate custom field names.");
            else if( tag.length() > 0 )
                tags.put(tag, value);
        }

        return tags;
    } // pick_up_tags(...)

    public String find_by_key(Hashtable hash, String search_key, String search_value, String return_key) {
        for( Hashtable record : (Collection<Hashtable>)hash.values() ) {
            if( Utils.nStr(record.get(search_key)).equals(search_value) )
                return (String)record.get(return_key);
        }
        return null;
    }

} // end of class ModifyNode
