/************************************************************************
 *
 * ModifyPage.java -
 *
 * $Id: ModifyPage.java,v 1.7 2006/12/14 21:33:45 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2004, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;
import javax.servlet.*;
import javax.servlet.http.*;
import su.netdb.parser.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public abstract class ModifyPage extends CommonPage {

    protected static String nextKey             = "0";
    private static String dataLock              = "lock";

    public static Comparator name_cmp           = null;
    public static Comparator ip_cmp             = null;
    public static Comparator full_ip_cmp        = null;
    public static Comparator as_ip_cmp          = null;

    static {
        name_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    return ((String)o1).compareToIgnoreCase((String)o2);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        long_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    long v1 = 0;
                    boolean number1 = true;
                    try {
                        v1 = (new Long((String)o1)).longValue();
                    } catch (Exception ex) {
                        number1 = false;
                        try {
                            v1 = (new Long( ((String)o1).substring(((String)o1).indexOf("_")+2) )).longValue();
                        } catch(Exception exx) {
                            v1 = 0;
                        }
                    }

                    long v2 = 0;
                    boolean number2 = true;
                    try {
                        v2 = (new Long((String)o2)).longValue();
                    } catch (Exception ex) {
                        number2 = false;
                        try {
                            v2 = (new Long( ((String)o2).substring(((String)o2).indexOf("_")+2) )).longValue();
                        } catch(Exception exx) {
                            v2 = 0;
                        }
                    }

                    if( number1 && !number2 )
                        return 1;

                    if( !number1 && number2 )
                        return -1;

                    try {
                        long cmp_val =  v1 - v2;
                        return cmp_val>0?1:(cmp_val<0?-1:0);
                    } catch (Exception ex) {}

                    return 0;
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        ip_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    try {
                        long cmp_val = Utils.ipstr2long((String)o1) -
                            Utils.ipstr2long((String)o2);
                        return cmp_val>0?1:(cmp_val<0?-1:0);
                    } catch (Exception ex) {}
                    return 0;
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        full_ip_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Hashtable ipH1 = (Hashtable)((Map.Entry)o1).getValue();
                    Hashtable ipH2 = (Hashtable)((Map.Entry)o2).getValue();

                    long cmp_val = Utils.ipstr2long((String)ipH1.get("ip")) -
                        Utils.ipstr2long((String)ipH2.get("ip"));

                    boolean active1 = ((Boolean)ipH1.get("active?")).booleanValue();
                    boolean active2 = ((Boolean)ipH2.get("active?")).booleanValue();

                    if( active1 == active2 )
                        return cmp_val>0?1:(cmp_val<0?-1:0);
                    else if( active1 && !active2 )
                        return -1;
                    else
                        return 1;
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        as_ip_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Hashtable as1 = (Hashtable)((Map.Entry)o1).getValue();
                    Hashtable as2 = (Hashtable)((Map.Entry)o2).getValue();

                    if( as1.get("addr_space_prefix") == null )
                        return -1;
                    else if( as2.get("addr_space_prefix") == null )
                        return 1;

                    long cmp_val = Utils.ipstr2long((String)as1.get("addr_space_prefix")) -
                        Utils.ipstr2long((String)as2.get("addr_space_prefix"));

                    return cmp_val>0?1:(cmp_val<0?-1:0);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };
    }

    public static String getNextKey() {
        synchronized(nextKey) {
            try {
                nextKey = (new Long(Long.parseLong(nextKey) + (long)1)).toString();
            } catch (Exception ex) {}
            return " _ "+nextKey;
        }
    }

    public static String getParameterNameStaringWith(HttpServletRequest request, String startStr)
        throws Exception
    {
        Enumeration enm =  request.getParameterNames();
        while( enm.hasMoreElements() ) {
            String key = URLDecoder.decode((String)enm.nextElement(), "UTF-8");
            if( key.startsWith(startStr) ) {
                return key;
            }
        }
        return null;
    }

    public static String[] getParameterNamesStaringWith(HttpServletRequest request, String startStr)
        throws Exception
    {
        Vector names = new Vector();
        Enumeration enm =  request.getParameterNames();
        while( enm.hasMoreElements() ) {
            String key = (String)enm.nextElement();;
            if( URLDecoder.decode(key, "UTF-8").startsWith(startStr) )
                names.addElement(key);
        }
        String[] namesArray = new String[names.size()];
        for( int i = 0; i < names.size(); i++ )
            namesArray[i] = (String)names.elementAt(i);
        return namesArray;
    }

    public static String getDecodedParameter(HttpServletRequest request, String param)
        throws Exception
    {
        String masterKey = null;
        Enumeration enm =  request.getParameterNames();
        while( enm.hasMoreElements() ) {
            String key = (String)enm.nextElement();
            if( URLDecoder.decode(key, "UTF-8").equals(param) ) {
                masterKey = key;
                break;
            }
        }
        if( masterKey == null )
            return null;
        return URLDecoder.decode(request.getParameter(masterKey), "UTF-8");
    }

    public static String[] getParametersRegex(HttpServletRequest request, String matchStr)
        throws Exception
    {
        Vector names = new Vector();
        Vector keys = new Vector();
        Enumeration enm =  request.getParameterNames();
        while( enm.hasMoreElements() ) {
            String key = (String)enm.nextElement();
            String name = URLDecoder.decode(key, "UTF-8");
            if( name.matches(matchStr) ) {
                names.addElement(name);
                keys.addElement(key);
            }
        }
        String[] params = new String[names.size()*2];
        for( int i = 0; i < names.size(); i++ ) {
            params[2*i] = (String)names.elementAt(i);
            params[2*i+1] = URLDecoder.decode(request.getParameter((String)keys.elementAt(i)), "UTF-8");
        }
        return params;
    }

    public static String[] getParametersStaringWith(HttpServletRequest request, String startStr)
        throws Exception
    {
        Vector names = new Vector();
        Vector keys = new Vector();
        Enumeration enm =  request.getParameterNames();
        while( enm.hasMoreElements() ) {
            String key = (String)enm.nextElement();
            String name = URLDecoder.decode(key, "UTF-8");
            if( name.startsWith(startStr) ) {
                names.addElement(name);
                keys.addElement(key);
            }
        }
        String[] params = new String[names.size()*2];
        for( int i = 0; i < names.size(); i++ ) {
            params[2*i] = (String)names.elementAt(i);
            params[2*i+1] = URLDecoder.decode(request.getParameter((String)keys.elementAt(i)), "UTF-8");
        }
        return params;
    }

    public void addSessionMsg(HttpSession session, String msgType, String errorMsgToAdd) {
        if( errorMsgToAdd == null || errorMsgToAdd.trim().length() == 0 )
            return;

        String errorMsg = (String)session.getAttribute(msgType);
        if( errorMsg == null || errorMsg.trim().length() == 0 )
            errorMsg = "";
        else
            errorMsg += "\n";
        errorMsg += errorMsgToAdd;
        session.setAttribute(msgType, errorMsg);
    }

    public String[] decode(String[] src) throws Exception {
        if( src == null )
            return new String[0];
        String[] result = new String[src.length];
        for( int i = 0; i < src.length; i++ )
            result[i] = URLDecoder.decode(src[i], "UTF-8");
        return result;
    }

    public boolean reserveIPandName(HttpSession session, NetDBpi netdb, Hashtable ips, Hashtable store,
                                    Boolean ipStatus, String requested_ip_addr,
                                    String requested_addr_space, String requested_ip_name,
                                    int count)
        throws NetDBException, Exception
    {
        return reserveIPandName(session, netdb, ips, store, ipStatus, requested_ip_addr,
                                requested_addr_space, requested_ip_name, count, "IPC", true, true);
    }

    public boolean reserveIPandName(HttpSession session, NetDBpi netdb, Hashtable ips, Hashtable store,
                                    Boolean ipStatus, String requested_ip_addr,
                                    String requested_addr_space, String requested_ip_name,
                                    int count, boolean show_success_message)
        throws NetDBException, Exception
    {
        return reserveIPandName(session, netdb, ips, store, ipStatus, requested_ip_addr,
                                requested_addr_space, requested_ip_name, count, "IPC", show_success_message,
                                true);
    }

    // for interface ip addresses cont must be equal zero
    public boolean reserveIPandName(HttpSession session, NetDBpi netdb, Hashtable ips, Hashtable store,
                                    Boolean ipStatus, String requested_ip_addr,
                                    String requested_addr_space, String requested_ip_name,
                                    int count, String msgPrefix, boolean show_success_message,
                                    boolean reserve_names_where_applicable)
        throws NetDBException, Exception
    {
        if( requested_addr_space != null &&
            ( requested_addr_space.trim().length() == 0 ||
              requested_addr_space.equals("select an address space") ) )
            requested_addr_space = null;

        if( requested_ip_addr != null && requested_ip_addr.trim().length() == 0 )
            requested_ip_addr = null;

        if( requested_ip_addr == null && requested_addr_space == null ) {
            return false;
        }

        // if the requested ip is identical to the address space (minus the prefix length),
        // null the requested ip and use the address space.
        if ( requested_ip_addr != null && requested_addr_space != null) {
            String addr_short = requested_addr_space.substring (0, requested_addr_space.indexOf("/"));

            if (addr_short.equals (requested_ip_addr))
                requested_ip_addr = null;
        }

        try {
            Parser parser = new Parser();
            if( requested_ip_addr != null ) {
                requested_addr_space = null;
                Hashtable result = parser.parse(requested_ip_addr);
                String ip_low = (String)result.get("ip_low");
                String ip_high = (String)result.get("ip_high");
                if( ip_low == null || ip_high == null )
                    throw new Exception("Malformed ip address '"+requested_ip_addr+"'.");
                if( !ip_low.equals(ip_high) )
                    throw new Exception("Malformed ip address '"+requested_ip_addr+"'.");
            }

            if( requested_addr_space != null ) {
                Hashtable result = parser.parse(requested_addr_space);
                if( result.get("ip_low") == null || result.get("ip_high") == null )
                    throw new Exception("Malformed ip address space '"+requested_addr_space+"'");
            }
        } catch(Exception ex) {
            addSessionMsg(session, "Error.errorMsg", "Error: "+ex.getMessage());
            return false;
        }

        String default_domain = (String)session.getAttribute("default_domain");

        Vector reserved_ip = new Vector();
        Vector reserved_name = new Vector();

        Hashtable reserved_ips = (Hashtable)store.get("reserved_ips");
        if( reserved_ips == null ) {
            reserved_ips = new Hashtable();
            store.put("reserved_ips", reserved_ips);
        }
        Hashtable reserved_names = (Hashtable)store.get("reserved_names");
        if( reserved_names == null ) {
            reserved_names = new Hashtable();
            store.put("reserved_names", reserved_names);
        }

        Hashtable troubled_ips = (Hashtable)store.get("troubled_ips");
        if( troubled_ips == null ) {
            troubled_ips = new Hashtable();
            store.put("troubled_ips", troubled_ips);
        }
        Hashtable troubled_names = (Hashtable)store.get("troubled_names");
        if( troubled_names == null ) {
            troubled_names = new Hashtable();
            store.put("troubled_names", troubled_names);
        }

        try {
            try {
                Hashtable result = netdb.reserveIP(requested_addr_space, requested_ip_addr, count==0?1:count);
                result = nHash(result.get(result.keys().nextElement()));
                Hashtable warnings = nHash(result.get("warnings"));
                if( warnings.size() > 0 ) {
                    for( String msg : (Set<String>)warnings.keySet() )
                        addSessionMsg(session, "Warning", msg.trim());
                }

                Hashtable ip_addresses = nHash(result.get("ip_addresses"));
                for( String ip : (Set<String>)ip_addresses.keySet() )
                    reserved_ip.
                        addElement(Utils.hash("ip", ip,
                                              "active?", ipStatus,
                                              "names", new Hashtable()));

                if( reserved_ip.size() == 0 )
                    throw new Exception("Error: Failed to reserve ip address.");

                if( count == 0) {
                    if( requested_ip_name != null && requested_ip_name.trim().length() > 0 ) {
                        if( requested_ip_name.length() > 255 )
                            throw new Exception("Error: Name '"+requested_ip_name+"' is too long");
                        String r_name = netdb.reserveName(requested_ip_name.trim(), Constants.ADDRESS_NAME_TYPE);
                        if( r_name != null && r_name.trim().length() > 0 ) {
                            synchronized(nextKey) {
                                Hashtable nameStore = new Hashtable();
                                nameStore.put("name", Utils.name(r_name));
                                nameStore.put("domain", Utils.domain(r_name, default_domain));
                                ((Hashtable)((Hashtable)reserved_ip.elementAt(0)).get("names")).put(r_name, nameStore);
                            }
                            if( show_success_message )
                                addSessionMsg(session, "info_msg", "Success: Interface IP address name verified.");
                        } else {
                            String requested_name_fqdn = Utils.name(requested_ip_name.trim()) + "." +
                                Utils.domain(requested_ip_name.trim(), default_domain);
                            synchronized(nextKey) {
                                Hashtable nameStore = new Hashtable();
                                nameStore.put("fqdn", requested_name_fqdn);
                                nameStore.put("type", Constants.ADDRESS_NAME_TYPE);
                                troubled_names.put(Constants.ADDRESS_NAME_TYPE+" "+requested_name_fqdn, nameStore);
                            }
                            throw new Exception("Error: Failed to reserve ip address name.");
                        }
                    }
                    if( show_success_message )
                        addSessionMsg(session, "info_msg", "Success: Interface IP address verified.");

                } else if( reserve_names_where_applicable ) {
                    // count > 0 which means it is ipc or dhcp ip address(es)
                    for( int i = 0; i < reserved_ip.size(); i++ ) {
                        Hashtable ipH = (Hashtable)reserved_ip.elementAt(i);
                        Hashtable generated_name = netdb.generateHexName((String)ipH.get("ip"), 0);
                        String generated_name_fqdn = ((String)generated_name.get("name")).trim()+"."+((String)generated_name.get("domain")).trim();
                        String r_name = null;
                        try {
                            r_name = netdb.reserveName(generated_name_fqdn, Constants.ADDRESS_NAME_TYPE);
                        } catch (Exception exxx) { }
                        if( r_name != null && r_name.trim().length() > 0 ) {
                            synchronized(nextKey) {
                                Hashtable nameStore = new Hashtable();
                                nameStore.put("name", Utils.name(r_name));
                                nameStore.put("domain", Utils.domain(r_name, default_domain));
                                ((Hashtable)ipH.get("names")).put(r_name, nameStore);
                            }
                        } else {
                            synchronized(nextKey) {
                                Hashtable nameStore = new Hashtable();
                                nameStore.put("fqdn",   generated_name_fqdn);
                                nameStore.put("name",   Utils.name(generated_name_fqdn));
                                nameStore.put("domain", Utils.domain(generated_name_fqdn, default_domain));
                                nameStore.put("type",   Constants.ADDRESS_NAME_TYPE);
                                troubled_names.put(Constants.ADDRESS_NAME_TYPE+" "+generated_name_fqdn, nameStore);
                                ((Hashtable)ipH.get("names")).put(generated_name_fqdn, nameStore);
                            }
                        }
                    }
                    if( reserved_ip.size() > 0 && show_success_message ) {
                        if( msgPrefix.equals("NodeCreate") ) msgPrefix = "";
                        addSessionMsg(session, "info_msg", "Success: "+msgPrefix+" IP address name(s) verified.");
                    }
                }


                synchronized(nextKey) {
                    for( int i = 0; i < reserved_ip.size(); i++ ) {
                        Hashtable ipH = (Hashtable)reserved_ip.elementAt(i);
                        ips.put(ipH.get("ip"), ipH);
                        String rip = (String)ipH.get("ip");
                        reserved_ips.put(rip, "");
                        troubled_ips.remove(rip.trim().toLowerCase());
                        Hashtable names = nHash(ipH.get("names"));
                        for( String name_fqdn : (Set<String>)names.keySet() )
                            reserved_names.put(Constants.ADDRESS_NAME_TYPE+" "+name_fqdn,
                                               Utils.hash("fqdn", name_fqdn,
                                                          "type", Constants.ADDRESS_NAME_TYPE));
                    }
                }

            } catch (NetDBException ex) {
                Vector errors = ex.getMessages();
                String msg = "";
                int esize = errors.size();
                for( int i = 0; i < esize; i++ ) {
                    String eMsg = ((String)Utils.n2s((String)errors.elementAt(i))).trim();
                    if( eMsg.matches("Address Space \".*\" does not exist\\.") )
                        setData(session, "myAddrSpaces", netdb.listMyAddrSpaces());
                    msg += "Error: "+eMsg+(i<esize-1?"\n":"");
                }
                throw new Exception(msg);
            }
            return true;

        } catch (Exception ex) {
            String msg = ex.getMessage();
            if( msg.equals("Error: The requested number of addresses is greater than the number of available addresses.") &&
                msgPrefix.equals("NodeCreate") )
                addSessionMsg(session, "Warning", msg.replaceAll("Error: ","Warning: "));
            else
                addSessionMsg(session, "Error.errorMsg", msg);

            try {
                synchronized(nextKey) {
                    for( int i = 0; i < reserved_ip.size(); i++ ) {
                        Hashtable ipH = (Hashtable)reserved_ip.elementAt(i);
                        String ipStr = (String)ipH.get("ip");
                        reserved_ips.remove(ipStr);
                        try {
                            netdb.cancelIP(ipStr);
                        } catch(Exception exxx) {}
                        for( String name_fqdn : (Set<String>)nHash(ipH.get("names")).keySet() ) {
                            reserved_names.remove(Constants.ADDRESS_NAME_TYPE+" "+name_fqdn);
                            try {
                                netdb.cancelName(name_fqdn, Constants.ADDRESS_NAME_TYPE);
                            } catch(Exception exxx) {}
                        }
                    }
                }
            } catch (Exception exx) { }
        }
        return false;
    } // end of reserveIPandName(...)

    public String checkDuplicateNames(Hashtable obj) {
        cleanUpDuplicateStatus(obj);

        String err_messages = "";
        Hashtable a_names = new Hashtable();
        Hashtable c_names = new Hashtable();
        Hashtable mx_names = new Hashtable();

        String err_msgs =
            processANames((String)obj.get("object"),
                          nHash(obj.get("names")),
                          a_names, c_names, mx_names);
        if( err_msgs != null && err_msgs.trim().length() > 0 )
            err_messages += (err_messages.length()>0?"\n":"")+err_msgs;

        // interfaces and ip names
        Hashtable intfcs = nHash(obj.get("interfaces"));
        for( Hashtable intfc : (Collection<Hashtable>)intfcs.values() ) {
            err_msgs =
                processANames((String)obj.get("object"),
                              nHash(intfc.get("names")),
                              a_names, c_names, mx_names);
            if( err_msgs != null && err_msgs.trim().length() > 0 )
                err_messages += (err_messages.length()>0?"\n":"")+err_msgs;

            Hashtable ips = nHash(intfc.get("ips"));
            for( Hashtable ip : (Collection<Hashtable>)ips.values() ) {
                err_msgs =
                    processANames((String)obj.get("object"),
                                  nHash(ip.get("names")),
                                  a_names, c_names, mx_names);
                if( err_msgs != null && err_msgs.trim().length() > 0 )
                    err_messages += (err_messages.length()>0?"\n":"")+err_msgs;
            }
        }

        // ipc names
        Hashtable ipc_addresses = nHash(obj.get("ipc_addresses"));
        for( Hashtable ip : (Collection<Hashtable>)ipc_addresses.values() ) {
            err_msgs =
                processANames((String)obj.get("object"),
                              nHash(ip.get("names")),
                              a_names, c_names, mx_names);
            if( err_msgs != null && err_msgs.trim().length() > 0 )
                err_messages += (err_messages.length()>0?"\n":"")+err_msgs;
        }

        // address spaces
        Hashtable ases = nHash(obj.get("address_spaces"));
        for( Hashtable as : (Collection<Hashtable>)ases.values() ) {
            Hashtable ips = nHash(as.get("dhcp_addresses"));
            for( Hashtable ip : (Collection<Hashtable>)ips.values() ) {
                err_msgs =
                    processANames((String)obj.get("object"),
                                  nHash(ip.get("names")),
                                  a_names, c_names, mx_names);
                if( err_msgs != null && err_msgs.trim().length() > 0 )
                    err_messages += (err_messages.length()>0?"\n":"")+err_msgs;
            }
        }

        if( err_messages != null && err_messages.trim().length() == 0 )
            return null;

        return err_messages;
    } // end of checkDuplicateNames(...)

    private String processANames(String oType, Hashtable names,
                                Hashtable a_names, Hashtable c_names,
                                Hashtable mx_names)
    {
        String err_messages = "";
        for( Hashtable nameH : (Collection<Hashtable>)names.values() ) {
            String baseName = ((String)nameH.get("name")).trim();
            String fqdn = baseName+"."+
                ((String)nameH.get("domain")).trim();
            if( baseName.length() > 0 && (a_names.get(fqdn) != null || c_names.get(fqdn) != null) ) {
                err_messages += (err_messages.length()>0?"\n":"")+"Error: "+
                    Utils.capitalize(oType)+" contains duplicate name '"+fqdn+"'.";
                nameH.put("error", "");

                if( a_names.get(fqdn) != null )
                    ((Hashtable)a_names.get(fqdn)).put("error", "");

                if( c_names.get(fqdn) != null )
                    ((Hashtable)c_names.get(fqdn)).put("error", "");
            }
            if( baseName.length() > 0 )
                a_names.put(fqdn, nameH);

            Hashtable aliases = nHash(nameH.get("aliases"));
            String err_msgs = processCNames(oType, nHash(nameH.get("aliases")), a_names,
                                            c_names, mx_names);
            if( err_msgs != null && err_msgs.trim().length() > 0 )
                err_messages += (err_messages.length()>0?"\n":"")+err_msgs;

            err_msgs = processMXNames(oType, nHash(nameH.get("mxs")), a_names,
                                      c_names, mx_names);
            if( err_msgs != null && err_msgs.trim().length() > 0 )
                err_messages += (err_messages.length()>0?"\n":"")+err_msgs;
        }
        return err_messages;
    } // processANames(...)

    private String processCNames(String oType, Hashtable names,
                                Hashtable a_names, Hashtable c_names,
                                Hashtable mx_names)
    {
        String err_messages = "";
        for( Hashtable nameH : (Collection<Hashtable>)names.values() ) {
            String baseName = ((String)nameH.get("name")).trim();
            String fqdn = baseName+"."+
                ((String)nameH.get("domain")).trim();

            if( baseName.length() > 0 &&
                ( a_names.get(fqdn)  != null ||
                  c_names.get(fqdn)  != null ||
                  mx_names.get(fqdn) != null ) )
                {
                    err_messages += (err_messages.length()>0?"\n":"")+"Error: "+
                        Utils.capitalize(oType)+" contains duplicate name '"+fqdn+"'.";

                    if( a_names.get(fqdn) != null )
                        ((Hashtable)a_names.get(fqdn)).put("error", "");

                    if( c_names.get(fqdn) != null )
                        ((Hashtable)c_names.get(fqdn)).put("error", "");

                    if( mx_names.get(fqdn) != null )
                        ((Hashtable)mx_names.get(fqdn)).put("error", "");

                    nameH.put("error", "");
                }
            if( baseName.length() > 0 )
                c_names.put(fqdn, nameH);
        }
        return err_messages;
    } // processCNames(...)

    private String processMXNames(String oType, Hashtable names,
                                Hashtable a_names, Hashtable c_names,
                                Hashtable mx_names)
    {
        String err_messages = "";
        for( Hashtable nameH : (Collection<Hashtable>)names.values() ) {
            String baseName = ((String)nameH.get("name")).trim();
            String fqdn = baseName+"."+
                ((String)nameH.get("domain")).trim();

            if( baseName.length() > 0 &&
                c_names.get(fqdn)  != null )
                {
                    err_messages += (err_messages.length()>0?"\n":"")+"Error: "+
                        Utils.capitalize(oType)+" contains duplicate name '"+fqdn+"'.";

                    if( c_names.get(fqdn) != null )
                        ((Hashtable)c_names.get(fqdn)).put("error", "");

                    if( mx_names.get(fqdn) != null )
                        ((Hashtable)mx_names.get(fqdn)).put("error", "");

                    nameH.put("error", "");
                }
            if( baseName.length() > 0 )
                mx_names.put(fqdn, nameH);
        }
        return err_messages;
    } // processMXNames(...)

    private void cleanUpDuplicateStatus(Hashtable obj) {
    }

    protected String nameType(String key) {
        if (     key.startsWith("NODE NAME ALIAS ") )
            return Constants.NAME_ALIAS_TYPE;
        else if( key.startsWith("NODE NAME MX ") )
            return Constants.MX_TYPE;
        else if( key.startsWith("NODE NAME ") )
            return Constants.NODE_NAME_TYPE;
        else if( key.startsWith("NET NAME ") )
            return Constants.NETWORK_NAME_TYPE;
        else if( key.startsWith("IP NAME ") )
            return Constants.ADDRESS_NAME_TYPE;
        else if( key.startsWith("INTFC NAME ") )
            return Constants.INTERFACE_NAME_TYPE;
        return "";
    } // end of nameType(...)

    protected String simplifiedType(String nameType) {
        if( nameType.equals(Constants.NAME_ALIAS_TYPE) )
            return "alias";
        else if( nameType.equals(Constants.MX_TYPE) )
            return "mx";
        else
            return "name";
    } // end of simplifiedType(...)

    public void modTop(HttpSession session, PrintWriter out, String object, String handle, String oType, String displayObjType,
                       String hlpPage, String sessionKey, String rootURL)
        throws Exception
    {
        Hashtable store = Utils.nHash(session.getAttribute(sessionKey));
        out.println(getCommonHtmlBegin((rootURL.equals("/Modify"+oType+"")?("Modify"+oType+": "+Utils.htmlize(object, false)):(""+oType+"Create")),
                                       Utils.nStr(store.get("PosX")), Utils.nStr(store.get("PosY")))+
                    commonHeader(session, "/Dispatch?Action=redirect&to=/home&pre=unlock&object="+oType+"&sessionKey="+
                                 encode(sessionKey),
                                 SHOW_HOME_LINK, NO_PROFILE_LINK, NO_LOGOUT_LINK,
                                 "", (rootURL.equals("/Modify"+oType+"")?("Modify "+displayObjType+"<br>"+
                                                                          Utils.htmlize(handle, false)):("Create "+displayObjType)),
                                 hlpPage)+
                    msgPanels(session)+
                    "<DIV id=\"modifyPage\">\n"+
                    "<FORM NAME=\"MainForm\" METHOD=\"POST\" "+
                    "onSubmit=\"document.MainForm.PosX.value = (document.all)?document.body.scrollLeft:window.pageXOffset; "+
                    "document.MainForm.PosY.value = (document.all)?document.body.scrollTop:window.pageYOffset;\" ACTION='"+rootURL+"'>"+
                    "<INPUT TYPE='HIDDEN' VALUE='' NAME='PosX'>"+
                    "<INPUT TYPE='HIDDEN' VALUE='' NAME='PosY'>"+
                    "<P CLASS=\"form_buttons\">\n"+
                    "<INPUT ID='Button_Save'   ACCESSKEY=\"S\" TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Save\"> "+
                    "<INPUT ID='Button_Verify' ACCESSKEY=\"V\" TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Verify\">\n"+
                    "<INPUT ID='Button_Reset'  ACCESSKEY=\"R\" TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Reset\">\n"+
                    "<INPUT ID='Button_Cancel' ACCESSKEY=\"C\" TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Cancel\">\n"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"sessionKey\" VALUE=\""+encode(sessionKey)+"\">"+
                    "</P>\n"+
                    "<DIV class='modHeader'>\n"+
                    "<TABLE WIDTH='100%'>\n"+
                    "<TR><TD WIDTH='1%' COLSPAN='2' NOWRAP><B>"+displayObjType+" Information</B><BR>"+
                    "(<font class='star'>*</font> = required)</TD></TR>\n"+
                    "</TABLE>\n"+
                    "</DIV>"+
                    "<TABLE CELLSPACING='0' WIDTH='100%'>");
    } // end of modTop(...)

    public void modBottom(PrintWriter out) {
        out.println("</TABLE>\n"+
                    "<HR NOSHADE SIZE=2>\n"+
                    "<P CLASS=\"form_buttons\">\n"+
                    "<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Save\">\n"+
                    "<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Verify\">\n"+
                    "<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Reset\">\n"+
                    "<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Cancel\">\n"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"ready\" VALUE=\"yes\">\n"+
                    "</P>\n"+
                    "</FORM>\n<BR>\n"+
                    getCommonBottomLabel()+
                    "</DIV>\n"+
                    "</BODY>\n"+
                    "</HTML>\n");
    } // end of modBottom(...)

    public void fillInPeople(Hashtable hash, Hashtable foundPeople) {
        boolean checked = foundPeople.size()>1?false:true;
        for( String key : (Set<String>)foundPeople.keySet() ) {
            Hashtable newPerson = (Hashtable)foundPeople.get(key);
            String newRegID = (String)newPerson.get("RegID");
            boolean add = true;
            for( Hashtable oldPerson : (Collection<Hashtable>)hash.values() ) {
                String oldRegID = (String)oldPerson.get("RegID");
                if( newRegID.equals(oldRegID) ) {
                    add = false;
                    break;
                }
            }

            if( !checked ) // setting this key means that check box won't be selected
                newPerson.put("not selected", "");

            if( add )
                hash.put(key, newPerson);
        }
    } // end of fillInPeople(...)

    public void updatePersons(NetDBpi netdb, Hashtable persons)
        throws Exception, NetDBException
    {
        for( Hashtable person : (Collection<Hashtable>)persons.values() ) {
            Object oType = person.get("object");
            if( oType != null && (oType instanceof String) &&
                ((String)oType).equals("person") )
                Utils.appendHash(person, netdb.update(person, person));
        }
    }

    // Given object obj and a name will extact this name from this object in its
    // original case.
    public String extractName(Hashtable obj, String name_any_case) {
        Hashtable names = Utils.nHash(obj.get("names"));
        for( String key : (Set<String>)names.keySet() ) {
            String name = key.trim().toLowerCase();
            if( name.equals(name_any_case.trim().toLowerCase()) ) {
                Hashtable nameH = (Hashtable)names.get(key);
                return nameH.get("name")+"."+nameH.get("domain");
            }
        }
        return "";
    }

    public boolean nameIn(String type, String fqdn, Hashtable nameStore) {
        for( Hashtable name : (Collection<Hashtable>)nameStore.values() ) {
            String nameFQDN = (String)name.get("fqdn");
            String nameType = (String)name.get("type");
            if( ( nameType.equals(Constants.MX_TYPE) && !type.equals(Constants.MX_TYPE) ) ||
                ( !nameType.equals(Constants.MX_TYPE) && type.equals(Constants.MX_TYPE) ) )
                continue;

            if( nameFQDN.toLowerCase().equals(fqdn.toLowerCase()) )
                return true;
        }
        return false;
    }

    public String firstName(Hashtable names) {
        Comparator deep_name_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Hashtable name1 = (Hashtable)((Map.Entry)o1).getValue();
                    Hashtable name2 = (Hashtable)((Map.Entry)o2).getValue();

                    String name_fqdn_1 =
                        ((String)Utils.n2s(name1.get("name"))).trim()+"."+
                        ((String)Utils.n2s(name1.get("domain"))).trim();

                    String name_fqdn_2 =
                        ((String)Utils.n2s(name2.get("name"))).trim()+"."+
                        ((String)Utils.n2s(name2.get("domain"))).trim();


                    return name_fqdn_1.compareToIgnoreCase(name_fqdn_2);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        java.util.List list =
            Arrays.asList(names.entrySet().toArray());
        Collections.sort(list, deep_name_cmp);
        Iterator it = list.iterator();
        while( it.hasNext() ) {
            Map.Entry entry = (Map.Entry)it.next();
            Hashtable name = (Hashtable)entry.getValue();
            return ((String)name.get("name")).trim()+"."+
                ((String)name.get("domain")).trim();
        }
        return "";
    }
}
