/************************************************************************
 *
 * MoveAS.java -
 *
 * $Id: MoveAS.java,v 1.3 2006/09/26 23:55:39 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2004, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.sql.*;
import java.net.*;
import java.util.*;
import java.text.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import su.netdb.parser.*;
import stanford.netdb.utils.*;

public class MoveAS extends ModifyPage {

    public MoveAS() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("type", "Network");
        Timestamp lockTm = null;
        String handle = null;
        NetDBpi netdb = null;

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        try {
            PrintWriter out = response.getWriter();
            String rootURL = request.getServletPath();

            netdb = (NetDBpi)session.getAttribute("netdb");
            String netID = (String)session.getAttribute("NetID");
            String default_domain = (String)session.getAttribute("default_domain");

            // obtain key network attributes from request
            String object = request.getParameter("handle");
            Hashtable sourceNet = null;

            if( object != null ) {
                try {
                    int oid = Integer.parseInt(object);
                    sourceNet = netdb.getInfo (Constants.NETWORK, oid);

                    if ( sourceNet == null || sourceNet.get("id") == null )
                        throw new Exception("Error: No info found for network "+object);

                    object = (String)nHash(sourceNet.get("names")).keys().nextElement();
                } catch(Exception ex) { }
                handle = new String(object);
                store.put("handle", handle);
                // lock
                try {
                    lockTm = netdb.lock(Constants.NETWORK, handle);
                    if( lockTm == null )
                        throw new Exception("Failed to lock network "+handle);

                } catch (Exception ex) {
                    ex.printStackTrace();
                    session.setAttribute("Error.errorMsg", ex.getMessage());
                    response.sendRedirect((String)store.get("fromPage"));
                    return;
                }
                store.put("lockTm", lockTm);
            }

            // If we were passed the ID instead of the handle, we got the info above.
            if (sourceNet == null)
                sourceNet = netdb.getInfo(Constants.NETWORK, object);

            if ( sourceNet == null || sourceNet.get("id") == null )
                throw new Exception("Error: No info found for network "+object);

            if( lockTm != null )
                sourceNet.put("locked", lockTm);

            store.put("sourceNet", sourceNet);

            Vector destNets = new Vector();
            Hashtable myNets = nHash(netdb.listMyNetworks());
            String handleLower = handle.toLowerCase().trim();
            java.util.List list = Arrays.asList(myNets.keySet().toArray());
            Collections.sort(list, name_cmp);
            Iterator it = list.iterator();
            while( it.hasNext() ) {
                String net = ((String)it.next()).trim();
                if( !net.toLowerCase().trim().equals(handleLower) )
                    destNets.addElement(net);
            }
            store.put("destNets", destNets);

            // extact name with original case
            object = extractName(sourceNet, object);

            // compress network name by stripping default domain
            object = compressName(object, default_domain);
            store.put("object", object);

            printOutModifyHTML(session, out, store, rootURL);

        } catch (Exception ex) {
            try {
                if( lockTm != null && handle != null && netdb != null )
                    netdb.unlock(Constants.NETWORK, handle, lockTm);
            } catch (Exception exx) {}
            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
            response.sendRedirect((String)store.get("fromPage"));
        }
    }

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        HttpSession session = begin(request, response, null);
        if( session == null )
            return;

        PrintWriter out = response.getWriter();
        String rootURL = request.getServletPath();

        String sessionKey = decode(request.getParameter("sessionKey"));
        String default_domain = (String)session.getAttribute("default_domain");
        Hashtable store = (Hashtable)session.getAttribute(sessionKey);
        NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
        String handle = (String)store.get("handle");
        Timestamp lockTm = (Timestamp)store.get("lockTm");

        Hashtable sourceNet = (Hashtable)store.get("sourceNet");

        String action = (String)Utils.n2s((String)request.getParameter("Action"));

        try {
            if( action.equals("Cancel") ) {
                try {
                    if( lockTm != null )
                        netdb.unlock(Constants.NETWORK, handle, lockTm);
                } catch (Exception ex) { }

                response.sendRedirect((String)store.get("fromPage"));
                return;

            } else {
                String destNet = decode(((String)Utils.n2s(request.getParameter("destNet"))));
                store.put("last_selected_destination", destNet);

                String[] ases = request.getParameterValues("as");

                Hashtable addrSpaces = nHash(sourceNet.get("address_spaces"));
                for( Object key : addrSpaces.keySet() )
                    nHash(addrSpaces.get(key)).remove("checked");
                if( ases != null ) {
                    for( int i = 0; i < ases.length; i++ ) {
                        String asPrefix = decode(ases[i]);
                        nHash(addrSpaces.get(asPrefix)).put("checked", "");
                    }
                }

                if( destNet.equals("--") )
                    addSessionMsg(session, "Error.errorMsg",
                                  "Error: Please select destination network.");
                else if( ases == null || ases.length == 0 )
                    addSessionMsg(session, "Error.errorMsg",
                                  "Error: Please select address spaces you want to move.");
                else {
                    // moving addr. spaces
                    Vector asList = new Vector();
                    for( int i = 0; i < ases.length; i++ ) {
                        Hashtable asToMove = new Hashtable();
                        asToMove.put("prefix", decode(ases[i]));
                        asToMove.put("lock", lockTm);
                        asList.addElement(asToMove);
                    }

                    try {
                        try {
                            netdb.moveAS(asList, destNet);

                            try {
                                netdb.unlock(Constants.NETWORK, handle, lockTm);
                            } catch (Exception ex ) { }

                            // reload my addr. spaces
                            setData(session, "myAddrSpaces", netdb.listMyAddrSpaces());

                            addSessionMsg(session, "info_msg", "SUCCESS: Address spaces were moved.");
                            response.sendRedirect((String)store.get("fromPage"));
                            return;

                        } catch (NetDBException ex) {
                            Vector errors = ex.getMessages();
                            String msg = "";
                            int esize = errors.size();
                            for( int j = 0; j < esize; j++ )
                                msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                            throw new Exception(msg);
                        }

                    } catch (Exception ex) {
                        addSessionMsg(session, "Error.errorMsg",ex.getMessage());
                    }
                }
            }
            printOutModifyHTML(session, out, store, rootURL);

        } catch (Exception ex) {
            // if we are here something unexpected happened
            // we should never end up here
            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
            response.sendRedirect((String)store.get("fromPage"));
        }
    }

    public void printOutModifyHTML(HttpSession session, PrintWriter out, Hashtable store, String rootURL)
        throws Exception
    {
        String default_domain = (String)session.getAttribute("default_domain");

        String sessionKey = (String)store.get("sessionKey");
        Timestamp lockTm = (Timestamp)store.get("lockTm");
        Hashtable sourceNet = (Hashtable)store.get("sourceNet");
        String handle = (String)store.get("handle");
        String object = (String)store.get("object");
        Vector destNets = (Vector)store.get("destNets");
        String last_selected_destination = (String)Utils.n2s(store.get("last_selected_destination"));

        out.println(commonHtmlBegin("Move Address Space(s)")+
                    commonHeader(session, "/Dispatch?Action=redirect&to=/home&pre=unlock&object=Network&sessionKey="+
                                 encode(sessionKey),
                                 SHOW_HOME_LINK, NO_PROFILE_LINK, NO_LOGOUT_LINK,
                                 "", "Move Address Space(s)<br>"+object,
                                 "hlpnetwork.html")+
                    msgPanels(session)+
                    "<DIV id=\"asPage\">\n"+
                    "<FORM METHOD=\"POST\" ACTION='"+rootURL+"'>"+
                    "<P CLASS=\"form_buttons\">\n"+
                    "&nbsp;<INPUT ID='Button_Move_Address_Space' TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Move Address Space(s)\"> "+
                    "<INPUT ID='Button_Cancel' TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Cancel\">\n"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"sessionKey\" VALUE=\""+encode(sessionKey)+"\">"+
                    "</P>\n"+
                    "<HR NOSHADE SIZE=2>\n"+
                    "<TABLE CELLSPACING='0' WIDTH='100%'>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%' NOWRAP>&nbsp;Source Network</TD><TD><B>"+object+"</B></TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%' NOWRAP>&nbsp;Destination Network</TD>"+
                    "<TD><SELECT ID='Selector_Destination_Network' SIZE=1 NAME='destNet'>"+
                    "<OPTION VALUE='"+encode("--")+"'>--");

        for( int i = 0; i < destNets.size(); i++ ) {
            String dest = (String)destNets.elementAt(i);
            out.println("<OPTION VALUE='"+encode(dest)+"'"+(dest.equals(last_selected_destination)?" SELECTED":"")+">"+dest);
        }

        out.println("</TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%' NOWRAP>&nbsp;Address Space(s)</TD><TD VALIGN='TOP'>");

        Hashtable ases = nHash(sourceNet.get("address_spaces"));
        java.util.List list = Arrays.asList(ases.entrySet().toArray());
        Collections.sort(list, as_ip_cmp);
        Iterator it = list.iterator();
        int index = 0;
        while( it.hasNext() ) {
            Map.Entry entry = (Map.Entry)it.next();
            Hashtable as = (Hashtable)entry.getValue();

            String asPrefix = (String)as.get("addr_space_prefix");
            out.println("<INPUT ID='CheckBox_Address_Space_"+(index++)+"' TYPE='checkbox' "+(as.get("checked")!=null?"CHECKED ":"")+"NAME='as' value='"+encode(asPrefix)+"'>"+
                            asPrefix+"<BR>");
        }

        out.println("</TD></TR>");

        out.println("</TABLE>\n"+
                    "<HR NOSHADE SIZE=2>\n"+
                    "<P CLASS=\"form_buttons\">\n"+
                    "&nbsp;<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Move Address Space(s)\"> "+
                    "<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Cancel\">\n"+
                    "</P>\n"+
                    "</FORM>\n<BR>\n"+
                    getCommonBottomLabel()+
                    "</DIV>\n"+
                    "</BODY>\n"+
                    "</HTML>\n");
    }
}
