/************************************************************************
 *
 * Profile.java -
 *
 * $Id: Profile.java,v 1.2 2006/11/06 21:16:33 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2004, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.sql.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import su.netdb.parser.*;
import stanford.netdb.utils.*;

public class Profile extends ModifyPage {

    public Profile() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        try {
            PrintWriter out = response.getWriter();
            String rootURL = request.getServletPath();

            Hashtable user = nHash(session.getAttribute("user"));
            store.put("newUser", Utils.copyHash(user));
            store.put("oldUser", Utils.copyHash(user));

            printOutModifyHTML(session, out, store, rootURL);

        } catch (Exception ex) {
            // display error page
            addSessionMsg(session, "Error.errorMsg", "Error: "+ex.getMessage());
            response.sendRedirect((String)store.get("fromPage"));
            return;
        }

    } // end of doGet(...)

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        HttpSession session = begin(request, response, null);
        if( session == null )
            return;

        String sessionKey = decode(request.getParameter("sessionKey"));
        Hashtable store = (Hashtable)session.getAttribute(sessionKey);
        try {
            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            PrintWriter out = response.getWriter();
            String rootURL = request.getServletPath();

            String action = (String)Utils.n2s((String)request.getParameter("Action"));

            if( action.equals("Cancel") ) {
                response.sendRedirect((String)store.get("fromPage"));
                return;
            } else if( action.equals("Reset") ) {
                store.put("newUser", Utils.copyHash((Hashtable)store.get("oldUser")));

            } else if( action.equals("Modify Profile") ) {
                try {
                    try {
                        Hashtable user = (Hashtable)store.get("newUser");
                        user.put("default_group", decode(request.getParameter("def_group")));
                        user.put("default_domain", decode(request.getParameter("def_domain")));
                        String def_as = decode(((String)Utils.n2s(request.getParameter("def_as")))).trim();
                        String def_as_txt = request.getParameter("def_as_txt").trim();
                        def_as = def_as_txt.length()>0?def_as_txt:def_as;
                        if( def_as == null || def_as.trim().length() == 0 || def_as.equals("---") )
                            user.remove("default_addr_space");
                        else
                            user.put("default_addr_space", def_as);
                        user.put("updProfile", ""); // this indicates that we are updating profile
                        user.put("def_as_txt", def_as_txt);
                        netdb.update(user, user);
                        session.setAttribute("info_msg", "SUCCESS: Your profile has been updated.");
                        user.remove("def_as_txt");

                        // Reload user. We are locking anything here, but why,?
                        Hashtable currentUser = netdb.getCurrentUser();
                        session.setAttribute("user", currentUser);
                        session.setAttribute("default_domain", currentUser.get("default_domain"));

                        response.sendRedirect((String)store.get("fromPage"));
                        return;

                    } catch (NetDBException ex) {
                        ex.printStackTrace();
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int j = 0; j < esize; j++ )
                            msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                        throw new Exception(msg);
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                    addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                }
            }

            printOutModifyHTML(session, out, store, rootURL);

        } catch (Exception ex) {
            // display error page
            addSessionMsg(session, "Error.errorMsg", "Error: "+ex.getMessage());
            response.sendRedirect((String)store.get("fromPage"));
            return;
        }
    } // end of doPost(...)

    public void printOutModifyHTML(HttpSession session, PrintWriter out, Hashtable store, String rootURL) throws Exception {
        Hashtable user = nHash(store.get("newUser"));

        // format dept/consultant lines
        Vector official_depts = new Vector();
        Vector other_depts = new Vector();
        Hashtable ht = (Hashtable)user.get("departments");
        if (ht != null) {
            Object[] oa = ht.keySet().toArray();
            for (int i = 0; i < oa.length; ++i) {
                String d_key = (String)oa[i];
                Hashtable ht2 = (Hashtable)ht.get(d_key);
                Boolean b = (Boolean)ht2.get("official?");
                boolean isOfficial = (b != null && b.booleanValue());
                String curr_dept = "";
                Hashtable ht3 = (Hashtable)ht2.get("consultants");
                if (ht3 != null) {
                    Object[] oa2 = ht3.keySet().toArray();
                    for (int j = 0; j < oa2.length; ++j) {
                        String c_key = (String)oa2[j];
                        Hashtable person = (Hashtable)ht3.get(c_key);
                        String c_wlink = makePersonLink(person);
                        String c_name =
                            (c_wlink != null ? "<A HREF='"+c_wlink+"'>" : "")+
                            (String)person.get("name") +
                            (c_wlink != null ? "</A>" : "");

                        if (curr_dept.length() > 0)
                            curr_dept += ", ";
                        curr_dept += c_name;
                    }
                    if (curr_dept.length() > 0)
                        curr_dept = "( " + curr_dept + " )";
                }
                curr_dept = Utils.htmlize(d_key, false) + " " + curr_dept;

                if (isOfficial)
                    official_depts.addElement(curr_dept);
                else
                    other_depts.addElement(curr_dept);
            }
        }

        // sort and format official depts
        String o_officials = "";
        if (official_depts.size() > 0) {
            java.util.List alist = Arrays.asList(official_depts.toArray());
            Collections.sort(alist, name_cmp);
            Object[] oa = alist.toArray();
            for (int i = 0; i < oa.length; ++i) {
                if (o_officials.length() > 0)
                    o_officials += "<BR>";
                o_officials += (String)oa[i];
            }
        }

        // sort and format other depts
        String o_others = "";
        if (other_depts.size() > 0) {
            java.util.List alist = Arrays.asList(other_depts.toArray());
            Collections.sort(alist, name_cmp);
            Object[] oa = alist.toArray();
            for (int i = 0; i < oa.length; ++i) {
                if (o_others.length() > 0)
                    o_others += "<BR>";
                o_others += (String)oa[i];
            }
        }

        String records = "";
        Boolean allObjects = (Boolean)user.get("all_objects?");
        if( allObjects != null && allObjects.booleanValue() )
            records += "All Records";

        Hashtable recordsH = nHash(user.get("records"));
        if( recordsH.size() > 0 ) {
            if( records.length() > 0 )
                records += "<BR>\n";
            java.util.List list = Arrays.asList(recordsH.keySet().toArray());
            Collections.sort(list, name_cmp);
            Object[] rec = list.toArray();
            for( int i = 0; i < rec.length; i++ ) {
                String label = (String)recordsH.get(rec[i]);
                records += (i>0?", ":"")+label.trim();
            }
        }

        String groups = "";
        Boolean allGroups = (Boolean)user.get("all_groups?");
        if( allGroups != null && allGroups.booleanValue() )
            groups += "All Groups";

        Hashtable groupsH = nHash(user.get("groups"));
        if( groupsH.size() > 0 ) {
            if( groups.length() > 0 )
                groups += "<BR>\n";
            java.util.List list = Arrays.asList(groupsH.keySet().toArray());
            Collections.sort(list, name_cmp);
            Object[] group = list.toArray();
            for( int i = 0; i < group.length; i++ )
                groups += (i>0?", ":"")+Utils.htmlize(((String)group[i]).trim(), false);
        }

        String def_group = (String)user.get("default_group");
        String def_domain = (String)user.get("default_domain");
        String def_as = (String)Utils.n2s(user.get("default_addr_space"));

        String sessionKey = (String)store.get("sessionKey");
        out.println(commonHtmlBegin("Your NetDB user profile.")+
                    commonHeader(session, "/home",
                                 SHOW_HOME_LINK, NO_PROFILE_LINK, NO_LOGOUT_LINK,
                                 "", "Modify your NetDB user profile",
                                 "hlpuserprofile.html")+
                    msgPanels(session)+
                    "<DIV id=\"infoPage\">\n"+
                    "<FORM METHOD=\"POST\" ACTION='"+rootURL+"'>"+
                    "<P CLASS=\"form_buttons\">\n"+
                    "<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Modify Profile\"> "+
                    "<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Reset\">\n"+
                    "<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Cancel\">\n"+
                    //"<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Refresh Session\">\n"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"sessionKey\" VALUE=\""+encode(sessionKey)+"\">"+
                    "</P>\n"+
                    "<TABLE WIDTH='100%'>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%'>Name</TD><TD><B>"+
                    Utils.htmlize((String)Utils.n2s(user.get("name")), false)+"</B></TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%'>Comment</TD><TD><B>"+
                    Utils.htmlize((String)Utils.n2s(user.get("comment")), false)+"</B></TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%' NOWRAP>LNA for these departments&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"+
                    "<br>&nbsp;(Network Consultant)</TD><TD><B>"+
                    o_officials+"<BR></B></TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%' NOWRAP>Other departments you're associated with"+
                    "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</TD><TD><B>"+
                    o_others+"<BR></B></TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%' NOWRAP>Record access</TD><TD><B>"+
                    records+"<BR></B></TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%' NOWRAP>NetDB groups you belong to</TD><TD><B>"+
                    groups+"<BR></B></TD></TR>\n"+
                    "<TR><TD COLSPAN='2'><HR NOSHADE SIZE=1></TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%' NOWRAP>Default NetDB group for new records</TD><TD>"+
                    "<SELECT NAME='def_group' SIZE='1'>");

        Hashtable myGroups = (Hashtable)getData(session, "myGroups");
        java.util.List list = Arrays.asList(myGroups.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();
        while( it.hasNext() ) {
            String group = (String)it.next();
            out.println("<OPTION VALUE='"+encode(group)+"'"+
                        (group.equals(def_group)?" SELECTED":"")+">"+Utils.htmlize(group.trim(), false)+"</OPTION>");
        }
        out.println("</SELECT></TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%' NOWRAP>Default Domain for new records</TD><TD>"+
                    "<SELECT NAME='def_domain' SIZE='1'>");

        Hashtable domains = (Hashtable)getData(session, "domains");
        list = Arrays.asList(domains.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        while( it.hasNext() ) {
            String domain = (String)it.next();
            out.println("<OPTION VALUE='"+encode(domain)+"'"+
                        (domain.equals(def_domain)?" SELECTED":"")+">"+domain.trim()+"</OPTION>");
        }

        out.println("</SELECT></TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%' NOWRAP>Default Address space for new records</TD><TD>"+
                    "<SELECT NAME='def_as' SIZE='1'>"+
                    "<OPTION VALUE='"+
                    encode("---")+"'"+
                    (def_as.trim().length()==0?" SELECTED":"")+">---");

        Hashtable myAddrSpaces = (Hashtable)getData(session, "myAddrSpaces");
        list = Arrays.asList(myAddrSpaces.keySet().toArray());
        Collections.sort(list, ip_str_cmp);
        it = list.iterator();
        while( it.hasNext() ) {
            String prefix = (String)((Hashtable)myAddrSpaces.get(it.next())).get("prefix");
            out.println("<OPTION VALUE='"+
                        encode(prefix)+"'"+
                        (prefix.equals(def_as)?" SELECTED":"")+">"+prefix);
        }
        out.println("</SELECT> or <INPUT TYPE='TEXT' SIZE='15' NAME='def_as_txt' value='"+
                    Utils.htmlize((String)(user.get("def_as_txt")!=null?user.get("def_as_txt"):""), false)+
                    "'></TD></TR>\n"+
                    "</TABLE>\n"+
                    "<HR NOSHADE SIZE=2>\n"+
                    "<P CLASS=\"form_buttons\">\n"+
                    "<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Modify Profile\"> "+
                    "<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Reset\">\n"+
                    "<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Cancel\">\n"+
                    //"<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Refresh Session\">\n"+
                    "</P>\n"+
                    "</FORM>\n<BR>\n"+
                    getCommonBottomLabel()+
                    "</DIV>\n"+
                    "</BODY>\n"+
                    "</HTML>\n");
    }
}
