/************************************************************************
 *
 * QSearchResults.java - perform pre-specified quicksearch and display results
 *
 * $Id: QSearchResults.java,v 1.3 2006/10/11 20:42:50 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;
import su.netdb.parser.Parser;

public class QSearchResults extends ModifyPage
{

    public QSearchResults() { }

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        HttpSession session = begin(request, response, null);
        if( session == null )
            return;

        try {

            Comparator matched_names_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Hashtable res1 = (Hashtable)((Map.Entry)o1).getValue();
                    Hashtable res2 = (Hashtable)((Map.Entry)o2).getValue();

                    String name1 = (String)Utils.n2s(res1.get("name"));
                    String name2 = (String)Utils.n2s(res2.get("name"));

                    return name1.compareToIgnoreCase(name2);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

            String fromPage = URLEncoder.encode("/qresults", "UTF-8");

            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String netID = (String)session.getAttribute("NetID");
            String default_domain = (String)session.getAttribute("default_domain");

            // obtain search parms from session
            String search_string = (String)session.getAttribute("search_string");
            search_string = (search_string == null ? "" : search_string.trim());
            String search_object = (String)session.getAttribute("search_object");
            search_object = (search_object == null ? "Nodes" : search_object.trim());

            // decide which searches to perform
            boolean wantAll = search_object.equals("All Types");
            boolean wantATeams = wantAll || search_object.equals("Admin Teams");
            boolean wantGroups = wantAll || search_object.equals("Groups");
            boolean wantMXes = false; // wantAll || search_object.equals("MXes");
            boolean wantNetworks = wantAll || search_object.equals("Networks");
            boolean wantNodes = wantAll || search_object.equals("Nodes");
            boolean wantUsers = wantAll || search_object.equals("Users");
            boolean wantDomains = wantAll || search_object.equals("Domains");
            boolean gotAny = false;

            // display results page header
            PrintWriter out = response.getWriter();
            out.println(getCommonHtmlBegin("Quick Search Results")+
                        commonHeader(session, "/home", SHOW_HOME_LINK, SHOW_PROFILE_LINK, SHOW_LOGOUT_LINK,
                                     "", "Search Result", "hlpqsearch.html",
                                     "/qresults")+
                        //getCommonHeaderTable("", "Search Results", netID, (String)session.getAttribute("user_name"), fromPage)+
                        "Quick Search results for \"<B>"+Utils.htmlize(search_string)+"</B>\"<BR><BR>\n");

            try {
                Hashtable results = new Hashtable();

                // NODES
                if (wantNodes) {
                    int searchType = Constants.NODE;
                    StringBuffer sb = new StringBuffer();
                    try {
                        try {
                            results = netdb.simpleSearch(searchType, search_string, sb);
                        } catch (NetDBException ex) {
                            Vector errors = ex.getMessages();
                            String msg = "";
                            int esize = errors.size();
                            for( int j = 0; j < esize; j++ )
                                msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                            throw new Exception(msg);
                        }
                    } catch (Exception ex) {
                        addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                    }
                    results = nHash(results);

                    boolean wantIPSort = sb.toString().equals("ip address");
                    boolean wantHW = sb.toString().equals("hw address");
                    int resultsCt = results.size();

                    gotAny = true;
                    htmlTop(session, search_string, out);
                    if( wantHW )
                        out.println(
                                    "<TR CLASS=\"qs_table_header\">\n"+
                                    "  <TD NOWRAP COLSPAN=7>Nodes ( "+resultsCt+" )</TD>\n"+
                                    "</TR>\n"+
                                    "<TR CLASS=\"qs_table_sub_header\">\n"+
                                    "  <TD WIDTH='5%' ALIGN='RIGHT' NOWRAP>#&nbsp;&nbsp;</TD>"+
                                    "  <TD NOWRAP>HW Address</TD>"+
                                    "  <TD NOWRAP>Actions</TD>"+
                                    "  <TD NOWRAP>Node</TD>"+
                                    "  <TD NOWRAP>IP Address</TD>"+
                                    "  <TD COLSPAN='2' NOWRAP>Node Type</TD>"+
                                    "</TR>\n"
                                    );
                    else
                        out.println(
                                    "<TR CLASS=\"qs_table_header\">\n"+
                                    "  <TD NOWRAP COLSPAN=7>Nodes ( "+resultsCt+" )</TD>\n"+
                                    "</TR>\n"+
                                    "<TR CLASS=\"qs_table_sub_header\">\n"+
                                    "  <TD WIDTH='5%' ALIGN='RIGHT' NOWRAP>#&nbsp;&nbsp;</TD>"+
                                    "  <TD NOWRAP>"+(wantIPSort?"IP Address":"Name")+"</TD>"+
                                    "  <TD NOWRAP>"+(wantIPSort?"IP Type":"Name Type")+"</TD>"+
                                    "  <TD NOWRAP>Actions</TD>"+
                                    "  <TD NOWRAP>Node</TD>"+
                                    (wantIPSort?"":"<TD NOWRAP>IP Address</TD>\n")+
                                    "  <TD "+(wantIPSort?"COLSPAN='2' ":"")+"NOWRAP>Node Type</TD>"+
                                    "</TR>\n"
                                    );

                    // check if user can modify
                    Boolean checkValue = (Boolean) session.getAttribute("hasNodeObjectAccess?");
                    boolean hasAccess = checkValue.booleanValue();

                    Object[] resultsArray = null;
                    if (wantIPSort) {
                        // sort the results by ip-addr
                        java.util.List alist = Arrays.asList(results.entrySet().toArray());
                        Collections.sort(alist, ipentry_cmp);
                        Object[] entriesArray = alist.toArray();
                        resultsArray = new Object[entriesArray.length];
                        for (int i = 0; i < entriesArray.length; ++i) {
                            java.util.Map.Entry entry = (java.util.Map.Entry)entriesArray[i];
                            resultsArray[i] = entry.getKey();
                        }

                    } else {
                        // reshuffle and add 'key' values
                        for( Object key : results.keySet() )
                            nHash(results.get(key)).put("key", key);

                        // sort the results by matched names
                        java.util.List alist = Arrays.asList(results.entrySet().toArray());
                        Collections.sort(alist, matched_names_cmp);
                        resultsArray = new Object[alist.toArray().length];

                        int i = 0;
                        Iterator it = alist.iterator();
                        while( it.hasNext() ) {
                            Map.Entry entry = (Map.Entry)it.next();
                            Hashtable res = (Hashtable)entry.getValue();
                            resultsArray[i++] = res.get("key");
                        }
                    }

                    int resultsIx = 0;
                    for (int i = 0; i < resultsArray.length; ++i) {
                        resultsIx = i + 1;
                        String key = (String) resultsArray[i];
                        Hashtable nodeVals = (Hashtable) results.get(key);
                        String handle = (String) nodeVals.get("handle");
                        String ip = (String)Utils.n2s(nodeVals.get("ip"));
                        String name_type = (String) nodeVals.get("name_type");

                        String typeStr = "";
                        Hashtable types = Utils.nHash(nodeVals.get("node_type"));
                        for( Map.Entry ent : (java.util.List<Map.Entry>)Utils.sort(types.entrySet(),
                                     new Comparator() {
                                         public int compare(Object o1, Object o2) {
                                             String type1  = (String)((Map.Entry)o1).getKey();
                                             Integer rank1 = new Integer((String)((Map.Entry)o1).getValue());

                                             String type2  = (String)((Map.Entry)o2).getKey();
                                             Integer rank2 = new Integer((String)((Map.Entry)o2).getValue());

                                             int cmp = rank1.compareTo(rank2);
                                             if( cmp == 0 ) return type1.compareToIgnoreCase(type2);
                                             return cmp;
                                         }

                                         public boolean equals(Object obj) {
                                             return true;
                                         }
                                     }) )
                            typeStr += (typeStr.length()>0?", ":"") + ent.getKey();

                        String nodeName = (String)Utils.n2s(nodeVals.get("node_name"));
                        String pref = (String)nodeVals.get("pref");
                        if( pref != null )
                            nodeName += " ("+pref+")";
                        String mName = (String)Utils.n2s(nodeVals.get("name"));
                        String nameType = (String)Utils.n2s(nodeVals.get("name_type"));
                        if( nameType.equals("node_name") )
                            nameType = "Node";
                        else if( nameType.equals("alias_name") )
                            nameType = "Alias";
                        else if( nameType.equals("intfc_name") )
                            nameType = "Interface";
                        else if( nameType.equals("ip_addr_name") )
                            nameType = "IP Addr.";
                        else if( nameType.equals("ipc_addr_name") )
                            nameType = "IPC Addr.";
                        else if( nameType.equals("mx") )
                            nameType = "MX";

                        String alias_of = (String)nodeVals.get("alias_of");

                        String ip_type = (String)Utils.n2s(nodeVals.get("ip_type"));
                        if( ip_type.equals("intfc_ip") )
                            ip_type = "Interface";
                        else if( ip_type.equals("ipc_ip") )
                            ip_type = "IPC";
                        else if( ip_type.equals("addr_space_ip") )
                            ip_type = "Addr Sp";

                        if( wantIPSort )
                            out.println(
                                        "<TR>\n"+
                                        "<TD WIDTH='5%' ALIGN='right' NOWRAP>"+resultsIx+"&nbsp;&nbsp;</TD>\n"+
                                        "<TD WIDTH='1%' NOWRAP><A ID='Link_Info_"+i+"' HREF='/Dispatch?Action=info&object=Node&fromPage="+fromPage+"&handle="+handle+"'>"+
                                        ip+"</A>&nbsp;</TD>\n"+
                                        "<TD WIDTH='1%' NOWRAP>"+ip_type+"&nbsp;</TD>"+
                                        "<TD WIDTH='1%' NOWRAP>"+
                                        (hasAccess?"  <A ID='Link_Modify_"+i+"' HREF='/Dispatch?Action=modify&object=Node&fromPage="+fromPage+"&handle="+handle+"'>Mod</A>":"")+
                                        "&nbsp;&nbsp;\n"+
                                        (hasAccess?"  <A ID='Link_Template_"+i+"' HREF='/Dispatch?Action=use&object=Node&fromPage="+fromPage+"&handle="+handle+"'>Tmpl</A>":"")+
                                        "&nbsp;&nbsp;\n"+
                                        (hasAccess?"  <A ID='Link_Delete_"+i+"' HREF='/Dispatch?Action=delete&object=Node&fromPage="+fromPage+"&handle="+handle+"'>Del</A>":"")+
                                        "&nbsp;&nbsp;&nbsp;&nbsp;</TD>\n"+
                                        "<TD WIDTH='1%' COLSPAN='2' NOWRAP><A ID='Link_Info_"+i+"' HREF='/Dispatch?Action=info&object=Node&fromPage="+fromPage+"&handle="+handle+"'>"+mName+"</A>&nbsp;</TD>\n"+
                                        "<TD WIDTH='90%' NOWRAP>"+typeStr+"</TD>\n"+
                                        "</TR>\n");
                        else if( wantHW ) {
                            Parser parser = new Parser();
                            String hw = (String)Utils.n2s(Utils.nHash(parser.parse(search_string)).get("hw"));
                            hw = hw.trim().toLowerCase();
                            hw = hw.substring(0, 4)+"."+hw.substring(4, 8)+"."+hw.substring(8);
                            out.println(
                                        "<TR>\n"+
                                        "<TD WIDTH='5%' ALIGN='right' NOWRAP>"+resultsIx+"&nbsp;&nbsp;</TD>\n"+
                                        "<TD WIDTH='1%' NOWRAP><A ID='Link_Info_"+i+"' HREF='/Dispatch?Action=info&object=Node&fromPage="+fromPage+"&handle="+handle+"'>"+
                                        hw+"</A>&nbsp;</TD>\n"+
                                        "<TD WIDTH='1%' NOWRAP>"+
                                        (hasAccess?"  <A ID='Link_Modify_"+i+"' HREF='/Dispatch?Action=modify&object=Node&fromPage="+fromPage+"&handle="+handle+"'>Mod</A>":"")+
                                        "&nbsp;&nbsp;\n"+
                                        (hasAccess?"  <A ID='Link_Template_"+i+"' HREF='/Dispatch?Action=use&object=Node&fromPage="+fromPage+"&handle="+handle+"'>Tmpl</A>":"")+
                                        "&nbsp;&nbsp;\n"+
                                        (hasAccess?"  <A ID='Link_Delete_"+i+"' HREF='/Dispatch?Action=delete&object=Node&fromPage="+fromPage+"&handle="+handle+"'>Del</A>":"")+
                                        "&nbsp;&nbsp;&nbsp;&nbsp;</TD>\n"+
                                        "<TD WIDTH='1%' NOWRAP>"+mName+"&nbsp;</TD>\n"+
                                        "<TD WIDTH='1%' NOWRAP><A ID='Link_Info_"+i+"' HREF='/Dispatch?Action=info&object=Node&fromPage="+fromPage+"&handle="+handle+"'>"+
                                        ip+"</A>&nbsp;</TD>\n"+
                                        "<TD WIDTH='90%' COLSPAN='2' NOWRAP>"+typeStr+"</TD>\n"+
                                        "</TR>\n");

                        } else
                            out.println(
                                        "<TR>\n"+
                                        "<TD WIDTH='5%' ALIGN='right' NOWRAP>"+resultsIx+"&nbsp;&nbsp;</TD>\n"+
                                        "<TD WIDTH='1%' NOWRAP><A ID='Link_Info_"+i+"' HREF='/Dispatch?Action=info&object=Node&fromPage="+fromPage+"&handle="+handle+"'>"+
                                        mName+"</A>&nbsp;</TD>\n"+
                                        "<TD WIDTH='1%' NOWRAP>"+nameType+"&nbsp;</TD>"+
                                        "<TD WIDTH='1%' NOWRAP>"+
                                        (hasAccess?"  <A ID='Link_Modify_"+i+"' HREF='/Dispatch?Action=modify&object=Node&fromPage="+fromPage+"&handle="+handle+"'>Mod</A>":"")+
                                        "&nbsp;&nbsp;\n"+
                                        (hasAccess?"  <A ID='Link_Template_"+i+"' HREF='/Dispatch?Action=use&object=Node&fromPage="+fromPage+"&handle="+handle+"'>Tmpl</A>":"")+
                                        "&nbsp;&nbsp;\n"+
                                        (hasAccess?"  <A ID='Link_Delete_"+i+"' HREF='/Dispatch?Action=delete&object=Node&fromPage="+fromPage+"&handle="+handle+"'>Del</A>":"")+
                                        "&nbsp;&nbsp;&nbsp;&nbsp;</TD>\n"+
                                        "<TD WIDTH='1%' NOWRAP>"+(alias_of!=null?alias_of:nodeName)+"&nbsp;</TD>\n"+
                                        "<TD WIDTH='1%' NOWRAP><A ID='Link_Info_"+i+"' HREF='/Dispatch?Action=info&object=Node&fromPage="+fromPage+"&handle="+handle+"'>"+
                                        ip+"</A>&nbsp;</TD>\n"+
                                        "<TD WIDTH='90%' NOWRAP>"+typeStr+"</TD>\n"+
                                        "</TR>\n");
                    }

                    printSeparator(out);
                    out.println("</TABLE>\n</DIV>\n");
                }

                // NETWORKS

                if (wantNetworks) {
                    int searchType = Constants.NETWORK;
                    StringBuffer sb = new StringBuffer();
                    //Hashtable results = null;
                    try {
                        try {
                            results = netdb.simpleSearch(searchType, search_string, sb);
                        } catch (NetDBException ex) {
                            Vector errors = ex.getMessages();
                            String msg = "";
                            int esize = errors.size();
                            for( int j = 0; j < esize; j++ )
                                msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                            throw new Exception(msg);
                        }
                    } catch (Exception ex) {
                        addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                    }
                    results = nHash(results);

                    boolean wantIPSort = sb.toString().equals("ip address");
                    int resultsCt = results.size();

                    gotAny = true;
                    htmlTop(session, search_string, out);
                    if( wantIPSort )
                        out.println(
                                    "<TR CLASS=\"qs_table_header\">\n"+
                                    "  <TD NOWRAP COLSPAN=7>Networks ( "+resultsCt+" )</TD>\n"+
                                    "</TR>\n"+
                                    "<TR CLASS=\"qs_table_sub_header\">\n"+
                                    "  <TD WIDTH='5%' ALIGN='RIGHT' NOWRAP>#&nbsp;&nbsp;</TD>"+
                                    "  <TD NOWRAP>IP Address</TD>"+
                                    "  <TD NOWRAP>IP Type</TD>"+
                                    "  <TD NOWRAP>Actions</TD>"+
                                    "  <TD COLSPAN=3 NOWRAP>Network</TD>\n"+
                                    "</TR>\n"
                                    );
                    else
                        out.println(
                                    "<TR CLASS=\"qs_table_header\">\n"+
                                    "  <TD NOWRAP COLSPAN=7>Networks ( "+resultsCt+" )</TD>\n"+
                                    "</TR>\n"+
                                    "<TR CLASS=\"qs_table_sub_header\">\n"+
                                    "  <TD WIDTH='5%' ALIGN='RIGHT' NOWRAP>#&nbsp;&nbsp;</TD>"+
                                    "  <TD NOWRAP>Name</TD>"+
                                    "  <TD NOWRAP>Name Type</TD>"+
                                    "  <TD NOWRAP>Actions</TD>"+
                                    "  <TD NOWRAP>Network</TD>"+
                                    "  <TD COLSPAN=2 NOWRAP>IP Address</TD>\n"+
                                    "</TR>\n"
                                    );

                    // check if user can modify
                    Boolean checkValue = (Boolean) session.getAttribute("hasNetworkObjectAccess?");
                    boolean hasAccess = checkValue.booleanValue();

                    Object[] resultsArray = null;
                    if (wantIPSort)
                        {
                            // sort the results by ip-addr
                            java.util.List alist = Arrays.asList(results.entrySet().toArray());
                            Collections.sort(alist, ipentry_cmp);
                            Object[] entriesArray = alist.toArray();
                            resultsArray = new Object[entriesArray.length];
                            for (int i = 0; i < entriesArray.length; ++i)
                                {
                                    java.util.Map.Entry entry = (java.util.Map.Entry)entriesArray[i];
                                    resultsArray[i] = entry.getKey();
                                }
                        }
                    else
                        {
                            // sort the results by node name
                            java.util.List alist = Arrays.asList(results.keySet().toArray());
                            Collections.sort(alist, name_cmp);
                            resultsArray = alist.toArray();
                        }

                    int resultsIx = 0;
                    for (int i = 0; i < resultsArray.length; ++i)
                        {
                            resultsIx = i + 1;
                            String netKey = (String) resultsArray[i];
                            Hashtable netVals = (Hashtable) results.get(netKey);
                            String handle = (String) netVals.get("handle");
                            String net_name = ((String)Utils.n2s(netVals.get("net_name"))).trim();
                            String mName = (String) netVals.get("dhcp_name");
                            String name_type = "Dyn DHCP";
                            if( mName == null ) {
                                mName = net_name;
                                name_type = "Network";
                            }

                            String ip = (String)Utils.n2s(netVals.get("ip"));
                            String ip_type = ip.indexOf("/")>-1?"Addr Sp":"Dyn DHCP";

                            if( wantIPSort )
                                out.println(
                                            "<TR>\n"+
                                            "<TD WIDTH='5%' ALIGN='right' NOWRAP>"+resultsIx+"&nbsp;&nbsp;</TD>\n"+
                                            "<TD WIDTH='1%' NOWRAP><A ID='Link_Info_"+i+"' HREF='/Dispatch?Action=info&object=Network&fromPage="+fromPage+"&handle="+handle+"'>"+
                                            ip+"</A>&nbsp;</TD>\n"+
                                            "<TD WIDTH='1%' NOWRAP>"+ip_type+"&nbsp;</TD>"+
                                            "<TD WIDTH='1%' NOWRAP>"+
                                            (hasAccess?"  <A ID='Link_Modify_"+i+"' HREF='/Dispatch?Action=modify&object=Network&fromPage="+fromPage+"&handle="+handle+"'>Mod</A>":"")+
                                            "&nbsp;&nbsp;\n"+
                                            (hasAccess?"  <A ID='Link_Template_"+i+"' HREF='/Dispatch?Action=use&object=Network&fromPage="+fromPage+"&handle="+handle+"'>Tmpl</A>":"")+
                                            "&nbsp;&nbsp;\n"+
                                            (hasAccess?"  <A ID='Link_Delete_"+i+"' HREF='/Dispatch?Action=delete&object=Network&fromPage="+fromPage+"&handle="+handle+"'>Del</A>":"")+
                                            "&nbsp;&nbsp;&nbsp;&nbsp;</TD>\n"+
                                            "<TD WIDTH='92%' COLSPAN='3' NOWRAP><A ID='Link_Info_"+i+"' HREF='/Dispatch?Action=info&object=Network&fromPage="+fromPage+"&handle="+handle+"'>"+
                                            net_name+"</A>&nbsp;</TD>\n"+
                                            "</TR>"
                                            );
                            else
                                out.println(
                                            "<TR>\n"+
                                            "<TD WIDTH='5%' ALIGN='right' NOWRAP>"+resultsIx+"&nbsp;&nbsp;</TD>\n"+
                                            "<TD WIDTH='1%' NOWRAP><A ID='Link_Info_"+i+"' HREF='/Dispatch?Action=info&object=Network&fromPage="+fromPage+"&handle="+handle+"'>"+
                                            mName+"</A>&nbsp;</TD>\n"+
                                            "<TD WIDTH='1%' NOWRAP>"+name_type+"&nbsp;</TD>"+
                                            "<TD WIDTH='1%' NOWRAP>"+
                                            (hasAccess?"  <A ID='Link_Modify_"+i+"' HREF='/Dispatch?Action=modify&object=Network&fromPage="+fromPage+"&handle="+handle+"'>Mod</A>":"")+
                                            "&nbsp;&nbsp;\n"+
                                            (hasAccess?"  <A ID='Link_Template_"+i+"' HREF='/Dispatch?Action=use&object=Network&fromPage="+fromPage+"&handle="+handle+"'>Tmpl</A>":"")+
                                            "&nbsp;&nbsp;\n"+
                                            (hasAccess?"  <A ID='Link_Delete_"+i+"' HREF='/Dispatch?Action=delete&object=Network&fromPage="+fromPage+"&handle="+handle+"'>Del</A>":"")+
                                            "&nbsp;&nbsp;&nbsp;&nbsp;</TD>\n"+
                                            "<TD WIDTH='1%' NOWRAP>"+net_name+"&nbsp;</TD>\n"+
                                            "<TD WIDTH='91%' COLSPAN='2' NOWRAP><A ID='Link_Info_"+i+"' HREF='/Dispatch?Action=info&object=Network&fromPage="+fromPage+"&handle="+handle+"'>"+
                                            ip+"</A>&nbsp;</TD>\n"+
                                            "</TR>"
                                            );
                        }

                    printSeparator(out);
                    out.println("</TABLE>\n</DIV>\n");
                }

                // MX

                if (wantMXes) {
                    int searchType = Constants.MX;
                    //Hashtable results = null;
                    try {
                        try {
                            results = netdb.simpleSearch(searchType, search_string);
                        } catch (NetDBException ex) {
                            Vector errors = ex.getMessages();
                            String msg = "";
                            int esize = errors.size();
                            for( int j = 0; j < esize; j++ )
                                msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                            throw new Exception(msg);
                        }
                    } catch (Exception ex) {
                        addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                    }
                    results = nHash(results);

                    int resultsCt = results.size();

                    gotAny = true;
                    htmlTop(session, search_string, out);
                    out.println(
                                "<TR CLASS=\"qs_table_header\">\n"+
                                "  <TD NOWRAP COLSPAN=7>MXes ( "+resultsCt+" )</TD>\n"+
                                "</TR>\n"+
                                "<TR CLASS=\"qs_table_sub_header\">\n"+
                                "  <TD WIDTH='5%' ALIGN='RIGHT' NOWRAP>#&nbsp;&nbsp;</TD>"+
                                "  <TD NOWRAP>Name</TD>"+
                                "  <TD NOWRAP>Name Type</TD>"+
                                "  <TD NOWRAP>Actions</TD>"+
                                "  <TD COLSPAN=\"3\" NOWRAP>Node</TD>\n"+
                                "</TR>\n"
                                );

                    // check if user can modify
                    Boolean checkValue = (Boolean) session.getAttribute("hasNodeObjectAccess?");  // ???
                    boolean hasAccess = checkValue.booleanValue();

                    // sort results by mx name
                    java.util.List alist = Arrays.asList(results.keySet().toArray());
                    Collections.sort(alist, name_cmp);
                    Object[] resultsArray = alist.toArray();

                    int resultsIx = 0;
                    for (int i = 0; i < resultsArray.length; ++i)
                        {
                            resultsIx = i + 1;
                            String mxKey = (String) resultsArray[i];
                            Hashtable mxVals = (Hashtable) results.get(mxKey);
                            String handle = (String) mxVals.get("handle");
                            String name = (String) mxVals.get("name");
                            String mx = (String) mxVals.get("mx");
                            Integer pref = (Integer) mxVals.get("pref");

                            out.println(
                                        "<TR>\n"+
                                        "<TD WIDTH='5%' ALIGN='right' NOWRAP><FONT SIZE='-1' FACE='Arial'>"+resultsIx+
                                        "&nbsp;&nbsp;</TD>\n"+
                                        "<TD WIDTH='1%' NOWRAP><A ID='Link_Info_"+i+"' HREF='/Dispatch?Action=info&object=Node&fromPage="+fromPage+"&handle="+handle+"'>"+mx+
                                        "</A>&nbsp;</TD>\n"+
                                        "<TD WIDTH='1%' NOWRAP>MX&nbsp;</TD>"+
                                        "<TD WIDTH='1%' NOWRAP>"+
                                        (hasAccess?"  <A ID='Link_Modify_"+i+"' HREF='/Dispatch?Action=modify&object=Node&fromPage="+fromPage+"&handle="+handle+"'>Mod</A>":"")+
                                        "&nbsp;&nbsp;\n"+
                                        (hasAccess?"  <A ID='Link_Template_"+i+"' HREF='/Dispatch?Action=use&object=Node&fromPage="+fromPage+"&handle="+handle+"'>Tmpl</A>":"")+
                                        "&nbsp;&nbsp;\n"+
                                        (hasAccess?"  <A ID='Link_Delete_"+i+"' HREF='/Dispatch?Action=delete&object=Node&fromPage="+fromPage+"&handle="+handle+"'>Del</A>":"")+
                                        "&nbsp;&nbsp;&nbsp;&nbsp;</TD>\n"+
                                        "<TD WIDTH='65%' COLSPAN='3' NOWRAP><A ID='Link_Info_"+i+"' HREF='/Dispatch?Action=info&object=Node&fromPage="+fromPage+"&handle="+
                                        handle+"'>"+name+"</A> ("+pref+")&nbsp;</TD>\n"+
                                        "</TR>"
                                        );
                        }

                    printSeparator(out);
                    out.println("</TABLE>\n</DIV>\n");
                }

                // USERS

                if (wantUsers) {
                    int searchType = Constants.USER;
                    //Hashtable results = null;
                    try {
                        try {
                            results = netdb.simpleSearch(searchType, search_string);
                        } catch (NetDBException ex) {
                            Vector errors = ex.getMessages();
                            String msg = "";
                            int esize = errors.size();
                            for( int j = 0; j < esize; j++ )
                                msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                            throw new Exception(msg);
                        }
                    } catch (Exception ex) {
                        addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                    }
                    results = nHash(results);

                    int resultsCt = results.size();

                    gotAny = true;
                    htmlTop(session, search_string, out);
                    out.println(
                                "<TR CLASS=\"qs_table_header\">\n"+
                                "  <TD NOWRAP COLSPAN=7>Users ( "+resultsCt+" )</TD>\n"+
                                "</TR>\n"+
                                "<TR CLASS=\"qs_table_sub_header\">\n"+
                                "  <TD WIDTH='5%' ALIGN='RIGHT' NOWRAP>#&nbsp;&nbsp;</TD>"+
                                "  <TD NOWRAP>User Name</TD>"+
                                "  <TD NOWRAP>Actions</TD>"+
                                "  <TD COLSPAN=\"4\" NOWRAP>NetID</TD>\n"+
                                "</TR>\n"
                                );

                    // check if user can modify
                    Boolean checkValue = (Boolean) session.getAttribute("hasUserObjectAccess?");
                    boolean hasAccess = checkValue.booleanValue();

                    // sort results by user name
                    java.util.List alist = Arrays.asList(results.keySet().toArray());
                    Collections.sort(alist, name_cmp);
                    Object[] resultsArray = alist.toArray();

                    int resultsIx = 0;
                    for (int i = 0; i < resultsArray.length; ++i)
                        {
                            resultsIx = i + 1;
                            String userKey = (String) resultsArray[i];
                            Hashtable userVals = (Hashtable) results.get(userKey);
                            String name = (String) userVals.get("name");
                            String user_id = (String) userVals.get("NetID");

                            out.println(
                                        "<TR>\n"+
                                        "<TD WIDTH='5%' ALIGN='right' NOWRAP>"+resultsIx+"&nbsp;&nbsp;</TD>\n"+
                                        "<TD WIDTH='1%' NOWRAP><A ID='Link_Info_"+i+"' HREF='/Dispatch?Action=info&object=User&fromPage="+fromPage+"&handle="+user_id+"'>"+
                                        Utils.htmlize(name, false)+"</A>&nbsp;</TD>\n"+
                                        "<TD WIDTH='1%' NOWRAP>"+
                                        (hasAccess?"  <A ID='Link_Modify_"+i+"' HREF='/Dispatch?Action=modify&object=User&fromPage="+fromPage+"&handle="+user_id+"'>Mod</A>":"")+
                                        "&nbsp;&nbsp;\n"+
                                        (hasAccess?"  <A ID='Link_Template_"+i+"' HREF='/Dispatch?Action=use&object=User&fromPage="+fromPage+"&handle="+user_id+"'>Tmpl</A>":"")+
                                        "&nbsp;&nbsp;\n"+
                                        (hasAccess?"  <A ID='Link_Delete_"+i+"' HREF='/Dispatch?Action=delete&object=User&fromPage="+fromPage+"&handle="+user_id+"'>Del</A>":"")+
                                        "&nbsp;&nbsp;&nbsp;&nbsp;</TD>\n"+
                                        "<TD WIDTH='93%' COLSPAN='4' NOWRAP><A ID='Link_Info_"+i+"' HREF='/Dispatch?Action=info&object=User&fromPage="+fromPage+"&handle="+user_id+"'>"+
                                        Utils.htmlize(user_id, false)+"</A></TD>\n"+
                                        "</TR>"
                                        );
                        }

                    printSeparator(out);
                    out.println("</TABLE>\n</DIV>\n");
                }

                // GROUPS

                if (wantGroups) {
                    int searchType = Constants.GROUP;
                    //Hashtable results = null;
                    try {
                        try {
                            results = netdb.simpleSearch(searchType, search_string);
                        } catch (NetDBException ex) {
                            Vector errors = ex.getMessages();
                            String msg = "";
                            int esize = errors.size();
                            for( int j = 0; j < esize; j++ )
                                msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                            throw new Exception(msg);
                        }
                    } catch (Exception ex) {
                        addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                    }
                    results = nHash(results);

                    int resultsCt = results.size();

                    gotAny = true;
                    htmlTop(session, search_string, out);
                    out.println(
                                "<TR CLASS=\"qs_table_header\">\n"+
                                "  <TD NOWRAP COLSPAN=7>Groups ( "+resultsCt+" )</TD>\n"+
                                "</TR>\n"+
                                "<TR CLASS=\"qs_table_sub_header\">\n"+
                                "  <TD WIDTH='5%' ALIGN='RIGHT' NOWRAP>#&nbsp;&nbsp;</TD>"+
                                "  <TD NOWRAP>Group Name</TD>"+
                                "  <TD NOWRAP COLSPAN=\"5\">Actions</TD>"+
                                "</TR>\n"
                                );

                    // check if user can modify
                    Boolean checkValue = (Boolean) session.getAttribute("hasGroupObjectAccess?");
                    boolean hasAccess = checkValue.booleanValue();

                    // sort results by group name
                    java.util.List alist = Arrays.asList(results.keySet().toArray());
                    Collections.sort(alist, name_cmp);
                    Object[] resultsArray = alist.toArray();

                    int resultsIx = 0;
                    for (int i = 0; i < resultsArray.length; ++i)
                        {
                            resultsIx = i + 1;
                            String groupKey = (String) resultsArray[i];
                            String group_id = URLEncoder.encode(groupKey, "UTF-8");

                            out.println(
                                        "<TR>\n"+
                                        "<TD WIDTH='5%' ALIGN='right' NOWRAP>"+resultsIx+"&nbsp;&nbsp;</TD>\n"+
                                        "<TD WIDTH='1%' NOWRAP><A ID='Link_Info_"+i+"' HREF='/Dispatch?Action=info&object=Group&fromPage="+fromPage+"&handle="+group_id+"'>"+
                                        Utils.htmlize(groupKey, false)+"</A>&nbsp;</TD>\n"+
                                        "<TD WIDTH='96%' COLSPAN='5' NOWRAP>"+
                                        (hasAccess?"  <A ID='Link_Modify_"+i+"' HREF='/Dispatch?Action=modify&object=Group&fromPage="+fromPage+"&handle="+group_id+"'>Mod</A>":"")+
                                        "&nbsp;&nbsp;\n"+
                                        (hasAccess?"  <A ID='Link_Template_"+i+"' HREF='/Dispatch?Action=use&object=Group&fromPage="+fromPage+"&handle="+group_id+"'>Tmpl</A>":"")+
                                        "&nbsp;&nbsp;\n"+
                                        (hasAccess?"  <A ID='Link_Delete_"+i+"' HREF='/Dispatch?Action=delete&object=Group&fromPage="+fromPage+"&handle="+group_id+"'>Del</A>":"")+
                                        "</TD>\n"+
                                        "</TR>"
                                        );
                        }

                    printSeparator(out);
                    out.println("</TABLE>\n</DIV>\n");
                }

                // ATEAMS

                if (wantATeams) {
                    int searchType = Constants.ATEAM;
                    //Hashtable results = null;
                    try {
                        try {
                            results = netdb.simpleSearch(searchType, search_string);
                        } catch (NetDBException ex) {
                            Vector errors = ex.getMessages();
                            String msg = "";
                            int esize = errors.size();
                            for( int j = 0; j < esize; j++ )
                                msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                            throw new Exception(msg);
                        }
                    } catch (Exception ex) {
                        addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                    }
                    results = nHash(results);

                    int resultsCt = results.size();

                    gotAny = true;
                    htmlTop(session, search_string, out);
                    out.println(
                                "<TR CLASS=\"qs_table_header\">\n"+
                                "  <TD NOWRAP COLSPAN='7'>Admin Teams ( "+resultsCt+" )</TD>\n"+
                                "</TR>\n"+
                                "<TR CLASS=\"qs_table_sub_header\">\n"+
                                "  <TD WIDTH='5%' ALIGN='RIGHT' NOWRAP>#&nbsp;&nbsp;</TD>"+
                                "  <TD NOWRAP>ATeam Name</TD>"+
                                "  <TD NOWRAP COLSPAN=\"5\">Actions</TD>"+
                                "</TR>\n"
                                );

                    // check if user can modify
                    Boolean checkValue = (Boolean)session.getAttribute("hasATeamObjectAccess?");
                    boolean hasAccess = checkValue.booleanValue();

                    // sort results by ateam name
                    java.util.List alist = Arrays.asList(results.keySet().toArray());
                    Collections.sort(alist, name_cmp);
                    Object[] resultsArray = alist.toArray();

                    int resultsIx = 0;
                    for (int i = 0; i < resultsArray.length; ++i)
                        {
                            resultsIx = i + 1;
                            String ateamKey = (String) resultsArray[i];
                            String ateam_id = URLEncoder.encode(ateamKey, "UTF-8");

                            out.println(
                                        "<TR>\n"+
                                        "<TD WIDTH='5%' ALIGN='right' NOWRAP>"+resultsIx+"&nbsp;&nbsp;</TD>\n"+
                                        "<TD WIDTH='1%' NOWRAP><A ID='Link_Info_"+i+"' HREF='/Dispatch?Action=info&object=AdminTeam&fromPage="+
                                        fromPage+"&handle="+ateam_id+"'>"+
                                        Utils.htmlize(ateamKey)+"</A>&nbsp;</TD>\n"+
                                        "<TD WIDTH='96%' COLSPAN='5' NOWRAP>"+
                                        (hasAccess?"  <A ID='Link_Modify_"+i+"' HREF='/Dispatch?Action=modify&object=AdminTeam&fromPage="+
                                         fromPage+"&handle="+ateam_id+"'>Mod</A>":"")+
                                        "&nbsp;&nbsp;\n"+
                                        (hasAccess?"  <A ID='Link_Template_"+i+"' HREF='/Dispatch?Action=use&object=AdminTeam&fromPage="+
                                         fromPage+"&handle="+ateam_id+"'>Tmpl</A>":"")+
                                        "&nbsp;&nbsp;\n"+
                                        (hasAccess?"  <A ID='Link_Delete_"+i+"' HREF='/Dispatch?Action=delete&object=AdminTeam&fromPage="+
                                         fromPage+"&handle="+ateam_id+"'>Del</A>":"")+
                                        "</TD>\n"+
                                        "</TR>"
                                        );
                        }

                    printSeparator(out);
                    out.println("</TABLE>\n</DIV>\n");
                }

                // DOMAINS

                if (wantDomains) {
                    int searchType = Constants.DOMAIN;
                    //Hashtable results = null;
                    try {
                        try {
                            results = netdb.simpleSearch(searchType, search_string);
                        } catch (NetDBException ex) {
                            Vector errors = ex.getMessages();
                            String msg = "";
                            int esize = errors.size();
                            for( int j = 0; j < esize; j++ )
                                msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                            throw new Exception(msg);
                        }
                    } catch (Exception ex) {
                        addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                    }
                    results = nHash(results);

                    int resultsCt = results.size();

                    gotAny = true;
                    htmlTop(session, search_string, out);
                    out.println(
                                "<TR CLASS=\"qs_table_header\">\n"+
                                "  <TD NOWRAP COLSPAN='7'>Domains ( "+resultsCt+" )</TD>\n"+
                                "</TR>\n"+
                                "<TR CLASS=\"qs_table_sub_header\">\n"+
                                "  <TD WIDTH='5%' ALIGN='RIGHT' NOWRAP>#&nbsp;&nbsp;</TD>"+
                                "  <TD NOWRAP>Domain Name</TD>"+
                                "  <TD NOWRAP COLSPAN=\"5\">Actions</TD>"+
                                "</TR>\n"
                                );

                    // check if user can modify
                    Boolean checkValue = (Boolean) session.getAttribute("hasDomainObjectAccess?");
                    boolean hasAccess = checkValue.booleanValue();

                    // sort results by domain name
                    java.util.List alist = Arrays.asList(results.keySet().toArray());
                    Collections.sort(alist, name_cmp);
                    Object[] resultsArray = alist.toArray();

                    int resultsIx = 0;
                    for (int i = 0; i < resultsArray.length; ++i)
                        {
                            resultsIx = i + 1;
                            String domainKey = (String) resultsArray[i];
                            String domain_id = URLEncoder.encode(domainKey, "UTF-8");

                            // /Dispatch?Action=info&object=Domain&fromPage="+fromPage+"&handle="+domain_id+"
                            out.println("<TR>\n"+
                                        "<TD WIDTH='5%' ALIGN='right' NOWRAP>"+resultsIx+"&nbsp;&nbsp;</TD>\n"+
                                        "<TD WIDTH='1%' NOWRAP><A ID='Link_Info_"+i+"' HREF='/Dispatch?Action=info&object=Domain&fromPage="+fromPage+"&handle="+domain_id+"'>"+
                                        domainKey+"</A>&nbsp;</TD>\n"+
                                        "<TD WIDTH='96%' NOWRAP>"+
                                        (hasAccess?"  <A ID='Link_Modify_"+i+"' HREF='/Dispatch?Action=modify&object=Domain&fromPage="+fromPage+"&handle="+domain_id+"'>Mod</A>":"")+
                                        "&nbsp;&nbsp;\n"+
                                        (hasAccess?"  <A ID='Link_Template_"+i+"' HREF='/Dispatch?Action=use&object=Domain&fromPage="+fromPage+"&handle="+domain_id+"'>Tmpl</A>":"")+
                                        "&nbsp;&nbsp;\n"+
                                        (hasAccess?"  <A ID='Link_Delete_"+i+"' HREF='/Dispatch?Action=delete&object=Domain&fromPage="+fromPage+"&handle="+domain_id+"'>Del</A>":"")+
                                        "</TD>\n"+
                                        "</TR>"
                                        );
                        }

                    printSeparator(out);
                    out.println("</TABLE>\n</DIV>\n");
                }

                // display error msg if zero results
                if (! gotAny) {
                    out.println(
                                "<TR>\n"+
                                "  <TD WIDTH='100%' HEIGHT=20 COLSPAN=5 ALIGN=CENTER BGCOLOR='#CC0000'><FONT COLOR='#FFFFFF'>"+
                                "ERROR: '"+Utils.htmlize(search_string)+"' was not found"+
                                "</FONT></TD>\n"+
                                "</TR>\n"
                                );
                }

            } catch (Exception exx) {
                exx.printStackTrace();

                // here an attempted search failed so display error
                String errMsg = exx.getMessage();
                errMsg = (errMsg == null ? "???" : errMsg);
                out.println(
                            "<TR>\n"+
                            "  <TD WIDTH='100%' HEIGHT=20 COLSPAN=5 ALIGN=CENTER BGCOLOR='#CC0000'><FONT COLOR='#FFFFFF'>"+
                            "ERROR: "+Utils.htmlize(errMsg)+"</FONT></TD>\n"+
                            "</TR>\n"
                            );
            }

            // display quicksearch panel and footer
            out.println(
                        //"</TABLE>\n</DIV>\n"+
                        getCommonQSearchForm(search_string, search_object, fromPage)+
                        "<BR>\n"+
                        getCommonBottomLabel("hlpqsearch.html")+
                        "</BODY>\n"+
                        "</HTML>\n"
                        );

        } catch (Exception ex) {
            printCommonErrorPage(response, ex);
            return;

        } finally {
            try {
                session.removeAttribute("info_msg");
            } catch (Exception ex) { }
            try {
                session.removeAttribute("Error.errorMsg");
            } catch (Exception ex) {}
            try {
                session.removeAttribute("Warning");
            } catch (Exception ex) { }
        }

    }

    private void printSeparator(PrintWriter out) throws Exception {
        out.println("<TR>\n"+
                    "  <TD COLSPAN=\"7\"><HR></TD>\n"+
                    "</TR>\n");
    }

    private void htmlTop(HttpSession session, String search_string, PrintWriter out) throws Exception {
        out.println(msgPanels(session)+
                    "<DIV id=\"qs_results\">\n"+
                    "<TABLE WIDTH='100%' CELLSPACING='1' CELLPADDING='0' BORDER='0'>\n");
    }
}
