/************************************************************************
 *
 * HtmlResponder.java -
 *
 * $Id: HtmlResponder.java,v 1.3 2007/03/28 16:48:31 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2006, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.whois;

import java.util.*;
import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.regex.*;
import stanford.netdb.utils.*;
import su.netdb.parser.Parser;

public class HtmlResponder implements Runnable {
    private static Parser parser = new Parser();
    private Socket client = null;

    public HtmlResponder(Socket client) {
        this.client = client;
    }

    /** Responds on html request */
    public void run() {
        try {
            Hashtable request = new Hashtable();
            InetAddress rAddr = client.getInetAddress();
            String remoteAddr = rAddr.toString();
            String remoteName = ((String)Utils.n2s(rAddr.getHostName())).toLowerCase();
            BufferedReader reader = new BufferedReader(new InputStreamReader(client.getInputStream()));
            PrintWriter writer = new PrintWriter(new OutputStreamWriter(client.getOutputStream()));

            // read request
            String search_string = (String)Utils.n2s(reader.readLine());
            if( Utils.nStr(System.getProperty("self.web.server")).equals("true") ) {
                // extract search string from GET request
                Matcher m = Pattern.compile("^GET /lookup\\?key=(.*) HTTP/.*$").matcher(search_string);
                if( m.matches() ) search_string = m.group(1);

                m = Pattern.compile("^GET /(.*) HTTP/.*$").matcher(search_string);
                if( m.matches() ) search_string = m.group(1);
            }

            String error_msg = "";
            String line = new String(search_string);

            // check remote addr for access
            if( System.getProperty("allow") != null &&
                !remoteName.endsWith(System.getProperty("allow")) ) {
                WhoIsServer.info("denied html query ["+line+"] from "+remoteAddr);
                client.close();
                return;
            }

            // here we do the actual search
            boolean do_search = false;
            int search_for = -1;
            StringBuffer result = new StringBuffer(10240);

            if( Utils.nStr(System.getProperty("self.web.server")).equals("true") )
                result.append("HTTP/1.1 200 OK\nDate: "+(new Date())+
                              "\nServer: netdb-whois-server"+
                              "\nConnection: close\n"+
                              "Content-Type: text/html\n\n<html>\n<body>\n");

            try {
                if( line.length() > 0 ) {
                    Hashtable parsing_result = WhoIsServer.parseInputString(parser, search_string);

                    if( parsing_result.get("hw") != null ) { // hardware address search
                        search_for = WhoIsServer.SEARCH_FOR_HW_ADDR;
                        do_search = true;

                    } else if( parsing_result.get("name") != null ) { // dns name
                        search_for = WhoIsServer.SEARCH_FOR_NAME;
                        do_search = true;

                    } else if( parsing_result.get("ip_low") != null ) { // ip address search
                        search_for = WhoIsServer.SEARCH_FOR_IP_ADDR;
                        do_search = true;
                    }

                    if( do_search )
                        (new HttpFormatter(WhoIsServer.search(search_string), search_for)).format(result, line);

                    // write out result
                    writer.print(result);

                } else {
                    writer.println("");
                }
                if( Utils.nStr(System.getProperty("self.web.server")).equals("true") ) 
                    writer.println("\n</body>\n</html>");

            } catch(Exception exx) {
                exx.printStackTrace(writer);
            }
            writer.flush();

            WhoIsServer.info("html query ["+line+"] from "+remoteAddr);

        } catch(Exception ex) {
            ex.printStackTrace();
            WhoIsServer.error("Exception in HtmlResponder ["+ex+"]");

        } finally {
            try {
                client.close();
            } catch(Exception exx) { }
        }
    }
}
